!******************************************************************************!
!                                                                              !
!    class_loop.f90                                                            !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_loop.f90                                 |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 07/2017                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2018-02-13 19:17             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_loop
  use class_vertices
  use constants_mdl
  use lcSUUa, only: loopSUUa
  use lcFFSa, only: loopFFSa
  use lcSSVV, only: loopSSVV
  use lcSSSS, only: loopSSSS
  use lcUUVVa, only: loopUUVVa
  use lcVVV, only: loopVVV
  use lcSSV, only: loopSSV
  use lcSSUUa, only: loopSSUUa
  use lcSVV, only: loopSVV
  use lcVVVV, only: loopVVVV
  use lcUUVa, only: loopUUVa
  use lcSSS, only: loopSSS
  use lcFFVa, only: loopFFVa
  implicit none

  interface compute_loop_mdl
    module procedure computeLoop2, computeLoop3, computeLoop4, computeLoop5,   &
    computeLoop6, computeLoop7, computeLoop8
  end interface 

  contains

  subroutine computeLoop2(ty, co, wl, P1, PL1, m, wp, riMaxIn, riMaxOut,       &
  rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop2

  subroutine computeLoop3(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,    &
  riMaxOut, rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case(UUaS0, USUa0, SUUa0, UaSU0, SUaU0, UaUS0)
      call loopSUUa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(FFaS0, FFaS1, FFaS2, FSFa0, FSFa1, FSFa2, FaFS0, FaFS1, FaFS2, SFFa0, &
    SFFa1, SFFa2, SFaF0, SFaF1, SFaF2, FaSF0, FaSF1, FaSF2)
      call loopFFSa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(VVV0, VVV1)
      call loopVVV(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut, &
      rankInc)
    case(SVS0, VSS0, VSS1, SSV0, SSV1)
      call loopSSV(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut, &
      rankInc)
    case(VVS0, SVV0, VSV0)
      call loopSVV(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut, &
      rankInc)
    case(UUaV0, UUaV1, UVUa0, UaUV0, UaUV1, UaVU0, VUUa0, VUUa1, VUaU0, VUaU1)
      call loopUUVa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(SSS0)
      call loopSSS(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut, &
      rankInc)
    case(FFaV0, FFaV1, VFaF0, VFaF1, FaVF0, FaVF1, FVFa0, FVFa1, FaFV0, FaFV1, &
    VFFa0, VFFa1)
      call loopFFVa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop3

  subroutine computeLoop4(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp &
  , riMaxIn, riMaxOut, rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case(SVVS0, VVSS0, SVSV0, VSVS0, SSVV0, VSSV0)
      call loopSSVV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,      &
      riMaxIn, riMaxOut, rankInc)
    case(SSSS0)
      call loopSSSS(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,      &
      riMaxIn, riMaxOut, rankInc)
    case(UaVUV0, VVUUa0, UUaVV0, VUaUV0, UaVVU0, VUVUa0, UVUaV0, VVUaU0, VUUaV0&
    , UaUVV0, UVVUa0, VUaVU0)
      call loopUUVVa(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(UaSSU0, SUSUa0, SUUaS0, SSUUa0, SUaSU0, UaUSS0, UaSUS0, USUaS0, SSUaU0&
    , USSUa0, UUaSS0, SUaUS0)
      call loopSSUUa(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(VVVV0, VVVV1)
      call loopVVVV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,      &
      riMaxIn, riMaxOut, rankInc)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop4

  subroutine computeLoop5(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, m, wp, riMaxIn, riMaxOut, rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop5

  subroutine computeLoop6(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, m, wp, riMaxIn, riMaxOut, rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop6

  subroutine computeLoop7(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, w6, P6, PL6, m, wp, riMaxIn, riMaxOut, rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (kind = dp), intent(in), dimension(0:3) :: P6, PL6
    complex (kind = dp), intent(in), dimension(0:3) :: w6
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop7

  subroutine computeLoop8(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, w6, P6, PL6, w7, P7, PL7, m, wp, riMaxIn, riMaxOut,      &
  rankInc)
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (kind = dp), intent(in), dimension(0:3) :: P6, PL6
    complex (kind = dp), intent(in), dimension(0:3) :: w6
    complex (kind = dp), intent(in), dimension(0:3) :: P7, PL7
    complex (kind = dp), intent(in), dimension(0:3) :: w7
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop8

end module class_loop
