!******************************************************************************!
!                                                                              !
!    class_vertices.f90                                                        !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_vertices.f90                             |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 07/2017                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2018-02-13 19:17             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_vertices
  use fill_couplings
  use class_particles
  use class_ctparameters
  use fill_ctparameters, only: init_CTparameters_mdl
  use particle_keys, only: createParticleConfig
  use dictionary, only: dict,dict_get_value,dict_init
  use constants_mdl, only: Nc
  implicit none
  save

  interface get_vertex_mdl
    module procedure get_vertex3_mdl, get_vertex4_mdl, get_vertexN_mdl
  end interface

  type Branch
    integer :: lorentz_id
    integer, allocatable :: couplings(:)
    integer, dimension(n_orders) :: coupling_order
    integer :: color_id
    integer :: colorfac_id
    ! Propagator extension (only supported for massive vectors)
    ! 0: Usual propagator
    ! 1: Rxi-propagator part with pole @ M_V^2
    ! 2: Rxi-propagator part with pole @ \xi*M_V^2
    integer :: pext
  end type

  type Vertex
    type(Branch), allocatable :: branch(:)
    type(Branch), allocatable :: branchCT(:)
    type(Branch), allocatable :: branchR2(:)
  end type

  logical :: vertices_init = .false.
  logical :: vertices_filled = .false.
    interface createCurrent
      module procedure createCurrent2, createCurrent3, createCurrent4,         &
      createCurrent5, createCurrent6, createCurrent7, createCurrent8
    end interface 
  integer, parameter :: n_vertices = 10782
  type(Vertex) :: vertices(0:n_vertices-1)

  ! Defines the maximum allowed number of legs participating at a vertex
  integer, parameter :: max_leg_multiplicity = 4

  integer, parameter :: FFaS0 = 1
  integer, parameter :: FFaS1 = 2
  integer, parameter :: FFaS2 = 3
  integer, parameter :: UaVUV0 = 4
  integer, parameter :: FFaV0 = 5
  integer, parameter :: FFaV1 = 6
  integer, parameter :: SVVS0 = 7
  integer, parameter :: UaSSU0 = 8
  integer, parameter :: VVS0 = 9
  integer, parameter :: UUaS0 = 10
  integer, parameter :: UUaV0 = 11
  integer, parameter :: UUaV1 = 12
  integer, parameter :: SUSUa0 = 13
  integer, parameter :: SUUaS0 = 14
  integer, parameter :: USUa0 = 15
  integer, parameter :: SSUUa0 = 16
  integer, parameter :: SUUa0 = 17
  integer, parameter :: VVSS0 = 18
  integer, parameter :: UVUa0 = 19
  integer, parameter :: UaUV0 = 20
  integer, parameter :: UaUV1 = 21
  integer, parameter :: VFaF0 = 22
  integer, parameter :: VFaF1 = 23
  integer, parameter :: SUaSU0 = 24
  integer, parameter :: VVV0 = 25
  integer, parameter :: VVV1 = 26
  integer, parameter :: FSFa0 = 27
  integer, parameter :: FSFa1 = 28
  integer, parameter :: FSFa2 = 29
  integer, parameter :: UaUSS0 = 30
  integer, parameter :: VVUUa0 = 31
  integer, parameter :: FaVF0 = 32
  integer, parameter :: FaVF1 = 33
  integer, parameter :: UUaVV0 = 34
  integer, parameter :: UaVU0 = 35
  integer, parameter :: VUaUV0 = 36
  integer, parameter :: FVFa0 = 37
  integer, parameter :: FVFa1 = 38
  integer, parameter :: UaSUS0 = 39
  integer, parameter :: UaSU0 = 40
  integer, parameter :: FaF0 = 41
  integer, parameter :: FaF1 = 42
  integer, parameter :: FaF2 = 43
  integer, parameter :: FaF3 = 44
  integer, parameter :: USUaS0 = 45
  integer, parameter :: VV0 = 46
  integer, parameter :: VV1 = 47
  integer, parameter :: UaVVU0 = 48
  integer, parameter :: SVSV0 = 49
  integer, parameter :: VUVUa0 = 50
  integer, parameter :: FaFS0 = 51
  integer, parameter :: FaFS1 = 52
  integer, parameter :: FaFS2 = 53
  integer, parameter :: FFa0 = 54
  integer, parameter :: FFa1 = 55
  integer, parameter :: FFa2 = 56
  integer, parameter :: FFa3 = 57
  integer, parameter :: FaFV0 = 58
  integer, parameter :: FaFV1 = 59
  integer, parameter :: SSSS0 = 60
  integer, parameter :: VSVS0 = 61
  integer, parameter :: UVUaV0 = 62
  integer, parameter :: VS0 = 63
  integer, parameter :: SFFa0 = 64
  integer, parameter :: SFFa1 = 65
  integer, parameter :: SFFa2 = 66
  integer, parameter :: VUUa0 = 67
  integer, parameter :: VUUa1 = 68
  integer, parameter :: SFaF0 = 69
  integer, parameter :: SFaF1 = 70
  integer, parameter :: SFaF2 = 71
  integer, parameter :: SV0 = 72
  integer, parameter :: SVV0 = 73
  integer, parameter :: FaSF0 = 74
  integer, parameter :: FaSF1 = 75
  integer, parameter :: FaSF2 = 76
  integer, parameter :: SVS0 = 77
  integer, parameter :: VVUaU0 = 78
  integer, parameter :: VUUaV0 = 79
  integer, parameter :: UaUVV0 = 80
  integer, parameter :: VSS0 = 81
  integer, parameter :: VSS1 = 82
  integer, parameter :: VSV0 = 83
  integer, parameter :: SS0 = 84
  integer, parameter :: SSVV0 = 85
  integer, parameter :: SSV0 = 86
  integer, parameter :: SSV1 = 87
  integer, parameter :: UVVUa0 = 88
  integer, parameter :: SSS0 = 89
  integer, parameter :: VUaU0 = 90
  integer, parameter :: VUaU1 = 91
  integer, parameter :: SSUaU0 = 92
  integer, parameter :: VSSV0 = 93
  integer, parameter :: VVVV0 = 94
  integer, parameter :: VVVV1 = 95
  integer, parameter :: SUaU0 = 96
  integer, parameter :: USSUa0 = 97
  integer, parameter :: UaUS0 = 98
  integer, parameter :: VFFa0 = 99
  integer, parameter :: VFFa1 = 100
  integer, parameter :: UUaSS0 = 101
  integer, parameter :: VUaVU0 = 102
  integer, parameter :: SUaUS0 = 103
  complex(dp), parameter :: c1 = complex(1d0, 0d0)
  character(len=20), parameter :: c1f = "1"
  complex(dp), parameter :: c2 = complex(sq2/2d0, 0d0)
  character(len=20), parameter :: c2f = "sq2/2"
  complex(dp), parameter :: c3 = complex(-sq2/(2d0*Nc), 0d0)
  character(len=20), parameter :: c3f = "-sq2/(2*Nc)"
  complex(dp), parameter :: c4 = complex(0d0, sq2/2d0)
  character(len=20), parameter :: c4f = "i_*sq2/2"
  complex(dp), parameter :: c5 = complex(0d0, -sq2/2d0)
  character(len=20), parameter :: c5f = "-i_*sq2/2"
  complex(dp), parameter :: c6 = complex(1d0/2d0, 0d0)
  character(len=20), parameter :: c6f = "1/2"
  complex(dp), parameter :: c7 = complex(-1d0/2d0, 0d0)
  character(len=20), parameter :: c7f = "-1/2"
  complex(dp), parameter :: c8 = complex(-1d0/Nc, 0d0)
  character(len=20), parameter :: c8f = "-1/Nc"
  complex(dp), dimension(1:8) :: colorcoeffs = (/c1, c2, c3, c4, c5, c6, c7, c8&
                                               /)
  character(len=20), dimension(1:8) :: colorcoeffs_form = (/c1f, c2f, c3f, c4f &
                 , c5f, c6f, c7f, c8f/)
  integer, dimension(1:103) :: lorentz_rank = (/0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                              , 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1&
                                              , 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1&
                                              , 0, 1, 0, 1, 1, 0, 0, -1, -1, -1&
                                              , -1, 0, -1, -1, 0, 0, 0, 0, 0, 0&
                                              , -1, -1, -1, -1, 0, 0, 0, 0, 0  &
                                              , -1, 1, 1, 1, 1, 1, 1, 1, 1, -1 &
                                              , 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0&
                                              , -1, 0, 1, 0, 0, 0, 1, 0, 0, 0, &
                                              0, 0, 0, 0, 0, 1, 1, 0, 0, 0/)
  integer, dimension(1:103) :: recola_base = (/5, 9, 7, -14, 15, 17, -2, -1, 2 &
                                             , 1, 161, 11, -1, -1, 1, -1, 1, -4&
                                             , 3, 164, 11, 58, 53, -1, 160, 12 &
                                             , 21, 25, 27, -1, -4, 51, 57, -12 &
                                             , 3, -13, 31, 35, -1, 1, 0, 0, 0, &
                                             0, -1, 0, 0, -2, -14, -3, 6, 10, 8&
                                             , 0, 0, 0, 0, 16, 18, -1, -3, -14 &
                                             , 0, 23, 28, 26, 163, 4, 43, 46,  &
                                             48, 0, 13, 41, 47, 45, 3, -4, -13 &
                                             , -12, 4, 162, 14, 0, -12, 11, 161&
                                             , -2, 1, 4, 162, -1, -13, -111, - &
                                             11, 1, -1, 1, 33, 36, -1, -3, -1/)
  integer, dimension(1:103) :: propagator_extension = (/0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0/)
  character(len=*), parameter :: model_timestamp = "Tue Feb 13 19:08:04 2018"

  contains
    function get_timestamp_mdl() result(ret_timestamp)
      character(len=len(model_timestamp)) :: ret_timestamp
      ret_timestamp = model_timestamp
    end function get_timestamp_mdl

    function get_lorentz_rank_mdl(l_id) result(ret_lorentz_rank)
      integer, intent(in) :: l_id
      integer :: ret_lorentz_rank
      ret_lorentz_rank = lorentz_rank(l_id)
    end function get_lorentz_rank_mdl

    function get_propagator_extension_mdl(l_id) result(ret_pext)
      integer, intent(in) :: l_id
      integer :: ret_pext
      ret_pext = propagator_extension(l_id)
    end function get_propagator_extension_mdl

    function get_max_leg_multiplicity_mdl() result(ret_max_leg_multiplicity)
      integer :: ret_max_leg_multiplicity
      ret_max_leg_multiplicity = max_leg_multiplicity
    end function get_max_leg_multiplicity_mdl

    function is_model_init_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_init
    end function is_model_init_mdl

    function is_model_filled_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_filled
    end function is_model_filled_mdl

    subroutine get_vertex3_mdl(p1, p2, p3, vertex_ret)
      integer, intent(out) :: vertex_ret
      integer, intent(in)  :: p1, p2, p3
      integer(kind=8)      :: config
      integer              :: address
      logical              :: found_vertex

      call createParticleConfig(p1, p2, p3, config)
      call dict(config, 1, address, found_vertex, 0)
      if(found_vertex) then
        vertex_ret = dict_get_value(address)
      else
        vertex_ret = -1
      end if

    end subroutine get_vertex3_mdl

    subroutine get_vertex4_mdl(p1, p2, p3, p4, vertex_ret)
      integer, intent(out) :: vertex_ret
      integer, intent(in)  :: p1, p2, p3, p4
      integer(kind=8)      :: config
      integer              :: address
      logical              :: found_vertex

      call createParticleConfig(p1, p2, p3, p4, config)
      call dict(config, 1, address, found_vertex, 0)
      if(found_vertex) then
        vertex_ret = dict_get_value(address)
      else
        vertex_ret = -1
      end if
    end subroutine get_vertex4_mdl

    subroutine get_vertexN_mdl(particles, particleOut, vertex_ret)
      integer, intent(out)              :: vertex_ret
      integer, dimension(:), intent(in) :: particles
      integer, intent(in)               :: particleOut
      integer(kind=8)                   :: config
      integer                           :: address
      logical                           :: found_vertex

      call createParticleConfig(particles, particleOut, config)
      call dict(config, 1, address, found_vertex, 0)
      if(found_vertex) then
        vertex_ret = dict_get_value(address)
      else
        vertex_ret = -1
      end if

    end subroutine get_vertexN_mdl

    subroutine av(vid,bid,cid,cfid,pext,lid,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branch(bid)%color_id = cid
      vertices(vid)%branch(bid)%colorfac_id = cfid
      vertices(vid)%branch(bid)%pext = pext
      vertices(vid)%branch(bid)%lorentz_id = lid
      vertices(vid)%branch(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branch(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branch(bid)%couplings(i) = cop(i)
      end do
    end subroutine av

    subroutine avCT(vid,bid,cid,cfid,pext,lid,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchCT(bid)%color_id = cid
      vertices(vid)%branchCT(bid)%colorfac_id = cfid
      vertices(vid)%branchCT(bid)%pext = pext
      vertices(vid)%branchCT(bid)%lorentz_id = lid
      vertices(vid)%branchCT(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchCT(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchCT(bid)%couplings(i) = cop(i)
      end do
    end subroutine avCT

    subroutine avR2(vid,bid,cid,cfid,pext,lid,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchR2(bid)%color_id = cid
      vertices(vid)%branchR2(bid)%colorfac_id = cfid
      vertices(vid)%branchR2(bid)%pext = pext
      vertices(vid)%branchR2(bid)%lorentz_id = lid
      vertices(vid)%branchR2(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchR2(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchR2(bid)%couplings(i) = cop(i)
      end do
    end subroutine avR2

    function get_colourfac_mdl(cfid)
      integer, intent(in) :: cfid
      complex(dp) :: get_colourfac_mdl
      get_colourfac_mdl = colorcoeffs(cfid)
    end function get_colourfac_mdl

    function get_colourfac_form_mdl(cfid)
      integer, intent(in) :: cfid
      character(len=20) :: get_colourfac_form_mdl
      get_colourfac_form_mdl = colorcoeffs_form(cfid)
    end function get_colourfac_form_mdl

  function get_recola_base_mdl(l_id) result(ret_rb)
      integer, intent(in) :: l_id
      integer :: ret_rb
      ret_rb = recola_base(l_id)
    end function get_recola_base_mdl

  subroutine get_vertex_branch_status_mdl(vx,xlp,branch_exists)
    integer, intent(in)  :: vx,xlp
    logical, intent(out) ::branch_exists
    select case (xlp)
    case(TreeBranch:LoopBranch)
      branch_exists = allocated(vertices(vx)%branch)
    case(CTBranch)
      branch_exists = allocated(vertices(vx)%branchCT)
    case(R2Branch)
      branch_exists = allocated(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_branch_status_mdl

  subroutine get_vertex_colour_branches_mdl(vx,xlp,cfids,nbranches)
    integer, allocatable, intent(out) :: cfids(:)
    integer, intent(out) :: nbranches
    integer, intent(in)  :: vx,xlp
    select case (xlp)
    case (TreeBranch:LoopBranch)
      allocate(cfids(size(vertices(vx)%branch)))
      cfids(:) = vertices(vx)%branch(0:)%color_id
      nbranches = size(vertices(vx)%branch)
    case(CTBranch)
      allocate(cfids(size(vertices(vx)%branchCT)))
      cfids(:) = vertices(vx)%branchCT(0:)%color_id
      nbranches = size(vertices(vx)%branchCT)
    case(R2Branch)
      allocate(cfids(size(vertices(vx)%branchR2)))
      cfids(:) = vertices(vx)%branchR2(0:)%color_id
      nbranches = size(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_colour_branches_mdl

  subroutine get_vertex_pext_mdl(vx,xlp,br,pext)
    integer, intent(in) :: vx,xlp,br
    integer, intent(out) :: pext
    select case (xlp)
    case (TreeBranch:LoopBranch)
      pext = vertices(vx)%branch(br)%pext
    case(CTBranch)
      pext = vertices(vx)%branchCT(br)%pext
    case(R2Branch)
      pext = vertices(vx)%branchR2(br)%pext
    end select
  end subroutine get_vertex_pext_mdl

  subroutine get_vertex_order_increase_mdl(vx,xlp,br,orderInc)
    integer, intent(in) :: vx,xlp,br
    integer, dimension(:), intent(out) :: orderInc
    select case (xlp)
    case (TreeBranch:LoopBranch)
      orderInc = vertices(vx)%branch(br)%coupling_order
    case(CTBranch)
      orderInc = vertices(vx)%branchCT(br)%coupling_order
    case(R2Branch)
      orderInc = vertices(vx)%branchR2(br)%coupling_order
    end select
  end subroutine get_vertex_order_increase_mdl

  subroutine get_vertex_lorentz_type_mdl(vx,xlp,br,lid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: lid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      lid = vertices(vx)%branch(br)%lorentz_id
    case(CTBranch)
      lid = vertices(vx)%branchCT(br)%lorentz_id
    case(R2Branch)
      lid = vertices(vx)%branchR2(br)%lorentz_id
    end select
  end subroutine get_vertex_lorentz_type_mdl

  subroutine get_vertex_colour_id_mdl(vx,xlp,br,cid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      cid = vertices(vx)%branch(br)%colorfac_id
    case(CTBranch)
      cid = vertices(vx)%branchCT(br)%colorfac_id
    case(R2Branch)
      cid = vertices(vx)%branchR2(br)%colorfac_id
    end select
  end subroutine get_vertex_colour_id_mdl

  function is_zero_coupling_vertex_mdl(vx,xlp,br) result(zero)
    use class_couplings, only: get_coupling_value_mdl
    use input_mdl, only: zerocut
    integer, intent(in)  :: vx,xlp,br
    logical              :: zero
    real(dp)             :: maxvalue
    integer              :: i

    zero = .false.
    select case (xlp)
    case (TreeBranch:LoopBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branch(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branch(br)%couplings(i-&
                1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(CTBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchCT(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchCT(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(R2Branch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchR2(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchR2(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    end select
  end function is_zero_coupling_vertex_mdl

  subroutine get_vertex_ncouplings_mdl(vx,xlp,br,ncouplings)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: ncouplings
    select case(xlp)
    case(TreeBranch, LoopBranch)
      ncouplings = size(vertices(vx)%branch(br)%couplings)
    case (CTBranch)
      ncouplings = size(vertices(vx)%branchCT(br)%couplings)
    case (R2Branch)
      ncouplings = size(vertices(vx)%branchR2(br)%couplings)
    end select
  end subroutine get_vertex_ncouplings_mdl

  subroutine get_vertex_coupling_ids_mdl(vx,xlp,br,cids)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cids(:)
    select case(xlp)
    case(TreeBranch, LoopBranch)
      cids(:) = vertices(vx)%branch(br)%couplings(:)
    case (CTBranch)
      cids(:) = vertices(vx)%branchCT(br)%couplings(:)
    case (R2Branch)
      cids(:) = vertices(vx)%branchR2(br)%couplings(:)
    end select
  end subroutine get_vertex_coupling_ids_mdl

    subroutine createCurrent2(P1, P2, id)
      integer, intent(in) :: P1, P2, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent2

    subroutine createCurrent3(P1, P2, P3, id)
      integer, intent(in) :: P1, P2, P3, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, P3, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent3

    subroutine createCurrent4(P1, P2, P3, P4, id)
      integer, intent(in) :: P1, P2, P3, P4, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent4

    subroutine createCurrent5(P1, P2, P3, P4, P5, id)
      integer, intent(in) :: P1, P2, P3, P4, P5, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent5

    subroutine createCurrent6(P1, P2, P3, P4, P5, P6, id)
      integer, intent(in) :: P1, P2, P3, P4, P5, P6, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, P6, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent6

    subroutine createCurrent7(P1, P2, P3, P4, P5, P6, P7, id)
      integer, intent(in) :: P1, P2, P3, P4, P5, P6, P7, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, P6, P7, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent7

    subroutine createCurrent8(P1, P2, P3, P4, P5, P6, P7, P8, id)
      integer, intent(in) :: P1, P2, P3, P4, P5, P6, P7, P8, id
      integer(kind=8) :: key
      integer :: address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, P6, P7, P8, key)
      call dict(key, 2, address, found, id)
    end subroutine createCurrent8

    subroutine init_vertices_mdl()
      use constants_mdl
      use input_mdl, only: nx, error_mdl
      integer :: address
      integer :: key
      logical :: found

      ! check if modelfile is compatible to drivers
      if (model_timestamp .ne. driver_timestamp) then
        call error_mdl("Model corrupted. Driver timestamp mismatch.", &
                      where='init_vertices')
      end if

      vertices_init = .true.
      call fill_couplings_mdl
      call init_CTparameters_mdl(.true.)
      call fill_CTcouplings_mdl
      call dict_init(21563)

      call createCurrent(P_ghWp, P_Hh, P_ghZAnti, P_HQMinus, 2208)
      call createCurrent(P_GMinus, P_G0, P_A, P_WPlus, 8258)
      call createCurrent(P_ghAAnti, P_ghZ, P_WMinus, P_WPlus, 7529)
      call createCurrent(P_GPlus, P_b, P_tAnti, 913)
      call createCurrent(P_ghZ, P_Ha, P_ghWmAnti, P_HMinus, 2667)
      call createCurrent(P_Z, P_ghWpAnti, P_ghWp, P_Z, 6486)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_G0, 10298)
      call createCurrent(P_WQPlus, P_WPlus, P_ghWpAnti, P_ghWm, 8333)
      call createCurrent(P_HQMinus, P_Hh, P_GPlus, P_HlQ, 5677)
      call createCurrent(P_Hl, P_Ha, P_A, 9346)
      call createCurrent(P_ghWm, P_Hh, P_ghWmAnti, P_Hl, 9133)
      call createCurrent(P_GQ0, P_Hh, P_G0, P_HlQ, 8535)
      call createCurrent(P_ghZAnti, P_Hh, P_ghZ, P_Hh, 1050)
      call createCurrent(P_HPlus, P_A, P_G0, P_WMinus, 10132)
      call createCurrent(P_WQMinus, P_Hl, P_Z, P_GQPlus, 8208)
      call createCurrent(P_HMinus, P_Ha, P_GPlus, 6208)
      call createCurrent(P_ghZAnti, P_G0, P_GPlus, P_ghWm, 3868)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghZ, P_Hh, 4522)
      call createCurrent(P_GPlus, P_ghWm, P_Ha, P_ghZAnti, 10264)
      call createCurrent(P_GPlus, P_A, P_HMinus, P_Z, 10087)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghA, P_HQPlus, 5713)
      call createCurrent(P_HMinus, P_ghWpAnti, P_HPlus, P_ghWp, 706)
      call createCurrent(P_b, P_bAnti, P_Ha, 4807)
      call createCurrent(P_GQPlus, P_WMinus, P_HhQ, 7063)
      call createCurrent(P_HQPlus, P_GMinus, P_Hh, P_HhQ, 3667)
      call createCurrent(P_G0, P_G0, P_G0, P_G0, 1022)
      call createCurrent(P_HQPlus, P_Hh, P_GMinus, P_HaQ, 3943)
      call createCurrent(P_G0, P_uAnti, P_u, 601)
      call createCurrent(P_ghZ, P_Hl, P_ghWpAnti, P_HQPlus, 3777)
      call createCurrent(P_HhQ, P_Ha, P_Ha, P_HhQ, 4261)
      call createCurrent(P_GMinus, P_GPlus, P_ghWm, P_ghWmAnti, 1941)
      call createCurrent(P_GPlus, P_GMinus, P_Ha, P_G0, 3259)
      call createCurrent(P_ghA, P_GMinus, P_GPlus, P_ghAAnti, 8715)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghZ, P_GPlus, 7989)
      call createCurrent(P_G0, P_ghWpAnti, P_ghWp, P_Ha, 9543)
      call createCurrent(P_G0, P_GPlus, P_Ha, P_GMinus, 7810)
      call createCurrent(P_A, P_A, P_ghWm, P_ghWmAnti, 6820)
      call createCurrent(P_GPlus, P_Hl, P_GMinus, P_Hl, 4497)
      call createCurrent(P_g, P_cAnti, P_c, 3128)
      call createCurrent(P_Z, P_Ha, P_Z, P_G0, 10175)
      call createCurrent(P_ghWm, P_WPlus, P_ghWpAnti, P_WQPlus, 5811)
      call createCurrent(P_HMinus, P_ghZ, P_HPlus, P_ghZAnti, 6917)
      call createCurrent(P_WQPlus, P_HMinus, P_HaQ, 6751)
      call createCurrent(P_Z, P_GPlus, P_GMinus, P_A, 4561)
      call createCurrent(P_HlQ, P_GPlus, P_HMinus, P_GQ0, 7062)
      call createCurrent(P_Z, P_Hh, P_A, P_Hh, 9544)
      call createCurrent(P_WQPlus, P_A, P_WQMinus, 4611)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghWp, P_GQ0, 8121)
      call createCurrent(P_HMinus, P_ghWp, P_Ha, P_ghZAnti, 8303)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hl, P_HQPlus, 6416)
      call createCurrent(P_WMinus, P_Ha, P_WPlus, P_Ha, 5900)
      call createCurrent(P_Hl, P_HMinus, P_ghZAnti, P_ghWp, 5065)
      call createCurrent(P_Hh, P_HMinus, P_A, P_WPlus, 8138)
      call createCurrent(P_ghA, P_Hh, P_ghWmAnti, P_HQMinus, 1534)
      call createCurrent(P_G0, P_GMinus, P_G0, P_HPlus, 2758)
      call createCurrent(P_WPlus, P_ghZ, P_WMinus, P_ghZAnti, 7650)
      call createCurrent(P_WQPlus, P_Hh, P_GMinus, P_AQ, 2787)
      call createCurrent(P_WPlus, P_Z, P_Hl, P_HMinus, 6722)
      call createCurrent(P_Ha, P_GMinus, P_GPlus, P_Hh, 9846)
      call createCurrent(P_Ha, P_HMinus, P_HPlus, 10188)
      call createCurrent(P_ghZAnti, P_ghZ, P_GMinus, P_GQPlus, 3156)
      call createCurrent(P_g, P_Hh, P_g, P_Hl, 10580)
      call createCurrent(P_WQPlus, P_Hh, P_WMinus, P_HhQ, 1146)
      call createCurrent(P_G0, P_GPlus, P_WMinus, P_A, 1965)
      call createCurrent(P_HQMinus, P_HPlus, P_AQ, 6714)
      call createCurrent(P_HaQ, P_HPlus, P_G0, P_HQMinus, 1192)
      call createCurrent(P_HhQ, P_ghWm, P_Ha, P_ghWmAnti, 900)
      call createCurrent(P_GPlus, P_Hl, P_Z, P_WMinus, 1232)
      call createCurrent(P_WQPlus, P_Hl, P_HMinus, P_ZQ, 7240)
      call createCurrent(P_tauMinus, P_tauPlus, P_Ha, 264)
      call createCurrent(P_ZQ, P_Hh, P_GPlus, P_WQMinus, 4402)
      call createCurrent(P_GQ0, P_Hl, P_ghWmAnti, P_ghWm, 2099)
      call createCurrent(P_Hh, P_Hh, P_ghWpAnti, P_ghWp, 7821)
      call createCurrent(P_HMinus, P_Z, P_HPlus, P_Z, 507)
      call createCurrent(P_Hl, P_ghWp, P_ghWpAnti, P_Hl, 69)
      call createCurrent(P_ghZAnti, P_Hh, P_ghZ, P_HhQ, 4267)
      call createCurrent(P_HaQ, P_Hh, P_GMinus, P_HQPlus, 4756)
      call createCurrent(P_Hh, P_HMinus, P_Hh, P_GPlus, 5173)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_HlQ, 2524)
      call createCurrent(P_G0, P_Hh, P_GMinus, P_GPlus, 9726)
      call createCurrent(P_ghWmAnti, P_Hh, P_HMinus, P_ghZ, 6490)
      call createCurrent(P_GPlus, P_HMinus, P_ghAAnti, P_ghZ, 10172)
      call createCurrent(P_Hh, P_HPlus, P_ghZAnti, P_ghWm, 7346)
      call createCurrent(P_Ha, P_ghZ, P_Ha, P_ghZAnti, 5345)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghA, P_HMinus, 8589)
      call createCurrent(P_HaQ, P_HMinus, P_G0, P_HQPlus, 1951)
      call createCurrent(P_GQ0, P_ghWmAnti, P_GMinus, P_ghZ, 1373)
      call createCurrent(P_A, P_Hl, P_A, P_Hl, 10520)
      call createCurrent(P_Hh, P_G0, P_Hl, P_Ha, 5780)
      call createCurrent(P_ghZ, P_WMinus, P_ghWmAnti, P_A, 1774)
      call createCurrent(P_WPlus, P_GMinus, P_Z, P_Ha, 9355)
      call createCurrent(P_HQMinus, P_ghWp, P_Ha, P_ghZAnti, 4087)
      call createCurrent(P_GMinus, P_GPlus, P_Hl, P_Hl, 5295)
      call createCurrent(P_HQMinus, P_ghWp, P_HPlus, P_ghWpAnti, 5943)
      call createCurrent(P_A, P_ghWm, P_ghWmAnti, 7174)
      call createCurrent(P_WPlus, P_ghZAnti, P_WMinus, P_ghA, 1248)
      call createCurrent(P_Hh, P_HPlus, P_Hl, P_HMinus, 302)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghA, P_G0, 8818)
      call createCurrent(P_HQPlus, P_ghZ, P_Hl, P_ghWpAnti, 558)
      call createCurrent(P_HMinus, P_ghA, P_GPlus, P_ghAAnti, 9318)
      call createCurrent(P_ghWpAnti, P_A, P_ghA, P_WPlus, 5994)
      call createCurrent(P_ghWm, P_ghZAnti, P_HQPlus, 8141)
      call createCurrent(P_ghZAnti, P_G0, P_ghWp, P_GMinus, 4301)
      call createCurrent(P_HaQ, P_Ha, P_ghWmAnti, P_ghWm, 1964)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_Hl, 2795)
      call createCurrent(P_Z, P_ePlus, P_eMinus, 7044)
      call createCurrent(P_HQPlus, P_HMinus, P_AQ, 2302)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Z, P_AQ, 2475)
      call createCurrent(P_G0, P_Hl, P_Ha, P_Hh, 5926)
      call createCurrent(P_G0, P_HMinus, P_Hl, P_GPlus, 6446)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_Hl, 7132)
      call createCurrent(P_GQMinus, P_G0, P_Z, P_WQPlus, 6492)
      call createCurrent(P_ZQ, P_WMinus, P_ghZAnti, P_ghWp, 6769)
      call createCurrent(P_A, P_ghWmAnti, P_WMinus, P_ghA, 3733)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hl, P_GPlus, 8406)
      call createCurrent(P_ghZ, P_ghZAnti, P_Ha, P_Ha, 8593)
      call createCurrent(P_HhQ, P_ghWp, P_HMinus, P_ghZAnti, 298)
      call createCurrent(P_ghWm, P_Ha, P_ghZAnti, P_HQPlus, 945)
      call createCurrent(P_ghWmAnti, P_ghWm, P_GMinus, P_GQPlus, 970)
      call createCurrent(P_HQPlus, P_HMinus, P_HlQ, 2881)
      call createCurrent(P_ghZ, P_ghWpAnti, P_A, P_WQPlus, 6716)
      call createCurrent(P_Hl, P_GPlus, P_A, P_WMinus, 2742)
      call createCurrent(P_ghWm, P_ghWmAnti, P_WMinus, P_WQPlus, 6996)
      call createCurrent(P_Hl, P_ghZ, P_GMinus, P_ghWmAnti, 4275)
      call createCurrent(P_GPlus, P_HPlus, P_GMinus, P_HMinus, 1539)
      call createCurrent(P_Hh, P_Hh, P_GMinus, P_GPlus, 8557)
      call createCurrent(P_ghZAnti, P_Hl, P_ghZ, P_Hh, 9088)
      call createCurrent(P_GQ0, P_GMinus, P_ghWp, P_ghAAnti, 716)
      call createCurrent(P_Ha, P_Hh, P_G0, P_Hl, 2939)
      call createCurrent(P_HhQ, P_GMinus, P_Hh, P_HQPlus, 8806)
      call createCurrent(P_A, P_Z, P_A, P_Z, 10639)
      call createCurrent(P_GQ0, P_ghWpAnti, P_ghWp, 4291)
      call createCurrent(P_Hh, P_ghWpAnti, P_GPlus, P_ghA, 734)
      call createCurrent(P_ghA, P_WMinus, P_ghWmAnti, P_A, 8528)
      call createCurrent(P_HQPlus, P_Hh, P_GMinus, P_HlQ, 8399)
      call createCurrent(P_Z, P_ghZAnti, P_WMinus, P_ghWp, 6625)
      call createCurrent(P_WQMinus, P_WPlus, P_HaQ, 3334)
      call createCurrent(P_A, P_Hl, P_WMinus, P_HPlus, 1515)
      call createCurrent(P_Hh, P_Hh, P_HMinus, P_GPlus, 7572)
      call createCurrent(P_HhQ, P_GMinus, P_HQPlus, 5040)
      call createCurrent(P_nu_mu, P_nu_muAnti, P_Z, 4899)
      call createCurrent(P_ZQ, P_WPlus, P_Ha, P_HQMinus, 6031)
      call createCurrent(P_ghA, P_Ha, P_ghWmAnti, P_GMinus, 9578)
      call createCurrent(P_AQ, P_A, P_ghWp, P_ghWpAnti, 2240)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, 400)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghWm, P_GPlus, 6426)
      call createCurrent(P_ZQ, P_ghWm, P_Z, P_ghWmAnti, 7517)
      call createCurrent(P_HQMinus, P_HPlus, P_HlQ, 1145)
      call createCurrent(P_GMinus, P_WPlus, P_Z, P_Hh, 3217)
      call createCurrent(P_g, P_WPlus, P_WMinus, P_g, 10518)
      call createCurrent(P_g, P_A, P_g, P_Z, 10653)
      call createCurrent(P_u, P_uAnti, P_Hh, 8757)
      call createCurrent(P_Hl, P_G0, P_Ha, P_Hh, 8429)
      call createCurrent(P_Hh, P_ghWp, P_ghWpAnti, 5724)
      call createCurrent(P_GQMinus, P_HPlus, P_G0, P_HhQ, 8620)
      call createCurrent(P_WMinus, P_GMinus, P_HPlus, P_WPlus, 10419)
      call createCurrent(P_GMinus, P_GPlus, P_Hl, P_Ha, 9178)
      call createCurrent(P_HhQ, P_u, P_uAnti, 4298)
      call createCurrent(P_GQMinus, P_Z, P_Z, P_GQPlus, 4100)
      call createCurrent(P_HQPlus, P_Hl, P_ghWpAnti, P_ghA, 3366)
      call createCurrent(P_GQPlus, P_Hh, P_GMinus, P_HhQ, 6396)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_G0, 2094)
      call createCurrent(P_ghG, P_g, P_ghGAnti, P_g, 3615)
      call createCurrent(P_GPlus, P_ghA, P_G0, P_ghWpAnti, 8793)
      call createCurrent(P_Z, P_HMinus, P_Z, P_GPlus, 8870)
      call createCurrent(P_Ha, P_WPlus, P_HMinus, 3555)
      call createCurrent(P_WMinus, P_HPlus, P_Ha, P_Z, 3831)
      call createCurrent(P_GMinus, P_ghZ, P_HPlus, P_ghZAnti, 9755)
      call createCurrent(P_tauMinus, P_tauPlus, P_Hh, 2887)
      call createCurrent(P_GQPlus, P_Ha, P_HMinus, P_HaQ, 3811)
      call createCurrent(P_ghZ, P_Ha, P_ghWmAnti, P_GMinus, 9806)
      call createCurrent(P_HQMinus, P_HPlus, P_GMinus, P_HQPlus, 7720)
      call createCurrent(P_Ha, P_G0, P_ghWp, P_ghWpAnti, 9717)
      call createCurrent(P_GPlus, P_nu_muAnti, P_muMinus, 3528)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_Ha, 9486)
      call createCurrent(P_HPlus, P_A, P_WMinus, P_G0, 8991)
      call createCurrent(P_ghAAnti, P_GPlus, P_HMinus, P_ghZ, 9329)
      call createCurrent(P_Hh, P_A, P_WPlus, P_GMinus, 6276)
      call createCurrent(P_G0, P_Ha, P_WMinus, P_WPlus, 9914)
      call createCurrent(P_nu_muAnti, P_muMinus, P_WPlus, 7505)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_Hh, P_ghZ, 3584)
      call createCurrent(P_GQMinus, P_GPlus, P_WMinus, P_WQPlus, 643)
      call createCurrent(P_AQ, P_ghZAnti, P_WMinus, P_ghWp, 4463)
      call createCurrent(P_Hl, P_Hl, P_G0, P_Ha, 4692)
      call createCurrent(P_WMinus, P_ghWp, P_A, P_ghAAnti, 94)
      call createCurrent(P_nu_e, P_HMinus, P_ePlus, 4945)
      call createCurrent(P_HQMinus, P_HPlus, P_GPlus, P_HQMinus, 5882)
      call createCurrent(P_ZQ, P_Z, P_HlQ, 933)
      call createCurrent(P_ghWp, P_GPlus, P_ghWpAnti, P_GQMinus, 8597)
      call createCurrent(P_HhQ, P_HMinus, P_WQPlus, 2515)
      call createCurrent(P_HQMinus, P_c, P_sAnti, 7851)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_Ha, 2163)
      call createCurrent(P_tAnti, P_t, P_A, 899)
      call createCurrent(P_HaQ, P_HPlus, P_GQMinus, 1012)
      call createCurrent(P_HQMinus, P_HPlus, P_Ha, P_HhQ, 5349)
      call createCurrent(P_HMinus, P_ghZ, P_ghAAnti, P_HPlus, 1799)
      call createCurrent(P_GQ0, P_WMinus, P_WPlus, P_GQ0, 6291)
      call createCurrent(P_Ha, P_HPlus, P_GMinus, P_G0, 2631)
      call createCurrent(P_Z, P_WMinus, P_Ha, P_GPlus, 9443)
      call createCurrent(P_HMinus, P_dAnti, P_u, 4413)
      call createCurrent(P_Hl, P_Z, P_Hh, P_Z, 9641)
      call createCurrent(P_ghWm, P_G0, P_GPlus, P_ghZAnti, 4579)
      call createCurrent(P_Z, P_WMinus, P_HPlus, P_Hl, 5931)
      call createCurrent(P_c, P_sAnti, P_WMinus, 6234)
      call createCurrent(P_GMinus, P_GPlus, P_ghAAnti, P_ghZ, 2615)
      call createCurrent(P_A, P_G0, P_A, P_Ha, 10712)
      call createCurrent(P_ghWm, P_ghAAnti, P_WQPlus, 104)
      call createCurrent(P_WPlus, P_ghZ, P_ghWpAnti, 7217)
      call createCurrent(P_GQ0, P_GPlus, P_G0, P_HQMinus, 5261)
      call createCurrent(P_HPlus, P_ghWm, P_GMinus, P_ghWmAnti, 9580)
      call createCurrent(P_HPlus, P_Ha, P_Hl, P_HMinus, 9620)
      call createCurrent(P_GQPlus, P_Hl, P_ghWpAnti, P_ghA, 728)
      call createCurrent(P_ghWmAnti, P_G0, P_ghWm, P_GQ0, 7646)
      call createCurrent(P_HPlus, P_GMinus, P_GMinus, P_GPlus, 8079)
      call createCurrent(P_GPlus, P_Hh, P_G0, P_GMinus, 10027)
      call createCurrent(P_G0, P_GMinus, P_ghA, P_ghWmAnti, 954)
      call createCurrent(P_HlQ, P_Hl, P_GMinus, P_GQPlus, 3527)
      call createCurrent(P_ghZ, P_Ha, P_ghWpAnti, P_HQPlus, 4219)
      call createCurrent(P_WMinus, P_ghWp, P_ghZAnti, P_A, 2627)
      call createCurrent(P_HPlus, P_GPlus, P_HMinus, P_GMinus, 3523)
      call createCurrent(P_WQPlus, P_G0, P_GQMinus, 6513)
      call createCurrent(P_WPlus, P_Z, P_ghZAnti, P_ghWm, 5352)
      call createCurrent(P_ZQ, P_WMinus, P_A, P_WQPlus, 7236)
      call createCurrent(P_cAnti, P_c, P_gQ, 6819)
      call createCurrent(P_bAnti, P_t, P_GQMinus, 7798)
      call createCurrent(P_ghWpAnti, P_ghZ, P_A, P_WQPlus, 2555)
      call createCurrent(P_WPlus, P_WMinus, P_G0, P_Ha, 9715)
      call createCurrent(P_HhQ, P_ghWp, P_GMinus, P_ghZAnti, 8614)
      call createCurrent(P_HPlus, P_HMinus, P_GPlus, P_HMinus, 8251)
      call createCurrent(P_Hh, P_Ha, P_Hl, P_Ha, 3399)
      call createCurrent(P_Ha, P_G0, P_Hl, P_G0, 10547)
      call createCurrent(P_Hh, P_A, P_A, P_Hh, 10743)
      call createCurrent(P_HlQ, P_Hl, P_Hh, P_HhQ, 3769)
      call createCurrent(P_AQ, P_c, P_cAnti, 8278)
      call createCurrent(P_GMinus, P_ghAAnti, P_HPlus, P_ghZ, 9365)
      call createCurrent(P_HMinus, P_GPlus, P_ghZAnti, P_ghZ, 9535)
      call createCurrent(P_HlQ, P_HMinus, P_ghWmAnti, P_ghZ, 1045)
      call createCurrent(P_HMinus, P_HPlus, P_Ha, P_Hh, 9194)
      call createCurrent(P_HhQ, P_HPlus, P_Hl, P_GQMinus, 6081)
      call createCurrent(P_GQ0, P_Hl, P_G0, P_HlQ, 6560)
      call createCurrent(P_HaQ, P_Hh, P_GMinus, P_GQPlus, 1208)
      call createCurrent(P_HMinus, P_G0, P_ghWmAnti, P_ghZ, 9345)
      call createCurrent(P_WMinus, P_A, P_GPlus, P_G0, 5235)
      call createCurrent(P_Z, P_G0, P_Ha, P_A, 10595)
      call createCurrent(P_ghWmAnti, P_ghZ, P_WMinus, P_A, 4911)
      call createCurrent(P_Z, P_A, P_Hl, 9283)
      call createCurrent(P_AQ, P_GPlus, P_A, P_GQMinus, 6405)
      call createCurrent(P_Hh, P_Ha, P_GMinus, P_HPlus, 7573)
      call createCurrent(P_u, P_uAnti, P_HhQ, 1758)
      call createCurrent(P_ghA, P_ghZAnti, P_WMinus, P_WPlus, 2181)
      call createCurrent(P_WQMinus, P_HPlus, P_Z, P_HlQ, 4688)
      call createCurrent(P_WQPlus, P_Hl, P_GMinus, P_AQ, 5506)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_Ha, 10301)
      call createCurrent(P_Hh, P_HMinus, P_HPlus, P_Hh, 8085)
      call createCurrent(P_HhQ, P_Hh, P_ghWmAnti, P_ghWm, 5277)
      call createCurrent(P_u, P_uAnti, P_Ha, 8257)
      call createCurrent(P_Z, P_HMinus, P_A, P_GPlus, 9206)
      call createCurrent(P_HPlus, P_ghZAnti, P_HMinus, P_ghA, 6425)
      call createCurrent(P_ghA, P_Hl, P_GPlus, P_ghWpAnti, 4585)
      call createCurrent(P_HMinus, P_WPlus, P_Hl, P_Z, 8744)
      call createCurrent(P_ghWp, P_G0, P_HMinus, P_ghAAnti, 8921)
      call createCurrent(P_HlQ, P_Ha, P_HPlus, P_GQMinus, 1030)
      call createCurrent(P_HQMinus, P_ghWp, P_Hh, P_ghZAnti, 7017)
      call createCurrent(P_ghZ, P_Hl, P_ghZAnti, 704)
      call createCurrent(P_GPlus, P_Hl, P_Hl, P_GMinus, 1745)
      call createCurrent(P_HhQ, P_G0, P_Hh, P_GQ0, 8805)
      call createCurrent(P_WMinus, P_Z, P_HPlus, P_Hl, 8641)
      call createCurrent(P_tAnti, P_t, P_Z, 4683)
      call createCurrent(P_ghWmAnti, P_Z, P_A, P_ghWm, 32)
      call createCurrent(P_GMinus, P_A, P_Hl, P_WPlus, 1093)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghA, P_HMinus, 9794)
      call createCurrent(P_ghWp, P_ghWpAnti, P_GPlus, P_HMinus, 10062)
      call createCurrent(P_Hl, P_Ha, P_Ha, 500)
      call createCurrent(P_WPlus, P_A, P_ghZ, P_ghWpAnti, 6997)
      call createCurrent(P_HMinus, P_WPlus, P_A, P_Hl, 4510)
      call createCurrent(P_WQMinus, P_ghAAnti, P_ghWp, 8816)
      call createCurrent(P_ghGAnti, P_g, P_ghG, P_gQ, 6677)
      call createCurrent(P_WQPlus, P_Hh, P_Z, P_GQMinus, 5019)
      call createCurrent(P_HPlus, P_Hl, P_Ha, P_HMinus, 9298)
      call createCurrent(P_ghWmAnti, P_ghA, P_HMinus, 1870)
      call createCurrent(P_Z, P_Ha, P_Z, P_Ha, 6916)
      call createCurrent(P_HQPlus, P_Z, P_WQMinus, 4872)
      call createCurrent(P_WQPlus, P_ghZ, P_ghWpAnti, 4743)
      call createCurrent(P_GQPlus, P_G0, P_Z, P_WQMinus, 8295)
      call createCurrent(P_Hl, P_HPlus, P_WMinus, P_Z, 6423)
      call createCurrent(P_ghZ, P_GPlus, P_HMinus, P_ghAAnti, 9932)
      call createCurrent(P_ghAAnti, P_GPlus, P_Ha, P_ghWm, 8928)
      call createCurrent(P_WQPlus, P_GMinus, P_Hl, P_ZQ, 4472)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_Hh, 2404)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hl, P_GQPlus, 2467)
      call createCurrent(P_Hh, P_Ha, P_GPlus, P_HMinus, 2125)
      call createCurrent(P_c, P_cAnti, P_G0, 8098)
      call createCurrent(P_HlQ, P_HPlus, P_Z, P_WQMinus, 6077)
      call createCurrent(P_Ha, P_ghWpAnti, P_G0, P_ghWp, 8873)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghZ, P_GPlus, 7489)
      call createCurrent(P_HhQ, P_GPlus, P_ghZ, P_ghWpAnti, 3675)
      call createCurrent(P_WPlus, P_A, P_G0, P_HMinus, 9783)
      call createCurrent(P_ghAAnti, P_HPlus, P_GMinus, P_ghZ, 9515)
      call createCurrent(P_HaQ, P_ghWp, P_HMinus, P_ghAAnti, 4363)
      call createCurrent(P_Ha, P_Ha, P_HPlus, P_GMinus, 217)
      call createCurrent(P_GMinus, P_ghWm, P_HPlus, P_ghWmAnti, 8964)
      call createCurrent(P_HaQ, P_ghAAnti, P_HPlus, P_ghWm, 6489)
      call createCurrent(P_HQPlus, P_WMinus, P_WPlus, P_HQMinus, 2461)
      call createCurrent(P_WPlus, P_GMinus, P_G0, 904)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_A, P_ghA, 6939)
      call createCurrent(P_dAnti, P_d, P_Ha, 6275)
      call createCurrent(P_GMinus, P_ghZ, P_ghWmAnti, P_Hh, 3077)
      call createCurrent(P_dAnti, P_d, P_g, 2176)
      call createCurrent(P_ghZ, P_ghWpAnti, P_G0, P_GQPlus, 242)
      call createCurrent(P_HPlus, P_GMinus, P_ghA, P_ghZAnti, 10347)
      call createCurrent(P_A, P_ghWp, P_Z, P_ghWpAnti, 4129)
      call createCurrent(P_GMinus, P_HPlus, P_Hh, P_Hl, 791)
      call createCurrent(P_ghWm, P_GPlus, P_Hh, P_ghAAnti, 2635)
      call createCurrent(P_Z, P_ghWpAnti, P_WPlus, P_ghZ, 2017)
      call createCurrent(P_Z, P_Hl, P_Z, 3685)
      call createCurrent(P_ghWpAnti, P_ghWp, P_GQ0, 8137)
      call createCurrent(P_Hl, P_A, P_A, P_Hh, 10730)
      call createCurrent(P_Hh, P_GPlus, P_WMinus, P_A, 7856)
      call createCurrent(P_GMinus, P_A, P_A, P_GPlus, 7859)
      call createCurrent(P_HMinus, P_Ha, P_HPlus, 10291)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_WMinus, P_ghWp, 5845)
      call createCurrent(P_muPlus, P_muMinus, P_A, 5541)
      call createCurrent(P_GQPlus, P_GMinus, P_GPlus, P_HQMinus, 8225)
      call createCurrent(P_HaQ, P_Hh, P_WMinus, P_WQPlus, 702)
      call createCurrent(P_HMinus, P_ghWp, P_Hh, P_ghZAnti, 1161)
      call createCurrent(P_ghWm, P_ghWmAnti, P_ZQ, 7208)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_Hl, 3201)
      call createCurrent(P_Hh, P_GMinus, P_HPlus, P_Ha, 1948)
      call createCurrent(P_HhQ, P_HMinus, P_ghWp, P_ghAAnti, 5392)
      call createCurrent(P_dAnti, P_u, P_HQMinus, 879)
      call createCurrent(P_Hh, P_tAnti, P_t, 6718)
      call createCurrent(P_A, P_Hh, P_A, P_Hh, 10569)
      call createCurrent(P_GMinus, P_ghWmAnti, P_G0, P_ghZ, 5247)
      call createCurrent(P_HaQ, P_muPlus, P_muMinus, 6752)
      call createCurrent(P_GQPlus, P_G0, P_G0, P_HQMinus, 7038)
      call createCurrent(P_d, P_dAnti, 9202)
      call createCurrent(P_u, P_dAnti, P_GMinus, 7269)
      call createCurrent(P_Z, P_WMinus, P_ghWp, P_ghAAnti, 6982)
      call createCurrent(P_HaQ, P_GPlus, P_Hl, P_HQMinus, 1339)
      call createCurrent(P_t, P_Hh, P_tAnti, 128)
      call createCurrent(P_G0, P_ghWmAnti, P_ghA, P_HMinus, 8980)
      call createCurrent(P_nu_e, P_ePlus, P_WMinus, 748)
      call createCurrent(P_GPlus, P_Z, P_Ha, P_WMinus, 9464)
      call createCurrent(P_WQMinus, P_A, P_A, P_WQPlus, 1362)
      call createCurrent(P_sAnti, P_s, P_Hl, 5068)
      call createCurrent(P_ghA, P_HMinus, P_ghZAnti, P_HQPlus, 7284)
      call createCurrent(P_GQPlus, P_HMinus, P_GMinus, P_GQPlus, 6663)
      call createCurrent(P_Z, P_tauPlus, P_tauMinus, 3229)
      call createCurrent(P_G0, P_HPlus, P_GMinus, P_Ha, 3235)
      call createCurrent(P_AQ, P_GPlus, P_WQMinus, 8400)
      call createCurrent(P_HlQ, P_ghAAnti, P_GPlus, P_ghWm, 2497)
      call createCurrent(P_WMinus, P_ghWm, P_ghWmAnti, P_WPlus, 7878)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_Ha, 9324)
      call createCurrent(P_HaQ, P_G0, P_Ha, P_GQ0, 721)
      call createCurrent(P_ghWm, P_ghZAnti, P_Ha, P_HPlus, 7509)
      call createCurrent(P_HlQ, P_WPlus, P_G0, P_WQMinus, 7963)
      call createCurrent(P_GQ0, P_GPlus, P_ghWm, P_ghAAnti, 3875)
      call createCurrent(P_HQPlus, P_ghA, P_HMinus, P_ghAAnti, 7437)
      call createCurrent(P_bAnti, P_G0, P_b, 7899)
      call createCurrent(P_WMinus, P_ghZAnti, P_A, P_ghWp, 7536)
      call createCurrent(P_GQ0, P_Hh, P_Hl, P_GQ0, 6089)
      call createCurrent(P_Hh, P_Hl, P_Ha, P_G0, 2966)
      call createCurrent(P_HhQ, P_ghZ, P_HMinus, P_ghWmAnti, 5218)
      call createCurrent(P_GMinus, P_Z, P_HPlus, 9603)
      call createCurrent(P_GQPlus, P_HMinus, P_GMinus, P_HQPlus, 7188)
      call createCurrent(P_Hl, P_Ha, P_GMinus, P_GPlus, 8954)
      call createCurrent(P_GPlus, P_G0, P_GMinus, P_G0, 2937)
      call createCurrent(P_A, P_HPlus, P_HMinus, P_Z, 8207)
      call createCurrent(P_GQPlus, P_nu_tauAnti, P_tauMinus, 7746)
      call createCurrent(P_HhQ, P_Hh, P_GPlus, P_GQMinus, 5573)
      call createCurrent(P_HlQ, P_cAnti, P_c, 6135)
      call createCurrent(P_HQMinus, P_dAnti, P_u, 4823)
      call createCurrent(P_HMinus, P_WPlus, P_HPlus, P_WMinus, 216)
      call createCurrent(P_ghWp, P_ghWpAnti, P_G0, P_G0, 1011)
      call createCurrent(P_GPlus, P_Ha, P_Hl, P_GMinus, 10185)
      call createCurrent(P_ghA, P_ghWpAnti, P_HPlus, P_G0, 8859)
      call createCurrent(P_ZQ, P_WPlus, P_Hh, P_HQMinus, 7952)
      call createCurrent(P_WQPlus, P_Z, P_Z, P_WQMinus, 1450)
      call createCurrent(P_G0, P_tauPlus, P_tauMinus, 8430)
      call createCurrent(P_G0, P_g, P_G0, P_g, 10618)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghWm, P_Hh, 8745)
      call createCurrent(P_HPlus, P_GMinus, P_GPlus, P_GMinus, 7819)
      call createCurrent(P_tAnti, P_Ha, P_t, 8127)
      call createCurrent(P_WQPlus, P_HMinus, P_A, P_HhQ, 8656)
      call createCurrent(P_HQPlus, P_G0, P_Ha, P_GQMinus, 4372)
      call createCurrent(P_WPlus, P_ghAAnti, P_ghWm, P_A, 2037)
      call createCurrent(P_HPlus, P_GMinus, P_ghWmAnti, P_ghWm, 9317)
      call createCurrent(P_GQMinus, P_GPlus, P_Z, P_AQ, 4878)
      call createCurrent(P_ghWm, P_G0, P_ghWmAnti, P_HhQ, 4984)
      call createCurrent(P_A, P_HPlus, P_WMinus, P_G0, 9097)
      call createCurrent(P_Hl, P_A, P_G0, 9258)
      call createCurrent(P_GQMinus, P_G0, P_GPlus, P_GQ0, 5182)
      call createCurrent(P_ghA, P_Hh, P_ghWpAnti, P_GPlus, 8743)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_Hl, P_ghA, 5177)
      call createCurrent(P_Ha, P_GPlus, P_GMinus, P_Ha, 2853)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hh, P_GQ0, 5722)
      call createCurrent(P_ghWp, P_ghWpAnti, P_GMinus, P_GPlus, 7914)
      call createCurrent(P_Hh, P_GPlus, P_Hh, P_HMinus, 870)
      call createCurrent(P_HaQ, P_ghZ, P_HMinus, P_ghWmAnti, 131)
      call createCurrent(P_HQPlus, P_Hl, P_WQMinus, 8345)
      call createCurrent(P_Ha, P_ghA, P_HMinus, P_ghWmAnti, 4257)
      call createCurrent(P_Z, P_ghWp, P_A, P_ghWpAnti, 5272)
      call createCurrent(P_ghWmAnti, P_ghA, P_GMinus, P_Ha, 10471)
      call createCurrent(P_Z, P_HPlus, P_HMinus, P_A, 2570)
      call createCurrent(P_A, P_A, P_GMinus, P_GPlus, 4064)
      call createCurrent(P_muPlus, P_muMinus, P_Hl, 6157)
      call createCurrent(P_GQ0, P_HPlus, P_HQMinus, 6238)
      call createCurrent(P_HPlus, P_G0, P_Hl, P_HMinus, 9789)
      call createCurrent(P_HaQ, P_Z, P_Ha, P_ZQ, 6245)
      call createCurrent(P_WMinus, P_A, P_G0, P_GPlus, 3377)
      call createCurrent(P_GPlus, P_A, P_A, P_GMinus, 48)
      call createCurrent(P_G0, P_ghWm, P_ghZAnti, P_HPlus, 9043)
      call createCurrent(P_ghWm, P_G0, P_G0, P_ghWmAnti, 7019)
      call createCurrent(P_HhQ, P_HPlus, P_Hl, P_HQMinus, 2064)
      call createCurrent(P_GMinus, P_GPlus, P_Hh, P_Hl, 8002)
      call createCurrent(P_Ha, P_WPlus, P_Ha, P_WMinus, 3177)
      call createCurrent(P_GPlus, P_HMinus, P_G0, P_Hl, 7024)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghA, P_GMinus, 8902)
      call createCurrent(P_HMinus, P_GPlus, P_Hl, P_Hh, 7391)
      call createCurrent(P_Hh, P_Z, P_A, P_Hl, 10702)
      call createCurrent(P_GMinus, P_ghWp, P_ghAAnti, 6018)
      call createCurrent(P_ghWp, P_ghAAnti, P_Hl, P_GMinus, 3829)
      call createCurrent(P_ghWm, P_A, P_Z, P_ghWmAnti, 7682)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghWp, P_Hl, 979)
      call createCurrent(P_HPlus, P_WMinus, P_WPlus, P_GMinus, 9047)
      call createCurrent(P_GQMinus, P_GPlus, P_Ha, P_HlQ, 1576)
      call createCurrent(P_HlQ, P_WMinus, P_GPlus, P_ZQ, 3737)
      call createCurrent(P_ghWmAnti, P_Ha, P_Ha, P_ghWm, 2377)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWp, P_GMinus, 110)
      call createCurrent(P_GQMinus, P_GPlus, P_GPlus, P_GQMinus, 3466)
      call createCurrent(P_HQPlus, P_HMinus, P_GQ0, 6322)
      call createCurrent(P_Z, P_Hh, P_Z, 6705)
      call createCurrent(P_ghWp, P_GMinus, P_ghWpAnti, P_HPlus, 8836)
      call createCurrent(P_Z, P_Z, P_Z, P_A, 10746)
      call createCurrent(P_sAnti, P_c, P_WMinus, 3359)
      call createCurrent(P_GQPlus, P_HMinus, P_HMinus, P_GQPlus, 6415)
      call createCurrent(P_WMinus, P_ghWp, P_A, P_ghZAnti, 3677)
      call createCurrent(P_ghA, P_ghZAnti, P_GMinus, P_GPlus, 8705)
      call createCurrent(P_HQMinus, P_Ha, P_ghZAnti, P_ghWp, 56)
      call createCurrent(P_GPlus, P_ghWp, P_ghWpAnti, P_HMinus, 9971)
      call createCurrent(P_ZQ, P_Z, P_ghWpAnti, P_ghWp, 5130)
      call createCurrent(P_GQ0, P_Hl, P_GMinus, P_GQPlus, 4776)
      call createCurrent(P_ghWpAnti, P_A, P_A, P_ghWp, 6954)
      call createCurrent(P_HaQ, P_c, P_cAnti, 8070)
      call createCurrent(P_ghWm, P_HPlus, P_HMinus, P_ghWmAnti, 4640)
      call createCurrent(P_ghZ, P_Hl, P_ghZAnti, P_HlQ, 8444)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_GQ0, 4027)
      call createCurrent(P_Hl, P_HMinus, P_HPlus, 3694)
      call createCurrent(P_G0, P_Hl, P_G0, P_Hl, 1025)
      call createCurrent(P_A, P_s, P_sAnti, 4182)
      call createCurrent(P_GQMinus, P_G0, P_HPlus, P_HhQ, 7374)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghA, P_Hh, 8634)
      call createCurrent(P_ghWm, P_ghZAnti, P_WPlus, P_Z, 6253)
      call createCurrent(P_GQMinus, P_GPlus, P_HlQ, 4534)
      call createCurrent(P_ghWpAnti, P_ghA, P_HPlus, P_G0, 9899)
      call createCurrent(P_ghZ, P_ghZAnti, P_GPlus, P_GQMinus, 7520)
      call createCurrent(P_Z, P_GPlus, P_WMinus, P_Hh, 5348)
      call createCurrent(P_g, P_t, P_tAnti, 6977)
      call createCurrent(P_HPlus, P_Ha, P_WMinus, P_A, 3172)
      call createCurrent(P_HQMinus, P_HPlus, P_Z, P_AQ, 1287)
      call createCurrent(P_ghWm, P_Ha, P_ghZAnti, P_HPlus, 2609)
      call createCurrent(P_Ha, P_ghWm, P_G0, P_ghWmAnti, 10135)
      call createCurrent(P_HlQ, P_ghWpAnti, P_Ha, P_ghWp, 1016)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_HlQ, 6469)
      call createCurrent(P_Hh, P_ghA, P_ghWmAnti, P_GMinus, 2214)
      call createCurrent(P_A, P_Z, P_Z, P_A, 10504)
      call createCurrent(P_GMinus, P_Hl, P_WPlus, 2740)
      call createCurrent(P_GMinus, P_Hh, P_Hh, P_GPlus, 2698)
      call createCurrent(P_WQPlus, P_GMinus, P_A, P_GQ0, 7209)
      call createCurrent(P_HPlus, P_ghA, P_GMinus, P_ghZAnti, 9735)
      call createCurrent(P_WMinus, P_Z, P_ghAAnti, P_ghWp, 4058)
      call createCurrent(P_HMinus, P_WPlus, P_Z, 7299)
      call createCurrent(P_GQPlus, P_G0, P_ghZAnti, P_ghWm, 6647)
      call createCurrent(P_Ha, P_Ha, P_HMinus, P_GPlus, 7494)
      call createCurrent(P_HhQ, P_Hl, P_Hl, P_HhQ, 7184)
      call createCurrent(P_s, P_Ha, P_sAnti, 6191)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghWp, P_GMinus, 4278)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Z, P_A, 5623)
      call createCurrent(P_G0, P_GMinus, P_HPlus, P_Hh, 8371)
      call createCurrent(P_GQPlus, P_G0, P_HMinus, P_HhQ, 3340)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hh, P_Hh, 662)
      call createCurrent(P_HPlus, P_Ha, P_GMinus, P_Hh, 6651)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_ghA, 3530)
      call createCurrent(P_GQMinus, P_Hl, P_HPlus, P_HaQ, 3238)
      call createCurrent(P_GQ0, P_ghWm, P_Hh, P_ghWmAnti, 3329)
      call createCurrent(P_HMinus, P_HPlus, P_ghA, P_ghZAnti, 7740)
      call createCurrent(P_HMinus, P_muPlus, P_nu_mu, 3470)
      call createCurrent(P_WQPlus, P_WMinus, P_ghWm, P_ghWmAnti, 330)
      call createCurrent(P_ePlus, P_eMinus, P_G0, 2623)
      call createCurrent(P_Hh, P_Hl, P_Hh, 4459)
      call createCurrent(P_G0, P_G0, P_Z, P_A, 10349)
      call createCurrent(P_ghZ, P_WPlus, P_ghWpAnti, P_Z, 5034)
      call createCurrent(P_HhQ, P_GMinus, P_ghWp, P_ghAAnti, 3437)
      call createCurrent(P_Hh, P_GPlus, P_Ha, P_HMinus, 2468)
      call createCurrent(P_A, P_Z, P_WPlus, P_WMinus, 7537)
      call createCurrent(P_GMinus, P_nu_tau, P_tauPlus, 4208)
      call createCurrent(P_Ha, P_Ha, P_Z, P_Z, 93)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_ghZ, 6355)
      call createCurrent(P_HPlus, P_HMinus, P_A, 3610)
      call createCurrent(P_ghWm, P_ghAAnti, P_Hh, P_GPlus, 471)
      call createCurrent(P_HaQ, P_ghAAnti, P_HMinus, P_ghWp, 5622)
      call createCurrent(P_HaQ, P_s, P_sAnti, 14)
      call createCurrent(P_HQMinus, P_WPlus, P_Z, P_HhQ, 593)
      call createCurrent(P_HQPlus, P_Hh, P_ghWpAnti, P_ghZ, 3474)
      call createCurrent(P_Hh, P_ghAAnti, P_ghWp, P_GMinus, 6543)
      call createCurrent(P_ghA, P_WPlus, P_ghWpAnti, P_Z, 3307)
      call createCurrent(P_Z, P_Hh, P_Ha, 3650)
      call createCurrent(P_A, P_Ha, P_Hh, 9172)
      call createCurrent(P_HPlus, P_ghWm, P_Hl, P_ghAAnti, 3421)
      call createCurrent(P_G0, P_G0, P_Ha, 10585)
      call createCurrent(P_WQMinus, P_WPlus, P_ghA, P_ghZAnti, 6181)
      call createCurrent(P_WMinus, P_ghAAnti, P_ghWp, P_A, 1503)
      call createCurrent(P_WQMinus, P_GPlus, P_Hl, P_ZQ, 6931)
      call createCurrent(P_ghWp, P_HMinus, P_Hh, P_ghZAnti, 1488)
      call createCurrent(P_Hl, P_GPlus, P_Hh, P_GMinus, 3317)
      call createCurrent(P_G0, P_WPlus, P_Z, P_HMinus, 10323)
      call createCurrent(P_ghWm, P_Hl, P_ghWmAnti, P_GQ0, 3009)
      call createCurrent(P_HhQ, P_G0, P_G0, P_HlQ, 1702)
      call createCurrent(P_ghWmAnti, P_ghA, P_A, P_WMinus, 323)
      call createCurrent(P_HPlus, P_HMinus, P_HPlus, P_HMinus, 6594)
      call createCurrent(P_HhQ, P_G0, P_Ha, P_HhQ, 5322)
      call createCurrent(P_ghA, P_ghWmAnti, P_GMinus, P_Hh, 3870)
      call createCurrent(P_sAnti, P_s, P_G0, 3573)
      call createCurrent(P_Z, P_Z, P_HPlus, P_HMinus, 5766)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghA, P_GQMinus, 3654)
      call createCurrent(P_ghWp, P_Ha, P_ghAAnti, P_HMinus, 4559)
      call createCurrent(P_Z, P_ghZAnti, P_ghWp, P_WMinus, 18)
      call createCurrent(P_WQPlus, P_G0, P_G0, P_WQMinus, 2335)
      call createCurrent(P_b, P_Hl, P_bAnti, 7056)
      call createCurrent(P_GPlus, P_Ha, P_G0, P_GMinus, 8626)
      call createCurrent(P_G0, P_HPlus, P_WMinus, 9068)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_Hl, 3167)
      call createCurrent(P_Ha, P_ghWp, P_ghWpAnti, P_Ha, 1361)
      call createCurrent(P_GPlus, P_Z, P_Z, P_HMinus, 9686)
      call createCurrent(P_WQMinus, P_t, P_bAnti, 308)
      call createCurrent(P_ghA, P_GMinus, P_Hh, P_ghWmAnti, 2438)
      call createCurrent(P_ghZ, P_Ha, P_ghWpAnti, P_GPlus, 9327)
      call createCurrent(P_HhQ, P_GPlus, P_WQMinus, 377)
      call createCurrent(P_GQPlus, P_GMinus, P_GMinus, P_GQPlus, 6315)
      call createCurrent(P_A, P_WMinus, P_ghWp, P_ghAAnti, 3388)
      call createCurrent(P_Hl, P_Ha, P_GPlus, P_GMinus, 9786)
      call createCurrent(P_Ha, P_GPlus, P_Hh, P_GMinus, 10191)
      call createCurrent(P_Ha, P_HPlus, P_Ha, P_GMinus, 469)
      call createCurrent(P_G0, P_ghWpAnti, P_GPlus, P_ghA, 16)
      call createCurrent(P_HhQ, P_HPlus, P_Hh, P_HQMinus, 5217)
      call createCurrent(P_GQPlus, P_GMinus, P_HPlus, P_HQMinus, 5047)
      call createCurrent(P_HPlus, P_Ha, P_Hh, P_HMinus, 9056)
      call createCurrent(P_ghZAnti, P_ghA, P_GMinus, P_GQPlus, 6740)
      call createCurrent(P_HhQ, P_Z, P_WMinus, P_GQPlus, 8239)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hh, P_GMinus, 1837)
      call createCurrent(P_G0, P_ghZ, P_HPlus, P_ghWpAnti, 10392)
      call createCurrent(P_A, P_HMinus, P_Z, P_HPlus, 4090)
      call createCurrent(P_HlQ, P_HMinus, P_GPlus, P_GQ0, 8252)
      call createCurrent(P_G0, P_ghZAnti, P_G0, P_ghZ, 5662)
      call createCurrent(P_HMinus, P_GPlus, P_ghWm, P_ghWmAnti, 9156)
      call createCurrent(P_HQPlus, P_GMinus, P_Hh, P_HlQ, 7343)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hh, P_Hl, 9412)
      call createCurrent(P_ghWpAnti, P_ghA, P_G0, P_HPlus, 8925)
      call createCurrent(P_HPlus, P_G0, P_GMinus, P_Ha, 5408)
      call createCurrent(P_A, P_Hl, P_Hh, P_A, 10670)
      call createCurrent(P_GMinus, P_Hh, P_ghZAnti, P_ghWp, 7073)
      call createCurrent(P_AQ, P_Ha, P_HPlus, P_WQMinus, 2988)
      call createCurrent(P_GQMinus, P_c, P_sAnti, 297)
      call createCurrent(P_WQMinus, P_Hl, P_HQPlus, 3603)
      call createCurrent(P_sAnti, P_s, P_ZQ, 4804)
      call createCurrent(P_WQPlus, P_A, P_GMinus, P_HlQ, 1225)
      call createCurrent(P_Z, P_HPlus, P_Hl, P_WMinus, 2681)
      call createCurrent(P_Hh, P_Hh, P_GPlus, P_GMinus, 3646)
      call createCurrent(P_GPlus, P_WMinus, P_Hh, P_Z, 1967)
      call createCurrent(P_GQ0, P_s, P_sAnti, 8530)
      call createCurrent(P_WQPlus, P_GMinus, P_GQ0, 1828)
      call createCurrent(P_GQMinus, P_Ha, P_HPlus, P_HlQ, 8352)
      call createCurrent(P_tauPlus, P_tauMinus, P_HlQ, 119)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_HaQ, 7898)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_HaQ, 1776)
      call createCurrent(P_Z, P_A, 9197)
      call createCurrent(P_GPlus, P_ghZAnti, P_GMinus, P_ghA, 4016)
      call createCurrent(P_Ha, P_GMinus, P_G0, P_GPlus, 8612)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, 7471)
      call createCurrent(P_Z, P_ghWp, P_ghAAnti, P_WMinus, 7826)
      call createCurrent(P_GQMinus, P_Hh, P_ghZAnti, P_ghWp, 6984)
      call createCurrent(P_ghZ, P_ghWmAnti, P_A, P_WQMinus, 2851)
      call createCurrent(P_Hl, P_ghWpAnti, P_HPlus, P_ghA, 6781)
      call createCurrent(P_ghZ, P_WPlus, P_WMinus, P_ghAAnti, 8168)
      call createCurrent(P_WMinus, P_Z, P_Ha, P_HPlus, 8233)
      call createCurrent(P_HaQ, P_Z, P_Z, P_HaQ, 8241)
      call createCurrent(P_HMinus, P_G0, P_Ha, P_HPlus, 3148)
      call createCurrent(P_HMinus, P_Hl, P_ghAAnti, P_ghWp, 6562)
      call createCurrent(P_HPlus, P_A, P_GMinus, 10040)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_HlQ, 5721)
      call createCurrent(P_GQPlus, P_GMinus, P_HMinus, P_HQPlus, 7969)
      call createCurrent(P_Hh, P_GPlus, P_ghWpAnti, P_ghA, 4614)
      call createCurrent(P_HQMinus, P_Hl, P_ghWmAnti, P_ghZ, 1773)
      call createCurrent(P_ghA, P_WPlus, P_WMinus, P_ghAAnti, 2386)
      call createCurrent(P_Ha, P_ghWpAnti, P_GPlus, P_ghA, 9992)
      call createCurrent(P_HhQ, P_WMinus, P_GQPlus, 2850)
      call createCurrent(P_ZQ, P_nu_e, P_nu_eAnti, 1917)
      call createCurrent(P_HhQ, P_Ha, P_HaQ, 1746)
      call createCurrent(P_GQ0, P_GPlus, P_ghAAnti, P_ghWm, 1117)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hl, P_HMinus, 7755)
      call createCurrent(P_HhQ, P_G0, P_GQ0, 2894)
      call createCurrent(P_HhQ, P_GPlus, P_ghWpAnti, P_ghZ, 2567)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_HhQ, 7218)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_G0, 10242)
      call createCurrent(P_WPlus, P_Hl, P_Z, P_HMinus, 3752)
      call createCurrent(P_Ha, P_WPlus, P_WMinus, P_Ha, 1312)
      call createCurrent(P_G0, P_GMinus, P_WPlus, P_Z, 2106)
      call createCurrent(P_tauPlus, P_tauMinus, P_Hl, 6943)
      call createCurrent(P_ghWp, P_HMinus, P_Hh, P_ghAAnti, 8710)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghWp, 3084)
      call createCurrent(P_g, P_g, 9525)
      call createCurrent(P_Hl, P_HPlus, P_ghWm, P_ghZAnti, 5870)
      call createCurrent(P_ghWm, P_Hh, P_ghZAnti, P_GPlus, 282)
      call createCurrent(P_G0, P_G0, P_Hh, 7263)
      call createCurrent(P_HPlus, P_ghWp, P_HMinus, P_ghWpAnti, 1163)
      call createCurrent(P_Hl, P_GPlus, P_Hl, P_GMinus, 3631)
      call createCurrent(P_Hh, P_HPlus, P_ghWpAnti, P_ghA, 311)
      call createCurrent(P_ghZAnti, P_GPlus, P_GMinus, P_ghZ, 626)
      call createCurrent(P_ghWpAnti, P_G0, P_ghWp, P_Ha, 10381)
      call createCurrent(P_GQMinus, P_GPlus, P_HPlus, P_GQMinus, 4304)
      call createCurrent(P_tAnti, P_b, P_HPlus, 883)
      call createCurrent(P_ghZ, P_ghZAnti, P_G0, P_Ha, 10025)
      call createCurrent(P_HlQ, P_HPlus, P_Ha, P_GQMinus, 7995)
      call createCurrent(P_Z, P_Ha, P_A, P_Ha, 9246)
      call createCurrent(P_ZQ, P_WMinus, P_Hl, P_HQPlus, 1814)
      call createCurrent(P_HaQ, P_Hh, P_ghWmAnti, P_ghWm, 5129)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghWp, P_HlQ, 1237)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HhQ, 4670)
      call createCurrent(P_G0, P_Ha, P_G0, P_Hl, 10742)
      call createCurrent(P_ghWmAnti, P_GMinus, P_Hl, P_ghZ, 7582)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghWp, P_HhQ, 1430)
      call createCurrent(P_u, P_uAnti, P_g, 1408)
      call createCurrent(P_ghWm, P_Hh, P_ghWmAnti, 160)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghA, 5205)
      call createCurrent(P_tauMinus, P_tauPlus, P_AQ, 8089)
      call createCurrent(P_HQPlus, P_GMinus, P_HhQ, 2864)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, 7127)
      call createCurrent(P_uAnti, P_u, P_AQ, 1378)
      call createCurrent(P_ZQ, P_t, P_tAnti, 563)
      call createCurrent(P_GQ0, P_Ha, P_Hl, P_HlQ, 4061)
      call createCurrent(P_HQPlus, P_Z, P_WMinus, P_HhQ, 2395)
      call createCurrent(P_Hh, P_WMinus, P_WPlus, 3898)
      call createCurrent(P_HPlus, P_A, P_HMinus, 163)
      call createCurrent(P_GQ0, P_WMinus, P_GPlus, P_ZQ, 4715)
      call createCurrent(P_Hh, P_ghA, P_GMinus, P_ghWmAnti, 7364)
      call createCurrent(P_WPlus, P_A, P_Hl, P_GMinus, 1853)
      call createCurrent(P_ghG, P_ghGAnti, P_g, P_g, 5589)
      call createCurrent(P_bAnti, P_HMinus, P_t, 4569)
      call createCurrent(P_HaQ, P_GPlus, P_Hh, P_GQMinus, 7367)
      call createCurrent(P_Ha, P_ghWpAnti, P_ghWp, P_Ha, 7976)
      call createCurrent(P_HlQ, P_Z, P_Hl, P_ZQ, 8067)
      call createCurrent(P_WQMinus, P_WPlus, P_ghA, P_ghAAnti, 2958)
      call createCurrent(P_HhQ, P_Ha, P_G0, P_HhQ, 2661)
      call createCurrent(P_HQPlus, P_GMinus, P_HMinus, P_HQPlus, 7857)
      call createCurrent(P_ghWm, P_Ha, P_ghZAnti, P_GPlus, 10083)
      call createCurrent(P_Ha, P_ghWp, P_ghZAnti, P_GMinus, 10248)
      call createCurrent(P_WMinus, P_A, P_ghZAnti, P_ghWp, 8367)
      call createCurrent(P_ZQ, P_uAnti, P_u, 4206)
      call createCurrent(P_ghZ, P_G0, P_G0, P_ghZAnti, 4144)
      call createCurrent(P_WMinus, P_Hl, P_WPlus, P_Hh, 9695)
      call createCurrent(P_WPlus, P_ghZAnti, P_A, P_ghWm, 592)
      call createCurrent(P_GMinus, P_ghA, P_ghAAnti, P_HPlus, 9081)
      call createCurrent(P_ghWpAnti, P_GPlus, P_Hl, P_ghZ, 4230)
      call createCurrent(P_GPlus, P_G0, P_Hl, P_GMinus, 8979)
      call createCurrent(P_GQPlus, P_Hl, P_G0, P_HQMinus, 383)
      call createCurrent(P_HQMinus, P_Hh, P_G0, P_GQPlus, 5707)
      call createCurrent(P_WPlus, P_Hl, P_HMinus, 6655)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GQPlus, 654)
      call createCurrent(P_HPlus, P_Ha, P_ghWpAnti, P_ghA, 3445)
      call createCurrent(P_HaQ, P_ghZAnti, P_HMinus, P_ghWp, 6270)
      call createCurrent(P_Hh, P_ghA, P_GPlus, P_ghWpAnti, 4892)
      call createCurrent(P_GQMinus, P_Hl, P_ghA, P_ghWmAnti, 4464)
      call createCurrent(P_Hl, P_GMinus, P_ghWp, P_ghAAnti, 6511)
      call createCurrent(P_ghZAnti, P_ghWm, P_G0, P_GPlus, 7238)
      call createCurrent(P_HhQ, P_GMinus, P_ghWp, P_ghZAnti, 7131)
      call createCurrent(P_HMinus, P_ghWp, P_ghAAnti, P_Hl, 5390)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, 7078)
      call createCurrent(P_HQMinus, P_ghAAnti, P_HPlus, P_ghA, 1303)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghWm, P_GQ0, 3385)
      call createCurrent(P_Ha, P_GMinus, P_A, P_WPlus, 9586)
      call createCurrent(P_HlQ, P_HMinus, P_ghAAnti, P_ghWp, 1084)
      call createCurrent(P_GQMinus, P_HPlus, P_HPlus, P_GQMinus, 369)
      call createCurrent(P_ghAAnti, P_ghA, P_HMinus, P_HQPlus, 3386)
      call createCurrent(P_HMinus, P_HPlus, P_WPlus, P_WMinus, 4658)
      call createCurrent(P_ghZ, P_ghAAnti, P_GPlus, P_HMinus, 9538)
      call createCurrent(P_GQMinus, P_ghAAnti, P_GPlus, P_ghZ, 482)
      call createCurrent(P_Hl, P_GPlus, P_ghWpAnti, P_ghZ, 8424)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghA, P_A, 1286)
      call createCurrent(P_GMinus, P_WPlus, P_Z, P_G0, 5645)
      call createCurrent(P_Hh, P_WMinus, P_A, P_GPlus, 2233)
      call createCurrent(P_GQ0, P_Hl, P_GMinus, P_HQPlus, 1188)
      call createCurrent(P_Hh, P_GPlus, P_HMinus, 4255)
      call createCurrent(P_HaQ, P_WPlus, P_HMinus, P_ZQ, 4875)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_Hh, 8728)
      call createCurrent(P_GMinus, P_ghWm, P_ghWmAnti, P_HPlus, 8881)
      call createCurrent(P_Hl, P_Hh, P_WMinus, P_WPlus, 8950)
      call createCurrent(P_nu_tauAnti, P_tauMinus, P_GQPlus, 701)
      call createCurrent(P_g, P_HMinus, P_g, P_HPlus, 10503)
      call createCurrent(P_HlQ, P_Hl, P_ghWm, P_ghWmAnti, 4850)
      call createCurrent(P_HaQ, P_Z, P_WPlus, P_HQMinus, 5163)
      call createCurrent(P_Z, P_ghZ, P_WPlus, P_ghWpAnti, 4412)
      call createCurrent(P_t, P_tAnti, P_G0, 743)
      call createCurrent(P_Hl, P_Hl, P_Z, P_Z, 664)
      call createCurrent(P_WQMinus, P_HPlus, P_Hl, P_AQ, 2483)
      call createCurrent(P_WMinus, P_WPlus, P_Z, 1528)
      call createCurrent(P_Hl, P_GPlus, P_HMinus, P_Hl, 1860)
      call createCurrent(P_HhQ, P_Ha, P_Hh, P_HaQ, 1552)
      call createCurrent(P_Ha, P_ghZ, P_ghWpAnti, P_GPlus, 8865)
      call createCurrent(P_GPlus, P_HMinus, P_Ha, P_G0, 6361)
      call createCurrent(P_GMinus, P_Ha, P_G0, P_HPlus, 1827)
      call createCurrent(P_A, P_Hh, P_Z, 9151)
      call createCurrent(P_HMinus, P_HPlus, P_Ha, 10432)
      call createCurrent(P_tauMinus, P_Hh, P_tauPlus, 2424)
      call createCurrent(P_Hh, P_WPlus, P_Z, P_GMinus, 7868)
      call createCurrent(P_Z, P_Z, P_HPlus, P_GMinus, 9643)
      call createCurrent(P_HPlus, P_HMinus, P_Z, P_A, 5465)
      call createCurrent(P_HhQ, P_GMinus, P_A, P_WQPlus, 5976)
      call createCurrent(P_GQMinus, P_ePlus, P_nu_e, 2735)
      call createCurrent(P_ZQ, P_HPlus, P_A, P_HQMinus, 22)
      call createCurrent(P_WMinus, P_HPlus, P_G0, P_Z, 10472)
      call createCurrent(P_ghA, P_HMinus, P_ghZAnti, P_HPlus, 5669)
      call createCurrent(P_WQPlus, P_tAnti, P_b, 6794)
      call createCurrent(P_AQ, P_ghWmAnti, P_A, P_ghWm, 7521)
      call createCurrent(P_Hl, P_WMinus, P_WPlus, 6429)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghWm, P_A, 8754)
      call createCurrent(P_HMinus, P_ghZAnti, P_GPlus, P_ghA, 8983)
      call createCurrent(P_b, P_bAnti, P_gQ, 2288)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_HlQ, 6414)
      call createCurrent(P_HMinus, P_ePlus, P_nu_e, 317)
      call createCurrent(P_HQMinus, P_Ha, P_ghWp, P_ghZAnti, 4798)
      call createCurrent(P_GPlus, P_A, P_WMinus, P_Hh, 1200)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghA, P_GMinus, 7334)
      call createCurrent(P_WPlus, P_g, P_g, P_WMinus, 10669)
      call createCurrent(P_GPlus, P_HMinus, P_A, P_Z, 10366)
      call createCurrent(P_HlQ, P_WMinus, P_A, P_HQPlus, 3324)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hl, P_GMinus, 2158)
      call createCurrent(P_GMinus, P_ghA, P_Ha, P_ghWmAnti, 9668)
      call createCurrent(P_Ha, P_Ha, P_Ha, P_Ha, 607)
      call createCurrent(P_GPlus, P_Hh, P_GMinus, P_G0, 8994)
      call createCurrent(P_ghZ, P_Hh, P_ghWmAnti, P_HMinus, 2237)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_Hl, 4004)
      call createCurrent(P_ghWp, P_HMinus, P_HPlus, P_ghWpAnti, 975)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghA, P_Hl, 1270)
      call createCurrent(P_ghZAnti, P_GPlus, P_G0, P_ghWm, 8175)
      call createCurrent(P_ghAAnti, P_WMinus, P_WPlus, P_ghA, 6033)
      call createCurrent(P_HQMinus, P_WPlus, P_Z, P_HaQ, 7170)
      call createCurrent(P_g, P_g, P_Hh, P_Hh, 10676)
      call createCurrent(P_GMinus, P_G0, P_HPlus, P_G0, 8090)
      call createCurrent(P_ghWpAnti, P_GPlus, P_HMinus, P_ghWp, 9632)
      call createCurrent(P_WMinus, P_HPlus, P_Hl, P_Z, 7515)
      call createCurrent(P_GMinus, P_Hl, P_Ha, P_GPlus, 10296)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_Ha, 2780)
      call createCurrent(P_ghZ, P_ghZAnti, P_WMinus, P_WPlus, 7291)
      call createCurrent(P_G0, P_A, P_WPlus, P_HMinus, 8935)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghZ, P_GPlus, 8468)
      call createCurrent(P_ghWm, P_ghZAnti, P_Ha, P_GPlus, 10255)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghZ, 2810)
      call createCurrent(P_GMinus, P_GPlus, P_GPlus, P_HMinus, 3899)
      call createCurrent(P_ZQ, P_WMinus, P_WPlus, P_AQ, 2263)
      call createCurrent(P_WPlus, P_d, P_uAnti, 5730)
      call createCurrent(P_ghWp, P_Hl, P_ghWpAnti, P_GQ0, 7872)
      call createCurrent(P_tAnti, P_t, P_HaQ, 6785)
      call createCurrent(P_GMinus, P_bAnti, P_t, 7271)
      call createCurrent(P_WMinus, P_GPlus, P_Ha, P_A, 10186)
      call createCurrent(P_G0, P_HMinus, P_ghWp, P_ghZAnti, 9084)
      call createCurrent(P_ghAAnti, P_HMinus, P_G0, P_ghWp, 10080)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_G0, 8909)
      call createCurrent(P_Hh, P_GPlus, P_WMinus, P_Z, 1635)
      call createCurrent(P_Ha, P_Hh, P_G0, P_G0, 10502)
      call createCurrent(P_GQMinus, P_GPlus, P_G0, P_HlQ, 3255)
      call createCurrent(P_GQPlus, P_HPlus, P_GMinus, P_HQMinus, 1139)
      call createCurrent(P_G0, P_g, P_g, P_G0, 10698)
      call createCurrent(P_GMinus, P_GPlus, P_ghZ, P_ghZAnti, 3458)
      call createCurrent(P_GMinus, P_WPlus, P_A, P_Ha, 9285)
      call createCurrent(P_Hl, P_Ha, P_G0, 5030)
      call createCurrent(P_GQPlus, P_GPlus, P_GMinus, P_HQMinus, 1738)
      call createCurrent(P_ghZAnti, P_HMinus, P_HPlus, P_ghA, 3551)
      call createCurrent(P_HQPlus, P_GMinus, P_G0, P_HhQ, 8072)
      call createCurrent(P_A, P_GPlus, P_WMinus, P_G0, 2920)
      call createCurrent(P_HQPlus, P_Hh, P_HQMinus, 261)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_HhQ, 3674)
      call createCurrent(P_eMinus, P_ePlus, P_Hl, 99)
      call createCurrent(P_ghWp, P_Ha, P_ghWpAnti, P_G0, 9221)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghA, 2349)
      call createCurrent(P_HMinus, P_ghWm, P_ghWmAnti, P_GPlus, 10235)
      call createCurrent(P_HMinus, P_HPlus, P_G0, P_G0, 3818)
      call createCurrent(P_ghWm, P_WPlus, P_ghAAnti, P_A, 2531)
      call createCurrent(P_ghAAnti, P_Z, P_WPlus, P_ghWm, 1510)
      call createCurrent(P_HMinus, P_G0, P_HPlus, P_Hl, 9399)
      call createCurrent(P_HQMinus, P_A, P_WPlus, P_HlQ, 380)
      call createCurrent(P_Hl, P_ghWp, P_ghZAnti, P_GMinus, 4274)
      call createCurrent(P_G0, P_ghA, P_ghWmAnti, P_HMinus, 9942)
      call createCurrent(P_HlQ, P_HPlus, P_HMinus, P_HlQ, 2777)
      call createCurrent(P_A, P_WMinus, P_GPlus, P_G0, 5758)
      call createCurrent(P_G0, P_Hh, P_Ha, P_Hh, 4873)
      call createCurrent(P_Hl, P_Hl, P_Ha, P_Hl, 10564)
      call createCurrent(P_HPlus, P_g, P_HMinus, P_g, 10693)
      call createCurrent(P_Hh, P_A, P_HPlus, P_WMinus, 6356)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_ghZ, 6717)
      call createCurrent(P_WMinus, P_WPlus, P_Hl, P_Hh, 10280)
      call createCurrent(P_GMinus, P_ghZ, P_HPlus, P_ghAAnti, 10118)
      call createCurrent(P_ZQ, P_WMinus, P_GPlus, P_HlQ, 5556)
      call createCurrent(P_HQPlus, P_GMinus, P_HPlus, P_HQMinus, 6649)
      call createCurrent(P_G0, P_Ha, P_Z, P_A, 10527)
      call createCurrent(P_ghZ, P_Ha, P_ghZAnti, P_G0, 9518)
      call createCurrent(P_Z, P_GPlus, P_WMinus, P_G0, 3679)
      call createCurrent(P_b, P_bAnti, P_AQ, 108)
      call createCurrent(P_HPlus, P_Ha, P_HMinus, 8874)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hl, P_GQPlus, 8823)
      call createCurrent(P_Z, P_WPlus, P_HMinus, P_Hl, 683)
      call createCurrent(P_HPlus, P_Hh, P_GMinus, 589)
      call createCurrent(P_HMinus, P_HPlus, P_Hh, 2501)
      call createCurrent(P_HQMinus, P_GPlus, P_Ha, P_GQ0, 1410)
      call createCurrent(P_tauPlus, P_A, P_tauMinus, 1448)
      call createCurrent(P_Z, P_Z, P_GPlus, P_GMinus, 3932)
      call createCurrent(P_GQ0, P_Hh, P_ghWpAnti, P_ghWp, 6914)
      call createCurrent(P_HQMinus, P_HPlus, P_G0, P_HhQ, 3972)
      call createCurrent(P_Ha, P_ghWmAnti, P_ghZ, P_GMinus, 9890)
      call createCurrent(P_gQ, P_uAnti, P_u, 1642)
      call createCurrent(P_GQPlus, P_Hh, P_HMinus, P_GQ0, 8140)
      call createCurrent(P_HaQ, P_GMinus, P_Hh, P_HQPlus, 7426)
      call createCurrent(P_Hh, P_GMinus, P_A, P_WPlus, 4307)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghA, 6858)
      call createCurrent(P_HMinus, P_WPlus, P_Hl, P_A, 3839)
      call createCurrent(P_HMinus, P_HPlus, P_Hl, P_Hh, 166)
      call createCurrent(P_GPlus, P_ghZ, P_ghWpAnti, P_Hl, 1098)
      call createCurrent(P_A, P_GPlus, P_WMinus, 3865)
      call createCurrent(P_Z, P_HMinus, P_G0, P_WPlus, 9883)
      call createCurrent(P_HPlus, P_ghWpAnti, P_Ha, P_ghA, 7620)
      call createCurrent(P_GQPlus, P_GMinus, P_Z, P_AQ, 1849)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, 3234)
      call createCurrent(P_GQMinus, P_GPlus, P_HMinus, P_HQPlus, 6962)
      call createCurrent(P_A, P_HPlus, P_GMinus, P_A, 10434)
      call createCurrent(P_GPlus, P_ghWp, P_ghWpAnti, P_GMinus, 795)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_HlQ, 2383)
      call createCurrent(P_A, P_WMinus, P_WPlus, 8594)
      call createCurrent(P_Hh, P_GMinus, P_GPlus, P_Ha, 10010)
      call createCurrent(P_HhQ, P_Z, P_Z, P_HhQ, 1130)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_HlQ, 793)
      call createCurrent(P_Hl, P_WMinus, P_GPlus, P_A, 539)
      call createCurrent(P_Ha, P_A, P_GPlus, P_WMinus, 10193)
      call createCurrent(P_Hh, P_ghZAnti, P_GMinus, P_ghWp, 2299)
      call createCurrent(P_HMinus, P_ghZAnti, P_G0, P_ghWp, 8974)
      call createCurrent(P_HMinus, P_A, P_GPlus, P_Z, 10219)
      call createCurrent(P_Hl, P_bAnti, P_b, 5260)
      call createCurrent(P_WQMinus, P_ghWp, P_ghAAnti, 1249)
      call createCurrent(P_A, P_Hh, P_HMinus, P_WPlus, 5432)
      call createCurrent(P_WPlus, P_ghWm, P_ghZAnti, 7966)
      call createCurrent(P_Hl, P_g, P_Hh, P_g, 10605)
      call createCurrent(P_HPlus, P_Hh, P_HMinus, P_Hh, 5815)
      call createCurrent(P_HPlus, P_Z, P_A, P_HMinus, 2746)
      call createCurrent(P_ghWp, P_HMinus, P_Hl, P_ghZAnti, 3341)
      call createCurrent(P_WMinus, P_Ha, P_Z, P_GPlus, 9163)
      call createCurrent(P_GQ0, P_Hh, P_HaQ, 1513)
      call createCurrent(P_Hl, P_ghZ, P_ghZAnti, P_Hh, 9721)
      call createCurrent(P_HPlus, P_ghA, P_ghWpAnti, P_Ha, 4590)
      call createCurrent(P_GMinus, P_ghZAnti, P_Hh, P_ghWp, 4043)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghZ, 4117)
      call createCurrent(P_Hl, P_WMinus, P_GPlus, P_Z, 8377)
      call createCurrent(P_WQMinus, P_G0, P_WPlus, P_GQ0, 6826)
      call createCurrent(P_GMinus, P_Hl, P_ghWp, P_ghAAnti, 6258)
      call createCurrent(P_GMinus, P_Ha, P_Hh, P_GPlus, 9139)
      call createCurrent(P_ghZAnti, P_ghWm, P_WPlus, P_AQ, 2089)
      call createCurrent(P_HMinus, P_WPlus, P_Z, P_Hh, 6550)
      call createCurrent(P_HQPlus, P_Ha, P_GQMinus, 1739)
      call createCurrent(P_ghZ, P_GPlus, P_HMinus, P_ghZAnti, 9037)
      call createCurrent(P_GMinus, P_ghA, P_ghWmAnti, 2474)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Ha, P_HPlus, 3119)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_Hl, 4535)
      call createCurrent(P_ghWm, P_Hh, P_ghWmAnti, P_HaQ, 2829)
      call createCurrent(P_GMinus, P_WPlus, P_Hl, P_A, 5624)
      call createCurrent(P_GQPlus, P_Hl, P_GMinus, P_HhQ, 7223)
      call createCurrent(P_WMinus, P_HMinus, P_WPlus, P_HPlus, 7129)
      call createCurrent(P_GPlus, P_ghWpAnti, P_Hh, P_ghZ, 357)
      call createCurrent(P_A, P_WPlus, P_ghWm, P_ghZAnti, 7281)
      call createCurrent(P_HPlus, P_GMinus, P_Hh, P_Ha, 6067)
      call createCurrent(P_Ha, P_G0, P_Hl, P_Hl, 7436)
      call createCurrent(P_Ha, P_WMinus, P_G0, P_WPlus, 9832)
      call createCurrent(P_WPlus, P_HMinus, P_Z, P_Hl, 7799)
      call createCurrent(P_WPlus, P_WMinus, P_ghWmAnti, P_ghWm, 982)
      call createCurrent(P_WPlus, P_Z, P_GMinus, P_G0, 5002)
      call createCurrent(P_WPlus, P_ghWm, P_A, P_ghZAnti, 566)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_HhQ, 4251)
      call createCurrent(P_A, P_WPlus, P_Hh, P_HMinus, 5472)
      call createCurrent(P_g, P_g, P_Hl, P_Hh, 10625)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghWm, P_ZQ, 7542)
      call createCurrent(P_GQPlus, P_WMinus, P_Z, P_HhQ, 5935)
      call createCurrent(P_Z, P_Hl, P_G0, 6239)
      call createCurrent(P_HQMinus, P_Ha, P_GQPlus, 7840)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghWm, P_Hh, 9849)
      call createCurrent(P_ZQ, P_Hl, P_WPlus, P_GQMinus, 5523)
      call createCurrent(P_d, P_dAnti, P_gQ, 5354)
      call createCurrent(P_WPlus, P_GMinus, P_Hh, 3841)
      call createCurrent(P_GQ0, P_GPlus, P_ghZAnti, P_ghWm, 6159)
      call createCurrent(P_HlQ, P_Hh, P_G0, P_HaQ, 5981)
      call createCurrent(P_WQPlus, P_Hl, P_WQMinus, 7586)
      call createCurrent(P_GMinus, P_GPlus, 10337)
      call createCurrent(P_ghWm, P_HPlus, P_G0, P_ghZAnti, 10387)
      call createCurrent(P_GQMinus, P_GPlus, P_Ha, P_GQ0, 1456)
      call createCurrent(P_ghWm, P_Hh, P_HPlus, P_ghAAnti, 672)
      call createCurrent(P_HaQ, P_Hl, P_Hl, P_HaQ, 3689)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_G0, 2147)
      call createCurrent(P_HQPlus, P_HMinus, P_ghZAnti, P_ghA, 3011)
      call createCurrent(P_GPlus, P_HMinus, P_WPlus, P_WMinus, 9103)
      call createCurrent(P_g, P_G0, P_G0, P_g, 10721)
      call createCurrent(P_tauPlus, P_tauMinus, P_ZQ, 685)
      call createCurrent(P_ghWmAnti, P_G0, P_ghZ, P_GQMinus, 8835)
      call createCurrent(P_GPlus, P_G0, P_HMinus, P_Hl, 5703)
      call createCurrent(P_ghZAnti, P_GMinus, P_HPlus, P_ghA, 9748)
      call createCurrent(P_WPlus, P_WMinus, P_Hl, P_Hl, 3426)
      call createCurrent(P_HMinus, P_ghZ, P_ghWmAnti, P_G0, 9908)
      call createCurrent(P_nu_mu, P_nu_muAnti, P_A, 8844)
      call createCurrent(P_WQMinus, P_Z, P_ghWmAnti, P_ghA, 347)
      call createCurrent(P_HQPlus, P_A, P_HMinus, P_ZQ, 3016)
      call createCurrent(P_HhQ, P_tauMinus, P_tauPlus, 2889)
      call createCurrent(P_HlQ, P_GMinus, P_ghAAnti, P_ghWp, 64)
      call createCurrent(P_g, P_Hh, P_g, 10682)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghZ, P_GPlus, 9491)
      call createCurrent(P_HhQ, P_G0, P_Ha, P_HlQ, 5944)
      call createCurrent(P_HQMinus, P_GPlus, P_Hh, P_HlQ, 3830)
      call createCurrent(P_GQMinus, P_Hh, P_GPlus, P_HaQ, 6521)
      call createCurrent(P_Ha, P_GMinus, P_Hl, P_HPlus, 2582)
      call createCurrent(P_ghA, P_ghZAnti, P_GPlus, P_HMinus, 10190)
      call createCurrent(P_ghZAnti, P_ghA, P_WMinus, P_WPlus, 6430)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hh, P_HQMinus, 4909)
      call createCurrent(P_GPlus, P_ghWm, P_ghAAnti, 197)
      call createCurrent(P_HlQ, P_GMinus, P_ghZ, P_ghWmAnti, 7406)
      call createCurrent(P_HQPlus, P_ghA, P_Hh, P_ghWpAnti, 3927)
      call createCurrent(P_Ha, P_A, P_Hh, 9810)
      call createCurrent(P_WMinus, P_WPlus, P_ghZ, P_ghZAnti, 8503)
      call createCurrent(P_ghWp, P_ghZAnti, P_WMinus, P_A, 1437)
      call createCurrent(P_GMinus, P_Ha, P_HPlus, 1295)
      call createCurrent(P_HaQ, P_A, P_WMinus, P_HQPlus, 5814)
      call createCurrent(P_WMinus, P_GPlus, P_Hh, P_Z, 320)
      call createCurrent(P_ZQ, P_Z, P_Ha, P_HaQ, 3869)
      call createCurrent(P_Ha, P_Ha, P_ghZ, P_ghZAnti, 6022)
      call createCurrent(P_G0, P_Z, P_Hh, 2443)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Ha, P_HhQ, 4706)
      call createCurrent(P_HMinus, P_Z, P_GPlus, P_A, 9519)
      call createCurrent(P_ghWm, P_HMinus, P_ghWmAnti, P_GPlus, 10426)
      call createCurrent(P_ZQ, P_HPlus, P_Hl, P_WQMinus, 1000)
      call createCurrent(P_Hh, P_WPlus, P_GMinus, P_A, 5378)
      call createCurrent(P_A, P_Hh, P_Z, P_Hh, 10042)
      call createCurrent(P_HQPlus, P_HMinus, P_ghWmAnti, P_ghWm, 5150)
      call createCurrent(P_GPlus, P_G0, P_ghWpAnti, P_ghZ, 881)
      call createCurrent(P_ghWpAnti, P_ghWp, P_GMinus, P_HPlus, 9869)
      call createCurrent(P_Z, P_A, P_GMinus, P_GPlus, 3047)
      call createCurrent(P_ghZ, P_Ha, P_Ha, P_ghZAnti, 3682)
      call createCurrent(P_WPlus, P_Hl, P_Z, P_GMinus, 6995)
      call createCurrent(P_AQ, P_tAnti, P_t, 661)
      call createCurrent(P_GPlus, P_HMinus, P_ghA, P_ghAAnti, 10405)
      call createCurrent(P_G0, P_G0, P_Hl, P_Ha, 10524)
      call createCurrent(P_AQ, P_Z, P_WMinus, P_WQPlus, 5663)
      call createCurrent(P_sAnti, P_c, P_GMinus, 5381)
      call createCurrent(P_GMinus, P_ghWmAnti, P_Hl, P_ghA, 5477)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, 6678)
      call createCurrent(P_g, P_ghGAnti, P_ghG, P_g, 1118)
      call createCurrent(P_GMinus, P_GPlus, P_HMinus, P_GPlus, 3611)
      call createCurrent(P_Ha, P_HMinus, P_GPlus, P_Ha, 3114)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghZ, P_GMinus, 8606)
      call createCurrent(P_ghWmAnti, P_ghA, P_Ha, P_HQMinus, 642)
      call createCurrent(P_Hh, P_G0, P_GMinus, P_GPlus, 9947)
      call createCurrent(P_G0, P_Hl, P_Hl, P_G0, 7210)
      call createCurrent(P_HhQ, P_ghWm, P_G0, P_ghWmAnti, 6363)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_G0, 3418)
      call createCurrent(P_AQ, P_GMinus, P_GPlus, P_ZQ, 3240)
      call createCurrent(P_WPlus, P_ghA, P_Z, P_ghWpAnti, 5636)
      call createCurrent(P_HlQ, P_ghWmAnti, P_ghWm, 857)
      call createCurrent(P_Hl, P_Z, P_Ha, 8579)
      call createCurrent(P_GMinus, P_Ha, P_ghWmAnti, P_ghZ, 9363)
      call createCurrent(P_HaQ, P_Hh, P_ghWp, P_ghWpAnti, 3325)
      call createCurrent(P_g, P_HPlus, P_GMinus, P_g, 10707)
      call createCurrent(P_GQPlus, P_Hh, P_ghWm, P_ghZAnti, 2589)
      call createCurrent(P_Hl, P_ghZAnti, P_ghZ, P_Hl, 4091)
      call createCurrent(P_HhQ, P_GMinus, P_ghZAnti, P_ghWp, 1903)
      call createCurrent(P_GMinus, P_ghZAnti, P_Hl, P_ghWp, 1318)
      call createCurrent(P_b, P_Z, P_bAnti, 1666)
      call createCurrent(P_ghZ, P_Hl, P_ghZAnti, P_Hh, 9616)
      call createCurrent(P_cAnti, P_WPlus, P_s, 3986)
      call createCurrent(P_ghWpAnti, P_Hl, P_HPlus, P_ghA, 7449)
      call createCurrent(P_ghZ, P_ghZAnti, P_HPlus, P_HMinus, 6506)
      call createCurrent(P_HaQ, P_WMinus, P_WPlus, P_HaQ, 1592)
      call createCurrent(P_Hl, P_GMinus, P_A, P_WPlus, 898)
      call createCurrent(P_HhQ, P_WMinus, P_Ha, P_WQPlus, 7678)
      call createCurrent(P_HaQ, P_GPlus, P_HMinus, P_HaQ, 8479)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_G0, 5828)
      call createCurrent(P_HMinus, P_HPlus, P_Ha, P_G0, 164)
      call createCurrent(P_WQPlus, P_WMinus, P_GPlus, P_GQMinus, 6922)
      call createCurrent(P_Z, P_GMinus, P_A, P_HPlus, 10151)
      call createCurrent(P_ghWm, P_WPlus, P_ghZAnti, 3613)
      call createCurrent(P_Ha, P_G0, P_Hh, P_G0, 10523)
      call createCurrent(P_HaQ, P_eMinus, P_ePlus, 6515)
      call createCurrent(P_ghWm, P_Hl, P_ghWmAnti, P_Hl, 2651)
      call createCurrent(P_HhQ, P_Hl, P_Hl, P_HlQ, 2027)
      call createCurrent(P_GMinus, P_GPlus, P_G0, P_Hl, 9320)
      call createCurrent(P_Z, P_ghWm, P_ghWmAnti, P_A, 2023)
      call createCurrent(P_WQPlus, P_Z, P_GMinus, P_HhQ, 2200)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_GQ0, 2213)
      call createCurrent(P_ZQ, P_HMinus, P_WPlus, P_HaQ, 7724)
      call createCurrent(P_Ha, P_ghAAnti, P_ghWp, P_GMinus, 9521)
      call createCurrent(P_Hh, P_ghAAnti, P_HPlus, P_ghWm, 17)
      call createCurrent(P_HlQ, P_HPlus, P_WQMinus, 1829)
      call createCurrent(P_Hh, P_G0, P_GMinus, P_HPlus, 5279)
      call createCurrent(P_Hh, P_GPlus, P_HMinus, P_Ha, 6874)
      call createCurrent(P_WMinus, P_nu_tau, P_tauPlus, 8576)
      call createCurrent(P_GQMinus, P_ghWp, P_GPlus, P_ghWpAnti, 7608)
      call createCurrent(P_HaQ, P_G0, P_G0, P_GQ0, 1869)
      call createCurrent(P_Hl, P_GMinus, P_G0, P_GPlus, 9232)
      call createCurrent(P_HlQ, P_Hh, P_GPlus, P_HQMinus, 3746)
      call createCurrent(P_ghWp, P_ghAAnti, P_WQMinus, 1526)
      call createCurrent(P_ZQ, P_Hl, P_Z, P_HlQ, 4436)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hl, P_GQMinus, 5700)
      call createCurrent(P_WQMinus, P_Hh, P_HQPlus, 8462)
      call createCurrent(P_ghZAnti, P_GMinus, P_GPlus, P_ghA, 5898)
      call createCurrent(P_WPlus, P_Z, P_Hl, P_GMinus, 3407)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghWm, P_HaQ, 4408)
      call createCurrent(P_nu_tau, P_Z, P_nu_tauAnti, 7654)
      call createCurrent(P_HhQ, P_Z, P_HaQ, 5918)
      call createCurrent(P_Hh, P_ghZAnti, P_Hl, P_ghZ, 9466)
      call createCurrent(P_Hh, P_Hl, P_HMinus, P_HPlus, 3209)
      call createCurrent(P_WPlus, P_Z, P_WMinus, P_A, 5895)
      call createCurrent(P_ghZ, P_ghZAnti, P_GMinus, P_HPlus, 9437)
      call createCurrent(P_ghWmAnti, P_HMinus, P_Ha, P_ghZ, 6754)
      call createCurrent(P_HhQ, P_G0, P_WPlus, P_WQMinus, 193)
      call createCurrent(P_Hl, P_Hh, P_HPlus, P_GMinus, 546)
      call createCurrent(P_A, P_nu_e, P_nu_eAnti, 10377)
      call createCurrent(P_WMinus, P_WPlus, P_ghWm, P_ghWmAnti, 2389)
      call createCurrent(P_GQ0, P_Hh, P_Ha, P_HlQ, 370)
      call createCurrent(P_bAnti, P_t, P_WMinus, 8015)
      call createCurrent(P_HMinus, P_GPlus, P_Z, 10331)
      call createCurrent(P_HQPlus, P_Hh, P_Hl, P_GQMinus, 4211)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HPlus, P_GMinus, 8842)
      call createCurrent(P_HlQ, P_Ha, P_G0, P_HhQ, 7328)
      call createCurrent(P_ghA, P_GPlus, P_ghWpAnti, P_Ha, 9115)
      call createCurrent(P_HlQ, P_Hh, P_Ha, P_GQ0, 4617)
      call createCurrent(P_ghWmAnti, P_A, P_A, P_ghWm, 3566)
      call createCurrent(P_WMinus, P_Hh, P_GPlus, P_Z, 7783)
      call createCurrent(P_WQPlus, P_Hh, P_HMinus, P_AQ, 1572)
      call createCurrent(P_GQPlus, P_Hl, P_ghZAnti, P_ghWm, 8344)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghWm, P_HlQ, 2931)
      call createCurrent(P_G0, P_HMinus, P_G0, P_GPlus, 6237)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hh, P_HhQ, 7554)
      call createCurrent(P_HQPlus, P_GMinus, P_Hl, P_HaQ, 2679)
      call createCurrent(P_HaQ, P_Ha, P_HPlus, P_GQMinus, 8104)
      call createCurrent(P_Ha, P_HPlus, P_GMinus, 1627)
      call createCurrent(P_HlQ, P_GPlus, P_ghWm, P_ghAAnti, 4094)
      call createCurrent(P_ghAAnti, P_ghZ, P_HMinus, P_HQPlus, 7074)
      call createCurrent(P_HlQ, P_Hl, P_ghWpAnti, P_ghWp, 4394)
      call createCurrent(P_HQMinus, P_Hh, P_GQPlus, 8698)
      call createCurrent(P_ghAAnti, P_ghZ, P_GPlus, P_GQMinus, 6882)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Ha, P_HaQ, 3398)
      call createCurrent(P_Hh, P_Z, P_WMinus, P_GPlus, 6973)
      call createCurrent(P_WPlus, P_ghWm, P_ghAAnti, P_A, 3142)
      call createCurrent(P_G0, P_Z, P_A, P_G0, 8892)
      call createCurrent(P_ghWmAnti, P_ghZ, P_A, P_WQMinus, 155)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Ha, P_HhQ, 3092)
      call createCurrent(P_ghZ, P_A, P_WMinus, P_ghWmAnti, 8013)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_Z, P_ghZ, 6656)
      call createCurrent(P_nu_eAnti, P_nu_e, P_A, 10336)
      call createCurrent(P_ghZ, P_ghWpAnti, P_WPlus, P_A, 6870)
      call createCurrent(P_A, P_WMinus, P_Hh, P_HPlus, 7748)
      call createCurrent(P_Z, P_ghWmAnti, P_WMinus, P_ghA, 5380)
      call createCurrent(P_ghAAnti, P_ghZ, P_HPlus, P_GMinus, 9048)
      call createCurrent(P_A, P_ghWpAnti, P_ghWp, 2592)
      call createCurrent(P_WPlus, P_A, P_ghWpAnti, P_ghA, 3349)
      call createCurrent(P_ghZAnti, P_HPlus, P_Hh, P_ghWm, 1134)
      call createCurrent(P_ghWpAnti, P_G0, P_HPlus, P_ghA, 10070)
      call createCurrent(P_GMinus, P_G0, P_Ha, P_HPlus, 7725)
      call createCurrent(P_Z, P_WPlus, P_GMinus, P_Ha, 9743)
      call createCurrent(P_Hh, P_ghWp, P_HMinus, P_ghZAnti, 4020)
      call createCurrent(P_ZQ, P_WPlus, P_GMinus, P_HhQ, 1778)
      call createCurrent(P_Z, P_Z, P_Ha, P_Ha, 1360)
      call createCurrent(P_ghA, P_ghWmAnti, P_A, P_WMinus, 867)
      call createCurrent(P_GQMinus, P_HPlus, P_Hl, P_GQ0, 2517)
      call createCurrent(P_GQMinus, P_HPlus, P_G0, P_HaQ, 4841)
      call createCurrent(P_Z, P_GMinus, P_GPlus, P_Z, 3379)
      call createCurrent(P_WQPlus, P_ghWm, P_ghZAnti, 1300)
      call createCurrent(P_ghWp, P_ghWmAnti, P_WMinus, P_WMinus, 6726)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghZ, P_GQPlus, 3104)
      call createCurrent(P_Ha, P_A, P_Ha, P_Z, 9919)
      call createCurrent(P_WQPlus, P_GMinus, P_Z, P_HlQ, 3481)
      call createCurrent(P_Z, P_A, P_WMinus, P_WPlus, 5008)
      call createCurrent(P_ghZAnti, P_A, P_WMinus, P_ghWp, 136)
      call createCurrent(P_cAnti, P_c, P_G0, 6243)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hl, P_Hh, 9017)
      call createCurrent(P_Hh, P_Hh, P_ghZ, P_ghZAnti, 3173)
      call createCurrent(P_Z, P_Hl, P_Hh, P_Z, 9180)
      call createCurrent(P_HQPlus, P_HMinus, P_A, P_ZQ, 4113)
      call createCurrent(P_g, P_Ha, P_Ha, P_g, 10526)
      call createCurrent(P_GPlus, P_nu_tauAnti, P_tauMinus, 8812)
      call createCurrent(P_ghZ, P_Hl, P_Hh, P_ghZAnti, 9533)
      call createCurrent(P_WMinus, P_Hh, P_GPlus, 5818)
      call createCurrent(P_Hl, P_HMinus, P_G0, P_GPlus, 301)
      call createCurrent(P_Ha, P_HPlus, P_ghWm, P_ghAAnti, 6645)
      call createCurrent(P_GQPlus, P_Z, P_GMinus, P_ZQ, 132)
      call createCurrent(P_A, P_WPlus, P_GMinus, P_Ha, 10448)
      call createCurrent(P_HPlus, P_ghWm, P_ghAAnti, P_Hl, 1375)
      call createCurrent(P_ghWm, P_Hh, P_Hh, P_ghWmAnti, 2350)
      call createCurrent(P_HhQ, P_A, P_HPlus, P_WQMinus, 8821)
      call createCurrent(P_HMinus, P_ghWmAnti, P_Hl, P_ghZ, 2174)
      call createCurrent(P_AQ, P_WPlus, P_Z, P_WQMinus, 1651)
      call createCurrent(P_muPlus, P_muMinus, P_HlQ, 8133)
      call createCurrent(P_HlQ, P_HMinus, P_Hh, P_GQPlus, 3797)
      call createCurrent(P_HaQ, P_HMinus, P_ghWp, P_ghZAnti, 4305)
      call createCurrent(P_HMinus, P_GPlus, P_GPlus, P_HMinus, 5865)
      call createCurrent(P_ghWmAnti, P_Hl, P_GMinus, P_ghA, 2823)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, 6421)
      call createCurrent(P_HQMinus, P_GPlus, P_HPlus, P_GQMinus, 6125)
      call createCurrent(P_GPlus, P_ghWm, P_ghAAnti, P_Hl, 3810)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghZ, P_HMinus, 9516)
      call createCurrent(P_Z, P_A, P_GPlus, P_GMinus, 5364)
      call createCurrent(P_GMinus, P_c, P_sAnti, 8178)
      call createCurrent(P_GQMinus, P_Z, P_G0, P_WQPlus, 2732)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_Hl, 6155)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, 8350)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hh, P_HMinus, 3514)
      call createCurrent(P_GPlus, P_GPlus, P_HMinus, P_HMinus, 5110)
      call createCurrent(P_G0, P_Hh, P_Ha, P_G0, 10490)
      call createCurrent(P_WQPlus, P_HMinus, P_Z, P_HhQ, 8222)
      call createCurrent(P_GQMinus, P_nu_mu, P_muPlus, 8813)
      call createCurrent(P_WPlus, P_WMinus, P_GPlus, P_GMinus, 8520)
      call createCurrent(P_HQMinus, P_HPlus, P_Hl, P_GQ0, 157)
      call createCurrent(P_WQMinus, P_Hl, P_WPlus, P_HaQ, 4284)
      call createCurrent(P_Ha, P_ghWmAnti, P_HMinus, P_ghZ, 3061)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_Hl, 6011)
      call createCurrent(P_GQMinus, P_A, P_Z, P_GQPlus, 1180)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_Ha, 9684)
      call createCurrent(P_GQPlus, P_WMinus, P_G0, P_ZQ, 6981)
      call createCurrent(P_HlQ, P_Hl, P_WMinus, P_WQPlus, 8164)
      call createCurrent(P_Hh, P_dAnti, P_d, 4179)
      call createCurrent(P_GPlus, P_d, P_uAnti, 5881)
      call createCurrent(P_Hh, P_HPlus, P_HMinus, P_Hl, 4746)
      call createCurrent(P_ghZ, P_G0, P_GMinus, P_ghWmAnti, 7785)
      call createCurrent(P_uAnti, P_u, P_GQ0, 5876)
      call createCurrent(P_HQPlus, P_Hh, P_ghWpAnti, P_ghA, 5652)
      call createCurrent(P_HMinus, P_HPlus, 9191)
      call createCurrent(P_ghZ, P_Hh, P_Hh, P_ghZAnti, 1817)
      call createCurrent(P_WQMinus, P_WPlus, P_WPlus, P_WQMinus, 651)
      call createCurrent(P_GMinus, P_G0, P_Z, P_WPlus, 6673)
      call createCurrent(P_ghWp, P_Hl, P_ghZAnti, P_GQMinus, 3559)
      call createCurrent(P_s, P_sAnti, P_Hl, 184)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Z, P_A, 4353)
      call createCurrent(P_HPlus, P_ghZAnti, P_GMinus, P_ghA, 10099)
      call createCurrent(P_GMinus, P_Ha, P_Hl, P_HPlus, 7459)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_HhQ, 3583)
      call createCurrent(P_HMinus, P_ghWp, P_ghZAnti, 5389)
      call createCurrent(P_ghWp, P_G0, P_ghZAnti, P_GQMinus, 6900)
      call createCurrent(P_HPlus, P_Ha, P_GMinus, P_Ha, 6069)
      call createCurrent(P_nu_tau, P_GMinus, P_tauPlus, 5770)
      call createCurrent(P_ghZAnti, P_A, P_ghWp, P_WMinus, 880)
      call createCurrent(P_Hh, P_GPlus, P_G0, P_GMinus, 9282)
      call createCurrent(P_HPlus, P_Ha, P_Ha, P_HMinus, 8643)
      call createCurrent(P_ghWpAnti, P_Z, P_ghWp, 1215)
      call createCurrent(P_HPlus, P_Ha, P_ghZ, P_ghWpAnti, 3134)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghZ, P_GQPlus, 8751)
      call createCurrent(P_HMinus, P_ghA, P_ghZAnti, P_HPlus, 6555)
      call createCurrent(P_HhQ, P_WMinus, P_Hh, P_WQPlus, 2968)
      call createCurrent(P_HQMinus, P_Hl, P_WPlus, P_ZQ, 7871)
      call createCurrent(P_HQPlus, P_ghZ, P_HMinus, P_ghZAnti, 4744)
      call createCurrent(P_ghWm, P_WPlus, P_Z, P_ghAAnti, 5293)
      call createCurrent(P_HhQ, P_GMinus, P_HPlus, P_HhQ, 1079)
      call createCurrent(P_GQ0, P_Ha, P_Hl, P_HhQ, 8802)
      call createCurrent(P_HMinus, P_WPlus, P_Z, P_G0, 9576)
      call createCurrent(P_GPlus, P_WMinus, P_Ha, P_Z, 10009)
      call createCurrent(P_HPlus, P_ghAAnti, P_HMinus, P_ghZ, 962)
      call createCurrent(P_ghWp, P_GMinus, P_ghWpAnti, P_GPlus, 3580)
      call createCurrent(P_eMinus, P_ePlus, P_HhQ, 5745)
      call createCurrent(P_HPlus, P_A, P_Z, P_HMinus, 8580)
      call createCurrent(P_Ha, P_uAnti, P_u, 733)
      call createCurrent(P_ePlus, P_Hl, P_eMinus, 4390)
      call createCurrent(P_HMinus, P_HPlus, P_Hh, P_Hh, 8831)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hh, P_HhQ, 7794)
      call createCurrent(P_HPlus, P_GMinus, P_Ha, P_Hh, 5572)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hl, P_Hh, 9419)
      call createCurrent(P_HMinus, P_ghA, P_ghWmAnti, P_G0, 8978)
      call createCurrent(P_HMinus, P_A, P_WPlus, 8432)
      call createCurrent(P_GQ0, P_G0, P_GPlus, P_GQMinus, 2396)
      call createCurrent(P_Ha, P_G0, P_G0, P_G0, 8114)
      call createCurrent(P_WQPlus, P_A, P_HMinus, P_HaQ, 5189)
      call createCurrent(P_HhQ, P_GMinus, P_WQPlus, 1483)
      call createCurrent(P_G0, P_GPlus, P_ghWm, P_ghZAnti, 5809)
      call createCurrent(P_GPlus, P_ghZ, P_HMinus, P_ghZAnti, 9907)
      call createCurrent(P_HlQ, P_GPlus, P_ghAAnti, P_ghWm, 8506)
      call createCurrent(P_WMinus, P_ghWp, P_ghZAnti, 5681)
      call createCurrent(P_GQ0, P_Hl, P_Hh, P_HaQ, 4649)
      call createCurrent(P_ghA, P_HMinus, P_ghWmAnti, P_G0, 10275)
      call createCurrent(P_WMinus, P_A, P_ghZ, P_ghWmAnti, 6216)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_Hh, 5249)
      call createCurrent(P_Hl, P_GMinus, P_Z, P_WPlus, 1377)
      call createCurrent(P_Ha, P_Hl, P_G0, 445)
      call createCurrent(P_tauMinus, P_Ha, P_tauPlus, 5664)
      call createCurrent(P_Hl, P_tauPlus, P_tauMinus, 1645)
      call createCurrent(P_Hl, P_HPlus, P_HMinus, P_Hh, 7782)
      call createCurrent(P_HaQ, P_G0, P_GMinus, P_HQPlus, 1621)
      call createCurrent(P_HQMinus, P_muPlus, P_nu_mu, 257)
      call createCurrent(P_ghWmAnti, P_WMinus, P_A, P_ghZ, 6144)
      call createCurrent(P_Ha, P_Hl, P_HMinus, P_GPlus, 3505)
      call createCurrent(P_ghWpAnti, P_HPlus, P_GMinus, P_ghWp, 9647)
      call createCurrent(P_HMinus, P_Hh, P_GPlus, P_Hh, 1885)
      call createCurrent(P_ghZ, P_G0, P_Ha, P_ghZAnti, 10415)
      call createCurrent(P_HPlus, P_GMinus, P_Z, 9498)
      call createCurrent(P_nu_mu, P_Z, P_nu_muAnti, 7105)
      call createCurrent(P_ghA, P_Hl, P_GMinus, P_ghWmAnti, 6864)
      call createCurrent(P_GQPlus, P_Ha, P_Ha, P_HQMinus, 1187)
      call createCurrent(P_GMinus, P_GPlus, P_GPlus, P_GMinus, 6609)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_GQ0, 1728)
      call createCurrent(P_HhQ, P_HPlus, P_G0, P_GQMinus, 1031)
      call createCurrent(P_Ha, P_GPlus, P_Ha, P_HMinus, 821)
      call createCurrent(P_GPlus, P_GMinus, P_G0, P_Ha, 3358)
      call createCurrent(P_ghG, P_g, P_ghGAnti, 2603)
      call createCurrent(P_HPlus, P_HMinus, P_ghWmAnti, P_ghWm, 5977)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghWp, P_Hl, 9799)
      call createCurrent(P_HaQ, P_GMinus, P_HPlus, P_HhQ, 6963)
      call createCurrent(P_GPlus, P_WMinus, P_Z, P_Hl, 1470)
      call createCurrent(P_ghZ, P_WMinus, P_ghZAnti, P_WPlus, 6151)
      call createCurrent(P_HhQ, P_HPlus, P_ghAAnti, P_ghWm, 7474)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_Hh, 5717)
      call createCurrent(P_ghWp, P_A, P_ghWpAnti, P_A, 4917)
      call createCurrent(P_A, P_g, P_g, P_A, 10741)
      call createCurrent(P_HlQ, P_Ha, P_HMinus, P_GQPlus, 1691)
      call createCurrent(P_HQPlus, P_A, P_HMinus, P_AQ, 834)
      call createCurrent(P_GPlus, P_HMinus, P_ghA, P_ghZAnti, 9023)
      call createCurrent(P_Z, P_ghA, P_ghWmAnti, P_WMinus, 5826)
      call createCurrent(P_GMinus, P_Hh, P_ghWmAnti, P_ghA, 5930)
      call createCurrent(P_WPlus, P_GMinus, P_Ha, P_A, 10218)
      call createCurrent(P_AQ, P_G0, P_GMinus, P_WQPlus, 4178)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghA, 2133)
      call createCurrent(P_g, P_Z, P_g, P_A, 10775)
      call createCurrent(P_ghA, P_HMinus, P_G0, P_ghWmAnti, 10019)
      call createCurrent(P_Hh, P_HPlus, P_Z, P_WMinus, 2268)
      call createCurrent(P_GQ0, P_HMinus, P_G0, P_GQPlus, 1840)
      call createCurrent(P_HQMinus, P_bAnti, P_t, 3736)
      call createCurrent(P_HMinus, P_HMinus, P_GPlus, P_HPlus, 130)
      call createCurrent(P_WQPlus, P_Z, P_GQMinus, 6450)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, 5546)
      call createCurrent(P_GPlus, P_G0, P_HMinus, 2086)
      call createCurrent(P_HaQ, P_WMinus, P_HPlus, P_ZQ, 4469)
      call createCurrent(P_muPlus, P_muMinus, P_HaQ, 2657)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_HlQ, 6134)
      call createCurrent(P_ghA, P_A, P_ghWmAnti, P_WQMinus, 3775)
      call createCurrent(P_Z, P_HPlus, P_A, P_HMinus, 6634)
      call createCurrent(P_HQPlus, P_Ha, P_HMinus, P_HaQ, 2425)
      call createCurrent(P_HaQ, P_Hl, P_ZQ, 4333)
      call createCurrent(P_ghWm, P_Hh, P_GPlus, P_ghAAnti, 1970)
      call createCurrent(P_ZQ, P_WMinus, P_HPlus, P_HhQ, 6857)
      call createCurrent(P_Hl, P_Hl, P_Hl, P_Hh, 2895)
      call createCurrent(P_ghWpAnti, P_Z, P_ghWp, P_Z, 7850)
      call createCurrent(P_GQMinus, P_GPlus, P_ghWp, P_ghWpAnti, 7322)
      call createCurrent(P_ghA, P_HMinus, P_Hl, P_ghWmAnti, 6269)
      call createCurrent(P_GPlus, P_Hl, P_HMinus, P_Hh, 2806)
      call createCurrent(P_GPlus, P_ghZ, P_GMinus, P_ghAAnti, 890)
      call createCurrent(P_Hh, P_HMinus, P_HPlus, P_G0, 9407)
      call createCurrent(P_A, P_A, P_G0, P_G0, 10581)
      call createCurrent(P_HMinus, P_GPlus, P_G0, P_G0, 8250)
      call createCurrent(P_Hl, P_Hh, P_Hl, 4478)
      call createCurrent(P_ghWpAnti, P_Ha, P_HPlus, P_ghZ, 6306)
      call createCurrent(P_Hl, P_HPlus, P_ghWpAnti, P_ghZ, 5071)
      call createCurrent(P_G0, P_s, P_sAnti, 4598)
      call createCurrent(P_Hh, P_HPlus, P_GMinus, P_Hh, 451)
      call createCurrent(P_GPlus, P_HMinus, P_g, P_g, 10762)
      call createCurrent(P_ghAAnti, P_ghWp, P_Ha, P_HMinus, 4629)
      call createCurrent(P_Ha, P_GMinus, P_ghAAnti, P_ghWp, 8976)
      call createCurrent(P_G0, P_A, P_Ha, P_A, 10655)
      call createCurrent(P_ghWpAnti, P_Hh, P_GPlus, P_ghZ, 2273)
      call createCurrent(P_ZQ, P_ghWmAnti, P_ghWm, 5611)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hl, P_HPlus, 1511)
      call createCurrent(P_uAnti, P_Hl, P_u, 7524)
      call createCurrent(P_GQPlus, P_GMinus, P_Ha, P_HaQ, 6892)
      call createCurrent(P_Ha, P_Hl, P_Ha, 766)
      call createCurrent(P_WQMinus, P_Hh, P_GPlus, P_ZQ, 4157)
      call createCurrent(P_HPlus, P_Hh, P_ghWpAnti, P_ghZ, 2979)
      call createCurrent(P_Hl, P_GPlus, P_WMinus, P_Z, 7897)
      call createCurrent(P_HMinus, P_ghAAnti, P_GPlus, P_ghA, 10049)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_Hh, 6932)
      call createCurrent(P_G0, P_Hl, P_HPlus, P_HMinus, 10449)
      call createCurrent(P_HlQ, P_HMinus, P_Ha, P_GQPlus, 3998)
      call createCurrent(P_HMinus, P_A, P_HPlus, P_A, 7177)
      call createCurrent(P_GMinus, P_Ha, P_G0, P_GPlus, 5490)
      call createCurrent(P_HhQ, P_WPlus, P_WMinus, P_HhQ, 1565)
      call createCurrent(P_b, P_bAnti, P_Hh, 7547)
      call createCurrent(P_ghZAnti, P_ghWm, P_Ha, P_HQPlus, 963)
      call createCurrent(P_GQPlus, P_WMinus, P_WPlus, P_GQMinus, 6504)
      call createCurrent(P_HMinus, P_G0, P_GPlus, 5297)
      call createCurrent(P_GMinus, P_ghWp, P_ghZAnti, 6224)
      call createCurrent(P_HPlus, P_HMinus, P_G0, P_Ha, 1132)
      call createCurrent(P_Ha, P_ghA, P_ghWpAnti, P_GPlus, 9946)
      call createCurrent(P_ghWmAnti, P_ghWm, P_G0, P_HlQ, 447)
      call createCurrent(P_Hl, P_ghZ, P_HMinus, P_ghWmAnti, 8810)
      call createCurrent(P_t, P_bAnti, P_GMinus, 1160)
      call createCurrent(P_HMinus, P_G0, P_ghWp, P_ghAAnti, 10456)
      call createCurrent(P_HMinus, P_GPlus, P_Hh, P_Ha, 6153)
      call createCurrent(P_WPlus, P_Z, P_HMinus, 8824)
      call createCurrent(P_GQ0, P_WPlus, P_GMinus, P_ZQ, 4893)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hl, P_Hh, 9623)
      call createCurrent(P_ghWmAnti, P_A, P_ghWm, P_Z, 4632)
      call createCurrent(P_GQ0, P_Z, P_G0, P_ZQ, 7778)
      call createCurrent(P_HaQ, P_HMinus, P_GPlus, P_HlQ, 1018)
      call createCurrent(P_c, P_cAnti, P_AQ, 5672)
      call createCurrent(P_WQPlus, P_A, P_HMinus, P_HlQ, 5549)
      call createCurrent(P_GMinus, P_WPlus, P_Hl, P_Z, 1612)
      call createCurrent(P_HMinus, P_GPlus, P_HPlus, P_GMinus, 1153)
      call createCurrent(P_GMinus, P_ghWp, P_ghZAnti, P_Ha, 9833)
      call createCurrent(P_HhQ, P_Ha, P_HPlus, P_GQMinus, 2700)
      call createCurrent(P_WQPlus, P_Z, P_WQMinus, 246)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hh, P_GQPlus, 248)
      call createCurrent(P_HaQ, P_Hh, P_Hl, P_HaQ, 1414)
      call createCurrent(P_ghWm, P_Hl, P_HPlus, P_ghZAnti, 1417)
      call createCurrent(P_GMinus, P_ghWp, P_HPlus, P_ghWpAnti, 10345)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_Hh, 869)
      call createCurrent(P_WPlus, P_Hh, P_GMinus, 8498)
      call createCurrent(P_WQPlus, P_Ha, P_Ha, P_WQMinus, 3536)
      call createCurrent(P_WPlus, P_Ha, P_WMinus, P_G0, 8872)
      call createCurrent(P_WQMinus, P_ghZ, P_A, P_ghWmAnti, 7942)
      call createCurrent(P_GMinus, P_Hl, P_HPlus, 6339)
      call createCurrent(P_ghZ, P_Hh, P_HPlus, P_ghWpAnti, 928)
      call createCurrent(P_ghAAnti, P_ghZ, P_GPlus, P_HMinus, 8955)
      call createCurrent(P_HhQ, P_ghZ, P_GPlus, P_ghWpAnti, 346)
      call createCurrent(P_Hh, P_G0, P_HMinus, P_GPlus, 3540)
      call createCurrent(P_HhQ, P_GPlus, P_Z, P_WQMinus, 2274)
      call createCurrent(P_ghWm, P_WPlus, P_A, P_ghZAnti, 3878)
      call createCurrent(P_g, P_GPlus, P_g, P_GMinus, 10600)
      call createCurrent(P_HhQ, P_ghWpAnti, P_ghWp, 5775)
      call createCurrent(P_Hh, P_GMinus, P_Z, P_WPlus, 5950)
      call createCurrent(P_WPlus, P_GMinus, P_Hh, P_A, 2001)
      call createCurrent(P_s, P_sAnti, P_A, 4434)
      call createCurrent(P_HPlus, P_d, P_uAnti, 1724)
      call createCurrent(P_A, P_Hl, P_HPlus, P_WMinus, 4938)
      call createCurrent(P_GQ0, P_HMinus, P_HQPlus, 5913)
      call createCurrent(P_HhQ, P_Hh, P_GMinus, P_GQPlus, 7601)
      call createCurrent(P_Hl, P_HPlus, P_GMinus, P_Ha, 5963)
      call createCurrent(P_Hl, P_Hl, P_Ha, P_Ha, 1559)
      call createCurrent(P_WQMinus, P_A, P_WPlus, P_ZQ, 1358)
      call createCurrent(P_Hl, P_GMinus, P_WPlus, P_A, 5154)
      call createCurrent(P_A, P_HPlus, P_GMinus, 10142)
      call createCurrent(P_ghA, P_ghWmAnti, P_Ha, P_GMinus, 10262)
      call createCurrent(P_GPlus, P_ghWp, P_HMinus, P_ghWpAnti, 10447)
      call createCurrent(P_Z, P_bAnti, P_b, 1546)
      call createCurrent(P_WQPlus, P_Hh, P_HMinus, P_ZQ, 754)
      call createCurrent(P_A, P_Hh, P_HPlus, P_WMinus, 7089)
      call createCurrent(P_HPlus, P_ghA, P_Ha, P_ghWpAnti, 1131)
      call createCurrent(P_AQ, P_HPlus, P_Hl, P_WQMinus, 6833)
      call createCurrent(P_HQMinus, P_HPlus, P_GPlus, P_GQMinus, 7812)
      call createCurrent(P_GQMinus, P_ghZ, P_GPlus, P_ghAAnti, 4490)
      call createCurrent(P_WMinus, P_ghWp, P_ghWpAnti, P_WPlus, 6885)
      call createCurrent(P_A, P_WMinus, P_A, P_WPlus, 6295)
      call createCurrent(P_Z, P_A, P_HPlus, P_HMinus, 3717)
      call createCurrent(P_WQMinus, P_Ha, P_HPlus, P_ZQ, 2771)
      call createCurrent(P_sAnti, P_Hh, P_s, 4539)
      call createCurrent(P_WPlus, P_A, P_GMinus, P_G0, 2774)
      call createCurrent(P_GPlus, P_Hh, P_ghWm, P_ghAAnti, 5529)
      call createCurrent(P_HlQ, P_A, P_WMinus, P_GQPlus, 5720)
      call createCurrent(P_HMinus, P_ghZ, P_ghAAnti, P_GPlus, 10477)
      call createCurrent(P_ZQ, P_Z, P_ghWmAnti, P_ghWm, 7564)
      call createCurrent(P_A, P_uAnti, P_u, 4802)
      call createCurrent(P_HMinus, P_ghWp, P_Hl, P_ghAAnti, 6109)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWm, P_GQPlus, 1788)
      call createCurrent(P_GPlus, P_HMinus, P_ghWmAnti, P_ghWm, 9027)
      call createCurrent(P_GQPlus, P_HMinus, P_HaQ, 6326)
      call createCurrent(P_ZQ, P_Hh, P_ZQ, 5578)
      call createCurrent(P_ghWp, P_ghZAnti, P_Z, P_WMinus, 2340)
      call createCurrent(P_uAnti, P_u, 9185)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_HaQ, 4661)
      call createCurrent(P_ghWpAnti, P_ghA, P_WPlus, P_A, 8830)
      call createCurrent(P_G0, P_G0, P_Z, P_Z, 2606)
      call createCurrent(P_HhQ, P_WPlus, P_WQMinus, 2725)
      call createCurrent(P_A, P_ghWp, P_ghZAnti, P_WMinus, 4400)
      call createCurrent(P_Hl, P_Hl, P_GPlus, P_GMinus, 6162)
      call createCurrent(P_ghWm, P_Ha, P_ghAAnti, P_HPlus, 6229)
      call createCurrent(P_Z, P_WMinus, P_ghWmAnti, P_ghA, 6549)
      call createCurrent(P_GPlus, P_HMinus, P_Ha, P_Hh, 2169)
      call createCurrent(P_HlQ, P_HMinus, P_ghWp, P_ghZAnti, 4190)
      call createCurrent(P_ghZ, P_A, P_ghWpAnti, P_WQPlus, 405)
      call createCurrent(P_WQMinus, P_Ha, P_HQPlus, 2065)
      call createCurrent(P_b, P_Hh, P_bAnti, 6816)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghZ, P_Z, 5474)
      call createCurrent(P_WMinus, P_Hh, P_HPlus, 1997)
      call createCurrent(P_ghWp, P_Hl, P_ghAAnti, P_GMinus, 5951)
      call createCurrent(P_ghWpAnti, P_ghWp, P_A, 4630)
      call createCurrent(P_s, P_sAnti, P_Z, 7172)
      call createCurrent(P_HPlus, P_G0, P_WMinus, P_A, 9055)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghZ, P_GPlus, 5824)
      call createCurrent(P_HhQ, P_ghWp, P_HMinus, P_ghAAnti, 5358)
      call createCurrent(P_HMinus, P_ghA, P_ghWmAnti, P_Ha, 206)
      call createCurrent(P_Z, P_ghWm, P_ghWmAnti, 3967)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghA, P_GMinus, 9439)
      call createCurrent(P_A, P_ghZAnti, P_ghWm, P_WPlus, 6087)
      call createCurrent(P_GPlus, P_ghWpAnti, P_G0, P_ghZ, 4399)
      call createCurrent(P_ghA, P_HPlus, P_G0, P_ghWpAnti, 9204)
      call createCurrent(P_Z, P_Hh, P_WMinus, P_GPlus, 951)
      call createCurrent(P_Hl, P_HPlus, P_WMinus, P_A, 7418)
      call createCurrent(P_Hl, P_Hl, P_A, P_Z, 9879)
      call createCurrent(P_ZQ, P_GMinus, P_A, P_GQPlus, 234)
      call createCurrent(P_GQPlus, P_GMinus, P_Hl, P_HhQ, 8068)
      call createCurrent(P_Hh, P_Z, P_A, 9887)
      call createCurrent(P_HMinus, P_bAnti, P_t, 1659)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_Ha, 9271)
      call createCurrent(P_ghZ, P_G0, P_ghZAnti, P_GQ0, 4565)
      call createCurrent(P_GPlus, P_ghWm, P_Hl, P_ghAAnti, 3940)
      call createCurrent(P_HMinus, P_g, P_GPlus, P_g, 10688)
      call createCurrent(P_ghWpAnti, P_ghA, P_G0, P_GQPlus, 3135)
      call createCurrent(P_HaQ, P_HPlus, P_Z, P_WQMinus, 3022)
      call createCurrent(P_HlQ, P_HPlus, P_WMinus, P_ZQ, 4023)
      call createCurrent(P_ghAAnti, P_G0, P_ghWp, P_GMinus, 6879)
      call createCurrent(P_WMinus, P_G0, P_WPlus, P_Ha, 10059)
      call createCurrent(P_GQPlus, P_GMinus, P_ghAAnti, P_ghA, 2730)
      call createCurrent(P_ZQ, P_HPlus, P_HMinus, P_AQ, 1740)
      call createCurrent(P_G0, P_Ha, P_HMinus, P_HPlus, 1493)
      call createCurrent(P_cAnti, P_Hl, P_c, 6553)
      call createCurrent(P_HhQ, P_GPlus, P_Hh, P_GQMinus, 4531)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hl, 4660)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghZ, P_Z, 6607)
      call createCurrent(P_Hh, P_Hl, P_A, P_Z, 10757)
      call createCurrent(P_ghZAnti, P_GPlus, P_HMinus, P_ghA, 9392)
      call createCurrent(P_WQPlus, P_GMinus, P_AQ, 8343)
      call createCurrent(P_G0, P_ghAAnti, P_ghWp, P_HMinus, 10438)
      call createCurrent(P_HhQ, P_HPlus, P_ghZ, P_ghWpAnti, 129)
      call createCurrent(P_GQPlus, P_ghA, P_GMinus, P_ghAAnti, 3680)
      call createCurrent(P_GQPlus, P_Z, P_WMinus, P_HhQ, 4281)
      call createCurrent(P_G0, P_HMinus, P_Z, P_WPlus, 9524)
      call createCurrent(P_HMinus, P_HPlus, P_Hl, P_G0, 9927)
      call createCurrent(P_ZQ, P_Hl, P_WMinus, P_HQPlus, 6516)
      call createCurrent(P_WQMinus, P_Hl, P_Z, P_HQPlus, 7923)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghZ, P_HMinus, 9441)
      call createCurrent(P_HhQ, P_WPlus, P_A, P_GQMinus, 5401)
      call createCurrent(P_ghWmAnti, P_ghA, P_WQMinus, 2149)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghA, P_HPlus, 7345)
      call createCurrent(P_GPlus, P_GMinus, P_ghZ, P_ghZAnti, 5351)
      call createCurrent(P_ZQ, P_WPlus, P_G0, P_GQMinus, 2436)
      call createCurrent(P_GMinus, P_HPlus, P_ghAAnti, P_ghZ, 9183)
      call createCurrent(P_ghWp, P_ghWpAnti, P_G0, P_Ha, 9740)
      call createCurrent(P_HQMinus, P_HPlus, P_Ha, P_HaQ, 1653)
      call createCurrent(P_u, P_Hh, P_uAnti, 1492)
      call createCurrent(P_HhQ, P_HMinus, P_WPlus, P_ZQ, 7790)
      call createCurrent(P_WMinus, P_GPlus, P_Z, 1499)
      call createCurrent(P_GQ0, P_ghWm, P_GPlus, P_ghAAnti, 6806)
      call createCurrent(P_WMinus, P_A, P_GPlus, P_Hl, 1603)
      call createCurrent(P_uAnti, P_G0, P_u, 4236)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghA, P_GPlus, 9111)
      call createCurrent(P_GQ0, P_GMinus, P_Ha, P_HQPlus, 1352)
      call createCurrent(P_WQMinus, P_WPlus, P_WMinus, P_WQPlus, 6759)
      call createCurrent(P_GQPlus, P_Hh, P_WMinus, P_ZQ, 2822)
      call createCurrent(P_dAnti, P_d, P_Z, 409)
      call createCurrent(P_ghA, P_Z, P_ghWpAnti, P_WQPlus, 46)
      call createCurrent(P_A, P_ghWmAnti, P_ghWm, 7010)
      call createCurrent(P_GMinus, P_WPlus, P_Hl, 2416)
      call createCurrent(P_HlQ, P_A, P_HMinus, P_WQPlus, 3693)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Ha, P_HlQ, 68)
      call createCurrent(P_ghWmAnti, P_G0, P_HMinus, P_ghZ, 9431)
      call createCurrent(P_HhQ, P_Z, P_GQ0, 6694)
      call createCurrent(P_ghZ, P_ghAAnti, P_HPlus, P_HMinus, 4099)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_HhQ, 1138)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_HlQ, 8450)
      call createCurrent(P_Z, P_G0, P_GPlus, P_WMinus, 2283)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_Ha, 10316)
      call createCurrent(P_WPlus, P_WMinus, P_GPlus, P_HMinus, 8924)
      call createCurrent(P_HaQ, P_Hh, P_G0, P_HhQ, 6509)
      call createCurrent(P_AQ, P_GMinus, P_GQPlus, 385)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hh, P_HQMinus, 7202)
      call createCurrent(P_ghWpAnti, P_ghZ, P_A, P_WPlus, 7749)
      call createCurrent(P_WPlus, P_G0, P_Z, P_GMinus, 2527)
      call createCurrent(P_g, P_Ha, P_G0, P_g, 10657)
      call createCurrent(P_ghA, P_HPlus, P_ghWpAnti, 4972)
      call createCurrent(P_ghZ, P_GMinus, P_ghAAnti, P_GQPlus, 436)
      call createCurrent(P_ZQ, P_ghWm, P_A, P_ghWmAnti, 4820)
      call createCurrent(P_GMinus, P_HPlus, P_HPlus, P_HMinus, 513)
      call createCurrent(P_Hh, P_Ha, P_Z, 8445)
      call createCurrent(P_ghWp, P_A, P_ghWpAnti, 4644)
      call createCurrent(P_Ha, P_Z, P_WPlus, P_HMinus, 529)
      call createCurrent(P_Z, P_ghWp, P_WMinus, P_ghZAnti, 2738)
      call createCurrent(P_eMinus, P_A, P_ePlus, 4668)
      call createCurrent(P_Hl, P_GPlus, P_HMinus, P_G0, 1902)
      call createCurrent(P_ghZ, P_ghWmAnti, P_G0, P_GMinus, 5754)
      call createCurrent(P_GMinus, P_ghWp, P_G0, P_ghZAnti, 5398)
      call createCurrent(P_A, P_WPlus, P_HMinus, P_Hl, 1772)
      call createCurrent(P_ghWmAnti, P_Ha, P_GMinus, P_ghA, 10174)
      call createCurrent(P_HlQ, P_Hh, P_Hh, P_HlQ, 2820)
      call createCurrent(P_A, P_ghWmAnti, P_ghA, P_WMinus, 3642)
      call createCurrent(P_tauMinus, P_tauPlus, P_GQ0, 3782)
      call createCurrent(P_ghZAnti, P_ghA, P_GPlus, P_GQMinus, 830)
      call createCurrent(P_HhQ, P_ePlus, P_eMinus, 2860)
      call createCurrent(P_g, P_g, P_GMinus, P_GPlus, 10594)
      call createCurrent(P_bAnti, P_Ha, P_b, 8801)
      call createCurrent(P_Hl, P_GPlus, P_GMinus, P_Ha, 9760)
      call createCurrent(P_AQ, P_ghAAnti, P_WPlus, P_ghWm, 8051)
      call createCurrent(P_HQPlus, P_HMinus, P_G0, P_HlQ, 906)
      call createCurrent(P_WQPlus, P_Hh, P_Hh, P_WQMinus, 5938)
      call createCurrent(P_Hl, P_ghZ, P_ghWmAnti, P_GMinus, 5125)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hh, P_HaQ, 2392)
      call createCurrent(P_muMinus, P_muPlus, P_Ha, 7751)
      call createCurrent(P_ZQ, P_WPlus, P_HMinus, P_HaQ, 2491)
      call createCurrent(P_WPlus, P_HMinus, P_Hl, P_Z, 7492)
      call createCurrent(P_GPlus, P_GMinus, P_Hl, P_G0, 8943)
      call createCurrent(P_HhQ, P_ghWpAnti, P_G0, P_ghWp, 4109)
      call createCurrent(P_HMinus, P_Hh, P_WPlus, P_Z, 3939)
      call createCurrent(P_ZQ, P_HMinus, P_Ha, P_WQPlus, 2870)
      call createCurrent(P_G0, P_Z, P_GPlus, P_WMinus, 8159)
      call createCurrent(P_AQ, P_Z, P_HPlus, P_HQMinus, 6431)
      call createCurrent(P_ZQ, P_WPlus, P_HMinus, P_HhQ, 7670)
      call createCurrent(P_G0, P_G0, P_WPlus, P_WMinus, 3502)
      call createCurrent(P_ghWmAnti, P_HPlus, P_GMinus, P_ghWm, 9303)
      call createCurrent(P_gQ, P_g, P_gQ, 6587)
      call createCurrent(P_HQPlus, P_ghWm, P_Hh, P_ghZAnti, 5801)
      call createCurrent(P_ghA, P_Hh, P_ghWmAnti, P_GMinus, 7237)
      call createCurrent(P_g, P_g, P_ghG, P_ghGAnti, 910)
      call createCurrent(P_t, P_Ha, P_tAnti, 1336)
      call createCurrent(P_ghWpAnti, P_G0, P_ghA, P_GPlus, 3004)
      call createCurrent(P_Hl, P_GMinus, P_Hl, P_HPlus, 537)
      call createCurrent(P_GMinus, P_HPlus, P_ghZ, P_ghAAnti, 10066)
      call createCurrent(P_ghAAnti, P_ghZ, P_HPlus, P_HMinus, 833)
      call createCurrent(P_HhQ, P_Hh, P_HMinus, P_GQPlus, 7318)
      call createCurrent(P_ePlus, P_nu_e, P_WMinus, 2358)
      call createCurrent(P_HhQ, P_HMinus, P_Hl, P_GQPlus, 7907)
      call createCurrent(P_A, P_WPlus, P_ghWm, P_ghAAnti, 3593)
      call createCurrent(P_cAnti, P_c, P_g, 3771)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghWm, 4341)
      call createCurrent(P_GQMinus, P_Hl, P_ghWmAnti, P_ghA, 4231)
      call createCurrent(P_ghAAnti, P_ghA, P_GMinus, P_GPlus, 2558)
      call createCurrent(P_dAnti, P_Hh, P_d, 5157)
      call createCurrent(P_GQMinus, P_A, P_A, P_GQPlus, 3627)
      call createCurrent(P_nu_e, P_ePlus, P_HMinus, 3122)
      call createCurrent(P_WMinus, P_WPlus, P_g, P_g, 10627)
      call createCurrent(P_HPlus, P_ghWm, P_Ha, P_ghZAnti, 7039)
      call createCurrent(P_GQPlus, P_HPlus, P_HMinus, P_HQMinus, 5602)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghWm, P_Hh, 9522)
      call createCurrent(P_A, P_A, P_Hl, P_Hl, 10556)
      call createCurrent(P_ghZAnti, P_G0, P_HPlus, P_ghWm, 9408)
      call createCurrent(P_GQ0, P_WPlus, P_A, P_GQMinus, 7080)
      call createCurrent(P_Hh, P_HPlus, P_Hh, P_HMinus, 7403)
      call createCurrent(P_GMinus, P_ghWpAnti, P_HPlus, P_ghWp, 9134)
      call createCurrent(P_A, P_sAnti, P_s, 6370)
      call createCurrent(P_GMinus, P_Ha, P_GPlus, P_Hl, 9402)
      call createCurrent(P_GMinus, P_GPlus, P_G0, P_Ha, 1356)
      call createCurrent(P_Ha, P_GMinus, P_G0, P_HPlus, 1896)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_G0, 9567)
      call createCurrent(P_Hl, P_ghWp, P_Hl, P_ghWpAnti, 5894)
      call createCurrent(P_ghWpAnti, P_ghA, P_Ha, P_HPlus, 1429)
      call createCurrent(P_AQ, P_b, P_bAnti, 8466)
      call createCurrent(P_WPlus, P_HMinus, P_HPlus, P_WMinus, 5006)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hh, P_GMinus, 4317)
      call createCurrent(P_G0, P_ghWpAnti, P_ghA, P_HPlus, 9449)
      call createCurrent(P_GPlus, P_ghA, P_Hh, P_ghWpAnti, 2890)
      call createCurrent(P_ghZAnti, P_ghWp, P_HQMinus, 540)
      call createCurrent(P_ghWp, P_WMinus, P_ghAAnti, P_A, 668)
      call createCurrent(P_WMinus, P_WPlus, P_HMinus, P_GPlus, 10177)
      call createCurrent(P_Ha, P_Ha, P_ghWm, P_ghWmAnti, 3001)
      call createCurrent(P_HaQ, P_GPlus, P_G0, P_GQMinus, 3543)
      call createCurrent(P_HaQ, P_HPlus, P_Hh, P_HQMinus, 8635)
      call createCurrent(P_ghA, P_Hl, P_HMinus, P_ghWmAnti, 7770)
      call createCurrent(P_ghA, P_GPlus, P_ghAAnti, P_GQMinus, 2652)
      call createCurrent(P_AQ, P_GMinus, P_Z, P_GQPlus, 2142)
      call createCurrent(P_ghZAnti, P_ghWp, P_Z, P_WMinus, 7903)
      call createCurrent(P_ghZ, P_ghZAnti, P_HPlus, P_GMinus, 10028)
      call createCurrent(P_HMinus, P_ghWp, P_ghZAnti, P_G0, 10389)
      call createCurrent(P_GMinus, P_GPlus, P_Ha, P_Hl, 9042)
      call createCurrent(P_WQMinus, P_ghWpAnti, P_WPlus, P_ghWp, 8136)
      call createCurrent(P_Z, P_WMinus, P_GPlus, P_Ha, 9089)
      call createCurrent(P_Hh, P_ghZ, P_HPlus, P_ghWpAnti, 2000)
      call createCurrent(P_HhQ, P_HMinus, P_GPlus, P_HlQ, 4653)
      call createCurrent(P_cAnti, P_s, P_GPlus, 2178)
      call createCurrent(P_G0, P_A, P_GMinus, P_WPlus, 6992)
      call createCurrent(P_Z, P_A, P_g, P_g, 10632)
      call createCurrent(P_HhQ, P_GPlus, P_A, P_WQMinus, 4153)
      call createCurrent(P_GQPlus, P_WMinus, P_HlQ, 2059)
      call createCurrent(P_Z, P_GMinus, P_A, P_GPlus, 389)
      call createCurrent(P_ghWpAnti, P_ghA, P_GPlus, P_G0, 4954)
      call createCurrent(P_Ha, P_Hh, P_Hl, P_G0, 3365)
      call createCurrent(P_ghWp, P_ghAAnti, P_WMinus, P_Z, 3282)
      call createCurrent(P_Ha, P_HPlus, P_Hl, P_GMinus, 826)
      call createCurrent(P_Hl, P_Ha, P_G0, P_Hl, 1304)
      call createCurrent(P_HhQ, P_ghWmAnti, P_G0, P_ghWm, 7411)
      call createCurrent(P_Z, P_WPlus, P_GMinus, 3293)
      call createCurrent(P_HPlus, P_ghZAnti, P_Hl, P_ghWm, 3840)
      call createCurrent(P_Ha, P_HMinus, P_WPlus, P_Z, 1517)
      call createCurrent(P_HQMinus, P_A, P_Z, P_HQPlus, 1573)
      call createCurrent(P_ghAAnti, P_HPlus, P_Hl, P_ghWm, 882)
      call createCurrent(P_ZQ, P_G0, P_WMinus, P_GQPlus, 4604)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghZ, P_WQPlus, 2786)
      call createCurrent(P_HPlus, P_Hl, P_Hl, P_HMinus, 350)
      call createCurrent(P_GPlus, P_Z, P_HMinus, P_A, 9409)
      call createCurrent(P_HPlus, P_ghA, P_G0, P_ghWpAnti, 10224)
      call createCurrent(P_ghZ, P_WPlus, P_ghZAnti, P_WQMinus, 973)
      call createCurrent(P_ghWm, P_Hl, P_ghZAnti, P_GQPlus, 7353)
      call createCurrent(P_Z, P_Z, P_G0, P_G0, 6866)
      call createCurrent(P_Ha, P_WMinus, P_HPlus, P_Z, 6616)
      call createCurrent(P_GQ0, P_WPlus, P_WMinus, P_GQ0, 4417)
      call createCurrent(P_GQ0, P_ghWmAnti, P_Hl, P_ghWm, 6748)
      call createCurrent(P_HlQ, P_Z, P_ZQ, 181)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_Hh, P_ghA, 8395)
      call createCurrent(P_GPlus, P_Ha, P_HMinus, P_Ha, 212)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWp, P_GQMinus, 5257)
      call createCurrent(P_HlQ, P_Hl, P_Hl, P_HlQ, 6)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghZ, P_GMinus, 8245)
      call createCurrent(P_Hh, P_ghWp, P_ghZAnti, P_HMinus, 6352)
      call createCurrent(P_HQPlus, P_Ha, P_ghWm, P_ghZAnti, 3118)
      call createCurrent(P_ghA, P_HPlus, P_ghAAnti, P_HMinus, 1472)
      call createCurrent(P_Ha, P_Hh, P_HMinus, P_GPlus, 2691)
      call createCurrent(P_A, P_Z, P_Hh, P_Hh, 9004)
      call createCurrent(P_HhQ, P_Z, P_Hh, P_ZQ, 873)
      call createCurrent(P_ghZAnti, P_G0, P_G0, P_ghZ, 5362)
      call createCurrent(P_A, P_Hh, P_GPlus, P_WMinus, 3218)
      call createCurrent(P_Hl, P_HPlus, P_ghWm, P_ghAAnti, 792)
      call createCurrent(P_WMinus, P_GPlus, P_GMinus, P_WPlus, 8016)
      call createCurrent(P_HMinus, P_Hh, P_ghZ, P_ghWmAnti, 31)
      call createCurrent(P_ghWmAnti, P_ghWm, P_WMinus, P_WPlus, 8541)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghZ, P_GQMinus, 5320)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghA, P_G0, 439)
      call createCurrent(P_ghWpAnti, P_Hl, P_Hl, P_ghWp, 4204)
      call createCurrent(P_HMinus, P_A, P_Z, P_HPlus, 2648)
      call createCurrent(P_A, P_HMinus, P_A, P_HPlus, 624)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghA, P_Z, 3433)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_Ha, 9340)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_HlQ, 7957)
      call createCurrent(P_ghAAnti, P_ghWp, P_Hl, P_HMinus, 6991)
      call createCurrent(P_A, P_GPlus, P_Z, P_GMinus, 3588)
      call createCurrent(P_muMinus, P_muPlus, P_Hh, 6024)
      call createCurrent(P_G0, P_HMinus, P_GPlus, P_Ha, 781)
      call createCurrent(P_A, P_ghWm, P_WPlus, P_ghAAnti, 3463)
      call createCurrent(P_ghWpAnti, P_G0, P_GPlus, P_ghZ, 2454)
      call createCurrent(P_WMinus, P_ghAAnti, P_ghWp, P_Z, 5073)
      call createCurrent(P_HPlus, P_HMinus, P_g, P_g, 10598)
      call createCurrent(P_HQPlus, P_WMinus, P_Z, P_HlQ, 1751)
      call createCurrent(P_HMinus, P_Ha, P_GPlus, P_G0, 2840)
      call createCurrent(P_GPlus, P_GMinus, P_A, P_A, 4958)
      call createCurrent(P_GQ0, P_ghWpAnti, P_G0, P_ghWp, 6171)
      call createCurrent(P_ghWm, P_ghWmAnti, P_A, P_Z, 77)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_A, P_ghZ, 8124)
      call createCurrent(P_u, P_uAnti, P_HaQ, 1322)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_HhQ, 8185)
      call createCurrent(P_AQ, P_Z, P_ghWpAnti, P_ghWp, 8549)
      call createCurrent(P_HlQ, P_ghWm, P_HPlus, P_ghAAnti, 5095)
      call createCurrent(P_GQMinus, P_GPlus, P_Hh, P_GQ0, 7541)
      call createCurrent(P_ghWp, P_ghWpAnti, P_G0, P_HlQ, 8115)
      call createCurrent(P_WPlus, P_Z, P_GMinus, 574)
      call createCurrent(P_GMinus, P_ghZ, P_Hl, P_ghWmAnti, 7499)
      call createCurrent(P_Hh, P_Ha, P_HPlus, P_GMinus, 528)
      call createCurrent(P_Hl, P_Ha, P_HPlus, P_GMinus, 2019)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghZ, P_Hh, 2090)
      call createCurrent(P_A, P_ghWpAnti, P_WPlus, P_ghA, 2808)
      call createCurrent(P_b, P_bAnti, P_HaQ, 6746)
      call createCurrent(P_HlQ, P_ghWp, P_HMinus, P_ghAAnti, 2172)
      call createCurrent(P_GMinus, P_A, P_GPlus, 2911)
      call createCurrent(P_gQ, P_d, P_dAnti, 888)
      call createCurrent(P_HPlus, P_A, P_HMinus, P_Z, 562)
      call createCurrent(P_GPlus, P_ghAAnti, P_G0, P_ghWm, 1538)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghA, P_HPlus, 6212)
      call createCurrent(P_GMinus, P_GPlus, P_GMinus, P_GPlus, 7461)
      call createCurrent(P_Z, P_HPlus, P_HMinus, 3275)
      call createCurrent(P_ghWmAnti, P_WMinus, P_A, P_ghA, 3971)
      call createCurrent(P_A, P_g, P_Z, P_g, 10636)
      call createCurrent(P_GMinus, P_GPlus, P_Hl, 4581)
      call createCurrent(P_g, P_uAnti, P_u, 4948)
      call createCurrent(P_ghWmAnti, P_GMinus, P_GPlus, P_ghWm, 8570)
      call createCurrent(P_GPlus, P_ghWm, P_ghAAnti, P_Ha, 8936)
      call createCurrent(P_ZQ, P_ghWp, P_WMinus, P_ghAAnti, 3747)
      call createCurrent(P_HlQ, P_HMinus, P_GPlus, P_HaQ, 2479)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, 4165)
      call createCurrent(P_GQ0, P_Hl, P_HPlus, P_HQMinus, 8100)
      call createCurrent(P_Z, P_WPlus, P_Hl, P_HMinus, 5449)
      call createCurrent(P_HQPlus, P_Z, P_WMinus, P_HlQ, 7557)
      call createCurrent(P_HlQ, P_GMinus, P_ghWmAnti, P_ghZ, 3792)
      call createCurrent(P_Ha, P_ghZAnti, P_HPlus, P_ghWm, 8617)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghZ, P_GQMinus, 4572)
      call createCurrent(P_Hh, P_G0, P_HMinus, P_HPlus, 10417)
      call createCurrent(P_tauMinus, P_Hl, P_tauPlus, 2612)
      call createCurrent(P_Ha, P_HMinus, P_WPlus, P_A, 5969)
      call createCurrent(P_HQMinus, P_WPlus, P_A, P_HlQ, 1786)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghA, P_Hl, 6386)
      call createCurrent(P_Hh, P_Z, P_Z, P_Hh, 5367)
      call createCurrent(P_HQPlus, P_GMinus, P_Hl, P_HhQ, 7021)
      call createCurrent(P_ghA, P_HMinus, P_ghZAnti, P_GPlus, 9552)
      call createCurrent(P_Hl, P_ghZAnti, P_Hl, P_ghZ, 6895)
      call createCurrent(P_WMinus, P_HPlus, P_Z, P_Hl, 1836)
      call createCurrent(P_muPlus, P_Z, P_muMinus, 8755)
      call createCurrent(P_ghAAnti, P_Ha, P_HMinus, P_ghWp, 3705)
      call createCurrent(P_HQPlus, P_GPlus, P_GMinus, P_GQMinus, 8661)
      call createCurrent(P_A, P_A, P_Hl, 10637)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghZ, 5629)
      call createCurrent(P_HPlus, P_g, P_g, P_GMinus, 10758)
      call createCurrent(P_ghZAnti, P_Hl, P_GMinus, P_ghWp, 3262)
      call createCurrent(P_Ha, P_Z, P_A, P_G0, 10493)
      call createCurrent(P_Hh, P_GPlus, P_ghZ, P_ghWpAnti, 7817)
      call createCurrent(P_Z, P_Hh, P_HMinus, P_WPlus, 2789)
      call createCurrent(P_HlQ, P_ghZAnti, P_GPlus, P_ghWm, 144)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_HhQ, 7680)
      call createCurrent(P_GMinus, P_Hh, P_GPlus, P_Hh, 4297)
      call createCurrent(P_GQPlus, P_Hl, P_GMinus, P_HaQ, 444)
      call createCurrent(P_HQPlus, P_Z, P_Z, P_HQMinus, 2817)
      call createCurrent(P_HMinus, P_GPlus, P_A, P_Z, 10357)
      call createCurrent(P_Hh, P_Hh, P_g, P_g, 10601)
      call createCurrent(P_ghA, P_ghWpAnti, P_GPlus, P_Ha, 9898)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_HhQ, 3395)
      call createCurrent(P_ZQ, P_HPlus, P_Hh, P_WQMinus, 8536)
      call createCurrent(P_Z, P_A, P_Z, P_A, 10748)
      call createCurrent(P_WPlus, P_Z, P_ghWpAnti, P_ghA, 8182)
      call createCurrent(P_A, P_Hl, P_Hl, P_Z, 10304)
      call createCurrent(P_Z, P_cAnti, P_c, 1172)
      call createCurrent(P_A, P_Ha, P_Z, P_G0, 10517)
      call createCurrent(P_Hl, P_G0, P_G0, P_Ha, 10511)
      call createCurrent(P_HPlus, P_HPlus, P_GMinus, P_HMinus, 3734)
      call createCurrent(P_GMinus, P_HPlus, P_Ha, P_Hh, 3594)
      call createCurrent(P_ZQ, P_GPlus, P_Z, P_GQMinus, 8363)
      call createCurrent(P_GQMinus, P_Hh, P_Hl, P_HQPlus, 3075)
      call createCurrent(P_HMinus, P_GPlus, P_HMinus, P_HPlus, 1590)
      call createCurrent(P_WMinus, P_HPlus, 9102)
      call createCurrent(P_A, P_A, P_Hl, P_Hh, 10592)
      call createCurrent(P_HhQ, P_Hh, P_ghZ, P_ghZAnti, 4749)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hl, P_HQPlus, 6003)
      call createCurrent(P_HQPlus, P_WMinus, P_Hh, P_ZQ, 7025)
      call createCurrent(P_GPlus, P_Ha, P_WMinus, P_Z, 10312)
      call createCurrent(P_HaQ, P_WMinus, P_Z, P_HQPlus, 3846)
      call createCurrent(P_WMinus, P_ghWmAnti, P_A, P_ghZ, 6551)
      call createCurrent(P_HPlus, P_Hh, P_WMinus, 6854)
      call createCurrent(P_GPlus, P_ghWm, P_GMinus, P_ghWmAnti, 3068)
      call createCurrent(P_GMinus, P_ghWp, P_ghAAnti, P_Hl, 7726)
      call createCurrent(P_GQPlus, P_HMinus, P_GPlus, P_GQMinus, 247)
      call createCurrent(P_HMinus, P_G0, P_G0, P_HPlus, 6951)
      call createCurrent(P_Ha, P_GMinus, P_ghWmAnti, P_ghZ, 9991)
      call createCurrent(P_u, P_uAnti, P_G0, 3759)
      call createCurrent(P_GQ0, P_GMinus, P_WQPlus, 1541)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_HlQ, 2859)
      call createCurrent(P_ZQ, P_muMinus, P_muPlus, 1434)
      call createCurrent(P_GPlus, P_GMinus, P_HPlus, P_GMinus, 426)
      call createCurrent(P_HPlus, P_HMinus, P_G0, P_G0, 6923)
      call createCurrent(P_HlQ, P_ghAAnti, P_GMinus, P_ghWp, 708)
      call createCurrent(P_Hh, P_GMinus, P_Ha, P_GPlus, 9823)
      call createCurrent(P_ghZ, P_HMinus, P_Hh, P_ghWmAnti, 1126)
      call createCurrent(P_WPlus, P_HMinus, P_WMinus, P_HPlus, 4568)
      call createCurrent(P_GQMinus, P_Hl, P_HPlus, P_GQ0, 915)
      call createCurrent(P_Z, P_Z, P_Hh, P_Hl, 9750)
      call createCurrent(P_ghWpAnti, P_Hl, P_Hh, P_ghWp, 9382)
      call createCurrent(P_HhQ, P_GPlus, P_ghWm, P_ghZAnti, 2168)
      call createCurrent(P_A, P_WPlus, P_Ha, P_GMinus, 9963)
      call createCurrent(P_HPlus, P_Hl, P_HMinus, P_Hh, 2075)
      call createCurrent(P_WQMinus, P_HPlus, P_A, P_HaQ, 1633)
      call createCurrent(P_WQPlus, P_ghA, P_Z, P_ghWpAnti, 4479)
      call createCurrent(P_WQMinus, P_A, P_GPlus, P_HlQ, 6372)
      call createCurrent(P_t, P_bAnti, P_HMinus, 1554)
      call createCurrent(P_Ha, P_A, P_WPlus, P_GMinus, 9954)
      call createCurrent(P_WPlus, P_Z, P_Ha, P_GMinus, 9380)
      call createCurrent(P_Hl, P_Hh, P_G0, P_Ha, 7228)
      call createCurrent(P_g, P_GMinus, P_g, P_HPlus, 10562)
      call createCurrent(P_Hl, P_WMinus, P_Hh, P_WPlus, 9350)
      call createCurrent(P_HMinus, P_Hh, P_ghWmAnti, P_ghA, 848)
      call createCurrent(P_HQMinus, P_WPlus, P_Z, P_HlQ, 5657)
      call createCurrent(P_GQPlus, P_WMinus, P_Hh, P_ZQ, 2747)
      call createCurrent(P_HlQ, P_Z, P_GQ0, 824)
      call createCurrent(P_ZQ, P_G0, P_GPlus, P_WQMinus, 2588)
      call createCurrent(P_HMinus, P_HPlus, P_ghWm, P_ghWmAnti, 1329)
      call createCurrent(P_HhQ, P_Z, P_WPlus, P_HQMinus, 3922)
      call createCurrent(P_HQMinus, P_HPlus, P_Hl, P_HlQ, 7383)
      call createCurrent(P_tAnti, P_G0, P_t, 2257)
      call createCurrent(P_GMinus, P_Ha, P_GPlus, 9493)
      call createCurrent(P_HPlus, P_HMinus, P_ghA, P_ghAAnti, 3305)
      call createCurrent(P_HlQ, P_HMinus, P_Hh, P_HQPlus, 6557)
      call createCurrent(P_ghAAnti, P_Ha, P_ghWm, P_HPlus, 3036)
      call createCurrent(P_GPlus, P_Hh, P_HMinus, P_G0, 4)
      call createCurrent(P_GPlus, P_WMinus, P_Hl, P_A, 4348)
      call createCurrent(P_WMinus, P_HPlus, P_A, P_G0, 9071)
      call createCurrent(P_HPlus, P_A, P_WMinus, 1383)
      call createCurrent(P_tAnti, P_Z, P_t, 3414)
      call createCurrent(P_ghWm, P_ghWmAnti, P_GMinus, P_GPlus, 5858)
      call createCurrent(P_Hh, P_G0, P_HPlus, P_GMinus, 5729)
      call createCurrent(P_Hh, P_muMinus, P_muPlus, 7203)
      call createCurrent(P_GPlus, P_Ha, P_Hl, P_HMinus, 3180)
      call createCurrent(P_GQ0, P_Z, P_WPlus, P_GQMinus, 8491)
      call createCurrent(P_WMinus, P_Hh, P_Z, P_GPlus, 4345)
      call createCurrent(P_HMinus, P_GPlus, P_Ha, P_Hl, 3885)
      call createCurrent(P_ghAAnti, P_ghA, P_GMinus, P_HPlus, 9835)
      call createCurrent(P_ghZAnti, P_ghA, P_GMinus, P_HPlus, 10249)
      call createCurrent(P_GQPlus, P_GMinus, P_G0, P_GQ0, 2411)
      call createCurrent(P_G0, P_WPlus, P_WMinus, P_G0, 3309)
      call createCurrent(P_AQ, P_WMinus, P_ghWp, P_ghAAnti, 2668)
      call createCurrent(P_HhQ, P_ghZ, P_HPlus, P_ghWpAnti, 7869)
      call createCurrent(P_Z, P_WMinus, P_Hh, P_GPlus, 831)
      call createCurrent(P_HPlus, P_G0, P_ghZ, P_ghWpAnti, 9766)
      call createCurrent(P_GQMinus, P_GPlus, P_ghAAnti, P_ghA, 3773)
      call createCurrent(P_ghZ, P_GPlus, P_GMinus, P_ghAAnti, 628)
      call createCurrent(P_G0, P_WPlus, P_GMinus, P_Z, 4635)
      call createCurrent(P_Ha, P_ghAAnti, P_GMinus, P_ghWp, 9200)
      call createCurrent(P_GPlus, P_Ha, P_GMinus, P_Hl, 9249)
      call createCurrent(P_HQPlus, P_GMinus, P_GMinus, P_GQPlus, 5107)
      call createCurrent(P_Hh, P_Hh, P_Z, P_Z, 8091)
      call createCurrent(P_Hl, P_WPlus, P_Hh, P_WMinus, 10036)
      call createCurrent(P_HQMinus, P_Hl, P_WQPlus, 3750)
      call createCurrent(P_Hl, P_ghZ, P_HPlus, P_ghWpAnti, 3364)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghZ, P_WPlus, 7685)
      call createCurrent(P_HaQ, P_GPlus, P_Hh, P_HQMinus, 648)
      call createCurrent(P_Ha, P_Ha, P_Hh, P_Hl, 1780)
      call createCurrent(P_Hl, P_G0, P_Hh, P_G0, 4772)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWp, P_HMinus, 7109)
      call createCurrent(P_WPlus, P_ghZ, P_Z, P_ghWpAnti, 2753)
      call createCurrent(P_Hl, P_HMinus, P_WPlus, P_A, 1696)
      call createCurrent(P_GMinus, P_Hl, P_ghZ, P_ghWmAnti, 730)
      call createCurrent(P_WMinus, P_Z, P_HPlus, 8003)
      call createCurrent(P_ghZ, P_ghAAnti, P_GPlus, P_GQMinus, 2238)
      call createCurrent(P_eMinus, P_ePlus, P_G0, 7526)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghWm, P_G0, 10023)
      call createCurrent(P_HlQ, P_WMinus, P_HPlus, P_ZQ, 6158)
      call createCurrent(P_Hh, P_Ha, P_A, 9639)
      call createCurrent(P_Hh, P_Ha, P_G0, P_G0, 10497)
      call createCurrent(P_c, P_cAnti, P_GQ0, 7438)
      call createCurrent(P_Hh, P_Hh, P_A, P_Z, 9295)
      call createCurrent(P_dAnti, P_A, P_d, 6225)
      call createCurrent(P_AQ, P_ghWmAnti, P_Z, P_ghWm, 6743)
      call createCurrent(P_GQ0, P_WPlus, P_Hh, P_WQMinus, 6196)
      call createCurrent(P_WPlus, P_ghA, P_ghWpAnti, P_Z, 4498)
      call createCurrent(P_HlQ, P_eMinus, P_ePlus, 255)
      call createCurrent(P_A, P_A, P_Z, P_Z, 10622)
      call createCurrent(P_WPlus, P_Z, P_ghWm, P_ghZAnti, 1805)
      call createCurrent(P_ghAAnti, P_ghA, P_GPlus, P_HMinus, 9417)
      call createCurrent(P_ghWpAnti, P_ghWp, P_G0, P_Ha, 10038)
      call createCurrent(P_ghZAnti, P_Hl, P_ghZ, P_HlQ, 8431)
      call createCurrent(P_A, P_G0, P_WPlus, P_HMinus, 9083)
      call createCurrent(P_GPlus, P_Hl, P_ghWm, P_ghZAnti, 8489)
      call createCurrent(P_Hh, P_Ha, P_HMinus, P_HPlus, 10154)
      call createCurrent(P_GQMinus, P_Z, P_A, P_GQPlus, 6458)
      call createCurrent(P_WMinus, P_ghAAnti, P_ghZ, P_WPlus, 8822)
      call createCurrent(P_Hl, P_ghWmAnti, P_HMinus, P_ghZ, 1154)
      call createCurrent(P_s, P_cAnti, P_HQPlus, 8354)
      call createCurrent(P_HQMinus, P_Hh, P_WPlus, P_ZQ, 585)
      call createCurrent(P_ghA, P_GMinus, P_ghWmAnti, P_Hh, 2595)
      call createCurrent(P_GPlus, P_G0, P_GMinus, P_Ha, 5184)
      call createCurrent(P_G0, P_ghWmAnti, P_HMinus, P_ghA, 10470)
      call createCurrent(P_tAnti, P_b, P_WQPlus, 6471)
      call createCurrent(P_Z, P_Z, P_GMinus, P_HPlus, 8985)
      call createCurrent(P_GPlus, P_ghWm, P_ghZAnti, P_Hl, 2217)
      call createCurrent(P_ZQ, P_HMinus, P_HQPlus, 7222)
      call createCurrent(P_ghWm, P_HPlus, P_Hl, P_ghAAnti, 5271)
      call createCurrent(P_ghWpAnti, P_Z, P_WPlus, P_ghA, 7451)
      call createCurrent(P_HPlus, P_Hh, P_GMinus, P_Hl, 6070)
      call createCurrent(P_GQ0, P_HMinus, P_HPlus, P_GQ0, 692)
      call createCurrent(P_GMinus, P_GPlus, P_Z, P_A, 4293)
      call createCurrent(P_HQMinus, P_HPlus, P_Hl, P_HhQ, 3707)
      call createCurrent(P_HQPlus, P_GMinus, P_GPlus, P_GQMinus, 1121)
      call createCurrent(P_Ha, P_ghWp, P_Ha, P_ghWpAnti, 6893)
      call createCurrent(P_Hh, P_WPlus, P_GMinus, 3204)
      call createCurrent(P_G0, P_G0, P_Hh, P_Hh, 3354)
      call createCurrent(P_Ha, P_ghA, P_HPlus, P_ghWpAnti, 7490)
      call createCurrent(P_nu_muAnti, P_nu_mu, P_Z, 5458)
      call createCurrent(P_HaQ, P_ghZ, P_HPlus, P_ghWpAnti, 180)
      call createCurrent(P_Hl, P_A, P_GMinus, P_WPlus, 3044)
      call createCurrent(P_uAnti, P_u, P_Z, 3278)
      call createCurrent(P_HQMinus, P_GPlus, P_GQ0, 7971)
      call createCurrent(P_GMinus, P_Ha, P_HPlus, P_G0, 8200)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghA, P_HPlus, 10164)
      call createCurrent(P_GPlus, P_HMinus, 8893)
      call createCurrent(P_ZQ, P_A, P_WPlus, P_WQMinus, 3355)
      call createCurrent(P_ghWmAnti, P_ghWm, P_GPlus, P_GMinus, 3760)
      call createCurrent(P_HPlus, P_ghZ, P_Ha, P_ghWpAnti, 6653)
      call createCurrent(P_ghWmAnti, P_ghA, P_HMinus, P_Hh, 5853)
      call createCurrent(P_WMinus, P_A, P_ghWp, P_ghZAnti, 8476)
      call createCurrent(P_WMinus, P_Ha, P_G0, P_WPlus, 9296)
      call createCurrent(P_Ha, P_WPlus, P_Z, P_GMinus, 9484)
      call createCurrent(P_ghWm, P_Ha, P_ghWmAnti, P_HlQ, 3072)
      call createCurrent(P_A, P_WMinus, P_G0, P_GPlus, 3450)
      call createCurrent(P_GPlus, P_Hl, P_WMinus, 4883)
      call createCurrent(P_ghZ, P_ghWmAnti, P_A, P_WMinus, 8044)
      call createCurrent(P_HMinus, P_Ha, P_WPlus, 2466)
      call createCurrent(P_WQPlus, P_WMinus, P_ghZ, P_ghAAnti, 7962)
      call createCurrent(P_GPlus, P_WMinus, P_Z, P_G0, 7973)
      call createCurrent(P_GQ0, P_Hh, P_ghWm, P_ghWmAnti, 5942)
      call createCurrent(P_HlQ, P_G0, P_HPlus, P_HQMinus, 4932)
      call createCurrent(P_HQMinus, P_Hl, P_Hl, P_HQPlus, 1241)
      call createCurrent(P_ZQ, P_WPlus, P_ghWm, P_ghZAnti, 4952)
      call createCurrent(P_Ha, P_Hl, P_Ha, P_Hh, 3403)
      call createCurrent(P_Hh, P_Hh, P_ghWm, P_ghWmAnti, 338)
      call createCurrent(P_HQPlus, P_ghWm, P_Hl, P_ghZAnti, 2685)
      call createCurrent(P_Hl, P_ghWm, P_ghZAnti, P_HPlus, 8274)
      call createCurrent(P_ZQ, P_WPlus, P_A, P_WQMinus, 3817)
      call createCurrent(P_gQ, P_dAnti, P_d, 6731)
      call createCurrent(P_GQMinus, P_WPlus, P_HhQ, 3888)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hl, P_HQPlus, 2400)
      call createCurrent(P_Hh, P_GMinus, P_GPlus, P_Hl, 4677)
      call createCurrent(P_Hh, P_HPlus, P_WMinus, 598)
      call createCurrent(P_HPlus, P_ghWm, P_ghAAnti, 6076)
      call createCurrent(P_Ha, P_GPlus, P_ghWm, P_ghZAnti, 10133)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghWm, P_WQPlus, 1803)
      call createCurrent(P_GPlus, P_Z, P_WMinus, 7915)
      call createCurrent(P_ghWp, P_Hh, P_ghWpAnti, P_Hh, 6719)
      call createCurrent(P_A, P_HPlus, P_A, P_HMinus, 431)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_G0, 9245)
      call createCurrent(P_HlQ, P_ghWpAnti, P_Hl, P_ghWp, 1345)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghWm, 6812)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_Ha, 5433)
      call createCurrent(P_WQMinus, P_Z, P_Hh, P_HQPlus, 1114)
      call createCurrent(P_ghWmAnti, P_WPlus, P_ghWm, P_WMinus, 732)
      call createCurrent(P_Hh, P_ghZAnti, P_ghWm, P_GPlus, 7532)
      call createCurrent(P_GQ0, P_ghWp, P_GMinus, P_ghZAnti, 2451)
      call createCurrent(P_Hl, P_ghZ, P_Hh, P_ghZAnti, 10427)
      call createCurrent(P_HhQ, P_Ha, P_Ha, P_HlQ, 8000)
      call createCurrent(P_ghZ, P_WMinus, P_ghWmAnti, P_Z, 2048)
      call createCurrent(P_GMinus, P_ghWm, P_ghWmAnti, P_GPlus, 2295)
      call createCurrent(P_Hl, P_Hh, P_Z, P_A, 10505)
      call createCurrent(P_HQMinus, P_GPlus, P_GPlus, P_HQMinus, 4943)
      call createCurrent(P_HPlus, P_HMinus, P_HPlus, P_GMinus, 1999)
      call createCurrent(P_GPlus, P_GMinus, P_A, 3532)
      call createCurrent(P_GQMinus, P_Hh, P_Hh, P_GQPlus, 5571)
      call createCurrent(P_HQMinus, P_Ha, P_HPlus, P_HaQ, 4920)
      call createCurrent(P_ZQ, P_Hh, P_HPlus, P_WQMinus, 5739)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghZ, P_Ha, 4602)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hl, P_GQMinus, 5787)
      call createCurrent(P_uAnti, P_u, P_HlQ, 6480)
      call createCurrent(P_HaQ, P_Z, P_HPlus, P_WQMinus, 2516)
      call createCurrent(P_HaQ, P_Hl, P_ghWp, P_ghWpAnti, 2264)
      call createCurrent(P_ghA, P_GPlus, P_HMinus, P_ghAAnti, 10073)
      call createCurrent(P_HMinus, P_HPlus, P_Z, P_A, 3026)
      call createCurrent(P_HQMinus, P_HMinus, P_HPlus, P_GQPlus, 5796)
      call createCurrent(P_ghWp, P_A, P_WMinus, P_ghAAnti, 7423)
      call createCurrent(P_ghA, P_ghWmAnti, P_GMinus, 1831)
      call createCurrent(P_GMinus, P_Z, P_WPlus, P_Hh, 8788)
      call createCurrent(P_WMinus, P_GPlus, P_A, P_G0, 1622)
      call createCurrent(P_A, P_ghWpAnti, P_Z, P_ghWp, 2928)
      call createCurrent(P_ghWpAnti, P_Z, P_ghZ, P_WQPlus, 3243)
      call createCurrent(P_Hh, P_ghAAnti, P_ghWp, P_HMinus, 2209)
      call createCurrent(P_Z, P_WPlus, P_ghWm, P_ghAAnti, 3236)
      call createCurrent(P_dAnti, P_d, P_gQ, 5213)
      call createCurrent(P_Hh, P_HPlus, P_G0, P_GMinus, 985)
      call createCurrent(P_bAnti, P_b, P_A, 3168)
      call createCurrent(P_WPlus, P_WMinus, P_ghA, P_ghZAnti, 2886)
      call createCurrent(P_GPlus, P_GMinus, P_ghWm, P_ghWmAnti, 735)
      call createCurrent(P_HPlus, P_Hh, P_Ha, P_HMinus, 10200)
      call createCurrent(P_d, P_uAnti, P_HQPlus, 1316)
      call createCurrent(P_ghWm, P_G0, P_ghZAnti, P_HPlus, 10350)
      call createCurrent(P_GQPlus, P_GMinus, P_HhQ, 455)
      call createCurrent(P_WMinus, P_HPlus, P_A, 5139)
      call createCurrent(P_ghZ, P_Hh, P_GMinus, P_ghWmAnti, 442)
      call createCurrent(P_GQMinus, P_Hh, P_ghWp, P_ghZAnti, 6394)
      call createCurrent(P_HPlus, P_ghAAnti, P_GMinus, P_ghA, 10078)
      call createCurrent(P_HaQ, P_Hh, P_Hh, P_GQ0, 5439)
      call createCurrent(P_Hh, P_Hl, P_HPlus, P_GMinus, 4300)
      call createCurrent(P_nu_mu, P_nu_muAnti, P_ZQ, 134)
      call createCurrent(P_Hl, P_Hl, P_Z, P_A, 9100)
      call createCurrent(P_Hh, P_A, P_WMinus, P_HPlus, 4669)
      call createCurrent(P_ghWp, P_WPlus, P_WMinus, P_ghWpAnti, 892)
      call createCurrent(P_HPlus, P_ghWpAnti, P_Hh, P_ghZ, 11)
      call createCurrent(P_ghZAnti, P_ghZ, P_Ha, P_Ha, 667)
      call createCurrent(P_ghZ, P_ghWmAnti, P_WQMinus, 7917)
      call createCurrent(P_ghWp, P_Hl, P_ghWpAnti, P_Hh, 10120)
      call createCurrent(P_ghA, P_GPlus, P_HMinus, P_ghZAnti, 10380)
      call createCurrent(P_GMinus, P_HPlus, P_GMinus, P_HPlus, 3639)
      call createCurrent(P_ghZAnti, P_ghWp, P_G0, P_GMinus, 3912)
      call createCurrent(P_b, P_GPlus, P_tAnti, 5124)
      call createCurrent(P_A, P_A, P_Ha, P_Ha, 10717)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Z, P_WQMinus, 4931)
      call createCurrent(P_GMinus, P_Hh, P_HPlus, P_Hh, 4473)
      call createCurrent(P_HQMinus, P_G0, P_HPlus, P_HlQ, 3017)
      call createCurrent(P_ghA, P_ghWmAnti, P_A, P_WQMinus, 7791)
      call createCurrent(P_GQ0, P_GPlus, P_G0, P_GQMinus, 6330)
      call createCurrent(P_ghZAnti, P_GMinus, P_Ha, P_ghWp, 9729)
      call createCurrent(P_Ha, P_HPlus, P_WMinus, 4114)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghA, P_A, 3867)
      call createCurrent(P_Ha, P_WMinus, P_A, P_GPlus, 9859)
      call createCurrent(P_HlQ, P_HMinus, P_Hl, P_HQPlus, 3088)
      call createCurrent(P_GMinus, P_HPlus, P_Z, 9488)
      call createCurrent(P_WMinus, P_A, P_Z, P_WPlus, 709)
      call createCurrent(P_WQMinus, P_Z, P_Hh, P_GQPlus, 3263)
      call createCurrent(P_b, P_WPlus, P_tAnti, 1313)
      call createCurrent(P_HMinus, P_ghA, P_ghWmAnti, P_Hl, 1100)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_Ha, 10321)
      call createCurrent(P_Hl, P_Ha, P_Hh, P_G0, 605)
      call createCurrent(P_t, P_bAnti, P_GQMinus, 3851)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_Hh, P_ghZ, 6112)
      call createCurrent(P_WMinus, P_A, P_HPlus, P_Hh, 726)
      call createCurrent(P_ghWpAnti, P_ghA, P_HQPlus, 8317)
      call createCurrent(P_GPlus, P_Hl, P_ghWpAnti, P_ghZ, 7631)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hl, P_HQMinus, 8605)
      call createCurrent(P_HPlus, P_WMinus, P_G0, 9649)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_Hl, 2616)
      call createCurrent(P_Ha, P_G0, P_HMinus, P_GPlus, 2653)
      call createCurrent(P_HQMinus, P_Hl, P_ghA, P_ghWmAnti, 3389)
      call createCurrent(P_GQMinus, P_nu_tau, P_tauPlus, 6558)
      call createCurrent(P_HMinus, P_ghAAnti, P_Hh, P_ghWp, 2580)
      call createCurrent(P_ghZ, P_Z, P_WPlus, P_ghWpAnti, 3849)
      call createCurrent(P_d, P_dAnti, P_HaQ, 6187)
      call createCurrent(P_HMinus, P_Hh, P_HPlus, P_Ha, 10365)
      call createCurrent(P_Ha, P_ghWpAnti, P_ghZ, P_HPlus, 3351)
      call createCurrent(P_u, P_dAnti, P_WQMinus, 3656)
      call createCurrent(P_A, P_HPlus, P_Ha, P_WMinus, 2105)
      call createCurrent(P_ghA, P_Ha, P_ghWmAnti, P_HMinus, 6584)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_HhQ, 3199)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_HaQ, 7277)
      call createCurrent(P_ghZAnti, P_G0, P_ghWm, P_GPlus, 8625)
      call createCurrent(P_GPlus, P_G0, P_Hh, P_HMinus, 5756)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghZ, P_Z, 2857)
      call createCurrent(P_Hh, P_Z, P_Z, 1943)
      call createCurrent(P_WPlus, P_G0, P_A, P_GMinus, 1956)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghA, P_GPlus, 5890)
      call createCurrent(P_ghWm, P_WMinus, P_WPlus, P_ghWmAnti, 7627)
      call createCurrent(P_Hl, P_uAnti, P_u, 4210)
      call createCurrent(P_HPlus, P_ghWm, P_ghZAnti, P_G0, 8989)
      call createCurrent(P_HQPlus, P_GMinus, P_G0, P_HlQ, 536)
      call createCurrent(P_ePlus, P_A, P_eMinus, 4846)
      call createCurrent(P_Hh, P_ghWm, P_ghAAnti, P_GPlus, 2267)
      call createCurrent(P_HhQ, P_ghWp, P_Ha, P_ghWpAnti, 4411)
      call createCurrent(P_ZQ, P_WMinus, P_Z, P_WQPlus, 2512)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Z, 2333)
      call createCurrent(P_ghWpAnti, P_ghWp, P_WMinus, P_WPlus, 4416)
      call createCurrent(P_HhQ, P_Hh, P_GPlus, P_HQMinus, 2643)
      call createCurrent(P_Z, P_WMinus, P_ghWp, P_ghZAnti, 6699)
      call createCurrent(P_G0, P_WMinus, P_A, P_HPlus, 9036)
      call createCurrent(P_A, P_GMinus, P_HPlus, 10155)
      call createCurrent(P_ZQ, P_WMinus, P_HQPlus, 2713)
      call createCurrent(P_ghWpAnti, P_WPlus, P_A, P_ghA, 5056)
      call createCurrent(P_HMinus, P_ghWpAnti, P_ghWp, P_HPlus, 6160)
      call createCurrent(P_ghZ, P_Hh, P_HMinus, P_ghWmAnti, 4973)
      call createCurrent(P_WQMinus, P_Hl, P_HPlus, P_ZQ, 7639)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_HlQ, 5566)
      call createCurrent(P_A, P_HPlus, P_GMinus, P_Z, 10092)
      call createCurrent(P_HMinus, P_HPlus, P_HMinus, P_HPlus, 6897)
      call createCurrent(P_HaQ, P_HPlus, P_A, P_WQMinus, 4373)
      call createCurrent(P_GQ0, P_GMinus, P_Hh, P_GQPlus, 5203)
      call createCurrent(P_HhQ, P_cAnti, P_c, 1069)
      call createCurrent(P_ghWm, P_ghWmAnti, P_GMinus, P_HPlus, 8984)
      call createCurrent(P_c, P_G0, P_cAnti, 4220)
      call createCurrent(P_ghWmAnti, P_ghWm, P_AQ, 467)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_G0, 3246)
      call createCurrent(P_WQMinus, P_ePlus, P_nu_e, 3678)
      call createCurrent(P_G0, P_HPlus, P_G0, P_HMinus, 2731)
      call createCurrent(P_GMinus, P_Ha, P_A, P_WPlus, 9336)
      call createCurrent(P_HMinus, P_GPlus, P_ghAAnti, P_ghZ, 9087)
      call createCurrent(P_Ha, P_ghWmAnti, P_GMinus, P_ghA, 8926)
      call createCurrent(P_ghWm, P_Hh, P_HPlus, P_ghZAnti, 5174)
      call createCurrent(P_HPlus, P_g, P_g, P_HMinus, 10648)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hl, P_Hh, 9277)
      call createCurrent(P_Ha, P_ghWm, P_HPlus, P_ghZAnti, 7787)
      call createCurrent(P_WQMinus, P_Hh, P_Z, P_HQPlus, 7372)
      call createCurrent(P_HhQ, P_s, P_sAnti, 2504)
      call createCurrent(P_muMinus, P_muPlus, P_G0, 3006)
      call createCurrent(P_Ha, P_HPlus, P_Ha, P_HMinus, 1664)
      call createCurrent(P_HQPlus, P_Z, P_A, P_HQMinus, 3154)
      call createCurrent(P_ghWp, P_ghAAnti, P_Hl, P_HMinus, 4650)
      call createCurrent(P_Hh, P_eMinus, P_ePlus, 3659)
      call createCurrent(P_HMinus, P_G0, P_HPlus, P_Hh, 9923)
      call createCurrent(P_ghWm, P_HPlus, P_Hh, P_ghZAnti, 8584)
      call createCurrent(P_WQPlus, P_ghZAnti, P_WMinus, P_ghZ, 6580)
      call createCurrent(P_ghWm, P_GMinus, P_ghWmAnti, P_HPlus, 10098)
      call createCurrent(P_ghWm, P_G0, P_ghWmAnti, P_Ha, 9709)
      call createCurrent(P_HlQ, P_WPlus, P_WMinus, P_HlQ, 7531)
      call createCurrent(P_Ha, P_HMinus, P_HPlus, P_Hl, 8861)
      call createCurrent(P_GQPlus, P_Hl, P_Z, P_WQMinus, 7562)
      call createCurrent(P_WMinus, P_A, P_G0, P_HPlus, 10238)
      call createCurrent(P_GQMinus, P_GPlus, P_GPlus, P_HQMinus, 2869)
      call createCurrent(P_HPlus, P_ghAAnti, P_Ha, P_ghWm, 6140)
      call createCurrent(P_HPlus, P_ghAAnti, P_Hl, P_ghWm, 7001)
      call createCurrent(P_ghG, P_ghGAnti, P_g, P_gQ, 3029)
      call createCurrent(P_ghZ, P_Hh, P_ghZAnti, P_HhQ, 8077)
      call createCurrent(P_ghAAnti, P_ghA, P_GMinus, P_GQPlus, 7759)
      call createCurrent(P_G0, P_ghZ, P_HMinus, P_ghWmAnti, 9602)
      call createCurrent(P_WMinus, P_ghZAnti, P_Z, P_ghWp, 3658)
      call createCurrent(P_ghWp, P_Hh, P_HMinus, P_ghZAnti, 3343)
      call createCurrent(P_HPlus, P_HMinus, P_A, P_Z, 1238)
      call createCurrent(P_A, P_GMinus, P_GPlus, 7713)
      call createCurrent(P_HlQ, P_G0, P_G0, P_HhQ, 2770)
      call createCurrent(P_ghAAnti, P_WPlus, P_WMinus, P_ghA, 4694)
      call createCurrent(P_ghWpAnti, P_GMinus, P_HPlus, P_ghWp, 10460)
      call createCurrent(P_GMinus, P_ghA, P_ghWmAnti, P_Hl, 7721)
      call createCurrent(P_c, P_sAnti, P_GMinus, 8341)
      call createCurrent(P_AQ, P_A, P_HMinus, P_HQPlus, 590)
      call createCurrent(P_ghA, P_Z, P_ghWmAnti, P_WMinus, 5338)
      call createCurrent(P_WMinus, P_Z, P_Hh, P_GPlus, 1116)
      call createCurrent(P_HlQ, P_Ha, P_Ha, P_HlQ, 5702)
      call createCurrent(P_GMinus, P_Hl, P_ghWp, P_ghZAnti, 7617)
      call createCurrent(P_G0, P_Hh, P_GPlus, P_HMinus, 1876)
      call createCurrent(P_d, P_dAnti, P_Hl, 3147)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Ha, P_HlQ, 6039)
      call createCurrent(P_GQ0, P_HPlus, P_Ha, P_HQMinus, 215)
      call createCurrent(P_ZQ, P_u, P_uAnti, 4521)
      call createCurrent(P_Hh, P_ghZ, P_ghZAnti, 612)
      call createCurrent(P_Hl, P_Hh, P_ghZ, P_ghZAnti, 9460)
      call createCurrent(P_G0, P_GMinus, P_Hl, P_GPlus, 9568)
      call createCurrent(P_HlQ, P_ghWmAnti, P_Ha, P_ghWm, 6679)
      call createCurrent(P_HlQ, P_ghZAnti, P_HPlus, P_ghWm, 1863)
      call createCurrent(P_HQPlus, P_WMinus, P_A, P_HaQ, 8621)
      call createCurrent(P_Z, P_ghAAnti, P_ghWm, P_WPlus, 6150)
      call createCurrent(P_HPlus, P_G0, P_GMinus, 5178)
      call createCurrent(P_HlQ, P_GPlus, P_GMinus, P_HlQ, 3454)
      call createCurrent(P_HlQ, P_muMinus, P_muPlus, 4651)
      call createCurrent(P_Hh, P_GMinus, P_Hl, P_GPlus, 6335)
      call createCurrent(P_GPlus, P_ghWpAnti, P_HMinus, P_ghWp, 9468)
      call createCurrent(P_WQPlus, P_Hh, P_WMinus, P_HaQ, 918)
      call createCurrent(P_HPlus, P_GMinus, P_G0, P_Hh, 1144)
      call createCurrent(P_GMinus, P_WPlus, P_Hh, P_Z, 8518)
      call createCurrent(P_G0, P_GPlus, P_ghZAnti, P_ghWm, 6768)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghZ, P_Hl, 8645)
      call createCurrent(P_HQPlus, P_ghWm, P_Ha, P_ghZAnti, 1421)
      call createCurrent(P_ghWpAnti, P_G0, P_ghZ, P_HPlus, 9425)
      call createCurrent(P_ghA, P_A, P_ghWpAnti, P_WQPlus, 5012)
      call createCurrent(P_ghWpAnti, P_G0, P_HPlus, P_ghZ, 10373)
      call createCurrent(P_HMinus, P_G0, P_Hh, P_GPlus, 6357)
      call createCurrent(P_ghWmAnti, P_Ha, P_G0, P_ghWm, 9418)
      call createCurrent(P_G0, P_ghAAnti, P_ghWm, P_GPlus, 4233)
      call createCurrent(P_WPlus, P_tauMinus, P_nu_tauAnti, 5646)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hh, P_Hh, 4138)
      call createCurrent(P_Hl, P_g, P_g, P_Hh, 10611)
      call createCurrent(P_ghWm, P_GPlus, P_Hl, P_ghAAnti, 7029)
      call createCurrent(P_Hh, P_ghA, P_ghWmAnti, P_HMinus, 1326)
      call createCurrent(P_WMinus, P_A, P_Hh, P_HPlus, 4570)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghWm, P_HhQ, 1476)
      call createCurrent(P_Hl, P_Hl, P_Ha, P_G0, 7994)
      call createCurrent(P_WPlus, P_ghA, P_ghAAnti, P_WMinus, 36)
      call createCurrent(P_WQMinus, P_HPlus, P_Ha, P_AQ, 5784)
      call createCurrent(P_G0, P_Z, P_Hl, 7413)
      call createCurrent(P_HPlus, P_HMinus, P_ghAAnti, P_ghZ, 2871)
      call createCurrent(P_HQMinus, P_ghZ, P_HPlus, P_ghAAnti, 7553)
      call createCurrent(P_G0, P_WMinus, P_Z, P_HPlus, 9384)
      call createCurrent(P_GPlus, P_Hl, P_WMinus, P_A, 8551)
      call createCurrent(P_ghWmAnti, P_ghWm, P_A, P_ZQ, 5769)
      call createCurrent(P_Hl, P_ghWm, P_ghAAnti, P_GPlus, 1802)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghZ, P_G0, 10466)
      call createCurrent(P_HQPlus, P_Hl, P_Hl, P_HQMinus, 2352)
      call createCurrent(P_HPlus, P_ghWmAnti, P_HMinus, P_ghWm, 8651)
      call createCurrent(P_HaQ, P_Hl, P_GMinus, P_HQPlus, 1852)
      call createCurrent(P_GQPlus, P_GMinus, P_A, P_ZQ, 6811)
      call createCurrent(P_A, P_GPlus, P_WMinus, P_Ha, 10101)
      call createCurrent(P_HhQ, P_HPlus, P_GQMinus, 7695)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_Ha, 8253)
      call createCurrent(P_ghAAnti, P_ghWp, P_WQMinus, 91)
      call createCurrent(P_ZQ, P_HMinus, P_HPlus, P_ZQ, 4839)
      call createCurrent(P_HlQ, P_Hl, P_HhQ, 8463)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_GQ0, 1325)
      call createCurrent(P_ghZ, P_WMinus, P_ghWmAnti, 3264)
      call createCurrent(P_GPlus, P_Hl, P_ghWm, P_ghAAnti, 2344)
      call createCurrent(P_GQPlus, P_Hh, P_Hh, P_GQMinus, 3353)
      call createCurrent(P_A, P_WMinus, P_GPlus, P_Hh, 3367)
      call createCurrent(P_GMinus, P_Ha, P_Z, P_WPlus, 9362)
      call createCurrent(P_d, P_HPlus, P_uAnti, 4374)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghWp, P_Hl, 7302)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghA, P_GPlus, 8778)
      call createCurrent(P_HlQ, P_Z, P_GPlus, P_WQMinus, 6042)
      call createCurrent(P_u, P_A, P_uAnti, 4842)
      call createCurrent(P_GMinus, P_Hh, P_Z, P_WPlus, 5909)
      call createCurrent(P_gQ, P_bAnti, P_b, 1302)
      call createCurrent(P_Hl, P_Ha, P_HPlus, P_HMinus, 9066)
      call createCurrent(P_ghZAnti, P_ghWp, P_A, P_WMinus, 5463)
      call createCurrent(P_G0, P_ghAAnti, P_HMinus, P_ghWp, 10061)
      call createCurrent(P_Hl, P_ghWpAnti, P_Hh, P_ghWp, 9916)
      call createCurrent(P_ghWm, P_ghZAnti, P_WQPlus, 4523)
      call createCurrent(P_GQ0, P_G0, P_ghWpAnti, P_ghWp, 4600)
      call createCurrent(P_ghWpAnti, P_ghZ, P_WPlus, 7630)
      call createCurrent(P_ghA, P_ghWpAnti, P_GPlus, 5873)
      call createCurrent(P_WQPlus, P_A, P_WMinus, P_AQ, 6349)
      call createCurrent(P_HaQ, P_G0, P_GPlus, P_HQMinus, 2999)
      call createCurrent(P_GMinus, P_Ha, P_Hh, P_HPlus, 4697)
      call createCurrent(P_G0, P_A, P_WPlus, P_GMinus, 2195)
      call createCurrent(P_GMinus, P_ghAAnti, P_GPlus, P_ghA, 8312)
      call createCurrent(P_Hl, P_GMinus, P_ghWmAnti, P_ghZ, 7756)
      call createCurrent(P_WPlus, P_GMinus, P_Z, P_G0, 4527)
      call createCurrent(P_g, P_A, P_g, P_g, 10509)
      call createCurrent(P_Ha, P_muMinus, P_muPlus, 3600)
      call createCurrent(P_Ha, P_ghWmAnti, P_GMinus, P_ghZ, 8887)
      call createCurrent(P_WMinus, P_Ha, P_WPlus, P_G0, 9005)
      call createCurrent(P_G0, P_GMinus, P_HPlus, P_Ha, 8436)
      call createCurrent(P_HPlus, P_Ha, P_WMinus, P_Z, 7527)
      call createCurrent(P_HQMinus, P_Ha, P_WPlus, P_AQ, 1309)
      call createCurrent(P_HlQ, P_Hl, P_GPlus, P_GQMinus, 7867)
      call createCurrent(P_HPlus, P_ghZ, P_ghAAnti, P_HMinus, 3455)
      call createCurrent(P_GQMinus, P_G0, P_GPlus, P_HaQ, 2047)
      call createCurrent(P_GQPlus, P_G0, P_ghWpAnti, P_ghA, 3432)
      call createCurrent(P_WPlus, P_GMinus, P_Ha, 10088)
      call createCurrent(P_HPlus, P_ghWm, P_ghZAnti, 1147)
      call createCurrent(P_HQMinus, P_Hl, P_Hl, P_GQPlus, 8107)
      call createCurrent(P_WPlus, P_HMinus, P_GPlus, P_WMinus, 9644)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HPlus, P_GMinus, 9507)
      call createCurrent(P_GQ0, P_HMinus, P_GQPlus, 7378)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_Z, P_ghA, 7326)
      call createCurrent(P_GQ0, P_HMinus, P_Ha, P_HQPlus, 770)
      call createCurrent(P_GQ0, P_Hh, P_GPlus, P_GQMinus, 3163)
      call createCurrent(P_GQMinus, P_A, P_WPlus, P_HlQ, 3903)
      call createCurrent(P_HaQ, P_Hl, P_GMinus, P_GQPlus, 8108)
      call createCurrent(P_GQ0, P_sAnti, P_s, 6619)
      call createCurrent(P_GMinus, P_GPlus, P_Ha, P_G0, 921)
      call createCurrent(P_Hl, P_ghWpAnti, P_GPlus, P_ghA, 6598)
      call createCurrent(P_GPlus, P_Hl, P_Hh, P_GMinus, 8443)
      call createCurrent(P_HhQ, P_ghAAnti, P_GPlus, P_ghWm, 3512)
      call createCurrent(P_c, P_sAnti, P_GQMinus, 2185)
      call createCurrent(P_GQPlus, P_GMinus, P_Hl, P_GQ0, 8679)
      call createCurrent(P_HhQ, P_Hl, P_Ha, P_HaQ, 1655)
      call createCurrent(P_HlQ, P_Ha, P_ZQ, 4271)
      call createCurrent(P_ghA, P_ghWpAnti, P_G0, P_GPlus, 5993)
      call createCurrent(P_GQMinus, P_u, P_dAnti, 6989)
      call createCurrent(P_ghAAnti, P_HPlus, P_HMinus, P_ghA, 2310)
      call createCurrent(P_Hh, P_G0, P_A, 9144)
      call createCurrent(P_HlQ, P_GPlus, P_WQMinus, 4243)
      call createCurrent(P_ghAAnti, P_G0, P_HPlus, P_ghWm, 10097)
      call createCurrent(P_g, P_g, P_G0, P_G0, 10697)
      call createCurrent(P_HlQ, P_ghWp, P_ghWpAnti, 5740)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_HlQ, 7979)
      call createCurrent(P_Hh, P_WPlus, P_Z, P_HMinus, 3281)
      call createCurrent(P_WPlus, P_ghZ, P_ghZAnti, P_WMinus, 2167)
      call createCurrent(P_WQPlus, P_HMinus, P_Ha, P_AQ, 84)
      call createCurrent(P_ghA, P_GMinus, P_Hl, P_ghWmAnti, 8075)
      call createCurrent(P_WPlus, P_ghWm, P_WMinus, P_ghWmAnti, 8658)
      call createCurrent(P_HPlus, P_ghA, P_HMinus, P_ghAAnti, 3974)
      call createCurrent(P_ghAAnti, P_ghWm, P_G0, P_GPlus, 5211)
      call createCurrent(P_muPlus, P_WMinus, P_nu_mu, 7839)
      call createCurrent(P_Ha, P_HPlus, P_HMinus, P_Hl, 9508)
      call createCurrent(P_HaQ, P_G0, P_Ha, P_HaQ, 2766)
      call createCurrent(P_Z, P_GPlus, P_Hh, P_WMinus, 5520)
      call createCurrent(P_ZQ, P_ghZAnti, P_WPlus, P_ghWm, 6177)
      call createCurrent(P_Hl, P_Hl, P_ghWm, P_ghWmAnti, 6575)
      call createCurrent(P_WQPlus, P_Hl, P_WMinus, P_GQ0, 2998)
      call createCurrent(P_A, P_WPlus, P_Ha, P_HMinus, 4862)
      call createCurrent(P_ghWp, P_HMinus, P_G0, P_ghZAnti, 9831)
      call createCurrent(P_s, P_sAnti, P_Ha, 3483)
      call createCurrent(P_HMinus, P_ghWmAnti, P_Ha, P_ghA, 7873)
      call createCurrent(P_HlQ, P_HPlus, P_Hl, P_GQMinus, 7084)
      call createCurrent(P_GQMinus, P_A, P_GPlus, P_AQ, 2417)
      call createCurrent(P_A, P_ghWmAnti, P_WMinus, P_ghZ, 8403)
      call createCurrent(P_AQ, P_A, P_WPlus, P_WQMinus, 7207)
      call createCurrent(P_ghWp, P_Z, P_ghWpAnti, P_A, 1435)
      call createCurrent(P_GQMinus, P_HPlus, P_Hl, P_HlQ, 6488)
      call createCurrent(P_GPlus, P_GMinus, P_Hl, P_Hh, 6544)
      call createCurrent(P_GMinus, P_G0, P_ghWp, P_ghAAnti, 8014)
      call createCurrent(P_uAnti, P_Hh, P_u, 7767)
      call createCurrent(P_muMinus, P_Ha, P_muPlus, 5907)
      call createCurrent(P_WPlus, P_Z, P_ghAAnti, P_ghWm, 1104)
      call createCurrent(P_d, P_dAnti, P_g, 3436)
      call createCurrent(P_Ha, P_g, P_g, P_G0, 10765)
      call createCurrent(P_ghZ, P_Z, P_ghWpAnti, P_WQPlus, 5302)
      call createCurrent(P_ghA, P_HPlus, P_GMinus, P_ghZAnti, 9132)
      call createCurrent(P_HQMinus, P_HPlus, P_ghA, P_ghAAnti, 2884)
      call createCurrent(P_ghWpAnti, P_ghA, P_HPlus, P_Hl, 140)
      call createCurrent(P_ZQ, P_WPlus, P_GQMinus, 7417)
      call createCurrent(P_GMinus, P_GPlus, P_WMinus, P_WPlus, 980)
      call createCurrent(P_AQ, P_WMinus, P_HQPlus, 8389)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_Hl, 2346)
      call createCurrent(P_ghWpAnti, P_ghA, P_GPlus, P_Hl, 4176)
      call createCurrent(P_GQMinus, P_GPlus, P_Hl, P_HaQ, 2123)
      call createCurrent(P_muMinus, P_muPlus, P_HlQ, 1960)
      call createCurrent(P_WMinus, P_Ha, P_HPlus, P_Z, 7674)
      call createCurrent(P_GQMinus, P_Ha, P_Ha, P_HQPlus, 2224)
      call createCurrent(P_t, P_tAnti, P_GQ0, 2585)
      call createCurrent(P_WQMinus, P_HPlus, P_AQ, 666)
      call createCurrent(P_HlQ, P_GMinus, P_GPlus, P_HhQ, 2809)
      call createCurrent(P_ghWp, P_WMinus, P_ghWmAnti, P_WMinus, 7421)
      call createCurrent(P_HlQ, P_Z, P_WPlus, P_GQMinus, 8387)
      call createCurrent(P_muMinus, P_WPlus, P_nu_muAnti, 6068)
      call createCurrent(P_GQPlus, P_HMinus, P_HPlus, P_HQMinus, 2677)
      call createCurrent(P_WQMinus, P_WPlus, P_ghZ, P_ghAAnti, 6242)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_Hl, 316)
      call createCurrent(P_HQPlus, P_HMinus, P_Hh, P_HlQ, 425)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_HaQ, 5242)
      call createCurrent(P_GQPlus, P_HMinus, P_GPlus, P_HQMinus, 356)
      call createCurrent(P_GQ0, P_Hl, P_Hh, P_GQ0, 8761)
      call createCurrent(P_WPlus, P_Ha, P_G0, P_WMinus, 8920)
      call createCurrent(P_A, P_ghZ, P_WMinus, P_ghWmAnti, 5704)
      call createCurrent(P_HaQ, P_Ha, P_Z, P_ZQ, 2452)
      call createCurrent(P_Hl, P_G0, P_Hh, P_Ha, 816)
      call createCurrent(P_GQ0, P_Ha, P_GPlus, P_HQMinus, 4198)
      call createCurrent(P_GQMinus, P_Hl, P_GPlus, P_HaQ, 3390)
      call createCurrent(P_HMinus, P_HPlus, P_G0, P_Ha, 1998)
      call createCurrent(P_Hh, P_G0, P_Ha, P_Hl, 7606)
      call createCurrent(P_WMinus, P_G0, P_Z, P_HPlus, 8968)
      call createCurrent(P_Hl, P_GMinus, P_HPlus, P_Hh, 3007)
      call createCurrent(P_GPlus, P_GMinus, P_WMinus, P_WPlus, 3274)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghZ, P_G0, 4592)
      call createCurrent(P_HPlus, P_GMinus, P_Hh, P_Hl, 4597)
      call createCurrent(P_ghWp, P_WMinus, P_ghWmAnti, P_WQMinus, 5137)
      call createCurrent(P_GMinus, P_Hl, P_Hh, P_GPlus, 6707)
      call createCurrent(P_GMinus, P_Ha, P_WPlus, P_A, 9583)
      call createCurrent(P_ghA, P_Z, P_WMinus, P_ghWmAnti, 2917)
      call createCurrent(P_A, P_WMinus, P_ghZAnti, P_ghWp, 3699)
      call createCurrent(P_GQMinus, P_GPlus, P_ghZ, P_ghZAnti, 8293)
      call createCurrent(P_GQMinus, P_Hl, P_ghZAnti, P_ghWp, 8325)
      call createCurrent(P_HlQ, P_WMinus, P_WPlus, P_HlQ, 4676)
      call createCurrent(P_dAnti, P_Z, P_d, 705)
      call createCurrent(P_HlQ, P_GMinus, P_WPlus, P_ZQ, 8624)
      call createCurrent(P_G0, P_GPlus, P_HMinus, P_G0, 8338)
      call createCurrent(P_HlQ, P_GMinus, P_HPlus, P_HaQ, 5220)
      call createCurrent(P_g, P_ghG, P_ghGAnti, 7566)
      call createCurrent(P_AQ, P_uAnti, P_u, 2373)
      call createCurrent(P_ghA, P_A, P_ghWmAnti, P_WMinus, 7439)
      call createCurrent(P_GQ0, P_G0, P_Hl, P_HlQ, 717)
      call createCurrent(P_A, P_WPlus, P_GMinus, P_G0, 4283)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghWp, P_WQMinus, 5747)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hl, P_GQPlus, 5183)
      call createCurrent(P_Hh, P_ghWm, P_GPlus, P_ghZAnti, 6210)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_Hl, P_ghZ, 1958)
      call createCurrent(P_HQMinus, P_Hl, P_GQPlus, 5737)
      call createCurrent(P_GQMinus, P_ghWp, P_Hl, P_ghZAnti, 7513)
      call createCurrent(P_G0, P_Hh, P_Z, 460)
      call createCurrent(P_ghWp, P_Ha, P_ghWpAnti, P_Ha, 6668)
      call createCurrent(P_ghWpAnti, P_ghWp, P_A, P_A, 8477)
      call createCurrent(P_GQPlus, P_ghZ, P_Hh, P_ghWpAnti, 375)
      call createCurrent(P_GPlus, P_Hl, P_Ha, P_HMinus, 366)
      call createCurrent(P_HQMinus, P_HPlus, P_ghZ, P_ghAAnti, 3116)
      call createCurrent(P_Z, P_WMinus, P_ghAAnti, P_ghWp, 3380)
      call createCurrent(P_GQMinus, P_GPlus, P_A, P_ZQ, 3350)
      call createCurrent(P_ghZ, P_G0, P_GPlus, P_ghWpAnti, 5510)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hh, P_HQMinus, 8135)
      call createCurrent(P_AQ, P_ghZAnti, P_WPlus, P_ghWm, 5043)
      call createCurrent(P_ghZAnti, P_ghWp, P_Ha, P_GMinus, 9446)
      call createCurrent(P_HQMinus, P_Hl, P_Z, P_WQPlus, 5988)
      call createCurrent(P_HQPlus, P_ghZAnti, P_Hl, P_ghWm, 5171)
      call createCurrent(P_WMinus, P_ghZ, P_WPlus, P_ghZAnti, 7386)
      call createCurrent(P_WQMinus, P_ghA, P_ghWmAnti, 2445)
      call createCurrent(P_HQMinus, P_G0, P_GQPlus, 8719)
      call createCurrent(P_GPlus, P_ghWm, P_ghZAnti, P_Ha, 9628)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghA, P_Hh, 4026)
      call createCurrent(P_A, P_Ha, P_WMinus, P_GPlus, 9553)
      call createCurrent(P_HQMinus, P_GPlus, P_HMinus, P_GQPlus, 6552)
      call createCurrent(P_Hh, P_WMinus, P_Z, P_GPlus, 476)
      call createCurrent(P_GQPlus, P_Z, P_GQMinus, 2055)
      call createCurrent(P_ghWpAnti, P_HPlus, P_Hh, P_ghA, 5719)
      call createCurrent(P_ghZAnti, P_HPlus, P_G0, P_ghWm, 10057)
      call createCurrent(P_Z, P_WMinus, P_HPlus, P_Ha, 2025)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_HlQ, 8675)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghWm, P_WPlus, 3089)
      call createCurrent(P_HPlus, P_Hh, P_Hl, P_HMinus, 8183)
      call createCurrent(P_WQMinus, P_WPlus, P_GMinus, P_GQPlus, 7538)
      call createCurrent(P_GQ0, P_GPlus, P_Ha, P_GQMinus, 2225)
      call createCurrent(P_AQ, P_Hl, P_GPlus, P_WQMinus, 6654)
      call createCurrent(P_GQPlus, P_GMinus, P_HlQ, 4217)
      call createCurrent(P_GQPlus, P_GMinus, P_ghZAnti, P_ghA, 3921)
      call createCurrent(P_HPlus, P_Hh, P_HMinus, 2159)
      call createCurrent(P_ZQ, P_GPlus, P_GMinus, P_ZQ, 5115)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_Ha, 10339)
      call createCurrent(P_HhQ, P_GPlus, P_Ha, P_HQMinus, 3914)
      call createCurrent(P_HMinus, P_ghZ, P_ghZAnti, P_HPlus, 6289)
      call createCurrent(P_HlQ, P_Hl, P_HPlus, P_GQMinus, 5375)
      call createCurrent(P_HPlus, P_ghWm, P_ghZAnti, P_Hh, 4854)
      call createCurrent(P_ghWp, P_G0, P_ghAAnti, P_GMinus, 1949)
      call createCurrent(P_GQ0, P_eMinus, P_ePlus, 8592)
      call createCurrent(P_GPlus, P_G0, P_Ha, P_GMinus, 2140)
      call createCurrent(P_HMinus, P_ghWp, P_ghZAnti, P_Hh, 2371)
      call createCurrent(P_A, P_eMinus, P_ePlus, 3087)
      call createCurrent(P_Ha, P_G0, P_Hh, P_Hh, 3578)
      call createCurrent(P_AQ, P_GPlus, P_G0, P_WQMinus, 5625)
      call createCurrent(P_WMinus, P_A, P_WPlus, 4387)
      call createCurrent(P_WPlus, P_A, P_Hh, P_HMinus, 6085)
      call createCurrent(P_WMinus, P_WPlus, P_HPlus, P_HMinus, 4587)
      call createCurrent(P_Hh, P_Hl, P_ghZAnti, P_ghZ, 9830)
      call createCurrent(P_Hl, P_G0, P_HPlus, P_GMinus, 5312)
      call createCurrent(P_GMinus, P_Ha, P_ghWp, P_ghAAnti, 9016)
      call createCurrent(P_WQPlus, P_muMinus, P_nu_muAnti, 5581)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_Hl, 2112)
      call createCurrent(P_G0, P_ghWmAnti, P_HMinus, P_ghZ, 9064)
      call createCurrent(P_Ha, P_muPlus, P_muMinus, 5618)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghZ, 5437)
      call createCurrent(P_HPlus, P_ghA, P_Hl, P_ghWpAnti, 5906)
      call createCurrent(P_ghZAnti, P_HPlus, P_GMinus, P_ghZ, 10026)
      call createCurrent(P_nu_tau, P_tauPlus, P_HMinus, 2617)
      call createCurrent(P_HlQ, P_GMinus, P_A, P_WQPlus, 4826)
      call createCurrent(P_HQPlus, P_A, P_Z, P_HQMinus, 1992)
      call createCurrent(P_Hh, P_ghZAnti, P_GPlus, P_ghWm, 2507)
      call createCurrent(P_GPlus, P_HMinus, P_GPlus, P_GMinus, 4038)
      call createCurrent(P_GPlus, P_Hh, P_GMinus, P_Ha, 9397)
      call createCurrent(P_WQPlus, P_Z, P_Hl, P_HQMinus, 4591)
      call createCurrent(P_ghWm, P_ghAAnti, P_WPlus, 8226)
      call createCurrent(P_HhQ, P_GPlus, P_Hl, P_HQMinus, 675)
      call createCurrent(P_GPlus, P_A, P_GMinus, P_Z, 274)
      call createCurrent(P_WMinus, P_WPlus, P_G0, P_G0, 4358)
      call createCurrent(P_GMinus, P_Ha, P_GPlus, P_G0, 609)
      call createCurrent(P_HlQ, P_Hl, P_ghWmAnti, P_ghWm, 8150)
      call createCurrent(P_WMinus, P_GPlus, P_A, P_Ha, 9234)
      call createCurrent(P_Hh, P_WPlus, P_WMinus, P_Hl, 9109)
      call createCurrent(P_HPlus, P_A, P_Ha, P_WMinus, 8670)
      call createCurrent(P_WQMinus, P_WPlus, P_Z, P_ZQ, 8058)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghA, P_GQPlus, 2883)
      call createCurrent(P_Hl, P_Hl, P_GPlus, P_HMinus, 1266)
      call createCurrent(P_Z, P_Z, P_HMinus, P_GPlus, 10234)
      call createCurrent(P_G0, P_HPlus, P_G0, P_GMinus, 1826)
      call createCurrent(P_ghWm, P_WPlus, P_ghWmAnti, P_WMinus, 2734)
      call createCurrent(P_WPlus, P_HMinus, P_A, P_Ha, 3907)
      call createCurrent(P_ghAAnti, P_WPlus, P_WMinus, P_ghZ, 263)
      call createCurrent(P_HhQ, P_WMinus, P_A, P_HQPlus, 1221)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hh, P_HQPlus, 6209)
      call createCurrent(P_G0, P_G0, P_GMinus, P_GPlus, 5298)
      call createCurrent(P_Hh, P_HMinus, P_G0, P_GPlus, 1255)
      call createCurrent(P_GQPlus, P_A, P_Z, P_GQMinus, 3531)
      call createCurrent(P_WPlus, P_WMinus, P_Hh, P_Hh, 1076)
      call createCurrent(P_Hl, P_HMinus, P_GPlus, P_G0, 4035)
      call createCurrent(P_Hl, P_GPlus, P_ghWm, P_ghAAnti, 464)
      call createCurrent(P_HMinus, P_ghWmAnti, P_HPlus, P_ghWm, 27)
      call createCurrent(P_AQ, P_WMinus, P_Hh, P_HQPlus, 2759)
      call createCurrent(P_ghWmAnti, P_HMinus, P_Hh, P_ghZ, 3804)
      call createCurrent(P_ghWp, P_Hl, P_ghAAnti, P_HMinus, 1231)
      call createCurrent(P_HaQ, P_bAnti, P_b, 2407)
      call createCurrent(P_HPlus, P_A, P_A, P_HMinus, 7014)
      call createCurrent(P_Hl, P_GPlus, P_ghAAnti, P_ghWm, 5795)
      call createCurrent(P_Z, P_Hl, P_GMinus, P_WPlus, 7175)
      call createCurrent(P_HPlus, P_GMinus, P_Hl, P_Hl, 105)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_G0, 2818)
      call createCurrent(P_HPlus, P_ghWpAnti, P_Hl, P_ghA, 5275)
      call createCurrent(P_ghAAnti, P_HPlus, P_Hh, P_ghWm, 421)
      call createCurrent(P_WPlus, P_WMinus, P_Ha, P_G0, 9342)
      call createCurrent(P_Ha, P_Z, P_Hl, 6131)
      call createCurrent(P_AQ, P_A, P_ghWmAnti, P_ghWm, 3999)
      call createCurrent(P_ghZAnti, P_ghZ, P_WMinus, P_WQPlus, 7980)
      call createCurrent(P_A, P_ghAAnti, P_ghWm, P_WPlus, 8223)
      call createCurrent(P_ZQ, P_Ha, P_Ha, P_ZQ, 4656)
      call createCurrent(P_WQPlus, P_ghWmAnti, P_WMinus, P_ghWm, 8756)
      call createCurrent(P_Hh, P_Z, P_HMinus, P_WPlus, 7267)
      call createCurrent(P_GQ0, P_ghZAnti, P_GPlus, P_ghWm, 8256)
      call createCurrent(P_GQ0, P_GMinus, P_Hh, P_HQPlus, 8699)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghZ, P_Ha, 10468)
      call createCurrent(P_G0, P_ghA, P_GMinus, P_ghWmAnti, 1415)
      call createCurrent(P_G0, P_GMinus, P_GPlus, P_Hh, 9652)
      call createCurrent(P_G0, P_Hl, P_Z, 1319)
      call createCurrent(P_GQ0, P_GMinus, P_Ha, P_GQPlus, 4320)
      call createCurrent(P_cAnti, P_c, P_Hh, 5166)
      call createCurrent(P_HPlus, P_ghZ, P_GMinus, P_ghZAnti, 9814)
      call createCurrent(P_G0, P_GMinus, P_G0, P_GPlus, 3819)
      call createCurrent(P_HMinus, P_WMinus, P_WPlus, P_GPlus, 9854)
      call createCurrent(P_HlQ, P_GPlus, P_Hh, P_HQMinus, 1983)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hl, P_HQPlus, 2563)
      call createCurrent(P_Hh, P_G0, P_G0, 8144)
      call createCurrent(P_ghAAnti, P_ghWm, P_Hl, P_GPlus, 1537)
      call createCurrent(P_HQMinus, P_Ha, P_Hl, P_GQPlus, 2716)
      call createCurrent(P_HPlus, P_Z, P_WMinus, P_Hl, 8359)
      call createCurrent(P_HPlus, P_Z, P_HMinus, P_A, 2922)
      call createCurrent(P_GQ0, P_G0, P_HPlus, P_HQMinus, 940)
      call createCurrent(P_HPlus, P_GMinus, P_Z, P_Z, 9257)
      call createCurrent(P_ghZAnti, P_G0, P_GMinus, P_ghWp, 2179)
      call createCurrent(P_ghWpAnti, P_Z, P_ghA, P_WPlus, 8288)
      call createCurrent(P_GPlus, P_Z, P_G0, P_WMinus, 211)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_GQ0, 6755)
      call createCurrent(P_HQMinus, P_GMinus, P_HPlus, P_HQPlus, 3352)
      call createCurrent(P_ghWm, P_G0, P_ghAAnti, P_GPlus, 4566)
      call createCurrent(P_ghWp, P_ghZAnti, P_HQMinus, 1083)
      call createCurrent(P_HhQ, P_Hl, P_HhQ, 6843)
      call createCurrent(P_AQ, P_Z, P_ghWmAnti, P_ghWm, 6040)
      call createCurrent(P_HPlus, P_Hl, P_G0, P_GMinus, 3094)
      call createCurrent(P_HhQ, P_GMinus, P_HPlus, P_HlQ, 6166)
      call createCurrent(P_Ha, P_GPlus, P_Hl, P_GMinus, 10289)
      call createCurrent(P_Hl, P_ghZAnti, P_HMinus, P_ghWp, 454)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, 2828)
      call createCurrent(P_HlQ, P_WPlus, P_GQMinus, 2297)
      call createCurrent(P_ghZ, P_WPlus, P_ghAAnti, P_WQMinus, 2838)
      call createCurrent(P_ghA, P_ghWmAnti, P_WMinus, 8347)
      call createCurrent(P_A, P_Z, P_GPlus, P_HMinus, 9825)
      call createCurrent(P_GMinus, P_HPlus, P_Hl, P_Hl, 5330)
      call createCurrent(P_HlQ, P_HMinus, P_G0, P_HQPlus, 3905)
      call createCurrent(P_HQPlus, P_HPlus, P_GMinus, P_GQMinus, 1809)
      call createCurrent(P_Hl, P_G0, P_GMinus, P_GPlus, 9160)
      call createCurrent(P_HQMinus, P_GPlus, P_G0, P_HaQ, 3871)
      call createCurrent(P_Hh, P_s, P_sAnti, 7317)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_Hh, 6217)
      call createCurrent(P_HPlus, P_GMinus, P_ghWm, P_ghWmAnti, 9022)
      call createCurrent(P_G0, P_A, P_Z, P_G0, 9096)
      call createCurrent(P_GQ0, P_GMinus, P_Z, P_WQPlus, 5468)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, 4196)
      call createCurrent(P_Z, P_ghWm, P_Z, P_ghWmAnti, 8553)
      call createCurrent(P_ghWmAnti, P_Hh, P_GMinus, P_ghZ, 1568)
      call createCurrent(P_Ha, P_Z, P_G0, P_A, 10613)
      call createCurrent(P_G0, P_WPlus, P_HMinus, P_Z, 9827)
      call createCurrent(P_GMinus, P_GPlus, P_ghWp, P_ghWpAnti, 351)
      call createCurrent(P_HhQ, P_Hl, P_Hh, P_HhQ, 7036)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghWm, P_Hl, 7110)
      call createCurrent(P_GMinus, P_Hl, P_ghAAnti, P_ghWp, 568)
      call createCurrent(P_GMinus, P_Z, P_WPlus, P_G0, 2799)
      call createCurrent(P_Hl, P_G0, P_GMinus, P_HPlus, 8198)
      call createCurrent(P_GQPlus, P_G0, P_ghWm, P_ghZAnti, 4808)
      call createCurrent(P_HMinus, P_ghWmAnti, P_Hh, P_ghA, 5262)
      call createCurrent(P_HhQ, P_ghWmAnti, P_Hh, P_ghWm, 3644)
      call createCurrent(P_HQPlus, P_Hl, P_ghZ, P_ghWpAnti, 4352)
      call createCurrent(P_GQMinus, P_A, P_WPlus, P_GQ0, 3681)
      call createCurrent(P_WPlus, P_WMinus, P_A, 4517)
      call createCurrent(P_WPlus, P_ghWmAnti, P_WMinus, P_ghWm, 4891)
      call createCurrent(P_HMinus, P_ghZ, P_HPlus, P_ghAAnti, 2477)
      call createCurrent(P_nu_muAnti, P_nu_mu, 9131)
      call createCurrent(P_HPlus, P_WMinus, P_Z, P_Ha, 8481)
      call createCurrent(P_ePlus, P_eMinus, P_HhQ, 3964)
      call createCurrent(P_Ha, P_Ha, P_G0, P_G0, 2161)
      call createCurrent(P_G0, P_Hh, P_HMinus, P_HPlus, 10050)
      call createCurrent(P_HaQ, P_tauPlus, P_tauMinus, 3057)
      call createCurrent(P_G0, P_GPlus, P_Hh, P_HMinus, 3676)
      call createCurrent(P_HMinus, P_HPlus, P_GPlus, P_HMinus, 5405)
      call createCurrent(P_Z, P_ghWmAnti, P_ghWm, 6252)
      call createCurrent(P_GQPlus, P_GMinus, P_HaQ, 7672)
      call createCurrent(P_HPlus, P_Hh, P_WMinus, P_Z, 4140)
      call createCurrent(P_Ha, P_GPlus, P_G0, P_HMinus, 4879)
      call createCurrent(P_HMinus, P_GMinus, P_GPlus, P_GPlus, 5482)
      call createCurrent(P_WQPlus, P_WMinus, P_Z, P_AQ, 1769)
      call createCurrent(P_GQMinus, P_GPlus, P_ghZAnti, P_ghA, 6482)
      call createCurrent(P_Ha, P_GMinus, P_Ha, P_GPlus, 6577)
      call createCurrent(P_dAnti, P_u, P_WMinus, 374)
      call createCurrent(P_HaQ, P_ghWmAnti, P_Hh, P_ghWm, 5222)
      call createCurrent(P_ghZ, P_ghAAnti, P_HMinus, P_GPlus, 9403)
      call createCurrent(P_g, P_g, P_GPlus, P_GMinus, 10510)
      call createCurrent(P_u, P_uAnti, P_Hl, 2835)
      call createCurrent(P_HaQ, P_Ha, P_HhQ, 7342)
      call createCurrent(P_muMinus, P_A, P_muPlus, 2378)
      call createCurrent(P_HMinus, P_ghA, P_ghAAnti, P_GPlus, 9949)
      call createCurrent(P_HQPlus, P_G0, P_GMinus, P_HaQ, 5555)
      call createCurrent(P_bAnti, P_b, P_GQ0, 1086)
      call createCurrent(P_HhQ, P_Z, P_ZQ, 407)
      call createCurrent(P_HQPlus, P_WMinus, P_ZQ, 6091)
      call createCurrent(P_eMinus, P_nu_eAnti, P_GPlus, 8671)
      call createCurrent(P_Ha, P_WPlus, P_WMinus, P_G0, 10352)
      call createCurrent(P_nu_tau, P_tauPlus, P_WQMinus, 2904)
      call createCurrent(P_Ha, P_Hh, P_GPlus, P_GMinus, 9660)
      call createCurrent(P_ghZ, P_Hh, P_ghWpAnti, P_HPlus, 6341)
      call createCurrent(P_HhQ, P_HPlus, P_Ha, P_HQMinus, 3097)
      call createCurrent(P_GQPlus, P_Hl, P_HMinus, P_HlQ, 6030)
      call createCurrent(P_G0, P_G0, P_ghWpAnti, P_ghWp, 3978)
      call createCurrent(P_ghWpAnti, P_GMinus, P_ghWp, P_GPlus, 674)
      call createCurrent(P_ghA, P_GPlus, P_ghZAnti, P_HMinus, 10122)
      call createCurrent(P_Z, P_t, P_tAnti, 4089)
      call createCurrent(P_d, P_G0, P_dAnti, 4240)
      call createCurrent(P_ghWpAnti, P_GPlus, P_G0, P_ghZ, 3402)
      call createCurrent(P_HaQ, P_Hh, P_HPlus, P_GQMinus, 8741)
      call createCurrent(P_AQ, P_Ha, P_HMinus, P_WQPlus, 7033)
      call createCurrent(P_WQMinus, P_G0, P_GPlus, P_ZQ, 5052)
      call createCurrent(P_ghWpAnti, P_HPlus, P_Hl, P_ghA, 4564)
      call createCurrent(P_ghWpAnti, P_ghWp, P_GPlus, P_HMinus, 9752)
      call createCurrent(P_Hh, P_Hl, P_WMinus, P_WPlus, 9943)
      call createCurrent(P_GMinus, P_GPlus, P_Hh, P_G0, 10425)
      call createCurrent(P_HaQ, P_Hh, P_Ha, P_HhQ, 2493)
      call createCurrent(P_ghA, P_HMinus, P_ghWmAnti, P_Ha, 4991)
      call createCurrent(P_muPlus, P_Hl, P_muMinus, 7325)
      call createCurrent(P_Hh, P_HMinus, P_GPlus, P_G0, 8419)
      call createCurrent(P_ghAAnti, P_HMinus, P_GPlus, P_ghA, 10052)
      call createCurrent(P_Ha, P_HMinus, P_GPlus, 3066)
      call createCurrent(P_HPlus, P_Hl, P_WMinus, 5910)
      call createCurrent(P_HQPlus, P_Hl, P_GQMinus, 1977)
      call createCurrent(P_ghWmAnti, P_ghA, P_GMinus, P_Hl, 3643)
      call createCurrent(P_Hh, P_ghWp, P_GMinus, P_ghAAnti, 861)
      call createCurrent(P_HMinus, P_Hl, P_GPlus, P_Hl, 8117)
      call createCurrent(P_ghA, P_Ha, P_GPlus, P_ghWpAnti, 9960)
      call createCurrent(P_WQMinus, P_Hh, P_Z, P_GQPlus, 3370)
      call createCurrent(P_HQMinus, P_HPlus, P_HMinus, P_GQPlus, 3994)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_Ha, 2665)
      call createCurrent(P_A, P_HPlus, P_WMinus, P_Ha, 5304)
      call createCurrent(P_GQPlus, P_ghWm, P_Hl, P_ghZAnti, 7159)
      call createCurrent(P_ZQ, P_WPlus, P_GMinus, P_GQ0, 2841)
      call createCurrent(P_HPlus, P_b, P_tAnti, 2244)
      call createCurrent(P_GMinus, P_G0, P_HPlus, P_Hh, 6110)
      call createCurrent(P_Ha, P_GPlus, P_ghAAnti, P_ghWm, 9166)
      call createCurrent(P_muPlus, P_muMinus, P_ZQ, 5649)
      call createCurrent(P_GQMinus, P_ghA, P_GPlus, P_ghAAnti, 729)
      call createCurrent(P_ghA, P_GMinus, P_ghWmAnti, P_G0, 2729)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Z, 1040)
      call createCurrent(P_WQPlus, P_d, P_uAnti, 7727)
      call createCurrent(P_GPlus, P_Ha, P_GMinus, P_Hh, 9870)
      call createCurrent(P_HMinus, P_HPlus, P_WMinus, P_WPlus, 8328)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HMinus, P_HQPlus, 1419)
      call createCurrent(P_GPlus, P_HPlus, P_HMinus, P_HMinus, 1216)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_HhQ, 4324)
      call createCurrent(P_HQMinus, P_ghZAnti, P_HPlus, P_ghZ, 1873)
      call createCurrent(P_HaQ, P_Hl, P_Ha, P_HlQ, 7103)
      call createCurrent(P_GQMinus, P_Hl, P_Hh, P_GQPlus, 5087)
      call createCurrent(P_HlQ, P_HPlus, P_ghZ, P_ghWpAnti, 5041)
      call createCurrent(P_WQPlus, P_WMinus, P_HhQ, 2902)
      call createCurrent(P_HQPlus, P_Hl, P_GMinus, P_HlQ, 7889)
      call createCurrent(P_HMinus, P_Hh, P_ghA, P_ghWmAnti, 6676)
      call createCurrent(P_G0, P_Hl, P_HPlus, P_GMinus, 3249)
      call createCurrent(P_G0, P_ghWpAnti, P_HPlus, P_ghA, 9812)
      call createCurrent(P_HQPlus, P_Hh, P_GMinus, P_GQ0, 4957)
      call createCurrent(P_HQMinus, P_HPlus, P_GMinus, P_GQPlus, 7327)
      call createCurrent(P_HQMinus, P_GPlus, P_HaQ, 4432)
      call createCurrent(P_tAnti, P_t, P_gQ, 4779)
      call createCurrent(P_Hh, P_ghAAnti, P_GMinus, P_ghWp, 5391)
      call createCurrent(P_Ha, P_Hl, P_Hh, P_Ha, 3805)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghA, 2575)
      call createCurrent(P_HQPlus, P_GPlus, P_GMinus, P_HQMinus, 6188)
      call createCurrent(P_ghAAnti, P_ghZ, P_WPlus, P_WQMinus, 6915)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_HlQ, 5143)
      call createCurrent(P_WPlus, P_G0, P_GMinus, P_A, 4529)
      call createCurrent(P_HaQ, P_HPlus, P_ghZ, P_ghWpAnti, 3184)
      call createCurrent(P_WMinus, P_ghAAnti, P_ghA, P_WPlus, 3982)
      call createCurrent(P_ghA, P_Hl, P_ghWmAnti, P_GMinus, 1186)
      call createCurrent(P_G0, P_ghZ, P_ghZAnti, P_G0, 2496)
      call createCurrent(P_Ha, P_A, P_A, P_G0, 10751)
      call createCurrent(P_ghWm, P_WMinus, P_ghWmAnti, P_WQPlus, 6132)
      call createCurrent(P_HPlus, P_ghWmAnti, P_ghWm, P_HMinus, 1024)
      call createCurrent(P_ghZ, P_HMinus, P_ghAAnti, P_HPlus, 3178)
      call createCurrent(P_A, P_WPlus, P_Hl, P_HMinus, 723)
      call createCurrent(P_HMinus, P_WPlus, P_WMinus, P_GPlus, 9182)
      call createCurrent(P_tauPlus, P_Ha, P_tauMinus, 989)
      call createCurrent(P_GMinus, P_GPlus, P_ghWmAnti, P_ghWm, 2444)
      call createCurrent(P_Z, P_GMinus, P_WPlus, P_Hl, 6554)
      call createCurrent(P_muPlus, P_muMinus, P_Z, 6499)
      call createCurrent(P_GQ0, P_ghWmAnti, P_Hh, P_ghWm, 8050)
      call createCurrent(P_HPlus, P_Z, P_A, P_GMinus, 9154)
      call createCurrent(P_HQMinus, P_HPlus, P_HPlus, P_GQMinus, 600)
      call createCurrent(P_Hh, P_GMinus, P_ghZAnti, P_ghWp, 7555)
      call createCurrent(P_HQMinus, P_GPlus, P_G0, P_GQ0, 3789)
      call createCurrent(P_Ha, P_HMinus, P_HPlus, P_Hh, 9129)
      call createCurrent(P_ghWm, P_Hh, P_ghWmAnti, P_GQ0, 5306)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Z, P_Z, 862)
      call createCurrent(P_Hh, P_ghWp, P_ghWpAnti, P_Hl, 8900)
      call createCurrent(P_A, P_GMinus, P_Z, P_GPlus, 5778)
      call createCurrent(P_GQ0, P_GPlus, P_ghZ, P_ghWpAnti, 269)
      call createCurrent(P_HhQ, P_HPlus, P_WMinus, P_ZQ, 3651)
      call createCurrent(P_GQPlus, P_Hl, P_HMinus, P_HhQ, 7565)
      call createCurrent(P_ZQ, P_Hl, P_WMinus, P_GQPlus, 2187)
      call createCurrent(P_ZQ, P_WMinus, P_HPlus, P_HaQ, 7697)
      call createCurrent(P_Z, P_Z, P_WMinus, P_WPlus, 1532)
      call createCurrent(P_WMinus, P_ghWpAnti, P_ghWp, P_WPlus, 5871)
      call createCurrent(P_t, P_A, P_tAnti, 893)
      call createCurrent(P_WMinus, P_Ha, P_HPlus, 2659)
      call createCurrent(P_ghWp, P_ghWpAnti, P_GPlus, P_GQMinus, 6683)
      call createCurrent(P_HPlus, P_A, P_Z, P_GMinus, 9585)
      call createCurrent(P_GQ0, P_HPlus, P_GMinus, P_HlQ, 5755)
      call createCurrent(P_HPlus, P_A, P_Hh, P_WMinus, 4975)
      call createCurrent(P_Z, P_A, P_Hh, 9090)
      call createCurrent(P_WQPlus, P_HMinus, P_Hl, P_ZQ, 2560)
      call createCurrent(P_GMinus, P_sAnti, P_c, 5503)
      call createCurrent(P_HPlus, P_ghZ, P_ghZAnti, P_HMinus, 7718)
      call createCurrent(P_HPlus, P_ghZAnti, P_Hh, P_ghWm, 8516)
      call createCurrent(P_GPlus, P_HMinus, P_GMinus, P_GPlus, 6184)
      call createCurrent(P_GMinus, P_Ha, P_WPlus, 10446)
      call createCurrent(P_Hh, P_Ha, P_Ha, 7092)
      call createCurrent(P_WQPlus, P_Z, P_GMinus, P_GQ0, 5996)
      call createCurrent(P_G0, P_Ha, P_g, P_g, 10626)
      call createCurrent(P_GQMinus, P_Hh, P_Z, P_WQPlus, 2703)
      call createCurrent(P_ghAAnti, P_GPlus, P_G0, P_ghWm, 8011)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghWm, P_Z, 7161)
      call createCurrent(P_ghZAnti, P_A, P_ghWm, P_WPlus, 4277)
      call createCurrent(P_HaQ, P_HMinus, P_GPlus, P_GQ0, 2514)
      call createCurrent(P_cAnti, P_c, P_AQ, 1257)
      call createCurrent(P_HPlus, P_ghZ, P_ghWpAnti, P_G0, 9861)
      call createCurrent(P_G0, P_ghWpAnti, P_G0, P_ghWp, 6287)
      call createCurrent(P_GQMinus, P_Hl, P_HQPlus, 7772)
      call createCurrent(P_HPlus, P_HMinus, P_A, P_A, 8073)
      call createCurrent(P_HMinus, P_Hl, P_A, P_WPlus, 3665)
      call createCurrent(P_ghA, P_GMinus, P_ghAAnti, P_GQPlus, 5813)
      call createCurrent(P_tauPlus, P_nu_tau, P_GQMinus, 4800)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghA, P_Ha, 9213)
      call createCurrent(P_ghWm, P_ghAAnti, P_A, P_WPlus, 7002)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hh, P_HhQ, 7634)
      call createCurrent(P_ghWm, P_HPlus, P_Hh, P_ghAAnti, 4055)
      call createCurrent(P_ghWm, P_Z, P_ghWmAnti, P_ZQ, 3589)
      call createCurrent(P_HlQ, P_HPlus, P_GMinus, P_HhQ, 1428)
      call createCurrent(P_HQPlus, P_Hl, P_HMinus, P_HlQ, 5244)
      call createCurrent(P_ghWmAnti, P_Hh, P_Hh, P_ghWm, 5999)
      call createCurrent(P_dAnti, P_G0, P_d, 1074)
      call createCurrent(P_ghZ, P_ghWpAnti, P_WPlus, P_Z, 8613)
      call createCurrent(P_tauPlus, P_tauMinus, P_Z, 2948)
      call createCurrent(P_ghZAnti, P_Hh, P_GMinus, P_ghWp, 3755)
      call createCurrent(P_GMinus, P_Hl, P_ghWmAnti, P_ghA, 2231)
      call createCurrent(P_A, P_A, P_g, P_g, 10692)
      call createCurrent(P_Z, P_G0, P_A, P_G0, 9901)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hh, P_GMinus, 1945)
      call createCurrent(P_GQ0, P_WMinus, P_G0, P_WQPlus, 1726)
      call createCurrent(P_GQMinus, P_G0, P_WPlus, P_AQ, 4120)
      call createCurrent(P_WPlus, P_cAnti, P_s, 5309)
      call createCurrent(P_G0, P_Hl, P_Ha, 5319)
      call createCurrent(P_G0, P_HPlus, P_WMinus, P_A, 9736)
      call createCurrent(P_ghWmAnti, P_Z, P_ghWm, P_A, 8361)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_HhQ, 1485)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_Hh, 1767)
      call createCurrent(P_GQ0, P_HMinus, P_G0, P_HQPlus, 8555)
      call createCurrent(P_Z, P_GPlus, P_HMinus, P_Z, 10251)
      call createCurrent(P_AQ, P_WMinus, P_WPlus, P_AQ, 7354)
      call createCurrent(P_ghWpAnti, P_G0, P_ghWp, P_GQ0, 2566)
      call createCurrent(P_GMinus, P_ghWpAnti, P_ghWp, P_HPlus, 10008)
      call createCurrent(P_ghZ, P_HPlus, P_Ha, P_ghWpAnti, 187)
      call createCurrent(P_HQMinus, P_G0, P_G0, P_HQPlus, 7625)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hh, P_HhQ, 8405)
      call createCurrent(P_A, P_Z, P_Ha, P_Ha, 9388)
      call createCurrent(P_HhQ, P_c, P_cAnti, 3653)
      call createCurrent(P_Ha, P_GMinus, P_Ha, P_HPlus, 4079)
      call createCurrent(P_ghAAnti, P_ghWm, P_G0, P_HPlus, 10103)
      call createCurrent(P_Hh, P_GMinus, P_G0, P_HPlus, 4207)
      call createCurrent(P_HhQ, P_ghWp, P_GMinus, P_ghAAnti, 8103)
      call createCurrent(P_GPlus, P_WMinus, P_G0, P_Z, 8664)
      call createCurrent(P_GPlus, P_Z, P_WMinus, P_Hh, 162)
      call createCurrent(P_Hh, P_g, P_g, P_Hl, 10500)
      call createCurrent(P_G0, P_ghWpAnti, P_GPlus, P_ghZ, 7844)
      call createCurrent(P_Hh, P_HPlus, P_ghWm, P_ghZAnti, 2607)
      call createCurrent(P_HQPlus, P_Hh, P_Hh, P_HQMinus, 5208)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hl, P_GQPlus, 2974)
      call createCurrent(P_ghZAnti, P_ghZ, P_HPlus, P_HMinus, 8001)
      call createCurrent(P_GMinus, P_HPlus, P_Hh, P_Ha, 1350)
      call createCurrent(P_GPlus, P_Ha, P_ghA, P_ghWpAnti, 9955)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_Ha, 10149)
      call createCurrent(P_g, P_Hh, P_Hh, P_g, 10686)
      call createCurrent(P_ghAAnti, P_Hl, P_HMinus, P_ghWp, 2779)
      call createCurrent(P_HlQ, P_GMinus, P_G0, P_HQPlus, 8534)
      call createCurrent(P_WMinus, P_Hl, P_GPlus, P_Z, 7779)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_Ha, 8879)
      call createCurrent(P_GPlus, P_WPlus, P_WMinus, P_GMinus, 8113)
      call createCurrent(P_HMinus, P_Hh, P_HPlus, P_Hl, 7447)
      call createCurrent(P_GMinus, P_GPlus, P_Ha, P_Ha, 7862)
      call createCurrent(P_WPlus, P_A, P_Hl, P_HMinus, 6680)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghWm, P_Hh, 5011)
      call createCurrent(P_GMinus, P_HPlus, P_G0, P_Hh, 4946)
      call createCurrent(P_Ha, P_HMinus, P_HPlus, P_G0, 4610)
      call createCurrent(P_WMinus, P_HPlus, P_Hh, P_A, 8083)
      call createCurrent(P_HQMinus, P_GPlus, P_Hl, P_HhQ, 3475)
      call createCurrent(P_HlQ, P_HMinus, P_HQPlus, 393)
      call createCurrent(P_WQPlus, P_Ha, P_WMinus, P_HlQ, 911)
      call createCurrent(P_ghAAnti, P_ghWp, P_A, P_WMinus, 7891)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_Ha, 9941)
      call createCurrent(P_WMinus, P_WPlus, P_GMinus, P_GPlus, 2359)
      call createCurrent(P_Hh, P_GMinus, P_HPlus, 7035)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, 2166)
      call createCurrent(P_HlQ, P_GPlus, P_ghZ, P_ghWpAnti, 7396)
      call createCurrent(P_Hh, P_A, P_G0, 8998)
      call createCurrent(P_GPlus, P_A, P_Z, P_GMinus, 2649)
      call createCurrent(P_HMinus, P_G0, P_HPlus, P_G0, 599)
      call createCurrent(P_GPlus, P_Hh, P_Hh, P_HMinus, 1900)
      call createCurrent(P_Hl, P_ghZAnti, P_ghWp, P_HMinus, 6293)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghA, P_WQMinus, 836)
      call createCurrent(P_HPlus, P_ghWp, P_GMinus, P_ghWpAnti, 8946)
      call createCurrent(P_G0, P_HMinus, P_Ha, P_GPlus, 8318)
      call createCurrent(P_eMinus, P_Z, P_ePlus, 8102)
      call createCurrent(P_HhQ, P_G0, P_GMinus, P_HQPlus, 2151)
      call createCurrent(P_g, P_g, P_Hl, P_Hl, 10586)
      call createCurrent(P_GQPlus, P_HMinus, P_GQ0, 582)
      call createCurrent(P_GQ0, P_GPlus, P_Hl, P_GQMinus, 5854)
      call createCurrent(P_G0, P_HPlus, P_GMinus, P_Hl, 7706)
      call createCurrent(P_HaQ, P_Hh, P_ghWm, P_ghWmAnti, 5039)
      call createCurrent(P_GQPlus, P_ghA, P_GMinus, P_ghZAnti, 7360)
      call createCurrent(P_GQMinus, P_GPlus, P_HaQ, 2498)
      call createCurrent(P_Hh, P_Hl, P_Hh, P_Hh, 4799)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hh, P_GQMinus, 8550)
      call createCurrent(P_WPlus, P_Hh, P_HMinus, P_Z, 719)
      call createCurrent(P_Ha, P_ghWm, P_HPlus, P_ghAAnti, 7205)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_Hl, 1233)
      call createCurrent(P_ghAAnti, P_GMinus, P_GPlus, P_ghA, 2970)
      call createCurrent(P_ZQ, P_ghZAnti, P_WMinus, P_ghWp, 4999)
      call createCurrent(P_Hl, P_Hl, P_ghZAnti, P_ghZ, 8496)
      call createCurrent(P_WMinus, P_HPlus, P_WPlus, P_HMinus, 7219)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghWp, P_Ha, 8694)
      call createCurrent(P_HQPlus, P_G0, P_HMinus, P_GQ0, 2946)
      call createCurrent(P_HPlus, P_ghWpAnti, P_Hl, P_ghZ, 7487)
      call createCurrent(P_ghZ, P_GMinus, P_ghZAnti, P_GQPlus, 8600)
      call createCurrent(P_ghWm, P_ghWmAnti, P_A, P_AQ, 6961)
      call createCurrent(P_GQMinus, P_A, P_WPlus, P_HhQ, 1744)
      call createCurrent(P_GQPlus, P_Z, P_WMinus, P_HlQ, 679)
      call createCurrent(P_Z, P_ghAAnti, P_ghWp, P_WMinus, 7420)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_GQ0, 6964)
      call createCurrent(P_ZQ, P_WMinus, P_G0, P_GQPlus, 1516)
      call createCurrent(P_Hl, P_HPlus, P_Ha, P_GMinus, 2836)
      call createCurrent(P_WQPlus, P_GMinus, P_A, P_HlQ, 8118)
      call createCurrent(P_HPlus, P_ghA, P_ghAAnti, P_GMinus, 9095)
      call createCurrent(P_GMinus, P_Hh, P_Hl, P_HPlus, 3546)
      call createCurrent(P_Ha, P_HPlus, P_ghAAnti, P_ghWm, 7221)
      call createCurrent(P_Hh, P_ghWmAnti, P_GMinus, P_ghA, 5231)
      call createCurrent(P_A, P_Ha, P_WPlus, P_HMinus, 1115)
      call createCurrent(P_ghZ, P_WPlus, P_WMinus, P_ghZAnti, 2972)
      call createCurrent(P_GQPlus, P_Hh, P_ghWpAnti, P_ghZ, 5893)
      call createCurrent(P_GQ0, P_HPlus, P_HMinus, P_HaQ, 3233)
      call createCurrent(P_GMinus, P_GMinus, P_HPlus, P_HPlus, 4627)
      call createCurrent(P_GMinus, P_Hl, P_GPlus, P_Hh, 6422)
      call createCurrent(P_ghWp, P_ghAAnti, P_Ha, P_GMinus, 10094)
      call createCurrent(P_WMinus, P_GPlus, P_Z, P_Hh, 1124)
      call createCurrent(P_ghA, P_WPlus, P_ghAAnti, P_WMinus, 6044)
      call createCurrent(P_WPlus, P_A, P_ghZAnti, P_ghWm, 3213)
      call createCurrent(P_nu_eAnti, P_eMinus, P_GPlus, 8232)
      call createCurrent(P_ZQ, P_WMinus, P_ghWp, P_ghZAnti, 4116)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_HhQ, 3735)
      call createCurrent(P_eMinus, P_ePlus, P_HlQ, 3144)
      call createCurrent(P_WMinus, P_G0, P_Ha, P_WPlus, 10479)
      call createCurrent(P_HPlus, P_A, P_A, P_GMinus, 9457)
      call createCurrent(P_muPlus, P_muMinus, P_Hh, 2341)
      call createCurrent(P_ghZAnti, P_Z, P_ghWp, P_WMinus, 3533)
      call createCurrent(P_HQMinus, P_HPlus, P_ghZAnti, P_ghZ, 3423)
      call createCurrent(P_A, P_Hl, P_GPlus, P_WMinus, 5616)
      call createCurrent(P_HPlus, P_GMinus, P_Z, P_A, 9889)
      call createCurrent(P_g, P_g, P_Ha, P_G0, 10492)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Ha, P_G0, 10359)
      call createCurrent(P_ghWm, P_ghZAnti, P_G0, P_GQPlus, 463)
      call createCurrent(P_GQ0, P_GPlus, P_Z, P_WQMinus, 7551)
      call createCurrent(P_Z, P_ghWmAnti, P_ghZ, P_WMinus, 2903)
      call createCurrent(P_HlQ, P_HMinus, P_GPlus, P_HlQ, 1041)
      call createCurrent(P_ghA, P_G0, P_HMinus, P_ghWmAnti, 9059)
      call createCurrent(P_A, P_WMinus, P_Z, P_WPlus, 7902)
      call createCurrent(P_ghWp, P_GMinus, P_Hl, P_ghZAnti, 6535)
      call createCurrent(P_ghWp, P_G0, P_ghZAnti, P_GMinus, 4678)
      call createCurrent(P_GPlus, P_ghA, P_ghZAnti, P_HMinus, 8915)
      call createCurrent(P_GPlus, P_Z, P_Hh, P_WMinus, 3438)
      call createCurrent(P_ghZAnti, P_G0, P_Ha, P_ghZ, 8992)
      call createCurrent(P_HaQ, P_HPlus, P_Ha, P_HQMinus, 7707)
      call createCurrent(P_WPlus, P_ghZAnti, P_ghWm, P_A, 360)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HPlus, P_HMinus, 4368)
      call createCurrent(P_HlQ, P_Hh, P_HMinus, P_HQPlus, 2876)
      call createCurrent(P_HaQ, P_ghWpAnti, P_Hl, P_ghWp, 9)
      call createCurrent(P_Ha, P_HPlus, P_ghWpAnti, P_ghZ, 2415)
      call createCurrent(P_G0, P_G0, P_HPlus, P_GMinus, 845)
      call createCurrent(P_HQPlus, P_GMinus, P_GPlus, P_HQMinus, 8167)
      call createCurrent(P_cAnti, P_Z, P_c, 3641)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghWp, P_Ha, 9427)
      call createCurrent(P_A, P_Ha, P_WMinus, P_HPlus, 4221)
      call createCurrent(P_eMinus, P_WPlus, P_nu_eAnti, 1851)
      call createCurrent(P_G0, P_Hl, P_Hh, P_Ha, 167)
      call createCurrent(P_A, P_HMinus, P_WPlus, P_Hl, 4040)
      call createCurrent(P_ghWmAnti, P_GMinus, P_G0, P_ghA, 6912)
      call createCurrent(P_HlQ, P_s, P_sAnti, 7622)
      call createCurrent(P_ghWmAnti, P_GMinus, P_HPlus, P_ghWm, 9179)
      call createCurrent(P_ghWm, P_GPlus, P_G0, P_ghAAnti, 4506)
      call createCurrent(P_WQPlus, P_WMinus, P_AQ, 6129)
      call createCurrent(P_Hh, P_ghWp, P_ghZAnti, P_GMinus, 3124)
      call createCurrent(P_HMinus, P_A, P_Ha, P_WPlus, 7453)
      call createCurrent(P_HhQ, P_Hl, P_HPlus, P_GQMinus, 2250)
      call createCurrent(P_ePlus, P_eMinus, P_ZQ, 8750)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghA, 1713)
      call createCurrent(P_Hl, P_Hh, P_A, P_A, 10681)
      call createCurrent(P_ghZ, P_WMinus, P_WPlus, P_ghZAnti, 860)
      call createCurrent(P_ghZ, P_HPlus, P_Hl, P_ghWpAnti, 8791)
      call createCurrent(P_ghWp, P_Hh, P_ghAAnti, P_GMinus, 4723)
      call createCurrent(P_Hl, P_Ha, P_GPlus, P_HMinus, 8730)
      call createCurrent(P_GQ0, P_Hl, P_Hl, P_GQ0, 7128)
      call createCurrent(P_HhQ, P_G0, P_GPlus, P_GQMinus, 6627)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hl, P_HMinus, 5161)
      call createCurrent(P_GMinus, P_dAnti, P_u, 415)
      call createCurrent(P_A, P_Ha, P_HPlus, P_WMinus, 411)
      call createCurrent(P_GQ0, P_ghZAnti, P_GMinus, P_ghWp, 5164)
      call createCurrent(P_ZQ, P_A, P_GMinus, P_GQPlus, 6953)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_HlQ, 7195)
      call createCurrent(P_ghWp, P_A, P_WMinus, P_ghZAnti, 1906)
      call createCurrent(P_A, P_muPlus, P_muMinus, 3295)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_G0, 9974)
      call createCurrent(P_HhQ, P_GPlus, P_GMinus, P_HlQ, 4941)
      call createCurrent(P_WQMinus, P_Hh, P_HPlus, P_ZQ, 3267)
      call createCurrent(P_Hh, P_HMinus, P_ghZAnti, P_ghWp, 1355)
      call createCurrent(P_ZQ, P_Ha, P_HlQ, 1791)
      call createCurrent(P_Ha, P_Hl, P_HPlus, P_HMinus, 9456)
      call createCurrent(P_GPlus, P_Hh, P_WMinus, P_A, 2232)
      call createCurrent(P_HlQ, P_Hh, P_HhQ, 5370)
      call createCurrent(P_Z, P_WPlus, P_ghWpAnti, P_ghZ, 5001)
      call createCurrent(P_GMinus, P_WPlus, P_G0, 3957)
      call createCurrent(P_ghWm, P_G0, P_ghAAnti, P_HPlus, 9375)
      call createCurrent(P_GQ0, P_ghWmAnti, P_ghWm, 5305)
      call createCurrent(P_GPlus, P_A, P_WMinus, P_G0, 3093)
      call createCurrent(P_GPlus, P_Hh, P_A, P_WMinus, 5542)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghZ, P_HQPlus, 2203)
      call createCurrent(P_GPlus, P_Z, P_HMinus, 9149)
      call createCurrent(P_Hl, P_ghZ, P_ghWpAnti, P_GPlus, 2755)
      call createCurrent(P_Hh, P_Hh, P_Hl, P_Hh, 944)
      call createCurrent(P_WMinus, P_g, P_WPlus, P_g, 10763)
      call createCurrent(P_ghZAnti, P_Hh, P_Hl, P_ghZ, 9599)
      call createCurrent(P_Hl, P_G0, P_Z, 6202)
      call createCurrent(P_WMinus, P_Z, P_GPlus, P_Hh, 1064)
      call createCurrent(P_WQMinus, P_G0, P_GQPlus, 2845)
      call createCurrent(P_G0, P_G0, P_ghWm, P_ghWmAnti, 2413)
      call createCurrent(P_muMinus, P_nu_muAnti, P_WPlus, 7604)
      call createCurrent(P_GPlus, P_G0, P_HMinus, P_Hh, 638)
      call createCurrent(P_Hl, P_HMinus, P_ghWmAnti, P_ghZ, 6780)
      call createCurrent(P_WMinus, P_A, P_Ha, P_GPlus, 9028)
      call createCurrent(P_Z, P_WMinus, P_GPlus, P_G0, 4594)
      call createCurrent(P_GQ0, P_Ha, P_Hh, P_HhQ, 135)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWm, P_HQPlus, 2455)
      call createCurrent(P_tauPlus, P_tauMinus, P_Ha, 5773)
      call createCurrent(P_WQPlus, P_Ha, P_WMinus, P_HaQ, 5286)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghWm, P_HaQ, 6364)
      call createCurrent(P_Ha, P_ghWpAnti, P_Ha, P_ghWp, 6929)
      call createCurrent(P_Z, P_WMinus, P_G0, P_GPlus, 6427)
      call createCurrent(P_AQ, P_Hl, P_WPlus, P_HQMinus, 4365)
      call createCurrent(P_AQ, P_WMinus, P_GQPlus, 4691)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghA, P_HPlus, 9969)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hh, P_HMinus, 6611)
      call createCurrent(P_HhQ, P_Ha, P_WPlus, P_WQMinus, 3422)
      call createCurrent(P_HMinus, P_ghWm, P_ghWmAnti, P_HPlus, 2032)
      call createCurrent(P_G0, P_A, P_Ha, P_Z, 10769)
      call createCurrent(P_HMinus, P_HPlus, P_A, 806)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_Ha, 8472)
      call createCurrent(P_WQPlus, P_WMinus, P_GQ0, 1388)
      call createCurrent(P_Ha, P_HPlus, P_G0, P_HMinus, 7644)
      call createCurrent(P_Hl, P_A, P_WPlus, P_GMinus, 7211)
      call createCurrent(P_GPlus, P_Hh, P_HMinus, 3710)
      call createCurrent(P_Hh, P_Hl, P_GPlus, P_GMinus, 1578)
      call createCurrent(P_HQMinus, P_nu_mu, P_muPlus, 5016)
      call createCurrent(P_Z, P_HMinus, P_HPlus, P_A, 1258)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_HaQ, 8482)
      call createCurrent(P_GMinus, P_HPlus, P_ghWpAnti, P_ghWp, 10058)
      call createCurrent(P_AQ, P_ghAAnti, P_WMinus, P_ghWp, 7569)
      call createCurrent(P_HPlus, P_ghZ, P_ghWpAnti, P_Ha, 3406)
      call createCurrent(P_c, P_cAnti, P_A, 2620)
      call createCurrent(P_ZQ, P_G0, P_G0, P_ZQ, 7708)
      call createCurrent(P_HaQ, P_WPlus, P_Z, P_HQMinus, 7196)
      call createCurrent(P_Hh, P_A, P_HMinus, P_WPlus, 7262)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghA, P_Hl, 6455)
      call createCurrent(P_Ha, P_sAnti, P_s, 3394)
      call createCurrent(P_ghZAnti, P_ghZ, P_HMinus, P_GPlus, 9956)
      call createCurrent(P_GQPlus, P_GMinus, P_G0, P_HhQ, 3417)
      call createCurrent(P_ghA, P_Hl, P_ghWmAnti, P_HMinus, 5363)
      call createCurrent(P_ghA, P_ghZAnti, P_GMinus, P_HPlus, 10113)
      call createCurrent(P_Hl, P_ghZAnti, P_ghWp, P_GMinus, 8292)
      call createCurrent(P_WMinus, P_WPlus, P_Hl, 295)
      call createCurrent(P_HMinus, P_G0, P_GPlus, P_Ha, 4238)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghZ, 8531)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_Ha, 9433)
      call createCurrent(P_HPlus, P_HMinus, P_GMinus, P_HPlus, 1095)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghWm, P_HPlus, 9424)
      call createCurrent(P_GQPlus, P_HMinus, P_HlQ, 7488)
      call createCurrent(P_WQMinus, P_WPlus, P_Ha, P_HaQ, 60)
      call createCurrent(P_ghZ, P_GPlus, P_Hl, P_ghWpAnti, 4084)
      call createCurrent(P_A, P_d, P_dAnti, 5591)
      call createCurrent(P_Ha, P_WMinus, P_Ha, P_WPlus, 5932)
      call createCurrent(P_GPlus, P_HMinus, P_G0, 2721)
      call createCurrent(P_GPlus, P_tauMinus, P_nu_tauAnti, 4870)
      call createCurrent(P_ghWm, P_ghWmAnti, P_G0, P_Ha, 10225)
      call createCurrent(P_GQMinus, P_Ha, P_GPlus, P_GQ0, 4259)
      call createCurrent(P_Hl, P_ghWm, P_Hl, P_ghWmAnti, 5596)
      call createCurrent(P_Ha, P_GMinus, P_HPlus, 8036)
      call createCurrent(P_nu_mu, P_muPlus, P_HQMinus, 1170)
      call createCurrent(P_ZQ, P_muPlus, P_muMinus, 5715)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_Ha, P_ghZ, 4992)
      call createCurrent(P_WQMinus, P_A, P_GPlus, P_HhQ, 1260)
      call createCurrent(P_ePlus, P_eMinus, P_Ha, 523)
      call createCurrent(P_WQPlus, P_WMinus, P_Hl, P_HaQ, 3537)
      call createCurrent(P_Hh, P_Hh, P_GMinus, P_HPlus, 519)
      call createCurrent(P_GMinus, P_GPlus, P_Hh, P_Ha, 9130)
      call createCurrent(P_Z, P_Hh, P_A, P_Hl, 10529)
      call createCurrent(P_HPlus, P_G0, P_GMinus, P_Hl, 874)
      call createCurrent(P_GPlus, P_Hl, P_HMinus, 7355)
      call createCurrent(P_A, P_Hl, P_Z, P_Hl, 9453)
      call createCurrent(P_HhQ, P_ghAAnti, P_HMinus, P_ghWp, 1787)
      call createCurrent(P_Ha, P_Ha, P_G0, P_Ha, 8224)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hl, P_GPlus, 6650)
      call createCurrent(P_HMinus, P_WPlus, P_GPlus, P_WMinus, 9435)
      call createCurrent(P_ghWpAnti, P_GPlus, P_Ha, P_ghA, 9742)
      call createCurrent(P_GMinus, P_Hh, P_ghAAnti, P_ghWp, 4286)
      call createCurrent(P_GPlus, P_GMinus, P_Hh, P_Ha, 9627)
      call createCurrent(P_A, P_nu_muAnti, P_nu_mu, 9808)
      call createCurrent(P_HQPlus, P_WMinus, P_HhQ, 3328)
      call createCurrent(P_Z, P_WPlus, P_ghZAnti, P_ghWm, 3112)
      call createCurrent(P_G0, P_WMinus, P_GPlus, P_A, 8262)
      call createCurrent(P_Hl, P_G0, P_HPlus, P_HMinus, 10422)
      call createCurrent(P_HMinus, P_GPlus, P_HPlus, P_HMinus, 4222)
      call createCurrent(P_G0, P_GPlus, P_HMinus, P_Hh, 4104)
      call createCurrent(P_G0, P_ghWpAnti, P_ghZ, P_GPlus, 3435)
      call createCurrent(P_GPlus, P_ghA, P_HMinus, P_ghZAnti, 9147)
      call createCurrent(P_ghWm, P_Ha, P_G0, P_ghWmAnti, 9401)
      call createCurrent(P_WQMinus, P_Z, P_Hl, P_GQPlus, 239)
      call createCurrent(P_u, P_uAnti, 10244)
      call createCurrent(P_HMinus, P_ghWp, P_ghWpAnti, P_HPlus, 3175)
      call createCurrent(P_G0, P_G0, P_HMinus, P_GPlus, 5230)
      call createCurrent(P_GPlus, P_Ha, P_Hh, P_GMinus, 10265)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghWp, 5475)
      call createCurrent(P_ghWm, P_GMinus, P_HPlus, P_ghWmAnti, 10236)
      call createCurrent(P_HPlus, P_GMinus, P_Hl, P_Ha, 4425)
      call createCurrent(P_ghAAnti, P_ghZ, P_GMinus, P_HPlus, 8840)
      call createCurrent(P_GPlus, P_HMinus, P_GPlus, P_HMinus, 5674)
      call createCurrent(P_tauPlus, P_tauMinus, P_Hh, 3073)
      call createCurrent(P_ghA, P_A, P_WPlus, P_ghWpAnti, 5857)
      call createCurrent(P_HPlus, P_G0, P_HMinus, P_Hl, 9964)
      call createCurrent(P_ghWm, P_GPlus, P_GMinus, P_ghWmAnti, 4735)
      call createCurrent(P_WQMinus, P_Hl, P_GQPlus, 844)
      call createCurrent(P_ghWp, P_WMinus, P_ghAAnti, P_AQ, 313)
      call createCurrent(P_GMinus, P_HPlus, P_Hh, 3054)
      call createCurrent(P_GQ0, P_ghWpAnti, P_Hh, P_ghWp, 7027)
      call createCurrent(P_ghWp, P_Ha, P_ghAAnti, P_GMinus, 9228)
      call createCurrent(P_ZQ, P_WPlus, P_GMinus, P_HlQ, 6946)
      call createCurrent(P_ZQ, P_Hl, P_HaQ, 3024)
      call createCurrent(P_HQPlus, P_Hl, P_Ha, P_GQMinus, 359)
      call createCurrent(P_ghWp, P_ghWpAnti, P_A, P_A, 2709)
      call createCurrent(P_HQPlus, P_GMinus, P_Hh, P_GQ0, 1785)
      call createCurrent(P_Ha, P_HPlus, P_GMinus, P_Ha, 4149)
      call createCurrent(P_ghA, P_HPlus, P_ghZAnti, P_HQMinus, 6790)
      call createCurrent(P_ghZ, P_WPlus, P_ghAAnti, P_WMinus, 6497)
      call createCurrent(P_ghWm, P_G0, P_HPlus, P_ghAAnti, 9292)
      call createCurrent(P_ghZAnti, P_ghZ, P_HPlus, P_HQMinus, 578)
      call createCurrent(P_HPlus, P_G0, P_Hh, P_GMinus, 6387)
      call createCurrent(P_HQMinus, P_ghZ, P_Ha, P_ghWmAnti, 4215)
      call createCurrent(P_Hh, P_G0, P_G0, P_Ha, 10615)
      call createCurrent(P_GPlus, P_HMinus, P_Hl, P_Hh, 5658)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWm, P_HPlus, 3257)
      call createCurrent(P_WPlus, P_A, P_HMinus, P_Hh, 6741)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_G0, 97)
      call createCurrent(P_HhQ, P_Z, P_WPlus, P_GQMinus, 8759)
      call createCurrent(P_HaQ, P_Ha, P_GMinus, P_HQPlus, 7298)
      call createCurrent(P_Hh, P_HMinus, P_ghZ, P_ghWmAnti, 1605)
      call createCurrent(P_Ha, P_Hl, P_Hl, P_Ha, 5804)
      call createCurrent(P_d, P_uAnti, P_HPlus, 3977)
      call createCurrent(P_WQPlus, P_G0, P_Z, P_GQMinus, 6986)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hh, P_HPlus, 6599)
      call createCurrent(P_ghWpAnti, P_ghWp, P_ZQ, 6264)
      call createCurrent(P_WQMinus, P_GPlus, P_Hh, P_AQ, 1667)
      call createCurrent(P_A, P_Z, P_ghWm, P_ghWmAnti, 8677)
      call createCurrent(P_GPlus, P_HMinus, P_GMinus, P_HPlus, 8833)
      call createCurrent(P_HlQ, P_Hl, P_HMinus, P_HQPlus, 3473)
      call createCurrent(P_ghWmAnti, P_ghA, P_GQMinus, 4634)
      call createCurrent(P_ghZ, P_Hh, P_ghWpAnti, P_HQPlus, 1582)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hh, 1709)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hl, P_GPlus, 7398)
      call createCurrent(P_ZQ, P_WMinus, P_ghWp, P_ghAAnti, 6218)
      call createCurrent(P_HlQ, P_G0, P_GQ0, 4556)
      call createCurrent(P_Hl, P_ghWm, P_ghWmAnti, P_Hh, 9226)
      call createCurrent(P_ghWpAnti, P_WPlus, P_Z, P_ghZ, 3786)
      call createCurrent(P_Ha, P_GMinus, P_ghWp, P_ghZAnti, 8975)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghWm, P_WPlus, 6906)
      call createCurrent(P_G0, P_Ha, P_Hl, P_Hh, 3043)
      call createCurrent(P_ghA, P_ghWmAnti, P_GQMinus, 4541)
      call createCurrent(P_GPlus, P_Hh, P_HMinus, P_Hh, 7924)
      call createCurrent(P_ePlus, P_eMinus, P_Hh, 2036)
      call createCurrent(P_WQMinus, P_GPlus, P_G0, P_AQ, 7676)
      call createCurrent(P_ghWp, P_Ha, P_ghWpAnti, P_HaQ, 8064)
      call createCurrent(P_WQPlus, P_GMinus, P_G0, P_AQ, 7801)
      call createCurrent(P_Hl, P_ghWp, P_ghAAnti, P_HMinus, 5984)
      call createCurrent(P_WMinus, P_HMinus, P_HPlus, P_WPlus, 74)
      call createCurrent(P_A, P_WMinus, P_Hl, P_GPlus, 4491)
      call createCurrent(P_Ha, P_HMinus, P_ghWp, P_ghZAnti, 3738)
      call createCurrent(P_s, P_sAnti, P_g, 5331)
      call createCurrent(P_ghWm, P_A, P_ghWmAnti, 953)
      call createCurrent(P_uAnti, P_Z, P_u, 1056)
      call createCurrent(P_ghAAnti, P_ghZ, P_GMinus, P_GQPlus, 3574)
      call createCurrent(P_GPlus, P_GMinus, P_ghZAnti, P_ghA, 4720)
      call createCurrent(P_ghWp, P_HPlus, P_GMinus, P_ghWpAnti, 9631)
      call createCurrent(P_GPlus, P_Hh, P_Hl, P_HMinus, 3529)
      call createCurrent(P_ghWp, P_ghAAnti, P_A, P_WMinus, 4431)
      call createCurrent(P_ghZ, P_ghZAnti, P_GPlus, P_GMinus, 5484)
      call createCurrent(P_G0, P_Ha, P_ghWmAnti, P_ghWm, 10115)
      call createCurrent(P_ghWm, P_Z, P_ghWmAnti, P_A, 1099)
      call createCurrent(P_ghAAnti, P_HPlus, P_Ha, P_ghWm, 6310)
      call createCurrent(P_Z, P_HMinus, P_WPlus, 3776)
      call createCurrent(P_Hh, P_Ha, P_Hh, P_Ha, 8006)
      call createCurrent(P_WPlus, P_Hh, P_HMinus, P_A, 7303)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_GQ0, 727)
      call createCurrent(P_WQPlus, P_HMinus, P_Hh, P_AQ, 7580)
      call createCurrent(P_ghWpAnti, P_HPlus, P_HMinus, P_ghWp, 7307)
      call createCurrent(P_HMinus, P_Z, P_WPlus, P_Hl, 2963)
      call createCurrent(P_Z, P_ghWmAnti, P_ghWm, P_A, 731)
      call createCurrent(P_ghZ, P_GPlus, P_Ha, P_ghWpAnti, 9478)
      call createCurrent(P_HPlus, P_ghZ, P_G0, P_ghWpAnti, 9025)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghWm, P_Ha, 5867)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_Hl, 3522)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HMinus, P_HPlus, 2656)
      call createCurrent(P_GQ0, P_Ha, P_Ha, P_HaQ, 4828)
      call createCurrent(P_WPlus, P_ghWp, P_ghWpAnti, P_WMinus, 6494)
      call createCurrent(P_Ha, P_HMinus, P_Hh, P_HPlus, 10334)
      call createCurrent(P_HMinus, P_nu_tau, P_tauPlus, 610)
      call createCurrent(P_GQMinus, P_HPlus, P_GPlus, P_HQMinus, 3783)
      call createCurrent(P_HMinus, P_Hh, P_Hh, P_GPlus, 6420)
      call createCurrent(P_HMinus, P_GPlus, P_G0, P_Hh, 5168)
      call createCurrent(P_GQPlus, P_WMinus, P_A, P_GQ0, 751)
      call createCurrent(P_A, P_GMinus, P_Hh, P_WPlus, 223)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_Hl, 1810)
      call createCurrent(P_G0, P_HPlus, P_Z, P_WMinus, 10128)
      call createCurrent(P_GQ0, P_GPlus, P_HMinus, P_HaQ, 1280)
      call createCurrent(P_Hl, P_ghWmAnti, P_GMinus, P_ghA, 5285)
      call createCurrent(P_G0, P_Ha, P_GPlus, P_HMinus, 3171)
      call createCurrent(P_HlQ, P_Ha, P_GPlus, P_GQMinus, 7774)
      call createCurrent(P_GMinus, P_A, P_Hh, P_WPlus, 2684)
      call createCurrent(P_G0, P_ghZ, P_ghWmAnti, P_HMinus, 9829)
      call createCurrent(P_WQMinus, P_GPlus, P_GMinus, P_WQPlus, 4633)
      call createCurrent(P_GMinus, P_HPlus, P_Ha, 2794)
      call createCurrent(P_WQMinus, P_GPlus, P_HlQ, 7645)
      call createCurrent(P_ghA, P_ghWmAnti, P_HQMinus, 4567)
      call createCurrent(P_HlQ, P_G0, P_GPlus, P_GQMinus, 5839)
      call createCurrent(P_Z, P_GPlus, P_Z, P_GMinus, 6440)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghWp, P_Hh, 10239)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghWm, P_GQ0, 1686)
      call createCurrent(P_Hl, P_HMinus, P_HPlus, P_Hh, 4599)
      call createCurrent(P_AQ, P_Z, P_GMinus, P_GQPlus, 6041)
      call createCurrent(P_WPlus, P_WMinus, P_WMinus, P_WPlus, 3430)
      call createCurrent(P_g, P_G0, P_g, P_Ha, 10642)
      call createCurrent(P_GMinus, P_ghA, P_HPlus, P_ghAAnti, 9165)
      call createCurrent(P_HMinus, P_Hl, P_GPlus, P_Ha, 3439)
      call createCurrent(P_GMinus, P_ghAAnti, P_Ha, P_ghWp, 9413)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_Hh, 999)
      call createCurrent(P_HlQ, P_GMinus, P_ghWp, P_ghZAnti, 2790)
      call createCurrent(P_HlQ, P_ghWm, P_HPlus, P_ghZAnti, 4445)
      call createCurrent(P_GQMinus, P_Hh, P_HQPlus, 3692)
      call createCurrent(P_G0, P_Z, P_WMinus, P_GPlus, 1354)
      call createCurrent(P_GQMinus, P_Hh, P_ghWmAnti, P_ghZ, 4791)
      call createCurrent(P_nu_tauAnti, P_A, P_nu_tau, 8971)
      call createCurrent(P_eMinus, P_Hl, P_ePlus, 1591)
      call createCurrent(P_Z, P_nu_tauAnti, P_nu_tau, 2992)
      call createCurrent(P_HlQ, P_ghZAnti, P_GMinus, P_ghWp, 1226)
      call createCurrent(P_ghWmAnti, P_G0, P_ghZ, P_GMinus, 699)
      call createCurrent(P_G0, P_HPlus, P_ghZAnti, P_ghWm, 9788)
      call createCurrent(P_ZQ, P_Z, P_HhQ, 5836)
      call createCurrent(P_HhQ, P_Hh, P_Hl, P_HlQ, 7279)
      call createCurrent(P_HQPlus, P_HMinus, P_A, P_AQ, 1888)
      call createCurrent(P_HaQ, P_WPlus, P_HQMinus, 6970)
      call createCurrent(P_Ha, P_G0, P_ghWmAnti, P_ghWm, 9565)
      call createCurrent(P_GMinus, P_WPlus, P_WMinus, P_HPlus, 10137)
      call createCurrent(P_HQMinus, P_GPlus, P_Hl, P_GQ0, 6840)
      call createCurrent(P_HQMinus, P_GPlus, P_Hh, P_GQ0, 4396)
      call createCurrent(P_WMinus, P_WPlus, P_ghWpAnti, P_ghWp, 7523)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghZ, P_GMinus, 3890)
      call createCurrent(P_Ha, P_tAnti, P_t, 4903)
      call createCurrent(P_A, P_ghWm, P_A, P_ghWmAnti, 518)
      call createCurrent(P_GMinus, P_Ha, P_ghWmAnti, P_ghA, 9368)
      call createCurrent(P_HhQ, P_G0, P_ZQ, 6484)
      call createCurrent(P_GQ0, P_GPlus, P_WQMinus, 5100)
      call createCurrent(P_dAnti, P_u, P_WQMinus, 6383)
      call createCurrent(P_GMinus, P_ghA, P_GPlus, P_ghZAnti, 7249)
      call createCurrent(P_HPlus, P_ghZ, P_Hl, P_ghWpAnti, 1824)
      call createCurrent(P_tAnti, P_b, P_GQPlus, 8610)
      call createCurrent(P_uAnti, P_d, P_HPlus, 2339)
      call createCurrent(P_ghWpAnti, P_Hl, P_HPlus, P_ghZ, 5528)
      call createCurrent(P_HQPlus, P_HMinus, P_ghA, P_ghAAnti, 3520)
      call createCurrent(P_HMinus, P_A, P_WPlus, P_Hl, 3444)
      call createCurrent(P_HMinus, P_GMinus, P_HPlus, P_GPlus, 6130)
      call createCurrent(P_HlQ, P_G0, P_GMinus, P_GQPlus, 4830)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_HlQ, 6869)
      call createCurrent(P_HhQ, P_GMinus, P_GPlus, P_HhQ, 6928)
      call createCurrent(P_dAnti, P_d, P_Hh, 4256)
      call createCurrent(P_Z, P_HPlus, P_Z, P_GMinus, 8993)
      call createCurrent(P_WQMinus, P_Hl, P_GPlus, P_AQ, 2052)
      call createCurrent(P_GQMinus, P_ghA, P_ghWmAnti, 2573)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghWp, P_G0, 230)
      call createCurrent(P_Z, P_A, P_ghWpAnti, P_ghWp, 8762)
      call createCurrent(P_AQ, P_Z, P_GPlus, P_GQMinus, 7632)
      call createCurrent(P_ghWpAnti, P_A, P_ghWp, 6792)
      call createCurrent(P_Ha, P_GPlus, P_HMinus, 1401)
      call createCurrent(P_G0, P_GMinus, P_Z, P_WPlus, 1165)
      call createCurrent(P_uAnti, P_u, P_Hh, 787)
      call createCurrent(P_HaQ, P_GMinus, P_G0, P_HQPlus, 5296)
      call createCurrent(P_HPlus, P_HPlus, P_GMinus, P_GMinus, 6382)
      call createCurrent(P_ghA, P_ghWmAnti, P_WMinus, P_A, 5269)
      call createCurrent(P_GQMinus, P_G0, P_ghWp, P_ghZAnti, 7442)
      call createCurrent(P_G0, P_GMinus, P_Ha, P_GPlus, 3609)
      call createCurrent(P_WQPlus, P_b, P_tAnti, 2414)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_HhQ, 2488)
      call createCurrent(P_Hh, P_G0, P_Hh, P_G0, 2534)
      call createCurrent(P_G0, P_ghZAnti, P_HPlus, P_ghWm, 9310)
      call createCurrent(P_ZQ, P_Z, P_ghWm, P_ghWmAnti, 5395)
      call createCurrent(P_GPlus, P_GMinus, P_ghA, P_ghZAnti, 5539)
      call createCurrent(P_HQPlus, P_ghAAnti, P_HMinus, P_ghZ, 214)
      call createCurrent(P_dAnti, P_d, P_HaQ, 8263)
      call createCurrent(P_HhQ, P_G0, P_HPlus, P_HQMinus, 5077)
      call createCurrent(P_HQMinus, P_ghZ, P_ghWmAnti, 2419)
      call createCurrent(P_Hl, P_Hh, P_Z, P_Z, 9981)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghA, P_HMinus, 5592)
      call createCurrent(P_Ha, P_WPlus, P_GMinus, 8863)
      call createCurrent(P_WPlus, P_GMinus, P_A, P_Hh, 5263)
      call createCurrent(P_ghA, P_WPlus, P_ghAAnti, P_WQMinus, 1676)
      call createCurrent(P_Hl, P_WMinus, P_HPlus, P_Z, 8497)
      call createCurrent(P_HPlus, P_GMinus, P_A, 9369)
      call createCurrent(P_GPlus, P_Ha, P_Ha, P_GMinus, 2374)
      call createCurrent(P_ghZ, P_ghZAnti, P_HhQ, 3575)
      call createCurrent(P_tAnti, P_t, P_ZQ, 1006)
      call createCurrent(P_HhQ, P_tAnti, P_t, 3208)
      call createCurrent(P_AQ, P_Z, P_HMinus, P_HQPlus, 6518)
      call createCurrent(P_ghZ, P_HPlus, P_ghAAnti, P_GMinus, 9236)
      call createCurrent(P_A, P_GPlus, P_A, P_HMinus, 9404)
      call createCurrent(P_ghAAnti, P_WMinus, P_A, P_ghWp, 1010)
      call createCurrent(P_HQPlus, P_nu_tauAnti, P_tauMinus, 5940)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghA, P_GPlus, 1062)
      call createCurrent(P_muMinus, P_muPlus, P_HaQ, 872)
      call createCurrent(P_Hl, P_ghA, P_GPlus, P_ghWpAnti, 2057)
      call createCurrent(P_WQMinus, P_GPlus, P_Z, P_GQ0, 2749)
      call createCurrent(P_WPlus, P_GMinus, P_Hh, P_Z, 7160)
      call createCurrent(P_dAnti, P_d, P_A, 5313)
      call createCurrent(P_ghZAnti, P_ghWp, P_G0, P_HMinus, 9774)
      call createCurrent(P_HhQ, P_HMinus, P_G0, P_GQPlus, 6568)
      call createCurrent(P_Hl, P_HPlus, P_ghWpAnti, P_ghA, 12)
      call createCurrent(P_HhQ, P_HMinus, P_Hh, P_HQPlus, 6448)
      call createCurrent(P_Ha, P_ghZ, P_ghWmAnti, P_GMinus, 10243)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hl, P_HQMinus, 1500)
      call createCurrent(P_tauMinus, P_tauPlus, 9015)
      call createCurrent(P_ghWm, P_HPlus, P_ghWmAnti, P_GMinus, 10404)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_HlQ, 5709)
      call createCurrent(P_GQPlus, P_GMinus, P_GQ0, 8205)
      call createCurrent(P_GPlus, P_G0, P_ghA, P_ghWpAnti, 5763)
      call createCurrent(P_GMinus, P_Ha, P_ghA, P_ghWmAnti, 9487)
      call createCurrent(P_GQPlus, P_GMinus, P_Hh, P_HlQ, 3042)
      call createCurrent(P_Hh, P_Hl, P_g, P_g, 10672)
      call createCurrent(P_ghA, P_WMinus, P_A, P_ghWmAnti, 4155)
      call createCurrent(P_G0, P_HPlus, P_Ha, P_HMinus, 7290)
      call createCurrent(P_g, P_b, P_bAnti, 7314)
      call createCurrent(P_GPlus, P_Hl, P_ghZ, P_ghWpAnti, 1750)
      call createCurrent(P_Z, P_Hh, P_Hh, P_A, 9667)
      call createCurrent(P_ghZAnti, P_Hh, P_Hh, P_ghZ, 5229)
      call createCurrent(P_Hh, P_HPlus, P_G0, P_HMinus, 10117)
      call createCurrent(P_GQ0, P_GMinus, P_HPlus, P_GQ0, 70)
      call createCurrent(P_AQ, P_tauPlus, P_tauMinus, 1372)
      call createCurrent(P_HMinus, P_WPlus, P_Hl, 8255)
      call createCurrent(P_Hh, P_HMinus, P_HPlus, P_Hl, 4364)
      call createCurrent(P_HlQ, P_WMinus, P_Z, P_GQPlus, 6463)
      call createCurrent(P_WQMinus, P_Z, P_GQPlus, 2805)
      call createCurrent(P_ghA, P_ghAAnti, P_HMinus, P_GPlus, 10386)
      call createCurrent(P_HhQ, P_G0, P_ghWp, P_ghWpAnti, 2446)
      call createCurrent(P_GPlus, P_WMinus, P_A, P_G0, 1198)
      call createCurrent(P_Ha, P_GMinus, P_GPlus, P_Hl, 10125)
      call createCurrent(P_HhQ, P_Z, P_HMinus, P_WQPlus, 7588)
      call createCurrent(P_GMinus, P_HPlus, P_ghWm, P_ghWmAnti, 8867)
      call createCurrent(P_g, P_Hh, P_g, P_Hh, 10496)
      call createCurrent(P_ghWp, P_Hh, P_Hl, P_ghWpAnti, 10024)
      call createCurrent(P_b, P_HPlus, P_tAnti, 7613)
      call createCurrent(P_HPlus, P_HMinus, P_Ha, P_G0, 2042)
      call createCurrent(P_HPlus, P_GMinus, P_HMinus, P_GPlus, 3412)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghZ, P_HPlus, 4042)
      call createCurrent(P_HlQ, P_Ha, P_Hl, P_HaQ, 6548)
      call createCurrent(P_ghWpAnti, P_ghWp, P_A, P_Z, 2347)
      call createCurrent(P_ghWpAnti, P_G0, P_ghWp, P_G0, 3688)
      call createCurrent(P_GQPlus, P_GMinus, P_WPlus, P_WQMinus, 2316)
      call createCurrent(P_ghZ, P_ghAAnti, P_HPlus, P_GMinus, 9858)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghWp, P_GMinus, 7192)
      call createCurrent(P_Ha, P_ghZ, P_GPlus, P_ghWpAnti, 9624)
      call createCurrent(P_G0, P_GPlus, P_GMinus, P_Hl, 9713)
      call createCurrent(P_HaQ, P_Ha, P_HMinus, P_HQPlus, 294)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_G0, 10396)
      call createCurrent(P_ghZ, P_Hh, P_Hl, P_ghZAnti, 9009)
      call createCurrent(P_ghA, P_ghAAnti, P_WMinus, P_WPlus, 1014)
      call createCurrent(P_HQPlus, P_Hl, P_HMinus, P_GQ0, 6015)
      call createCurrent(P_GQMinus, P_Z, P_WPlus, P_HhQ, 4111)
      call createCurrent(P_WPlus, P_ghZAnti, P_ghWm, P_Z, 5099)
      call createCurrent(P_ghWm, P_A, P_ghZAnti, P_WPlus, 3672)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWm, P_HPlus, 7194)
      call createCurrent(P_Ha, P_WMinus, P_GPlus, 9613)
      call createCurrent(P_Hl, P_GMinus, P_ghAAnti, P_ghWp, 6163)
      call createCurrent(P_WQMinus, P_WMinus, P_ghWp, P_ghWmAnti, 955)
      call createCurrent(P_A, P_HPlus, P_WMinus, 1179)
      call createCurrent(P_Hh, P_GPlus, P_ghZAnti, P_ghWm, 145)
      call createCurrent(P_HMinus, P_ghWp, P_ghAAnti, 1558)
      call createCurrent(P_HlQ, P_GPlus, P_ghWm, P_ghZAnti, 7814)
      call createCurrent(P_A, P_Hl, P_A, P_Hh, 10491)
      call createCurrent(P_u, P_G0, P_uAnti, 4827)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hh, P_HQMinus, 6161)
      call createCurrent(P_ghAAnti, P_HMinus, P_GPlus, P_ghZ, 9765)
      call createCurrent(P_HhQ, P_WMinus, P_HQPlus, 4588)
      call createCurrent(P_GQ0, P_GPlus, P_Hl, P_HQMinus, 2439)
      call createCurrent(P_AQ, P_u, P_uAnti, 4175)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_Hl, 7431)
      call createCurrent(P_GMinus, P_GPlus, P_Hl, P_Hh, 240)
      call createCurrent(P_Hh, P_WMinus, P_HPlus, P_A, 4803)
      call createCurrent(P_G0, P_sAnti, P_s, 4902)
      call createCurrent(P_Hl, P_GPlus, P_GMinus, P_G0, 9322)
      call createCurrent(P_Z, P_g, P_g, P_g, 10534)
      call createCurrent(P_GPlus, P_Hl, P_G0, P_HMinus, 4057)
      call createCurrent(P_Hl, P_A, P_HPlus, P_WMinus, 4173)
      call createCurrent(P_Z, P_Hl, P_WPlus, P_GMinus, 2098)
      call createCurrent(P_HlQ, P_WMinus, P_GQPlus, 3993)
      call createCurrent(P_ZQ, P_HMinus, P_A, P_HQPlus, 8808)
      call createCurrent(P_WPlus, P_A, P_WMinus, 8763)
      call createCurrent(P_uAnti, P_u, P_Ha, 3448)
      call createCurrent(P_G0, P_Hl, P_Ha, P_Hl, 5255)
      call createCurrent(P_Z, P_Hh, P_WMinus, P_HPlus, 202)
      call createCurrent(P_ghWmAnti, P_Hl, P_HMinus, P_ghZ, 72)
      call createCurrent(P_WMinus, P_t, P_bAnti, 5533)
      call createCurrent(P_HlQ, P_WMinus, P_A, P_GQPlus, 3361)
      call createCurrent(P_Z, P_WPlus, P_ghWm, P_ghZAnti, 8474)
      call createCurrent(P_G0, P_WPlus, P_Ha, P_WMinus, 10433)
      call createCurrent(P_GPlus, P_GMinus, P_g, P_g, 10718)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghWm, P_Ha, 9625)
      call createCurrent(P_GPlus, P_Hl, P_Hl, P_HMinus, 6567)
      call createCurrent(P_WPlus, P_A, P_Hh, P_GMinus, 3916)
      call createCurrent(P_GMinus, P_Ha, P_Ha, P_HPlus, 8714)
      call createCurrent(P_A, P_ghWp, P_WMinus, P_ghAAnti, 2867)
      call createCurrent(P_WPlus, P_GMinus, P_Hl, 85)
      call createCurrent(P_HhQ, P_G0, P_HMinus, P_GQPlus, 490)
      call createCurrent(P_ghWp, P_ghZAnti, P_WMinus, P_AQ, 5467)
      call createCurrent(P_GMinus, P_GPlus, P_Z, 7877)
      call createCurrent(P_HlQ, P_GMinus, P_HPlus, P_HlQ, 3758)
      call createCurrent(P_ZQ, P_ghWp, P_ghWpAnti, 6956)
      call createCurrent(P_HQMinus, P_WPlus, P_Ha, P_AQ, 5007)
      call createCurrent(P_ghZAnti, P_Ha, P_HPlus, P_ghWm, 2285)
      call createCurrent(P_GPlus, P_G0, P_WMinus, P_Z, 2331)
      call createCurrent(P_ghZAnti, P_Hl, P_ghZ, P_Hl, 5760)
      call createCurrent(P_WPlus, P_WMinus, P_G0, P_G0, 644)
      call createCurrent(P_ghWm, P_ghZAnti, P_WPlus, 4549)
      call createCurrent(P_Ha, P_Hh, P_G0, P_Hh, 1804)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_HaQ, 7399)
      call createCurrent(P_Hl, P_ghWp, P_Hh, P_ghWpAnti, 9650)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_Hl, 4716)
      call createCurrent(P_ghWm, P_HMinus, P_ghWmAnti, P_HQPlus, 8201)
      call createCurrent(P_Ha, P_Hl, P_G0, P_G0, 10760)
      call createCurrent(P_Hh, P_Hh, P_A, P_A, 10539)
      call createCurrent(P_Hh, P_HPlus, P_HMinus, P_Ha, 8973)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_Hh, 1436)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghWp, P_Hl, 7950)
      call createCurrent(P_g, P_g, P_g, P_Z, 10774)
      call createCurrent(P_nu_tau, P_A, P_nu_tauAnti, 9646)
      call createCurrent(P_WMinus, P_Hh, P_WPlus, P_Hh, 4686)
      call createCurrent(P_A, P_G0, P_G0, P_A, 10781)
      call createCurrent(P_bAnti, P_WMinus, P_t, 2893)
      call createCurrent(P_s, P_sAnti, P_HaQ, 5402)
      call createCurrent(P_HhQ, P_ghWmAnti, P_HMinus, P_ghZ, 5799)
      call createCurrent(P_HlQ, P_Ha, P_ghWp, P_ghWpAnti, 535)
      call createCurrent(P_Ha, P_ghWpAnti, P_ghZ, P_GPlus, 9886)
      call createCurrent(P_muPlus, P_GMinus, P_nu_mu, 5485)
      call createCurrent(P_HhQ, P_Ha, P_Hl, P_HaQ, 3182)
      call createCurrent(P_HQMinus, P_Hh, P_ghA, P_ghWmAnti, 1820)
      call createCurrent(P_Z, P_A, P_Ha, P_Ha, 10187)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_HlQ, 4695)
      call createCurrent(P_Z, P_WMinus, P_HPlus, 2423)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_G0, 9570)
      call createCurrent(P_ghWpAnti, P_Hl, P_GPlus, P_ghZ, 703)
      call createCurrent(P_ghWm, P_ghZAnti, P_A, P_WPlus, 3459)
      call createCurrent(P_ghWmAnti, P_Hl, P_Hh, P_ghWm, 9574)
      call createCurrent(P_tauMinus, P_tauPlus, P_G0, 1842)
      call createCurrent(P_ghA, P_WMinus, P_ghZAnti, P_WQPlus, 828)
      call createCurrent(P_ghZ, P_ghZAnti, P_HMinus, P_HPlus, 143)
      call createCurrent(P_ghA, P_HPlus, P_ghAAnti, P_GMinus, 9997)
      call createCurrent(P_HQPlus, P_HMinus, P_HMinus, P_GQPlus, 8652)
      call createCurrent(P_ghZAnti, P_HMinus, P_Ha, P_ghWp, 5937)
      call createCurrent(P_HaQ, P_Hh, P_HaQ, 6281)
      call createCurrent(P_ZQ, P_Hl, P_HPlus, P_WQMinus, 5885)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghWm, P_HPlus, 8271)
      call createCurrent(P_c, P_Z, P_cAnti, 3063)
      call createCurrent(P_HPlus, P_WMinus, P_Hl, P_Z, 8787)
      call createCurrent(P_ghZAnti, P_ghWp, P_Ha, P_HMinus, 1770)
      call createCurrent(P_Z, P_HPlus, P_WMinus, 2994)
      call createCurrent(P_Ha, P_G0, P_Ha, P_Ha, 2645)
      call createCurrent(P_ZQ, P_Hl, P_GQ0, 6008)
      call createCurrent(P_Z, P_HPlus, P_WMinus, P_Hh, 7933)
      call createCurrent(P_HPlus, P_Hl, P_WMinus, P_A, 1833)
      call createCurrent(P_ghAAnti, P_WPlus, P_Z, P_ghWm, 3326)
      call createCurrent(P_HhQ, P_HMinus, P_Z, P_WQPlus, 1442)
      call createCurrent(P_Ha, P_G0, P_A, P_Z, 10779)
      call createCurrent(P_ghAAnti, P_Hh, P_ghWp, P_GMinus, 1955)
      call createCurrent(P_G0, P_WPlus, P_HMinus, 9612)
      call createCurrent(P_Hl, P_HPlus, P_ghA, P_ghWpAnti, 361)
      call createCurrent(P_Ha, P_GPlus, P_Hl, P_HMinus, 4964)
      call createCurrent(P_WPlus, P_A, P_G0, P_GMinus, 7990)
      call createCurrent(P_Z, P_Z, 10116)
      call createCurrent(P_Hh, P_GPlus, P_GMinus, P_Hh, 3181)
      call createCurrent(P_HQMinus, P_ghZ, P_Hl, P_ghWmAnti, 1571)
      call createCurrent(P_ghAAnti, P_HMinus, P_HPlus, P_ghA, 3727)
      call createCurrent(P_HaQ, P_HMinus, P_Ha, P_GQPlus, 8646)
      call createCurrent(P_GPlus, P_GMinus, P_ghA, P_ghAAnti, 484)
      call createCurrent(P_HPlus, P_HMinus, P_WMinus, P_WPlus, 2959)
      call createCurrent(P_WMinus, P_WPlus, P_A, P_A, 3926)
      call createCurrent(P_GMinus, P_Hh, P_G0, P_HPlus, 6561)
      call createCurrent(P_G0, P_G0, P_Hl, P_Hl, 2253)
      call createCurrent(P_Hh, P_ghWmAnti, P_Hl, P_ghWm, 10183)
      call createCurrent(P_HPlus, P_WMinus, P_Hl, 1901)
      call createCurrent(P_HlQ, P_Ha, P_ghWpAnti, P_ghWp, 3897)
      call createCurrent(P_WQMinus, P_GPlus, P_G0, P_ZQ, 4916)
      call createCurrent(P_HlQ, P_WPlus, P_Ha, P_WQMinus, 2816)
      call createCurrent(P_HhQ, P_GPlus, P_HMinus, P_HaQ, 7649)
      call createCurrent(P_HPlus, P_A, P_WMinus, P_Ha, 1341)
      call createCurrent(P_Hl, P_ghA, P_HPlus, P_ghWpAnti, 964)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_G0, 9426)
      call createCurrent(P_Z, P_A, P_Hl, P_Hh, 10766)
      call createCurrent(P_HlQ, P_G0, P_ghWm, P_ghWmAnti, 103)
      call createCurrent(P_HMinus, P_Ha, P_ghWp, P_ghZAnti, 3780)
      call createCurrent(P_ghWpAnti, P_Z, P_ghWp, P_AQ, 1567)
      call createCurrent(P_Hh, P_GMinus, P_HPlus, P_G0, 3289)
      call createCurrent(P_Hl, P_ghZAnti, P_GMinus, P_ghWp, 7116)
      call createCurrent(P_HQPlus, P_Hl, P_WMinus, P_ZQ, 1376)
      call createCurrent(P_GQMinus, P_Hh, P_GPlus, P_HhQ, 4671)
      call createCurrent(P_A, P_WMinus, P_Ha, P_GPlus, 10055)
      call createCurrent(P_HhQ, P_WMinus, P_Z, P_HQPlus, 1830)
      call createCurrent(P_GQ0, P_ghWp, P_GMinus, P_ghAAnti, 7610)
      call createCurrent(P_G0, P_GPlus, P_Hl, P_GMinus, 8889)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_ghZ, 2152)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghWm, P_GPlus, 9410)
      call createCurrent(P_A, P_WPlus, P_Hh, P_GMinus, 8794)
      call createCurrent(P_WPlus, P_Z, P_ghZ, P_ghWpAnti, 4524)
      call createCurrent(P_ghZ, P_ghAAnti, P_GPlus, P_GMinus, 5878)
      call createCurrent(P_Hh, P_Hl, P_ghWmAnti, P_ghWm, 8853)
      call createCurrent(P_d, P_GPlus, P_uAnti, 7750)
      call createCurrent(P_nu_tauAnti, P_GPlus, P_tauMinus, 2076)
      call createCurrent(P_ghWp, P_HMinus, P_ghWpAnti, P_HPlus, 7607)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_HaQ, 7124)
      call createCurrent(P_G0, P_HPlus, P_ghA, P_ghWpAnti, 10136)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_HhQ, 3504)
      call createCurrent(P_WQPlus, P_GMinus, P_GPlus, P_WQMinus, 8267)
      call createCurrent(P_GQPlus, P_Hl, P_Hh, P_GQMinus, 5505)
      call createCurrent(P_HMinus, P_Ha, P_GPlus, P_Ha, 5299)
      call createCurrent(P_HPlus, P_Hh, P_ghA, P_ghWpAnti, 7310)
      call createCurrent(P_HaQ, P_A, P_WPlus, P_HQMinus, 4554)
      call createCurrent(P_WMinus, P_GPlus, P_Z, P_Ha, 10254)
      call createCurrent(P_HhQ, P_GPlus, P_GMinus, P_HhQ, 1290)
      call createCurrent(P_HhQ, P_Ha, P_GPlus, P_HQMinus, 6736)
      call createCurrent(P_HMinus, P_G0, P_G0, P_GPlus, 6215)
      call createCurrent(P_Hh, P_Hh, P_ghWp, P_ghWpAnti, 3743)
      call createCurrent(P_HhQ, P_Hl, P_HlQ, 769)
      call createCurrent(P_Ha, P_WMinus, P_WPlus, P_Ha, 1039)
      call createCurrent(P_GQMinus, P_G0, P_GPlus, P_HlQ, 8784)
      call createCurrent(P_Ha, P_ghAAnti, P_ghWm, P_GPlus, 10421)
      call createCurrent(P_muMinus, P_nu_muAnti, P_GPlus, 832)
      call createCurrent(P_GQMinus, P_GPlus, P_G0, P_HhQ, 6778)
      call createCurrent(P_WPlus, P_ghAAnti, P_WMinus, P_ghZ, 5499)
      call createCurrent(P_WPlus, P_GMinus, P_G0, P_Z, 2680)
      call createCurrent(P_ghWmAnti, P_ghWm, P_A, P_Z, 6445)
      call createCurrent(P_t, P_tAnti, P_A, 7894)
      call createCurrent(P_bAnti, P_b, P_HaQ, 6050)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_HhQ, 2638)
      call createCurrent(P_WMinus, P_WPlus, P_ghAAnti, P_ghA, 7741)
      call createCurrent(P_G0, P_G0, P_G0, P_Ha, 96)
      call createCurrent(P_WPlus, P_Hl, P_A, P_HMinus, 126)
      call createCurrent(P_WMinus, P_Z, P_GPlus, P_G0, 8510)
      call createCurrent(P_Ha, P_A, P_G0, P_Z, 10561)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghZ, P_A, 660)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_G0, 9233)
      call createCurrent(P_Z, P_G0, P_G0, P_A, 9174)
      call createCurrent(P_HMinus, P_HPlus, P_Ha, P_Hl, 10483)
      call createCurrent(P_WQMinus, P_Z, P_Z, P_WQPlus, 5671)
      call createCurrent(P_A, P_ghWp, P_WMinus, P_ghZAnti, 8063)
      call createCurrent(P_WMinus, P_ghWp, P_ghZAnti, P_Z, 4821)
      call createCurrent(P_HlQ, P_HPlus, P_Hl, P_HQMinus, 1259)
      call createCurrent(P_GQ0, P_WPlus, P_GQMinus, 2605)
      call createCurrent(P_GQ0, P_GMinus, P_ghWp, P_ghZAnti, 4314)
      call createCurrent(P_G0, P_ghWm, P_ghZAnti, P_GPlus, 4185)
      call createCurrent(P_ghA, P_GMinus, P_ghAAnti, P_HPlus, 9904)
      call createCurrent(P_ghWmAnti, P_WMinus, P_Z, P_ghZ, 4881)
      call createCurrent(P_WQPlus, P_A, P_A, P_WQMinus, 4607)
      call createCurrent(P_HQMinus, P_A, P_A, P_HQPlus, 8020)
      call createCurrent(P_HlQ, P_WMinus, P_G0, P_WQPlus, 8145)
      call createCurrent(P_GMinus, P_HPlus, P_ghZAnti, P_ghA, 9373)
      call createCurrent(P_GQ0, P_Hh, P_Hh, P_GQ0, 127)
      call createCurrent(P_ghWm, P_G0, P_ghWmAnti, P_G0, 1459)
      call createCurrent(P_Z, P_Hl, P_WMinus, P_GPlus, 4898)
      call createCurrent(P_tauMinus, P_HPlus, P_nu_tauAnti, 737)
      call createCurrent(P_HPlus, P_Ha, P_G0, P_HMinus, 7992)
      call createCurrent(P_HQMinus, P_A, P_HPlus, P_AQ, 4062)
      call createCurrent(P_GPlus, P_G0, P_A, P_WMinus, 7042)
      call createCurrent(P_HlQ, P_G0, P_ghWmAnti, P_ghWm, 6593)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_Hh, 1690)
      call createCurrent(P_HQMinus, P_G0, P_HPlus, P_HaQ, 2975)
      call createCurrent(P_WMinus, P_nu_mu, P_muPlus, 362)
      call createCurrent(P_AQ, P_GMinus, P_Hl, P_WQPlus, 7804)
      call createCurrent(P_WMinus, P_Z, P_HPlus, P_Hh, 3225)
      call createCurrent(P_AQ, P_WPlus, P_ghZAnti, P_ghWm, 6072)
      call createCurrent(P_WPlus, P_Z, P_G0, P_GMinus, 1469)
      call createCurrent(P_GQPlus, P_A, P_WMinus, P_GQ0, 2148)
      call createCurrent(P_Ha, P_GPlus, P_HMinus, P_Hl, 5621)
      call createCurrent(P_HQMinus, P_HPlus, P_G0, P_HlQ, 7216)
      call createCurrent(P_WPlus, P_GMinus, P_A, 7339)
      call createCurrent(P_g, P_g, P_Hh, P_Hl, 10650)
      call createCurrent(P_GQ0, P_GMinus, P_Hl, P_HQPlus, 43)
      call createCurrent(P_G0, P_WMinus, P_A, P_GPlus, 7158)
      call createCurrent(P_HPlus, P_ghZ, P_ghWpAnti, P_Hl, 3376)
      call createCurrent(P_HQPlus, P_HMinus, P_GPlus, P_HQMinus, 4654)
      call createCurrent(P_u, P_dAnti, P_WMinus, 6057)
      call createCurrent(P_tAnti, P_GPlus, P_b, 5685)
      call createCurrent(P_Hl, P_g, P_g, 10643)
      call createCurrent(P_HMinus, P_WPlus, P_A, P_Hh, 8229)
      call createCurrent(P_WMinus, P_Z, P_ghZAnti, P_ghWp, 5933)
      call createCurrent(P_ghZ, P_HMinus, P_Hl, P_ghWmAnti, 3850)
      call createCurrent(P_ghWm, P_ghAAnti, P_Z, P_WPlus, 4601)
      call createCurrent(P_HhQ, P_tauPlus, P_tauMinus, 3620)
      call createCurrent(P_ghWp, P_ghWmAnti, P_WMinus, P_WQMinus, 190)
      call createCurrent(P_HQMinus, P_HPlus, P_Ha, P_GQ0, 7747)
      call createCurrent(P_Hl, P_A, P_A, 10596)
      call createCurrent(P_Hh, P_u, P_uAnti, 5659)
      call createCurrent(P_Z, P_Z, P_g, P_g, 10606)
      call createCurrent(P_WQPlus, P_A, P_ghZ, P_ghWpAnti, 3892)
      call createCurrent(P_GPlus, P_Hh, P_ghZAnti, P_ghWm, 4435)
      call createCurrent(P_HMinus, P_GPlus, P_Hh, P_Hl, 3883)
      call createCurrent(P_WQPlus, P_WMinus, P_ghAAnti, P_ghA, 4452)
      call createCurrent(P_HQMinus, P_Ha, P_Z, P_WQPlus, 3314)
      call createCurrent(P_G0, P_WMinus, P_Z, P_GPlus, 62)
      call createCurrent(P_HlQ, P_GPlus, P_GMinus, P_HhQ, 7143)
      call createCurrent(P_GQMinus, P_Hh, P_Ha, P_HQPlus, 3360)
      call createCurrent(P_Hl, P_Hh, P_ghZAnti, P_ghZ, 9067)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Ha, P_HMinus, 6180)
      call createCurrent(P_GPlus, P_ghZAnti, P_Hl, P_ghWm, 1193)
      call createCurrent(P_Ha, P_ghWmAnti, P_ghWm, P_G0, 9337)
      call createCurrent(P_HMinus, P_Hh, P_HPlus, 2830)
      call createCurrent(P_A, P_WMinus, P_GPlus, P_Hl, 7472)
      call createCurrent(P_ZQ, P_WPlus, P_ghZAnti, P_ghWm, 3194)
      call createCurrent(P_sAnti, P_c, P_HMinus, 4995)
      call createCurrent(P_GMinus, P_HPlus, P_g, P_g, 10559)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_HhQ, 4988)
      call createCurrent(P_HMinus, P_ghZ, P_Hh, P_ghWmAnti, 887)
      call createCurrent(P_GPlus, P_Hh, P_ghA, P_ghWpAnti, 8640)
      call createCurrent(P_Z, P_ghA, P_WPlus, P_ghWpAnti, 4410)
      call createCurrent(P_HQPlus, P_GMinus, P_HaQ, 6723)
      call createCurrent(P_Z, P_G0, P_G0, P_Z, 219)
      call createCurrent(P_HPlus, P_ghWm, P_G0, P_ghAAnti, 9718)
      call createCurrent(P_GQMinus, P_Ha, P_HQPlus, 3544)
      call createCurrent(P_WPlus, P_ghWm, P_ghWmAnti, P_WMinus, 3227)
      call createCurrent(P_GQPlus, P_ghA, P_G0, P_ghWpAnti, 6711)
      call createCurrent(P_GPlus, P_WMinus, P_Hh, P_A, 5610)
      call createCurrent(P_WMinus, P_WPlus, P_GPlus, P_HMinus, 9776)
      call createCurrent(P_ghZ, P_Hl, P_Hl, P_ghZAnti, 3251)
      call createCurrent(P_cAnti, P_A, P_c, 3451)
      call createCurrent(P_WPlus, P_G0, P_GMinus, P_Z, 6732)
      call createCurrent(P_Hl, P_A, P_A, P_Hl, 10659)
      call createCurrent(P_AQ, P_ghWp, P_Z, P_ghWpAnti, 201)
      call createCurrent(P_AQ, P_WPlus, P_G0, P_GQMinus, 318)
      call createCurrent(P_GQPlus, P_Hh, P_GMinus, P_GQ0, 2165)
      call createCurrent(P_GMinus, P_Z, P_WPlus, P_Hl, 8188)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_HhQ, 4376)
      call createCurrent(P_ghA, P_ghZAnti, P_HPlus, P_HMinus, 386)
      call createCurrent(P_A, P_WPlus, P_ghWpAnti, P_ghZ, 7183)
      call createCurrent(P_WQMinus, P_Hh, P_HPlus, P_AQ, 1077)
      call createCurrent(P_G0, P_Ha, P_GPlus, P_GMinus, 8413)
      call createCurrent(P_G0, P_Hl, P_GMinus, P_GPlus, 9030)
      call createCurrent(P_ghA, P_GMinus, P_G0, P_ghWmAnti, 6808)
      call createCurrent(P_HPlus, P_s, P_cAnti, 4354)
      call createCurrent(P_ghWp, P_WMinus, P_ghZAnti, P_ZQ, 8638)
      call createCurrent(P_ghWmAnti, P_WMinus, P_WMinus, P_ghWp, 8180)
      call createCurrent(P_bAnti, P_b, P_G0, 6825)
      call createCurrent(P_WQPlus, P_WMinus, P_Hh, P_HaQ, 2291)
      call createCurrent(P_HQPlus, P_G0, P_Ha, P_HQMinus, 5291)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hl, P_GQ0, 8308)
      call createCurrent(P_GMinus, P_GPlus, P_GMinus, P_HPlus, 5256)
      call createCurrent(P_Hl, P_ghWp, P_ghAAnti, P_GMinus, 526)
      call createCurrent(P_GPlus, P_GMinus, P_Hh, P_Hl, 8459)
      call createCurrent(P_WPlus, P_nu_muAnti, P_muMinus, 5956)
      call createCurrent(P_HQMinus, P_Hh, P_HQPlus, 3250)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghWp, P_Z, 7118)
      call createCurrent(P_WMinus, P_c, P_sAnti, 3296)
      call createCurrent(P_ghZAnti, P_ghWm, P_Ha, P_GPlus, 10157)
      call createCurrent(P_GMinus, P_Hl, P_A, P_WPlus, 1922)
      call createCurrent(P_HaQ, P_Hh, P_HMinus, P_GQPlus, 1001)
      call createCurrent(P_HaQ, P_HMinus, P_HQPlus, 4768)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hl, P_GQ0, 8110)
      call createCurrent(P_ghWp, P_G0, P_ghWpAnti, P_G0, 6350)
      call createCurrent(P_Hl, P_G0, P_Hl, P_Ha, 6090)
      call createCurrent(P_Hh, P_ghZAnti, P_HMinus, P_ghWp, 199)
      call createCurrent(P_Hl, P_ghAAnti, P_HPlus, P_ghWm, 4454)
      call createCurrent(P_Hl, P_WPlus, P_Hl, P_WMinus, 4959)
      call createCurrent(P_GMinus, P_Z, P_A, P_HPlus, 9475)
      call createCurrent(P_s, P_sAnti, P_Hh, 3935)
      call createCurrent(P_GMinus, P_Z, P_HPlus, P_Z, 9430)
      call createCurrent(P_HPlus, P_Z, P_Hl, P_WMinus, 63)
      call createCurrent(P_HMinus, P_Hh, P_ghWp, P_ghZAnti, 4151)
      call createCurrent(P_GQ0, P_GPlus, P_ghWpAnti, P_ghZ, 4088)
      call createCurrent(P_ghWpAnti, P_HMinus, P_GPlus, P_ghWp, 10401)
      call createCurrent(P_tauMinus, P_tauPlus, P_A, 4989)
      call createCurrent(P_Hl, P_Hh, P_ghWpAnti, P_ghWp, 9099)
      call createCurrent(P_GQ0, P_WPlus, P_Hl, P_WQMinus, 1625)
      call createCurrent(P_A, P_Hh, P_Hh, P_A, 10489)
      call createCurrent(P_Hh, P_ghWm, P_ghWmAnti, P_Hl, 9479)
      call createCurrent(P_GMinus, P_Hh, P_Ha, P_HPlus, 6702)
      call createCurrent(P_Hh, P_ghZ, P_ghWmAnti, P_HMinus, 6837)
      call createCurrent(P_HQMinus, P_GPlus, P_Hh, P_HaQ, 2719)
      call createCurrent(P_WMinus, P_GPlus, P_A, 5340)
      call createCurrent(P_Ha, P_ghWpAnti, P_HPlus, P_ghA, 7675)
      call createCurrent(P_ZQ, P_Hh, P_WPlus, P_HQMinus, 854)
      call createCurrent(P_HMinus, P_GMinus, P_HPlus, P_HPlus, 6406)
      call createCurrent(P_Hl, P_HMinus, P_GPlus, P_Ha, 335)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghWm, P_Hh, 2865)
      call createCurrent(P_WPlus, P_A, P_WMinus, P_A, 8410)
      call createCurrent(P_A, P_Z, P_Z, P_Z, 10588)
      call createCurrent(P_WQPlus, P_Z, P_G0, P_GQMinus, 6327)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hh, P_GPlus, 498)
      call createCurrent(P_Z, P_Hl, P_HMinus, P_WPlus, 499)
      call createCurrent(P_A, P_ghZAnti, P_WPlus, P_ghWm, 1854)
      call createCurrent(P_HhQ, P_Hh, P_WMinus, P_WQPlus, 8537)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_HaQ, 1229)
      call createCurrent(P_Hh, P_HPlus, P_HMinus, P_G0, 9252)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghWm, P_Hl, 9772)
      call createCurrent(P_GMinus, P_HPlus, P_ghA, P_ghAAnti, 9259)
      call createCurrent(P_HQPlus, P_Hl, P_Hh, P_GQMinus, 2825)
      call createCurrent(P_tauPlus, P_tauMinus, P_HhQ, 7502)
      call createCurrent(P_ghA, P_ghWpAnti, P_Ha, P_HQPlus, 3894)
      call createCurrent(P_HQMinus, P_HPlus, P_Hh, P_HhQ, 3873)
      call createCurrent(P_G0, P_GMinus, P_Ha, P_HPlus, 7296)
      call createCurrent(P_GMinus, P_ghZ, P_GPlus, P_ghAAnti, 4664)
      call createCurrent(P_u, P_uAnti, P_AQ, 1166)
      call createCurrent(P_Ha, P_ghWm, P_ghZAnti, P_GPlus, 10199)
      call createCurrent(P_HMinus, P_Ha, P_G0, P_GPlus, 8803)
      call createCurrent(P_ghA, P_ghWpAnti, P_G0, P_GQPlus, 920)
      call createCurrent(P_Z, P_WMinus, P_ghA, P_ghWmAnti, 4711)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghA, P_HMinus, 3767)
      call createCurrent(P_Hh, P_G0, P_G0, P_Hl, 3556)
      call createCurrent(P_Hl, P_HMinus, P_ghZ, P_ghWmAnti, 8765)
      call createCurrent(P_nu_eAnti, P_nu_e, 10227)
      call createCurrent(P_GQ0, P_Ha, P_GPlus, P_GQMinus, 5038)
      call createCurrent(P_A, P_GMinus, P_GPlus, P_A, 1866)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghZ, 8286)
      call createCurrent(P_GPlus, P_ghAAnti, P_GMinus, P_ghZ, 1301)
      call createCurrent(P_Hh, P_G0, P_GPlus, P_HMinus, 198)
      call createCurrent(P_G0, P_ghAAnti, P_HPlus, P_ghWm, 9075)
      call createCurrent(P_Z, P_HPlus, P_A, P_GMinus, 10267)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hl, P_HlQ, 3781)
      call createCurrent(P_GQ0, P_GMinus, P_ghAAnti, P_ghWp, 3517)
      call createCurrent(P_Ha, P_Z, P_HMinus, P_WPlus, 5791)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, 1638)
      call createCurrent(P_ghA, P_Hh, P_ghWmAnti, P_HMinus, 5196)
      call createCurrent(P_HhQ, P_Hl, P_G0, P_GQ0, 4628)
      call createCurrent(P_HMinus, P_G0, P_WPlus, P_A, 10450)
      call createCurrent(P_G0, P_HPlus, P_HMinus, 9640)
      call createCurrent(P_Hh, P_ghZ, P_ghWmAnti, P_GMinus, 4822)
      call createCurrent(P_GPlus, P_HMinus, P_G0, P_Hh, 1848)
      call createCurrent(P_HlQ, P_dAnti, P_d, 6919)
      call createCurrent(P_GMinus, P_Z, P_Hl, P_WPlus, 1289)
      call createCurrent(P_ZQ, P_G0, P_WPlus, P_GQMinus, 4077)
      call createCurrent(P_HMinus, P_GPlus, P_Hl, P_Hl, 2242)
      call createCurrent(P_GQMinus, P_ghZ, P_ghWmAnti, 3253)
      call createCurrent(P_HMinus, P_GPlus, P_GPlus, P_GMinus, 7204)
      call createCurrent(P_GPlus, P_WMinus, P_WPlus, P_GMinus, 3788)
      call createCurrent(P_ghZAnti, P_ghWm, P_GQPlus, 4562)
      call createCurrent(P_tauMinus, P_nu_tauAnti, P_HQPlus, 855)
      call createCurrent(P_ghZAnti, P_HPlus, P_HMinus, P_ghA, 2435)
      call createCurrent(P_Hh, P_Ha, P_G0, P_Hh, 4103)
      call createCurrent(P_GQPlus, P_Ha, P_HMinus, P_HlQ, 8283)
      call createCurrent(P_ghZ, P_Hl, P_ghWmAnti, P_HQMinus, 691)
      call createCurrent(P_GQ0, P_G0, P_HlQ, 4906)
      call createCurrent(P_HQPlus, P_Hh, P_HMinus, P_HhQ, 8095)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghA, P_HQMinus, 7651)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hl, P_GQMinus, 6476)
      call createCurrent(P_GPlus, P_ghWpAnti, P_Hl, P_ghA, 3497)
      call createCurrent(P_HPlus, P_HMinus, P_ghWpAnti, P_ghWp, 6945)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Z, P_WPlus, 4453)
      call createCurrent(P_HQMinus, P_HPlus, P_Hh, P_HaQ, 1649)
      call createCurrent(P_A, P_GPlus, P_Hl, P_WMinus, 1584)
      call createCurrent(P_GMinus, P_ghWp, P_Hh, P_ghAAnti, 943)
      call createCurrent(P_ghWm, P_HMinus, P_HPlus, P_ghWmAnti, 811)
      call createCurrent(P_Ha, P_ghWm, P_ghAAnti, P_GPlus, 9188)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Ha, P_HQPlus, 5580)
      call createCurrent(P_G0, P_Ha, P_ghZ, P_ghZAnti, 10362)
      call createCurrent(P_ZQ, P_WMinus, P_Hl, P_GQPlus, 4500)
      call createCurrent(P_WPlus, P_GMinus, P_HPlus, P_WMinus, 9033)
      call createCurrent(P_b, P_tAnti, P_HPlus, 3137)
      call createCurrent(P_c, P_sAnti, P_HMinus, 6503)
      call createCurrent(P_GQPlus, P_G0, P_WMinus, P_ZQ, 804)
      call createCurrent(P_WMinus, P_A, P_A, P_WPlus, 5416)
      call createCurrent(P_G0, P_Hl, P_G0, P_Hh, 6710)
      call createCurrent(P_HhQ, P_HPlus, P_A, P_WQMinus, 446)
      call createCurrent(P_WPlus, P_WMinus, P_ghAAnti, P_ghZ, 7385)
      call createCurrent(P_nu_eAnti, P_eMinus, P_HQPlus, 959)
      call createCurrent(P_HlQ, P_HMinus, P_Z, P_WQPlus, 7664)
      call createCurrent(P_A, P_Hh, P_WPlus, P_GMinus, 8349)
      call createCurrent(P_GQMinus, P_Hh, P_HPlus, P_HlQ, 2013)
      call createCurrent(P_A, P_Hl, P_A, 10623)
      call createCurrent(P_ghWp, P_Hl, P_Hh, P_ghWpAnti, 9465)
      call createCurrent(P_nu_tau, P_tauPlus, P_HQMinus, 7653)
      call createCurrent(P_HhQ, P_GPlus, P_Hl, P_GQMinus, 2402)
      call createCurrent(P_Hh, P_Z, P_G0, 2388)
      call createCurrent(P_Z, P_GPlus, P_WMinus, P_Hl, 289)
      call createCurrent(P_tauPlus, P_tauMinus, 8922)
      call createCurrent(P_Z, P_ghZ, P_WMinus, P_ghWmAnti, 5767)
      call createCurrent(P_WQMinus, P_Z, P_ghWmAnti, P_ghZ, 5410)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghWm, P_AQ, 957)
      call createCurrent(P_G0, P_HPlus, P_HMinus, P_G0, 580)
      call createCurrent(P_HhQ, P_Hh, P_ghWpAnti, P_ghWp, 2562)
      call createCurrent(P_Ha, P_ghZAnti, P_Ha, P_ghZ, 2038)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, 8215)
      call createCurrent(P_WPlus, P_GPlus, P_WMinus, P_HMinus, 10382)
      call createCurrent(P_ghWmAnti, P_Z, P_ghA, P_WMinus, 7861)
      call createCurrent(P_ghWp, P_G0, P_G0, P_ghWpAnti, 4684)
      call createCurrent(P_ghWmAnti, P_Z, P_ghWm, P_ZQ, 4996)
      call createCurrent(P_HQPlus, P_Hl, P_ghWpAnti, P_ghZ, 2587)
      call createCurrent(P_Ha, P_WPlus, P_G0, P_WMinus, 9034)
      call createCurrent(P_HPlus, P_A, P_Hl, P_WMinus, 4709)
      call createCurrent(P_Z, P_HMinus, P_GPlus, P_A, 10112)
      call createCurrent(P_GQPlus, P_GPlus, P_HMinus, P_GQMinus, 3640)
      call createCurrent(P_GMinus, P_HPlus, P_ghZ, P_ghZAnti, 9589)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hh, P_Hl, 9294)
      call createCurrent(P_dAnti, P_d, P_GQ0, 3188)
      call createCurrent(P_GQPlus, P_HMinus, P_Hh, P_GQ0, 5693)
      call createCurrent(P_t, P_tAnti, P_HaQ, 7574)
      call createCurrent(P_WMinus, P_Z, P_WPlus, P_Z, 7776)
      call createCurrent(P_Hh, P_WMinus, P_WPlus, P_Hh, 8228)
      call createCurrent(P_Z, P_WPlus, P_WMinus, 842)
      call createCurrent(P_ghWpAnti, P_ghA, P_WPlus, P_Z, 278)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HPlus, P_HMinus, 3493)
      call createCurrent(P_ghA, P_ghZAnti, P_HMinus, P_HPlus, 2256)
      call createCurrent(P_ghZ, P_ghWmAnti, P_WMinus, P_A, 169)
      call createCurrent(P_ghAAnti, P_HMinus, P_HPlus, P_ghZ, 4181)
      call createCurrent(P_WQMinus, P_WPlus, P_Z, P_AQ, 1412)
      call createCurrent(P_WPlus, P_uAnti, P_d, 1101)
      call createCurrent(P_GQMinus, P_HPlus, P_HPlus, P_HQMinus, 8561)
      call createCurrent(P_HQPlus, P_Ha, P_HMinus, P_GQ0, 7652)
      call createCurrent(P_ghWm, P_Hl, P_ghWmAnti, P_HlQ, 2936)
      call createCurrent(P_HQMinus, P_Hh, P_Z, P_WQPlus, 5949)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_Hl, 4017)
      call createCurrent(P_WQPlus, P_Hl, P_Z, P_HQMinus, 4492)
      call createCurrent(P_GPlus, P_ghZ, P_Ha, P_ghWpAnti, 9754)
      call createCurrent(P_Hh, P_GPlus, P_HMinus, P_Hh, 3302)
      call createCurrent(P_ghWmAnti, P_GPlus, P_ghWm, P_GMinus, 5120)
      call createCurrent(P_ghWm, P_Hh, P_ghAAnti, P_HPlus, 8558)
      call createCurrent(P_WQPlus, P_GMinus, P_Z, P_GQ0, 1159)
      call createCurrent(P_ghWmAnti, P_ghWm, P_A, 7311)
      call createCurrent(P_G0, P_HMinus, P_A, P_WPlus, 10237)
      call createCurrent(P_uAnti, P_A, P_u, 5365)
      call createCurrent(P_G0, P_ghWp, P_ghZAnti, P_HMinus, 9758)
      call createCurrent(P_GQPlus, P_HMinus, P_Hl, P_HaQ, 3391)
      call createCurrent(P_ghA, P_ghZAnti, P_GPlus, P_GMinus, 6666)
      call createCurrent(P_Ha, P_ghZ, P_ghZAnti, P_Ha, 3266)
      call createCurrent(P_WPlus, P_Ha, P_HMinus, P_A, 8062)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghZ, P_HQMinus, 2547)
      call createCurrent(P_HPlus, P_G0, P_WMinus, 10044)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghWm, P_HQPlus, 6169)
      call createCurrent(P_HMinus, P_GPlus, P_g, P_g, 10651)
      call createCurrent(P_HMinus, P_A, P_GPlus, P_A, 9371)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_ghA, 7808)
      call createCurrent(P_Ha, P_GMinus, P_ghWp, P_ghAAnti, 8854)
      call createCurrent(P_A, P_nu_eAnti, P_nu_e, 10424)
      call createCurrent(P_HQPlus, P_GMinus, P_G0, P_HaQ, 623)
      call createCurrent(P_A, P_Hh, P_Hh, P_Z, 9804)
      call createCurrent(P_GQPlus, P_ghZ, P_GMinus, P_ghAAnti, 6844)
      call createCurrent(P_HPlus, P_ghA, P_HMinus, P_ghZAnti, 6278)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hl, P_GMinus, 2143)
      call createCurrent(P_HMinus, P_Z, P_Z, P_HPlus, 2041)
      call createCurrent(P_HMinus, P_Hh, P_A, P_WPlus, 6465)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Z, 7958)
      call createCurrent(P_nu_eAnti, P_Z, P_nu_e, 6621)
      call createCurrent(P_HaQ, P_Ha, P_G0, P_GQ0, 5643)
      call createCurrent(P_AQ, P_Hl, P_HPlus, P_WQMinus, 4052)
      call createCurrent(P_HMinus, P_GPlus, P_ghWmAnti, P_ghWm, 10313)
      call createCurrent(P_ghA, P_HMinus, P_Hh, P_ghWmAnti, 6658)
      call createCurrent(P_Ha, P_Hl, P_GMinus, P_GPlus, 9622)
      call createCurrent(P_HaQ, P_ghWpAnti, P_HPlus, P_ghZ, 7404)
      call createCurrent(P_WPlus, P_Ha, P_WMinus, P_Ha, 7256)
      call createCurrent(P_HMinus, P_Hh, P_GPlus, 6681)
      call createCurrent(P_WQPlus, P_A, P_GMinus, P_GQ0, 5270)
      call createCurrent(P_Ha, P_c, P_cAnti, 6791)
      call createCurrent(P_Hl, P_GMinus, P_GPlus, P_Ha, 9554)
      call createCurrent(P_Hl, P_Hh, P_GPlus, P_GMinus, 3844)
      call createCurrent(P_ZQ, P_HMinus, P_HPlus, P_AQ, 8619)
      call createCurrent(P_WQPlus, P_Hh, P_GQMinus, 2628)
      call createCurrent(P_HlQ, P_A, P_WPlus, P_GQMinus, 8385)
      call createCurrent(P_GQMinus, P_GPlus, P_HPlus, P_HQMinus, 8307)
      call createCurrent(P_GQPlus, P_GMinus, P_AQ, 3633)
      call createCurrent(P_WQMinus, P_GMinus, P_WPlus, P_GQPlus, 1914)
      call createCurrent(P_ZQ, P_WMinus, P_WPlus, P_ZQ, 154)
      call createCurrent(P_GMinus, P_ghAAnti, P_HPlus, P_ghA, 10204)
      call createCurrent(P_G0, P_b, P_bAnti, 7665)
      call createCurrent(P_Hh, P_GPlus, P_Ha, P_GMinus, 10033)
      call createCurrent(P_HPlus, P_Ha, P_Hl, P_GMinus, 4470)
      call createCurrent(P_HMinus, P_GMinus, P_GPlus, P_HPlus, 1586)
      call createCurrent(P_Z, P_ghWm, P_A, P_ghWmAnti, 309)
      call createCurrent(P_HaQ, P_HMinus, P_Hh, P_GQPlus, 7621)
      call createCurrent(P_HQPlus, P_GMinus, P_Ha, P_GQ0, 123)
      call createCurrent(P_ghA, P_ghAAnti, P_WPlus, P_WMinus, 8525)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghWm, P_Hl, 9120)
      call createCurrent(P_AQ, P_GMinus, P_G0, P_WQPlus, 2450)
      call createCurrent(P_Hl, P_HMinus, P_WPlus, P_Z, 6850)
      call createCurrent(P_GMinus, P_WPlus, P_Ha, 9309)
      call createCurrent(P_ghWm, P_GPlus, P_ghWmAnti, P_HMinus, 10229)
      call createCurrent(P_WQPlus, P_HMinus, P_HlQ, 3346)
      call createCurrent(P_ghZ, P_Ha, P_ghWpAnti, P_HPlus, 2087)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HMinus, P_HPlus, 226)
      call createCurrent(P_uAnti, P_Ha, P_u, 4843)
      call createCurrent(P_GQ0, P_Ha, P_G0, P_GQ0, 3224)
      call createCurrent(P_WQPlus, P_WMinus, P_WPlus, P_WQMinus, 6433)
      call createCurrent(P_ghWm, P_G0, P_Ha, P_ghWmAnti, 9502)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghWp, P_Hl, 9661)
      call createCurrent(P_HQPlus, P_WMinus, P_Hl, P_AQ, 7272)
      call createCurrent(P_HPlus, P_WMinus, P_Hh, P_Z, 5917)
      call createCurrent(P_ghWmAnti, P_ghA, P_Ha, P_GMinus, 10110)
      call createCurrent(P_HPlus, P_ghWm, P_HMinus, P_ghWmAnti, 2846)
      call createCurrent(P_GQPlus, P_G0, P_ghZ, P_ghWpAnti, 6640)
      call createCurrent(P_HMinus, P_GPlus, P_ghZAnti, P_ghA, 9429)
      call createCurrent(P_Hh, P_Ha, P_G0, P_Hl, 8637)
      call createCurrent(P_ghZ, P_WMinus, P_ghAAnti, P_WQPlus, 4415)
      call createCurrent(P_GPlus, P_Ha, P_Ha, P_HMinus, 5003)
      call createCurrent(P_GPlus, P_ghAAnti, P_Ha, P_ghWm, 10351)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghZ, P_Z, 1097)
      call createCurrent(P_ghZAnti, P_ghZ, P_G0, P_Ha, 9123)
      call createCurrent(P_ghZ, P_Z, P_ghWmAnti, P_WQMinus, 1085)
      call createCurrent(P_ghZAnti, P_GMinus, P_Hl, P_ghWp, 480)
      call createCurrent(P_tAnti, P_WPlus, P_b, 3405)
      call createCurrent(P_Ha, P_HPlus, P_ghZAnti, P_ghWm, 808)
      call createCurrent(P_HhQ, P_HPlus, P_Hh, P_GQMinus, 4813)
      call createCurrent(P_GQ0, P_Z, P_WMinus, P_GQPlus, 8586)
      call createCurrent(P_WPlus, P_Z, P_HMinus, P_G0, 9125)
      call createCurrent(P_uAnti, P_WPlus, P_d, 4818)
      call createCurrent(P_GMinus, P_Ha, P_WPlus, P_Z, 10416)
      call createCurrent(P_ghZ, P_ghAAnti, P_HPlus, P_HQMinus, 2804)
      call createCurrent(P_muMinus, P_muPlus, P_ZQ, 4721)
      call createCurrent(P_Z, P_HMinus, P_WPlus, P_Hh, 1963)
      call createCurrent(P_s, P_sAnti, P_gQ, 7276)
      call createCurrent(P_G0, P_ghWmAnti, P_ghWm, P_Ha, 9692)
      call createCurrent(P_g, P_Hl, P_g, P_Hh, 10560)
      call createCurrent(P_Hh, P_WPlus, P_HMinus, 3828)
      call createCurrent(P_HhQ, P_Hl, P_GPlus, P_HQMinus, 1663)
      call createCurrent(P_ghZ, P_Hl, P_HPlus, P_ghWpAnti, 6274)
      call createCurrent(P_HhQ, P_G0, P_ghWpAnti, P_ghWp, 8517)
      call createCurrent(P_GMinus, P_HPlus, P_Z, P_A, 10414)
      call createCurrent(P_GPlus, P_ghA, P_Ha, P_ghWpAnti, 9595)
      call createCurrent(P_Ha, P_ePlus, P_eMinus, 2252)
      call createCurrent(P_HMinus, P_sAnti, P_c, 6539)
      call createCurrent(P_GQPlus, P_HMinus, P_G0, P_HaQ, 3621)
      call createCurrent(P_Hl, P_ghZ, P_ghWpAnti, P_HPlus, 6417)
      call createCurrent(P_Hl, P_WPlus, P_WMinus, 6479)
      call createCurrent(P_WQMinus, P_Z, P_G0, P_GQPlus, 6531)
      call createCurrent(P_AQ, P_G0, P_GPlus, P_WQMinus, 6733)
      call createCurrent(P_ghZ, P_Ha, P_GMinus, P_ghWmAnti, 10411)
      call createCurrent(P_G0, P_Ha, P_G0, P_Hh, 10572)
      call createCurrent(P_GMinus, P_g, P_HPlus, P_g, 10617)
      call createCurrent(P_HPlus, P_ghZ, P_ghWpAnti, 4690)
      call createCurrent(P_GMinus, P_ghWp, P_ghAAnti, P_Ha, 8981)
      call createCurrent(P_HaQ, P_HPlus, P_HQMinus, 514)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hh, P_GMinus, 5252)
      call createCurrent(P_HMinus, P_Z, P_Hh, P_WPlus, 4783)
      call createCurrent(P_ePlus, P_nu_e, P_HMinus, 3768)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghWp, P_HMinus, 9312)
      call createCurrent(P_AQ, P_HMinus, P_Hl, P_WQPlus, 3169)
      call createCurrent(P_HMinus, P_HPlus, P_ghWpAnti, P_ghWp, 23)
      call createCurrent(P_HaQ, P_Hl, P_G0, P_HlQ, 3634)
      call createCurrent(P_ghWm, P_ghZAnti, P_WPlus, P_ZQ, 6601)
      call createCurrent(P_ghZAnti, P_Hh, P_HPlus, P_ghWm, 7765)
      call createCurrent(P_Ha, P_A, P_Z, P_Ha, 9996)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hl, P_HlQ, 7135)
      call createCurrent(P_Ha, P_A, P_G0, P_A, 10633)
      call createCurrent(P_HQMinus, P_WPlus, P_Hh, P_AQ, 3787)
      call createCurrent(P_AQ, P_GPlus, P_Z, P_GQMinus, 8575)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_GQ0, 1687)
      call createCurrent(P_ghWmAnti, P_HPlus, P_HMinus, P_ghWm, 3192)
      call createCurrent(P_Hh, P_HPlus, P_A, P_WMinus, 6762)
      call createCurrent(P_GQ0, P_muMinus, P_muPlus, 6369)
      call createCurrent(P_WMinus, P_ghA, P_ghZAnti, P_WPlus, 3486)
      call createCurrent(P_WPlus, P_Hl, P_GMinus, 2448)
      call createCurrent(P_G0, P_GPlus, P_WMinus, 8289)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghWp, P_GQ0, 3039)
      call createCurrent(P_Hl, P_G0, P_GPlus, P_GMinus, 10445)
      call createCurrent(P_ghZAnti, P_ghA, P_WPlus, P_WQMinus, 7233)
      call createCurrent(P_ghZAnti, P_ghZ, P_WMinus, P_WPlus, 2773)
      call createCurrent(P_GMinus, P_ghWmAnti, P_Hh, P_ghA, 8392)
      call createCurrent(P_GQ0, P_HMinus, P_GPlus, P_HhQ, 1157)
      call createCurrent(P_WPlus, P_HMinus, P_Ha, 6308)
      call createCurrent(P_WQPlus, P_Hl, P_Z, P_GQMinus, 6359)
      call createCurrent(P_Z, P_d, P_dAnti, 159)
      call createCurrent(P_G0, P_WMinus, P_WPlus, P_G0, 7879)
      call createCurrent(P_GMinus, P_Hl, P_ghA, P_ghWmAnti, 5816)
      call createCurrent(P_ghWmAnti, P_ghZ, P_WQMinus, 4615)
      call createCurrent(P_HPlus, P_WMinus, P_A, P_Hl, 7911)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_HlQ, 4398)
      call createCurrent(P_GPlus, P_GMinus, P_Ha, P_Hh, 9267)
      call createCurrent(P_G0, P_Ha, P_Z, P_Z, 9815)
      call createCurrent(P_A, P_WMinus, P_ghZ, P_ghWmAnti, 7287)
      call createCurrent(P_AQ, P_ghWpAnti, P_Z, P_ghWp, 4476)
      call createCurrent(P_WQPlus, P_WMinus, P_A, P_AQ, 4631)
      call createCurrent(P_HaQ, P_GMinus, P_GQPlus, 7260)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_G0, 3722)
      call createCurrent(P_WPlus, P_ghWm, P_ghZAnti, P_Z, 5604)
      call createCurrent(P_GQ0, P_Hh, P_GPlus, P_HQMinus, 4753)
      call createCurrent(P_ghWpAnti, P_A, P_ghA, P_WQPlus, 2354)
      call createCurrent(P_ghWp, P_WMinus, P_ghWpAnti, P_WPlus, 3190)
      call createCurrent(P_ghA, P_ghWpAnti, P_WPlus, P_A, 5290)
      call createCurrent(P_Z, P_ghWpAnti, P_Z, P_ghWp, 3239)
      call createCurrent(P_GMinus, P_WPlus, P_Hh, 7397)
      call createCurrent(P_HQPlus, P_Ha, P_Hh, P_GQMinus, 594)
      call createCurrent(P_Hl, P_WPlus, P_A, P_GMinus, 5925)
      call createCurrent(P_GPlus, P_HMinus, P_Hh, P_G0, 3946)
      call createCurrent(P_cAnti, P_s, P_WPlus, 4655)
      call createCurrent(P_WQMinus, P_WPlus, P_A, P_ZQ, 2675)
      call createCurrent(P_GQ0, P_Z, P_Z, P_GQ0, 3037)
      call createCurrent(P_HlQ, P_Hl, P_G0, P_HaQ, 1368)
      call createCurrent(P_HMinus, P_ghWmAnti, P_G0, P_ghA, 9266)
      call createCurrent(P_ghWp, P_GPlus, P_ghWpAnti, P_GMinus, 2457)
      call createCurrent(P_HMinus, P_WMinus, P_WPlus, P_HPlus, 3872)
      call createCurrent(P_ghWmAnti, P_G0, P_G0, P_ghWm, 5862)
      call createCurrent(P_GQPlus, P_Ha, P_GMinus, P_HhQ, 7400)
      call createCurrent(P_AQ, P_WMinus, P_ghWp, P_ghZAnti, 6340)
      call createCurrent(P_WQPlus, P_Z, P_Hh, P_HQMinus, 2549)
      call createCurrent(P_A, P_nu_tau, P_nu_tauAnti, 9657)
      call createCurrent(P_Hl, P_GPlus, P_HMinus, P_Hh, 5519)
      call createCurrent(P_GMinus, P_WPlus, P_Z, P_Ha, 10322)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hh, 7434)
      call createCurrent(P_HhQ, P_ghWm, P_ghWmAnti, 6987)
      call createCurrent(P_ghWpAnti, P_A, P_ghZ, P_WQPlus, 8081)
      call createCurrent(P_tauPlus, P_nu_tau, P_WQMinus, 2317)
      call createCurrent(P_GMinus, P_WPlus, 9527)
      call createCurrent(P_Ha, P_G0, P_GMinus, P_HPlus, 1393)
      call createCurrent(P_ZQ, P_ghAAnti, P_WPlus, P_ghWm, 588)
      call createCurrent(P_A, P_t, P_tAnti, 2664)
      call createCurrent(P_HMinus, P_Ha, P_WPlus, P_Z, 1834)
      call createCurrent(P_b, P_G0, P_bAnti, 3166)
      call createCurrent(P_GPlus, P_ghZAnti, P_G0, P_ghWm, 4343)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghZ, P_HPlus, 10179)
      call createCurrent(P_A, P_HMinus, P_Z, P_GPlus, 10257)
      call createCurrent(P_Hh, P_Z, P_GMinus, P_WPlus, 1495)
      call createCurrent(P_Hl, P_WMinus, P_GPlus, 8727)
      call createCurrent(P_Hl, P_Z, P_A, P_Hh, 10647)
      call createCurrent(P_GQMinus, P_Ha, P_GPlus, P_HaQ, 61)
      call createCurrent(P_Hl, P_WMinus, P_WPlus, P_Hl, 8700)
      call createCurrent(P_GMinus, P_A, P_Z, P_HPlus, 10308)
      call createCurrent(P_G0, P_WPlus, P_HMinus, P_A, 9739)
      call createCurrent(P_G0, P_Hh, P_G0, P_Hh, 1581)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_HlQ, 3247)
      call createCurrent(P_Ha, P_t, P_tAnti, 5226)
      call createCurrent(P_AQ, P_ghWm, P_WPlus, P_ghAAnti, 5554)
      call createCurrent(P_HMinus, P_ghA, P_G0, P_ghWmAnti, 9114)
      call createCurrent(P_ghWpAnti, P_ghZ, P_WPlus, P_Z, 3170)
      call createCurrent(P_GQPlus, P_HMinus, P_HMinus, P_HQPlus, 3696)
      call createCurrent(P_HMinus, P_ghZAnti, P_Hl, P_ghWp, 146)
      call createCurrent(P_ghA, P_ghAAnti, P_HPlus, P_HMinus, 2205)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_HaQ, 6976)
      call createCurrent(P_tAnti, P_t, P_g, 2054)
      call createCurrent(P_ghWm, P_Hl, P_HPlus, P_ghAAnti, 6527)
      call createCurrent(P_Hh, P_ghZAnti, P_HPlus, P_ghWm, 1246)
      call createCurrent(P_Hl, P_GPlus, P_GMinus, P_Hl, 6118)
      call createCurrent(P_G0, P_Ha, P_HMinus, P_GPlus, 1003)
      call createCurrent(P_GPlus, P_ghZ, P_ghWpAnti, 7416)
      call createCurrent(P_GPlus, P_Ha, P_GMinus, P_G0, 2412)
      call createCurrent(P_GMinus, P_ghWp, P_ghAAnti, P_G0, 4642)
      call createCurrent(P_HPlus, P_GMinus, P_WPlus, P_WMinus, 9251)
      call createCurrent(P_nu_e, P_ePlus, P_WQMinus, 5687)
      call createCurrent(P_HPlus, P_ghA, P_GMinus, P_ghAAnti, 9473)
      call createCurrent(P_HQMinus, P_WPlus, P_ZQ, 3845)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghA, P_HQMinus, 1715)
      call createCurrent(P_ghWm, P_ghAAnti, P_G0, P_HPlus, 9731)
      call createCurrent(P_GMinus, P_WPlus, P_GPlus, P_WMinus, 8819)
      call createCurrent(P_AQ, P_HMinus, P_HPlus, P_ZQ, 6698)
      call createCurrent(P_WPlus, P_GMinus, P_A, P_G0, 4704)
      call createCurrent(P_s, P_sAnti, P_HlQ, 2014)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_ghA, 3214)
      call createCurrent(P_GPlus, P_WMinus, P_G0, 649)
      call createCurrent(P_GQPlus, P_WPlus, P_WMinus, P_GQMinus, 690)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghA, P_GPlus, 1194)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Ha, P_GPlus, 10003)
      call createCurrent(P_A, P_Hl, P_WPlus, P_HMinus, 4169)
      call createCurrent(P_ghZ, P_ghZAnti, P_GPlus, P_HMinus, 9344)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hl, P_GQPlus, 5844)
      call createCurrent(P_HaQ, P_Hh, P_Hh, P_HaQ, 8665)
      call createCurrent(P_ZQ, P_dAnti, P_d, 1947)
      call createCurrent(P_HPlus, P_GMinus, P_ghWp, P_ghWpAnti, 10307)
      call createCurrent(P_GPlus, P_WMinus, P_Hh, 1884)
      call createCurrent(P_Hh, P_GPlus, P_ghAAnti, P_ghWm, 1455)
      call createCurrent(P_AQ, P_GPlus, P_GQMinus, 6296)
      call createCurrent(P_WPlus, P_HMinus, P_Ha, P_A, 584)
      call createCurrent(P_Ha, P_Ha, P_GPlus, P_GMinus, 453)
      call createCurrent(P_HPlus, P_HPlus, P_HMinus, P_GMinus, 7083)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HPlus, P_HMinus, 5287)
      call createCurrent(P_HhQ, P_bAnti, P_b, 8423)
      call createCurrent(P_GQ0, P_GPlus, P_ghWm, P_ghZAnti, 7199)
      call createCurrent(P_ghWmAnti, P_HMinus, P_G0, P_ghA, 9076)
      call createCurrent(P_g, P_Hl, P_Hh, P_g, 10599)
      call createCurrent(P_GPlus, P_ghWp, P_GMinus, P_ghWpAnti, 4457)
      call createCurrent(P_GPlus, P_ghA, P_ghWpAnti, P_Hh, 4011)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghWm, P_G0, 8729)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hl, P_HlQ, 8320)
      call createCurrent(P_ghZAnti, P_WPlus, P_WMinus, P_ghA, 2908)
      call createCurrent(P_GPlus, P_A, P_Z, P_HMinus, 10258)
      call createCurrent(P_GMinus, P_GPlus, P_Ha, 10160)
      call createCurrent(P_WPlus, P_ghZ, P_A, P_ghWpAnti, 1502)
      call createCurrent(P_Z, P_WMinus, P_GPlus, 5116)
      call createCurrent(P_HlQ, P_Ha, P_HPlus, P_HQMinus, 2762)
      call createCurrent(P_nu_tauAnti, P_tauMinus, P_HQPlus, 6933)
      call createCurrent(P_ghWpAnti, P_WMinus, P_ghWp, P_WPlus, 798)
      call createCurrent(P_t, P_tAnti, P_ZQ, 2409)
      call createCurrent(P_GPlus, P_Hh, P_ghWm, P_ghZAnti, 4483)
      call createCurrent(P_HlQ, P_c, P_cAnti, 4139)
      call createCurrent(P_tauPlus, P_Hh, P_tauMinus, 7539)
      call createCurrent(P_sAnti, P_s, P_HhQ, 3155)
      call createCurrent(P_Ha, P_HMinus, P_Hl, P_HPlus, 9775)
      call createCurrent(P_GQMinus, P_GPlus, P_ghAAnti, P_ghZ, 2993)
      call createCurrent(P_WQMinus, P_c, P_sAnti, 5500)
      call createCurrent(P_ghWp, P_ghWpAnti, P_GPlus, P_GMinus, 3003)
      call createCurrent(P_WPlus, P_WMinus, P_GMinus, P_GPlus, 2898)
      call createCurrent(P_A, P_WPlus, P_Z, P_WMinus, 7506)
      call createCurrent(P_G0, P_GMinus, P_Hl, P_HPlus, 3962)
      call createCurrent(P_ghGAnti, P_g, P_g, P_ghG, 5411)
      call createCurrent(P_G0, P_Hl, P_Ha, P_G0, 10614)
      call createCurrent(P_HaQ, P_Hl, P_ghWm, P_ghWmAnti, 3176)
      call createCurrent(P_A, P_g, P_g, P_Z, 10691)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghA, P_HMinus, 522)
      call createCurrent(P_WPlus, P_GMinus, P_Hl, P_Z, 4389)
      call createCurrent(P_WQMinus, P_GMinus, P_GPlus, P_WQPlus, 6978)
      call createCurrent(P_ghWp, P_WMinus, P_ghZAnti, P_A, 2266)
      call createCurrent(P_Hl, P_tauMinus, P_tauPlus, 3884)
      call createCurrent(P_A, P_WMinus, P_ghAAnti, P_ghWp, 6822)
      call createCurrent(P_Hh, P_b, P_bAnti, 2251)
      call createCurrent(P_HlQ, P_Hl, P_GPlus, P_HQMinus, 7584)
      call createCurrent(P_Hh, P_HMinus, P_GPlus, 7648)
      call createCurrent(P_Hh, P_Hh, P_Hl, P_Hl, 6016)
      call createCurrent(P_HMinus, P_ghWp, P_Hh, P_ghAAnti, 4785)
      call createCurrent(P_HMinus, P_ghA, P_Hl, P_ghWmAnti, 4330)
      call createCurrent(P_GPlus, P_HPlus, P_HMinus, P_GMinus, 3210)
      call createCurrent(P_ghWp, P_ghWpAnti, P_A, 2924)
      call createCurrent(P_ZQ, P_Hl, P_ZQ, 5803)
      call createCurrent(P_GPlus, P_Hl, P_GMinus, P_Ha, 10314)
      call createCurrent(P_G0, P_ghZ, P_ghWpAnti, P_GPlus, 5586)
      call createCurrent(P_WPlus, P_WMinus, P_Hh, 150)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_G0, P_ghZ, 173)
      call createCurrent(P_GQMinus, P_ghZAnti, P_GPlus, P_ghZ, 34)
      call createCurrent(P_GQMinus, P_Hl, P_HPlus, P_HlQ, 4837)
      call createCurrent(P_Ha, P_eMinus, P_ePlus, 6365)
      call createCurrent(P_WQMinus, P_HPlus, P_Hh, P_AQ, 8639)
      call createCurrent(P_HMinus, P_HPlus, P_Hl, P_Hl, 5221)
      call createCurrent(P_G0, P_tauMinus, P_tauPlus, 8082)
      call createCurrent(P_GPlus, P_ghZ, P_ghWpAnti, P_Hh, 1343)
      call createCurrent(P_Hl, P_ghWmAnti, P_HMinus, P_ghA, 7893)
      call createCurrent(P_HaQ, P_WMinus, P_A, P_HQPlus, 153)
      call createCurrent(P_A, P_HMinus, P_WPlus, P_Ha, 7066)
      call createCurrent(P_Hh, P_A, P_Z, P_Hh, 9381)
      call createCurrent(P_HhQ, P_GMinus, P_GQPlus, 3495)
      call createCurrent(P_AQ, P_d, P_dAnti, 3)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hh, P_HaQ, 6006)
      call createCurrent(P_muMinus, P_muPlus, P_Hl, 124)
      call createCurrent(P_nu_tau, P_nu_tauAnti, 8869)
      call createCurrent(P_HhQ, P_Hl, P_Ha, P_GQ0, 4008)
      call createCurrent(P_WMinus, P_Hl, P_GPlus, 5593)
      call createCurrent(P_GQ0, P_G0, P_GMinus, P_GQPlus, 7071)
      call createCurrent(P_Z, P_g, P_A, P_g, 10666)
      call createCurrent(P_WPlus, P_WMinus, P_HPlus, P_GMinus, 9334)
      call createCurrent(P_HQPlus, P_nu_muAnti, P_muMinus, 1894)
      call createCurrent(P_ghWpAnti, P_G0, P_ghZ, P_GQPlus, 7577)
      call createCurrent(P_ghWm, P_Z, P_Z, P_ghWmAnti, 4154)
      call createCurrent(P_A, P_HMinus, P_GPlus, P_Z, 8927)
      call createCurrent(P_ghA, P_HMinus, P_ghAAnti, P_HQPlus, 1315)
      call createCurrent(P_G0, P_HMinus, P_GPlus, P_Hl, 5156)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Ha, P_GMinus, 9708)
      call createCurrent(P_Z, P_A, P_GPlus, P_HMinus, 9571)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_Ha, 3109)
      call createCurrent(P_Hl, P_Hh, P_Hl, P_Hl, 4342)
      call createCurrent(P_HQMinus, P_Hl, P_HPlus, P_HlQ, 3230)
      call createCurrent(P_WPlus, P_ghZ, P_ghWpAnti, P_A, 3005)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghWp, P_GMinus, 9732)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HPlus, P_HMinus, 2239)
      call createCurrent(P_ePlus, P_eMinus, P_Hl, 5859)
      call createCurrent(P_HQPlus, P_ghZ, P_ghWpAnti, 7153)
      call createCurrent(P_HhQ, P_GPlus, P_G0, P_HQMinus, 4819)
      call createCurrent(P_WPlus, P_Hh, P_WMinus, P_Hl, 9167)
      call createCurrent(P_ghWmAnti, P_Hh, P_HMinus, P_ghA, 65)
      call createCurrent(P_Hh, P_WPlus, P_GMinus, P_Z, 7796)
      call createCurrent(P_HMinus, P_Hh, P_WPlus, P_A, 1741)
      call createCurrent(P_A, P_WPlus, P_HMinus, P_G0, 9912)
      call createCurrent(P_HlQ, P_A, P_HPlus, P_WQMinus, 4053)
      call createCurrent(P_GPlus, P_G0, P_WMinus, 4045)
      call createCurrent(P_ghWp, P_Ha, P_G0, P_ghWpAnti, 9250)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hh, P_HhQ, 3179)
      call createCurrent(P_eMinus, P_nu_eAnti, P_HPlus, 8382)
      call createCurrent(P_HMinus, P_WPlus, P_Z, P_Hl, 2858)
      call createCurrent(P_WMinus, P_WPlus, P_ghZ, P_ghAAnti, 1918)
      call createCurrent(P_WPlus, P_ghAAnti, P_ghA, P_WMinus, 7430)
      call createCurrent(P_HQPlus, P_G0, P_GMinus, P_HhQ, 2050)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_HhQ, 2382)
      call createCurrent(P_GQ0, P_HPlus, P_HMinus, P_GQ0, 7926)
      call createCurrent(P_A, P_A, P_GMinus, P_HPlus, 10269)
      call createCurrent(P_Hl, P_Ha, P_Hl, P_Hl, 10703)
      call createCurrent(P_GQ0, P_tauPlus, P_tauMinus, 2882)
      call createCurrent(P_GMinus, P_GPlus, P_Hh, 796)
      call createCurrent(P_HhQ, P_ghZAnti, P_GPlus, P_ghWm, 8287)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_Hl, 1059)
      call createCurrent(P_ghWm, P_Hl, P_GPlus, P_ghZAnti, 4512)
      call createCurrent(P_GPlus, P_A, P_HMinus, P_A, 9828)
      call createCurrent(P_AQ, P_Hh, P_WMinus, P_GQPlus, 7243)
      call createCurrent(P_Hl, P_ghAAnti, P_ghWp, P_HMinus, 1910)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_HaQ, 3132)
      call createCurrent(P_HPlus, P_Ha, P_Ha, P_GMinus, 6247)
      call createCurrent(P_tAnti, P_A, P_t, 4732)
      call createCurrent(P_HQMinus, P_Hh, P_WQPlus, 4612)
      call createCurrent(P_GQPlus, P_Hl, P_Hl, P_GQMinus, 5807)
      call createCurrent(P_HlQ, P_ghAAnti, P_HMinus, P_ghWp, 6060)
      call createCurrent(P_ZQ, P_tauPlus, P_tauMinus, 652)
      call createCurrent(P_HMinus, P_ghZAnti, P_HPlus, P_ghZ, 7920)
      call createCurrent(P_HaQ, P_Hl, P_Hh, P_GQ0, 6635)
      call createCurrent(P_u, P_Z, P_uAnti, 8236)
      call createCurrent(P_Ha, P_G0, P_ghWpAnti, P_ghWp, 9730)
      call createCurrent(P_HQMinus, P_Ha, P_Ha, P_GQPlus, 5268)
      call createCurrent(P_WMinus, P_A, P_WPlus, P_A, 8820)
      call createCurrent(P_WMinus, P_G0, P_HPlus, P_A, 8952)
      call createCurrent(P_WPlus, P_ghWpAnti, P_A, P_ghA, 1178)
      call createCurrent(P_HQPlus, P_G0, P_Hh, P_GQMinus, 3655)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_G0, 4953)
      call createCurrent(P_GPlus, P_ghA, P_ghWpAnti, 5908)
      call createCurrent(P_ghWp, P_G0, P_ghWpAnti, P_GQ0, 3807)
      call createCurrent(P_GQPlus, P_G0, P_HMinus, P_GQ0, 3110)
      call createCurrent(P_WQPlus, P_HMinus, P_Hl, P_AQ, 2855)
      call createCurrent(P_GQPlus, P_Hh, P_WQMinus, 6178)
      call createCurrent(P_ghA, P_GMinus, P_ghZAnti, P_GPlus, 946)
      call createCurrent(P_G0, P_WPlus, P_GMinus, P_A, 6115)
      call createCurrent(P_GQPlus, P_A, P_A, P_GQMinus, 7026)
      call createCurrent(P_Z, P_dAnti, P_d, 835)
      call createCurrent(P_HMinus, P_Hh, P_Ha, P_HPlus, 9662)
      call createCurrent(P_g, P_Ha, P_g, P_G0, 10558)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghZ, P_GPlus, 10407)
      call createCurrent(P_WPlus, P_Z, P_ghA, P_ghWpAnti, 4515)
      call createCurrent(P_ghZAnti, P_ghWm, P_WQPlus, 2068)
      call createCurrent(P_Hh, P_ghA, P_ghWpAnti, P_GPlus, 8311)
      call createCurrent(P_HPlus, P_Hl, P_G0, P_HMinus, 10423)
      call createCurrent(P_G0, P_Hl, P_GMinus, P_HPlus, 6108)
      call createCurrent(P_GPlus, P_GMinus, P_ghWp, P_ghWpAnti, 1268)
      call createCurrent(P_HQMinus, P_Ha, P_WPlus, P_ZQ, 5982)
      call createCurrent(P_GMinus, P_GPlus, P_g, P_g, 10690)
      call createCurrent(P_HQPlus, P_Hh, P_ghZ, P_ghWpAnti, 1195)
      call createCurrent(P_GQPlus, P_Hl, P_HMinus, P_GQ0, 3917)
      call createCurrent(P_HlQ, P_uAnti, P_u, 3462)
      call createCurrent(P_ghZ, P_HPlus, P_HMinus, P_ghAAnti, 8054)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghA, P_A, 3153)
      call createCurrent(P_HQMinus, P_GPlus, P_G0, P_HhQ, 1463)
      call createCurrent(P_HlQ, P_HPlus, P_G0, P_GQMinus, 6213)
      call createCurrent(P_ghA, P_WMinus, P_WPlus, P_ghZAnti, 8023)
      call createCurrent(P_Z, P_g, P_Z, P_g, 10678)
      call createCurrent(P_Ha, P_ghWp, P_ghZAnti, P_HMinus, 4775)
      call createCurrent(P_eMinus, P_nu_eAnti, P_WPlus, 2275)
      call createCurrent(P_ghA, P_ghZAnti, P_WMinus, P_WQPlus, 7330)
      call createCurrent(P_nu_tau, P_nu_tauAnti, P_ZQ, 4186)
      call createCurrent(P_s, P_g, P_sAnti, 8706)
      call createCurrent(P_ghZAnti, P_HPlus, P_Ha, P_ghWm, 1685)
      call createCurrent(P_ghAAnti, P_GMinus, P_Ha, P_ghWp, 9223)
      call createCurrent(P_GPlus, P_HMinus, P_Hh, P_Ha, 4730)
      call createCurrent(P_Hh, P_GPlus, P_GMinus, P_Hl, 6875)
      call createCurrent(P_ghWm, P_ghAAnti, P_WPlus, P_ZQ, 416)
      call createCurrent(P_GQPlus, P_G0, P_GMinus, P_HaQ, 8471)
      call createCurrent(P_A, P_g, P_g, P_g, 10646)
      call createCurrent(P_ZQ, P_WMinus, P_Ha, P_HQPlus, 5092)
      call createCurrent(P_HaQ, P_G0, P_Hh, P_HhQ, 5288)
      call createCurrent(P_s, P_cAnti, P_GQPlus, 2843)
      call createCurrent(P_Hh, P_GPlus, P_GMinus, P_G0, 9314)
      call createCurrent(P_HaQ, P_Hh, P_G0, P_HlQ, 851)
      call createCurrent(P_G0, P_Hh, P_Hh, P_G0, 7058)
      call createCurrent(P_HQMinus, P_ePlus, P_nu_e, 2320)
      call createCurrent(P_ghAAnti, P_ghWp, P_WMinus, P_Z, 6127)
      call createCurrent(P_Hl, P_G0, P_Ha, P_G0, 10567)
      call createCurrent(P_HaQ, P_GMinus, P_Hl, P_HQPlus, 1721)
      call createCurrent(P_Hh, P_WPlus, P_WMinus, 5233)
      call createCurrent(P_GMinus, P_ghZ, P_ghWmAnti, P_G0, 367)
      call createCurrent(P_HPlus, P_Hl, P_Z, P_WMinus, 4375)
      call createCurrent(P_GQPlus, P_Hh, P_GMinus, P_HlQ, 1994)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_Hh, P_ghA, 7)
      call createCurrent(P_A, P_Hl, P_Ha, 10114)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hl, P_HaQ, 4028)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghA, P_GPlus, 9386)
      call createCurrent(P_WPlus, P_HMinus, P_Hh, P_Z, 1731)
      call createCurrent(P_ghWp, P_ghZAnti, P_WMinus, P_Z, 4657)
      call createCurrent(P_WMinus, P_Z, P_ghWmAnti, P_ghZ, 6805)
      call createCurrent(P_ghA, P_ghZAnti, P_HMinus, P_HQPlus, 258)
      call createCurrent(P_GPlus, P_Hh, P_WMinus, 3991)
      call createCurrent(P_ghA, P_Hh, P_HPlus, P_ghWpAnti, 1278)
      call createCurrent(P_HMinus, P_GPlus, P_Hl, P_Ha, 4950)
      call createCurrent(P_Z, P_Z, P_G0, P_Ha, 9759)
      call createCurrent(P_ghZ, P_ghAAnti, P_HMinus, P_HPlus, 5337)
      call createCurrent(P_ghAAnti, P_ghWm, P_Ha, P_GPlus, 10360)
      call createCurrent(P_WQPlus, P_WMinus, P_HlQ, 6761)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghWp, P_GMinus, 10228)
      call createCurrent(P_nu_eAnti, P_eMinus, P_WQPlus, 1218)
      call createCurrent(P_HQMinus, P_HPlus, P_ghAAnti, P_ghZ, 4682)
      call createCurrent(P_ghZAnti, P_Hl, P_HMinus, P_ghWp, 6073)
      call createCurrent(P_nu_tau, P_tauPlus, P_GQMinus, 470)
      call createCurrent(P_GMinus, P_GPlus, P_ghZAnti, P_ghA, 6576)
      call createCurrent(P_HlQ, P_GMinus, P_GPlus, P_HlQ, 2801)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghWm, P_Hl, 8060)
      call createCurrent(P_Hl, P_ghZAnti, P_ghZ, P_Hh, 10368)
      call createCurrent(P_ghWp, P_Z, P_WMinus, P_ghZAnti, 5579)
      call createCurrent(P_ghWmAnti, P_ghA, P_HMinus, P_G0, 9126)
      call createCurrent(P_GQMinus, P_Hh, P_HPlus, P_HhQ, 2278)
      call createCurrent(P_ZQ, P_GMinus, P_WPlus, P_HhQ, 2372)
      call createCurrent(P_Z, P_Z, P_Z, P_Z, 10641)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghA, P_GPlus, 5819)
      call createCurrent(P_GQMinus, P_ghWpAnti, P_GPlus, P_ghWp, 2051)
      call createCurrent(P_HlQ, P_Ha, P_GQ0, 2712)
      call createCurrent(P_GQPlus, P_GPlus, P_HMinus, P_HQMinus, 3415)
      call createCurrent(P_ghZ, P_Ha, P_HPlus, P_ghWpAnti, 3115)
      call createCurrent(P_t, P_g, P_tAnti, 8028)
      call createCurrent(P_AQ, P_WPlus, P_Hl, P_GQMinus, 7097)
      call createCurrent(P_Hl, P_Z, P_HMinus, P_WPlus, 2459)
      call createCurrent(P_Hl, P_ghAAnti, P_ghWm, P_HPlus, 2281)
      call createCurrent(P_ghWp, P_ghWpAnti, P_WMinus, P_WPlus, 3827)
      call createCurrent(P_GMinus, P_G0, P_ghWmAnti, P_ghZ, 552)
      call createCurrent(P_GMinus, P_G0, P_GPlus, 9274)
      call createCurrent(P_WQMinus, P_HPlus, P_Hh, P_ZQ, 1974)
      call createCurrent(P_ghA, P_WPlus, P_ghWpAnti, 7922)
      call createCurrent(P_G0, P_ghWpAnti, P_ghZ, P_HPlus, 9300)
      call createCurrent(P_G0, P_u, P_uAnti, 310)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hh, P_HhQ, 1684)
      call createCurrent(P_A, P_WMinus, P_HPlus, P_Ha, 4379)
      call createCurrent(P_ghWp, P_Ha, P_Ha, P_ghWpAnti, 7996)
      call createCurrent(P_GQPlus, P_ghZAnti, P_Hh, P_ghWm, 3604)
      call createCurrent(P_Z, P_nu_muAnti, P_nu_mu, 6248)
      call createCurrent(P_G0, P_G0, P_Ha, P_G0, 6034)
      call createCurrent(P_HlQ, P_Hh, P_HPlus, P_HQMinus, 5666)
      call createCurrent(P_HQMinus, P_HPlus, P_Hh, P_GQ0, 6119)
      call createCurrent(P_HhQ, P_HMinus, P_Hl, P_HQPlus, 1204)
      call createCurrent(P_GQ0, P_GMinus, P_WPlus, P_ZQ, 919)
      call createCurrent(P_GQMinus, P_GPlus, P_Z, P_ZQ, 8047)
      call createCurrent(P_ghWmAnti, P_ghWm, P_GMinus, P_GPlus, 4574)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_Ha, 7393)
      call createCurrent(P_ghA, P_G0, P_ghWmAnti, P_HMinus, 8966)
      call createCurrent(P_HPlus, P_HMinus, P_GMinus, P_GPlus, 1331)
      call createCurrent(P_ghZ, P_GPlus, P_ghAAnti, P_GQMinus, 5355)
      call createCurrent(P_HQMinus, P_Hh, P_HPlus, P_HlQ, 6032)
      call createCurrent(P_ghWp, P_Z, P_ghAAnti, P_WMinus, 1507)
      call createCurrent(P_Hh, P_A, P_Hl, P_A, 10732)
      call createCurrent(P_Hl, P_u, P_uAnti, 998)
      call createCurrent(P_WPlus, P_WMinus, P_g, P_g, 10635)
      call createCurrent(P_WQMinus, P_WPlus, P_Hh, P_HhQ, 5570)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_Hl, 3924)
      call createCurrent(P_WPlus, P_Ha, P_HMinus, 5102)
      call createCurrent(P_ghWp, P_HMinus, P_G0, P_ghAAnti, 9777)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hh, P_GQ0, 8615)
      call createCurrent(P_HhQ, P_Hh, P_ghWp, P_ghWpAnti, 5797)
      call createCurrent(P_HPlus, P_G0, P_HMinus, P_G0, 4895)
      call createCurrent(P_HMinus, P_Hl, P_Hl, P_GPlus, 3542)
      call createCurrent(P_HPlus, P_Hh, P_G0, P_HMinus, 9104)
      call createCurrent(P_HaQ, P_HMinus, P_WQPlus, 1471)
      call createCurrent(P_WMinus, P_A, P_ghA, P_ghWmAnti, 8322)
      call createCurrent(P_HMinus, P_WMinus, P_HPlus, P_WPlus, 1596)
      call createCurrent(P_g, P_u, P_uAnti, 4637)
      call createCurrent(P_Hl, P_HPlus, P_GMinus, P_G0, 2226)
      call createCurrent(P_AQ, P_GPlus, P_GMinus, P_ZQ, 8376)
      call createCurrent(P_A, P_Z, P_GMinus, P_HPlus, 10213)
      call createCurrent(P_HPlus, P_WMinus, P_G0, P_Z, 9880)
      call createCurrent(P_ghZ, P_Hl, P_ghZAnti, P_Hl, 6324)
      call createCurrent(P_nu_muAnti, P_GPlus, P_muMinus, 8502)
      call createCurrent(P_WQPlus, P_Hl, P_GMinus, P_ZQ, 3000)
      call createCurrent(P_ghZ, P_WPlus, P_Z, P_ghWpAnti, 5725)
      call createCurrent(P_ghZAnti, P_ghA, P_HMinus, P_HQPlus, 7666)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghWp, P_HMinus, 171)
      call createCurrent(P_ghZ, P_GMinus, P_Hh, P_ghWmAnti, 1931)
      call createCurrent(P_HMinus, P_GPlus, P_Hl, P_G0, 5887)
      call createCurrent(P_HhQ, P_HMinus, P_GPlus, P_HhQ, 1043)
      call createCurrent(P_G0, P_GMinus, P_GPlus, P_G0, 4582)
      call createCurrent(P_A, P_GMinus, P_A, P_GPlus, 6260)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghZ, P_GMinus, 8949)
      call createCurrent(P_Ha, P_Hh, P_Ha, P_Hh, 7182)
      call createCurrent(P_HQMinus, P_G0, P_Ha, P_HQPlus, 7910)
      call createCurrent(P_ghA, P_ghWmAnti, P_GMinus, P_Ha, 9405)
      call createCurrent(P_eMinus, P_ePlus, P_Z, 7087)
      call createCurrent(P_HQPlus, P_Ha, P_GMinus, P_GQ0, 805)
      call createCurrent(P_G0, P_Z, P_Ha, P_A, 10679)
      call createCurrent(P_HPlus, P_WMinus, P_Hh, P_A, 8173)
      call createCurrent(P_t, P_tAnti, P_Hh, 1789)
      call createCurrent(P_AQ, P_WMinus, P_ghAAnti, P_ghWp, 6652)
      call createCurrent(P_HPlus, P_GMinus, P_ghZAnti, P_ghA, 10241)
      call createCurrent(P_ghWmAnti, P_ghA, P_A, P_WQMinus, 6343)
      call createCurrent(P_HPlus, P_WPlus, P_GMinus, P_WMinus, 8882)
      call createCurrent(P_ZQ, P_WPlus, P_Hl, P_GQMinus, 4979)
      call createCurrent(P_Hh, P_HPlus, P_WMinus, P_Z, 5574)
      call createCurrent(P_g, P_g, P_Z, P_A, 10684)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_G0, 1460)
      call createCurrent(P_WPlus, P_ghAAnti, P_ghWm, 3937)
      call createCurrent(P_GPlus, P_ghZAnti, P_Hh, P_ghWm, 776)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghWm, 4315)
      call createCurrent(P_HQMinus, P_Hl, P_ghZAnti, P_ghWp, 5151)
      call createCurrent(P_WQMinus, P_A, P_ghWmAnti, P_ghA, 7716)
      call createCurrent(P_GMinus, P_G0, P_G0, P_GPlus, 7901)
      call createCurrent(P_G0, P_G0, P_GPlus, P_HMinus, 6059)
      call createCurrent(P_HPlus, P_Hl, P_HMinus, P_G0, 9770)
      call createCurrent(P_ghWp, P_HPlus, P_HMinus, P_ghWpAnti, 1008)
      call createCurrent(P_WQMinus, P_GPlus, P_A, P_HhQ, 1034)
      call createCurrent(P_Hh, P_Hl, P_HMinus, P_GPlus, 1061)
      call createCurrent(P_HPlus, P_Z, P_GMinus, P_Z, 10150)
      call createCurrent(P_Hh, P_Hl, P_GPlus, P_HMinus, 2519)
      call createCurrent(P_ghZAnti, P_HPlus, P_GMinus, P_ghA, 9619)
      call createCurrent(P_WQPlus, P_GPlus, P_GMinus, P_WQMinus, 6721)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghA, P_GQPlus, 3784)
      call createCurrent(P_HPlus, P_Ha, P_ghWm, P_ghAAnti, 5397)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghA, P_HMinus, 6080)
      call createCurrent(P_ZQ, P_A, P_HMinus, P_HQPlus, 2538)
      call createCurrent(P_HaQ, P_WMinus, P_HQPlus, 4747)
      call createCurrent(P_WPlus, P_ghWp, P_WMinus, P_ghWpAnti, 8040)
      call createCurrent(P_Z, P_ghWp, P_ghWpAnti, 2537)
      call createCurrent(P_Ha, P_HMinus, P_Ha, P_GPlus, 5952)
      call createCurrent(P_GQ0, P_Hl, P_HMinus, P_HQPlus, 1604)
      call createCurrent(P_d, P_dAnti, P_Z, 5258)
      call createCurrent(P_WQMinus, P_HPlus, P_HhQ, 8161)
      call createCurrent(P_ZQ, P_Hh, P_Hh, P_ZQ, 271)
      call createCurrent(P_WPlus, P_A, P_ghWm, P_ghZAnti, 5823)
      call createCurrent(P_ghWpAnti, P_ghA, P_G0, P_GPlus, 6828)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghA, P_Z, 5172)
      call createCurrent(P_G0, P_HPlus, P_ghAAnti, P_ghWm, 10037)
      call createCurrent(P_Hh, P_d, P_dAnti, 5997)
      call createCurrent(P_Ha, P_ghZAnti, P_ghWp, P_HMinus, 7348)
      call createCurrent(P_HPlus, P_WPlus, P_WMinus, P_HMinus, 3815)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_Hh, 2509)
      call createCurrent(P_AQ, P_A, P_ghWpAnti, P_ghWp, 4538)
      call createCurrent(P_eMinus, P_Ha, P_ePlus, 8449)
      call createCurrent(P_A, P_Z, P_A, P_A, 10673)
      call createCurrent(P_AQ, P_WPlus, P_A, P_WQMinus, 8636)
      call createCurrent(P_WMinus, P_ghA, P_ghWmAnti, P_Z, 4502)
      call createCurrent(P_ZQ, P_G0, P_HhQ, 1168)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_Hh, 939)
      call createCurrent(P_GQ0, P_HMinus, P_Hl, P_HQPlus, 6227)
      call createCurrent(P_HaQ, P_HMinus, P_Hl, P_GQPlus, 1524)
      call createCurrent(P_HlQ, P_WPlus, P_HMinus, P_ZQ, 581)
      call createCurrent(P_Z, P_ghWpAnti, P_ghWp, 1819)
      call createCurrent(P_ghA, P_HPlus, P_GMinus, P_ghAAnti, 10084)
      call createCurrent(P_ghWp, P_WMinus, P_Z, P_ghZAnti, 7273)
      call createCurrent(P_GPlus, P_G0, P_ghAAnti, P_ghWm, 3108)
      call createCurrent(P_Hh, P_Z, P_Hl, P_Z, 8894)
      call createCurrent(P_ghZAnti, P_GPlus, P_GMinus, P_ghA, 8230)
      call createCurrent(P_Hh, P_Hh, P_Hh, P_Hl, 1240)
      call createCurrent(P_ghZ, P_HPlus, P_ghAAnti, P_HQMinus, 2913)
      call createCurrent(P_A, P_GMinus, P_Z, P_HPlus, 9092)
      call createCurrent(P_ghWm, P_Hl, P_GPlus, P_ghAAnti, 8237)
      call createCurrent(P_Hl, P_g, P_Hl, P_g, 10754)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_GQ0, 8693)
      call createCurrent(P_ghWp, P_Ha, P_HMinus, P_ghAAnti, 4929)
      call createCurrent(P_HQMinus, P_ghZ, P_HPlus, P_ghZAnti, 2391)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hl, P_GMinus, 914)
      call createCurrent(P_HhQ, P_WMinus, P_HPlus, P_ZQ, 5678)
      call createCurrent(P_ghZAnti, P_ghWp, P_G0, P_GQMinus, 2981)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, 8022)
      call createCurrent(P_ghWp, P_A, P_ghWpAnti, P_AQ, 935)
      call createCurrent(P_GMinus, P_G0, P_Hl, P_HPlus, 4344)
      call createCurrent(P_Ha, P_WPlus, P_Z, P_HMinus, 633)
      call createCurrent(P_HPlus, P_g, P_GMinus, P_g, 10488)
      call createCurrent(P_Hl, P_Hl, P_ghWmAnti, P_ghWm, 354)
      call createCurrent(P_HhQ, P_WPlus, P_Z, P_GQMinus, 4127)
      call createCurrent(P_GPlus, P_GMinus, P_HMinus, P_HPlus, 5138)
      call createCurrent(P_GQMinus, P_tauPlus, P_nu_tau, 5361)
      call createCurrent(P_HhQ, P_A, P_WPlus, P_GQMinus, 7825)
      call createCurrent(P_Ha, P_Hl, P_A, 9629)
      call createCurrent(P_HlQ, P_Hl, P_ghZAnti, P_ghZ, 2644)
      call createCurrent(P_HlQ, P_G0, P_WMinus, P_WQPlus, 5710)
      call createCurrent(P_HPlus, P_Ha, P_GMinus, P_G0, 1265)
      call createCurrent(P_ZQ, P_Hh, P_HaQ, 6667)
      call createCurrent(P_HPlus, P_WPlus, P_HMinus, P_WMinus, 465)
      call createCurrent(P_WMinus, P_Hl, P_HPlus, 2896)
      call createCurrent(P_WMinus, P_WPlus, P_Ha, P_Ha, 7176)
      call createCurrent(P_ghZ, P_GMinus, P_ghZAnti, P_GPlus, 4108)
      call createCurrent(P_ghA, P_ghWpAnti, P_HPlus, P_Hh, 1293)
      call createCurrent(P_HMinus, P_Hl, P_ghWp, P_ghZAnti, 7163)
      call createCurrent(P_HaQ, P_ghWp, P_Hh, P_ghWpAnti, 6845)
      call createCurrent(P_A, P_A, P_A, P_A, 10530)
      call createCurrent(P_Hh, P_WMinus, P_Hl, P_WPlus, 8880)
      call createCurrent(P_HPlus, P_GMinus, P_Ha, P_Hl, 3159)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hl, P_HMinus, 5548)
      call createCurrent(P_WMinus, P_G0, P_GPlus, 7043)
      call createCurrent(P_ghZAnti, P_G0, P_ghWp, P_HMinus, 9658)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_Ha, 8825)
      call createCurrent(P_t, P_tAnti, P_Ha, 5614)
      call createCurrent(P_HPlus, P_WMinus, P_G0, P_A, 9601)
      call createCurrent(P_gQ, P_b, P_bAnti, 4759)
      call createCurrent(P_GQPlus, P_G0, P_Hh, P_HQMinus, 3374)
      call createCurrent(P_ghA, P_Ha, P_GMinus, P_ghWmAnti, 10039)
      call createCurrent(P_Hl, P_Hh, P_Hh, 697)
      call createCurrent(P_A, P_GPlus, P_GMinus, 7381)
      call createCurrent(P_ghWmAnti, P_G0, P_ghWm, P_HhQ, 41)
      call createCurrent(P_GQMinus, P_GPlus, P_GMinus, P_HQPlus, 4621)
      call createCurrent(P_HPlus, P_Z, P_WMinus, P_G0, 9577)
      call createCurrent(P_u, P_dAnti, P_HQMinus, 8153)
      call createCurrent(P_ghA, P_HPlus, P_HMinus, P_ghAAnti, 2557)
      call createCurrent(P_HMinus, P_ghWmAnti, P_Ha, P_ghZ, 5212)
      call createCurrent(P_A, P_A, P_A, P_Z, 10728)
      call createCurrent(P_HMinus, P_Hl, P_ghWp, P_ghAAnti, 7777)
      call createCurrent(P_ghWmAnti, P_Hh, P_GMinus, P_ghA, 2328)
      call createCurrent(P_Hl, P_HMinus, P_Hh, P_GPlus, 670)
      call createCurrent(P_u, P_dAnti, P_HMinus, 3130)
      call createCurrent(P_WPlus, P_WMinus, P_ghWpAnti, P_ghWp, 3321)
      call createCurrent(P_Hl, P_Hh, P_G0, P_G0, 1602)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_HlQ, 7197)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghWp, P_Hl, 6693)
      call createCurrent(P_Hh, P_g, P_Hl, P_g, 10568)
      call createCurrent(P_HMinus, P_A, P_HPlus, P_Z, 3706)
      call createCurrent(P_WPlus, P_WPlus, P_ghWm, P_ghWpAnti, 2996)
      call createCurrent(P_GMinus, P_GMinus, P_HPlus, P_GPlus, 8378)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghWp, P_HaQ, 3745)
      call createCurrent(P_HhQ, P_Hh, P_Ha, P_HaQ, 8446)
      call createCurrent(P_ghAAnti, P_GPlus, P_Hh, P_ghWm, 8519)
      call createCurrent(P_HMinus, P_G0, P_WPlus, P_Z, 8891)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghA, P_GMinus, 8154)
      call createCurrent(P_cAnti, P_c, P_Ha, 3586)
      call createCurrent(P_GQMinus, P_GPlus, P_ghZ, P_ghAAnti, 4557)
      call createCurrent(P_HPlus, P_ghZ, P_HMinus, P_ghAAnti, 4643)
      call createCurrent(P_HQPlus, P_WMinus, P_Hl, P_ZQ, 8817)
      call createCurrent(P_Ha, P_Hh, P_G0, 3164)
      call createCurrent(P_HlQ, P_tAnti, P_t, 655)
      call createCurrent(P_HQPlus, P_Hl, P_G0, P_GQMinus, 986)
      call createCurrent(P_HMinus, P_WPlus, P_G0, P_Z, 10462)
      call createCurrent(P_GQ0, P_ghWpAnti, P_Hl, P_ghWp, 984)
      call createCurrent(P_Hl, P_HMinus, P_ghWp, P_ghZAnti, 6569)
      call createCurrent(P_GQPlus, P_Hl, P_ghA, P_ghWpAnti, 2066)
      call createCurrent(P_HlQ, P_sAnti, P_s, 7954)
      call createCurrent(P_Hh, P_ghZAnti, P_ghWp, P_GMinus, 749)
      call createCurrent(P_WMinus, P_A, P_Ha, P_HPlus, 1521)
      call createCurrent(P_nu_mu, P_GMinus, P_muPlus, 3373)
      call createCurrent(P_nu_eAnti, P_eMinus, P_WPlus, 6147)
      call createCurrent(P_ghWp, P_ghAAnti, P_WMinus, 6474)
      call createCurrent(P_GMinus, P_G0, P_GPlus, P_Ha, 7874)
      call createCurrent(P_Hh, P_bAnti, P_b, 6456)
      call createCurrent(P_ghA, P_Hh, P_HMinus, P_ghWmAnti, 2009)
      call createCurrent(P_ghWp, P_G0, P_GMinus, P_ghAAnti, 8809)
      call createCurrent(P_HhQ, P_Hl, P_GMinus, P_HQPlus, 3404)
      call createCurrent(P_HhQ, P_GPlus, P_WMinus, P_ZQ, 2674)
      call createCurrent(P_HPlus, P_GMinus, P_WMinus, P_WPlus, 9785)
      call createCurrent(P_GQPlus, P_Hh, P_HQMinus, 8480)
      call createCurrent(P_Z, P_Hl, P_Z, P_Hl, 3254)
      call createCurrent(P_Ha, P_HMinus, P_WPlus, 1088)
      call createCurrent(P_A, P_A, P_Z, P_A, 10578)
      call createCurrent(P_GQMinus, P_Ha, P_G0, P_GQPlus, 4985)
      call createCurrent(P_g, P_WMinus, P_WPlus, P_g, 10574)
      call createCurrent(P_Hl, P_GMinus, P_WPlus, P_Z, 6507)
      call createCurrent(P_HQMinus, P_Ha, P_Ha, P_HQPlus, 178)
      call createCurrent(P_Hh, P_HMinus, P_HPlus, 4888)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghZ, P_A, 462)
      call createCurrent(P_ghZAnti, P_ghA, P_GMinus, P_GPlus, 3058)
      call createCurrent(P_ghWpAnti, P_ghA, P_WQPlus, 6608)
      call createCurrent(P_Hl, P_Hl, P_Hl, P_Hl, 4266)
      call createCurrent(P_ghWpAnti, P_ghA, P_HPlus, P_Hh, 5086)
      call createCurrent(P_GPlus, P_Hh, P_GMinus, P_Hl, 4477)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghZ, P_HPlus, 543)
      call createCurrent(P_G0, P_Ha, P_HPlus, P_GMinus, 1886)
      call createCurrent(P_HhQ, P_GPlus, P_ghZAnti, P_ghWm, 2353)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghZ, P_A, 1135)
      call createCurrent(P_Hl, P_ghZ, P_ghZAnti, 2437)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, 1264)
      call createCurrent(P_AQ, P_GMinus, P_Hh, P_WQPlus, 7335)
      call createCurrent(P_nu_tauAnti, P_WPlus, P_tauMinus, 8768)
      call createCurrent(P_Z, P_ghA, P_ghWpAnti, P_WPlus, 3794)
      call createCurrent(P_eMinus, P_ePlus, P_HaQ, 3048)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_Hl, P_ghA, 1219)
      call createCurrent(P_GMinus, P_ghA, P_ghWmAnti, P_Ha, 9818)
      call createCurrent(P_AQ, P_WPlus, P_WQMinus, 4576)
      call createCurrent(P_WQMinus, P_WPlus, P_ghZAnti, P_ghZ, 1067)
      call createCurrent(P_GQMinus, P_Hl, P_Ha, P_HQPlus, 2583)
      call createCurrent(P_G0, P_GMinus, P_HPlus, P_G0, 434)
      call createCurrent(P_s, P_cAnti, P_GPlus, 8760)
      call createCurrent(P_cAnti, P_c, P_A, 7365)
      call createCurrent(P_WMinus, P_Hl, P_A, P_GPlus, 2536)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_G0, 10318)
      call createCurrent(P_G0, P_Ha, P_ghWpAnti, P_ghWp, 10202)
      call createCurrent(P_HQPlus, P_Hl, P_GMinus, P_HhQ, 2671)
      call createCurrent(P_GMinus, P_HPlus, P_WPlus, P_WMinus, 9452)
      call createCurrent(P_ghWpAnti, P_ghA, P_GQPlus, 373)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_Hl, 2223)
      call createCurrent(P_Hl, P_Hl, P_A, P_A, 10665)
      call createCurrent(P_GQMinus, P_muPlus, P_nu_mu, 4098)
      call createCurrent(P_ghWpAnti, P_A, P_ghWp, P_Z, 3766)
      call createCurrent(P_Ha, P_WPlus, P_A, P_GMinus, 10095)
      call createCurrent(P_HhQ, P_WPlus, P_HMinus, P_ZQ, 2421)
      call createCurrent(P_WPlus, P_HMinus, P_Hh, P_A, 1905)
      call createCurrent(P_Hh, P_WPlus, P_HMinus, P_Z, 1369)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_G0, 9878)
      call createCurrent(P_HlQ, P_ghWpAnti, P_ghWp, 5567)
      call createCurrent(P_ghWp, P_HMinus, P_Hl, P_ghAAnti, 2202)
      call createCurrent(P_HPlus, P_HMinus, P_Hh, P_Ha, 10046)
      call createCurrent(P_ZQ, P_HMinus, P_Z, P_HQPlus, 5922)
      call createCurrent(P_WMinus, P_Ha, P_Ha, P_WPlus, 4786)
      call createCurrent(P_GMinus, P_Ha, P_HPlus, P_Ha, 7231)
      call createCurrent(P_WQPlus, P_Z, P_ghWpAnti, P_ghZ, 6047)
      call createCurrent(P_Hh, P_HMinus, P_Hl, P_GPlus, 4005)
      call createCurrent(P_t, P_tAnti, 8990)
      call createCurrent(P_G0, P_Ha, P_GMinus, P_GPlus, 5082)
      call createCurrent(P_ghWpAnti, P_HMinus, P_HPlus, P_ghWp, 2104)
      call createCurrent(P_eMinus, P_ePlus, P_AQ, 5134)
      call createCurrent(P_WQPlus, P_HMinus, P_HPlus, P_WQMinus, 8467)
      call createCurrent(P_HMinus, P_ghWpAnti, P_ghWp, P_GPlus, 10324)
      call createCurrent(P_Ha, P_ghZAnti, P_GMinus, P_ghWp, 9512)
      call createCurrent(P_Z, P_G0, P_Hh, 4262)
      call createCurrent(P_WQMinus, P_ghWp, P_WMinus, P_ghWmAnti, 4845)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghWp, 1616)
      call createCurrent(P_GQPlus, P_HMinus, P_HhQ, 2505)
      call createCurrent(P_Ha, P_ghWm, P_GPlus, P_ghAAnti, 9436)
      call createCurrent(P_GMinus, P_Ha, P_ghZ, P_ghWmAnti, 9222)
      call createCurrent(P_WMinus, P_A, P_ghWmAnti, P_ghA, 724)
      call createCurrent(P_WQMinus, P_Hh, P_GQPlus, 5806)
      call createCurrent(P_ghZAnti, P_ghZ, P_GPlus, P_HMinus, 9903)
      call createCurrent(P_WQMinus, P_ghWp, P_ghZAnti, 2433)
      call createCurrent(P_WQPlus, P_Z, P_Ha, P_HQMinus, 3552)
      call createCurrent(P_HPlus, P_GMinus, P_G0, P_Hl, 4771)
      call createCurrent(P_HlQ, P_GMinus, P_ghZAnti, P_ghWp, 8672)
      call createCurrent(P_HhQ, P_GPlus, P_ghAAnti, P_ghWm, 4810)
      call createCurrent(P_WMinus, P_HPlus, P_A, P_Hh, 972)
      call createCurrent(P_WQMinus, P_WPlus, P_Ha, P_HhQ, 5419)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_Hl, 969)
      call createCurrent(P_uAnti, P_u, P_A, 8326)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_Hh, 7144)
      call createCurrent(P_ghWp, P_Hl, P_ghZAnti, P_HQMinus, 8074)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghWm, 4700)
      call createCurrent(P_ghAAnti, P_ghWp, P_Ha, P_GMinus, 9843)
      call createCurrent(P_GQMinus, P_WPlus, P_A, P_HlQ, 6692)
      call createCurrent(P_WMinus, P_ghWp, P_ghWmAnti, P_WMinus, 6312)
      call createCurrent(P_ghZAnti, P_ghWp, P_WMinus, 5718)
      call createCurrent(P_GQMinus, P_Hh, P_ghZ, P_ghWmAnti, 8190)
      call createCurrent(P_WQMinus, P_ghZAnti, P_WPlus, P_ghZ, 6371)
      call createCurrent(P_G0, P_Ha, P_Hh, P_Hl, 3981)
      call createCurrent(P_HQMinus, P_Ha, P_ghA, P_ghWmAnti, 847)
      call createCurrent(P_HlQ, P_GPlus, P_GQMinus, 2247)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_Hh, 371)
      call createCurrent(P_ghWp, P_Ha, P_HMinus, P_ghZAnti, 8056)
      call createCurrent(P_tauPlus, P_HMinus, P_nu_tau, 5954)
      call createCurrent(P_ghZ, P_HMinus, P_Ha, P_ghWmAnti, 3322)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hh, P_GPlus, 7181)
      call createCurrent(P_GMinus, P_ghWp, P_ghZAnti, P_G0, 152)
      call createCurrent(P_Z, P_WMinus, P_HPlus, P_Hh, 8111)
      call createCurrent(P_Hl, P_GPlus, P_ghZAnti, P_ghWm, 5582)
      call createCurrent(P_ghA, P_HMinus, P_ghAAnti, P_HPlus, 542)
      call createCurrent(P_GQMinus, P_G0, P_Hh, P_HQPlus, 5149)
      call createCurrent(P_GQMinus, P_Z, P_WPlus, P_GQ0, 1123)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hl, P_HQMinus, 799)
      call createCurrent(P_GQ0, P_Z, P_HhQ, 5059)
      call createCurrent(P_GMinus, P_HPlus, P_GPlus, P_GMinus, 6088)
      call createCurrent(P_Ha, P_ghWp, P_HMinus, P_ghAAnti, 1399)
      call createCurrent(P_HMinus, P_A, P_A, P_HPlus, 3500)
      call createCurrent(P_ghWmAnti, P_Ha, P_HMinus, P_ghA, 5972)
      call createCurrent(P_GQ0, P_tAnti, P_t, 8181)
      call createCurrent(P_ghA, P_Z, P_ghWmAnti, P_WQMinus, 275)
      call createCurrent(P_ghAAnti, P_A, P_ghWp, P_WMinus, 3113)
      call createCurrent(P_ghWp, P_WMinus, P_ghZAnti, P_Z, 864)
      call createCurrent(P_WQMinus, P_A, P_ghA, P_ghWmAnti, 2950)
      call createCurrent(P_WQMinus, P_ghZ, P_Z, P_ghWmAnti, 8401)
      call createCurrent(P_GQ0, P_G0, P_ghZ, P_ghZAnti, 7076)
      call createCurrent(P_HQPlus, P_HMinus, P_ghAAnti, P_ghZ, 2473)
      call createCurrent(P_GQPlus, P_Hl, P_ghZ, P_ghWpAnti, 5600)
      call createCurrent(P_G0, P_Ha, P_G0, 10593)
      call createCurrent(P_sAnti, P_Z, P_s, 934)
      call createCurrent(P_g, P_g, P_WMinus, P_WPlus, 10701)
      call createCurrent(P_Hh, P_GMinus, P_GPlus, 2030)
      call createCurrent(P_GQPlus, P_ghWm, P_Hh, P_ghZAnti, 4990)
      call createCurrent(P_G0, P_Ha, P_Hh, 324)
      call createCurrent(P_Z, P_muMinus, P_muPlus, 7384)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghWm, P_HaQ, 4773)
      call createCurrent(P_ghA, P_ghWpAnti, P_GQPlus, 976)
      call createCurrent(P_HQMinus, P_ghA, P_Hl, P_ghWmAnti, 2356)
      call createCurrent(P_b, P_tAnti, P_GPlus, 7841)
      call createCurrent(P_HMinus, P_HMinus, P_GPlus, P_GPlus, 8300)
      call createCurrent(P_GMinus, P_WPlus, P_G0, P_A, 713)
      call createCurrent(P_GQMinus, P_HPlus, P_G0, P_GQ0, 3509)
      call createCurrent(P_WMinus, P_G0, P_A, P_HPlus, 10159)
      call createCurrent(P_A, P_Hl, P_Z, P_Hh, 10695)
      call createCurrent(P_ghZ, P_GMinus, P_ghAAnti, P_GPlus, 6664)
      call createCurrent(P_GQMinus, P_ghZAnti, P_G0, P_ghWp, 687)
      call createCurrent(P_Hl, P_ghA, P_ghWmAnti, P_GMinus, 3772)
      call createCurrent(P_HaQ, P_GMinus, P_G0, P_GQPlus, 5921)
      call createCurrent(P_Ha, P_WMinus, P_Z, P_HPlus, 6574)
      call createCurrent(P_Ha, P_ghWp, P_HMinus, P_ghZAnti, 6512)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghA, P_GPlus, 4036)
      call createCurrent(P_HlQ, P_ghWp, P_Ha, P_ghWpAnti, 5329)
      call createCurrent(P_HlQ, P_tauMinus, P_tauPlus, 7986)
      call createCurrent(P_GMinus, P_WPlus, P_Z, 6063)
      call createCurrent(P_WQPlus, P_A, P_Z, P_WQMinus, 3228)
      call createCurrent(P_muMinus, P_nu_muAnti, P_WQPlus, 3748)
      call createCurrent(P_ghZ, P_G0, P_ghZAnti, P_G0, 7164)
      call createCurrent(P_A, P_ghZAnti, P_ghWp, P_WMinus, 195)
      call createCurrent(P_WQMinus, P_Z, P_Ha, P_HQPlus, 3211)
      call createCurrent(P_HPlus, P_ghA, P_ghZAnti, P_GMinus, 9664)
      call createCurrent(P_WMinus, P_ghA, P_A, P_ghWmAnti, 4471)
      call createCurrent(P_GPlus, P_ghWpAnti, P_Ha, P_ghA, 10287)
      call createCurrent(P_WMinus, P_GPlus, P_Hl, 8669)
      call createCurrent(P_HMinus, P_HPlus, P_HPlus, P_HMinus, 7516)
      call createCurrent(P_GQMinus, P_Z, P_GPlus, P_AQ, 3515)
      call createCurrent(P_GQMinus, P_G0, P_WQPlus, 5409)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghWm, P_HPlus, 8404)
      call createCurrent(P_ghWm, P_G0, P_GPlus, P_ghAAnti, 4356)
      call createCurrent(P_GQ0, P_ghWp, P_Hh, P_ghWpAnti, 7193)
      call createCurrent(P_WMinus, P_Z, P_G0, P_HPlus, 8945)
      call createCurrent(P_GQMinus, P_Hl, P_ghWp, P_ghZAnti, 7805)
      call createCurrent(P_GMinus, P_ghWp, P_ghZAnti, P_Hl, 3090)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_G0, 10168)
      call createCurrent(P_u, P_uAnti, P_HlQ, 6737)
      call createCurrent(P_ghZ, P_GMinus, P_GPlus, P_ghAAnti, 5850)
      call createCurrent(P_ghWpAnti, P_GMinus, P_ghWp, P_HPlus, 9881)
      call createCurrent(P_ghA, P_HPlus, P_HMinus, P_ghZAnti, 6219)
      call createCurrent(P_ghZAnti, P_HMinus, P_Hl, P_ghWp, 6318)
      call createCurrent(P_WQMinus, P_sAnti, P_c, 5698)
      call createCurrent(P_ghZAnti, P_Ha, P_ghWm, P_HQPlus, 172)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghZ, P_WQMinus, 947)
      call createCurrent(P_HQMinus, P_Hh, P_HPlus, P_GQ0, 7061)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghWm, P_WPlus, 6974)
      call createCurrent(P_GQMinus, P_ghWp, P_ghZAnti, 5300)
      call createCurrent(P_ghZ, P_HPlus, P_ghZAnti, P_HQMinus, 2737)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hl, P_GQMinus, 1296)
      call createCurrent(P_HhQ, P_GPlus, P_HMinus, P_HlQ, 6998)
      call createCurrent(P_Z, P_A, P_GMinus, P_HPlus, 9663)
      call createCurrent(P_WPlus, P_Hh, P_Z, P_GMinus, 1752)
      call createCurrent(P_HaQ, P_HPlus, P_Hh, P_GQMinus, 5265)
      call createCurrent(P_GQ0, P_Z, P_GMinus, P_WQPlus, 5126)
      call createCurrent(P_GQ0, P_A, P_WPlus, P_GQMinus, 5438)
      call createCurrent(P_A, P_Hh, P_WMinus, P_GPlus, 8832)
      call createCurrent(P_ghZ, P_Ha, P_GPlus, P_ghWpAnti, 9811)
      call createCurrent(P_Ha, P_HPlus, P_HMinus, P_Ha, 822)
      call createCurrent(P_WMinus, P_bAnti, P_t, 183)
      call createCurrent(P_ghAAnti, P_A, P_WMinus, P_ghWp, 4871)
      call createCurrent(P_Hh, P_WPlus, P_A, P_HMinus, 8299)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_Hl, 6054)
      call createCurrent(P_HlQ, P_WPlus, P_WQMinus, 8666)
      call createCurrent(P_G0, P_Z, P_A, P_Ha, 10638)
      call createCurrent(P_G0, P_ghA, P_ghWpAnti, P_HPlus, 10000)
      call createCurrent(P_AQ, P_WMinus, P_Z, P_WQPlus, 4928)
      call createCurrent(P_HQPlus, P_Ha, P_ghZ, P_ghWpAnti, 8457)
      call createCurrent(P_ZQ, P_HMinus, P_WPlus, P_HhQ, 4015)
      call createCurrent(P_GQMinus, P_HPlus, P_GQ0, 3506)
      call createCurrent(P_Z, P_HMinus, P_Hh, P_WPlus, 2897)
      call createCurrent(P_GQ0, P_ghAAnti, P_GMinus, P_ghWp, 2190)
      call createCurrent(P_AQ, P_HMinus, P_Hh, P_WQPlus, 6282)
      call createCurrent(P_Z, P_HMinus, P_Hl, P_WPlus, 4712)
      call createCurrent(P_WQMinus, P_HPlus, P_Ha, P_ZQ, 3602)
      call createCurrent(P_HhQ, P_Hh, P_HhQ, 2768)
      call createCurrent(P_ghA, P_ghWmAnti, P_Z, P_WMinus, 4235)
      call createCurrent(P_GMinus, P_ghA, P_G0, P_ghWmAnti, 2070)
      call createCurrent(P_HMinus, P_ghA, P_ghZAnti, P_GPlus, 9905)
      call createCurrent(P_G0, P_WPlus, P_A, P_HMinus, 9459)
      call createCurrent(P_WQMinus, P_A, P_ghZ, P_ghWmAnti, 4191)
      call createCurrent(P_Hh, P_HPlus, P_HMinus, 2196)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghWm, P_GPlus, 10205)
      call createCurrent(P_Ha, P_GPlus, P_A, P_WMinus, 9977)
      call createCurrent(P_HMinus, P_G0, P_GPlus, P_G0, 8707)
      call createCurrent(P_ghWpAnti, P_ghA, P_GPlus, 4128)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWm, P_GQPlus, 8368)
      call createCurrent(P_ghA, P_Hl, P_ghWmAnti, P_GQMinus, 5603)
      call createCurrent(P_nu_eAnti, P_GPlus, P_eMinus, 1813)
      call createCurrent(P_GQ0, P_GMinus, P_ghWmAnti, P_ghZ, 1505)
      call createCurrent(P_G0, P_Hh, P_GMinus, P_HPlus, 2705)
      call createCurrent(P_HMinus, P_HPlus, P_A, P_Z, 4166)
      call createCurrent(P_Z, P_WMinus, P_WPlus, P_A, 5488)
      call createCurrent(P_GPlus, P_ghWm, P_Hl, P_ghZAnti, 2486)
      call createCurrent(P_HaQ, P_ghWmAnti, P_Ha, P_ghWm, 8142)
      call createCurrent(P_GPlus, P_G0, P_Hh, P_GMinus, 10209)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HPlus, P_HQMinus, 232)
      call createCurrent(P_Hh, P_Hh, P_Ha, P_Ha, 6942)
      call createCurrent(P_GQMinus, P_Hh, P_GPlus, P_HlQ, 1052)
      call createCurrent(P_GQPlus, P_WMinus, P_Hl, P_ZQ, 1395)
      call createCurrent(P_HhQ, P_WMinus, P_G0, P_WQPlus, 4930)
      call createCurrent(P_GQMinus, P_WPlus, P_HlQ, 4071)
      call createCurrent(P_g, P_bAnti, P_b, 4384)
      call createCurrent(P_ghZAnti, P_GPlus, P_Hh, P_ghWm, 1277)
      call createCurrent(P_WQMinus, P_WPlus, P_AQ, 5777)
      call createCurrent(P_ghZ, P_GPlus, P_G0, P_ghWpAnti, 5111)
      call createCurrent(P_GQPlus, P_Hl, P_HQMinus, 3626)
      call createCurrent(P_HaQ, P_Ha, P_GPlus, P_HQMinus, 5089)
      call createCurrent(P_HQPlus, P_HMinus, P_GMinus, P_HQPlus, 1344)
      call createCurrent(P_ghA, P_ghWmAnti, P_WMinus, P_Z, 3071)
      call createCurrent(P_ghWmAnti, P_HMinus, P_GPlus, P_ghWm, 10253)
      call createCurrent(P_WPlus, P_HMinus, P_Z, 7288)
      call createCurrent(P_HQPlus, P_GMinus, P_Ha, P_HaQ, 4197)
      call createCurrent(P_GQ0, P_A, P_WMinus, P_GQPlus, 489)
      call createCurrent(P_GPlus, P_ghA, P_ghZAnti, P_GMinus, 7270)
      call createCurrent(P_G0, P_Ha, P_Hl, P_Hl, 8248)
      call createCurrent(P_Ha, P_G0, P_Ha, P_G0, 7331)
      call createCurrent(P_HPlus, P_ghZAnti, P_GMinus, P_ghZ, 9573)
      call createCurrent(P_c, P_sAnti, P_WQMinus, 233)
      call createCurrent(P_WMinus, P_sAnti, P_c, 7904)
      call createCurrent(P_Hl, P_G0, P_Hl, P_G0, 629)
      call createCurrent(P_Hh, P_ghZAnti, P_ghZ, P_Hh, 2127)
      call createCurrent(P_ghA, P_WPlus, P_WMinus, P_ghZAnti, 268)
      call createCurrent(P_ghAAnti, P_GMinus, P_Hh, P_ghWp, 6891)
      call createCurrent(P_Ha, P_HMinus, P_ghA, P_ghWmAnti, 7232)
      call createCurrent(P_GPlus, P_GMinus, P_G0, 9853)
      call createCurrent(P_ghAAnti, P_ghA, P_GPlus, P_GMinus, 2046)
      call createCurrent(P_GQ0, P_Hh, P_WPlus, P_WQMinus, 8725)
      call createCurrent(P_HQMinus, P_WPlus, P_HaQ, 7723)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HMinus, P_GPlus, 10047)
      call createCurrent(P_ghWp, P_Hh, P_ghAAnti, P_HMinus, 905)
      call createCurrent(P_HQPlus, P_ghZ, P_Ha, P_ghWpAnti, 5967)
      call createCurrent(P_G0, P_muPlus, P_muMinus, 3951)
      call createCurrent(P_A, P_ghWmAnti, P_ghWm, P_Z, 5188)
      call createCurrent(P_GPlus, P_ghWm, P_G0, P_ghZAnti, 3597)
      call createCurrent(P_HQMinus, P_WPlus, P_Hl, P_AQ, 1065)
      call createCurrent(P_Ha, P_Z, P_A, P_Ha, 9039)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_G0, 9065)
      call createCurrent(P_GPlus, P_A, P_WMinus, 8478)
      call createCurrent(P_HlQ, P_G0, P_GMinus, P_HQPlus, 5601)
      call createCurrent(P_ghWmAnti, P_ghZ, P_WMinus, 8560)
      call createCurrent(P_Hl, P_GMinus, P_HPlus, P_G0, 2689)
      call createCurrent(P_ghWpAnti, P_HPlus, P_Hl, P_ghZ, 5843)
      call createCurrent(P_Hh, P_G0, P_Ha, P_G0, 10579)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_Hl, P_ghA, 8204)
      call createCurrent(P_HlQ, P_GPlus, P_HMinus, P_HaQ, 7999)
      call createCurrent(P_nu_tau, P_WMinus, P_tauPlus, 5429)
      call createCurrent(P_HhQ, P_G0, P_Hl, P_GQ0, 5934)
      call createCurrent(P_Ha, P_Hh, P_HPlus, P_GMinus, 4264)
      call createCurrent(P_GPlus, P_ghWm, P_ghZAnti, P_G0, 8771)
      call createCurrent(P_HhQ, P_Hl, P_HPlus, P_HQMinus, 8769)
      call createCurrent(P_ghWpAnti, P_GMinus, P_GPlus, P_ghWp, 2987)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghWp, 2482)
      call createCurrent(P_WQMinus, P_WPlus, P_ghWp, P_ghWpAnti, 4143)
      call createCurrent(P_GPlus, P_ghZ, P_ghAAnti, P_GMinus, 5022)
      call createCurrent(P_HhQ, P_WPlus, P_A, P_HQMinus, 8485)
      call createCurrent(P_HPlus, P_Ha, P_Z, P_WMinus, 6795)
      call createCurrent(P_ghWp, P_ghWpAnti, P_WPlus, P_WMinus, 5049)
      call createCurrent(P_ghWmAnti, P_A, P_Z, P_ghWm, 5617)
      call createCurrent(P_ghWm, P_Ha, P_GPlus, P_ghZAnti, 9980)
      call createCurrent(P_G0, P_GMinus, P_A, P_WPlus, 189)
      call createCurrent(P_A, P_WPlus, P_HMinus, P_Hh, 6146)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_Hh, 7370)
      call createCurrent(P_A, P_HPlus, P_WMinus, P_Hl, 5688)
      call createCurrent(P_GQ0, P_ghWpAnti, P_GPlus, P_ghZ, 4209)
      call createCurrent(P_WMinus, P_Hh, P_WPlus, P_Hl, 10340)
      call createCurrent(P_ghA, P_ghWmAnti, P_GMinus, P_G0, 1509)
      call createCurrent(P_HhQ, P_Hh, P_GMinus, P_HQPlus, 7736)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghA, P_Ha, 10217)
      call createCurrent(P_ghWp, P_HPlus, P_ghWpAnti, P_GMinus, 8986)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_HhQ, 2111)
      call createCurrent(P_GPlus, P_WMinus, P_A, P_Ha, 9545)
      call createCurrent(P_ghWmAnti, P_A, P_ghWm, P_A, 6856)
      call createCurrent(P_WPlus, P_G0, P_A, P_HMinus, 10182)
      call createCurrent(P_HaQ, P_GPlus, P_HMinus, P_HlQ, 3965)
      call createCurrent(P_WMinus, P_Hl, P_HPlus, P_A, 2118)
      call createCurrent(P_ZQ, P_GMinus, P_WPlus, P_HlQ, 7528)
      call createCurrent(P_Z, P_ghWpAnti, P_ghZ, P_WPlus, 7147)
      call createCurrent(P_ghWpAnti, P_A, P_ghZ, P_WPlus, 4361)
      call createCurrent(P_A, P_GPlus, P_HMinus, P_Z, 9374)
      call createCurrent(P_ghZ, P_G0, P_ghWmAnti, P_GMinus, 3724)
      call createCurrent(P_Ha, P_HMinus, P_GPlus, P_G0, 118)
      call createCurrent(P_ghA, P_ghWmAnti, P_HMinus, P_Ha, 6434)
      call createCurrent(P_GMinus, P_WMinus, P_WPlus, P_GPlus, 3582)
      call createCurrent(P_GPlus, P_A, P_Ha, P_WMinus, 9389)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghWp, P_G0, 1966)
      call createCurrent(P_GQPlus, P_HMinus, P_G0, P_HhQ, 5493)
      call createCurrent(P_ghWmAnti, P_ghWm, P_G0, P_G0, 788)
      call createCurrent(P_GMinus, P_GPlus, P_G0, P_G0, 5840)
      call createCurrent(P_HMinus, P_ghAAnti, P_Hl, P_ghWp, 5966)
      call createCurrent(P_GQPlus, P_Hl, P_WMinus, P_AQ, 7978)
      call createCurrent(P_GPlus, P_GMinus, P_GMinus, P_HPlus, 2129)
      call createCurrent(P_Z, P_ghWpAnti, P_ghA, P_WPlus, 49)
      call createCurrent(P_HaQ, P_ghWm, P_Ha, P_ghWmAnti, 7265)
      call createCurrent(P_A, P_WPlus, P_ghAAnti, P_ghWm, 3140)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hl, P_HQMinus, 3051)
      call createCurrent(P_ghWm, P_WPlus, P_ghZAnti, P_A, 7951)
      call createCurrent(P_ghWpAnti, P_ghZ, P_G0, P_HPlus, 9462)
      call createCurrent(P_tAnti, P_t, P_HhQ, 8425)
      call createCurrent(P_GQMinus, P_GPlus, P_Hl, P_HhQ, 7484)
      call createCurrent(P_HQPlus, P_HMinus, P_G0, P_GQ0, 3961)
      call createCurrent(P_Ha, P_bAnti, P_b, 7428)
      call createCurrent(P_GMinus, P_G0, P_ghWp, P_ghZAnti, 7264)
      call createCurrent(P_WQMinus, P_Z, P_ghZ, P_ghWmAnti, 7658)
      call createCurrent(P_GQMinus, P_WPlus, P_A, P_HhQ, 902)
      call createCurrent(P_Z, P_muPlus, P_muMinus, 1486)
      call createCurrent(P_b, P_Ha, P_bAnti, 4978)
      call createCurrent(P_A, P_Hh, P_WPlus, P_HMinus, 2215)
      call createCurrent(P_GPlus, P_A, P_Hh, P_WMinus, 5251)
      call createCurrent(P_ghWp, P_ghZAnti, P_Ha, P_HQMinus, 2831)
      call createCurrent(P_ghZ, P_HPlus, P_Hh, P_ghWpAnti, 7491)
      call createCurrent(P_G0, P_Hh, P_Hh, P_Ha, 6045)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghA, P_WPlus, 8147)
      call createCurrent(P_dAnti, P_d, P_G0, 1271)
      call createCurrent(P_Hh, P_HMinus, P_WPlus, P_A, 7600)
      call createCurrent(P_G0, P_cAnti, P_c, 7732)
      call createCurrent(P_G0, P_ghWp, P_ghZAnti, P_GMinus, 3806)
      call createCurrent(P_G0, P_ghA, P_ghWpAnti, P_GPlus, 6174)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Z, P_Z, 213)
      call createCurrent(P_HMinus, P_HPlus, P_ghA, P_ghAAnti, 4030)
      call createCurrent(P_ZQ, P_HPlus, P_Z, P_HQMinus, 5608)
      call createCurrent(P_GQ0, P_ghWm, P_Hl, P_ghWmAnti, 6062)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghA, P_GPlus, 9146)
      call createCurrent(P_WPlus, P_WPlus, P_ghWpAnti, P_ghWm, 576)
      call createCurrent(P_ghWm, P_ghZAnti, P_Z, P_WPlus, 5090)
      call createCurrent(P_HhQ, P_Hh, P_Hh, P_HhQ, 2293)
      call createCurrent(P_HPlus, P_Hl, P_ghWpAnti, P_ghA, 1668)
      call createCurrent(P_ghWpAnti, P_ghWp, P_WPlus, P_WQMinus, 2775)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_Hl, 5886)
      call createCurrent(P_ghAAnti, P_ghA, P_WPlus, P_WQMinus, 8381)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_Hl, 4085)
      call createCurrent(P_WQPlus, P_Z, P_HMinus, P_HhQ, 7293)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_Hh, 2925)
      call createCurrent(P_Hl, P_ghZAnti, P_Hh, P_ghZ, 10161)
      call createCurrent(P_nu_eAnti, P_nu_e, P_ZQ, 3637)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_HlQ, 8092)
      call createCurrent(P_WPlus, P_ghAAnti, P_ghWm, P_Z, 4393)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghA, P_HMinus, 3363)
      call createCurrent(P_GQ0, P_GMinus, P_Hl, P_GQPlus, 3929)
      call createCurrent(P_Hl, P_Ha, P_G0, P_Hh, 6419)
      call createCurrent(P_Z, P_HPlus, P_GMinus, P_A, 9438)
      call createCurrent(P_Ha, P_WMinus, P_GPlus, P_Z, 10341)
      call createCurrent(P_G0, P_G0, P_A, P_A, 10537)
      call createCurrent(P_HPlus, P_Hl, P_ghZAnti, P_ghWm, 1140)
      call createCurrent(P_uAnti, P_d, P_WPlus, 4136)
      call createCurrent(P_GMinus, P_WPlus, P_WMinus, P_GPlus, 1868)
      call createCurrent(P_AQ, P_sAnti, P_s, 7960)
      call createCurrent(P_ghWp, P_Hl, P_ghWpAnti, P_HlQ, 5112)
      call createCurrent(P_ghA, P_GPlus, P_ghWpAnti, P_Hh, 322)
      call createCurrent(P_GQ0, P_A, P_GPlus, P_WQMinus, 4907)
      call createCurrent(P_G0, P_WMinus, P_WPlus, P_Ha, 9683)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghA, P_Hl, 7077)
      call createCurrent(P_WMinus, P_A, P_Hh, P_GPlus, 968)
      call createCurrent(P_Ha, P_Z, P_Hh, 2312)
      call createCurrent(P_g, P_g, P_g, P_A, 10675)
      call createCurrent(P_d, P_Z, P_dAnti, 115)
      call createCurrent(P_ghA, P_WMinus, P_ghWmAnti, P_Z, 6223)
      call createCurrent(P_ghZ, P_ghAAnti, P_GMinus, P_HPlus, 9937)
      call createCurrent(P_HQPlus, P_Hh, P_Hh, P_GQMinus, 1846)
      call createCurrent(P_ghWmAnti, P_G0, P_ghZ, P_HMinus, 9626)
      call createCurrent(P_A, P_Z, P_HPlus, P_HMinus, 3101)
      call createCurrent(P_Hl, P_Z, P_Z, P_Hl, 784)
      call createCurrent(P_GQ0, P_GMinus, P_G0, P_HQPlus, 8170)
      call createCurrent(P_A, P_G0, P_Ha, P_A, 10514)
      call createCurrent(P_Hl, P_ghWm, P_ghAAnti, P_HPlus, 5459)
      call createCurrent(P_Hl, P_Hl, P_G0, P_G0, 6849)
      call createCurrent(P_GQPlus, P_GMinus, P_G0, P_HlQ, 7113)
      call createCurrent(P_cAnti, P_c, P_Hl, 276)
      call createCurrent(P_GPlus, P_GPlus, P_GMinus, P_GMinus, 5789)
      call createCurrent(P_GQPlus, P_Hh, P_Z, P_WQMinus, 1105)
      call createCurrent(P_gQ, P_c, P_cAnti, 2952)
      call createCurrent(P_ZQ, P_HMinus, P_Hh, P_WQPlus, 1497)
      call createCurrent(P_Hl, P_ghWmAnti, P_Hh, P_ghWm, 10180)
      call createCurrent(P_HQMinus, P_HPlus, P_ZQ, 7405)
      call createCurrent(P_GQPlus, P_Hl, P_WMinus, P_ZQ, 1942)
      call createCurrent(P_HlQ, P_Z, P_WMinus, P_GQPlus, 2453)
      call createCurrent(P_Hl, P_A, P_Z, P_Hh, 10747)
      call createCurrent(P_Z, P_ghWmAnti, P_WMinus, P_ghZ, 1717)
      call createCurrent(P_Z, P_WMinus, P_ghZAnti, P_ghWp, 4346)
      call createCurrent(P_eMinus, P_HPlus, P_nu_eAnti, 7838)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hh, P_HQPlus, 8342)
      call createCurrent(P_HaQ, P_HPlus, P_GMinus, P_GQ0, 5028)
      call createCurrent(P_ghWmAnti, P_ghZ, P_G0, P_GQMinus, 292)
      call createCurrent(P_Hl, P_GMinus, P_ghZ, P_ghWmAnti, 7101)
      call createCurrent(P_ghWm, P_A, P_ghWmAnti, P_Z, 392)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Z, P_AQ, 8488)
      call createCurrent(P_t, P_G0, P_tAnti, 6004)
      call createCurrent(P_g, P_Hl, P_g, 10533)
      call createCurrent(P_Ha, P_GPlus, P_Hh, P_HMinus, 3335)
      call createCurrent(P_WQPlus, P_WMinus, P_ghWmAnti, P_ghWm, 4046)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HlQ, 889)
      call createCurrent(P_ghZ, P_Hl, P_ghWpAnti, P_GPlus, 483)
      call createCurrent(P_HQMinus, P_Z, P_WPlus, P_HaQ, 5451)
      call createCurrent(P_HPlus, P_WMinus, P_Z, P_G0, 8904)
      call createCurrent(P_Hh, P_HMinus, P_GPlus, P_Hh, 1654)
      call createCurrent(P_WMinus, P_Ha, P_GPlus, P_Z, 9505)
      call createCurrent(P_ghA, P_GPlus, P_GMinus, P_ghZAnti, 4560)
      call createCurrent(P_Ha, P_ghWmAnti, P_ghA, P_GMinus, 10109)
      call createCurrent(P_A, P_Z, P_Ha, P_G0, 10499)
      call createCurrent(P_AQ, P_WPlus, P_Ha, P_HQMinus, 2369)
      call createCurrent(P_HaQ, P_Ha, P_ghWp, P_ghWpAnti, 8365)
      call createCurrent(P_Hl, P_Hh, P_Ha, P_G0, 5421)
      call createCurrent(P_GQMinus, P_WPlus, P_WMinus, P_GQPlus, 2487)
      call createCurrent(P_HQMinus, P_Hh, P_Hl, P_HQPlus, 6052)
      call createCurrent(P_G0, P_GMinus, P_GPlus, 9451)
      call createCurrent(P_G0, P_ghWm, P_ghAAnti, P_GPlus, 5455)
      call createCurrent(P_HPlus, P_G0, P_G0, P_HMinus, 4295)
      call createCurrent(P_GPlus, P_ghWpAnti, P_Hh, P_ghA, 8088)
      call createCurrent(P_Ha, P_Z, P_Z, P_G0, 10233)
      call createCurrent(P_GQMinus, P_ghA, P_G0, P_ghWmAnti, 4831)
      call createCurrent(P_GMinus, P_ghZAnti, P_HPlus, P_ghA, 9196)
      call createCurrent(P_ZQ, P_GPlus, P_A, P_GQMinus, 3711)
      call createCurrent(P_A, P_Hh, P_Ha, 10443)
      call createCurrent(P_GPlus, P_WMinus, P_Z, 236)
      call createCurrent(P_HQMinus, P_ghWpAnti, P_HPlus, P_ghWp, 2130)
      call createCurrent(P_WPlus, P_HMinus, P_Z, P_Ha, 3369)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_Ha, 3471)
      call createCurrent(P_G0, P_ghZAnti, P_ghZ, P_Ha, 9687)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghWp, P_ZQ, 949)
      call createCurrent(P_Hh, P_Hh, P_G0, P_Ha, 3554)
      call createCurrent(P_t, P_GMinus, P_bAnti, 5193)
      call createCurrent(P_HaQ, P_ghWpAnti, P_Ha, P_ghWp, 1790)
      call createCurrent(P_ghWp, P_Hh, P_HMinus, P_ghAAnti, 8659)
      call createCurrent(P_GMinus, P_Z, P_Z, P_GPlus, 5655)
      call createCurrent(P_GQ0, P_HPlus, P_Hl, P_HQMinus, 8172)
      call createCurrent(P_HPlus, P_HMinus, P_ghZ, P_ghZAnti, 7587)
      call createCurrent(P_A, P_Ha, P_Ha, P_Z, 10127)
      call createCurrent(P_HlQ, P_HMinus, P_WQPlus, 3664)
      call createCurrent(P_G0, P_WPlus, P_WMinus, P_Ha, 9448)
      call createCurrent(P_G0, P_WPlus, P_A, P_GMinus, 5983)
      call createCurrent(P_Ha, P_ghAAnti, P_HMinus, P_ghWp, 556)
      call createCurrent(P_GQMinus, P_Z, P_GQPlus, 6399)
      call createCurrent(P_ghWp, P_ghWpAnti, P_AQ, 5970)
      call createCurrent(P_ghWp, P_Hl, P_ghZAnti, P_HMinus, 4351)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_Hl, P_ghZ, 6545)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, 3189)
      call createCurrent(P_HMinus, P_Z, P_WPlus, P_Hh, 5051)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_Hh, P_ghZ, 3145)
      call createCurrent(P_WPlus, P_G0, P_HMinus, P_A, 9215)
      call createCurrent(P_AQ, P_Hh, P_WPlus, P_HQMinus, 3120)
      call createCurrent(P_WPlus, P_Hl, P_GMinus, P_A, 2427)
      call createCurrent(P_WMinus, P_HPlus, P_Z, P_Hh, 7012)
      call createCurrent(P_HhQ, P_GMinus, P_WPlus, P_ZQ, 7683)
      call createCurrent(P_G0, P_HPlus, P_HMinus, P_Hh, 10398)
      call createCurrent(P_G0, P_Ha, P_Ha, P_Ha, 7120)
      call createCurrent(P_HQMinus, P_Hl, P_WPlus, P_AQ, 3223)
      call createCurrent(P_HMinus, P_HPlus, P_G0, P_Hl, 10007)
      call createCurrent(P_GMinus, P_A, P_Z, P_GPlus, 6154)
      call createCurrent(P_Ha, P_G0, P_A, P_A, 10543)
      call createCurrent(P_gQ, P_g, P_ghGAnti, P_ghG, 7947)
      call createCurrent(P_G0, P_HMinus, P_HPlus, 10437)
      call createCurrent(P_HaQ, P_Ha, P_GMinus, P_GQPlus, 2084)
      call createCurrent(P_Hl, P_ghZAnti, P_GPlus, P_ghWm, 7715)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hl, P_HMinus, 3215)
      call createCurrent(P_HlQ, P_ghWp, P_HMinus, P_ghZAnti, 3968)
      call createCurrent(P_Hl, P_HPlus, P_ghZAnti, P_ghWm, 4860)
      call createCurrent(P_Hh, P_Hh, P_Ha, P_G0, 343)
      call createCurrent(P_HaQ, P_Ha, P_ghZ, P_ghZAnti, 1864)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghZ, P_GMinus, 2625)
      call createCurrent(P_WPlus, P_Z, P_Z, P_WMinus, 2873)
      call createCurrent(P_A, P_GPlus, P_GMinus, P_Z, 8660)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_HhQ, 6644)
      call createCurrent(P_HQMinus, P_Hl, P_GPlus, P_GQ0, 6824)
      call createCurrent(P_GQMinus, P_G0, P_ghZAnti, P_ghWp, 403)
      call createCurrent(P_GQPlus, P_Hh, P_ghWpAnti, P_ghA, 4583)
      call createCurrent(P_HlQ, P_Hl, P_HMinus, P_GQPlus, 823)
      call createCurrent(P_G0, P_muMinus, P_muPlus, 8811)
      call createCurrent(P_A, P_A, P_HMinus, P_HPlus, 2044)
      call createCurrent(P_GPlus, P_Hh, P_Hh, P_GMinus, 6637)
      call createCurrent(P_HMinus, P_ghZ, P_ghWmAnti, P_Hh, 6261)
      call createCurrent(P_nu_tauAnti, P_nu_tau, P_ZQ, 1162)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hl, P_GQMinus, 1938)
      call createCurrent(P_WMinus, P_WPlus, P_A, P_Z, 1725)
      call createCurrent(P_ghA, P_ghWpAnti, P_GPlus, P_G0, 7919)
      call createCurrent(P_HQMinus, P_ghWp, P_Hl, P_ghZAnti, 6493)
      call createCurrent(P_WMinus, P_WPlus, P_HMinus, P_HPlus, 1433)
      call createCurrent(P_Hh, P_ghAAnti, P_ghWm, P_GPlus, 7072)
      call createCurrent(P_ghZ, P_WPlus, P_A, P_ghWpAnti, 895)
      call createCurrent(P_Ha, P_Hh, P_Ha, 5117)
      call createCurrent(P_WPlus, P_Z, P_HMinus, P_Ha, 1017)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hh, 2933)
      call createCurrent(P_HMinus, P_HPlus, P_HMinus, P_GPlus, 3203)
      call createCurrent(P_ghA, P_HMinus, P_ghWmAnti, P_Hl, 5508)
      call createCurrent(P_GQ0, P_HPlus, P_GMinus, P_HaQ, 8696)
      call createCurrent(P_GQ0, P_Ha, P_HMinus, P_HQPlus, 5833)
      call createCurrent(P_HPlus, P_Ha, P_HMinus, P_Hh, 9593)
      call createCurrent(P_AQ, P_muMinus, P_muPlus, 5989)
      call createCurrent(P_ghWm, P_ghWmAnti, P_GPlus, P_HMinus, 9844)
      call createCurrent(P_GQ0, P_cAnti, P_c, 2082)
      call createCurrent(P_ghWpAnti, P_Ha, P_G0, P_ghWp, 10361)
      call createCurrent(P_AQ, P_GPlus, P_Hh, P_WQMinus, 6787)
      call createCurrent(P_GQPlus, P_G0, P_HMinus, P_HlQ, 7040)
      call createCurrent(P_Z, P_GPlus, P_Z, P_HMinus, 9537)
      call createCurrent(P_HhQ, P_Hh, P_G0, P_HaQ, 186)
      call createCurrent(P_ZQ, P_WMinus, P_Hh, P_GQPlus, 1760)
      call createCurrent(P_HMinus, P_A, P_Hl, P_WPlus, 6380)
      call createCurrent(P_WMinus, P_ghZAnti, P_ghA, P_WPlus, 8426)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hl, P_GQMinus, 125)
      call createCurrent(P_ghZ, P_ghWpAnti, P_A, P_WPlus, 1569)
      call createCurrent(P_Ha, P_G0, P_G0, P_Ha, 707)
      call createCurrent(P_uAnti, P_u, P_gQ, 7967)
      call createCurrent(P_ghAAnti, P_Ha, P_ghWp, P_HMinus, 5971)
      call createCurrent(P_GQ0, P_Hh, P_G0, P_HhQ, 2629)
      call createCurrent(P_G0, P_HMinus, P_ghAAnti, P_ghWp, 10166)
      call createCurrent(P_HPlus, P_Z, P_HMinus, P_Z, 5118)
      call createCurrent(P_HlQ, P_HPlus, P_GMinus, P_HaQ, 6547)
      call createCurrent(P_GMinus, P_Z, P_GPlus, 1597)
      call createCurrent(P_GQPlus, P_ghZ, P_G0, P_ghWpAnti, 7151)
      call createCurrent(P_HQMinus, P_HPlus, P_ghZ, P_ghZAnti, 6807)
      call createCurrent(P_muPlus, P_muMinus, P_AQ, 7507)
      call createCurrent(P_AQ, P_WMinus, P_Ha, P_HQPlus, 636)
      call createCurrent(P_HQPlus, P_HMinus, P_Z, P_ZQ, 2490)
      call createCurrent(P_Hh, P_Z, P_Hl, P_A, 10542)
      call createCurrent(P_HQMinus, P_WPlus, P_HhQ, 1094)
      call createCurrent(P_Hl, P_Hl, P_ghWp, P_ghWpAnti, 6262)
      call createCurrent(P_bAnti, P_t, P_HMinus, 7446)
      call createCurrent(P_ghZ, P_G0, P_ghWmAnti, P_HMinus, 9872)
      call createCurrent(P_ghZAnti, P_ghWm, P_G0, P_HPlus, 9082)
      call createCurrent(P_GQMinus, P_GPlus, P_A, P_AQ, 2485)
      call createCurrent(P_HlQ, P_GMinus, P_Ha, P_HQPlus, 6818)
      call createCurrent(P_WMinus, P_Hl, P_Hl, P_WPlus, 1783)
      call createCurrent(P_ePlus, P_WMinus, P_nu_e, 3453)
      call createCurrent(P_t, P_tAnti, P_AQ, 7392)
      call createCurrent(P_g, P_HPlus, P_g, P_HMinus, 10667)
      call createCurrent(P_Z, P_Hh, P_GMinus, P_WPlus, 7807)
      call createCurrent(P_ghZAnti, P_Ha, P_ghWm, P_HPlus, 458)
      call createCurrent(P_WQMinus, P_Ha, P_HPlus, P_AQ, 4961)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghA, P_HQPlus, 4857)
      call createCurrent(P_HMinus, P_ghWp, P_ghAAnti, P_G0, 10439)
      call createCurrent(P_ghZ, P_HPlus, P_ghZAnti, P_HMinus, 3700)
      call createCurrent(P_A, P_WMinus, P_WPlus, P_A, 6055)
      call createCurrent(P_ghZ, P_Hh, P_ghZAnti, P_Hh, 5450)
      call createCurrent(P_GPlus, P_GMinus, P_GMinus, P_GPlus, 15)
      call createCurrent(P_Z, P_GMinus, P_WPlus, P_Ha, 10325)
      call createCurrent(P_GMinus, P_HPlus, P_ghAAnti, P_ghA, 9428)
      call createCurrent(P_HhQ, P_GMinus, P_Hl, P_HQPlus, 5551)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hh, P_GPlus, 3843)
      call createCurrent(P_HQMinus, P_nu_e, P_ePlus, 8334)
      call createCurrent(P_HPlus, P_ghWm, P_ghWmAnti, P_HMinus, 7691)
      call createCurrent(P_GPlus, P_Hl, P_HMinus, P_Hl, 6464)
      call createCurrent(P_Hh, P_A, P_GMinus, P_WPlus, 7813)
      call createCurrent(P_A, P_Z, P_Hl, P_Hh, 10522)
      call createCurrent(P_Hh, P_ghWp, P_HMinus, P_ghAAnti, 414)
      call createCurrent(P_GQMinus, P_GPlus, P_ghZAnti, P_ghZ, 1364)
      call createCurrent(P_GQ0, P_WPlus, P_G0, P_WQMinus, 2254)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HMinus, P_HPlus, 4021)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_HaQ, 2248)
      call createCurrent(P_WPlus, P_ghZ, P_ghWpAnti, P_Z, 6684)
      call createCurrent(P_WQPlus, P_WMinus, P_ZQ, 4306)
      call createCurrent(P_GPlus, P_GMinus, 9347)
      call createCurrent(P_ghWm, P_Z, P_ghAAnti, P_WPlus, 220)
      call createCurrent(P_c, P_g, P_cAnti, 2552)
      call createCurrent(P_GPlus, P_HMinus, P_Hh, P_Hl, 2298)
      call createCurrent(P_Ha, P_WPlus, P_HMinus, P_Z, 80)
      call createCurrent(P_HQPlus, P_GPlus, P_HMinus, P_GQMinus, 5346)
      call createCurrent(P_Ha, P_G0, P_G0, 10722)
      call createCurrent(P_HhQ, P_G0, P_Hh, P_HaQ, 5514)
      call createCurrent(P_GQPlus, P_GMinus, P_ghA, P_ghAAnti, 3080)
      call createCurrent(P_GQPlus, P_HPlus, P_GMinus, P_GQMinus, 6847)
      call createCurrent(P_GPlus, P_ghWpAnti, P_Ha, P_ghZ, 8923)
      call createCurrent(P_muMinus, P_muPlus, P_AQ, 700)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hh, P_GQ0, 6331)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_Hh, 5042)
      call createCurrent(P_GMinus, P_Hh, P_ghWp, P_ghAAnti, 325)
      call createCurrent(P_GPlus, P_Hl, P_HMinus, P_Ha, 5483)
      call createCurrent(P_G0, P_ghWm, P_HPlus, P_ghAAnti, 9973)
      call createCurrent(P_u, P_Ha, P_uAnti, 1533)
      call createCurrent(P_ghZAnti, P_ghA, P_HMinus, P_GPlus, 8895)
      call createCurrent(P_HPlus, P_WMinus, P_Z, 3200)
      call createCurrent(P_HMinus, P_HPlus, P_ghZAnti, P_ghZ, 5953)
      call createCurrent(P_Hl, P_HMinus, P_ghWmAnti, P_ghA, 746)
      call createCurrent(P_GPlus, P_G0, P_WMinus, P_A, 8493)
      call createCurrent(P_HPlus, P_ghAAnti, P_Hh, P_ghWm, 5899)
      call createCurrent(P_ghZAnti, P_WPlus, P_Z, P_ghWm, 7802)
      call createCurrent(P_HaQ, P_HMinus, P_GQPlus, 3975)
      call createCurrent(P_cAnti, P_c, P_ZQ, 2955)
      call createCurrent(P_GPlus, P_ghWm, P_ghWmAnti, P_GMinus, 7993)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_G0, 9634)
      call createCurrent(P_Hl, P_HMinus, P_GPlus, 6268)
      call createCurrent(P_WMinus, P_Z, P_ghWp, P_ghAAnti, 1753)
      call createCurrent(P_ghA, P_HMinus, P_ghAAnti, P_GPlus, 9046)
      call createCurrent(P_Hh, P_HPlus, P_GMinus, P_Hl, 5521)
      call createCurrent(P_ghZ, P_WPlus, P_ghWpAnti, 5359)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_WPlus, P_ghWm, 4885)
      call createCurrent(P_GQMinus, P_WMinus, P_WPlus, P_GQPlus, 6565)
      call createCurrent(P_HQPlus, P_tAnti, P_b, 4318)
      call createCurrent(P_ghWmAnti, P_ghWp, P_WMinus, P_WMinus, 8721)
      call createCurrent(P_ghWm, P_Hl, P_Hh, P_ghWmAnti, 9053)
      call createCurrent(P_HPlus, P_Hh, P_GMinus, P_Hh, 2426)
      call createCurrent(P_AQ, P_ghWp, P_ghWpAnti, 1771)
      call createCurrent(P_ghWm, P_ghAAnti, P_Hl, P_HPlus, 2018)
      call createCurrent(P_ghAAnti, P_ghWp, P_WMinus, P_A, 4619)
      call createCurrent(P_GMinus, P_HPlus, P_HPlus, P_GMinus, 8668)
      call createCurrent(P_GMinus, P_Hl, P_GPlus, P_Ha, 8941)
      call createCurrent(P_HPlus, P_HMinus, P_Hh, P_Hl, 3232)
      call createCurrent(P_G0, P_G0, P_Hh, P_Hl, 4890)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghZ, P_HPlus, 2863)
      call createCurrent(P_HQPlus, P_Hh, P_HMinus, P_HlQ, 2906)
      call createCurrent(P_s, P_sAnti, P_ZQ, 4544)
      call createCurrent(P_ghWm, P_HPlus, P_GMinus, P_ghWmAnti, 9360)
      call createCurrent(P_WQMinus, P_WPlus, P_Ha, P_HlQ, 1583)
      call createCurrent(P_HMinus, P_Ha, P_Hl, P_HPlus, 9254)
      call createCurrent(P_GPlus, P_Hh, P_Ha, P_HMinus, 6841)
      call createCurrent(P_ghWm, P_Z, P_ghWmAnti, P_AQ, 283)
      call createCurrent(P_Z, P_Z, P_Hh, 2138)
      call createCurrent(P_Z, P_WPlus, P_Ha, P_HMinus, 8538)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, 7593)
      call createCurrent(P_ghZAnti, P_ghZ, P_GMinus, P_GPlus, 6292)
      call createCurrent(P_AQ, P_ghWp, P_WMinus, P_ghAAnti, 6354)
      call createCurrent(P_WMinus, P_Z, P_Z, P_WPlus, 3764)
      call createCurrent(P_GMinus, P_ghWmAnti, P_GPlus, P_ghWm, 2944)
      call createCurrent(P_HaQ, P_Hh, P_ZQ, 3757)
      call createCurrent(P_Ha, P_ghZ, P_HPlus, P_ghWpAnti, 4578)
      call createCurrent(P_ghWp, P_G0, P_ghAAnti, P_HMinus, 10085)
      call createCurrent(P_ghZ, P_HMinus, P_ghZAnti, P_GPlus, 9902)
      call createCurrent(P_Hh, P_ghWp, P_Hl, P_ghWpAnti, 9321)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hh, P_HaQ, 8562)
      call createCurrent(P_HPlus, P_Z, P_G0, P_WMinus, 9255)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWp, P_HMinus, 6285)
      call createCurrent(P_ghZ, P_Hl, P_ghWpAnti, P_HPlus, 6475)
      call createCurrent(P_HQPlus, P_HMinus, P_WPlus, P_WQMinus, 3708)
      call createCurrent(P_WMinus, P_GMinus, P_WPlus, P_GPlus, 5509)
      call createCurrent(P_muPlus, P_nu_mu, P_GQMinus, 2062)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWm, P_GPlus, 5447)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hh, 2745)
      call createCurrent(P_WPlus, P_Z, P_Ha, P_HMinus, 1211)
      call createCurrent(P_Ha, P_ghWm, P_ghWmAnti, P_Ha, 8602)
      call createCurrent(P_G0, P_ghWp, P_ghAAnti, P_GMinus, 6149)
      call createCurrent(P_s, P_Hl, P_sAnti, 4308)
      call createCurrent(P_cAnti, P_s, P_GQPlus, 5985)
      call createCurrent(P_Hh, P_GMinus, P_ghWmAnti, P_ghZ, 6025)
      call createCurrent(P_ghA, P_ghZAnti, P_GMinus, P_GQPlus, 8393)
      call createCurrent(P_b, P_bAnti, P_Z, 1939)
      call createCurrent(P_ghZAnti, P_ghZ, P_G0, P_GQ0, 4125)
      call createCurrent(P_Hl, P_Hl, P_GMinus, P_HPlus, 6413)
      call createCurrent(P_Hl, P_Ha, P_Ha, P_Hh, 6848)
      call createCurrent(P_WPlus, P_GMinus, P_Z, P_Hh, 6542)
      call createCurrent(P_cAnti, P_HPlus, P_s, 6889)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghA, P_HMinus, 9873)
      call createCurrent(P_HPlus, P_ghWm, P_ghAAnti, P_Ha, 5728)
      call createCurrent(P_GPlus, P_HMinus, P_HMinus, P_GPlus, 2727)
      call createCurrent(P_Hl, P_HPlus, P_GMinus, 7051)
      call createCurrent(P_G0, P_GPlus, P_HMinus, 28)
      call createCurrent(P_Hh, P_Z, P_WPlus, P_GMinus, 5000)
      call createCurrent(P_ghWm, P_WPlus, P_ghAAnti, 7468)
      call createCurrent(P_HPlus, P_WMinus, P_Ha, P_A, 6320)
      call createCurrent(P_GMinus, P_ghZ, P_ghAAnti, P_GPlus, 3449)
      call createCurrent(P_ghWpAnti, P_GPlus, P_Ha, P_ghZ, 9235)
      call createCurrent(P_ghAAnti, P_ghZ, P_HPlus, P_HQMinus, 5915)
      call createCurrent(P_ghWmAnti, P_ghA, P_G0, P_GQMinus, 3443)
      call createCurrent(P_GPlus, P_GMinus, P_ghZ, P_ghAAnti, 1733)
      call createCurrent(P_WPlus, P_WMinus, P_ghZAnti, P_ghA, 2060)
      call createCurrent(P_HaQ, P_Hh, P_ghWpAnti, P_ghWp, 4580)
      call createCurrent(P_Ha, P_HMinus, P_Hl, P_GPlus, 8601)
      call createCurrent(P_GQ0, P_GMinus, P_GPlus, P_HaQ, 5223)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWm, P_GPlus, 4974)
      call createCurrent(P_Hl, P_GPlus, P_HMinus, 3420)
      call createCurrent(P_ghWmAnti, P_GMinus, P_Hh, P_ghZ, 107)
      call createCurrent(P_WPlus, P_ghWm, P_ghZAnti, P_A, 3431)
      call createCurrent(P_WMinus, P_ghA, P_ghWmAnti, 4784)
      call createCurrent(P_AQ, P_Ha, P_WMinus, P_HQPlus, 8316)
      call createCurrent(P_ghWp, P_ghZAnti, P_Ha, P_HMinus, 8030)
      call createCurrent(P_Z, P_WMinus, P_Ha, P_HPlus, 6402)
      call createCurrent(P_Hl, P_eMinus, P_ePlus, 188)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghWp, P_WMinus, 1348)
      call createCurrent(P_HlQ, P_Hh, P_Hl, P_HhQ, 8261)
      call createCurrent(P_HlQ, P_GPlus, P_Ha, P_GQMinus, 8581)
      call createCurrent(P_Hh, P_tauMinus, P_tauPlus, 1657)
      call createCurrent(P_GQPlus, P_GMinus, P_HPlus, P_GQMinus, 8708)
      call createCurrent(P_Z, P_Ha, P_Hl, 488)
      call createCurrent(P_Z, P_GPlus, P_WMinus, 2481)
      call createCurrent(P_WPlus, P_GMinus, P_G0, P_A, 7452)
      call createCurrent(P_AQ, P_HPlus, P_WQMinus, 3980)
      call createCurrent(P_Z, P_Z, P_WPlus, P_WMinus, 5670)
      call createCurrent(P_HlQ, P_HMinus, P_G0, P_GQPlus, 8412)
      call createCurrent(P_A, P_ghWpAnti, P_ghZ, P_WPlus, 7220)
      call createCurrent(P_g, P_g, P_HPlus, P_GMinus, 10576)
      call createCurrent(P_ghAAnti, P_Hl, P_HPlus, P_ghWm, 7320)
      call createCurrent(P_WPlus, P_HMinus, P_Hh, 4422)
      call createCurrent(P_ghWp, P_Hl, P_ghWpAnti, P_HaQ, 747)
      call createCurrent(P_Ha, P_Ha, P_Hl, 1152)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghA, P_Ha, 5564)
      call createCurrent(P_HaQ, P_HPlus, P_WQMinus, 2973)
      call createCurrent(P_WMinus, P_g, P_g, P_WPlus, 10570)
      call createCurrent(P_ghWpAnti, P_GPlus, P_Hh, P_ghZ, 3258)
      call createCurrent(P_tAnti, P_t, P_HlQ, 3371)
      call createCurrent(P_Z, P_WMinus, P_ghWmAnti, P_ghZ, 2193)
      call createCurrent(P_GQPlus, P_WMinus, P_G0, P_AQ, 4757)
      call createCurrent(P_HPlus, P_HMinus, P_Z, 8685)
      call createCurrent(P_WQPlus, P_WPlus, P_WMinus, P_WQMinus, 1757)
      call createCurrent(P_G0, P_A, P_HPlus, P_WMinus, 10211)
      call createCurrent(P_GMinus, P_Hh, P_HPlus, P_Hl, 859)
      call createCurrent(P_ghWp, P_A, P_ghZAnti, P_WMinus, 5507)
      call createCurrent(P_GQ0, P_G0, P_GMinus, P_HQPlus, 3833)
      call createCurrent(P_GPlus, P_ghWmAnti, P_ghWm, P_HMinus, 10051)
      call createCurrent(P_Hl, P_A, P_Hl, P_Z, 9176)
      call createCurrent(P_ghWpAnti, P_ghWp, P_G0, P_G0, 8027)
      call createCurrent(P_GPlus, P_GMinus, P_Hh, 7581)
      call createCurrent(P_Hh, P_HMinus, P_ghWmAnti, P_ghZ, 3948)
      call createCurrent(P_HhQ, P_G0, P_HMinus, P_HQPlus, 6437)
      call createCurrent(P_HaQ, P_Hl, P_ghWpAnti, P_ghWp, 5651)
      call createCurrent(P_Z, P_HMinus, P_Z, P_HPlus, 6103)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hl, 6017)
      call createCurrent(P_ZQ, P_Ha, P_WMinus, P_HQPlus, 4107)
      call createCurrent(P_HMinus, P_HPlus, P_ghWp, P_ghWpAnti, 5866)
      call createCurrent(P_HMinus, P_ghA, P_ghWmAnti, P_Hh, 3798)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghA, P_Ha, 8588)
      call createCurrent(P_nu_e, P_ePlus, P_GQMinus, 5417)
      call createCurrent(P_G0, P_GPlus, P_ghWpAnti, P_ghA, 1818)
      call createCurrent(P_Z, P_A, P_A, P_Z, 10644)
      call createCurrent(P_Ha, P_Hl, P_Hl, P_Hl, 10516)
      call createCurrent(P_HMinus, P_HPlus, P_ghAAnti, P_ghZ, 4699)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_HaQ, 5794)
      call createCurrent(P_G0, P_GPlus, P_GMinus, P_G0, 4325)
      call createCurrent(P_WMinus, P_A, P_HPlus, P_Hl, 174)
      call createCurrent(P_HQPlus, P_Z, P_HMinus, P_ZQ, 2819)
      call createCurrent(P_Ha, P_A, P_HPlus, P_WMinus, 4724)
      call createCurrent(P_Ha, P_Hl, P_HMinus, P_HPlus, 10305)
      call createCurrent(P_A, P_GMinus, P_HPlus, P_Z, 9691)
      call createCurrent(P_ghA, P_ghWmAnti, P_G0, P_GQMinus, 1051)
      call createCurrent(P_ghA, P_HPlus, P_ghAAnti, P_HQMinus, 6612)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghZ, P_GMinus, 3518)
      call createCurrent(P_GMinus, P_G0, P_ghZAnti, P_ghWp, 441)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_GQ0, 8402)
      call createCurrent(P_G0, P_WMinus, P_GPlus, 7242)
      call createCurrent(P_Hl, P_Ha, P_Hl, P_Ha, 646)
      call createCurrent(P_ghZ, P_ghWmAnti, P_G0, P_GQMinus, 7681)
      call createCurrent(P_HlQ, P_HMinus, P_ghWp, P_ghAAnti, 3596)
      call createCurrent(P_HMinus, P_WPlus, P_G0, 9018)
      call createCurrent(P_WQMinus, P_WPlus, P_G0, P_HlQ, 358)
      call createCurrent(P_WMinus, P_WPlus, P_Z, P_A, 2290)
      call createCurrent(P_G0, P_Z, 10355)
      call createCurrent(P_muPlus, P_muMinus, P_Ha, 802)
      call createCurrent(P_A, P_G0, P_GPlus, P_WMinus, 2338)
      call createCurrent(P_GQMinus, P_HPlus, P_Hh, P_HlQ, 2301)
      call createCurrent(P_A, P_GMinus, P_HPlus, P_A, 9330)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghA, P_WQMinus, 602)
      call createCurrent(P_AQ, P_WMinus, P_WQPlus, 6279)
      call createCurrent(P_WMinus, P_ghWmAnti, P_Z, P_ghA, 4441)
      call createCurrent(P_HMinus, P_HPlus, P_GMinus, P_HPlus, 1202)
      call createCurrent(P_Z, P_Ha, P_A, P_G0, 10734)
      call createCurrent(P_GQ0, P_G0, P_HhQ, 1107)
      call createCurrent(P_WMinus, P_A, P_ghWp, P_ghAAnti, 4710)
      call createCurrent(P_HMinus, P_HPlus, P_Ha, P_Ha, 7257)
      call createCurrent(P_nu_tauAnti, P_tauMinus, P_WQPlus, 7448)
      call createCurrent(P_HMinus, P_ghZ, P_GPlus, P_ghZAnti, 9306)
      call createCurrent(P_GQMinus, P_WPlus, P_G0, P_ZQ, 501)
      call createCurrent(P_ghAAnti, P_Hh, P_ghWm, P_GPlus, 740)
      call createCurrent(P_GQMinus, P_HPlus, P_Ha, P_HhQ, 3268)
      call createCurrent(P_ghWp, P_WPlus, P_ghWpAnti, P_WMinus, 773)
      call createCurrent(P_WPlus, P_HMinus, P_Ha, P_Z, 8059)
      call createCurrent(P_HQPlus, P_Hh, P_GMinus, P_HhQ, 4646)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Z, P_WMinus, 7155)
      call createCurrent(P_ghZAnti, P_ghZ, P_HPlus, P_GMinus, 9895)
      call createCurrent(P_GPlus, P_GMinus, P_G0, P_Hl, 10069)
      call createCurrent(P_HlQ, P_GPlus, P_Ha, P_HQMinus, 6397)
      call createCurrent(P_HMinus, P_g, P_g, P_GPlus, 10735)
      call createCurrent(P_Hl, P_GPlus, P_WMinus, P_A, 577)
      call createCurrent(P_ZQ, P_WPlus, P_ghWm, P_ghAAnti, 1297)
      call createCurrent(P_ghWm, P_WPlus, P_ghZAnti, P_ZQ, 3855)
      call createCurrent(P_ZQ, P_nu_eAnti, P_nu_e, 1606)
      call createCurrent(P_ghA, P_ghWmAnti, P_G0, P_HMinus, 9596)
      call createCurrent(P_WMinus, P_A, P_Hl, P_HPlus, 4825)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Z, P_A, 7444)
      call createCurrent(P_Hl, P_ghA, P_HMinus, P_ghWmAnti, 6412)
      call createCurrent(P_HPlus, P_ghWm, P_Hl, P_ghZAnti, 7115)
      call createCurrent(P_WQMinus, P_GPlus, P_Z, P_HhQ, 2462)
      call createCurrent(P_GMinus, P_Hh, P_Ha, P_GPlus, 8885)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghWp, 5661)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghA, P_HMinus, 9700)
      call createCurrent(P_ZQ, P_Hl, P_GMinus, P_WQPlus, 5583)
      call createCurrent(P_HMinus, P_Ha, P_G0, P_HPlus, 8514)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hl, P_Hl, 6000)
      call createCurrent(P_WMinus, P_ghAAnti, P_A, P_ghWp, 5103)
      call createCurrent(P_Ha, P_ghZAnti, P_ghZ, P_Ha, 249)
      call createCurrent(P_nu_e, P_ePlus, P_HQMinus, 8146)
      call createCurrent(P_WQPlus, P_Hh, P_HQMinus, 4252)
      call createCurrent(P_ghWmAnti, P_Hh, P_Hl, P_ghWm, 9764)
      call createCurrent(P_GPlus, P_WMinus, P_Ha, 9359)
      call createCurrent(P_b, P_bAnti, P_GQ0, 4770)
      call createCurrent(P_Z, P_ghWm, P_WPlus, P_ghAAnti, 6114)
      call createCurrent(P_A, P_Z, P_GMinus, P_GPlus, 4748)
      call createCurrent(P_WPlus, P_A, P_GMinus, P_Ha, 9968)
      call createCurrent(P_GMinus, P_ghZ, P_ghWmAnti, 4912)
      call createCurrent(P_GQMinus, P_HPlus, P_Ha, P_HlQ, 6913)
      call createCurrent(P_GQPlus, P_GMinus, P_A, P_AQ, 4367)
      call createCurrent(P_Hl, P_GPlus, P_Hl, P_HMinus, 6859)
      call createCurrent(P_ghZAnti, P_HMinus, P_G0, P_ghWp, 9517)
      call createCurrent(P_Ha, P_HPlus, P_HMinus, P_Hh, 9958)
      call createCurrent(P_GQPlus, P_Hh, P_G0, P_HQMinus, 6979)
      call createCurrent(P_Ha, P_HMinus, P_Z, P_WPlus, 4760)
      call createCurrent(P_HQPlus, P_tauMinus, P_nu_tauAnti, 7315)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, 1811)
      call createCurrent(P_HPlus, P_HMinus, P_ghZAnti, P_ghA, 4852)
      call createCurrent(P_ghWp, P_Ha, P_ghWpAnti, P_HhQ, 4936)
      call createCurrent(P_HaQ, P_WPlus, P_Hl, P_WQMinus, 1615)
      call createCurrent(P_WPlus, P_WMinus, P_HPlus, P_HMinus, 7389)
      call createCurrent(P_WQMinus, P_WPlus, P_G0, P_HhQ, 5366)
      call createCurrent(P_HaQ, P_GPlus, P_HMinus, P_HhQ, 7668)
      call createCurrent(P_nu_e, P_ePlus, P_GMinus, 2695)
      call createCurrent(P_HhQ, P_HMinus, P_HPlus, P_HlQ, 715)
      call createCurrent(P_Ha, P_GPlus, P_ghWm, P_ghAAnti, 10451)
      call createCurrent(P_A, P_GMinus, P_Ha, P_WPlus, 9906)
      call createCurrent(P_WQPlus, P_ghWm, P_WMinus, P_ghWmAnti, 8447)
      call createCurrent(P_sAnti, P_Ha, P_s, 897)
      call createCurrent(P_Hl, P_HMinus, P_WPlus, 3476)
      call createCurrent(P_Ha, P_HMinus, P_ghWmAnti, P_ghA, 1934)
      call createCurrent(P_ghWpAnti, P_HPlus, P_Hh, P_ghZ, 7519)
      call createCurrent(P_g, P_g, P_Ha, P_Ha, 10677)
      call createCurrent(P_g, P_Z, P_A, P_g, 10604)
      call createCurrent(P_ghWp, P_Hh, P_ghZAnti, P_GMinus, 4927)
      call createCurrent(P_GMinus, P_ghWp, P_Ha, P_ghZAnti, 8906)
      call createCurrent(P_tauPlus, P_nu_tau, P_GMinus, 7543)
      call createCurrent(P_Hh, P_HPlus, P_ghWm, P_ghAAnti, 6038)
      call createCurrent(P_b, P_g, P_bAnti, 2525)
      call createCurrent(P_HPlus, P_Hl, P_Hh, P_GMinus, 7295)
      call createCurrent(P_ghA, P_GPlus, P_Hh, P_ghWpAnti, 6185)
      call createCurrent(P_WQMinus, P_Z, P_WPlus, P_AQ, 6887)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghA, P_Hh, 5837)
      call createCurrent(P_ghG, P_g, P_ghGAnti, P_gQ, 3931)
      call createCurrent(P_ePlus, P_G0, P_eMinus, 8086)
      call createCurrent(P_HPlus, P_HMinus, P_G0, P_Hl, 10230)
      call createCurrent(P_Ha, P_ghAAnti, P_GPlus, P_ghWm, 10124)
      call createCurrent(P_GMinus, P_G0, P_ghAAnti, P_ghWp, 575)
      call createCurrent(P_WMinus, P_WPlus, P_Hh, 5187)
      call createCurrent(P_GQPlus, P_GMinus, P_Hh, P_GQ0, 270)
      call createCurrent(P_GQMinus, P_GPlus, P_Hl, P_HlQ, 1907)
      call createCurrent(P_ZQ, P_WMinus, P_Hh, P_HQPlus, 1987)
      call createCurrent(P_g, P_Hh, P_Hl, P_g, 10535)
      call createCurrent(P_GMinus, P_ghWp, P_GPlus, P_ghWpAnti, 3304)
      call createCurrent(P_G0, P_GMinus, P_HPlus, 6727)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghZ, P_Hh, 4152)
      call createCurrent(P_G0, P_HPlus, P_ghWpAnti, P_ghA, 9600)
      call createCurrent(P_ghZAnti, P_GPlus, P_Ha, P_ghWm, 10259)
      call createCurrent(P_HMinus, P_G0, P_GPlus, P_Hl, 253)
      call createCurrent(P_HQMinus, P_WPlus, P_AQ, 8565)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hh, P_Hl, 9265)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_Hl, 5537)
      call createCurrent(P_Hl, P_g, P_g, P_Hl, 10532)
      call createCurrent(P_ghWpAnti, P_HPlus, P_Ha, P_ghZ, 7020)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_G0, 9834)
      call createCurrent(P_ZQ, P_WPlus, P_WQMinus, 1127)
      call createCurrent(P_WPlus, P_WMinus, P_HMinus, P_HPlus, 1092)
      call createCurrent(P_WPlus, P_Z, P_HMinus, P_Hl, 5344)
      call createCurrent(P_WQPlus, P_uAnti, P_d, 2633)
      call createCurrent(P_A, P_Hh, P_Hl, P_Z, 10744)
      call createCurrent(P_WPlus, P_Z, P_ghWm, P_ghAAnti, 5464)
      call createCurrent(P_HQPlus, P_ghA, P_Ha, P_ghWpAnti, 8160)
      call createCurrent(P_GQMinus, P_HPlus, P_Hh, P_HhQ, 1007)
      call createCurrent(P_G0, P_Hl, P_HMinus, P_GPlus, 7786)
      call createCurrent(P_ghWm, P_ghWmAnti, P_G0, P_GQ0, 2092)
      call createCurrent(P_GQMinus, P_GPlus, P_ghWpAnti, P_ghWp, 2956)
      call createCurrent(P_HaQ, P_Hl, P_Hl, P_GQ0, 627)
      call createCurrent(P_WMinus, P_GPlus, P_Ha, P_Z, 9985)
      call createCurrent(P_ZQ, P_ghAAnti, P_WMinus, P_ghWp, 1832)
      call createCurrent(P_ghZAnti, P_ghZ, P_WPlus, P_WQMinus, 1213)
      call createCurrent(P_AQ, P_WPlus, P_GQMinus, 6602)
      call createCurrent(P_WMinus, P_ghWpAnti, P_WPlus, P_ghWp, 6638)
      call createCurrent(P_WPlus, P_HMinus, P_G0, 9611)
      call createCurrent(P_GMinus, P_g, P_GPlus, P_g, 10687)
      call createCurrent(P_GQPlus, P_GMinus, P_ghZ, P_ghAAnti, 8654)
      call createCurrent(P_HaQ, P_b, P_bAnti, 5113)
      call createCurrent(P_Hh, P_GPlus, P_ghWm, P_ghZAnti, 300)
      call createCurrent(P_ghZ, P_HPlus, P_HMinus, P_ghZAnti, 6388)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghA, P_GQMinus, 7642)
      call createCurrent(P_HhQ, P_HMinus, P_GPlus, P_HaQ, 352)
      call createCurrent(P_WPlus, P_ghWmAnti, P_ghWm, P_WMinus, 5289)
      call createCurrent(P_WQPlus, P_tauMinus, P_nu_tauAnti, 2375)
      call createCurrent(P_Hl, P_HPlus, P_GMinus, P_Hl, 5335)
      call createCurrent(P_HlQ, P_ghZ, P_Hl, P_ghZAnti, 2008)
      call createCurrent(P_GMinus, P_t, P_bAnti, 1306)
      call createCurrent(P_HhQ, P_b, P_bAnti, 2394)
      call createCurrent(P_WQPlus, P_HMinus, P_A, P_HlQ, 671)
      call createCurrent(P_GQPlus, P_Hh, P_Ha, P_HQMinus, 1564)
      call createCurrent(P_HhQ, P_GPlus, P_G0, P_GQMinus, 6779)
      call createCurrent(P_HQMinus, P_ghA, P_HPlus, P_ghZAnti, 5284)
      call createCurrent(P_HQPlus, P_HMinus, P_Ha, P_HhQ, 1335)
      call createCurrent(P_A, P_ghZ, P_WPlus, P_ghWpAnti, 2144)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghWp, P_Hh, 1874)
      call createCurrent(P_HlQ, P_HPlus, P_Ha, P_HQMinus, 6925)
      call createCurrent(P_HlQ, P_ghWm, P_G0, P_ghWmAnti, 5706)
      call createCurrent(P_WQPlus, P_Ha, P_WMinus, P_HhQ, 4525)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hl, P_GQ0, 924)
      call createCurrent(P_ghWmAnti, P_ghWm, P_A, P_A, 339)
      call createCurrent(P_A, P_ghWp, P_ghWpAnti, P_A, 3911)
      call createCurrent(P_ghZAnti, P_ghZ, P_G0, P_G0, 5241)
      call createCurrent(P_ghWp, P_ghAAnti, P_WMinus, P_A, 1129)
      call createCurrent(P_HlQ, P_ghZ, P_HMinus, P_ghWmAnti, 6323)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghWp, P_HhQ, 1549)
      call createCurrent(P_Z, P_Z, P_A, P_A, 10710)
      call createCurrent(P_HQMinus, P_GMinus, P_HPlus, P_GQPlus, 3472)
      call createCurrent(P_WMinus, P_GPlus, P_G0, 6742)
      call createCurrent(P_HQMinus, P_A, P_WPlus, P_HaQ, 2325)
      call createCurrent(P_WPlus, P_WMinus, P_ghZ, P_ghZAnti, 2670)
      call createCurrent(P_HQPlus, P_Hh, P_WMinus, P_ZQ, 6051)
      call createCurrent(P_GPlus, P_Ha, P_HMinus, P_Hh, 937)
      call createCurrent(P_HMinus, P_GPlus, P_ghA, P_ghAAnti, 9469)
      call createCurrent(P_GQPlus, P_GMinus, P_ghWmAnti, P_ghWm, 3887)
      call createCurrent(P_GMinus, P_HPlus, P_Hl, 1680)
      call createCurrent(P_WMinus, P_G0, P_GPlus, P_Z, 5325)
      call createCurrent(P_ghWp, P_ghWpAnti, P_A, P_Z, 4908)
      call createCurrent(P_GMinus, P_Hh, P_A, P_WPlus, 1294)
      call createCurrent(P_AQ, P_WPlus, P_Hl, P_HQMinus, 3712)
      call createCurrent(P_A, P_HMinus, P_HPlus, P_A, 6367)
      call createCurrent(P_GPlus, P_HMinus, P_A, 10441)
      call createCurrent(P_Hh, P_ghA, P_HPlus, P_ghWpAnti, 6983)
      call createCurrent(P_GQPlus, P_WMinus, P_AQ, 1882)
      call createCurrent(P_ghA, P_ghAAnti, P_GMinus, P_GQPlus, 2495)
      call createCurrent(P_g, P_ghGAnti, P_ghG, 4777)
      call createCurrent(P_G0, P_G0, P_g, P_g, 10634)
      call createCurrent(P_Hh, P_A, P_GPlus, P_WMinus, 82)
      call createCurrent(P_ghZAnti, P_GMinus, P_G0, P_ghWp, 7049)
      call createCurrent(P_Hh, P_muPlus, P_muMinus, 3440)
      call createCurrent(P_GQPlus, P_Hl, P_GQMinus, 4382)
      call createCurrent(P_HPlus, P_GMinus, P_A, P_Z, 9354)
      call createCurrent(P_HlQ, P_GPlus, P_WMinus, P_ZQ, 2708)
      call createCurrent(P_Hh, P_WMinus, P_A, P_HPlus, 6603)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_HlQ, 4141)
      call createCurrent(P_ghWp, P_WMinus, P_WPlus, P_ghWpAnti, 142)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_G0, 2856)
      call createCurrent(P_Hh, P_t, P_tAnti, 8630)
      call createCurrent(P_Z, P_Z, P_ghWpAnti, P_ghWp, 6657)
      call createCurrent(P_ZQ, P_A, P_GPlus, P_GQMinus, 1033)
      call createCurrent(P_G0, P_Z, P_Z, P_G0, 6725)
      call createCurrent(P_ePlus, P_nu_e, P_GQMinus, 1671)
      call createCurrent(P_ghAAnti, P_WMinus, P_Z, P_ghWp, 254)
      call createCurrent(P_GQMinus, P_G0, P_HPlus, P_GQ0, 863)
      call createCurrent(P_HMinus, P_Hh, P_ghAAnti, P_ghWp, 1338)
      call createCurrent(P_Hl, P_HMinus, P_HPlus, P_G0, 9141)
      call createCurrent(P_HQPlus, P_ghZ, P_Hh, P_ghWpAnti, 2277)
      call createCurrent(P_WMinus, P_ghWp, P_ghAAnti, P_Z, 2102)
      call createCurrent(P_HaQ, P_Hh, P_GQ0, 1928)
      call createCurrent(P_GPlus, P_Z, P_Hl, P_WMinus, 4423)
      call createCurrent(P_ghZAnti, P_WMinus, P_A, P_ghWp, 109)
      call createCurrent(P_GMinus, P_ghWp, P_Hl, P_ghAAnti, 3186)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hh, P_HaQ, 1032)
      call createCurrent(P_ghWmAnti, P_G0, P_ghA, P_GQMinus, 843)
      call createCurrent(P_ghWm, P_GPlus, P_Ha, P_ghAAnti, 10486)
      call createCurrent(P_Z, P_ghWpAnti, P_WPlus, P_ghA, 4194)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghWp, P_WQMinus, 7739)
      call createCurrent(P_WQMinus, P_Z, P_HPlus, P_HhQ, 5559)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_Ha, 9513)
      call createCurrent(P_GMinus, P_ghWm, P_GPlus, P_ghWmAnti, 6985)
      call createCurrent(P_ghA, P_WMinus, P_Z, P_ghWmAnti, 4740)
      call createCurrent(P_AQ, P_WPlus, P_ghWm, P_ghZAnti, 6738)
      call createCurrent(P_ghWpAnti, P_ghWp, P_G0, P_HlQ, 2188)
      call createCurrent(P_GMinus, P_HPlus, P_Hl, P_Hh, 1673)
      call createCurrent(P_ghWmAnti, P_Z, P_ghA, P_WQMinus, 7591)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghWm, 3023)
      call createCurrent(P_Ha, P_ghA, P_GMinus, P_ghWmAnti, 10412)
      call createCurrent(P_sAnti, P_s, P_GQ0, 6636)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_HaQ, 2862)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_Ha, P_ghA, 6926)
      call createCurrent(P_GMinus, P_Hl, P_Hl, P_HPlus, 2259)
      call createCurrent(P_GQMinus, P_Hl, P_ghZ, P_ghWmAnti, 5044)
      call createCurrent(P_GQPlus, P_GMinus, P_ghAAnti, P_ghZ, 4075)
      call createCurrent(P_ghWmAnti, P_Hl, P_HMinus, P_ghA, 4718)
      call createCurrent(P_GQMinus, P_HPlus, P_HaQ, 4573)
      call createCurrent(P_Hl, P_ghZ, P_ghWmAnti, P_HMinus, 1694)
      call createCurrent(P_GPlus, P_GMinus, P_Hl, P_Hl, 1427)
      call createCurrent(P_HPlus, P_ghWm, P_Ha, P_ghAAnti, 5939)
      call createCurrent(P_HPlus, P_Hh, P_ghAAnti, P_ghWm, 7833)
      call createCurrent(P_ghZ, P_Hh, P_ghWpAnti, P_GQPlus, 4867)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghZ, P_GMinus, 5158)
      call createCurrent(P_HQPlus, P_Ha, P_G0, P_HQMinus, 3434)
      call createCurrent(P_HQMinus, P_Hh, P_Hh, P_HQPlus, 1149)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Ha, P_Ha, 1027)
      call createCurrent(P_Ha, P_ghWpAnti, P_ghWp, P_G0, 9711)
      call createCurrent(P_WPlus, P_HPlus, P_WMinus, P_HMinus, 161)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_Hh, 7500)
      call createCurrent(P_WPlus, P_GPlus, P_GMinus, P_WMinus, 3248)
      call createCurrent(P_AQ, P_Hh, P_HPlus, P_WQMinus, 4965)
      call createCurrent(P_HQPlus, P_ghA, P_HMinus, P_ghZAnti, 5283)
      call createCurrent(P_WQMinus, P_WPlus, P_ghZAnti, P_ghA, 8362)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_G0, 711)
      call createCurrent(P_WMinus, P_Z, P_Hl, P_HPlus, 8216)
      call createCurrent(P_Z, P_GPlus, P_HMinus, 9749)
      call createCurrent(P_ghA, P_Hl, P_ghWpAnti, P_GQPlus, 291)
      call createCurrent(P_GPlus, P_Hl, P_GMinus, 7410)
      call createCurrent(P_HQPlus, P_Hl, P_Z, P_WQMinus, 6728)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghZ, P_HPlus, 9050)
      call createCurrent(P_Hl, P_WMinus, P_A, P_GPlus, 4874)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_Hh, 596)
      call createCurrent(P_WQMinus, P_HMinus, P_WPlus, P_HQPlus, 1822)
      call createCurrent(P_HQPlus, P_Z, P_WMinus, P_HaQ, 1619)
      call createCurrent(P_ghWpAnti, P_ghA, P_A, P_WPlus, 7673)
      call createCurrent(P_Z, P_Z, P_GPlus, P_HMinus, 9756)
      call createCurrent(P_AQ, P_WPlus, P_WMinus, P_AQ, 1151)
      call createCurrent(P_WMinus, P_A, P_GPlus, P_Ha, 10194)
      call createCurrent(P_ghZ, P_G0, P_ghWpAnti, P_GPlus, 381)
      call createCurrent(P_HPlus, P_G0, P_Ha, P_GMinus, 4150)
      call createCurrent(P_ghAAnti, P_ghA, P_WPlus, P_WMinus, 8284)
      call createCurrent(P_G0, P_ghWpAnti, P_Ha, P_ghWp, 9978)
      call createCurrent(P_ghA, P_ghWpAnti, P_GPlus, P_Hh, 8157)
      call createCurrent(P_HPlus, P_GMinus, P_GPlus, P_HMinus, 6250)
      call createCurrent(P_HlQ, P_Ha, P_GMinus, P_HQPlus, 4347)
      call createCurrent(P_ghZAnti, P_ghZ, P_Ha, P_HaQ, 7247)
      call createCurrent(P_GMinus, P_ghWp, P_G0, P_ghAAnti, 191)
      call createCurrent(P_ghWp, P_G0, P_HMinus, P_ghZAnti, 10260)
      call createCurrent(P_HPlus, P_GMinus, P_Hh, P_Hh, 990)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_Ha, 6182)
      call createCurrent(P_HaQ, P_HPlus, P_GMinus, P_HlQ, 1285)
      call createCurrent(P_Z, P_GPlus, P_Ha, P_WMinus, 10212)
      call createCurrent(P_Hl, P_G0, P_G0, P_Hh, 2282)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghZ, P_HPlus, 2153)
      call createCurrent(P_muPlus, P_muMinus, P_HhQ, 4234)
      call createCurrent(P_sAnti, P_g, P_s, 2772)
      call createCurrent(P_HMinus, P_Hh, P_Hh, P_HPlus, 3191)
      call createCurrent(P_A, P_ePlus, P_eMinus, 8119)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghZ, P_GQMinus, 7570)
      call createCurrent(P_HMinus, P_G0, P_Ha, P_GPlus, 7609)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_G0, 4161)
      call createCurrent(P_A, P_GMinus, P_A, P_HPlus, 9885)
      call createCurrent(P_ZQ, P_GPlus, P_WMinus, P_HhQ, 4044)
      call createCurrent(P_ghWm, P_ghWpAnti, P_WPlus, P_WQPlus, 1669)
      call createCurrent(P_AQ, P_HMinus, P_HQPlus, 7107)
      call createCurrent(P_GQMinus, P_Ha, P_GPlus, P_HhQ, 775)
      call createCurrent(P_ghAAnti, P_GPlus, P_HMinus, P_ghA, 9579)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghA, P_G0, 10315)
      call createCurrent(P_ghWp, P_G0, P_ghWpAnti, P_HlQ, 398)
      call createCurrent(P_GPlus, P_HMinus, P_HMinus, P_HPlus, 8545)
      call createCurrent(P_ZQ, P_s, P_sAnti, 7845)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_HhQ, 1044)
      call createCurrent(P_tauMinus, P_nu_tauAnti, P_WPlus, 4923)
      call createCurrent(P_G0, P_Hl, P_HMinus, P_HPlus, 9256)
      call createCurrent(P_ghWmAnti, P_WPlus, P_WMinus, P_ghWm, 1705)
      call createCurrent(P_WMinus, P_dAnti, P_u, 5692)
      call createCurrent(P_G0, P_G0, 9490)
      call createCurrent(P_ghWm, P_Ha, P_ghWmAnti, P_G0, 9158)
      call createCurrent(P_ghZ, P_Ha, P_HMinus, P_ghWmAnti, 1601)
      call createCurrent(P_WPlus, P_G0, P_G0, P_WMinus, 6410)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghA, 5069)
      call createCurrent(P_GPlus, P_ghA, P_ghAAnti, P_HMinus, 9477)
      call createCurrent(P_Z, P_WPlus, P_A, P_WMinus, 3652)
      call createCurrent(P_GQPlus, P_ghZAnti, P_ghWm, 8500)
      call createCurrent(P_nu_muAnti, P_muMinus, P_GQPlus, 4530)
      call createCurrent(P_GQPlus, P_ghA, P_ghWpAnti, 7130)
      call createCurrent(P_Z, P_Hh, P_WPlus, P_GMinus, 7815)
      call createCurrent(P_HlQ, P_Ha, P_G0, P_HlQ, 7795)
      call createCurrent(P_ghWm, P_ghAAnti, P_WPlus, P_A, 4795)
      call createCurrent(P_GMinus, P_Hh, P_WPlus, 2162)
      call createCurrent(P_nu_muAnti, P_nu_mu, P_A, 9989)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghA, P_Hl, 1347)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghZ, P_GMinus, 9070)
      call createCurrent(P_G0, P_G0, P_GMinus, P_HPlus, 1644)
      call createCurrent(P_ghZ, P_ghWmAnti, P_WMinus, P_Z, 5315)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghZ, P_G0, 10410)
      call createCurrent(P_Hl, P_ghWm, P_GPlus, P_ghAAnti, 6520)
      call createCurrent(P_HaQ, P_Hl, P_GQ0, 3669)
      call createCurrent(P_Hl, P_ghZAnti, P_ghWm, P_GPlus, 6530)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghA, P_G0, 9725)
      call createCurrent(P_HlQ, P_ghWmAnti, P_HMinus, P_ghZ, 8065)
      call createCurrent(P_ghWmAnti, P_ghWm, P_GMinus, P_HPlus, 8995)
      call createCurrent(P_HQMinus, P_Ha, P_ghWmAnti, P_ghA, 6948)
      call createCurrent(P_s, P_cAnti, P_HPlus, 5927)
      call createCurrent(P_ghZ, P_ghZAnti, P_HPlus, P_HQMinus, 2848)
      call createCurrent(P_Hl, P_HMinus, P_Ha, P_HPlus, 9558)
      call createCurrent(P_dAnti, P_d, P_Hl, 8422)
      call createCurrent(P_HMinus, P_HPlus, P_ghWmAnti, P_ghWm, 7876)
      call createCurrent(P_HaQ, P_G0, P_GPlus, P_GQMinus, 3698)
      call createCurrent(P_nu_eAnti, P_WPlus, P_eMinus, 7522)
      call createCurrent(P_HMinus, P_Ha, P_HPlus, P_Hh, 8857)
      call createCurrent(P_HQMinus, P_ghA, P_ghWmAnti, 3541)
      call createCurrent(P_uAnti, P_u, P_Hl, 4571)
      call createCurrent(P_ghZ, P_GPlus, P_ghAAnti, P_GMinus, 1720)
      call createCurrent(P_HlQ, P_Z, P_HaQ, 2235)
      call createCurrent(P_A, P_tauPlus, P_tauMinus, 7351)
      call createCurrent(P_Ha, P_Z, P_Z, P_Ha, 7974)
      call createCurrent(P_HPlus, P_ghWm, P_ghZAnti, P_Hl, 8629)
      call createCurrent(P_Hh, P_ghZAnti, P_ghWm, P_HPlus, 5318)
      call createCurrent(P_A, P_Hh, P_A, 10767)
      call createCurrent(P_Z, P_WPlus, P_WMinus, P_Z, 1703)
      call createCurrent(P_HQMinus, P_GPlus, P_HMinus, P_HQPlus, 4279)
      call createCurrent(P_G0, P_GPlus, P_Hl, P_HMinus, 7712)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_HlQ, 280)
      call createCurrent(P_ghGAnti, P_ghG, P_g, P_gQ, 6167)
      call createCurrent(P_GPlus, P_Hl, P_A, P_WMinus, 3567)
      call createCurrent(P_HaQ, P_ghZAnti, P_Ha, P_ghZ, 1005)
      call createCurrent(P_GMinus, P_G0, P_GPlus, P_Hh, 9447)
      call createCurrent(P_GQMinus, P_G0, P_Hl, P_HQPlus, 8339)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghA, P_WQPlus, 2343)
      call createCurrent(P_A, P_Ha, P_Ha, P_A, 10733)
      call createCurrent(P_ghZAnti, P_ghWp, P_WMinus, P_AQ, 440)
      call createCurrent(P_GMinus, P_ghWp, P_ghWpAnti, P_HPlus, 9572)
      call createCurrent(P_WQPlus, P_A, P_WMinus, P_ZQ, 8703)
      call createCurrent(P_WQMinus, P_Hl, P_Hl, P_WQPlus, 6058)
      call createCurrent(P_GQ0, P_Ha, P_GMinus, P_GQPlus, 3808)
      call createCurrent(P_GPlus, P_Hh, P_ghAAnti, P_ghWm, 2480)
      call createCurrent(P_Hh, P_cAnti, P_c, 8021)
      call createCurrent(P_HQMinus, P_Hl, P_GPlus, P_HhQ, 2319)
      call createCurrent(P_Ha, P_G0, P_ghZAnti, P_ghZ, 10203)
      call createCurrent(P_AQ, P_Z, P_WPlus, P_WQMinus, 1991)
      call createCurrent(P_t, P_Hl, P_tAnti, 8766)
      call createCurrent(P_ghWpAnti, P_ghA, P_A, P_WQPlus, 6581)
      call createCurrent(P_HMinus, P_ghZ, P_Ha, P_ghWmAnti, 1775)
      call createCurrent(P_G0, P_HPlus, P_GMinus, P_G0, 1379)
      call createCurrent(P_WMinus, P_GMinus, P_GPlus, P_WPlus, 7858)
      call createCurrent(P_HQPlus, P_ghZAnti, P_ghWm, 7780)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_ghZ, 4913)
      call createCurrent(P_HlQ, P_G0, P_ZQ, 2728)
      call createCurrent(P_HhQ, P_Hl, P_GPlus, P_GQMinus, 4282)
      call createCurrent(P_GPlus, P_Hh, P_WMinus, P_Z, 7382)
      call createCurrent(P_HPlus, P_Hh, P_Hl, P_GMinus, 4048)
      call createCurrent(P_GQMinus, P_Hh, P_HPlus, P_HaQ, 886)
      call createCurrent(P_HMinus, P_Z, P_HPlus, 7663)
      call createCurrent(P_A, P_G0, P_Z, P_G0, 9975)
      call createCurrent(P_ghA, P_GPlus, P_ghAAnti, P_HMinus, 9118)
      call createCurrent(P_Hh, P_Hl, P_Hl, P_Hl, 4868)
      call createCurrent(P_GMinus, P_HPlus, P_ghZAnti, P_ghZ, 9761)
      call createCurrent(P_Z, P_GMinus, P_HPlus, P_A, 9307)
      call createCurrent(P_HlQ, P_HPlus, P_Hh, P_HQMinus, 7729)
      call createCurrent(P_HlQ, P_Hl, P_HPlus, P_HQMinus, 6907)
      call createCurrent(P_HPlus, P_Hh, P_GMinus, P_G0, 7150)
      call createCurrent(P_HhQ, P_GPlus, P_Ha, P_GQMinus, 6591)
      call createCurrent(P_GQMinus, P_HPlus, P_GMinus, P_GQPlus, 6098)
      call createCurrent(P_HMinus, P_WPlus, P_Hh, 4350)
      call createCurrent(P_Z, P_WMinus, P_GPlus, P_Hh, 4926)
      call createCurrent(P_A, P_G0, P_Hh, 9467)
      call createCurrent(P_GMinus, P_GPlus, P_ghA, P_ghAAnti, 8165)
      call createCurrent(P_ghWpAnti, P_ghZ, P_WPlus, P_A, 1217)
      call createCurrent(P_WQPlus, P_ghWm, P_WPlus, P_ghWpAnti, 3647)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hl, P_GPlus, 3291)
      call createCurrent(P_HQMinus, P_Hl, P_ghWmAnti, P_ghA, 7169)
      call createCurrent(P_GMinus, P_ghZ, P_Hh, P_ghWmAnti, 4728)
      call createCurrent(P_Hh, P_ghA, P_ghWpAnti, P_HPlus, 3496)
      call createCurrent(P_nu_tauAnti, P_nu_tau, P_Z, 194)
      call createCurrent(P_ghA, P_GMinus, P_ghWmAnti, P_Ha, 10076)
      call createCurrent(P_Z, P_ghZ, P_ghWpAnti, P_WPlus, 7629)
      call createCurrent(P_Z, P_uAnti, P_u, 7722)
      call createCurrent(P_GPlus, P_GMinus, P_A, P_Z, 1311)
      call createCurrent(P_ghWm, P_A, P_ghAAnti, P_WPlus, 7895)
      call createCurrent(P_Ha, P_GMinus, P_GPlus, P_Ha, 7929)
      call createCurrent(P_GQMinus, P_GMinus, P_HPlus, P_HQPlus, 6935)
      call createCurrent(P_muMinus, P_GPlus, P_nu_muAnti, 5452)
      call createCurrent(P_HMinus, P_ghZAnti, P_Hh, P_ghWp, 8773)
      call createCurrent(P_Ha, P_ghZ, P_ghZAnti, P_G0, 9659)
      call createCurrent(P_ZQ, P_GMinus, P_GPlus, P_ZQ, 6969)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghZ, P_Ha, 10465)
      call createCurrent(P_dAnti, P_g, P_d, 2980)
      call createCurrent(P_Ha, P_dAnti, P_d, 5165)
      call createCurrent(P_Hl, P_ghAAnti, P_HMinus, P_ghWp, 2069)
      call createCurrent(P_HPlus, P_nu_tauAnti, P_tauMinus, 4074)
      call createCurrent(P_WPlus, P_ghAAnti, P_A, P_ghWm, 2150)
      call createCurrent(P_WMinus, P_G0, P_G0, P_WPlus, 3553)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_HlQ, 7278)
      call createCurrent(P_ghAAnti, P_ghA, P_GPlus, P_GQMinus, 1119)
      call createCurrent(P_ghA, P_G0, P_ghWpAnti, P_HPlus, 9938)
      call createCurrent(P_GQ0, P_G0, P_GPlus, P_HQMinus, 7636)
      call createCurrent(P_Hl, P_HPlus, P_Z, P_WMinus, 2241)
      call createCurrent(P_HQPlus, P_Hh, P_WQMinus, 6559)
      call createCurrent(P_G0, P_A, P_Hh, 10129)
      call createCurrent(P_WPlus, P_Hl, P_Hl, P_WMinus, 1812)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_HhQ, 5342)
      call createCurrent(P_GMinus, P_HMinus, P_GPlus, P_GPlus, 7717)
      call createCurrent(P_ghWmAnti, P_ghWm, P_ZQ, 3105)
      call createCurrent(P_nu_mu, P_HMinus, P_muPlus, 5568)
      call createCurrent(P_Z, P_HPlus, P_Ha, P_WMinus, 5236)
      call createCurrent(P_ghWpAnti, P_G0, P_ghWp, P_HhQ, 3749)
      call createCurrent(P_ghAAnti, P_HPlus, P_GMinus, P_ghA, 9813)
      call createCurrent(P_GQMinus, P_HPlus, P_GMinus, P_HQPlus, 83)
      call createCurrent(P_ghWp, P_WMinus, P_ghZAnti, P_AQ, 6297)
      call createCurrent(P_HMinus, P_Ha, P_Hh, P_HPlus, 9866)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, 6838)
      call createCurrent(P_HPlus, P_Hh, P_A, P_WMinus, 5686)
      call createCurrent(P_HhQ, P_Hh, P_HlQ, 6623)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghWm, P_A, 4080)
      call createCurrent(P_WPlus, P_HPlus, P_HMinus, P_WMinus, 7853)
      call createCurrent(P_WMinus, P_ghZ, P_ghWmAnti, P_A, 2610)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_HhQ, 6758)
      call createCurrent(P_HlQ, P_Ha, P_GMinus, P_GQPlus, 3821)
      call createCurrent(P_HlQ, P_GPlus, P_ghWpAnti, P_ghZ, 4638)
      call createCurrent(P_GQMinus, P_Ha, P_HPlus, P_HhQ, 3131)
      call createCurrent(P_ghZ, P_A, P_WPlus, P_ghWpAnti, 5462)
      call createCurrent(P_HQPlus, P_WMinus, P_Z, P_HhQ, 5415)
      call createCurrent(P_GPlus, P_ghZ, P_ghZAnti, P_GMinus, 7409)
      call createCurrent(P_GQ0, P_d, P_dAnti, 7450)
      call createCurrent(P_GPlus, P_HMinus, P_HPlus, P_HMinus, 8184)
      call createCurrent(P_HMinus, P_WMinus, P_GPlus, P_WPlus, 9504)
      call createCurrent(P_Hl, P_Z, P_G0, 8026)
      call createCurrent(P_GMinus, P_HPlus, P_G0, P_G0, 6904)
      call createCurrent(P_HPlus, P_HPlus, P_HMinus, P_HMinus, 5682)
      call createCurrent(P_Hh, P_GMinus, P_HPlus, P_Hh, 7540)
      call createCurrent(P_WQMinus, P_WPlus, P_A, P_AQ, 7842)
      call createCurrent(P_HMinus, P_Hl, P_Hl, P_HPlus, 5723)
      call createCurrent(P_Hh, P_GMinus, P_Hh, P_HPlus, 5641)
      call createCurrent(P_ghA, P_HPlus, P_Hl, P_ghWpAnti, 8240)
      call createCurrent(P_HMinus, P_Z, P_GPlus, 9807)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_HlQ, 224)
      call createCurrent(P_sAnti, P_A, P_s, 1366)
      call createCurrent(P_HlQ, P_GPlus, P_G0, P_HQMinus, 1742)
      call createCurrent(P_HMinus, P_GPlus, P_A, P_A, 9862)
      call createCurrent(P_WMinus, P_GPlus, P_WPlus, P_GMinus, 7698)
      call createCurrent(P_A, P_ghWmAnti, P_Z, P_ghWm, 1133)
      call createCurrent(P_WQMinus, P_Z, P_WQPlus, 838)
      call createCurrent(P_ghZ, P_Ha, P_ghZAnti, P_Ha, 2535)
      call createCurrent(P_GPlus, P_Ha, P_ghAAnti, P_ghWm, 8916)
      call createCurrent(P_HaQ, P_ghWpAnti, P_ghWp, 7178)
      call createCurrent(P_GMinus, P_WPlus, P_A, P_Hh, 5240)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_Ha, 8420)
      call createCurrent(P_AQ, P_G0, P_WMinus, P_GQPlus, 1589)
      call createCurrent(P_G0, P_A, P_A, P_G0, 10494)
      call createCurrent(P_WQPlus, P_WMinus, P_Ha, P_HlQ, 481)
      call createCurrent(P_GMinus, P_Z, P_WPlus, P_Ha, 9856)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_HhQ, 8742)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghZ, P_GQMinus, 8786)
      call createCurrent(P_HlQ, P_ghWmAnti, P_G0, P_ghWm, 7677)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghWp, 7615)
      call createCurrent(P_Ha, P_Hh, P_Hl, P_Ha, 2216)
      call createCurrent(P_GMinus, P_ghWpAnti, P_ghWp, P_GPlus, 3942)
      call createCurrent(P_Z, P_nu_mu, P_nu_muAnti, 7486)
      call createCurrent(P_HMinus, P_ghA, P_Hh, P_ghWmAnti, 7114)
      call createCurrent(P_uAnti, P_u, P_g, 4853)
      call createCurrent(P_WMinus, P_Hh, P_HPlus, P_A, 7148)
      call createCurrent(P_HQMinus, P_G0, P_HPlus, P_HhQ, 5545)
      call createCurrent(P_ghWmAnti, P_G0, P_ghWm, P_G0, 8372)
      call createCurrent(P_A, P_Ha, P_G0, P_Z, 10609)
      call createCurrent(P_g, P_ghG, P_g, P_ghGAnti, 2702)
      call createCurrent(P_Hh, P_Hh, P_GPlus, P_HMinus, 6152)
      call createCurrent(P_ZQ, P_GMinus, P_GPlus, P_AQ, 757)
      call createCurrent(P_HMinus, P_WPlus, P_Ha, 7096)
      call createCurrent(P_HhQ, P_GMinus, P_G0, P_GQPlus, 952)
      call createCurrent(P_ghZAnti, P_ghZ, P_HMinus, P_HQPlus, 102)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghA, P_Hl, 4102)
      call createCurrent(P_HaQ, P_WMinus, P_WQPlus, 8109)
      call createCurrent(P_HQMinus, P_GPlus, P_HPlus, P_HQMinus, 486)
      call createCurrent(P_GPlus, P_Hl, P_HMinus, P_G0, 758)
      call createCurrent(P_A, P_A, P_GPlus, P_HMinus, 9520)
      call createCurrent(P_HMinus, P_ghZ, P_ghZAnti, P_GPlus, 9926)
      call createCurrent(P_Z, P_Hh, P_GPlus, P_WMinus, 2915)
      call createCurrent(P_ghAAnti, P_GMinus, P_Hl, P_ghWp, 849)
      call createCurrent(P_Ha, P_GMinus, P_Z, P_WPlus, 10346)
      call createCurrent(P_HlQ, P_G0, P_HMinus, P_HQPlus, 2135)
      call createCurrent(P_G0, P_Hh, P_G0, 1332)
      call createCurrent(P_WPlus, P_HMinus, P_Hl, 7460)
      call createCurrent(P_AQ, P_A, P_WMinus, P_WQPlus, 3086)
      call createCurrent(P_WPlus, P_ghWm, P_A, P_ghAAnti, 5628)
      call createCurrent(P_WPlus, P_GMinus, P_WMinus, P_GPlus, 4494)
      call createCurrent(P_Hl, P_A, P_Hh, P_Z, 10515)
      call createCurrent(P_HhQ, P_ghWm, P_GPlus, P_ghAAnti, 3079)
      call createCurrent(P_Ha, P_GPlus, P_GMinus, P_Hh, 9934)
      call createCurrent(P_Ha, P_ghWmAnti, P_ghZ, P_HMinus, 6617)
      call createCurrent(P_ZQ, P_HMinus, P_WQPlus, 7784)
      call createCurrent(P_GPlus, P_HMinus, P_G0, P_G0, 4475)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_Hh, 3880)
      call createCurrent(P_Hl, P_Ha, P_GMinus, P_HPlus, 6930)
      call createCurrent(P_Ha, P_HPlus, P_Hh, P_HMinus, 9610)
      call createCurrent(P_GQMinus, P_HPlus, P_Hl, P_HaQ, 2132)
      call createCurrent(P_Hh, P_Hh, P_ghZAnti, P_ghZ, 5731)
      call createCurrent(P_ZQ, P_WMinus, P_HPlus, P_HlQ, 1768)
      call createCurrent(P_Ha, P_ghWm, P_GPlus, P_ghZAnti, 9474)
      call createCurrent(P_ghZAnti, P_ghZ, P_HlQ, 8008)
      call createCurrent(P_Z, P_Hl, P_Z, P_Hh, 10406)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghA, P_WMinus, 5667)
      call createCurrent(P_GMinus, P_HPlus, P_Ha, P_G0, 6618)
      call createCurrent(P_WMinus, P_A, P_HPlus, P_Ha, 6682)
      call createCurrent(P_G0, P_ghA, P_ghWmAnti, P_GMinus, 5522)
      call createCurrent(P_ghWmAnti, P_ghA, P_GMinus, P_Hh, 7329)
      call createCurrent(P_HaQ, P_Ha, P_Hh, P_HlQ, 4148)
      call createCurrent(P_HPlus, P_WMinus, P_Hh, 6890)
      call createCurrent(P_ghWmAnti, P_ghWm, P_A, P_AQ, 698)
      call createCurrent(P_HMinus, P_ghZ, P_ghWmAnti, 5169)
      call createCurrent(P_Hh, P_ghZAnti, P_ghZ, 8061)
      call createCurrent(P_HPlus, P_ghZ, P_Hh, P_ghWpAnti, 1391)
      call createCurrent(P_Hh, P_Hh, P_WPlus, P_WMinus, 6899)
      call createCurrent(P_G0, P_Z, P_WPlus, P_GMinus, 8713)
      call createCurrent(P_GMinus, P_G0, P_HPlus, P_Hl, 2465)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_Hl, 7763)
      call createCurrent(P_HPlus, P_ghWpAnti, P_Ha, P_ghZ, 5694)
      call createCurrent(P_HlQ, P_HMinus, P_ghZ, P_ghWmAnti, 6120)
      call createCurrent(P_HlQ, P_Z, P_HPlus, P_WQMinus, 6901)
      call createCurrent(P_HQPlus, P_Ha, P_ghWpAnti, P_ghA, 6524)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghZ, P_HMinus, 7909)
      call createCurrent(P_GQMinus, P_Hl, P_GPlus, P_HhQ, 8439)
      call createCurrent(P_WMinus, P_Z, P_ghA, P_ghWmAnti, 4018)
      call createCurrent(P_WQPlus, P_WMinus, P_WMinus, P_WQPlus, 5469)
      call createCurrent(P_ghAAnti, P_ghWp, P_G0, P_HMinus, 9261)
      call createCurrent(P_s, P_sAnti, P_GQ0, 527)
      call createCurrent(P_HQPlus, P_G0, P_HMinus, P_HlQ, 800)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_Hh, 1441)
      call createCurrent(P_Z, P_Hh, P_Hl, P_A, 10780)
      call createCurrent(P_HPlus, P_HMinus, P_ghWm, P_ghWmAnti, 4024)
      call createCurrent(P_ghWp, P_Z, P_ghZAnti, P_WMinus, 466)
      call createCurrent(P_Hl, P_Z, P_GMinus, P_WPlus, 5445)
      call createCurrent(P_ghWp, P_Hh, P_GMinus, P_ghZAnti, 2243)
      call createCurrent(P_HQMinus, P_HPlus, P_Hl, P_HaQ, 2929)
      call createCurrent(P_HaQ, P_Hl, P_GPlus, P_GQMinus, 3174)
      call createCurrent(P_GPlus, P_HMinus, P_Ha, P_Hl, 5805)
      call createCurrent(P_WMinus, P_G0, P_A, P_GPlus, 8826)
      call createCurrent(P_A, P_ghA, P_WPlus, P_ghWpAnti, 936)
      call createCurrent(P_HQMinus, P_HPlus, P_G0, P_HaQ, 5711)
      call createCurrent(P_GQMinus, P_G0, P_ghZ, P_ghWmAnti, 2078)
      call createCurrent(P_ghWm, P_WPlus, P_ghZAnti, P_Z, 8129)
      call createCurrent(P_HQPlus, P_Ha, P_Ha, P_HQMinus, 3207)
      call createCurrent(P_Hh, P_A, P_WPlus, P_HMinus, 8767)
      call createCurrent(P_HQMinus, P_Z, P_WPlus, P_HhQ, 7699)
      call createCurrent(P_ZQ, P_A, P_ghWpAnti, P_ghWp, 2141)
      call createCurrent(P_HPlus, P_Z, P_Z, P_GMinus, 9423)
      call createCurrent(P_HlQ, P_GMinus, P_Z, P_WQPlus, 4121)
      call createCurrent(P_uAnti, P_GPlus, P_d, 1048)
      call createCurrent(P_HlQ, P_Hh, P_GMinus, P_HQPlus, 5245)
      call createCurrent(P_ZQ, P_Hl, P_WPlus, P_HQMinus, 2739)
      call createCurrent(P_HPlus, P_ghA, P_ghAAnti, P_HMinus, 6010)
      call createCurrent(P_GMinus, P_Hl, P_Z, P_WPlus, 6578)
      call createCurrent(P_Ha, P_Ha, P_GPlus, P_HMinus, 2990)
      call createCurrent(P_Hh, P_Hl, P_A, P_A, 10603)
      call createCurrent(P_d, P_dAnti, P_HhQ, 192)
      call createCurrent(P_WQMinus, P_HPlus, P_A, P_HlQ, 1658)
      call createCurrent(P_HMinus, P_GPlus, P_Ha, P_Ha, 3761)
      call createCurrent(P_tauMinus, P_tauPlus, P_ZQ, 5094)
      call createCurrent(P_HaQ, P_HMinus, P_GPlus, P_HhQ, 4624)
      call createCurrent(P_HhQ, P_Hh, P_WPlus, P_WQMinus, 3592)
      call createCurrent(P_nu_muAnti, P_muMinus, P_HQPlus, 1990)
      call createCurrent(P_GQ0, P_Ha, P_HhQ, 1718)
      call createCurrent(P_HhQ, P_ghZ, P_Hh, P_ghZAnti, 3666)
      call createCurrent(P_G0, P_GPlus, P_Ha, P_HMinus, 5471)
      call createCurrent(P_Hl, P_Z, P_Z, P_Hh, 8858)
      call createCurrent(P_Hh, P_ghWp, P_ghWpAnti, P_Hh, 938)
      call createCurrent(P_A, P_Ha, P_GPlus, P_WMinus, 9416)
      call createCurrent(P_Z, P_nu_e, P_nu_eAnti, 2669)
      call createCurrent(P_ghWmAnti, P_Z, P_WMinus, P_ghA, 1856)
      call createCurrent(P_ghWp, P_Ha, P_ghWpAnti, P_HlQ, 8720)
      call createCurrent(P_HQPlus, P_Hh, P_HMinus, P_HaQ, 6122)
      call createCurrent(P_GQ0, P_G0, P_ghWp, P_ghWpAnti, 2119)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghWp, P_WMinus, 5830)
      call createCurrent(P_Ha, P_Z, P_G0, P_Z, 9720)
      call createCurrent(P_GQ0, P_GMinus, P_HPlus, P_HlQ, 6207)
      call createCurrent(P_GPlus, P_Ha, P_GMinus, 9079)
      call createCurrent(P_ZQ, P_Ha, P_HhQ, 5615)
      call createCurrent(P_Ha, P_HPlus, P_G0, P_GMinus, 54)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_Hh, 7215)
      call createCurrent(P_G0, P_GPlus, P_ghWm, P_ghAAnti, 4180)
      call createCurrent(P_HaQ, P_WMinus, P_Ha, P_WQPlus, 4012)
      call createCurrent(P_Hl, P_HMinus, P_A, P_WPlus, 8269)
      call createCurrent(P_muMinus, P_nu_muAnti, P_HQPlus, 5127)
      call createCurrent(P_WQPlus, P_HPlus, P_WMinus, P_HQMinus, 8417)
      call createCurrent(P_ghZ, P_HPlus, P_GMinus, P_ghAAnti, 8898)
      call createCurrent(P_HaQ, P_HMinus, P_ghZ, P_ghWmAnti, 3198)
      call createCurrent(P_ZQ, P_WMinus, P_GPlus, P_GQ0, 570)
      call createCurrent(P_ghWmAnti, P_GPlus, P_HMinus, P_ghWm, 10181)
      call createCurrent(P_u, P_uAnti, P_A, 413)
      call createCurrent(P_HlQ, P_ghWp, P_GMinus, P_ghZAnti, 7764)
      call createCurrent(P_GQMinus, P_GPlus, P_ghA, P_ghZAnti, 8529)
      call createCurrent(P_HMinus, P_GPlus, P_G0, P_Hl, 681)
      call createCurrent(P_A, P_Z, P_Hl, P_Hl, 9863)
      call createCurrent(P_HQMinus, P_GPlus, P_Ha, P_HlQ, 5121)
      call createCurrent(P_Z, P_GMinus, P_Hh, P_WPlus, 565)
      call createCurrent(P_Hh, P_ghWm, P_Hh, P_ghWmAnti, 176)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_G0, 2701)
      call createCurrent(P_A, P_ghWmAnti, P_ghWm, P_A, 810)
      call createCurrent(P_Hh, P_Ha, P_Ha, P_Hl, 3606)
      call createCurrent(P_cAnti, P_s, P_WQPlus, 8604)
      call createCurrent(P_ghWm, P_GMinus, P_ghWmAnti, P_GQPlus, 3947)
      call createCurrent(P_WPlus, P_G0, P_WMinus, P_Ha, 10215)
      call createCurrent(P_ghG, P_ghGAnti, P_gQ, 8566)
      call createCurrent(P_HMinus, P_WPlus, 9341)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HQMinus, 205)
      call createCurrent(P_AQ, P_Hh, P_WPlus, P_GQMinus, 780)
      call createCurrent(P_ghWpAnti, P_HMinus, P_ghWp, P_HPlus, 6784)
      call createCurrent(P_WQMinus, P_A, P_WQPlus, 6366)
      call createCurrent(P_GQMinus, P_WPlus, P_ZQ, 4675)
      call createCurrent(P_GQMinus, P_G0, P_ghWmAnti, P_ghA, 3330)
      call createCurrent(P_GPlus, P_G0, P_GMinus, P_Hl, 9135)
      call createCurrent(P_HlQ, P_G0, P_Hh, P_HaQ, 1845)
      call createCurrent(P_HPlus, P_HMinus, P_Hh, P_Hh, 7704)
      call createCurrent(P_g, P_GPlus, P_g, P_HMinus, 10513)
      call createCurrent(P_Ha, P_G0, P_GPlus, P_HMinus, 8655)
      call createCurrent(P_HPlus, P_WMinus, P_Ha, 3021)
      call createCurrent(P_G0, P_Hl, P_GPlus, P_GMinus, 10106)
      call createCurrent(P_G0, P_G0, P_ghWp, P_ghWpAnti, 4438)
      call createCurrent(P_WPlus, P_WMinus, 10369)
      call createCurrent(P_GQPlus, P_Ha, P_HMinus, P_GQ0, 6661)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_HhQ, 3628)
      call createCurrent(P_G0, P_GPlus, P_GMinus, P_Hh, 9186)
      call createCurrent(P_HQPlus, P_G0, P_HMinus, P_HaQ, 5181)
      call createCurrent(P_HhQ, P_Hh, P_G0, P_GQ0, 7733)
      call createCurrent(P_muPlus, P_Hh, P_muMinus, 8734)
      call createCurrent(P_c, P_cAnti, P_HaQ, 3598)
      call createCurrent(P_GMinus, P_Hl, P_Ha, P_HPlus, 2079)
      call createCurrent(P_WPlus, P_GMinus, P_A, P_Ha, 9528)
      call createCurrent(P_GQMinus, P_GPlus, P_HhQ, 7478)
      call createCurrent(P_ghZAnti, P_ghWp, P_WMinus, P_ZQ, 7070)
      call createCurrent(P_WPlus, P_Z, P_A, P_WMinus, 2699)
      call createCurrent(P_Hl, P_GMinus, P_Ha, P_HPlus, 1461)
      call createCurrent(P_Ha, P_ghWm, P_Ha, P_ghWmAnti, 1908)
      call createCurrent(P_HMinus, P_WPlus, P_A, P_G0, 10048)
      call createCurrent(P_WQPlus, P_Hl, P_HMinus, P_AQ, 4296)
      call createCurrent(P_HlQ, P_HMinus, P_WPlus, P_ZQ, 8331)
      call createCurrent(P_HhQ, P_Ha, P_ghWp, P_ghWpAnti, 4884)
      call createCurrent(P_Hl, P_Hl, P_Hh, P_Hl, 2618)
      call createCurrent(P_HMinus, P_GPlus, P_WMinus, P_WPlus, 9688)
      call createCurrent(P_s, P_WPlus, P_cAnti, 5141)
      call createCurrent(P_WQPlus, P_WMinus, P_Hl, P_HlQ, 4189)
      call createCurrent(P_ghZAnti, P_ghA, P_HPlus, P_HQMinus, 2916)
      call createCurrent(P_HPlus, P_Hh, P_Hh, P_HMinus, 8275)
      call createCurrent(P_GPlus, P_Z, P_WMinus, P_Hl, 5227)
      call createCurrent(P_dAnti, P_WMinus, P_u, 1276)
      call createCurrent(P_Hh, P_g, P_g, P_Hh, 10555)
      call createCurrent(P_HMinus, P_GPlus, P_ghZ, P_ghAAnti, 9107)
      call createCurrent(P_GQPlus, P_ghZ, P_Hl, P_ghWpAnti, 260)
      call createCurrent(P_Z, P_WPlus, P_GMinus, P_Hh, 2120)
      call createCurrent(P_ghZ, P_HMinus, P_HPlus, P_ghAAnti, 5225)
      call createCurrent(P_G0, P_ghWpAnti, P_ghA, P_GPlus, 6519)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hh, P_HMinus, 2408)
      call createCurrent(P_GQ0, P_ePlus, P_eMinus, 6674)
      call createCurrent(P_Ha, P_A, P_A, P_Ha, 10536)
      call createCurrent(P_HQPlus, P_Hl, P_ghA, P_ghWpAnti, 4848)
      call createCurrent(P_GQPlus, P_Ha, P_GMinus, P_HaQ, 8195)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Ha, P_HaQ, 7800)
      call createCurrent(P_Ha, P_Ha, 9333)
      call createCurrent(P_Ha, P_G0, 9999)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghWm, P_Hh, 116)
      call createCurrent(P_WMinus, P_Hl, P_Z, P_HPlus, 1665)
      call createCurrent(P_sAnti, P_s, P_gQ, 8369)
      call createCurrent(P_Ha, P_GMinus, P_Hh, P_GPlus, 10141)
      call createCurrent(P_HMinus, P_Hh, P_HPlus, P_Hh, 6305)
      call createCurrent(P_GMinus, P_A, P_G0, P_WPlus, 2977)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghWm, P_Hl, 686)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_ghA, 3344)
      call createCurrent(P_WPlus, P_G0, P_Ha, P_WMinus, 9072)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghA, P_HPlus, 8996)
      call createCurrent(P_HQMinus, P_WMinus, P_HPlus, P_WQPlus, 3800)
      call createCurrent(P_Hh, P_Z, P_GPlus, P_WMinus, 8456)
      call createCurrent(P_GMinus, P_HPlus, P_G0, P_Hl, 1292)
      call createCurrent(P_WMinus, P_HPlus, P_G0, 9541)
      call createCurrent(P_HQPlus, P_WMinus, P_Z, P_HaQ, 3220)
      call createCurrent(P_AQ, P_WMinus, P_Hl, P_GQPlus, 3136)
      call createCurrent(P_WQPlus, P_Hh, P_Z, P_HQMinus, 7332)
      call createCurrent(P_HQMinus, P_HPlus, P_G0, P_GQ0, 8523)
      call createCurrent(P_A, P_Z, P_WMinus, P_WPlus, 7275)
      call createCurrent(P_Hl, P_Ha, P_Hh, P_Ha, 1855)
      call createCurrent(P_GQPlus, P_HMinus, P_Hh, P_HlQ, 659)
      call createCurrent(P_WQMinus, P_HPlus, P_HlQ, 3139)
      call createCurrent(P_HQMinus, P_GPlus, P_Ha, P_HhQ, 8329)
      call createCurrent(P_WMinus, P_A, P_WPlus, P_Z, 1976)
      call createCurrent(P_Ha, P_Hl, P_Hh, P_G0, 871)
      call createCurrent(P_HMinus, P_HPlus, P_A, P_A, 8709)
      call createCurrent(P_Hl, P_G0, P_GPlus, P_HMinus, 3315)
      call createCurrent(P_GQPlus, P_Hh, P_Hl, P_GQMinus, 7603)
      call createCurrent(P_Hl, P_A, P_WPlus, P_HMinus, 3557)
      call createCurrent(P_HQMinus, P_GMinus, P_GPlus, P_HQPlus, 4009)
      call createCurrent(P_ghWm, P_ghZAnti, P_GQPlus, 7065)
      call createCurrent(P_G0, P_Z, P_G0, P_Z, 7154)
      call createCurrent(P_Ha, P_Ha, P_g, P_g, 10620)
      call createCurrent(P_G0, P_HPlus, P_GMinus, P_Hh, 4371)
      call createCurrent(P_HQMinus, P_HMinus, P_GPlus, P_HQPlus, 4461)
      call createCurrent(P_Hh, P_Hl, P_Hh, P_Hl, 1176)
      call createCurrent(P_HhQ, P_d, P_dAnti, 7286)
      call createCurrent(P_nu_mu, P_nu_muAnti, 9642)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_Hh, 3812)
      call createCurrent(P_GMinus, P_A, P_WPlus, 477)
      call createCurrent(P_GMinus, P_WMinus, P_WPlus, P_HPlus, 9920)
      call createCurrent(P_g, P_Z, P_g, P_g, 10528)
      call createCurrent(P_Hl, P_GMinus, P_ghWmAnti, P_ghA, 4041)
      call createCurrent(P_u, P_uAnti, P_Z, 3050)
      call createCurrent(P_GPlus, P_GMinus, P_Hh, P_G0, 10074)
      call createCurrent(P_ghZ, P_Ha, P_G0, P_ghZAnti, 9617)
      call createCurrent(P_HPlus, P_WMinus, P_A, 5897)
      call createCurrent(P_GQPlus, P_HPlus, P_HMinus, P_GQMinus, 3638)
      call createCurrent(P_nu_eAnti, P_nu_e, P_Z, 7390)
      call createCurrent(P_HPlus, P_ghZ, P_ghZAnti, P_GMinus, 10077)
      call createCurrent(P_GQPlus, P_cAnti, P_s, 435)
      call createCurrent(P_HMinus, P_ghWmAnti, P_G0, P_ghZ, 9289)
      call createCurrent(P_Hh, P_c, P_cAnti, 2399)
      call createCurrent(P_GQ0, P_GMinus, P_ghZAnti, P_ghWp, 6256)
      call createCurrent(P_GPlus, P_G0, P_ghZ, P_ghWpAnti, 2601)
      call createCurrent(P_A, P_b, P_bAnti, 1683)
      call createCurrent(P_WMinus, P_ghWp, P_Z, P_ghZAnti, 6871)
      call createCurrent(P_G0, P_HMinus, P_GPlus, 2521)
      call createCurrent(P_HQMinus, P_HPlus, P_Z, P_ZQ, 3507)
      call createCurrent(P_ghA, P_ghZAnti, P_WPlus, P_WQMinus, 8398)
      call createCurrent(P_Z, P_Hl, P_Ha, 106)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghA, P_GMinus, 665)
      call createCurrent(P_HPlus, P_GMinus, P_G0, P_G0, 1915)
      call createCurrent(P_cAnti, P_c, 8953)
      call createCurrent(P_HaQ, P_GMinus, P_HQPlus, 2561)
      call createCurrent(P_ghWpAnti, P_Z, P_Z, P_ghWp, 8304)
      call createCurrent(P_WQPlus, P_ghZ, P_WMinus, P_ghZAnti, 1587)
      call createCurrent(P_HhQ, P_GMinus, P_Hh, P_GQPlus, 7561)
      call createCurrent(P_ZQ, P_Hh, P_WMinus, P_GQPlus, 4285)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_G0, 777)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hh, P_HPlus, 4232)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_GQ0, 8264)
      call createCurrent(P_HPlus, P_ghWm, P_ghAAnti, P_G0, 10245)
      call createCurrent(P_ghWm, P_ghAAnti, P_Ha, P_GPlus, 9411)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghWp, P_HQMinus, 6529)
      call createCurrent(P_u, P_uAnti, P_ZQ, 515)
      call createCurrent(P_g, P_GPlus, P_GMinus, P_g, 10685)
      call createCurrent(P_WQPlus, P_s, P_cAnti, 6452)
      call createCurrent(P_A, P_HMinus, P_Ha, P_WPlus, 8776)
      call createCurrent(P_Hh, P_ghWpAnti, P_Hh, P_ghWp, 3900)
      call createCurrent(P_HPlus, P_A, P_GMinus, P_Z, 10162)
      call createCurrent(P_Ha, P_Hh, P_HMinus, P_HPlus, 9802)
      call createCurrent(P_GMinus, P_ghA, P_ghWmAnti, P_G0, 4101)
      call createCurrent(P_HPlus, P_WMinus, P_Z, P_Hh, 1766)
      call createCurrent(P_A, P_Z, P_ghWpAnti, P_ghWp, 6075)
      call createCurrent(P_WQMinus, P_ghA, P_WPlus, P_ghZAnti, 8391)
      call createCurrent(P_HPlus, P_WMinus, P_Hl, P_A, 6467)
      call createCurrent(P_HMinus, P_HMinus, P_HPlus, P_HPlus, 8360)
      call createCurrent(P_WPlus, P_A, P_Ha, P_HMinus, 2533)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_G0, 5987)
      call createCurrent(P_AQ, P_Z, P_ghWp, P_ghWpAnti, 4302)
      call createCurrent(P_Hl, P_ghWm, P_Hh, P_ghWmAnti, 9590)
      call createCurrent(P_ghAAnti, P_ghZ, P_GMinus, P_GPlus, 8834)
      call createCurrent(P_HlQ, P_bAnti, P_b, 7688)
      call createCurrent(P_G0, P_HMinus, P_Hl, P_HPlus, 10464)
      call createCurrent(P_ghWm, P_Hl, P_ghZAnti, P_HQPlus, 618)
      call createCurrent(P_HQMinus, P_Hl, P_GPlus, P_HaQ, 5961)
      call createCurrent(P_A, P_G0, P_G0, P_Z, 9169)
      call createCurrent(P_WPlus, P_WMinus, P_ghWp, P_ghWpAnti, 1762)
      call createCurrent(P_WQPlus, P_Ha, P_HMinus, P_AQ, 1881)
      call createCurrent(P_HMinus, P_ghWpAnti, P_GPlus, P_ghWp, 9953)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hh, P_Hh, 5735)
      call createCurrent(P_ghZAnti, P_Ha, P_G0, P_ghZ, 9073)
      call createCurrent(P_G0, P_Ha, P_Hl, P_G0, 10545)
      call createCurrent(P_WQPlus, P_G0, P_GMinus, P_AQ, 8650)
      call createCurrent(P_HlQ, P_Ha, P_WMinus, P_WQPlus, 1439)
      call createCurrent(P_ghWp, P_A, P_ghWpAnti, P_Z, 7953)
      call createCurrent(P_ghWpAnti, P_A, P_WPlus, P_ghZ, 8112)
      call createCurrent(P_Hh, P_A, P_Hh, P_Z, 9117)
      call createCurrent(P_A, P_HMinus, P_WPlus, P_Hh, 7753)
      call createCurrent(P_HMinus, P_Hh, P_WPlus, 6687)
      call createCurrent(P_Hh, P_WMinus, P_HPlus, P_Z, 8374)
      call createCurrent(P_g, P_g, P_GMinus, P_HPlus, 10498)
      call createCurrent(P_HaQ, P_HPlus, P_ghWpAnti, P_ghZ, 5492)
      call createCurrent(P_HMinus, P_nu_e, P_ePlus, 7905)
      call createCurrent(P_Ha, P_G0, P_Z, P_Z, 9445)
      call createCurrent(P_HQPlus, P_Ha, P_WMinus, P_AQ, 8509)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_HlQ, 252)
      call createCurrent(P_HQPlus, P_Hl, P_HQMinus, 4934)
      call createCurrent(P_GQPlus, P_ghZAnti, P_GMinus, P_ghA, 794)
      call createCurrent(P_HMinus, P_u, P_dAnti, 238)
      call createCurrent(P_G0, P_Hh, P_Hl, P_Ha, 4956)
      call createCurrent(P_ZQ, P_WPlus, P_HMinus, P_HlQ, 3966)
      call createCurrent(P_WQPlus, P_GMinus, P_ZQ, 569)
      call createCurrent(P_ghAAnti, P_HPlus, P_HMinus, P_ghZ, 1551)
      call createCurrent(P_Hl, P_dAnti, P_d, 4894)
      call createCurrent(P_ghAAnti, P_Hl, P_ghWp, P_HMinus, 8433)
      call createCurrent(P_Z, P_GMinus, P_Z, P_GPlus, 3396)
      call createCurrent(P_ZQ, P_ghWmAnti, P_Z, P_ghWm, 4158)
      call createCurrent(P_d, P_dAnti, P_G0, 5448)
      call createCurrent(P_WMinus, P_Z, P_ghZ, P_ghWmAnti, 2663)
      call createCurrent(P_HMinus, P_WPlus, P_Ha, P_A, 1755)
      call createCurrent(P_HQMinus, P_GPlus, P_Hh, P_HhQ, 235)
      call createCurrent(P_Hl, P_HMinus, P_G0, P_HPlus, 9703)
      call createCurrent(P_HaQ, P_HPlus, P_GMinus, P_HhQ, 6403)
      call createCurrent(P_WQMinus, P_WPlus, P_Hh, P_GQ0, 6757)
      call createCurrent(P_HPlus, P_Ha, P_GMinus, 3511)
      call createCurrent(P_HMinus, P_GPlus, P_ghAAnti, P_ghA, 9387)
      call createCurrent(P_HPlus, P_Hh, P_WMinus, P_A, 8591)
      call createCurrent(P_ghAAnti, P_ghWm, P_Hh, P_HPlus, 7997)
      call createCurrent(P_GPlus, P_GMinus, P_WPlus, P_WMinus, 4809)
      call createCurrent(P_HPlus, P_HMinus, P_G0, 8929)
      call createCurrent(P_GQ0, P_ghAAnti, P_GPlus, P_ghWm, 1480)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_HhQ, 1923)
      call createCurrent(P_Hl, P_HPlus, P_ghZ, P_ghWpAnti, 3271)
      call createCurrent(P_GPlus, P_Hl, P_ghA, P_ghWpAnti, 6273)
      call createCurrent(P_Z, P_eMinus, P_ePlus, 8272)
      call createCurrent(P_Hl, P_GMinus, P_HPlus, P_Ha, 5207)
      call createCurrent(P_ghZ, P_ghZAnti, P_WPlus, P_WMinus, 4218)
      call createCurrent(P_ghWm, P_Z, P_WPlus, P_ghZAnti, 7121)
      call createCurrent(P_GQMinus, P_Hl, P_G0, P_HQPlus, 1729)
      call createCurrent(P_HMinus, P_Z, P_GPlus, P_Z, 9511)
      call createCurrent(P_ghWmAnti, P_ghA, P_Z, P_WQMinus, 4789)
      call createCurrent(P_Hh, P_Ha, P_HMinus, P_GPlus, 2814)
      call createCurrent(P_G0, P_Hh, P_HMinus, P_GPlus, 2540)
      call createCurrent(P_GQMinus, P_Ha, P_GPlus, P_HlQ, 4388)
      call createCurrent(P_WMinus, P_Hh, P_Hh, P_WPlus, 5214)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghZ, P_GQPlus, 3684)
      call createCurrent(P_ghZ, P_WMinus, P_ghAAnti, P_WPlus, 8134)
      call createCurrent(P_HhQ, P_WPlus, P_G0, P_WQMinus, 4359)
      call createCurrent(P_Z, P_A, P_Z, P_Z, 10752)
      call createCurrent(P_Hh, P_GPlus, P_G0, P_HMinus, 6572)
      call createCurrent(P_GQ0, P_WMinus, P_Hh, P_WQPlus, 2175)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghA, P_G0, 573)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hh, P_HPlus, 7811)
      call createCurrent(P_GQ0, P_GPlus, P_HMinus, P_HhQ, 3539)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghZ, P_HMinus, 742)
      call createCurrent(P_GPlus, P_ghAAnti, P_HMinus, P_ghZ, 10005)
      call createCurrent(P_HQPlus, P_Ha, P_ghWpAnti, P_ghZ, 1496)
      call createCurrent(P_GMinus, P_WPlus, P_A, 5013)
      call createCurrent(P_ghGAnti, P_g, P_ghG, P_g, 4428)
      call createCurrent(P_ZQ, P_WPlus, P_HQMinus, 7379)
      call createCurrent(P_ghWm, P_Z, P_A, P_ghWmAnti, 1929)
      call createCurrent(P_tauMinus, P_nu_tauAnti, P_GQPlus, 7935)
      call createCurrent(P_AQ, P_A, P_GMinus, P_GQPlus, 1305)
      call createCurrent(P_G0, P_GPlus, P_WMinus, P_Z, 6288)
      call createCurrent(P_GPlus, P_cAnti, P_s, 7744)
      call createCurrent(P_A, P_Hl, P_Hl, P_A, 10565)
      call createCurrent(P_HMinus, P_ghWp, P_ghZAnti, P_Hl, 6013)
      call createCurrent(P_HlQ, P_GMinus, P_Hh, P_GQPlus, 7888)
      call createCurrent(P_GPlus, P_A, P_WMinus, P_Ha, 9976)
      call createCurrent(P_HPlus, P_G0, P_Ha, P_HMinus, 2234)
      call createCurrent(P_t, P_tAnti, P_gQ, 5106)
      call createCurrent(P_GQ0, P_Hl, P_ghWm, P_ghWmAnti, 762)
      call createCurrent(P_HQMinus, P_Hh, P_WPlus, P_AQ, 7162)
      call createCurrent(P_Hh, P_ghZ, P_ghWpAnti, P_GPlus, 2401)
      call createCurrent(P_GPlus, P_GMinus, P_Ha, 9779)
      call createCurrent(P_WMinus, P_ghZ, P_ghWmAnti, 7422)
      call createCurrent(P_Hh, P_Hl, P_ghWp, P_ghWpAnti, 9162)
      call createCurrent(P_GPlus, P_Ha, P_ghWpAnti, P_ghZ, 10376)
      call createCurrent(P_HQPlus, P_GMinus, P_Hl, P_GQ0, 2941)
      call createCurrent(P_HlQ, P_WPlus, P_A, P_GQMinus, 3384)
      call createCurrent(P_WPlus, P_Ha, P_GMinus, P_A, 9006)
      call createCurrent(P_ghA, P_ghZAnti, P_GPlus, P_GQMinus, 138)
      call createCurrent(P_HQPlus, P_G0, P_HMinus, P_HhQ, 5880)
      call createCurrent(P_nu_muAnti, P_nu_mu, P_ZQ, 6485)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_G0, P_ghZ, 4981)
      call createCurrent(P_WPlus, P_ghA, P_ghWpAnti, 2847)
      call createCurrent(P_HMinus, P_HPlus, P_GMinus, P_GPlus, 6309)
      call createCurrent(P_GQMinus, P_G0, P_Ha, P_HQPlus, 7248)
      call createCurrent(P_Z, P_WPlus, P_ghAAnti, P_ghWm, 1850)
      call createCurrent(P_GQMinus, P_Hh, P_G0, P_HQPlus, 3923)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_Hh, 4093)
      call createCurrent(P_WQMinus, P_Z, P_HQPlus, 7016)
      call createCurrent(P_HQMinus, P_Ha, P_HPlus, P_GQ0, 2593)
      call createCurrent(P_HQPlus, P_WMinus, P_HaQ, 2877)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghWm, P_WPlus, 6863)
      call createCurrent(P_HaQ, P_HMinus, P_A, P_WQPlus, 4518)
      call createCurrent(P_GPlus, P_ghWm, P_Ha, P_ghAAnti, 9058)
      call createCurrent(P_ghWm, P_Ha, P_ghWmAnti, P_HaQ, 4550)
      call createCurrent(P_ghZAnti, P_WMinus, P_WPlus, P_ghZ, 4164)
      call createCurrent(P_Hh, P_ghWm, P_GPlus, P_ghAAnti, 1112)
      call createCurrent(P_HPlus, P_HMinus, P_Hl, P_G0, 9984)
      call createCurrent(P_GQ0, P_Hl, P_HaQ, 2422)
      call createCurrent(P_HQMinus, P_HPlus, P_A, P_ZQ, 6206)
      call createCurrent(P_GMinus, P_GMinus, P_GPlus, P_HPlus, 2792)
      call createCurrent(P_HQPlus, P_WMinus, P_Ha, P_ZQ, 5569)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_GQ0, 479)
      call createCurrent(P_GQPlus, P_GMinus, P_ghZAnti, P_ghZ, 8702)
      call createCurrent(P_Hl, P_HPlus, P_HMinus, P_G0, 9635)
      call createCurrent(P_Hh, P_g, P_g, 10487)
      call createCurrent(P_HPlus, P_GMinus, P_Hl, 1446)
      call createCurrent(P_Hl, P_GPlus, P_Ha, P_GMinus, 9723)
      call createCurrent(P_GQMinus, P_Hh, P_Hl, P_GQPlus, 4766)
      call createCurrent(P_Z, P_Ha, P_GMinus, P_WPlus, 10294)
      call createCurrent(P_ghZ, P_ghWmAnti, P_G0, P_HMinus, 10156)
      call createCurrent(P_ghA, P_ghAAnti, P_HPlus, P_GMinus, 9656)
      call createCurrent(P_HaQ, P_Hl, P_HMinus, P_GQPlus, 7655)
      call createCurrent(P_Hl, P_G0, P_G0, 6407)
      call createCurrent(P_Hl, P_ghA, P_ghWpAnti, P_GPlus, 2227)
      call createCurrent(P_HMinus, P_Hl, P_ghZ, P_ghWmAnti, 5919)
      call createCurrent(P_Z, P_WMinus, P_Hh, P_HPlus, 852)
      call createCurrent(P_ghWpAnti, P_Z, P_ghWp, P_ZQ, 2327)
      call createCurrent(P_Hl, P_GMinus, P_GPlus, P_Hl, 8397)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HMinus, P_HPlus, 3083)
      call createCurrent(P_HhQ, P_dAnti, P_d, 4955)
      call createCurrent(P_ghWp, P_G0, P_ghWpAnti, P_Ha, 8852)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_Ha, 10198)
      call createCurrent(P_HQMinus, P_HPlus, P_HaQ, 6517)
      call createCurrent(P_WQMinus, P_Ha, P_WPlus, P_HhQ, 2289)
      call createCurrent(P_Ha, P_Hh, P_HPlus, P_HMinus, 9291)
      call createCurrent(P_HPlus, P_G0, P_GMinus, P_Hh, 1036)
      call createCurrent(P_ghWp, P_ghWpAnti, P_G0, P_HhQ, 468)
      call createCurrent(P_WPlus, P_G0, P_GMinus, 218)
      call createCurrent(P_HQMinus, P_Ha, P_WQPlus, 1490)
      call createCurrent(P_tauMinus, P_tauPlus, P_HlQ, 1029)
      call createCurrent(P_ZQ, P_Ha, P_WPlus, P_HQMinus, 406)
      call createCurrent(P_ghAAnti, P_ghWp, P_Hh, P_GMinus, 4951)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, 4876)
      call createCurrent(P_AQ, P_Hl, P_WPlus, P_GQMinus, 1897)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hl, P_HMinus, 6967)
      call createCurrent(P_HPlus, P_G0, P_ghWm, P_ghAAnti, 9108)
      call createCurrent(P_A, P_ghWm, P_Z, P_ghWmAnti, 753)
      call createCurrent(P_HMinus, P_Z, P_WPlus, P_G0, 10311)
      call createCurrent(P_GMinus, P_ghWmAnti, P_G0, P_ghA, 4465)
      call createCurrent(P_HaQ, P_HMinus, P_G0, P_GQPlus, 6767)
      call createCurrent(P_HPlus, P_Ha, P_A, P_WMinus, 7069)
      call createCurrent(P_A, P_nu_mu, P_nu_muAnti, 9790)
      call createCurrent(P_GMinus, P_ghZ, P_ghAAnti, P_HPlus, 9822)
      call createCurrent(P_ghWm, P_HPlus, P_G0, P_ghAAnti, 10079)
      call createCurrent(P_WMinus, P_GPlus, P_A, P_Hh, 378)
      call createCurrent(P_HhQ, P_Ha, P_GMinus, P_GQPlus, 2961)
      call createCurrent(P_GPlus, P_A, P_GMinus, P_A, 3876)
      call createCurrent(P_G0, P_G0, P_ghWmAnti, P_ghWm, 7761)
      call createCurrent(P_HaQ, P_GPlus, P_HQMinus, 6894)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghWp, P_G0, 9734)
      call createCurrent(P_Hl, P_HPlus, P_G0, P_GMinus, 1623)
      call createCurrent(P_Z, P_A, P_Hl, P_Hl, 9529)
      call createCurrent(P_ghZ, P_HMinus, P_ghAAnti, P_HQPlus, 520)
      call createCurrent(P_GPlus, P_ghZAnti, P_GMinus, P_ghZ, 738)
      call createCurrent(P_GPlus, P_Z, P_Z, P_GMinus, 8735)
      call createCurrent(P_HPlus, P_GMinus, P_g, P_g, 10662)
      call createCurrent(P_Ha, P_Ha, P_GMinus, P_GPlus, 6514)
      call createCurrent(P_WQPlus, P_WMinus, P_HaQ, 1340)
      call createCurrent(P_HhQ, P_Hh, P_HMinus, P_HQPlus, 2006)
      call createCurrent(P_d, P_WPlus, P_uAnti, 1158)
      call createCurrent(P_G0, P_G0, P_Ha, P_Hh, 10582)
      call createCurrent(P_ZQ, P_sAnti, P_s, 7510)
      call createCurrent(P_GMinus, P_ghAAnti, P_GPlus, P_ghZ, 1594)
      call createCurrent(P_muMinus, P_Hh, P_muPlus, 1089)
      call createCurrent(P_GPlus, P_Hl, P_G0, P_GMinus, 10090)
      call createCurrent(P_Ha, P_GPlus, P_HMinus, P_Hh, 554)
      call createCurrent(P_GMinus, P_WPlus, P_G0, P_Z, 4312)
      call createCurrent(P_b, P_bAnti, P_HlQ, 5587)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hh, P_HQPlus, 1071)
      call createCurrent(P_Hl, P_Hl, P_HPlus, P_GMinus, 4796)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghWp, P_HMinus, 9948)
      call createCurrent(P_HhQ, P_ghWmAnti, P_GMinus, P_ghZ, 850)
      call createCurrent(P_HPlus, P_G0, P_HMinus, P_Hh, 9270)
      call createCurrent(P_HPlus, P_ghZ, P_GMinus, P_ghAAnti, 9809)
      call createCurrent(P_G0, P_eMinus, P_ePlus, 221)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghZ, P_Hl, 8598)
      call createCurrent(P_Z, P_GMinus, P_Z, P_HPlus, 10403)
      call createCurrent(P_Ha, P_Hl, P_GPlus, P_HMinus, 5357)
      call createCurrent(P_GQ0, P_GMinus, P_A, P_WQPlus, 2647)
      call createCurrent(P_WPlus, P_G0, P_HMinus, 10165)
      call createCurrent(P_HhQ, P_GPlus, P_Hh, P_HQMinus, 521)
      call createCurrent(P_A, P_WMinus, P_G0, P_HPlus, 9293)
      call createCurrent(P_WPlus, P_Hh, P_Hh, P_WMinus, 2619)
      call createCurrent(P_G0, P_GMinus, P_GPlus, P_Ha, 6404)
      call createCurrent(P_WPlus, P_HMinus, P_Hl, P_A, 3816)
      call createCurrent(P_ghZAnti, P_Z, P_ghWm, P_WPlus, 3762)
      call createCurrent(P_G0, P_A, P_G0, P_A, 10552)
      call createCurrent(P_Hl, P_ghZ, P_GPlus, P_ghWpAnti, 8315)
      call createCurrent(P_HhQ, P_Ha, P_Hl, P_GQ0, 2021)
      call createCurrent(P_GMinus, P_HPlus, P_ghWmAnti, P_ghWm, 9280)
      call createCurrent(P_ZQ, P_A, P_ghWm, P_ghWmAnti, 5147)
      call createCurrent(P_Hl, P_ghWmAnti, P_ghA, P_HMinus, 6190)
      call createCurrent(P_WQMinus, P_Ha, P_Z, P_HQPlus, 8603)
      call createCurrent(P_HMinus, P_GPlus, P_WPlus, P_WMinus, 8868)
      call createCurrent(P_GQMinus, P_ghZ, P_GPlus, P_ghZAnti, 5324)
      call createCurrent(P_GQMinus, P_ghZ, P_Hl, P_ghWmAnti, 2307)
      call createCurrent(P_A, P_WPlus, P_GMinus, P_Hl, 7187)
      call createCurrent(P_Z, P_A, P_A, P_A, 10753)
      call createCurrent(P_G0, P_Z, P_HMinus, P_WPlus, 9840)
      call createCurrent(P_GPlus, P_HMinus, P_Hl, P_Hl, 7503)
      call createCurrent(P_ghWmAnti, P_HMinus, P_HPlus, P_ghWm, 3913)
      call createCurrent(P_ghWmAnti, P_GMinus, P_Hl, P_ghA, 992)
      call createCurrent(P_HPlus, P_ghA, P_ghWpAnti, P_Hh, 5800)
      call createCurrent(P_ZQ, P_WPlus, P_Z, P_WQMinus, 1189)
      call createCurrent(P_HlQ, P_ghWmAnti, P_GMinus, P_ghZ, 5145)
      call createCurrent(P_dAnti, P_d, P_ZQ, 1385)
      call createCurrent(P_ghWpAnti, P_Z, P_ghZ, P_WPlus, 6940)
      call createCurrent(P_Hh, P_ghWm, P_ghZAnti, P_GPlus, 3906)
      call createCurrent(P_Hh, P_HMinus, P_Ha, P_GPlus, 8290)
      call createCurrent(P_WQPlus, P_A, P_ghWpAnti, P_ghZ, 3687)
      call createCurrent(P_HMinus, P_g, P_g, P_HPlus, 10680)
      call createCurrent(P_ghA, P_ghAAnti, P_WMinus, P_WQPlus, 5341)
      call createCurrent(P_Z, P_A, P_G0, P_G0, 8938)
      call createCurrent(P_GMinus, P_A, P_A, P_HPlus, 10104)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_HaQ, 1736)
      call createCurrent(P_Hh, P_Hl, P_G0, P_Ha, 5832)
      call createCurrent(P_GQMinus, P_G0, P_G0, P_HQPlus, 8105)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_Ha, 9763)
      call createCurrent(P_HPlus, P_Z, P_GMinus, P_A, 9153)
      call createCurrent(P_WQMinus, P_Hl, P_GPlus, P_ZQ, 6861)
      call createCurrent(P_Hl, P_ghWm, P_ghWmAnti, P_Hl, 2110)
      call createCurrent(P_WPlus, P_Ha, P_A, P_HMinus, 837)
      call createCurrent(P_HlQ, P_HPlus, P_ghWm, P_ghAAnti, 3441)
      call createCurrent(P_WQPlus, P_HMinus, P_Z, P_HlQ, 4504)
      call createCurrent(P_tauMinus, P_WPlus, P_nu_tauAnti, 6302)
      call createCurrent(P_nu_mu, P_muPlus, P_GQMinus, 4663)
      call createCurrent(P_AQ, P_ePlus, P_eMinus, 5131)
      call createCurrent(P_g, P_g, P_Hl, 10671)
      call createCurrent(P_nu_e, P_nu_eAnti, P_Z, 225)
      call createCurrent(P_HlQ, P_Hh, P_G0, P_GQ0, 6241)
      call createCurrent(P_ghAAnti, P_ghWp, P_Hl, P_GMinus, 5146)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghWm, P_G0, 2900)
      call createCurrent(P_HPlus, P_Ha, P_ghWpAnti, P_ghZ, 7373)
      call createCurrent(P_Hh, P_Hh, P_Hl, 4662)
      call createCurrent(P_ghZAnti, P_ghA, P_HPlus, P_HMinus, 6582)
      call createCurrent(P_AQ, P_HPlus, P_HMinus, P_AQ, 5385)
      call createCurrent(P_ghWpAnti, P_ghA, P_GPlus, P_Ha, 9224)
      call createCurrent(P_ghA, P_ghAAnti, P_HMinus, P_HPlus, 4383)
      call createCurrent(P_ghWm, P_ghZAnti, P_G0, P_GPlus, 5526)
      call createCurrent(P_WQPlus, P_Hl, P_WMinus, P_HlQ, 8039)
      call createCurrent(P_Z, P_nu_tau, P_nu_tauAnti, 1203)
      call createCurrent(P_Hh, P_HMinus, P_ghA, P_ghWmAnti, 2751)
      call createCurrent(P_muMinus, P_muPlus, P_A, 5062)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_Hl, 7126)
      call createCurrent(P_GQPlus, P_HMinus, P_Hl, P_HlQ, 4858)
      call createCurrent(P_Z, P_WMinus, P_Hl, P_GPlus, 2160)
      call createCurrent(P_WQPlus, P_nu_tauAnti, P_tauMinus, 4244)
      call createCurrent(P_WQPlus, P_WMinus, P_ghZAnti, P_ghA, 1719)
      call createCurrent(P_HaQ, P_HPlus, P_HMinus, P_GQ0, 8370)
      call createCurrent(P_Ha, P_ghWpAnti, P_ghA, P_HPlus, 4072)
      call createCurrent(P_ghZ, P_GMinus, P_ghAAnti, P_HPlus, 8875)
      call createCurrent(P_GMinus, P_GPlus, P_HPlus, P_GMinus, 3487)
      call createCurrent(P_ghWm, P_Hl, P_ghWmAnti, P_HaQ, 7388)
      call createCurrent(P_ghZ, P_ghZAnti, P_Ha, P_G0, 10015)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_HaQ, 4555)
      call createCurrent(P_WPlus, P_Hh, P_WMinus, 1624)
      call createCurrent(P_HMinus, P_G0, P_ghZAnti, P_ghWp, 9481)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hl, 3570)
      call createCurrent(P_G0, P_A, P_WMinus, P_GPlus, 6502)
      call createCurrent(P_b, P_bAnti, P_G0, 2854)
      call createCurrent(P_HQMinus, P_Ha, P_G0, P_HQPlus, 6908)
      call createCurrent(P_GQ0, P_GPlus, P_Hh, P_GQMinus, 1660)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_Hh, 7145)
      call createCurrent(P_HMinus, P_Ha, P_ghAAnti, P_ghWp, 621)
      call createCurrent(P_HhQ, P_sAnti, P_s, 1704)
      call createCurrent(P_GQMinus, P_ghZAnti, P_ghWp, 8673)
      call createCurrent(P_HMinus, P_A, P_G0, P_WPlus, 9262)
      call createCurrent(P_G0, P_GPlus, P_HMinus, P_Ha, 4814)
      call createCurrent(P_Ha, P_Hh, P_A, 10299)
      call createCurrent(P_HhQ, P_GMinus, P_ghWmAnti, P_ghZ, 2258)
      call createCurrent(P_HMinus, P_Hl, P_GPlus, P_G0, 2139)
      call createCurrent(P_GQMinus, P_GPlus, P_ghA, P_ghAAnti, 3287)
      call createCurrent(P_HaQ, P_HPlus, P_Hl, P_HQMinus, 8815)
      call createCurrent(P_GQ0, P_c, P_cAnti, 1209)
      call createCurrent(P_ghWm, P_Hh, P_ghZAnti, P_HQPlus, 5014)
      call createCurrent(P_Z, P_HPlus, P_WMinus, P_G0, 9982)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghWp, P_Hl, 5557)
      call createCurrent(P_WQMinus, P_dAnti, P_u, 1020)
      call createCurrent(P_A, P_GMinus, P_WPlus, P_G0, 8628)
      call createCurrent(P_WPlus, P_WMinus, P_Ha, P_Ha, 7504)
      call createCurrent(P_ZQ, P_eMinus, P_ePlus, 5928)
      call createCurrent(P_ghWm, P_Ha, P_GPlus, P_ghAAnti, 10086)
      call createCurrent(P_ghZAnti, P_ghWm, P_HQPlus, 1779)
      call createCurrent(P_G0, P_HPlus, P_Hh, P_GMinus, 684)
      call createCurrent(P_ghWmAnti, P_G0, P_ghA, P_HMinus, 10002)
      call createCurrent(P_Hl, P_WMinus, P_HPlus, P_A, 4733)
      call createCurrent(P_WMinus, P_Hl, P_WPlus, 3581)
      call createCurrent(P_Ha, P_Hh, P_Hh, P_G0, 2109)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_Hl, 8212)
      call createCurrent(P_WPlus, P_b, P_tAnti, 4993)
      call createCurrent(P_HaQ, P_ghWp, P_Hl, P_ghWpAnti, 5535)
      call createCurrent(P_HhQ, P_ghWpAnti, P_Ha, P_ghWp, 5640)
      call createCurrent(P_ghA, P_GPlus, P_ghWpAnti, 7057)
      call createCurrent(P_GQMinus, P_sAnti, P_c, 7462)
      call createCurrent(P_GMinus, P_A, P_WPlus, P_Ha, 9597)
      call createCurrent(P_G0, P_WMinus, P_HPlus, P_Z, 10029)
      call createCurrent(P_HPlus, P_GMinus, P_ghA, P_ghAAnti, 10482)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghZ, P_GPlus, 10327)
      call createCurrent(P_Z, P_A, P_G0, P_Ha, 10777)
      call createCurrent(P_GPlus, P_Ha, P_GMinus, P_Ha, 1128)
      call createCurrent(P_WQPlus, P_Z, P_ghA, P_ghWpAnti, 3297)
      call createCurrent(P_GMinus, P_WMinus, P_HPlus, P_WPlus, 10442)
      call createCurrent(P_nu_muAnti, P_A, P_nu_mu, 10261)
      call createCurrent(P_HaQ, P_HPlus, P_HMinus, P_HaQ, 4755)
      call createCurrent(P_GQMinus, P_Hl, P_WQPlus, 4409)
      call createCurrent(P_WMinus, P_WPlus, P_G0, P_Ha, 9988)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghA, P_HPlus, 9187)
      call createCurrent(P_ZQ, P_WPlus, P_WMinus, P_ZQ, 2844)
      call createCurrent(P_HaQ, P_GMinus, P_HPlus, P_HlQ, 4745)
      call createCurrent(P_GQMinus, P_HPlus, P_Ha, P_HaQ, 6774)
      call createCurrent(P_GPlus, P_WMinus, P_Hl, 6878)
      call createCurrent(P_HPlus, P_WMinus, P_HMinus, P_WPlus, 7934)
      call createCurrent(P_GQ0, P_ghWp, P_Hl, P_ghWpAnti, 2949)
      call createCurrent(P_Z, P_g, P_g, P_Z, 10719)
      call createCurrent(P_Z, P_ghZ, P_ghWmAnti, P_WMinus, 4378)
      call createCurrent(P_HQMinus, P_Hl, P_G0, P_GQPlus, 1806)
      call createCurrent(P_GPlus, P_g, P_HMinus, P_g, 10602)
      call createCurrent(P_Z, P_GMinus, P_WPlus, P_G0, 4319)
      call createCurrent(P_Ha, P_GPlus, P_ghA, P_ghWpAnti, 9052)
      call createCurrent(P_HMinus, P_GPlus, P_HMinus, P_GPlus, 5576)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Z, P_ZQ, 8007)
      call createCurrent(P_HMinus, P_WPlus, P_Hh, P_A, 767)
      call createCurrent(P_GQ0, P_G0, P_HMinus, P_GQPlus, 1698)
      call createCurrent(P_Hl, P_WPlus, P_HMinus, 2954)
      call createCurrent(P_HlQ, P_Hl, P_Hh, P_HlQ, 7435)
      call createCurrent(P_ghWp, P_A, P_A, P_ghWpAnti, 6541)
      call createCurrent(P_WQPlus, P_ghZ, P_Z, P_ghWpAnti, 148)
      call createCurrent(P_HhQ, P_GPlus, P_GQMinus, 2800)
      call createCurrent(P_HlQ, P_A, P_WMinus, P_HQPlus, 5793)
      call createCurrent(P_cAnti, P_c, P_HaQ, 7094)
      call createCurrent(P_GPlus, P_ghZ, P_GMinus, P_ghZAnti, 3751)
      call createCurrent(P_AQ, P_ghWmAnti, P_ghWm, 534)
      call createCurrent(P_G0, P_Ha, P_WPlus, P_WMinus, 10060)
      call createCurrent(P_HMinus, P_A, P_WPlus, P_Hh, 7138)
      call createCurrent(P_Hl, P_GMinus, P_WPlus, 2909)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_Hl, 4971)
      call createCurrent(P_ghZAnti, P_ghA, P_GPlus, P_HMinus, 9045)
      call createCurrent(P_ghZ, P_GPlus, P_ghZAnti, P_HMinus, 10184)
      call createCurrent(P_GMinus, P_HMinus, P_HPlus, P_GPlus, 6639)
      call createCurrent(P_nu_muAnti, P_HPlus, P_muMinus, 7011)
      call createCurrent(P_ghZ, P_WMinus, P_A, P_ghWmAnti, 1632)
      call createCurrent(P_WMinus, P_Hh, P_WPlus, 3824)
      call createCurrent(P_WPlus, P_WMinus, P_A, P_Z, 1847)
      call createCurrent(P_WQPlus, P_ghAAnti, P_WMinus, P_ghA, 8653)
      call createCurrent(P_GMinus, P_A, P_WPlus, P_Hh, 147)
      call createCurrent(P_WMinus, P_HPlus, P_GMinus, P_WPlus, 9929)
      call createCurrent(P_gQ, P_t, P_tAnti, 5280)
      call createCurrent(P_HaQ, P_HMinus, P_Ha, P_HQPlus, 2191)
      call createCurrent(P_ghWmAnti, P_ghA, P_WMinus, P_A, 8156)
      call createCurrent(P_GQ0, P_ghZ, P_G0, P_ghZAnti, 3252)
      call createCurrent(P_HMinus, P_nu_mu, P_muPlus, 1199)
      call createCurrent(P_A, P_GMinus, P_G0, P_WPlus, 4437)
      call createCurrent(P_ghA, P_HPlus, P_ghWpAnti, P_Hh, 591)
      call createCurrent(P_Hh, P_Hh, P_WMinus, P_WPlus, 619)
      call createCurrent(P_ghWm, P_Hl, P_ghZAnti, P_HPlus, 90)
      call createCurrent(P_G0, P_GMinus, P_Hh, P_HPlus, 6701)
      call createCurrent(P_HlQ, P_Ha, P_ghWm, P_ghWmAnti, 101)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HPlus, P_HQMinus, 1299)
      call createCurrent(P_Ha, P_GPlus, P_HMinus, P_Ha, 7968)
      call createCurrent(P_HaQ, P_u, P_uAnti, 5998)
      call createCurrent(P_G0, P_GMinus, P_ghZ, P_ghWmAnti, 7703)
      call createCurrent(P_tauPlus, P_tauMinus, P_GQ0, 2458)
      call createCurrent(P_eMinus, P_ePlus, P_GQ0, 669)
      call createCurrent(P_A, P_WMinus, P_HPlus, P_Hl, 694)
      call createCurrent(P_ghWmAnti, P_ghA, P_Z, P_WMinus, 7797)
      call createCurrent(P_WQMinus, P_muPlus, P_nu_mu, 1544)
      call createCurrent(P_A, P_GPlus, P_WMinus, P_Hl, 1946)
      call createCurrent(P_nu_tauAnti, P_nu_tau, 9164)
      call createCurrent(P_GQPlus, P_WMinus, P_Hh, P_AQ, 1108)
      call createCurrent(P_Ha, P_ghWp, P_GMinus, P_ghAAnti, 8939)
      call createCurrent(P_WQPlus, P_Hh, P_GMinus, P_ZQ, 8033)
      call createCurrent(P_ghAAnti, P_ghWp, P_WMinus, P_AQ, 8409)
      call createCurrent(P_GPlus, P_Hh, P_Hl, P_GMinus, 994)
      call createCurrent(P_WQMinus, P_Hh, P_GPlus, P_AQ, 3452)
      call createCurrent(P_dAnti, P_u, P_HMinus, 5785)
      call createCurrent(P_Z, P_WMinus, P_Z, P_WPlus, 5668)
      call createCurrent(P_g, P_Ha, P_g, P_Ha, 10759)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_HaQ, 5684)
      call createCurrent(P_WPlus, P_WMinus, P_Hl, 6626)
      call createCurrent(P_Hl, P_WPlus, P_GMinus, 6165)
      call createCurrent(P_ZQ, P_bAnti, P_b, 5912)
      call createCurrent(P_WQMinus, P_Hh, P_WPlus, P_GQ0, 3864)
      call createCurrent(P_WPlus, P_HMinus, P_A, P_Hl, 3949)
      call createCurrent(P_ghWp, P_Hl, P_GMinus, P_ghAAnti, 2781)
      call createCurrent(P_WQMinus, P_Z, P_A, P_WQPlus, 8101)
      call createCurrent(P_tauMinus, P_tauPlus, P_HaQ, 2115)
      call createCurrent(P_G0, P_GMinus, P_ghWmAnti, P_ghA, 6851)
      call createCurrent(P_WPlus, P_A, P_ghWm, P_ghAAnti, 4625)
      call createCurrent(P_ghA, P_HMinus, P_HPlus, P_ghZAnti, 3032)
      call createCurrent(P_HaQ, P_Hh, P_Hl, P_GQ0, 2692)
      call createCurrent(P_ghWmAnti, P_HMinus, P_Hh, P_ghA, 4545)
      call createCurrent(P_G0, P_HPlus, P_Hl, P_GMinus, 6708)
      call createCurrent(P_HlQ, P_HPlus, P_ghWm, P_ghZAnti, 7931)
      call createCurrent(P_GMinus, P_Hh, P_HPlus, P_G0, 5626)
      call createCurrent(P_ghWm, P_GPlus, P_ghWmAnti, P_GMinus, 7754)
      call createCurrent(P_GQMinus, P_ghZAnti, P_Hh, P_ghWp, 1106)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, 3294)
      call createCurrent(P_HQMinus, P_WPlus, P_A, P_HaQ, 3256)
      call createCurrent(P_HPlus, P_ghWpAnti, P_G0, P_ghA, 10428)
      call createCurrent(P_GPlus, P_GMinus, P_Z, P_Z, 3725)
      call createCurrent(P_Hh, P_WMinus, P_HPlus, 6342)
      call createCurrent(P_WQMinus, P_Hl, P_WPlus, P_HlQ, 8758)
      call createCurrent(P_HQMinus, P_Hh, P_ghWmAnti, P_ghZ, 5609)
      call createCurrent(P_HPlus, P_ghZAnti, P_Ha, P_ghWm, 5473)
      call createCurrent(P_HMinus, P_GPlus, P_Z, P_Z, 9696)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hh, P_HQPlus, 394)
      call createCurrent(P_Ha, P_Ha, P_ghWmAnti, P_ghWm, 4468)
      call createCurrent(P_WMinus, P_Ha, P_GPlus, P_A, 10390)
      call createCurrent(P_HMinus, P_G0, P_Hh, P_HPlus, 9792)
      call createCurrent(P_ghAAnti, P_Hh, P_HPlus, P_ghWm, 3221)
      call createCurrent(P_GMinus, P_WPlus, P_HPlus, P_WMinus, 9276)
      call createCurrent(P_ghWpAnti, P_G0, P_ghZ, P_GPlus, 3896)
      call createCurrent(P_GQ0, P_Hh, P_GMinus, P_HQPlus, 4499)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_ghZ, 572)
      call createCurrent(P_HPlus, P_Hh, P_ghWm, P_ghZAnti, 7508)
      call createCurrent(P_Hl, P_ghWm, P_ghWmAnti, 4253)
      call createCurrent(P_ghWp, P_ghAAnti, P_WMinus, P_AQ, 4726)
      call createCurrent(P_WMinus, P_ghAAnti, P_Z, P_ghWp, 4160)
      call createCurrent(P_GQPlus, P_GMinus, P_Hl, P_HlQ, 8434)
      call createCurrent(P_Hh, P_GMinus, P_ghA, P_ghWmAnti, 2440)
      call createCurrent(P_A, P_GMinus, P_GPlus, P_Z, 7045)
      call createCurrent(P_HMinus, P_c, P_sAnti, 2300)
      call createCurrent(P_WMinus, P_Z, P_GPlus, 2967)
      call createCurrent(P_c, P_cAnti, P_Hl, 3763)
      call createCurrent(P_GPlus, P_Z, P_WMinus, P_Ha, 10232)
      call createCurrent(P_HPlus, P_Z, P_WMinus, P_Ha, 1009)
      call createCurrent(P_ghAAnti, P_WMinus, P_WPlus, P_ghZ, 5743)
      call createCurrent(P_WPlus, P_Hh, P_GMinus, P_Z, 6540)
      call createCurrent(P_ghWmAnti, P_ghA, P_GMinus, P_G0, 663)
      call createCurrent(P_WQMinus, P_GPlus, P_A, P_GQ0, 3345)
      call createCurrent(P_WQMinus, P_ghA, P_Z, P_ghWmAnti, 3729)
      call createCurrent(P_HPlus, P_GMinus, P_HMinus, P_HPlus, 7849)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_Hh, 6301)
      call createCurrent(P_ghZ, P_HPlus, P_G0, P_ghWpAnti, 9091)
      call createCurrent(P_Hl, P_WPlus, P_Z, P_HMinus, 8461)
      call createCurrent(P_Hh, P_Z, P_A, P_Hh, 10167)
      call createCurrent(P_G0, P_GMinus, P_ghWp, P_ghAAnti, 7106)
      call createCurrent(P_WPlus, P_A, P_GMinus, 1859)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghWp, 1465)
      call createCurrent(P_A, P_Z, P_HMinus, P_HPlus, 6872)
      call createCurrent(P_ZQ, P_ghWm, P_ghWmAnti, 6401)
      call createCurrent(P_GQ0, P_ghZ, P_GMinus, P_ghWmAnti, 2907)
      call createCurrent(P_HMinus, P_ghZAnti, P_Ha, P_ghWp, 8448)
      call createCurrent(P_GMinus, P_GPlus, P_A, P_A, 929)
      call createCurrent(P_d, P_uAnti, P_WPlus, 8676)
      call createCurrent(P_ghWp, P_GMinus, P_HPlus, P_ghWpAnti, 9738)
      call createCurrent(P_tauPlus, P_WMinus, P_nu_tau, 6002)
      call createCurrent(P_b, P_tAnti, P_WPlus, 7930)
      call createCurrent(P_ghWpAnti, P_ghWp, P_A, P_AQ, 45)
      call createCurrent(P_Ha, P_GPlus, P_G0, P_GMinus, 5088)
      call createCurrent(P_G0, P_ghZAnti, P_Ha, P_ghZ, 9877)
      call createCurrent(P_u, P_HMinus, P_dAnti, 3478)
      call createCurrent(P_ghAAnti, P_ghWp, P_WMinus, 3848)
      call createCurrent(P_HhQ, P_HPlus, P_GMinus, P_HaQ, 7667)
      call createCurrent(P_ZQ, P_A, P_ghWp, P_ghWpAnti, 5779)
      call createCurrent(P_GPlus, P_Hl, P_WMinus, P_Z, 3308)
      call createCurrent(P_WMinus, P_Z, P_Hh, P_HPlus, 3272)
      call createCurrent(P_ghWmAnti, P_Hl, P_GMinus, P_ghZ, 693)
      call createCurrent(P_HMinus, P_WPlus, P_Hh, P_Z, 303)
      call createCurrent(P_ghWmAnti, P_ghA, P_WMinus, 7139)
      call createCurrent(P_GPlus, P_A, P_Hl, P_WMinus, 640)
      call createCurrent(P_ghWp, P_A, P_ghAAnti, P_WMinus, 8120)
      call createCurrent(P_A, P_HPlus, P_Z, P_HMinus, 1171)
      call createCurrent(P_A, P_G0, P_A, P_G0, 10714)
      call createCurrent(P_HQPlus, P_cAnti, P_s, 1458)
      call createCurrent(P_ghZ, P_GPlus, P_Hh, P_ghWpAnti, 1390)
      call createCurrent(P_nu_eAnti, P_eMinus, P_GQPlus, 2072)
      call createCurrent(P_Z, P_HMinus, P_GPlus, P_Z, 9636)
      call createCurrent(P_HQMinus, P_Z, P_Hl, P_WQPlus, 1716)
      call createCurrent(P_ghAAnti, P_GMinus, P_G0, P_ghWp, 4489)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghWp, P_G0, 9791)
      call createCurrent(P_HQMinus, P_Hh, P_ghWp, P_ghZAnti, 689)
      call createCurrent(P_ghWm, P_ghAAnti, P_Hl, P_GPlus, 8738)
      call createCurrent(P_HQMinus, P_ghAAnti, P_HPlus, P_ghZ, 5619)
      call createCurrent(P_Z, P_A, P_Ha, P_G0, 10704)
      call createCurrent(P_ZQ, P_Hl, P_HMinus, P_WQPlus, 4216)
      call createCurrent(P_ghWp, P_GMinus, P_ghZAnti, P_Hh, 6771)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghWm, 5396)
      call createCurrent(P_Hl, P_GPlus, P_ghWpAnti, P_ghA, 6139)
      call createCurrent(P_HQPlus, P_HMinus, P_ghZAnti, P_ghZ, 7458)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghWp, P_Hh, 597)
      call createCurrent(P_A, P_A, P_ghWmAnti, P_ghWm, 5386)
      call createCurrent(P_ghWmAnti, P_GMinus, P_Ha, P_ghZ, 10474)
      call createCurrent(P_HMinus, P_WPlus, P_A, 6099)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghWp, 6198)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hh, P_HMinus, 7961)
      call createCurrent(P_A, P_WPlus, P_G0, P_HMinus, 9220)
      call createCurrent(P_GQ0, P_WMinus, P_A, P_GQPlus, 3879)
      call createCurrent(P_gQ, P_ghG, P_g, P_ghGAnti, 1196)
      call createCurrent(P_HhQ, P_HPlus, P_ghZAnti, P_ghWm, 6179)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Z, P_Z, 2189)
      call createCurrent(P_HPlus, P_G0, P_A, P_WMinus, 9014)
      call createCurrent(P_Hl, P_Z, P_Hl, P_Z, 7469)
      call createCurrent(P_ghZAnti, P_ghWm, P_Z, P_WPlus, 6037)
      call createCurrent(P_GPlus, P_Z, P_WMinus, P_G0, 6104)
      call createCurrent(P_GQ0, P_tauMinus, P_tauPlus, 4713)
      call createCurrent(P_HPlus, P_ghWpAnti, P_G0, P_ghZ, 9888)
      call createCurrent(P_Ha, P_HPlus, P_Z, P_WMinus, 1930)
      call createCurrent(P_tauPlus, P_nu_tau, P_HQMinus, 5527)
      call createCurrent(P_Z, P_HPlus, P_GMinus, 9029)
      call createCurrent(P_WPlus, P_eMinus, P_nu_eAnti, 6392)
      call createCurrent(P_GQMinus, P_Hl, P_GQPlus, 6418)
      call createCurrent(P_Hl, P_HPlus, P_A, P_WMinus, 5588)
      call createCurrent(P_WMinus, P_ghAAnti, P_ghWp, 1102)
      call createCurrent(P_GPlus, P_HMinus, P_WMinus, P_WPlus, 9458)
      call createCurrent(P_WMinus, P_ghAAnti, P_WPlus, P_ghA, 113)
      call createCurrent(P_HhQ, P_Hh, P_Z, P_ZQ, 5960)
      call createCurrent(P_WMinus, P_A, P_ghWmAnti, P_ghZ, 47)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghWp, P_AQ, 4395)
      call createCurrent(P_HMinus, P_ghAAnti, P_HPlus, P_ghZ, 4618)
      call createCurrent(P_HQMinus, P_HPlus, P_GQ0, 2646)
      call createCurrent(P_ghWmAnti, P_HPlus, P_ghWm, P_HMinus, 5540)
      call createCurrent(P_WMinus, P_WPlus, P_Ha, P_G0, 8838)
      call createCurrent(P_GQMinus, P_WPlus, P_Hh, P_AQ, 6035)
      call createCurrent(P_ZQ, P_A, P_ghWmAnti, P_ghWm, 5246)
      call createCurrent(P_GPlus, P_G0, P_ghWpAnti, P_ghA, 4496)
      call createCurrent(P_ZQ, P_HMinus, P_WPlus, P_HlQ, 412)
      call createCurrent(P_HMinus, P_HPlus, P_Z, 5712)
      call createCurrent(P_GQPlus, P_nu_muAnti, P_muMinus, 1072)
      call createCurrent(P_HMinus, P_G0, P_WPlus, 9867)
      call createCurrent(P_ghWp, P_ghZAnti, P_WQMinus, 8202)
      call createCurrent(P_Hl, P_ghZ, P_Hl, P_ghZAnti, 376)
      call createCurrent(P_HhQ, P_GMinus, P_Hl, P_GQPlus, 4414)
      call createCurrent(P_HPlus, P_Hh, P_G0, P_GMinus, 1243)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_HlQ, 1708)
      call createCurrent(P_ghZ, P_HPlus, P_ghAAnti, P_HMinus, 4722)
      call createCurrent(P_WPlus, P_GMinus, P_Z, 250)
      call createCurrent(P_AQ, P_ghWm, P_ghWmAnti, 2765)
      call createCurrent(P_G0, P_G0, P_Hl, P_Hh, 2769)
      call createCurrent(P_ghZAnti, P_Hh, P_ghZ, 6596)
      call createCurrent(P_HQPlus, P_HMinus, P_GMinus, P_GQPlus, 8785)
      call createCurrent(P_HhQ, P_HPlus, P_GMinus, P_HlQ, 6137)
      call createCurrent(P_GQ0, P_HMinus, P_Hh, P_GQPlus, 5889)
      call createCurrent(P_Hh, P_Hl, P_HPlus, P_HMinus, 4338)
      call createCurrent(P_ghZAnti, P_ghZ, P_GPlus, P_GQMinus, 2330)
      call createCurrent(P_Hh, P_ghZ, P_HMinus, P_ghWmAnti, 456)
      call createCurrent(P_eMinus, P_ePlus, 8914)
      call createCurrent(P_GMinus, P_Z, P_WPlus, 6764)
      call createCurrent(P_Ha, P_HMinus, P_GPlus, P_Hh, 3482)
      call createCurrent(P_Z, P_HMinus, P_Ha, P_WPlus, 2198)
      call createCurrent(P_GPlus, P_Hh, P_ghWpAnti, P_ghZ, 3716)
      call createCurrent(P_Hl, P_HPlus, P_HMinus, P_Ha, 9248)
      call createCurrent(P_HQPlus, P_HMinus, P_HPlus, P_GQMinus, 8452)
      call createCurrent(P_ghWm, P_Z, P_ghWmAnti, 4213)
      call createCurrent(P_HPlus, P_ghWpAnti, P_HMinus, P_ghWp, 1699)
      call createCurrent(P_G0, P_GMinus, P_Hh, P_GPlus, 8883)
      call createCurrent(P_A, P_WMinus, P_WPlus, P_Z, 2406)
      call createCurrent(P_HMinus, P_g, P_HPlus, P_g, 10495)
      call createCurrent(P_HMinus, P_G0, P_Hl, P_GPlus, 6665)
      call createCurrent(P_A, P_ghWp, P_ghAAnti, P_WMinus, 7595)
      call createCurrent(P_Ha, P_GPlus, P_ghZAnti, P_ghWm, 9094)
      call createCurrent(P_G0, P_HMinus, P_Hh, P_GPlus, 5202)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghWp, P_WMinus, 1109)
      call createCurrent(P_HlQ, P_ghWm, P_ghWmAnti, 6873)
      call createCurrent(P_GQMinus, P_GMinus, P_HPlus, P_GQPlus, 4626)
      call createCurrent(P_ghA, P_GMinus, P_ghZAnti, P_HPlus, 9332)
      call createCurrent(P_GQPlus, P_Hl, P_ghWm, P_ghZAnti, 5142)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWm, P_HQPlus, 5748)
      call createCurrent(P_b, P_tAnti, P_GQPlus, 3702)
      call createCurrent(P_Ha, P_Ha, P_ghZAnti, P_ghZ, 2598)
      call createCurrent(P_ghZ, P_G0, P_ghWpAnti, P_GQPlus, 6592)
      call createCurrent(P_GPlus, P_Z, P_GMinus, P_A, 8139)
      call createCurrent(P_ZQ, P_Hl, P_Hl, P_ZQ, 4719)
      call createCurrent(P_HlQ, P_GPlus, P_Hl, P_GQMinus, 5461)
      call createCurrent(P_HPlus, P_Ha, P_ghAAnti, P_ghWm, 7728)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hh, P_HQPlus, 5444)
      call createCurrent(P_Ha, P_Z, 9757)
      call createCurrent(P_HQPlus, P_HMinus, P_G0, P_HaQ, 5282)
      call createCurrent(P_Hh, P_ghWp, P_ghAAnti, P_GMinus, 5053)
      call createCurrent(P_ghAAnti, P_HMinus, P_Hh, P_ghWp, 2932)
      call createCurrent(P_HaQ, P_Ha, P_ghWpAnti, P_ghWp, 5606)
      call createCurrent(P_ghZAnti, P_HMinus, P_GPlus, P_ghZ, 9398)
      call createCurrent(P_ZQ, P_Z, P_HMinus, P_HQPlus, 1737)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Ha, P_Ha, 8032)
      call createCurrent(P_GQPlus, P_G0, P_GMinus, P_HhQ, 2146)
      call createCurrent(P_HMinus, P_WPlus, P_WMinus, P_HPlus, 3013)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Ha, P_GPlus, 9378)
      call createCurrent(P_g, P_c, P_cAnti, 6395)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hh, P_GQ0, 4439)
      call createCurrent(P_ghWp, P_WMinus, P_ghAAnti, P_ZQ, 6921)
      call createCurrent(P_GMinus, P_ghZ, P_Ha, P_ghWmAnti, 10285)
      call createCurrent(P_GQPlus, P_Z, P_GMinus, P_AQ, 2204)
      call createCurrent(P_ghWmAnti, P_ghZ, P_A, P_WMinus, 6690)
      call createCurrent(P_Z, P_WPlus, P_HMinus, P_Ha, 4331)
      call createCurrent(P_nu_mu, P_muPlus, P_HMinus, 4966)
      call createCurrent(P_Z, P_ghWp, P_Z, P_ghWpAnti, 345)
      call createCurrent(P_G0, P_ghZAnti, P_ghWp, P_GMinus, 8749)
      call createCurrent(P_Hh, P_G0, P_GPlus, P_GMinus, 9710)
      call createCurrent(P_GPlus, P_Ha, P_G0, P_HMinus, 4540)
      call createCurrent(P_HhQ, P_GMinus, P_HPlus, P_GQ0, 5776)
      call createCurrent(P_HQPlus, P_GMinus, P_GQ0, 1392)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghWp, P_A, 4769)
      call createCurrent(P_G0, P_GPlus, P_G0, P_GMinus, 7037)
      call createCurrent(P_A, P_nu_tauAnti, P_nu_tau, 10105)
      call createCurrent(P_HlQ, P_G0, P_WPlus, P_WQMinus, 2318)
      call createCurrent(P_Z, P_GMinus, P_HPlus, P_Z, 10020)
      call createCurrent(P_WMinus, P_Z, P_GPlus, P_Ha, 9669)
      call createCurrent(P_WMinus, P_ghA, P_WPlus, P_ghAAnti, 7616)
      call createCurrent(P_HPlus, P_Z, P_HMinus, 2197)
      call createCurrent(P_ZQ, P_ghWpAnti, P_A, P_ghWp, 3378)
      call createCurrent(P_ghWmAnti, P_HMinus, P_Ha, P_ghA, 4708)
      call createCurrent(P_WMinus, P_A, P_GPlus, 2379)
      call createCurrent(P_A, P_G0, P_Z, P_Ha, 10563)
      call createCurrent(P_A, P_GMinus, P_WPlus, P_Ha, 8862)
      call createCurrent(P_ghWpAnti, P_G0, P_ghA, P_GQPlus, 8609)
      call createCurrent(P_A, P_Ha, P_GMinus, P_WPlus, 9689)
      call createCurrent(P_Z, P_Hl, P_Hl, P_Z, 6867)
      call createCurrent(P_GQMinus, P_WPlus, P_A, P_GQ0, 2688)
      call createCurrent(P_Hh, P_WMinus, P_GPlus, P_Z, 321)
      call createCurrent(P_Hh, P_HPlus, P_ghAAnti, P_ghWm, 8543)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_HlQ, 2550)
      call createCurrent(P_WQMinus, P_Hh, P_Hh, P_WQPlus, 1466)
      call createCurrent(P_Z, P_s, P_sAnti, 2568)
      call createCurrent(P_ghWm, P_Hh, P_Hl, P_ghWmAnti, 10372)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hl, P_HMinus, 786)
      call createCurrent(P_Hl, P_Hh, P_GPlus, P_HMinus, 2919)
      call createCurrent(P_HQMinus, P_HPlus, P_Ha, P_HlQ, 290)
      call createCurrent(P_G0, P_A, P_HMinus, P_WPlus, 10021)
      call createCurrent(P_GPlus, P_Hl, P_ghAAnti, P_ghWm, 3129)
      call createCurrent(P_HMinus, P_ghAAnti, P_HPlus, P_ghA, 1629)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HhQ, 1626)
      call createCurrent(P_G0, P_HPlus, P_Hl, P_HMinus, 9970)
      call createCurrent(P_HQPlus, P_HMinus, P_Ha, P_HaQ, 2093)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hl, P_Hl, 4078)
      call createCurrent(P_HPlus, P_ghZ, P_ghWpAnti, P_Hh, 7806)
      call createCurrent(P_HaQ, P_Hl, P_WMinus, P_WQPlus, 1639)
      call createCurrent(P_HhQ, P_ghZ, P_ghZAnti, 8572)
      call createCurrent(P_WMinus, P_Hl, P_HPlus, P_Z, 25)
      call createCurrent(P_ghZAnti, P_Z, P_WMinus, P_ghWp, 7085)
      call createCurrent(P_GMinus, P_HPlus, P_A, 9311)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_HhQ, 5063)
      call createCurrent(P_GMinus, P_A, P_HPlus, P_A, 9482)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, 1867)
      call createCurrent(P_GQ0, P_GPlus, P_A, P_WQMinus, 6173)
      call createCurrent(P_HMinus, P_GPlus, P_Hh, P_Hh, 3954)
      call createCurrent(P_Ha, P_Hh, P_GMinus, P_HPlus, 5869)
      call createCurrent(P_t, P_tAnti, P_Hl, 6777)
      call createCurrent(P_AQ, P_GPlus, P_GMinus, P_AQ, 288)
      call createCurrent(P_HhQ, P_WPlus, P_GQMinus, 1838)
      call createCurrent(P_ghWpAnti, P_Hh, P_HPlus, P_ghA, 5026)
      call createCurrent(P_WPlus, P_A, P_ghWpAnti, P_ghZ, 2608)
      call createCurrent(P_GQ0, P_Ha, P_G0, P_HaQ, 5650)
      call createCurrent(P_HQMinus, P_HPlus, P_ghA, P_ghZAnti, 8080)
      call createCurrent(P_ghZ, P_HMinus, P_GPlus, P_ghZAnti, 10017)
      call createCurrent(P_ghWm, P_WPlus, P_WMinus, P_ghWmAnti, 942)
      call createCurrent(P_Ha, P_GPlus, P_GMinus, P_G0, 5005)
      call createCurrent(P_GMinus, P_ghZAnti, P_GPlus, P_ghA, 4316)
      call createCurrent(P_tAnti, P_b, P_HQPlus, 1262)
      call createCurrent(P_GMinus, P_ghWp, P_ghZAnti, P_Hh, 6772)
      call createCurrent(P_Hh, P_A, P_WMinus, P_GPlus, 7340)
      call createCurrent(P_Hh, P_ghWmAnti, P_HMinus, P_ghZ, 6133)
      call createCurrent(P_Hl, P_ghWm, P_HPlus, P_ghAAnti, 8227)
      call createCurrent(P_GQPlus, P_ghAAnti, P_GMinus, P_ghA, 1381)
      call createCurrent(P_Hl, P_Ha, P_HMinus, P_GPlus, 2145)
      call createCurrent(P_WQPlus, P_WMinus, P_Hl, P_GQ0, 622)
      call createCurrent(P_G0, P_GPlus, P_ghAAnti, P_ghWm, 5198)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_Ha, 8886)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghWm, P_ZQ, 8336)
      call createCurrent(P_ghAAnti, P_Z, P_ghWm, P_WPlus, 1952)
      call createCurrent(P_HQMinus, P_Hl, P_HPlus, P_HhQ, 8469)
      call createCurrent(P_ghA, P_HMinus, P_HPlus, P_ghAAnti, 8231)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghA, P_GMinus, 10270)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, 88)
      call createCurrent(P_HQMinus, P_G0, P_Hl, P_GQPlus, 4250)
      call createCurrent(P_HlQ, P_HPlus, P_GQMinus, 8726)
      call createCurrent(P_GPlus, P_HMinus, P_Hl, 8171)
      call createCurrent(P_GQMinus, P_GPlus, P_Hh, P_HlQ, 3212)
      call createCurrent(P_Ha, P_Hl, P_G0, P_Hh, 196)
      call createCurrent(P_ghZAnti, P_G0, P_ghZ, P_GQ0, 5560)
      call createCurrent(P_GQ0, P_WMinus, P_WQPlus, 5965)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_Hl, 3690)
      call createCurrent(P_HQPlus, P_Z, P_HMinus, P_AQ, 3858)
      call createCurrent(P_u, P_WMinus, P_dAnti, 1637)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, P_HhQ, 3138)
      call createCurrent(P_GQPlus, P_ghWm, P_G0, P_ghZAnti, 5109)
      call createCurrent(P_HlQ, P_GPlus, P_HMinus, P_HhQ, 1330)
      call createCurrent(P_GQMinus, P_GPlus, P_GQ0, 51)
      call createCurrent(P_HlQ, P_GMinus, P_GQPlus, 1425)
      call createCurrent(P_HQPlus, P_G0, P_GMinus, P_HlQ, 3976)
      call createCurrent(P_ghWpAnti, P_A, P_ghWp, P_AQ, 6313)
      call createCurrent(P_HaQ, P_ghWm, P_Hl, P_ghWmAnti, 4762)
      call createCurrent(P_sAnti, P_HMinus, P_c, 6304)
      call createCurrent(P_tauMinus, P_Z, P_tauPlus, 4142)
      call createCurrent(P_HaQ, P_HMinus, P_HPlus, P_HaQ, 6660)
      call createCurrent(P_HaQ, P_ghWp, P_HMinus, P_ghZAnti, 764)
      call createCurrent(P_A, P_ghA, P_ghWpAnti, P_WPlus, 960)
      call createCurrent(P_HPlus, P_HMinus, P_Z, P_Z, 7710)
      call createCurrent(P_HhQ, P_HMinus, P_Ha, P_HQPlus, 5762)
      call createCurrent(P_HPlus, P_Hl, P_ghZ, P_ghWpAnti, 7282)
      call createCurrent(P_GPlus, P_GMinus, P_ghAAnti, P_ghA, 1989)
      call createCurrent(P_WQMinus, P_ghA, P_A, P_ghWmAnti, 3126)
      call createCurrent(P_ghZAnti, P_Z, P_WPlus, P_ghWm, 6472)
      call createCurrent(P_GPlus, P_GMinus, P_Hh, P_Hh, 3219)
      call createCurrent(P_ghA, P_GPlus, P_ghAAnti, P_GMinus, 7864)
      call createCurrent(P_WMinus, P_WPlus, P_GPlus, P_GMinus, 8421)
      call createCurrent(P_ZQ, P_HPlus, P_WMinus, P_HaQ, 3008)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Ha, P_GMinus, 10329)
      call createCurrent(P_ghZAnti, P_ghWm, P_G0, P_GQPlus, 3549)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghA, P_HPlus, 2058)
      call createCurrent(P_muPlus, P_nu_mu, P_WMinus, 7068)
      call createCurrent(P_Ha, P_GPlus, P_Z, P_WMinus, 9471)
      call createCurrent(P_ghWp, P_Hl, P_Hl, P_ghWpAnti, 6368)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HQMinus, 3205)
      call createCurrent(P_WQPlus, P_HMinus, P_HhQ, 7200)
      call createCurrent(P_Z, P_HMinus, P_GPlus, 9339)
      call createCurrent(P_ghWmAnti, P_A, P_ghA, P_WQMinus, 7614)
      call createCurrent(P_bAnti, P_b, 10163)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_HhQ, 3020)
      call createCurrent(P_GQMinus, P_Z, P_GPlus, P_ZQ, 5683)
      call createCurrent(P_Z, P_A, P_WPlus, P_WMinus, 2986)
      call createCurrent(P_ghWpAnti, P_WPlus, P_WMinus, P_ghWp, 7054)
      call createCurrent(P_GMinus, P_A, P_WPlus, P_G0, 8616)
      call createCurrent(P_HPlus, P_Hl, P_A, P_WMinus, 8048)
      call createCurrent(P_Z, P_ghWp, P_WMinus, P_ghAAnti, 7949)
      call createCurrent(P_WQPlus, P_Hh, P_WQMinus, 227)
      call createCurrent(P_ghWpAnti, P_ghWp, P_G0, P_GQ0, 1230)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_HaQ, 2367)
      call createCurrent(P_GMinus, P_Hl, P_Hl, P_GPlus, 6588)
      call createCurrent(P_WQMinus, P_WPlus, P_HlQ, 1656)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghA, P_HQMinus, 2081)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghWm, P_Hl, 8583)
      call createCurrent(P_GQPlus, P_HMinus, P_Hl, P_GQ0, 6898)
      call createCurrent(P_WQMinus, P_HPlus, P_HMinus, P_WQPlus, 6106)
      call createCurrent(P_HlQ, P_HMinus, P_GQPlus, 6145)
      call createCurrent(P_HlQ, P_HPlus, P_HQMinus, 8260)
      call createCurrent(P_ghAAnti, P_Ha, P_GPlus, P_ghWm, 9714)
      call createCurrent(P_Ha, P_Ha, P_GMinus, P_HPlus, 1749)
      call createCurrent(P_GQ0, P_Hl, P_ghWpAnti, P_ghWp, 7441)
      call createCurrent(P_Hl, P_G0, P_HMinus, P_GPlus, 5414)
      call createCurrent(P_ZQ, P_G0, P_Z, P_GQ0, 7408)
      call createCurrent(P_Hh, P_Hl, P_GMinus, P_HPlus, 6240)
      call createCurrent(P_HQMinus, P_GPlus, P_G0, P_HlQ, 5829)
      call createCurrent(P_G0, P_ghA, P_HPlus, P_ghWpAnti, 9420)
      call createCurrent(P_muPlus, P_HMinus, P_nu_mu, 5430)
      call createCurrent(P_HPlus, P_HMinus, P_Hl, P_Hl, 4258)
      call createCurrent(P_GQPlus, P_HMinus, P_G0, P_GQ0, 1986)
      call createCurrent(P_ghWmAnti, P_Z, P_ghZ, P_WMinus, 4687)
      call createCurrent(P_GQMinus, P_GPlus, P_Hh, P_HaQ, 1269)
      call createCurrent(P_tAnti, P_t, P_G0, 3779)
      call createCurrent(P_HhQ, P_G0, P_GMinus, P_GQPlus, 2071)
      call createCurrent(P_AQ, P_A, P_ghWm, P_ghWmAnti, 3149)
      call createCurrent(P_WPlus, P_Hl, P_A, P_GMinus, 1574)
      call createCurrent(P_Hh, P_G0, P_G0, P_Hh, 8722)
      call createCurrent(P_GPlus, P_HMinus, P_Hl, P_Ha, 5986)
      call createCurrent(P_HhQ, P_muMinus, P_muPlus, 6952)
      call createCurrent(P_GQMinus, P_WPlus, P_G0, P_AQ, 7477)
      call createCurrent(P_GQ0, P_Hl, P_G0, P_HhQ, 1748)
      call createCurrent(P_WPlus, P_A, P_HMinus, P_Ha, 1595)
      call createCurrent(P_A, P_ghWm, P_ghAAnti, P_WPlus, 304)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_GQ0, 5427)
      call createCurrent(P_GQ0, P_G0, P_Hh, P_HlQ, 6462)
      call createCurrent(P_GQ0, P_Hh, P_ZQ, 8684)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghA, P_GQMinus, 2910)
      call createCurrent(P_Hh, P_G0, P_Hl, P_G0, 6773)
      call createCurrent(P_A, P_Z, P_GPlus, P_GMinus, 7095)
      call createCurrent(P_ghZAnti, P_Ha, P_ghWp, P_HMinus, 7534)
      call createCurrent(P_HMinus, P_Ha, P_ghZAnti, P_ghWp, 4833)
      call createCurrent(P_HQMinus, P_Hl, P_Hh, P_HQPlus, 2879)
      call createCurrent(P_GQPlus, P_A, P_GMinus, P_ZQ, 3686)
      call createCurrent(P_HlQ, P_GPlus, P_HQMinus, 2591)
      call createCurrent(P_Ha, P_ghWp, P_ghWpAnti, P_G0, 10485)
      call createCurrent(P_G0, P_Ha, P_Hh, P_G0, 10725)
      call createCurrent(P_G0, P_HPlus, P_ghZ, P_ghWpAnti, 9480)
      call createCurrent(P_ghWp, P_Hh, P_Hh, P_ghWpAnti, 3347)
      call createCurrent(P_HMinus, P_HPlus, P_G0, 9633)
      call createCurrent(P_Z, P_G0, P_Z, P_Ha, 9238)
      call createCurrent(P_Ha, P_ghZAnti, P_G0, P_ghZ, 9391)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_Ha, 1969)
      call createCurrent(P_GMinus, P_HPlus, P_Hl, P_G0, 1367)
      call createCurrent(P_GMinus, P_Ha, P_GPlus, P_Hh, 9127)
      call createCurrent(P_GQPlus, P_Hl, P_Ha, P_HQMinus, 6734)
      call createCurrent(P_ZQ, P_GPlus, P_G0, P_WQMinus, 8155)
      call createCurrent(P_GMinus, P_Ha, P_ghAAnti, P_ghWp, 10134)
      call createCurrent(P_WMinus, P_tauPlus, P_nu_tau, 7292)
      call createCurrent(P_ghWp, P_Z, P_Z, P_ghWpAnti, 329)
      call createCurrent(P_HPlus, P_GMinus, P_A, P_A, 10413)
      call createCurrent(P_ghWm, P_HPlus, P_Ha, P_ghAAnti, 4339)
      call createCurrent(P_HMinus, P_Hh, P_GPlus, P_Ha, 1046)
      call createCurrent(P_HPlus, P_WPlus, P_WMinus, P_GMinus, 9143)
      call createCurrent(P_HaQ, P_dAnti, P_d, 5764)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghZ, P_HPlus, 1501)
      call createCurrent(P_muMinus, P_muPlus, P_HhQ, 5781)
      call createCurrent(P_WPlus, P_WMinus, P_GMinus, P_HPlus, 9608)
      call createCurrent(P_GMinus, P_ghZAnti, P_Ha, P_ghWp, 9305)
      call createCurrent(P_HlQ, P_GPlus, P_G0, P_GQMinus, 5436)
      call createCurrent(P_Hh, P_Hh, P_HPlus, P_HMinus, 907)
      call createCurrent(P_HPlus, P_GMinus, P_ghZ, P_ghZAnti, 9951)
      call createCurrent(P_HaQ, P_HPlus, P_Hl, P_GQMinus, 6799)
      call createCurrent(P_Ha, P_s, P_sAnti, 3723)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghWp, P_HaQ, 5792)
      call createCurrent(P_Z, P_A, P_ghWm, P_ghWmAnti, 1244)
      call createCurrent(P_t, P_tAnti, P_g, 1961)
      call createCurrent(P_A, P_ghWpAnti, P_ghWp, P_Z, 2286)
      call createCurrent(P_WMinus, P_Hh, P_Hl, P_WPlus, 9741)
      call createCurrent(P_GQ0, P_Ha, P_Ha, P_GQ0, 3959)
      call createCurrent(P_HMinus, P_Hl, P_HPlus, P_G0, 10429)
      call createCurrent(P_AQ, P_t, P_tAnti, 4349)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_G0, 9555)
      call createCurrent(P_G0, P_HMinus, P_ghWmAnti, P_ghA, 9826)
      call createCurrent(P_GQ0, P_G0, P_WMinus, P_WQPlus, 1800)
      call createCurrent(P_Hh, P_Ha, P_Hh, P_G0, 1308)
      call createCurrent(P_tauMinus, P_nu_tauAnti, P_GPlus, 2905)
      call createCurrent(P_G0, P_HMinus, P_ghA, P_ghWmAnti, 9966)
      call createCurrent(P_Ha, P_Hl, P_Z, 595)
      call createCurrent(P_HQMinus, P_Hh, P_HPlus, P_HaQ, 7544)
      call createCurrent(P_GQMinus, P_GPlus, P_AQ, 4226)
      call createCurrent(P_Ha, P_ghWmAnti, P_ghWm, P_Ha, 2510)
      call createCurrent(P_WMinus, P_HPlus, P_Z, 1314)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hl, 6796)
      call createCurrent(P_bAnti, P_b, P_Z, 7046)
      call createCurrent(P_G0, P_Ha, P_A, P_A, 10566)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hh, P_GQPlus, 7946)
      call createCurrent(P_ghWm, P_G0, P_ghWmAnti, P_HlQ, 1333)
      call createCurrent(P_A, P_c, P_cAnti, 3479)
      call createCurrent(P_ghZ, P_WMinus, P_Z, P_ghWmAnti, 1402)
      call createCurrent(P_HPlus, P_Hl, P_ghWm, P_ghZAnti, 349)
      call createCurrent(P_HMinus, P_ghWmAnti, P_GPlus, P_ghWm, 9557)
      call createCurrent(P_GQPlus, P_GPlus, P_GMinus, P_GQMinus, 3244)
      call createCurrent(P_A, P_GPlus, P_GMinus, P_A, 8017)
      call createCurrent(P_GPlus, P_Z, P_GMinus, 5350)
      call createCurrent(P_GQMinus, P_WPlus, P_AQ, 5368)
      call createCurrent(P_HlQ, P_Ha, P_Hh, P_GQ0, 761)
      call createCurrent(P_muPlus, P_muMinus, P_GQ0, 3197)
      call createCurrent(P_nu_eAnti, P_HPlus, P_eMinus, 2710)
      call createCurrent(P_GMinus, P_Z, P_GPlus, P_Z, 5153)
      call createCurrent(P_HaQ, P_A, P_HMinus, P_WQPlus, 2476)
      call createCurrent(P_Ha, P_Z, P_Ha, P_Z, 6473)
      call createCurrent(P_c, P_cAnti, P_HlQ, 2484)
      call createCurrent(P_Hl, P_Ha, P_Z, 4106)
      call createCurrent(P_GPlus, P_ghZ, P_Hh, P_ghWpAnti, 6453)
      call createCurrent(P_GQPlus, P_tAnti, P_b, 7075)
      call createCurrent(P_GMinus, P_Hh, P_GPlus, P_G0, 9745)
      call createCurrent(P_Hh, P_WMinus, P_WPlus, P_Hl, 10045)
      call createCurrent(P_ghA, P_ghWmAnti, P_Hl, P_GMinus, 4115)
      call createCurrent(P_Hl, P_Ha, P_HMinus, P_HPlus, 9150)
      call createCurrent(P_HQMinus, P_ghZAnti, P_Ha, P_ghWp, 8169)
      call createCurrent(P_G0, P_Ha, P_HPlus, P_HMinus, 6670)
      call createCurrent(P_WQMinus, P_WPlus, P_Hl, P_HlQ, 3796)
      call createCurrent(P_Z, P_GMinus, P_WPlus, 2578)
      call createCurrent(P_Z, P_Ha, P_WPlus, P_GMinus, 9198)
      call createCurrent(P_WMinus, P_GPlus, P_Ha, 9957)
      call createCurrent(P_G0, P_ghWm, P_ghWmAnti, P_Ha, 8847)
      call createCurrent(P_nu_e, P_GMinus, P_ePlus, 8238)
      call createCurrent(P_WQPlus, P_A, P_ghA, P_ghWpAnti, 2590)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_Hh, 1334)
      call createCurrent(P_Hl, P_GMinus, P_GPlus, 5413)
      call createCurrent(P_A, P_Hl, P_HMinus, P_WPlus, 7769)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hh, P_HQMinus, 2752)
      call createCurrent(P_Hh, P_GPlus, P_HMinus, P_Hl, 5470)
      call createCurrent(P_GQ0, P_Hl, P_WPlus, P_WQMinus, 6855)
      call createCurrent(P_WMinus, P_WPlus, P_HPlus, P_GMinus, 10367)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_Ha, 8956)
      call createCurrent(P_GQ0, P_Hl, P_Ha, P_HlQ, 2565)
      call createCurrent(P_HQPlus, P_WMinus, P_A, P_HhQ, 3002)
      call createCurrent(P_Hh, P_Ha, P_G0, 8203)
      call createCurrent(P_GQMinus, P_GPlus, P_Ha, P_HhQ, 5466)
      call createCurrent(P_A, P_ghZ, P_ghWpAnti, P_WPlus, 1214)
      call createCurrent(P_GMinus, P_GPlus, P_ghZ, P_ghAAnti, 7137)
      call createCurrent(P_WQMinus, P_Z, P_GPlus, P_GQ0, 4781)
      call createCurrent(P_dAnti, P_d, 9349)
      call createCurrent(P_ghZ, P_ghZAnti, P_HMinus, P_HQPlus, 3601)
      call createCurrent(P_GPlus, P_Hl, P_ghZAnti, P_ghWm, 3983)
      call createCurrent(P_HaQ, P_HMinus, P_Hl, P_HQPlus, 5875)
      call createCurrent(P_HaQ, P_Ha, P_HlQ, 2361)
      call createCurrent(P_HPlus, P_HMinus, P_Hh, P_G0, 8965)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, 4097)
      call createCurrent(P_Z, P_G0, P_Hl, 504)
      call createCurrent(P_GMinus, P_Ha, P_HPlus, P_Hl, 8631)
      call createCurrent(P_ghA, P_ghWmAnti, P_HMinus, P_G0, 9534)
      call createCurrent(P_ghA, P_ghWpAnti, P_HQPlus, 6703)
      call createCurrent(P_WQPlus, P_Ha, P_HQMinus, 4124)
      call createCurrent(P_cAnti, P_g, P_c, 6325)
      call createCurrent(P_HaQ, P_Ha, P_GPlus, P_GQMinus, 3856)
      call createCurrent(P_HPlus, P_Hl, P_Ha, P_GMinus, 6633)
      call createCurrent(P_GQ0, P_HMinus, P_GPlus, P_GQ0, 978)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HQPlus, 2947)
      call createCurrent(P_GMinus, P_Z, P_A, P_GPlus, 7032)
      call createCurrent(P_AQ, P_GPlus, P_Hl, P_WQMinus, 6193)
      call createCurrent(P_Ha, P_ghWmAnti, P_Ha, P_ghWm, 417)
      call createCurrent(P_Ha, P_A, P_WMinus, P_HPlus, 7633)
      call createCurrent(P_AQ, P_HPlus, P_HMinus, P_ZQ, 1520)
      call createCurrent(P_WQMinus, P_WPlus, P_HMinus, P_HQPlus, 98)
      call createCurrent(P_ghWpAnti, P_GPlus, P_GMinus, P_ghWp, 6831)
      call createCurrent(P_HMinus, P_Hl, P_ghWmAnti, P_ghA, 5565)
      call createCurrent(P_HlQ, P_WMinus, P_Hl, P_WQPlus, 4063)
      call createCurrent(P_HhQ, P_WPlus, P_Ha, P_WQMinus, 7429)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Ha, P_HlQ, 6776)
      call createCurrent(P_ghWmAnti, P_HMinus, P_Hl, P_ghZ, 2157)
      call createCurrent(P_GQ0, P_ghWp, P_ghWpAnti, 6614)
      call createCurrent(P_ghWp, P_ghWpAnti, P_GQ0, 1707)
      call createCurrent(P_HMinus, P_Z, P_WPlus, P_Ha, 7412)
      call createCurrent(P_t, P_tAnti, P_HlQ, 8795)
      call createCurrent(P_ghWm, P_Hh, P_ghAAnti, P_GPlus, 2662)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_HhQ, 2117)
      call createCurrent(P_Z, P_Hh, P_Hh, P_Z, 3990)
      call createCurrent(P_ghWpAnti, P_G0, P_ghWp, P_HlQ, 7419)
      call createCurrent(P_HaQ, P_ghZAnti, P_HPlus, P_ghWm, 89)
      call createCurrent(P_Z, P_Z, P_Ha, P_G0, 10131)
      call createCurrent(P_ghWm, P_WPlus, P_ghWpAnti, P_WPlus, 736)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghWm, P_G0, 9106)
      call createCurrent(P_ZQ, P_WMinus, P_ghAAnti, P_ghWp, 3062)
      call createCurrent(P_WPlus, P_ghAAnti, P_Z, P_ghWm, 6347)
      call createCurrent(P_A, P_ghWp, P_ghWpAnti, P_Z, 6082)
      call createCurrent(P_GPlus, P_A, P_HMinus, 8884)
      call createCurrent(P_Hl, P_A, P_Z, P_Hl, 9983)
      call createCurrent(P_HMinus, P_Hl, P_GPlus, 8475)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HaQ, 6321)
      call createCurrent(P_GQ0, P_Hl, P_GPlus, P_GQMinus, 3283)
      call createCurrent(P_Hl, P_d, P_dAnti, 5215)
      call createCurrent(P_ghZAnti, P_Hh, P_HMinus, P_ghWp, 2028)
      call createCurrent(P_Z, P_HPlus, P_WMinus, P_Hl, 2776)
      call createCurrent(P_WQPlus, P_WMinus, P_G0, P_HhQ, 4551)
      call createCurrent(P_tauPlus, P_tauMinus, P_A, 2878)
      call createCurrent(P_HQPlus, P_ghZAnti, P_HMinus, P_ghZ, 387)
      call createCurrent(P_HlQ, P_G0, P_Hl, P_GQ0, 6086)
      call createCurrent(P_GMinus, P_Hh, P_Hl, P_GPlus, 1082)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghWm, P_Hh, 4450)
      call createCurrent(P_G0, P_Ha, P_Hl, 4076)
      call createCurrent(P_Ha, P_Hh, P_Z, 3368)
      call createCurrent(P_ghWm, P_WPlus, P_ghAAnti, P_Z, 6944)
      call createCurrent(P_GQPlus, P_Z, P_A, P_GQMinus, 1464)
      call createCurrent(P_HaQ, P_GPlus, P_GMinus, P_HaQ, 1957)
      call createCurrent(P_HMinus, P_Z, P_Z, P_GPlus, 10130)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_Ha, 1026)
      call createCurrent(P_GMinus, P_Hh, P_HPlus, P_Ha, 8197)
      call createCurrent(P_HaQ, P_Hl, P_HMinus, P_HQPlus, 4577)
      call createCurrent(P_Hl, P_ghZAnti, P_ghWm, P_HPlus, 2872)
      call createCurrent(P_d, P_Hl, P_dAnti, 5647)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghWp, P_Hh, 3045)
      call createCurrent(P_HQMinus, P_Hh, P_Ha, P_GQPlus, 4126)
      call createCurrent(P_Ha, P_Hh, P_GPlus, P_HMinus, 6827)
      call createCurrent(P_Hl, P_ghWp, P_ghWpAnti, 6027)
      call createCurrent(P_ghZAnti, P_ghZ, P_Ha, P_G0, 9952)
      call createCurrent(P_HQPlus, P_GMinus, P_HMinus, P_GQPlus, 5114)
      call createCurrent(P_HlQ, P_Hh, P_Ha, P_HaQ, 3310)
      call createCurrent(P_HhQ, P_G0, P_G0, P_HhQ, 2430)
      call createCurrent(P_GMinus, P_G0, P_WPlus, P_Z, 7592)
      call createCurrent(P_Ha, P_G0, P_ghWm, P_ghWmAnti, 9385)
      call createCurrent(P_Z, P_G0, P_WMinus, P_HPlus, 10006)
      call createCurrent(P_G0, P_G0, P_Ha, P_Hl, 10607)
      call createCurrent(P_HlQ, P_HMinus, P_HPlus, P_HhQ, 2260)
      call createCurrent(P_ghA, P_GMinus, P_Ha, P_ghWmAnti, 9733)
      call createCurrent(P_HhQ, P_HMinus, P_ghZAnti, P_ghWp, 3286)
      call createCurrent(P_WMinus, P_nu_e, P_ePlus, 6990)
      call createCurrent(P_WMinus, P_GPlus, P_Hh, 4725)
      call createCurrent(P_GQMinus, P_Hl, P_Hl, P_HQPlus, 8186)
      call createCurrent(P_HlQ, P_G0, P_ghWp, P_ghWpAnti, 7108)
      call createCurrent(P_GQMinus, P_GMinus, P_GPlus, P_HQPlus, 1498)
      call createCurrent(P_AQ, P_GMinus, P_A, P_GQPlus, 4863)
      call createCurrent(P_G0, P_ghWm, P_Ha, P_ghWmAnti, 8969)
      call createCurrent(P_ghAAnti, P_G0, P_GMinus, P_ghWp, 1674)
      call createCurrent(P_nu_tauAnti, P_Z, P_nu_tau, 2314)
      call createCurrent(P_WQPlus, P_GMinus, P_HhQ, 2460)
      call createCurrent(P_Z, P_HMinus, P_WPlus, P_Hl, 926)
      call createCurrent(P_Ha, P_WPlus, P_GMinus, P_A, 10054)
      call createCurrent(P_GPlus, P_GMinus, P_Z, 6231)
      call createCurrent(P_WQMinus, P_GPlus, P_Hh, P_ZQ, 5856)
      call createCurrent(P_g, P_Hl, P_g, P_Hl, 10591)
      call createCurrent(P_HQMinus, P_Hh, P_GPlus, P_GQ0, 5057)
      call createCurrent(P_Z, P_Hh, P_Z, P_Hh, 5209)
      call createCurrent(P_WQPlus, P_Z, P_Hl, P_GQMinus, 5746)
      call createCurrent(P_HlQ, P_HPlus, P_G0, P_HQMinus, 4247)
      call createCurrent(P_GQ0, P_Hh, P_Hh, P_HaQ, 2005)
      call createCurrent(P_HhQ, P_A, P_GMinus, P_WQPlus, 3662)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hl, P_HQMinus, 532)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWp, P_GQMinus, 4835)
      call createCurrent(P_Hl, P_HPlus, P_WMinus, 4987)
      call createCurrent(P_ghAAnti, P_Hh, P_ghWp, P_HMinus, 3866)
      call createCurrent(P_GPlus, P_ghWpAnti, P_G0, P_ghA, 1440)
      call createCurrent(P_WPlus, P_GPlus, P_WMinus, P_GMinus, 2706)
      call createCurrent(P_HQPlus, P_A, P_WMinus, P_HhQ, 7662)
      call createCurrent(P_Hh, P_Z, P_WMinus, P_HPlus, 395)
      call createCurrent(P_AQ, P_Hh, P_GPlus, P_WQMinus, 1274)
      call createCurrent(P_Ha, P_ghWm, P_ghWmAnti, P_G0, 10286)
      call createCurrent(P_WQPlus, P_A, P_GMinus, P_HhQ, 6329)
      call createCurrent(P_ghA, P_GPlus, P_Hl, P_ghWpAnti, 4925)
      call createCurrent(P_ghZ, P_ghWpAnti, P_WPlus, 1550)
      call createCurrent(P_ePlus, P_Ha, P_eMinus, 5175)
      call createCurrent(P_A, P_A, P_Hh, P_Hh, 10630)
      call createCurrent(P_A, P_WPlus, P_ghWpAnti, P_ghA, 2637)
      call createCurrent(P_GPlus, P_Ha, P_WMinus, 9803)
      call createCurrent(P_ZQ, P_GMinus, P_G0, P_WQPlus, 7885)
      call createCurrent(P_GMinus, P_ghZAnti, P_HPlus, P_ghZ, 9995)
      call createCurrent(P_Hh, P_ghAAnti, P_GPlus, P_ghWm, 5732)
      call createCurrent(P_HQMinus, P_ghZAnti, P_Hl, P_ghWp, 3117)
      call createCurrent(P_HQPlus, P_WMinus, P_HlQ, 8375)
      call createCurrent(P_G0, P_Hl, P_Hl, P_Ha, 7470)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghZ, P_HMinus, 5382)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Z, P_ZQ, 3457)
      call createCurrent(P_ghA, P_GPlus, P_ghZAnti, P_GMinus, 2073)
      call createCurrent(P_HQPlus, P_HMinus, P_Hl, P_HhQ, 932)
      call createCurrent(P_A, P_g, P_A, P_g, 10768)
      call createCurrent(P_GQMinus, P_Ha, P_HPlus, P_HaQ, 6186)
      call createCurrent(P_HPlus, P_Ha, P_Hh, P_GMinus, 8678)
      call createCurrent(P_HMinus, P_Ha, P_A, P_WPlus, 3409)
      call createCurrent(P_Hl, P_Hl, P_Hl, P_Ha, 10616)
      call createCurrent(P_GQPlus, P_Ha, P_Ha, P_GQMinus, 5888)
      call createCurrent(P_HPlus, P_WMinus, P_A, P_Hh, 4170)
      call createCurrent(P_G0, P_ghZ, P_ghZAnti, P_Ha, 10145)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_HhQ, 8662)
      call createCurrent(P_G0, P_HMinus, P_HPlus, P_Hl, 10196)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_Hh, 7598)
      call createCurrent(P_WMinus, P_Ha, P_A, P_GPlus, 10319)
      call createCurrent(P_ghWmAnti, P_GMinus, P_Hh, P_ghA, 820)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_Z, P_ghA, 1514)
      call createCurrent(P_Ha, P_GPlus, P_HMinus, P_G0, 6172)
      call createCurrent(P_ghZAnti, P_Ha, P_ghWm, P_GPlus, 9085)
      call createCurrent(P_GQ0, P_G0, P_Ha, P_HaQ, 7882)
      call createCurrent(P_GPlus, P_ghA, P_GMinus, P_ghZAnti, 8484)
      call createCurrent(P_ZQ, P_Z, P_G0, P_GQ0, 4829)
      call createCurrent(P_GQ0, P_Hh, P_HMinus, P_GQPlus, 241)
      call createCurrent(P_WQMinus, P_Z, P_GPlus, P_HhQ, 2953)
      call createCurrent(P_HQMinus, P_t, P_bAnti, 2782)
      call createCurrent(P_A, P_Z, P_ghWmAnti, P_ghWm, 5633)
      call createCurrent(P_GQPlus, P_GMinus, P_Hl, P_HaQ, 20)
      call createCurrent(P_ghWp, P_A, P_Z, P_ghWpAnti, 8723)
      call createCurrent(P_WPlus, P_ghWm, P_ghAAnti, 8828)
      call createCurrent(P_G0, P_g, P_g, P_Ha, 10705)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Hh, P_GQMinus, 2365)
      call createCurrent(P_A, P_Z, P_ghWp, P_ghWpAnti, 1508)
      call createCurrent(P_Hl, P_A, P_Z, 9105)
      call createCurrent(P_GQPlus, P_ghZAnti, P_Hl, P_ghWm, 4815)
      call createCurrent(P_GQMinus, P_WPlus, P_Hh, P_ZQ, 6097)
      call createCurrent(P_GMinus, P_u, P_dAnti, 4059)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_Ha, 4921)
      call createCurrent(P_HaQ, P_ghWmAnti, P_Hl, P_ghWm, 6744)
      call createCurrent(P_GMinus, P_Hh, P_GPlus, P_Ha, 8851)
      call createCurrent(P_WPlus, P_ghA, P_WMinus, P_ghAAnti, 7167)
      call createCurrent(P_Z, P_Ha, P_HPlus, P_WMinus, 7179)
      call createCurrent(P_Ha, P_A, P_WMinus, P_GPlus, 10173)
      call createCurrent(P_ZQ, P_Z, P_WMinus, P_WQPlus, 3765)
      call createCurrent(P_HMinus, P_Hl, P_HPlus, P_Ha, 10310)
      call createCurrent(P_WMinus, P_G0, P_WPlus, P_G0, 645)
      call createCurrent(P_A, P_Z, P_Hl, 8933)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghWp, P_HMinus, 5132)
      call createCurrent(P_HPlus, P_HMinus, P_Ha, P_Hh, 8937)
      call createCurrent(P_sAnti, P_G0, P_s, 2744)
      call createCurrent(P_ghGAnti, P_ghG, P_g, 1662)
      call createCurrent(P_Hh, P_G0, P_Z, 293)
      call createCurrent(P_GMinus, P_HPlus, P_Ha, P_Ha, 4177)
      call createCurrent(P_ghWpAnti, P_HMinus, P_ghWp, P_GPlus, 9319)
      call createCurrent(P_GQ0, P_HPlus, P_G0, P_GQMinus, 505)
      call createCurrent(P_G0, P_G0, P_WMinus, P_WPlus, 3041)
      call createCurrent(P_GMinus, P_Hh, P_WPlus, P_Z, 4323)
      call createCurrent(P_A, P_ghWmAnti, P_ghZ, P_WMinus, 840)
      call createCurrent(P_ghWp, P_HMinus, P_Ha, P_ghAAnti, 5105)
      call createCurrent(P_AQ, P_ghWp, P_A, P_ghWpAnti, 4310)
      call createCurrent(P_G0, P_GPlus, P_Hh, P_GMinus, 9998)
      call createCurrent(P_ghWpAnti, P_Hh, P_GPlus, P_ghA, 4446)
      call createCurrent(P_HMinus, P_HPlus, P_ghZ, P_ghAAnti, 3133)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghZ, P_WQPlus, 7870)
      call createCurrent(P_AQ, P_WMinus, P_Hl, P_HQPlus, 2927)
      call createCurrent(P_Hl, P_WMinus, P_WPlus, P_Hh, 10111)
      call createCurrent(P_WPlus, P_ghWpAnti, P_WPlus, P_ghWm, 2943)
      call createCurrent(P_WPlus, P_ghZAnti, P_ghWm, 4146)
      call createCurrent(P_HMinus, P_Z, P_A, P_GPlus, 10231)
      call createCurrent(P_Hl, P_ghWp, P_GMinus, P_ghZAnti, 6760)
      call createCurrent(P_AQ, P_HPlus, P_A, P_HQMinus, 2370)
      call createCurrent(P_tAnti, P_b, P_WPlus, 2634)
      call createCurrent(P_G0, P_Hh, P_G0, P_Hl, 5332)
      call createCurrent(P_WMinus, P_HPlus, P_Hh, 739)
      call createCurrent(P_HMinus, P_Hl, P_HPlus, 5656)
      call createCurrent(P_Hl, P_ghAAnti, P_ghWm, P_GPlus, 912)
      call createCurrent(P_HMinus, P_HPlus, P_ghAAnti, P_ghA, 1075)
      call createCurrent(P_GMinus, P_ghA, P_GPlus, P_ghAAnti, 3558)
      call createCurrent(P_Z, P_G0, P_HMinus, P_WPlus, 10290)
      call createCurrent(P_bAnti, P_b, P_Hh, 5705)
      call createCurrent(P_Hh, P_GPlus, P_HMinus, P_G0, 8217)
      call createCurrent(P_GQPlus, P_Hl, P_WQMinus, 6393)
      call createCurrent(P_GPlus, P_Ha, P_ghWm, P_ghZAnti, 9316)
      call createCurrent(P_GPlus, P_WMinus, P_Ha, P_A, 9218)
      call createCurrent(P_Ha, P_G0, P_G0, P_Hl, 10663)
      call createCurrent(P_cAnti, P_s, P_HQPlus, 8281)
      call createCurrent(P_ghWm, P_GPlus, P_G0, P_ghZAnti, 5407)
      call createCurrent(P_ghAAnti, P_ghWp, P_G0, P_GMinus, 3778)
      call createCurrent(P_WQMinus, P_G0, P_G0, P_WQPlus, 7628)
      call createCurrent(P_t, P_WMinus, P_bAnti, 5631)
      call createCurrent(P_Hl, P_GPlus, P_GMinus, P_Hh, 7731)
      call createCurrent(P_Hl, P_c, P_cAnti, 7693)
      call createCurrent(P_HMinus, P_Hh, P_Z, P_WPlus, 1210)
      call createCurrent(P_HMinus, P_ghZAnti, P_ghA, P_GPlus, 9865)
      call createCurrent(P_HlQ, P_Z, P_WPlus, P_HQMinus, 2201)
      call createCurrent(P_A, P_GMinus, P_WPlus, P_Hl, 502)
      call createCurrent(P_bAnti, P_b, P_HhQ, 7684)
      call createCurrent(P_GPlus, P_WPlus, P_HMinus, P_WMinus, 10303)
      call createCurrent(P_WMinus, P_ghWp, P_ghAAnti, 1857)
      call createCurrent(P_HaQ, P_WPlus, P_WMinus, P_HaQ, 67)
      call createCurrent(P_HlQ, P_WMinus, P_HQPlus, 6747)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghZ, P_HPlus, 7965)
      call createCurrent(P_Hh, P_ghZ, P_ghZAnti, P_Hh, 4623)
      call createCurrent(P_A, P_HPlus, P_Hl, P_WMinus, 1614)
      call createCurrent(P_HPlus, P_ghWp, P_ghWpAnti, P_GMinus, 9816)
      call createCurrent(P_ghZ, P_GPlus, P_ghZAnti, P_GQMinus, 4086)
      call createCurrent(P_Hl, P_GPlus, P_Hh, P_HMinus, 40)
      call createCurrent(P_sAnti, P_s, P_g, 5584)
      call createCurrent(P_HPlus, P_G0, P_ghZAnti, P_ghWm, 10388)
      call createCurrent(P_WQPlus, P_WMinus, P_ghA, P_ghAAnti, 2888)
      call createCurrent(P_HMinus, P_GPlus, P_Ha, P_G0, 4081)
      call createCurrent(P_HQMinus, P_Hl, P_ghWp, P_ghZAnti, 8249)
      call createCurrent(P_ghWmAnti, P_ghWm, P_G0, P_Ha, 9024)
      call createCurrent(P_G0, P_t, P_tAnti, 6102)
      call createCurrent(P_GMinus, P_GPlus, P_ghAAnti, P_ghA, 2874)
      call createCurrent(P_WQPlus, P_nu_eAnti, P_eMinus, 8246)
      call createCurrent(P_tAnti, P_Hl, P_t, 1530)
      call createCurrent(P_ghA, P_HPlus, P_ghWpAnti, P_G0, 9697)
      call createCurrent(P_AQ, P_Hl, P_WMinus, P_GQPlus, 7988)
      call createCurrent(P_bAnti, P_g, P_b, 3616)
      call createCurrent(P_GMinus, P_Hl, P_GPlus, 331)
      call createCurrent(P_WQMinus, P_GPlus, P_Z, P_HlQ, 6107)
      call createCurrent(P_ghZ, P_A, P_ghWpAnti, P_WPlus, 6589)
      call createCurrent(P_Z, P_WMinus, P_Hl, P_HPlus, 4082)
      call createCurrent(P_Hh, P_GPlus, P_Hl, P_GMinus, 185)
      call createCurrent(P_HQPlus, P_HMinus, P_ghZ, P_ghZAnti, 4616)
      call createCurrent(P_Ha, P_HMinus, P_ghWp, P_ghAAnti, 8148)
      call createCurrent(P_HhQ, P_A, P_WPlus, P_HQMinus, 1307)
      call createCurrent(P_HPlus, P_ghA, P_ghWpAnti, P_G0, 10481)
      call createCurrent(P_tAnti, P_t, P_AQ, 2756)
      call createCurrent(P_HQMinus, P_Hl, P_HQPlus, 285)
      call createCurrent(P_GQPlus, P_Hl, P_ghWpAnti, P_ghZ, 2)
      call createCurrent(P_GMinus, P_A, P_GPlus, P_A, 475)
      call createCurrent(P_HPlus, P_ghWmAnti, P_GMinus, P_ghWm, 9003)
      call createCurrent(P_ghZAnti, P_Ha, P_ghWp, P_HQMinus, 7827)
      call createCurrent(P_GPlus, P_ghA, P_ghWpAnti, P_Hl, 5516)
      call createCurrent(P_ghWp, P_ghWpAnti, P_A, P_AQ, 7512)
      call createCurrent(P_Z, P_Hl, P_A, P_Hh, 10652)
      call createCurrent(P_HhQ, P_WMinus, P_WQPlus, 4866)
      call createCurrent(P_ghAAnti, P_ghWm, P_Z, P_WPlus, 4589)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghA, 2265)
      call createCurrent(P_GPlus, P_Z, P_A, P_GMinus, 1641)
      call createCurrent(P_G0, P_ghWp, P_ghWpAnti, P_G0, 6868)
      call createCurrent(P_GQMinus, P_Hl, P_Z, P_WQPlus, 1735)
      call createCurrent(P_Ha, P_HPlus, P_HMinus, 9930)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghWm, P_GPlus, 2622)
      call createCurrent(P_muMinus, P_G0, P_muPlus, 930)
      call createCurrent(P_HaQ, P_HPlus, P_ghWm, P_ghAAnti, 5025)
      call createCurrent(P_GQ0, P_GMinus, P_ghZ, P_ghWmAnti, 3059)
      call createCurrent(P_GMinus, P_ghWmAnti, P_Ha, P_ghA, 9013)
      call createCurrent(P_HMinus, P_Hl, P_WPlus, 2080)
      call createCurrent(P_G0, P_Ha, P_A, P_Z, 10550)
      call createCurrent(P_GPlus, P_WMinus, P_A, 6235)
      call createCurrent(P_HQMinus, P_u, P_dAnti, 8453)
      call createCurrent(P_Ha, P_HPlus, P_Hh, P_GMinus, 6334)
      call createCurrent(P_ghZ, P_Hl, P_ghWmAnti, P_HMinus, 4290)
      call createCurrent(P_tauPlus, P_tauMinus, P_G0, 8270)
      call createCurrent(P_HlQ, P_G0, P_G0, P_HlQ, 4241)
      call createCurrent(P_Hh, P_Hh, P_Z, P_A, 9539)
      call createCurrent(P_HMinus, P_WPlus, P_Ha, P_Z, 6205)
      call createCurrent(P_Z, P_HPlus, P_GMinus, P_Z, 9719)
      call createCurrent(P_ghWm, P_A, P_WPlus, P_ghZAnti, 3419)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghZ, P_Ha, 3587)
      call createCurrent(P_HMinus, P_G0, P_ghWmAnti, P_ghA, 8999)
      call createCurrent(P_Hh, P_HMinus, P_WPlus, 6556)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Ha, P_HPlus, 2108)
      call createCurrent(P_WQMinus, P_ghAAnti, P_WPlus, P_ghA, 6078)
      call createCurrent(P_AQ, P_ghWm, P_Z, P_ghWmAnti, 8418)
      call createCurrent(P_HMinus, P_HPlus, P_ghZAnti, P_ghA, 6695)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Ha, P_HhQ, 7362)
      call createCurrent(P_Hh, P_Hh, P_ghWmAnti, P_ghWm, 3908)
      call createCurrent(P_HlQ, P_Hl, P_Ha, P_GQ0, 100)
      call createCurrent(P_HMinus, P_Hl, P_GPlus, P_Hh, 1261)
      call createCurrent(P_Hh, P_Hl, P_Ha, P_Ha, 6286)
      call createCurrent(P_HaQ, P_GMinus, P_GPlus, P_HaQ, 8254)
      call createCurrent(P_GQMinus, P_G0, P_ghWmAnti, P_ghZ, 7034)
      call createCurrent(P_ghWp, P_ghZAnti, P_Ha, P_GMinus, 10332)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hl, P_HQPlus, 8642)
      call createCurrent(P_A, P_ghA, P_ghWmAnti, P_WMinus, 7082)
      call createCurrent(P_g, P_g, P_WPlus, P_WMinus, 10689)
      call createCurrent(P_HQPlus, P_Hh, P_GQMinus, 340)
      call createCurrent(P_GQPlus, P_HMinus, P_Hh, P_HhQ, 8464)
      call createCurrent(P_ZQ, P_cAnti, P_c, 6839)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghZ, P_GPlus, 8319)
      call createCurrent(P_HhQ, P_Z, P_WMinus, P_HQPlus, 2177)
      call createCurrent(P_WMinus, P_HPlus, P_Ha, 2270)
      call createCurrent(P_G0, P_ghZ, P_GPlus, P_ghWpAnti, 6585)
      call createCurrent(P_HaQ, P_Z, P_HhQ, 8116)
      call createCurrent(P_Ha, P_GMinus, P_ghA, P_ghWmAnti, 8876)
      call createCurrent(P_bAnti, P_b, P_Ha, 4132)
      call createCurrent(P_HMinus, P_GPlus, P_Z, P_A, 9990)
      call createCurrent(P_Ha, P_ghWp, P_ghAAnti, P_HMinus, 3874)
      call createCurrent(P_Hh, P_WMinus, P_Hh, P_WPlus, 1823)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_HhQ, 2837)
      call createCurrent(P_A, P_WMinus, P_GPlus, 2206)
      call createCurrent(P_HPlus, P_HMinus, P_HMinus, P_GPlus, 3076)
      call createCurrent(P_Hl, P_GMinus, P_HPlus, 3099)
      call createCurrent(P_ZQ, P_ghWp, P_A, P_ghWpAnti, 7530)
      call createCurrent(P_tauMinus, P_G0, P_tauPlus, 6164)
      call createCurrent(P_HPlus, P_GMinus, P_Ha, P_G0, 5133)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_Hh, 8277)
      call createCurrent(P_WPlus, P_A, P_HMinus, P_G0, 9390)
      call createCurrent(P_ZQ, P_ghWm, P_WPlus, P_ghZAnti, 2376)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghA, P_G0, 5479)
      call createCurrent(P_GQ0, P_b, P_bAnti, 7112)
      call createCurrent(P_G0, P_ghAAnti, P_ghWp, P_GMinus, 5968)
      call createCurrent(P_G0, P_HPlus, P_ghWpAnti, P_ghZ, 9348)
      call createCurrent(P_Ha, P_Ha, P_WPlus, P_WMinus, 5454)
      call createCurrent(P_Ha, P_ghZAnti, P_ghWp, P_GMinus, 10012)
      call createCurrent(P_Hl, P_Hh, P_Hl, P_Hh, 259)
      call createCurrent(P_HQMinus, P_Ha, P_G0, P_GQPlus, 1904)
      call createCurrent(P_ghAAnti, P_Z, P_ghWp, P_WMinus, 7306)
      call createCurrent(P_ghWpAnti, P_ghWm, P_WPlus, P_WPlus, 6830)
      call createCurrent(P_Hl, P_WMinus, P_HPlus, 7830)
      call createCurrent(P_WPlus, P_Hh, P_WMinus, P_Hh, 5224)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hh, P_Hh, 5055)
      call createCurrent(P_HhQ, P_G0, P_GPlus, P_HQMinus, 8306)
      call createCurrent(P_g, P_g, P_GPlus, P_HMinus, 10729)
      call createCurrent(P_ghA, P_HPlus, P_ghZAnti, P_GMinus, 9892)
      call createCurrent(P_ghWpAnti, P_Ha, P_GPlus, P_ghA, 9712)
      call createCurrent(P_GMinus, P_GPlus, P_HPlus, P_HMinus, 6265)
      call createCurrent(P_ghWmAnti, P_ghWm, P_WPlus, P_WMinus, 8739)
      call createCurrent(P_HQPlus, P_Hl, P_HMinus, P_HhQ, 2997)
      call createCurrent(P_GMinus, P_HPlus, P_ghA, P_ghZAnti, 9911)
      call createCurrent(P_HaQ, P_Ha, P_Ha, P_HaQ, 4596)
      call createCurrent(P_HhQ, P_GPlus, P_HQMinus, 8607)
      call createCurrent(P_Hh, P_GPlus, P_Hh, P_GMinus, 6971)
      call createCurrent(P_WQMinus, P_A, P_HPlus, P_HhQ, 553)
      call createCurrent(P_HMinus, P_Hl, P_G0, P_GPlus, 3985)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HhQ, 3619)
      call createCurrent(P_GPlus, P_ghWm, P_Hh, P_ghZAnti, 8647)
      call createCurrent(P_Z, P_A, P_HPlus, P_GMinus, 9967)
      call createCurrent(P_GQMinus, P_Hh, P_WQPlus, 7274)
      call createCurrent(P_GQMinus, P_G0, P_WPlus, P_ZQ, 7852)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hl, P_Hl, 2031)
      call createCurrent(P_HlQ, P_HPlus, P_HMinus, P_HhQ, 7987)
      call createCurrent(P_HlQ, P_ghWp, P_GMinus, P_ghAAnti, 7119)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghZ, P_HMinus, 6968)
      call createCurrent(P_HQMinus, P_ghA, P_Hh, P_ghWmAnti, 7789)
      call createCurrent(P_HQPlus, P_d, P_uAnti, 8388)
      call createCurrent(P_GQPlus, P_Hh, P_HMinus, P_HaQ, 7432)
      call createCurrent(P_ghWp, P_GMinus, P_Ha, P_ghAAnti, 9116)
      call createCurrent(P_Z, P_nu_eAnti, P_nu_e, 4511)
      call createCurrent(P_ghAAnti, P_ghWp, P_HMinus, P_Hh, 6148)
      call createCurrent(P_GQPlus, P_Z, P_G0, P_WQMinus, 1103)
      call createCurrent(P_HlQ, P_ghZAnti, P_ghZ, 5311)
      call createCurrent(P_HMinus, P_Ha, P_Ha, P_HPlus, 8218)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_HlQ, 5630)
      call createCurrent(P_HMinus, P_Ha, P_ghA, P_ghWmAnti, 2832)
      call createCurrent(P_WMinus, P_u, P_dAnti, 1197)
      call createCurrent(P_ghA, P_ghWmAnti, P_WQMinus, 2040)
      call createCurrent(P_d, P_Ha, P_dAnti, 1185)
      call createCurrent(P_ghWp, P_HMinus, P_Ha, P_ghZAnti, 312)
      call createCurrent(P_GMinus, P_HPlus, P_HMinus, P_GPlus, 6195)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_Hl, 4939)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_ghA, 1706)
      call createCurrent(P_Hl, P_A, P_GPlus, P_WMinus, 3564)
      call createCurrent(P_HMinus, P_ghWp, P_ghAAnti, P_Hh, 2526)
      call createCurrent(P_ghWpAnti, P_ghWp, P_A, P_ZQ, 5443)
      call createCurrent(P_AQ, P_HMinus, P_Z, P_HQPlus, 3714)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghA, P_GMinus, 9909)
      call createCurrent(P_HQMinus, P_G0, P_Ha, P_GQPlus, 38)
      call createCurrent(P_HaQ, P_Hh, P_HPlus, P_HQMinus, 673)
      call createCurrent(P_ghWp, P_ghZAnti, P_G0, P_GMinus, 6798)
      call createCurrent(P_HQMinus, P_G0, P_GPlus, P_GQ0, 6821)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghZ, P_Hh, 996)
      call createCurrent(P_A, P_ghZ, P_ghWmAnti, P_WMinus, 336)
      call createCurrent(P_GPlus, P_eMinus, P_nu_eAnti, 8332)
      call createCurrent(P_tauPlus, P_tauMinus, P_HaQ, 7743)
      call createCurrent(P_ghWm, P_ghZAnti, P_WPlus, P_A, 4068)
      call createCurrent(P_AQ, P_Hh, P_WMinus, P_HQPlus, 8220)
      call createCurrent(P_ghWp, P_Hl, P_ghZAnti, P_GMinus, 8649)
      call createCurrent(P_HaQ, P_tAnti, P_t, 7556)
      call createCurrent(P_WMinus, P_WMinus, P_ghWmAnti, P_ghWp, 5782)
      call createCurrent(P_Ha, P_g, P_g, P_Ha, 10668)
      call createCurrent(P_ePlus, P_nu_e, P_WQMinus, 8546)
      call createCurrent(P_HlQ, P_HPlus, P_GMinus, P_HlQ, 5074)
      call createCurrent(P_HPlus, P_Z, P_WMinus, 1899)
      call createCurrent(P_HhQ, P_HMinus, P_ghWp, P_ghZAnti, 333)
      call createCurrent(P_HhQ, P_ghWpAnti, P_Hh, P_ghWp, 8711)
      call createCurrent(P_ghWp, P_Z, P_ghWpAnti, P_ZQ, 7201)
      call createCurrent(P_AQ, P_WPlus, P_ghAAnti, P_ghWm, 2236)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghA, P_GPlus, 615)
      call createCurrent(P_AQ, P_tauMinus, P_tauPlus, 3183)
      call createCurrent(P_HaQ, P_HMinus, P_ghWmAnti, P_ghZ, 1474)
      call createCurrent(P_HPlus, P_Hh, P_Ha, P_GMinus, 2639)
      call createCurrent(P_HQMinus, P_Ha, P_HPlus, P_HlQ, 6384)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghA, P_GMinus, 2697)
      call createCurrent(P_ghWpAnti, P_HPlus, P_G0, P_ghZ, 10278)
      call createCurrent(P_b, P_tAnti, P_HQPlus, 5424)
      call createCurrent(P_ghA, P_WMinus, P_ghAAnti, P_WQPlus, 8608)
      call createCurrent(P_HPlus, P_GMinus, P_Hh, 3668)
      call createCurrent(P_HQMinus, P_Z, P_WQPlus, 7599)
      call createCurrent(P_Hl, P_G0, P_A, 9301)
      call createCurrent(P_Z, P_WPlus, P_HMinus, 353)
      call createCurrent(P_Z, P_tAnti, P_t, 5905)
      call createCurrent(P_ghZ, P_HMinus, P_ghAAnti, P_GPlus, 10418)
      call createCurrent(P_AQ, P_HPlus, P_HQMinus, 7321)
      call createCurrent(P_HaQ, P_HPlus, P_ghZAnti, P_ghWm, 3488)
      call createCurrent(P_A, P_ghAAnti, P_WPlus, P_ghWm, 2096)
      call createCurrent(P_HQPlus, P_WMinus, P_AQ, 6532)
      call createCurrent(P_Z, P_G0, P_WPlus, P_GMinus, 78)
      call createCurrent(P_Z, P_G0, P_Z, P_G0, 5486)
      call createCurrent(P_ghWp, P_Ha, P_ghZAnti, P_GMinus, 9231)
      call createCurrent(P_Z, P_ghWmAnti, P_ghA, P_WMinus, 768)
      call createCurrent(P_Ha, P_Ha, P_HMinus, P_HPlus, 8130)
      call createCurrent(P_Hh, P_ghWm, P_ghWmAnti, P_Hh, 7041)
      call createCurrent(P_HaQ, P_G0, P_HMinus, P_HQPlus, 2296)
      call createCurrent(P_ghWmAnti, P_A, P_WMinus, P_ghZ, 8285)
      call createCurrent(P_HMinus, P_Z, P_Hl, P_WPlus, 3464)
      call createCurrent(P_ghWpAnti, P_G0, P_G0, P_ghWp, 6466)
      call createCurrent(P_GPlus, P_ghZAnti, P_Ha, P_ghWm, 10295)
      call createCurrent(P_Ha, P_HPlus, P_A, P_WMinus, 5032)
      call createCurrent(P_ghAAnti, P_Hl, P_ghWp, P_GMinus, 7090)
      call createCurrent(P_ZQ, P_G0, P_HlQ, 6094)
      call createCurrent(P_HQPlus, P_Hl, P_Hl, P_GQMinus, 5752)
      call createCurrent(P_G0, P_GPlus, P_ghWpAnti, P_ghZ, 1096)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_HlQ, 6236)
      call createCurrent(P_ghWp, P_ghAAnti, P_Z, P_WMinus, 6829)
      call createCurrent(P_g, P_G0, P_g, P_G0, 10549)
      call createCurrent(P_GPlus, P_ghA, P_GMinus, P_ghAAnti, 4918)
      call createCurrent(P_A, P_ghWm, P_WPlus, P_ghZAnti, 8005)
      call createCurrent(P_GQMinus, P_Ha, P_Ha, P_GQPlus, 8265)
      call createCurrent(P_GQ0, P_HMinus, P_Hl, P_GQPlus, 8582)
      call createCurrent(P_GQMinus, P_HPlus, P_Hl, P_HhQ, 1371)
      call createCurrent(P_ghWmAnti, P_ghWm, P_GQ0, 4919)
      call createCurrent(P_GQPlus, P_Hh, P_Hh, P_HQMinus, 6026)
      call createCurrent(P_WPlus, P_HMinus, P_A, 3429)
      call createCurrent(P_Ha, P_Ha, P_Hl, P_Hl, 759)
      call createCurrent(P_GQPlus, P_G0, P_GMinus, P_GQ0, 2714)
      call createCurrent(P_Ha, P_G0, P_Z, P_A, 10771)
      call createCurrent(P_GQPlus, P_WMinus, P_A, P_HlQ, 6896)
      call createCurrent(P_HPlus, P_ghWmAnti, P_ghWm, P_GMinus, 8897)
      call createCurrent(P_G0, P_HPlus, P_WMinus, P_Z, 9434)
      call createCurrent(P_ZQ, P_nu_muAnti, P_nu_mu, 3467)
      call createCurrent(P_GMinus, P_Ha, P_Hl, P_GPlus, 9237)
      call createCurrent(P_c, P_cAnti, P_g, 6019)
      call createCurrent(P_HPlus, P_A, P_WMinus, P_Hl, 5697)
      call createCurrent(P_g, P_g, P_A, P_A, 10778)
      call createCurrent(P_HlQ, P_ghWp, P_Hl, P_ghWpAnti, 4982)
      call createCurrent(P_WQPlus, P_Ha, P_Z, P_HQMinus, 1801)
      call createCurrent(P_ghWp, P_Hh, P_ghZAnti, P_HMinus, 2502)
      call createCurrent(P_ghA, P_Hh, P_GMinus, P_ghWmAnti, 1545)
      call createCurrent(P_nu_tau, P_HMinus, P_tauPlus, 533)
      call createCurrent(P_ghZ, P_Ha, P_ghWmAnti, P_HQMinus, 7140)
      call createCurrent(P_GMinus, P_GPlus, P_ghWpAnti, P_ghWp, 437)
      call createCurrent(P_A, P_HPlus, P_Z, P_GMinus, 9893)
      call createCurrent(P_HlQ, P_G0, P_HMinus, P_GQPlus, 2004)
      call createCurrent(P_WMinus, P_HPlus, P_G0, P_A, 10484)
      call createCurrent(P_GMinus, P_Z, P_Hh, P_WPlus, 327)
      call createCurrent(P_GMinus, P_ghWmAnti, P_Hh, P_ghZ, 1891)
      call createCurrent(P_Hh, P_WMinus, P_Z, P_HPlus, 7234)
      call createCurrent(P_ghWmAnti, P_Ha, P_GMinus, P_ghZ, 10221)
      call createCurrent(P_WMinus, P_WPlus, P_A, 6477)
      call createCurrent(P_Z, P_G0, P_GMinus, P_WPlus, 332)
      call createCurrent(P_GQPlus, P_GMinus, P_Ha, P_HhQ, 6096)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghWp, P_Hh, 3012)
      call createCurrent(P_HlQ, P_HPlus, P_Hh, P_GQMinus, 5831)
      call createCurrent(P_ghAAnti, P_GPlus, P_GMinus, P_ghA, 3040)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hl, P_GMinus, 8800)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghZ, P_WMinus, 7959)
      call createCurrent(P_HMinus, P_HPlus, P_ghZ, P_ghZAnti, 8353)
      call createCurrent(P_Hl, P_Z, P_WPlus, P_HMinus, 8069)
      call createCurrent(P_ZQ, P_nu_mu, P_nu_muAnti, 965)
      call createCurrent(P_Ha, P_GPlus, P_ghZ, P_ghWpAnti, 9699)
      call createCurrent(P_Hh, P_sAnti, P_s, 5197)
      call createCurrent(P_nu_muAnti, P_muMinus, P_WQPlus, 2513)
      call createCurrent(P_WQPlus, P_G0, P_WMinus, P_HhQ, 4369)
      call createCurrent(P_HQMinus, P_Ha, P_GPlus, P_HlQ, 5457)
      call createCurrent(P_GQMinus, P_G0, P_HQPlus, 5122)
      call createCurrent(P_GQMinus, P_dAnti, P_u, 2678)
      call createCurrent(P_Hl, P_HPlus, P_G0, P_HMinus, 9432)
      call createCurrent(P_ZQ, P_Z, P_Hh, P_HhQ, 2015)
      call createCurrent(P_GPlus, P_ghWm, P_ghZAnti, 5067)
      call createCurrent(P_ghZAnti, P_HPlus, P_Hl, P_ghWm, 5675)
      call createCurrent(P_G0, P_HMinus, P_HPlus, P_G0, 1184)
      call createCurrent(P_G0, P_Ha, P_ghWm, P_ghWmAnti, 8942)
      call createCurrent(P_GQ0, P_HMinus, P_Ha, P_GQPlus, 3158)
      call createCurrent(P_Hh, P_GMinus, P_GPlus, P_G0, 8907)
      call createCurrent(P_GQMinus, P_HPlus, P_HhQ, 4123)
      call createCurrent(P_WMinus, P_WPlus, P_ghAAnti, P_ghZ, 2673)
      call createCurrent(P_G0, P_ghZ, P_ghWpAnti, P_HPlus, 8845)
      call createCurrent(P_Z, P_Hl, P_WMinus, P_HPlus, 506)
      call createCurrent(P_ghWp, P_GPlus, P_GMinus, P_ghWpAnti, 5513)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghZ, P_HQMinus, 319)
      call createCurrent(P_WMinus, P_WPlus, P_ghZAnti, P_ghA, 2736)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hh, P_GPlus, 4778)
      call createCurrent(P_muPlus, P_nu_mu, P_WQMinus, 2067)
      call createCurrent(P_GMinus, P_GPlus, P_HMinus, P_HPlus, 4202)
      call createCurrent(P_GQ0, P_G0, P_Hl, P_HhQ, 1585)
      call createCurrent(P_G0, P_ghWp, P_G0, P_ghWpAnti, 789)
      call createCurrent(P_WQPlus, P_eMinus, P_nu_eAnti, 5727)
      call createCurrent(P_WMinus, P_muPlus, P_nu_mu, 1053)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghZ, P_HMinus, 1190)
      call createCurrent(P_GQMinus, P_G0, P_GPlus, P_HhQ, 1291)
      call createCurrent(P_Hl, P_Hl, P_Hh, 4680)
      call createCurrent(P_WPlus, P_ghZAnti, P_WMinus, P_ghZ, 4889)
      call createCurrent(P_ghAAnti, P_Hh, P_GPlus, P_ghWm, 3599)
      call createCurrent(P_HQMinus, P_WPlus, P_Ha, P_ZQ, 1677)
      call createCurrent(P_Ha, P_Z, P_WMinus, P_GPlus, 10282)
      call createCurrent(P_HQMinus, P_Hh, P_GPlus, P_HaQ, 8309)
      call createCurrent(P_ghWm, P_GPlus, P_Hh, P_ghZAnti, 7773)
      call createCurrent(P_ghA, P_ghWpAnti, P_WPlus, 3381)
      call createCurrent(P_GPlus, P_ghWpAnti, P_GMinus, P_ghWp, 2287)
      call createCurrent(P_HQMinus, P_ghZAnti, P_Hh, P_ghWp, 2984)
      call createCurrent(P_ghAAnti, P_ghA, P_HPlus, P_HMinus, 4397)
      call createCurrent(P_ghWp, P_G0, P_ghWpAnti, P_HhQ, 4006)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghA, P_GQMinus, 6457)
      call createCurrent(P_ghA, P_ghWpAnti, P_A, P_WPlus, 7848)
      call createCurrent(P_A, P_Z, P_HMinus, P_GPlus, 9086)
      call createCurrent(P_G0, P_G0, P_GPlus, P_GMinus, 6360)
      call createCurrent(P_WPlus, P_Ha, P_Z, P_HMinus, 5744)
      call createCurrent(P_ZQ, P_HMinus, P_Hl, P_WQPlus, 4603)
      call createCurrent(P_Hl, P_ghWmAnti, P_Hl, P_ghWm, 8366)
      call createCurrent(P_GMinus, P_muPlus, P_nu_mu, 3920)
      call createCurrent(P_GQ0, P_Hl, P_Ha, P_HhQ, 7766)
      call createCurrent(P_Z, P_WPlus, P_Hh, P_GMinus, 983)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_Hh, 7612)
      call createCurrent(P_HaQ, P_GPlus, P_Ha, P_GQMinus, 3494)
      call createCurrent(P_GMinus, P_Hl, P_WPlus, P_A, 8680)
      call createCurrent(P_HaQ, P_G0, P_HPlus, P_GQMinus, 2784)
      call createCurrent(P_HlQ, P_Hh, P_GMinus, P_GQPlus, 2542)
      call createCurrent(P_HMinus, P_ghWp, P_G0, P_ghZAnti, 9884)
      call createCurrent(P_Z, P_WPlus, P_ghA, P_ghWpAnti, 2294)
      call createCurrent(P_GQPlus, P_GMinus, P_GMinus, P_HQPlus, 7757)
      call createCurrent(P_eMinus, P_nu_eAnti, P_GQPlus, 4727)
      call createCurrent(P_HlQ, P_Hl, P_GMinus, P_HQPlus, 6204)
      call createCurrent(P_d, P_dAnti, P_GQ0, 7394)
      call createCurrent(P_ghWmAnti, P_Z, P_Z, P_ghWm, 6675)
      call createCurrent(P_GPlus, P_GMinus, P_ghAAnti, P_ghZ, 7816)
      call createCurrent(P_GQMinus, P_WPlus, P_Hl, P_ZQ, 2572)
      call createCurrent(P_HQMinus, P_GPlus, P_GMinus, P_HQPlus, 3820)
      call createCurrent(P_HaQ, P_muMinus, P_muPlus, 4503)
      call createCurrent(P_Hl, P_A, P_Ha, 9540)
      call createCurrent(P_HQPlus, P_ghA, P_Hl, P_ghWpAnti, 5128)
      call createCurrent(P_GMinus, P_Hh, P_Hh, P_HPlus, 1793)
      call createCurrent(P_HhQ, P_HPlus, P_Z, P_WQMinus, 2743)
      call createCurrent(P_ghWpAnti, P_ghWp, P_WPlus, P_WMinus, 4505)
      call createCurrent(P_WQPlus, P_ghZ, P_WMinus, P_ghAAnti, 5491)
      call createCurrent(P_AQ, P_Hh, P_HMinus, P_WQPlus, 4355)
      call createCurrent(P_g, P_GPlus, P_HMinus, P_g, 10761)
      call createCurrent(P_HhQ, P_ghWpAnti, P_HPlus, P_ghZ, 7745)
      call createCurrent(P_ghWm, P_Z, P_ghZAnti, P_WPlus, 7319)
      call createCurrent(P_HlQ, P_ghZ, P_GMinus, P_ghWmAnti, 7583)
      call createCurrent(P_Ha, P_ghZAnti, P_GPlus, P_ghWm, 9542)
      call createCurrent(P_Ha, P_Hh, P_Ha, P_Hl, 2022)
      call createCurrent(P_WMinus, P_ghZ, P_ghZAnti, P_WPlus, 2212)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GQMinus, 2581)
      call createCurrent(P_b, P_bAnti, 9648)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hl, P_HlQ, 7152)
      call createCurrent(P_HPlus, P_GMinus, P_G0, P_Ha, 6244)
      call createCurrent(P_g, P_HPlus, P_g, P_GMinus, 10512)
      call createCurrent(P_GQPlus, P_GMinus, P_Hh, P_HhQ, 8247)
      call createCurrent(P_HaQ, P_ghWmAnti, P_HMinus, P_ghZ, 2763)
      call createCurrent(P_dAnti, P_u, P_GQMinus, 8522)
      call createCurrent(P_GQMinus, P_HMinus, P_GPlus, P_HQPlus, 6065)
      call createCurrent(P_WQPlus, P_WMinus, P_ghAAnti, P_ghZ, 7098)
      call createCurrent(P_G0, P_GPlus, P_A, P_WMinus, 5080)
      call createCurrent(P_Z, P_G0, 9190)
      call createCurrent(P_ghWpAnti, P_ghA, P_Z, P_WQPlus, 1600)
      call createCurrent(P_HaQ, P_HMinus, P_Hh, P_HQPlus, 6975)
      call createCurrent(P_ghWmAnti, P_Z, P_ghZ, P_WQMinus, 2508)
      call createCurrent(P_tauMinus, P_A, P_tauPlus, 8717)
      call createCurrent(P_ghA, P_G0, P_ghWpAnti, P_GPlus, 4731)
      call createCurrent(P_HQMinus, P_HMinus, P_GPlus, P_GQPlus, 6373)
      call createCurrent(P_WQPlus, P_Hl, P_Hl, P_WQMinus, 7009)
      call createCurrent(P_Z, P_WPlus, P_G0, P_HMinus, 10158)
      call createCurrent(P_ghWmAnti, P_HMinus, P_G0, P_ghZ, 10139)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hl, 7775)
      call createCurrent(P_uAnti, P_g, P_u, 4886)
      call createCurrent(P_G0, P_ghWp, P_GMinus, P_ghZAnti, 4844)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Z, P_WQPlus, 2121)
      call createCurrent(P_ghAAnti, P_ghWm, P_Hl, P_HPlus, 5176)
      call createCurrent(P_WQPlus, P_ghA, P_ghWpAnti, 5558)
      call createCurrent(P_ghA, P_ghAAnti, P_HPlus, P_HQMinus, 429)
      call createCurrent(P_GMinus, P_ghZ, P_ghWmAnti, P_Ha, 9175)
      call createCurrent(P_ghWpAnti, P_WMinus, P_WPlus, P_ghWp, 4357)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hh, P_GPlus, 6338)
      call createCurrent(P_G0, P_HMinus, P_G0, P_HPlus, 307)
      call createCurrent(P_WQMinus, P_G0, P_WPlus, P_HlQ, 1858)
      call createCurrent(P_WQPlus, P_A, P_HMinus, P_HhQ, 1911)
      call createCurrent(P_WQPlus, P_WMinus, P_G0, P_HlQ, 3605)
      call createCurrent(P_gQ, P_u, P_uAnti, 6468)
      call createCurrent(P_HaQ, P_HPlus, P_G0, P_GQMinus, 586)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HMinus, P_GPlus, 9302)
      call createCurrent(P_GPlus, P_Z, P_GMinus, P_Z, 2942)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghWp, P_HaQ, 7820)
      call createCurrent(P_GPlus, P_WMinus, P_G0, P_A, 6358)
      call createCurrent(P_HQMinus, P_sAnti, P_c, 1610)
      call createCurrent(P_GPlus, P_G0, P_G0, P_HMinus, 8797)
      call createCurrent(P_HQMinus, P_GPlus, P_GMinus, P_GQPlus, 5446)
      call createCurrent(P_HlQ, P_ghZ, P_ghZAnti, 631)
      call createCurrent(P_HPlus, P_Hl, P_GMinus, P_Hh, 7308)
      call createCurrent(P_HMinus, P_Hh, P_GPlus, P_G0, 2186)
      call createCurrent(P_GMinus, P_ghWp, P_Ha, P_ghAAnti, 9421)
      call createCurrent(P_WQMinus, P_ghZ, P_WPlus, P_ghAAnti, 3242)
      call createCurrent(P_Ha, P_HMinus, P_ghZ, P_ghWmAnti, 6613)
      call createCurrent(P_ghZAnti, P_ghZ, P_HhQ, 4105)
      call createCurrent(P_GQ0, P_Z, P_HlQ, 1245)
      call createCurrent(P_GQMinus, P_nu_e, P_ePlus, 5045)
      call createCurrent(P_GPlus, P_G0, P_Ha, P_HMinus, 3096)
      call createCurrent(P_ghA, P_Ha, P_HMinus, P_ghWmAnti, 4280)
      call createCurrent(P_ZQ, P_GMinus, P_WQPlus, 4067)
      call createCurrent(P_ghZAnti, P_GPlus, P_Hl, P_ghWm, 7828)
      call createCurrent(P_Z, P_WPlus, P_HMinus, P_G0, 9546)
      call createCurrent(P_ghZ, P_ghZAnti, P_HMinus, P_GPlus, 10220)
      call createCurrent(P_GQ0, P_GPlus, P_GQMinus, 3081)
      call createCurrent(P_WPlus, P_HMinus, P_G0, P_Z, 9061)
      call createCurrent(P_Hl, P_WMinus, P_Z, P_GPlus, 6138)
      call createCurrent(P_HlQ, P_HMinus, P_HPlus, P_HlQ, 5901)
      call createCurrent(P_WMinus, P_Z, P_ghWp, P_ghZAnti, 3568)
      call createCurrent(P_Z, P_Ha, P_Hh, 4248)
      call createCurrent(P_ghA, P_GPlus, P_Ha, P_ghWpAnti, 9229)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_Hl, 8386)
      call createCurrent(P_HPlus, P_Hh, P_Hh, P_GMinus, 4249)
      call createCurrent(P_Hl, P_GMinus, P_GPlus, P_G0, 9357)
      call createCurrent(P_Ha, P_ghA, P_ghWpAnti, P_HPlus, 3162)
      call createCurrent(P_nu_muAnti, P_muMinus, P_GPlus, 750)
      call createCurrent(P_GQMinus, P_ghZAnti, P_GPlus, P_ghA, 5079)
      call createCurrent(P_ghZ, P_Z, P_ghWmAnti, P_WMinus, 2543)
      call createCurrent(P_c, P_cAnti, P_Ha, 1693)
      call createCurrent(P_ghWp, P_ghAAnti, P_Hh, P_GMinus, 5035)
      call createCurrent(P_bAnti, P_b, P_gQ, 7376)
      call createCurrent(P_A, P_Hh, P_WMinus, P_HPlus, 5046)
      call createCurrent(P_HQPlus, P_Z, P_HQMinus, 2345)
      call createCurrent(P_Ha, P_Ha, P_Hh, 4443)
      call createCurrent(P_Hh, P_GPlus, P_ghWpAnti, P_ghZ, 2003)
      call createCurrent(P_WPlus, P_GMinus, P_Ha, P_Z, 8901)
      call createCurrent(P_ghA, P_Ha, P_ghWpAnti, P_GPlus, 9796)
      call createCurrent(P_ghWmAnti, P_ghA, P_HQMinus, 5695)
      call createCurrent(P_HQMinus, P_Hh, P_ghZ, P_ghWmAnti, 7944)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghA, P_WMinus, 4449)
      call createCurrent(P_g, P_ghGAnti, P_g, P_ghG, 4133)
      call createCurrent(P_Ha, P_G0, P_g, P_g, 10716)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_GQ0, 3715)
      call createCurrent(P_G0, P_ghZAnti, P_ghWm, P_HPlus, 9140)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_GQ0, 6993)
      call createCurrent(P_G0, P_ghWm, P_HPlus, P_ghZAnti, 10072)
      call createCurrent(P_gQ, P_ghG, P_ghGAnti, 617)
      call createCurrent(P_Hh, P_ghWm, P_Hl, P_ghWmAnti, 9272)
      call createCurrent(P_HlQ, P_ePlus, P_eMinus, 6214)
      call createCurrent(P_Hl, P_HMinus, P_GPlus, P_Hl, 7359)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HPlus, P_GMinus, 10096)
      call createCurrent(P_A, P_dAnti, P_d, 1253)
      call createCurrent(P_A, P_bAnti, P_b, 6669)
      call createCurrent(P_WQMinus, P_A, P_HPlus, P_HlQ, 5123)
      call createCurrent(P_gQ, P_ghGAnti, P_ghG, 1394)
      call createCurrent(P_Hh, P_Hh, P_Hh, P_Hh, 3383)
      call createCurrent(P_sAnti, P_s, 9918)
      call createCurrent(P_HQMinus, P_GPlus, P_GPlus, P_GQMinus, 151)
      call createCurrent(P_HPlus, P_ghA, P_ghWpAnti, 1236)
      call createCurrent(P_ghA, P_ghWpAnti, P_A, P_WQPlus, 8323)
      call createCurrent(P_GPlus, P_GMinus, P_Hl, 3730)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_HhQ, 1431)
      call createCurrent(P_GQPlus, P_Hl, P_GMinus, P_GQ0, 3740)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hh, 2210)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_Z, P_ghZ, 7605)
      call createCurrent(P_ghAAnti, P_GMinus, P_GPlus, P_ghZ, 8578)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghZ, P_G0, 7948)
      call createCurrent(P_ghWp, P_WMinus, P_ghZAnti, 3265)
      call createCurrent(P_HaQ, P_uAnti, P_u, 606)
      call createCurrent(P_Z, P_Z, P_Hl, 4112)
      call createCurrent(P_WQMinus, P_G0, P_WPlus, P_HhQ, 6865)
      call createCurrent(P_Hl, P_HMinus, P_Hl, P_GPlus, 6093)
      call createCurrent(P_ghA, P_ghWpAnti, P_HPlus, P_Hl, 2351)
      call createCurrent(P_G0, P_Ha, P_G0, P_Ha, 33)
      call createCurrent(P_cAnti, P_G0, P_c, 7007)
      call createCurrent(P_tAnti, P_t, P_GQ0, 6832)
      call createCurrent(P_HaQ, P_HMinus, P_ghAAnti, P_ghWp, 8321)
      call createCurrent(P_HMinus, P_A, P_A, P_GPlus, 9060)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghA, P_GQPlus, 3319)
      call createCurrent(P_dAnti, P_Hl, P_d, 3425)
      call createCurrent(P_ZQ, P_GPlus, P_WQMinus, 8294)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_HlQ, 6460)
      call createCurrent(P_ghZ, P_GPlus, P_ghZAnti, P_GMinus, 6271)
      call createCurrent(P_ZQ, P_GPlus, P_GMinus, P_AQ, 7333)
      call createCurrent(P_Ha, P_HMinus, P_Ha, P_HPlus, 877)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_G0, 9011)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Ha, P_HQPlus, 8149)
      call createCurrent(P_WQPlus, P_WMinus, P_A, P_ZQ, 6424)
      call createCurrent(P_ghZAnti, P_ghWp, P_WQMinus, 812)
      call createCurrent(P_HPlus, P_Hl, P_HMinus, 7344)
      call createCurrent(P_Hl, P_Hl, P_g, P_g, 10750)
      call createCurrent(P_AQ, P_A, P_HPlus, P_HQMinus, 5)
      call createCurrent(P_WPlus, P_Z, P_GMinus, P_Hh, 763)
      call createCurrent(P_Z, P_WPlus, P_Hl, P_GMinus, 4674)
      call createCurrent(P_G0, P_Ha, 9422)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghWp, P_Z, 7048)
      call createCurrent(P_Hl, P_Hh, P_Hh, P_Hh, 3055)
      call createCurrent(P_Hl, P_HMinus, P_Hh, P_HPlus, 3721)
      call createCurrent(P_HMinus, P_ghWp, P_ghAAnti, P_Ha, 402)
      call createCurrent(P_GPlus, P_GMinus, P_Ha, P_Ha, 2309)
      call createCurrent(P_GMinus, P_GPlus, P_A, P_Z, 3774)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghZ, P_HPlus, 4386)
      call createCurrent(P_HlQ, P_ghWpAnti, P_GPlus, P_ghZ, 4187)
      call createCurrent(P_ghZ, P_Hl, P_HMinus, P_ghWmAnti, 427)
      call createCurrent(P_ghA, P_WMinus, P_ghZAnti, P_WPlus, 5159)
      call createCurrent(P_ghWp, P_ghAAnti, P_G0, P_HMinus, 9177)
      call createCurrent(P_Hh, P_ghWp, P_GMinus, P_ghZAnti, 5204)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghZ, P_HPlus, 6835)
      call createCurrent(P_GQPlus, P_Ha, P_GMinus, P_GQ0, 7525)
      call createCurrent(P_HQMinus, P_WPlus, P_A, P_HhQ, 6955)
      call createCurrent(P_GQ0, P_G0, P_ghZAnti, P_ghZ, 7939)
      call createCurrent(P_ghZAnti, P_Ha, P_ghZ, P_HaQ, 6659)
      call createCurrent(P_WMinus, P_WPlus, P_GMinus, P_HPlus, 9704)
      call createCurrent(P_Ha, P_WMinus, P_A, P_HPlus, 7829)
      call createCurrent(P_ghA, P_HMinus, P_ghWmAnti, 1212)
      call createCurrent(P_GPlus, P_G0, P_ghWm, P_ghAAnti, 4811)
      call createCurrent(P_ghZ, P_GMinus, P_Ha, P_ghWmAnti, 9547)
      call createCurrent(P_WPlus, P_Hl, P_WMinus, P_Hh, 9136)
      call createCurrent(P_HPlus, P_Hl, P_GMinus, P_G0, 3934)
      call createCurrent(P_Hl, P_WPlus, P_WMinus, P_Hl, 256)
      call createCurrent(P_WPlus, P_A, P_GMinus, P_Hl, 5547)
      call createCurrent(P_Hl, P_GPlus, P_HMinus, P_Ha, 6228)
      call createCurrent(P_HMinus, P_Hl, P_HPlus, P_Hl, 6691)
      call createCurrent(P_ghA, P_GMinus, P_HPlus, P_ghZAnti, 9939)
      call createCurrent(P_GQPlus, P_G0, P_WMinus, P_AQ, 1566)
      call createCurrent(P_GQ0, P_ghWm, P_G0, P_ghWmAnti, 1282)
      call createCurrent(P_GPlus, P_A, P_G0, P_WMinus, 6606)
      call createCurrent(P_bAnti, P_Hl, P_b, 6823)
      call createCurrent(P_Ha, P_GMinus, P_WPlus, P_A, 8910)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_Ha, 7261)
      call createCurrent(P_GQ0, P_Hl, P_WMinus, P_WQPlus, 3918)
      call createCurrent(P_GMinus, P_Hh, P_WPlus, P_A, 4430)
      call createCurrent(P_ghWp, P_ghZAnti, P_A, P_WMinus, 5434)
      call createCurrent(P_GPlus, P_Hh, P_GMinus, P_Hh, 8199)
      call createCurrent(P_GMinus, P_GPlus, P_WPlus, P_WMinus, 2989)
      call createCurrent(P_HaQ, P_Hl, P_GPlus, P_HQMinus, 4019)
      call createCurrent(P_WQMinus, P_WPlus, P_ghAAnti, P_ghA, 3428)
      call createCurrent(P_Z, P_Hl, P_Hl, P_A, 10370)
      call createCurrent(P_HhQ, P_Ha, P_GQ0, 3160)
      call createCurrent(P_GMinus, P_GPlus, P_G0, 9936)
      call createCurrent(P_WMinus, P_Z, P_WPlus, P_A, 1489)
      call createCurrent(P_HQPlus, P_Ha, P_GMinus, P_HlQ, 284)
      call createCurrent(P_WMinus, P_GPlus, P_Hh, P_A, 5383)
      call createCurrent(P_nu_mu, P_muPlus, P_WMinus, 4462)
      call createCurrent(P_g, P_g, P_ghGAnti, P_ghG, 1734)
      call createCurrent(P_HaQ, P_Hl, P_Ha, P_HhQ, 6226)
      call createCurrent(P_ghA, P_Z, P_ghWpAnti, P_WPlus, 3622)
      call createCurrent(P_Ha, P_Ha, P_Z, P_A, 10292)
      call createCurrent(P_HaQ, P_G0, P_HMinus, P_GQPlus, 8106)
      call createCurrent(P_GQMinus, P_Hl, P_GPlus, P_HlQ, 3933)
      call createCurrent(P_WPlus, P_Hh, P_GMinus, P_A, 1424)
      call createCurrent(P_GQMinus, P_Hl, P_WPlus, P_ZQ, 3413)
      call createCurrent(P_GQMinus, P_Hh, P_WPlus, P_AQ, 2449)
      call createCurrent(P_ghAAnti, P_Hh, P_GMinus, P_ghWp, 3863)
      call createCurrent(P_WPlus, P_GMinus, P_Z, P_Hl, 1426)
      call createCurrent(P_Ha, P_ghZ, P_ghWpAnti, P_HPlus, 6537)
      call createCurrent(P_GQPlus, P_HMinus, P_Hh, P_HaQ, 7700)
      call createCurrent(P_WQPlus, P_Z, P_HQMinus, 512)
      call createCurrent(P_Hh, P_tauPlus, P_tauMinus, 8737)
      call createCurrent(P_WPlus, P_A, P_ghAAnti, P_ghWm, 4480)
      call createCurrent(P_AQ, P_HMinus, P_WQPlus, 3503)
      call createCurrent(P_g, P_WMinus, P_g, P_WPlus, 10720)
      call createCurrent(P_GQMinus, P_A, P_GQPlus, 3195)
      call createCurrent(P_GPlus, P_ghWpAnti, P_ghA, P_Ha, 10379)
      call createCurrent(P_ghWpAnti, P_Z, P_WPlus, P_ghZ, 7843)
      call createCurrent(P_A, P_A, P_WPlus, P_WMinus, 6495)
      call createCurrent(P_Z, P_GMinus, P_GPlus, P_A, 7548)
      call createCurrent(P_WMinus, P_ghWmAnti, P_WPlus, P_ghWm, 8781)
      call createCurrent(P_GQPlus, P_G0, P_Hl, P_HQMinus, 7758)
      call createCurrent(P_ghAAnti, P_Ha, P_ghWm, P_GPlus, 10371)
      call createCurrent(P_GPlus, P_muMinus, P_nu_muAnti, 2733)
      call createCurrent(P_c, P_Ha, P_cAnti, 4467)
      call createCurrent(P_Ha, P_ghA, P_ghWmAnti, P_HMinus, 7064)
      call createCurrent(P_GQ0, P_HMinus, P_HPlus, P_HaQ, 5333)
      call createCurrent(P_WQPlus, P_GMinus, P_G0, P_ZQ, 3280)
      call createCurrent(P_Z, P_GMinus, P_HPlus, 8917)
      call createCurrent(P_WPlus, P_ghWm, P_Z, P_ghZAnti, 1111)
      call createCurrent(P_ghAAnti, P_GPlus, P_GMinus, P_ghZ, 8094)
      call createCurrent(P_Hh, P_Hl, P_Z, P_Z, 9701)
      call createCurrent(P_G0, P_A, P_WMinus, P_HPlus, 10444)
      call createCurrent(P_ghZAnti, P_Hl, P_GPlus, P_ghWm, 4444)
      call createCurrent(P_GQMinus, P_Hh, P_WPlus, P_ZQ, 7100)
      call createCurrent(P_WQMinus, P_GPlus, P_Hl, P_AQ, 5195)
      call createCurrent(P_Ha, P_ghAAnti, P_ghWm, P_HPlus, 3754)
      call createCurrent(P_Hh, P_ghA, P_HMinus, P_ghWmAnti, 6965)
      call createCurrent(P_g, P_A, P_g, P_A, 10554)
      call createCurrent(P_HMinus, P_A, P_HPlus, 5868)
      call createCurrent(P_HhQ, P_Hh, P_HPlus, P_GQMinus, 4466)
      call createCurrent(P_ZQ, P_GMinus, P_GQPlus, 541)
      call createCurrent(P_GMinus, P_HMinus, P_GPlus, P_HPlus, 7884)
      call createCurrent(P_WMinus, P_HPlus, P_A, P_Hl, 6043)
      call createCurrent(P_ghZAnti, P_HMinus, P_HPlus, P_ghZ, 8814)
      call createCurrent(P_ghA, P_Hh, P_ghWmAnti, P_GQMinus, 3320)
      call createCurrent(P_ghWp, P_ghAAnti, P_WMinus, P_ZQ, 7395)
      call createCurrent(P_HPlus, P_HMinus, P_ghAAnti, P_ghA, 8066)
      call createCurrent(P_nu_tauAnti, P_HPlus, P_tauMinus, 7705)
      call createCurrent(P_GQMinus, P_Z, P_WPlus, P_HlQ, 7837)
      call createCurrent(P_WPlus, P_Hl, P_HMinus, P_A, 2124)
      call createCurrent(P_GQ0, P_HPlus, P_Ha, P_GQMinus, 4969)
      call createCurrent(P_ghWp, P_WMinus, P_WMinus, P_ghWmAnti, 5835)
      call createCurrent(P_G0, P_Ha, P_Hh, P_Hh, 379)
      call createCurrent(P_HlQ, P_GMinus, P_Hl, P_GQPlus, 7305)
      call createCurrent(P_d, P_uAnti, P_GPlus, 6685)
      call createCurrent(P_HQMinus, P_tauPlus, P_nu_tau, 3837)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Z, P_WQMinus, 7465)
      call createCurrent(P_ghZ, P_ghAAnti, P_HMinus, P_HQPlus, 8627)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghA, 5849)
      call createCurrent(P_GQ0, P_G0, P_Z, P_ZQ, 1462)
      call createCurrent(P_HMinus, P_G0, P_HPlus, P_Ha, 5538)
      call createCurrent(P_ghWm, P_Hh, P_ghWmAnti, P_Hh, 779)
      call createCurrent(P_G0, P_HMinus, P_ghWp, P_ghAAnti, 10480)
      call createCurrent(P_ghWmAnti, P_G0, P_GMinus, P_ghA, 8296)
      call createCurrent(P_ePlus, P_eMinus, P_Z, 3327)
      call createCurrent(P_ghWp, P_GMinus, P_G0, P_ghZAnti, 6696)
      call createCurrent(P_Hh, P_HMinus, P_ghAAnti, P_ghWp, 1169)
      call createCurrent(P_cAnti, P_Hh, P_c, 7497)
      call createCurrent(P_ghZAnti, P_ghWp, P_WMinus, P_Z, 6095)
      call createCurrent(P_GQMinus, P_WPlus, P_Z, P_GQ0, 1275)
      call createCurrent(P_ghZAnti, P_Hh, P_ghZ, P_Hl, 8948)
      call createCurrent(P_GPlus, P_HMinus, P_HPlus, P_GMinus, 6800)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghZ, P_Hl, 2020)
      call createCurrent(P_ghA, P_WMinus, P_WPlus, P_ghAAnti, 2821)
      call createCurrent(P_GMinus, P_ghWp, P_Hh, P_ghZAnti, 4552)
      call createCurrent(P_HaQ, P_G0, P_Hl, P_HlQ, 384)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Z, P_ZQ, 7008)
      call createCurrent(P_HPlus, P_WMinus, P_Ha, P_Z, 3187)
      call createCurrent(P_HPlus, P_GPlus, P_GMinus, P_GMinus, 7483)
      call createCurrent(P_G0, P_ghA, P_HMinus, P_ghWmAnti, 8987)
      call createCurrent(P_c, P_cAnti, P_Z, 7656)
      call createCurrent(P_G0, P_HPlus, P_ghWm, P_ghAAnti, 10457)
      call createCurrent(P_Hl, P_Hh, P_Ha, P_Ha, 5627)
      call createCurrent(P_GQ0, P_HPlus, P_Hh, P_HQMinus, 4983)
      call createCurrent(P_GPlus, P_Hl, P_GMinus, P_Hh, 725)
      call createCurrent(P_HQPlus, P_Ha, P_GMinus, P_HaQ, 1522)
      call createCurrent(P_Ha, P_Ha, P_HPlus, P_HMinus, 5243)
      call createCurrent(P_WQMinus, P_tauPlus, P_nu_tau, 6272)
      call createCurrent(P_ghWp, P_Hl, P_HMinus, P_ghZAnti, 7380)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghWp, P_G0, 9693)
      call createCurrent(P_Hh, P_Hh, P_G0, P_G0, 7964)
      call createCurrent(P_AQ, P_GMinus, P_WQPlus, 2815)
      call createCurrent(P_HMinus, P_Hl, P_G0, P_HPlus, 9671)
      call createCurrent(P_d, P_dAnti, P_HlQ, 5061)
      call createCurrent(P_WQMinus, P_bAnti, P_t, 8151)
      call createCurrent(P_GPlus, P_HMinus, P_Ha, 2693)
      call createCurrent(P_HlQ, P_Ha, P_GPlus, P_HQMinus, 8189)
      call createCurrent(P_A, P_GMinus, P_WPlus, 1374)
      call createCurrent(P_GQ0, P_Hl, P_HPlus, P_GQMinus, 5206)
      call createCurrent(P_ghA, P_G0, P_ghWmAnti, P_GMinus, 4897)
      call createCurrent(P_dAnti, P_d, P_AQ, 7250)
      call createCurrent(P_HlQ, P_Hh, P_Hl, P_HlQ, 3490)
      call createCurrent(P_WMinus, P_WPlus, P_ghWmAnti, P_ghWm, 6029)
      call createCurrent(P_HQPlus, P_Z, P_Hl, P_WQMinus, 4548)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghA, P_HQPlus, 2114)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghA, P_Ha, 10317)
      call createCurrent(P_HaQ, P_GPlus, P_G0, P_HQMinus, 5689)
      call createCurrent(P_HlQ, P_ghZ, P_GPlus, P_ghWpAnti, 2614)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Z, P_A, 7560)
      call createCurrent(P_HaQ, P_ghWp, P_Ha, P_ghWpAnti, 1795)
      call createCurrent(P_GQMinus, P_Hh, P_Hh, P_HQPlus, 3241)
      call createCurrent(P_HhQ, P_Ha, P_ghWmAnti, P_ghWm, 6949)
      call createCurrent(P_HQMinus, P_ghZ, P_Hh, P_ghWmAnti, 1491)
      call createCurrent(P_ghAAnti, P_ghA, P_HPlus, P_HQMinus, 4780)
      call createCurrent(P_WQPlus, P_Ha, P_HMinus, P_ZQ, 5496)
      call createCurrent(P_Hl, P_b, P_bAnti, 5543)
      call createCurrent(P_Ha, P_GMinus, P_WPlus, P_Z, 9352)
      call createCurrent(P_HlQ, P_ghWm, P_GPlus, P_ghZAnti, 4533)
      call createCurrent(P_Z, P_ghWm, P_ghAAnti, P_WPlus, 6782)
      call createCurrent(P_HlQ, P_G0, P_Ha, P_HlQ, 7168)
      call createCurrent(P_Hl, P_ghWp, P_ghWpAnti, P_Hh, 9592)
      call createCurrent(P_WPlus, P_ghZ, P_ghAAnti, P_WMinus, 1993)
      call createCurrent(P_HlQ, P_Hl, P_ghZ, P_ghZAnti, 772)
      call createCurrent(P_Ha, P_G0, P_ghZ, P_ghZAnti, 9965)
      call createCurrent(P_ghZ, P_ghZAnti, P_G0, P_GQ0, 7900)
      call createCurrent(P_WMinus, P_WPlus, P_ghA, P_ghZAnti, 2469)
      call createCurrent(P_Hl, P_A, P_Hh, P_A, 10507)
      call createCurrent(P_HhQ, P_ghWpAnti, P_GPlus, P_ghZ, 878)
      call createCurrent(P_HPlus, P_GMinus, P_ghZAnti, P_ghZ, 10271)
      call createCurrent(P_g, P_g, P_A, P_Z, 10583)
      call createCurrent(P_G0, P_ghWpAnti, P_HPlus, P_ghZ, 9549)
      call createCurrent(P_uAnti, P_u, P_HaQ, 7832)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_HhQ, 1985)
      call createCurrent(P_GQMinus, P_ghA, P_GPlus, P_ghZAnti, 200)
      call createCurrent(P_ghZ, P_HMinus, P_GPlus, P_ghAAnti, 9594)
      call createCurrent(P_Hl, P_GMinus, P_Hh, P_HPlus, 6526)
      call createCurrent(P_HQPlus, P_ghZ, P_HMinus, P_ghAAnti, 8511)
      call createCurrent(P_WQMinus, P_A, P_GPlus, P_GQ0, 1078)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghZ, P_HMinus, 4737)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_HlQ, 653)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_HlQ, 8501)
      call createCurrent(P_HaQ, P_HPlus, P_ghAAnti, P_ghWm, 3657)
      call createCurrent(P_Hh, P_Ha, P_Hl, P_G0, 2551)
      call createCurrent(P_G0, P_Z, P_WPlus, P_HMinus, 10328)
      call createCurrent(P_HMinus, P_HMinus, P_HPlus, P_GPlus, 8055)
      call createCurrent(P_Hh, P_ghWpAnti, P_Hl, P_ghWp, 9054)
      call createCurrent(P_Z, P_HMinus, P_A, P_HPlus, 305)
      call createCurrent(P_Z, P_Hh, P_WPlus, P_HMinus, 5639)
      call createCurrent(P_ghWm, P_ghWmAnti, P_A, P_A, 4696)
      call createCurrent(P_HPlus, P_G0, P_Hl, P_GMinus, 7602)
      call createCurrent(P_HlQ, P_HPlus, P_ghAAnti, P_ghWm, 7575)
      call createCurrent(P_WPlus, P_Ha, P_GMinus, P_Z, 10119)
      call createCurrent(P_ghWpAnti, P_GPlus, P_Hl, P_ghA, 5810)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hl, P_GMinus, 3516)
      call createCurrent(P_t, P_bAnti, P_HQMinus, 6500)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_HhQ, 2420)
      call createCurrent(P_HaQ, P_WPlus, P_A, P_HQMinus, 7368)
      call createCurrent(P_ghAAnti, P_ghWm, P_Ha, P_HPlus, 1871)
      call createCurrent(P_Z, P_G0, P_HPlus, P_WMinus, 10041)
      call createCurrent(P_ghWp, P_GPlus, P_ghWpAnti, P_HMinus, 9253)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghWm, P_AQ, 1640)
      call createCurrent(P_HlQ, P_Hl, P_Z, P_ZQ, 7387)
      call createCurrent(P_HQMinus, P_G0, P_GPlus, P_HlQ, 5892)
      call createCurrent(P_GPlus, P_ghA, P_Hl, P_ghWpAnti, 4900)
      call createCurrent(P_ghZ, P_WPlus, P_ghZAnti, P_WMinus, 6686)
      call createCurrent(P_HQMinus, P_Ha, P_HPlus, P_HhQ, 2722)
      call createCurrent(P_GPlus, P_HMinus, P_Z, 10201)
      call createCurrent(P_g, P_Z, P_g, P_Z, 10706)
      call createCurrent(P_A, P_ghWp, P_ghWpAnti, 6643)
      call createCurrent(P_WQPlus, P_GMinus, P_A, P_HhQ, 4407)
      call createCurrent(P_GQPlus, P_A, P_GQMinus, 7047)
      call createCurrent(P_ZQ, P_Hh, P_GMinus, P_WQPlus, 3950)
      call createCurrent(P_Hl, P_GMinus, P_Hh, P_GPlus, 4337)
      call createCurrent(P_WMinus, P_ghWp, P_WMinus, P_ghWmAnti, 613)
      call createCurrent(P_cAnti, P_c, P_HhQ, 7117)
      call createCurrent(P_WPlus, P_ghA, P_ghZAnti, P_WMinus, 8695)
      call createCurrent(P_ghWp, P_Z, P_A, P_ghWpAnti, 7711)
      call createCurrent(P_GQPlus, P_Ha, P_G0, P_HQMinus, 1267)
      call createCurrent(P_HPlus, P_tauMinus, P_nu_tauAnti, 410)
      call createCurrent(P_A, P_ghWm, P_ghWmAnti, P_Z, 637)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghA, P_HMinus, 1387)
      call createCurrent(P_Ha, P_WMinus, P_HPlus, P_A, 87)
      call createCurrent(P_HQMinus, P_Hh, P_ghWmAnti, P_ghA, 8324)
      call createCurrent(P_GQMinus, P_Hl, P_WPlus, P_AQ, 4119)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_G0, 9495)
      call createCurrent(P_G0, P_Z, P_WMinus, P_HPlus, 10153)
      call createCurrent(P_HPlus, P_ghAAnti, P_HMinus, P_ghA, 8213)
      call createCurrent(P_GMinus, P_A, P_Ha, P_WPlus, 9171)
      call createCurrent(P_HPlus, P_tAnti, P_b, 1926)
      call createCurrent(P_Hh, P_ghZAnti, P_ghZ, P_Hl, 10279)
      call createCurrent(P_ghWm, P_WPlus, P_ghZAnti, P_AQ, 5879)
      call createCurrent(P_GMinus, P_WPlus, P_Z, P_Hl, 6538)
      call createCurrent(P_HMinus, P_G0, P_Z, P_WPlus, 10169)
      call createCurrent(P_Hl, P_G0, P_Ha, P_Hl, 2053)
      call createCurrent(P_GQMinus, P_GPlus, P_Hh, P_HhQ, 6905)
      call createCurrent(P_HlQ, P_Hl, P_ghWp, P_ghWpAnti, 4595)
      call createCurrent(P_HlQ, P_HMinus, P_GPlus, P_HhQ, 1971)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghA, P_G0, 9219)
      call createCurrent(P_GQ0, P_Hl, P_ghWp, P_ghWpAnti, 4947)
      call createCurrent(P_GPlus, P_GPlus, P_GMinus, P_HMinus, 8692)
      call createCurrent(P_ghAAnti, P_ghWm, P_WPlus, P_AQ, 4882)
      call createCurrent(P_HMinus, P_ghZAnti, P_GPlus, P_ghZ, 9771)
      call createCurrent(P_Hl, P_ghA, P_ghWmAnti, P_HMinus, 3550)
      call createCurrent(P_c, P_GMinus, P_sAnti, 6770)
      call createCurrent(P_Hh, P_WPlus, P_HMinus, P_A, 6487)
      call createCurrent(P_Hh, P_GPlus, P_Z, P_WMinus, 7245)
      call createCurrent(P_WPlus, P_s, P_cAnti, 995)
      call createCurrent(P_HhQ, P_WPlus, P_GMinus, P_ZQ, 7977)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghA, P_HMinus, 8210)
      call createCurrent(P_Hh, P_ghZ, P_Hl, P_ghZAnti, 9203)
      call createCurrent(P_Z, P_Hh, P_HPlus, P_WMinus, 3629)
      call createCurrent(P_GQMinus, P_ghZ, P_Hh, P_ghWmAnti, 7294)
      call createCurrent(P_GMinus, P_G0, P_WPlus, 7511)
      call createCurrent(P_A, P_HPlus, P_WMinus, P_Hh, 6005)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghZ, P_HQMinus, 7252)
      call createCurrent(P_HPlus, P_uAnti, P_d, 4474)
      call createCurrent(P_WPlus, P_HPlus, P_GMinus, P_WMinus, 9727)
      call createCurrent(P_Ha, P_G0, P_GPlus, P_GMinus, 7985)
      call createCurrent(P_HQPlus, P_HMinus, P_HaQ, 6277)
      call createCurrent(P_ghWp, P_GMinus, P_G0, P_ghAAnti, 5033)
      call createCurrent(P_ghWm, P_WPlus, P_ghAAnti, P_AQ, 6564)
      call createCurrent(P_GQMinus, P_WPlus, P_Z, P_HhQ, 3956)
      call createCurrent(P_g, P_s, P_sAnti, 4321)
      call createCurrent(P_Ha, P_G0, P_WMinus, P_WPlus, 10399)
      call createCurrent(P_ghZ, P_HMinus, P_G0, P_ghWmAnti, 8860)
      call createCurrent(P_WMinus, P_ghZAnti, P_ghZ, P_WPlus, 6712)
      call createCurrent(P_ghWm, P_GPlus, P_HMinus, P_ghWmAnti, 9575)
      call createCurrent(P_GQPlus, P_ghZAnti, P_G0, P_ghWm, 6910)
      call createCurrent(P_ghWmAnti, P_Ha, P_HMinus, P_ghZ, 3809)
      call createCurrent(P_ghWm, P_Hl, P_ghAAnti, P_GPlus, 4276)
      call createCurrent(P_HQMinus, P_ghZAnti, P_HPlus, P_ghA, 1143)
      call createCurrent(P_A, P_WPlus, P_ghA, P_ghWpAnti, 948)
      call createCurrent(P_G0, P_WPlus, P_G0, P_WMinus, 2280)
      call createCurrent(P_Z, P_Hh, P_Hl, P_Z, 9665)
      call createCurrent(P_ghWmAnti, P_A, P_ghWm, 1889)
      call createCurrent(P_ZQ, P_GPlus, P_Hh, P_WQMinus, 2218)
      call createCurrent(P_WQPlus, P_G0, P_WMinus, P_HlQ, 8087)
      call createCurrent(P_Hl, P_t, P_tAnti, 2682)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hh, P_Hh, 7970)
      call createCurrent(P_ghWmAnti, P_GMinus, P_G0, P_ghZ, 1962)
      call createCurrent(P_HlQ, P_Ha, P_Ha, P_HhQ, 1298)
      call createCurrent(P_Z, P_WMinus, P_A, P_WPlus, 7831)
      call createCurrent(P_Hh, P_g, P_Hh, P_g, 10506)
      call createCurrent(P_g, P_g, P_g, 3290)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, 8407)
      call createCurrent(P_GQPlus, P_A, P_WMinus, P_HhQ, 6012)
      call createCurrent(P_GMinus, P_g, P_g, P_GPlus, 10738)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghWm, P_GQPlus, 8542)
      call createCurrent(P_GQ0, P_bAnti, P_b, 3331)
      call createCurrent(P_HhQ, P_HMinus, P_G0, P_HQPlus, 6337)
      call createCurrent(P_cAnti, P_c, P_GQ0, 559)
      call createCurrent(P_Hh, P_HMinus, P_WPlus, P_Z, 2442)
      call createCurrent(P_Hl, P_WMinus, P_A, P_HPlus, 8532)
      call createCurrent(P_A, P_A, P_Ha, P_G0, 10740)
      call createCurrent(P_HQPlus, P_ghZAnti, P_Hh, P_ghWm, 4508)
      call createCurrent(P_A, P_G0, P_HMinus, P_WPlus, 10063)
      call createCurrent(P_GPlus, P_uAnti, P_d, 137)
      call createCurrent(P_HPlus, P_ghWm, P_ghZAnti, P_Ha, 7659)
      call createCurrent(P_HQPlus, P_Hh, P_ghWm, P_ghZAnti, 8209)
      call createCurrent(P_Hl, P_ghWp, P_HMinus, P_ghAAnti, 2707)
      call createCurrent(P_ghZ, P_HMinus, P_ghZAnti, P_HQPlus, 4147)
      call createCurrent(P_Hl, P_Hh, P_ghWp, P_ghWpAnti, 9531)
      call createCurrent(P_HPlus, P_Hl, P_HMinus, P_Hl, 8567)
      call createCurrent(P_Hh, P_ghZ, P_ghWpAnti, P_HPlus, 1531)
      call createCurrent(P_tAnti, P_Hh, P_t, 1743)
      call createCurrent(P_GPlus, P_G0, P_HMinus, P_G0, 4639)
      call createCurrent(P_HQPlus, P_Hl, P_WMinus, P_AQ, 5326)
      call createCurrent(P_g, P_g, P_HMinus, P_HPlus, 10764)
      call createCurrent(P_ghWp, P_Hl, P_GMinus, P_ghZAnti, 2574)
      call createCurrent(P_GMinus, P_WMinus, P_GPlus, P_WPlus, 7111)
      call createCurrent(P_HhQ, P_ghWp, P_ghWpAnti, 7495)
      call createCurrent(P_ePlus, P_eMinus, P_AQ, 1423)
      call createCurrent(P_GPlus, P_Hl, P_ghWpAnti, P_ghA, 6853)
      call createCurrent(P_HMinus, P_ghWp, P_GPlus, P_ghWpAnti, 10309)
      call createCurrent(P_Hh, P_A, P_A, P_Hl, 10589)
      call createCurrent(P_GQ0, P_GPlus, P_HMinus, P_HlQ, 73)
      call createCurrent(P_eMinus, P_GPlus, P_nu_eAnti, 8455)
      call createCurrent(P_ZQ, P_b, P_bAnti, 4528)
      call createCurrent(P_HQMinus, P_Z, P_HQPlus, 4224)
      call createCurrent(P_Z, P_WPlus, P_Ha, P_GMinus, 10138)
      call createCurrent(P_GQ0, P_Hh, P_Ha, P_HhQ, 3970)
      call createCurrent(P_ghWmAnti, P_G0, P_ghA, P_GMinus, 4167)
      call createCurrent(P_Z, P_WPlus, P_ghWpAnti, P_ghA, 5877)
      call createCurrent(P_ePlus, P_Z, P_eMinus, 2024)
      call createCurrent(P_HPlus, P_nu_muAnti, P_muMinus, 6200)
      call createCurrent(P_Hh, P_G0, P_Hh, P_Ha, 4520)
      call createCurrent(P_ghZ, P_G0, P_ghWmAnti, P_GQMinus, 4214)
      call createCurrent(P_ghZAnti, P_Hl, P_Hh, P_ghZ, 10353)
      call createCurrent(P_HQMinus, P_Z, P_WPlus, P_HlQ, 5585)
      call createCurrent(P_G0, P_ghWm, P_ghWmAnti, P_G0, 2360)
      call createCurrent(P_ghWp, P_HMinus, P_ghZAnti, P_Hh, 2245)
      call createCurrent(P_ghZAnti, P_GMinus, P_GPlus, P_ghZ, 5096)
      call createCurrent(P_HhQ, P_WMinus, P_Z, P_GQPlus, 5690)
      call createCurrent(P_G0, P_GPlus, P_G0, P_HMinus, 3744)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_Hl, 2599)
      call createCurrent(P_nu_e, P_Z, P_nu_eAnti, 4289)
      call createCurrent(P_G0, P_Hh, P_A, 9582)
      call createCurrent(P_A, P_ghAAnti, P_WMinus, P_ghWp, 7013)
      call createCurrent(P_GMinus, P_A, P_WPlus, P_Hl, 5751)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hh, P_HPlus, 449)
      call createCurrent(P_A, P_G0, P_Hl, 9290)
      call createCurrent(P_uAnti, P_d, P_GPlus, 6156)
      call createCurrent(P_HaQ, P_Hl, P_HPlus, P_HQMinus, 7480)
      call createCurrent(P_Hl, P_Z, P_GPlus, P_WMinus, 5957)
      call createCurrent(P_HhQ, P_GMinus, P_ghAAnti, P_ghWp, 6729)
      call createCurrent(P_tauMinus, P_tauPlus, P_HhQ, 4805)
      call createCurrent(P_G0, P_Hh, P_Ha, 1697)
      call createCurrent(P_ZQ, P_Ha, P_HPlus, P_WQMinus, 3292)
      call createCurrent(P_WMinus, P_GMinus, P_WPlus, P_HPlus, 9848)
      call createCurrent(P_ghWm, P_Z, P_ghWmAnti, P_Z, 956)
      call createCurrent(P_ghA, P_GPlus, P_ghWpAnti, P_Hl, 3649)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_G0, P_ghA, 8009)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghA, P_Hh, 1909)
      call createCurrent(P_Hh, P_Ha, P_GMinus, P_GPlus, 10306)
      call createCurrent(P_HMinus, P_HPlus, P_Hl, P_Ha, 9148)
      call createCurrent(P_GPlus, P_G0, P_ghWm, P_ghZAnti, 7341)
      call createCurrent(P_HMinus, P_Hl, P_Ha, P_HPlus, 8913)
      call createCurrent(P_HaQ, P_sAnti, P_s, 314)
      call createCurrent(P_GQ0, P_Hh, P_ghWmAnti, P_ghWm, 3010)
      call createCurrent(P_HlQ, P_d, P_dAnti, 6583)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hl, P_HaQ, 5155)
      call createCurrent(P_Z, P_Z, P_ghWm, P_ghWmAnti, 8356)
      call createCurrent(P_Hl, P_Hh, 9489)
      call createCurrent(P_ghWmAnti, P_GMinus, P_Ha, P_ghA, 9536)
      call createCurrent(P_GPlus, P_GMinus, P_ghWmAnti, P_ghWm, 818)
      call createCurrent(P_WQMinus, P_Ha, P_Ha, P_WQPlus, 2602)
      call createCurrent(P_GQ0, P_Hh, P_HMinus, P_HQPlus, 841)
      call createCurrent(P_G0, P_A, P_Hl, 9925)
      call createCurrent(P_Ha, P_HMinus, P_G0, P_GPlus, 2868)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Z, P_WPlus, 6461)
      call createCurrent(P_HlQ, P_G0, P_HaQ, 8494)
      call createCurrent(P_HQPlus, P_Hl, P_ghZAnti, P_ghWm, 8379)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_HlQ, 8221)
      call createCurrent(P_gQ, P_g, P_ghG, P_ghGAnti, 2063)
      call createCurrent(P_GQ0, P_A, P_GMinus, P_WQPlus, 6201)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hh, P_Hl, 9377)
      call createCurrent(P_ZQ, P_HPlus, P_HMinus, P_ZQ, 6909)
      call createCurrent(P_GMinus, P_Hl, P_ghZAnti, P_ghWp, 6566)
      call createCurrent(P_GQ0, P_GPlus, P_Hh, P_HQMinus, 5524)
      call createCurrent(P_GPlus, P_HMinus, P_Z, P_Z, 9702)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_Hl, 1630)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_Hh, P_ghZ, 3492)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hl, P_HaQ, 5423)
      call createCurrent(P_HPlus, P_Hh, P_HMinus, P_Ha, 9606)
      call createCurrent(P_WPlus, P_Ha, P_HMinus, P_Z, 2012)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_Hh, 4001)
      call createCurrent(P_G0, P_ghWm, P_GPlus, P_ghZAnti, 209)
      call createCurrent(P_WPlus, P_nu_tauAnti, P_tauMinus, 2336)
      call createCurrent(P_ZQ, P_WMinus, P_GQPlus, 5946)
      call createCurrent(P_ghZAnti, P_GMinus, P_Hh, P_ghWp, 3150)
      call createCurrent(P_GQPlus, P_Z, P_Hh, P_WQMinus, 8335)
      call createCurrent(P_HPlus, P_G0, P_WMinus, P_Z, 9857)
      call createCurrent(P_Hh, P_HMinus, P_GPlus, P_Hl, 5310)
      call createCurrent(P_Hl, P_Hl, P_WPlus, P_WMinus, 5512)
      call createCurrent(P_t, P_HMinus, P_bAnti, 7863)
      call createCurrent(P_ghWp, P_Hh, P_GMinus, P_ghAAnti, 5425)
      call createCurrent(P_WQMinus, P_Ha, P_WPlus, P_HlQ, 6662)
      call createCurrent(P_GMinus, P_ghZ, P_ghWmAnti, P_Hl, 6763)
      call createCurrent(P_GQMinus, P_Hh, P_GQPlus, 525)
      call createCurrent(P_ZQ, P_ePlus, P_eMinus, 8779)
      call createCurrent(P_GQ0, P_Hh, P_HPlus, P_GQMinus, 4051)
      call createCurrent(P_GMinus, P_HPlus, P_Hh, P_Hh, 4406)
      call createCurrent(P_HhQ, P_Hh, P_Hl, P_HhQ, 7352)
      call createCurrent(P_WQMinus, P_ghZAnti, P_WPlus, P_ghA, 6124)
      call createCurrent(P_G0, P_HPlus, P_ghWm, P_ghZAnti, 9063)
      call createCurrent(P_WQMinus, P_HPlus, P_ZQ, 5561)
      call createCurrent(P_HMinus, P_WPlus, P_Z, P_Ha, 7313)
      call createCurrent(P_ghWm, P_G0, P_ghWmAnti, P_GQ0, 7737)
      call createCurrent(P_Z, P_HPlus, P_Hh, P_WMinus, 8792)
      call createCurrent(P_AQ, P_A, P_GPlus, P_GQMinus, 6311)
      call createCurrent(P_GQMinus, P_GPlus, P_ZQ, 8123)
      call createCurrent(P_A, P_A, P_HPlus, P_HMinus, 5920)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_HaQ, 5612)
      call createCurrent(P_ghZ, P_Hl, P_ghWmAnti, P_GMinus, 1283)
      call createCurrent(P_HaQ, P_G0, P_HlQ, 8780)
      call createCurrent(P_GPlus, P_ghZAnti, P_ghWm, P_Ha, 9216)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hl, P_Hh, 10302)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghWm, P_Hh, 7623)
      call createCurrent(P_A, P_WMinus, P_HPlus, 3375)
      call createCurrent(P_HPlus, P_GMinus, P_ghWpAnti, P_ghWp, 9494)
      call createCurrent(P_Ha, P_HMinus, P_G0, P_HPlus, 7053)
      call createCurrent(P_Ha, P_HPlus, P_ghA, P_ghWpAnti, 1893)
      call createCurrent(P_Z, P_Z, P_Hl, P_Hh, 10393)
      call createCurrent(P_Z, P_Ha, P_WPlus, P_HMinus, 1984)
      call createCurrent(P_HPlus, P_Ha, P_ghZAnti, P_ghWm, 6920)
      call createCurrent(P_WPlus, P_WMinus, P_ghWm, P_ghWmAnti, 5435)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_HlQ, 4636)
      call createCurrent(P_HMinus, P_GPlus, P_ghZ, P_ghZAnti, 10170)
      call createCurrent(P_GMinus, P_GPlus, P_ghZAnti, P_ghZ, 4263)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hl, P_GQPlus, 7142)
      call createCurrent(P_g, P_dAnti, P_d, 4652)
      call createCurrent(P_ghWm, P_ghWmAnti, P_GQ0, 3591)
      call createCurrent(P_uAnti, P_d, P_WQPlus, 8035)
      call createCurrent(P_G0, P_Ha, P_Ha, P_G0, 8099)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hh, P_HQMinus, 5248)
      call createCurrent(P_ghAAnti, P_Hl, P_ghWm, P_GPlus, 8131)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghWm, P_G0, 9584)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_HaQ, 8547)
      call createCurrent(P_HaQ, P_Hh, P_GPlus, P_HQMinus, 3065)
      call createCurrent(P_GQPlus, P_Ha, P_HMinus, P_HhQ, 3074)
      call createCurrent(P_ghWp, P_G0, P_GMinus, P_ghZAnti, 5017)
      call createCurrent(P_Ha, P_ghWmAnti, P_HMinus, P_ghA, 7055)
      call createCurrent(P_G0, P_Z, P_G0, P_A, 9035)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_Ha, 3941)
      call createCurrent(P_GPlus, P_WPlus, P_WMinus, P_HMinus, 9562)
      call createCurrent(P_ghWp, P_Hl, P_ghWpAnti, P_Hl, 2539)
      call createCurrent(P_GMinus, P_Hl, P_G0, P_HPlus, 1679)
      call createCurrent(P_WPlus, P_Z, P_GMinus, P_Hl, 4362)
      call createCurrent(P_Hl, P_Hh, P_Hh, P_Hl, 8259)
      call createCurrent(P_A, P_tauMinus, P_tauPlus, 4000)
      call createCurrent(P_GQ0, P_GPlus, P_GMinus, P_HaQ, 7402)
      call createCurrent(P_WQMinus, P_Hl, P_HPlus, P_AQ, 4801)
      call createCurrent(P_HhQ, P_ghWm, P_GPlus, P_ghZAnti, 5716)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, P_HlQ, 7883)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghA, P_GMinus, 5696)
      call createCurrent(P_WMinus, P_GPlus, P_Z, P_G0, 5820)
      call createCurrent(P_HaQ, P_HPlus, P_GMinus, P_HaQ, 8716)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghWp, P_Hh, 9244)
      call createCurrent(P_HhQ, P_HMinus, P_GQPlus, 551)
      call createCurrent(P_A, P_HMinus, P_G0, P_WPlus, 9463)
      call createCurrent(P_ghZ, P_Hh, P_ghWmAnti, P_HQMinus, 1070)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_ghZ, 3928)
      call createCurrent(P_GQPlus, P_GMinus, P_ghWm, P_ghWmAnti, 5847)
      call createCurrent(P_HaQ, P_Hl, P_HPlus, P_GQMinus, 5404)
      call createCurrent(P_HPlus, P_Ha, P_GMinus, P_Hl, 4707)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghA, P_HMinus, 9670)
      call createCurrent(P_GPlus, P_WMinus, P_WPlus, P_HMinus, 9724)
      call createCurrent(P_G0, P_HMinus, P_Ha, P_HPlus, 1453)
      call createCurrent(P_Hl, P_Hh, P_GMinus, P_GPlus, 5393)
      call createCurrent(P_Z, P_GPlus, P_A, P_GMinus, 5292)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghWm, P_WQPlus, 8674)
      call createCurrent(P_ghZAnti, P_Hh, P_GPlus, P_ghWm, 6788)
      call createCurrent(P_GQMinus, P_ghZ, P_G0, P_ghWmAnti, 3034)
      call createCurrent(P_HMinus, P_ghZ, P_GPlus, P_ghAAnti, 9550)
      call createCurrent(P_HaQ, P_G0, P_HPlus, P_HQMinus, 6143)
      call createCurrent(P_GQPlus, P_b, P_tAnti, 1579)
      call createCurrent(P_Hl, P_Hl, P_HMinus, P_HPlus, 6720)
      call createCurrent(P_HMinus, P_ghZ, P_ghWmAnti, P_Hl, 2584)
      call createCurrent(P_GPlus, P_HMinus, P_ghZ, P_ghZAnti, 9871)
      call createCurrent(P_ghZ, P_GMinus, P_HPlus, P_ghAAnti, 9012)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_Hl, 5431)
      call createCurrent(P_HhQ, P_ghAAnti, P_HPlus, P_ghWm, 8158)
      call createCurrent(P_muMinus, P_muPlus, P_Z, 2554)
      call createCurrent(P_G0, P_G0, P_HMinus, P_HPlus, 4172)
      call createCurrent(P_ghA, P_ghWpAnti, P_HPlus, P_Ha, 6786)
      call createCurrent(P_ghAAnti, P_Z, P_WMinus, P_ghWp, 5995)
      call createCurrent(P_ghA, P_HMinus, P_ghWmAnti, P_Hh, 635)
      call createCurrent(P_HQMinus, P_HPlus, P_ghZAnti, P_ghA, 7619)
      call createCurrent(P_Hh, P_G0, P_HPlus, P_HMinus, 9894)
      call createCurrent(P_HlQ, P_Z, P_Z, P_HlQ, 243)
      call createCurrent(P_ghWmAnti, P_ghWm, P_WMinus, P_WQPlus, 1451)
      call createCurrent(P_ZQ, P_HPlus, P_WQMinus, 6170)
      call createCurrent(P_HQMinus, P_WPlus, P_HlQ, 6014)
      call createCurrent(P_WQPlus, P_WMinus, P_ghZ, P_ghZAnti, 4914)
      call createCurrent(P_ghWm, P_ghWmAnti, P_G0, P_G0, 1935)
      call createCurrent(P_WMinus, P_WMinus, P_WPlus, P_WPlus, 5860)
      call createCurrent(P_G0, P_HPlus, P_Hh, P_HMinus, 9351)
      call createCurrent(P_b, P_bAnti, P_ZQ, 6100)
      call createCurrent(P_tAnti, P_t, P_Hh, 6409)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hh, P_Hl, 9855)
      call createCurrent(P_c, P_WMinus, P_sAnti, 4790)
      call createCurrent(P_HaQ, P_G0, P_Hh, P_HlQ, 1631)
      call createCurrent(P_GPlus, P_Ha, P_ghZ, P_ghWpAnti, 9705)
      call createCurrent(P_Hl, P_Hh, P_GMinus, P_HPlus, 1557)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghA, P_HPlus, 7533)
      call createCurrent(P_GMinus, P_ghWmAnti, P_Hl, P_ghZ, 6189)
      call createCurrent(P_GPlus, P_GMinus, P_Z, P_A, 2368)
      call createCurrent(P_ghWpAnti, P_WPlus, P_Z, P_ghA, 4482)
      call createCurrent(P_Ha, P_Ha, P_A, P_Z, 9767)
      call createCurrent(P_GQ0, P_GPlus, P_GMinus, P_GQ0, 2741)
      call createCurrent(P_GQ0, P_G0, P_ghWmAnti, P_ghWm, 1403)
      call createCurrent(P_HlQ, P_Ha, P_Hh, P_HaQ, 5562)
      call createCurrent(P_Hh, P_Hl, P_G0, P_G0, 4485)
      call createCurrent(P_g, P_Hl, P_Hl, P_g, 10658)
      call createCurrent(P_G0, P_Hl, P_GPlus, P_HMinus, 5281)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghWp, P_Hh, 4584)
      call createCurrent(P_Z, P_GPlus, P_A, P_HMinus, 9367)
      call createCurrent(P_HQPlus, P_Ha, P_Z, P_WQMinus, 7768)
      call createCurrent(P_GQMinus, P_HPlus, P_GPlus, P_GQMinus, 2337)
      call createCurrent(P_WQMinus, P_Z, P_Hl, P_HQPlus, 19)
      call createCurrent(P_HhQ, P_GMinus, P_Ha, P_GQPlus, 2750)
      call createCurrent(P_HQPlus, P_GMinus, P_Hl, P_HlQ, 3318)
      call createCurrent(P_GQMinus, P_GMinus, P_GPlus, P_GQPlus, 1890)
      call createCurrent(P_ghAAnti, P_G0, P_HMinus, P_ghWp, 9051)
      call createCurrent(P_G0, P_ePlus, P_eMinus, 7640)
      call createCurrent(P_ghAAnti, P_WPlus, P_A, P_ghWm, 7571)
      call createCurrent(P_HPlus, P_ghZ, P_ghAAnti, P_GMinus, 10178)
      call createCurrent(P_HlQ, P_Ha, P_ghWmAnti, P_ghWm, 3337)
      call createCurrent(P_HPlus, P_Ha, P_HMinus, P_Hl, 9891)
      call createCurrent(P_GPlus, P_Z, P_HMinus, P_Z, 9847)
      call createCurrent(P_ghA, P_Hh, P_ghWpAnti, P_HPlus, 4960)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Hh, P_GQMinus, 5497)
      call createCurrent(P_ghWpAnti, P_ghWp, P_G0, P_HhQ, 7337)
      call createCurrent(P_ghZ, P_Hh, P_ghZAnti, P_Hl, 9690)
      call createCurrent(P_AQ, P_GMinus, P_GPlus, P_AQ, 6546)
      call createCurrent(P_g, P_HMinus, P_GPlus, P_g, 10715)
      call createCurrent(P_WQMinus, P_ghZAnti, P_ghWp, 8611)
      call createCurrent(P_Ha, P_WPlus, P_HMinus, P_A, 6604)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghZ, P_HQMinus, 8505)
      call createCurrent(P_HaQ, P_cAnti, P_c, 1175)
      call createCurrent(P_ghG, P_g, P_g, P_ghGAnti, 8314)
      call createCurrent(P_ghAAnti, P_ghWm, P_WPlus, P_A, 5974)
      call createCurrent(P_Hh, P_HPlus, P_Hh, P_GMinus, 4924)
      call createCurrent(P_ghWmAnti, P_A, P_ghWm, P_AQ, 7226)
      call createCurrent(P_Ha, P_HMinus, P_HPlus, P_Ha, 79)
      call createCurrent(P_Z, P_Hl, P_GPlus, P_WMinus, 5219)
      call createCurrent(P_ghA, P_ghAAnti, P_GMinus, P_HPlus, 8982)
      call createCurrent(P_GMinus, P_G0, P_G0, P_HPlus, 5632)
      call createCurrent(P_ghZAnti, P_ghWm, P_A, P_WPlus, 6966)
      call createCurrent(P_ghWmAnti, P_HPlus, P_ghWm, P_GMinus, 9137)
      call createCurrent(P_Hh, P_G0, P_Ha, 2134)
      call createCurrent(P_HQPlus, P_HMinus, P_Hl, P_HaQ, 839)
      call createCurrent(P_GQ0, P_HMinus, P_GPlus, P_HaQ, 2891)
      call createCurrent(P_ghAAnti, P_ghA, P_HMinus, P_GPlus, 9762)
      call createCurrent(P_ghA, P_ghAAnti, P_GPlus, P_GMinus, 4254)
      call createCurrent(P_GPlus, P_GMinus, P_ghZAnti, P_ghZ, 4532)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_Ha, 182)
      call createCurrent(P_HlQ, P_HPlus, P_GMinus, P_GQ0, 1397)
      call createCurrent(P_G0, P_GMinus, P_HPlus, P_Hl, 3152)
      call createCurrent(P_ghWpAnti, P_ghA, P_HPlus, P_Ha, 1529)
      call createCurrent(P_Hl, P_Z, P_Z, 8777)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghWm, P_Hl, 8829)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_HlQ, 5759)
      call createCurrent(P_Z, P_WMinus, P_WPlus, 3098)
      call createCurrent(P_ghWmAnti, P_ghZ, P_G0, P_HMinus, 9868)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_Hl, P_ghZ, 7407)
      call createCurrent(P_WQMinus, P_u, P_dAnti, 7687)
      call createCurrent(P_ghWp, P_Hh, P_ghWpAnti, 916)
      call createCurrent(P_ghWmAnti, P_WMinus, P_WPlus, P_ghWm, 2362)
      call createCurrent(P_A, P_Z, P_G0, P_Ha, 10640)
      call createCurrent(P_Hl, P_GMinus, P_ghZAnti, P_ghWp, 2938)
      call createCurrent(P_HMinus, P_Ha, P_ghWmAnti, P_ghA, 4905)
      call createCurrent(P_GQ0, P_G0, P_G0, P_GQ0, 2640)
      call createCurrent(P_ghWm, P_Z, P_WPlus, P_ghAAnti, 4767)
      call createCurrent(P_HlQ, P_GMinus, P_Ha, P_GQPlus, 341)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghA, P_HQMinus, 4223)
      call createCurrent(P_HPlus, P_Hh, P_ghWpAnti, P_ghA, 5179)
      call createCurrent(P_A, P_Ha, P_G0, P_A, 10525)
      call createCurrent(P_HMinus, P_ghAAnti, P_GPlus, P_ghZ, 9598)
      call createCurrent(P_HQMinus, P_G0, P_HPlus, P_GQ0, 8125)
      call createCurrent(P_HMinus, P_Hl, P_ghA, P_ghWmAnti, 6389)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hl, P_GQ0, 8577)
      call createCurrent(P_A, P_HMinus, P_HPlus, P_Z, 1254)
      call createCurrent(P_ghZ, P_ghZAnti, P_HlQ, 7004)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghWp, P_ZQ, 2261)
      call createCurrent(P_Hh, P_ghWp, P_Hh, P_ghWpAnti, 4606)
      call createCurrent(P_HlQ, P_GPlus, P_Hl, P_HQMinus, 2472)
      call createCurrent(P_HhQ, P_GMinus, P_GPlus, P_HlQ, 6505)
      call createCurrent(P_Hl, P_ghA, P_GMinus, P_ghWmAnti, 7415)
      call createCurrent(P_GQ0, P_Hh, P_Hl, P_HaQ, 785)
      call createCurrent(P_WMinus, P_Z, P_Ha, P_GPlus, 8888)
      call createCurrent(P_HMinus, P_Hh, P_G0, P_HPlus, 10206)
      call createCurrent(P_AQ, P_G0, P_WPlus, P_GQMinus, 3123)
      call createCurrent(P_WQPlus, P_cAnti, P_s, 3125)
      call createCurrent(P_HhQ, P_G0, P_HPlus, P_GQMinus, 1478)
      call createCurrent(P_ghWpAnti, P_Hh, P_Hh, P_ghWp, 237)
      call createCurrent(P_d, P_uAnti, P_WQPlus, 4849)
      call createCurrent(P_ghAAnti, P_ghA, P_WMinus, P_WPlus, 7363)
      call createCurrent(P_ghA, P_ghWmAnti, P_GMinus, P_Hl, 7550)
      call createCurrent(P_A, P_HMinus, P_WPlus, P_G0, 8877)
      call createCurrent(P_WMinus, P_A, P_HPlus, 6197)
      call createCurrent(P_Hl, P_HPlus, P_Hl, P_GMinus, 2478)
      call createCurrent(P_GMinus, P_ghWmAnti, P_Ha, P_ghZ, 9007)
      call createCurrent(P_WPlus, P_GMinus, P_A, P_Hl, 3285)
      call createCurrent(P_GQMinus, P_ghWp, P_G0, P_ghZAnti, 3988)
      call createCurrent(P_ghWm, P_Hl, P_ghWmAnti, 901)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghA, P_GMinus, 3216)
      call createCurrent(P_Z, P_WPlus, P_Z, P_WMinus, 3825)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, P_Hh, 6860)
      call createCurrent(P_GQPlus, P_HMinus, P_Ha, P_HlQ, 4537)
      call createCurrent(P_A, P_GPlus, P_Z, P_HMinus, 10354)
      call createCurrent(P_G0, P_HPlus, P_GMinus, 7060)
      call createCurrent(P_A, P_Hl, P_Hh, P_Z, 10575)
      call createCurrent(P_ghWmAnti, P_ghA, P_G0, P_GMinus, 5864)
      call createCurrent(P_A, P_WMinus, P_ghWp, P_ghZAnti, 8633)
      call createCurrent(P_HMinus, P_HPlus, P_Hh, P_G0, 10210)
      call createCurrent(P_HaQ, P_HMinus, P_ghWp, P_ghAAnti, 1396)
      call createCurrent(P_HPlus, P_G0, P_GMinus, P_G0, 1467)
      call createCurrent(P_WMinus, P_HPlus, P_A, P_Ha, 1682)
      call createCurrent(P_GQPlus, P_GMinus, P_Ha, P_GQ0, 987)
      call createCurrent(P_bAnti, P_b, P_g, 8731)
      call createCurrent(P_GQMinus, P_Ha, P_HPlus, P_GQ0, 4935)
      call createCurrent(P_Ha, P_G0, P_Hh, 3732)
      call createCurrent(P_HhQ, P_ghWm, P_HPlus, P_ghZAnti, 8037)
      call createCurrent(P_WQMinus, P_Z, P_WPlus, P_ZQ, 328)
      call createCurrent(P_HlQ, P_tauPlus, P_tauMinus, 1363)
      call createCurrent(P_ghA, P_ghZAnti, P_WPlus, P_WMinus, 6116)
      call createCurrent(P_WQMinus, P_Hl, P_WQPlus, 6960)
      call createCurrent(P_HMinus, P_G0, P_ghAAnti, P_ghWp, 8912)
      call createCurrent(P_HlQ, P_Hh, P_GPlus, P_GQMinus, 8327)
      call createCurrent(P_GMinus, P_HPlus, P_A, P_A, 9609)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Ha, P_G0, 10192)
      call createCurrent(P_ghZ, P_ghZAnti, P_WMinus, P_WQPlus, 4401)
      call createCurrent(P_t, P_Z, P_tAnti, 6046)
      call createCurrent(P_HQPlus, P_ghAAnti, P_HMinus, P_ghA, 5420)
      call createCurrent(P_G0, P_dAnti, P_d, 3919)
      call createCurrent(P_HaQ, P_ghWmAnti, P_ghWm, 1617)
      call createCurrent(P_Ha, P_GPlus, P_GMinus, 9615)
      call createCurrent(P_Hl, P_Ha, P_Ha, P_Hl, 3312)
      call createCurrent(P_ghA, P_WPlus, P_Z, P_ghWpAnti, 1816)
      call createCurrent(P_s, P_Z, P_sAnti, 4047)
      call createCurrent(P_GMinus, P_A, P_GPlus, P_Z, 7059)
      call createCurrent(P_Hl, P_muMinus, P_muPlus, 8053)
      call createCurrent(P_HMinus, P_HPlus, P_GPlus, P_GMinus, 8733)
      call createCurrent(P_nu_e, P_WMinus, P_ePlus, 3661)
      call createCurrent(P_HaQ, P_Hl, P_G0, P_HhQ, 55)
      call createCurrent(P_GPlus, P_ghZ, P_Hl, P_ghWpAnti, 8122)
      call createCurrent(P_Hl, P_GMinus, P_Hl, P_GPlus, 5597)
      call createCurrent(P_eMinus, P_Hh, P_ePlus, 5761)
      call createCurrent(P_Hh, P_Z, P_HPlus, P_WMinus, 1927)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hh, P_Hh, 5736)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_Hh, 4065)
      call createCurrent(P_GQPlus, P_ghZ, P_ghWpAnti, 2272)
      call createCurrent(P_GPlus, P_Hh, P_G0, P_HMinus, 4536)
      call createCurrent(P_s, P_sAnti, P_HhQ, 5838)
      call createCurrent(P_HlQ, P_A, P_GMinus, P_WQPlus, 1548)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hh, P_HMinus, 7938)
      call createCurrent(P_GQMinus, P_Ha, P_Hl, P_HQPlus, 5487)
      call createCurrent(P_HQMinus, P_ghA, P_Ha, P_ghWmAnti, 3997)
      call createCurrent(P_A, P_WMinus, P_HPlus, P_G0, 9184)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hl, P_HPlus, 6924)
      call createCurrent(P_HlQ, P_Hl, P_HlQ, 5160)
      call createCurrent(P_HaQ, P_HPlus, P_WMinus, P_ZQ, 3861)
      call createCurrent(P_G0, P_c, P_cAnti, 71)
      call createCurrent(P_WMinus, P_ghZ, P_Z, P_ghWmAnti, 650)
      call createCurrent(P_G0, P_GMinus, P_ghWmAnti, P_ghZ, 7455)
      call createCurrent(P_GQ0, P_HMinus, P_GPlus, P_HlQ, 2757)
      call createCurrent(P_ghZ, P_HMinus, P_ghZAnti, P_HPlus, 8078)
      call createCurrent(P_HQPlus, P_HMinus, P_Hl, P_HlQ, 5303)
      call createCurrent(P_HlQ, P_Hl, P_Ha, P_HaQ, 5575)
      call createCurrent(P_ghWm, P_HPlus, P_ghWmAnti, P_HMinus, 1224)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_G0, 1598)
      call createCurrent(P_Ha, P_b, P_bAnti, 7088)
      call createCurrent(P_Ha, P_Ha, P_Ha, P_G0, 2991)
      call createCurrent(P_Ha, P_g, P_G0, P_g, 10724)
      call createCurrent(P_nu_tau, P_nu_tauAnti, P_A, 10385)
      call createCurrent(P_HMinus, P_Z, P_WPlus, 231)
      call createCurrent(P_ghAAnti, P_HMinus, P_Ha, P_ghWp, 3953)
      call createCurrent(P_GMinus, P_GMinus, P_GPlus, P_GPlus, 7424)
      call createCurrent(P_HaQ, P_Hl, P_HaQ, 1142)
      call createCurrent(P_HlQ, P_HMinus, P_A, P_WQPlus, 4311)
      call createCurrent(P_HPlus, P_G0, P_ghWm, P_ghZAnti, 8947)
      call createCurrent(P_AQ, P_HPlus, P_Z, P_HQMinus, 2696)
      call createCurrent(P_ghA, P_ghWpAnti, P_Ha, P_HPlus, 5144)
      call createCurrent(P_HQMinus, P_Z, P_HPlus, P_AQ, 8568)
      call createCurrent(P_WQPlus, P_GMinus, P_Hh, P_ZQ, 6793)
      call createCurrent(P_GMinus, P_Ha, P_ghWp, P_ghZAnti, 9279)
      call createCurrent(P_A, P_G0, P_WPlus, P_GMinus, 5665)
      call createCurrent(P_ghWmAnti, P_ghA, P_Ha, P_HMinus, 4237)
      call createCurrent(P_GQPlus, P_eMinus, P_nu_eAnti, 2978)
      call createCurrent(P_HQMinus, P_GPlus, P_HlQ, 5550)
      call createCurrent(P_Hl, P_Hl, P_Hl, 896)
      call createCurrent(P_tAnti, P_t, P_Ha, 4832)
      call createCurrent(P_WPlus, P_HMinus, P_G0, P_A, 9057)
      call createCurrent(P_HhQ, P_ghZ, P_GMinus, P_ghWmAnti, 2357)
      call createCurrent(P_Ha, P_GMinus, P_GPlus, 9501)
      call createCurrent(P_G0, P_HMinus, P_HPlus, P_Hh, 10240)
      call createCurrent(P_Ha, P_ghWm, P_ghAAnti, P_HPlus, 1473)
      call createCurrent(P_ghWm, P_A, P_ghWmAnti, P_A, 4003)
      call createCurrent(P_HPlus, P_GMinus, P_ghZ, P_ghAAnti, 9654)
      call createCurrent(P_HMinus, P_Ha, P_Hh, P_GPlus, 3525)
      call createCurrent(P_GMinus, P_G0, P_HPlus, 7301)
      call createCurrent(P_Ha, P_HPlus, P_GMinus, P_Hl, 8689)
      call createCurrent(P_ghWpAnti, P_A, P_ghWp, P_A, 876)
      call createCurrent(P_WMinus, P_HPlus, P_Ha, P_A, 8752)
      call createCurrent(P_WMinus, P_HPlus, P_HMinus, P_WPlus, 7259)
      call createCurrent(P_HhQ, P_HPlus, P_HMinus, P_HlQ, 4685)
      call createCurrent(P_HMinus, P_HPlus, P_Hh, P_Ha, 9093)
      call createCurrent(P_HPlus, P_GMinus, P_Ha, P_Ha, 3025)
      call createCurrent(P_HMinus, P_GPlus, P_Hl, 1570)
      call createCurrent(P_HMinus, P_ghWp, P_G0, P_ghAAnti, 9364)
      call createCurrent(P_dAnti, P_d, P_HhQ, 5489)
      call createCurrent(P_WPlus, P_g, P_WMinus, P_g, 10661)
      call createCurrent(P_ghZ, P_ghZAnti, P_Hl, P_HlQ, 7945)
      call createCurrent(P_WQMinus, P_Hh, P_WPlus, P_HaQ, 1883)
      call createCurrent(P_tauMinus, P_nu_tauAnti, P_WQPlus, 1122)
      call createCurrent(P_Z, P_Ha, P_GPlus, P_WMinus, 9500)
      call createCurrent(P_ZQ, P_Z, P_WPlus, P_WQMinus, 1279)
      call createCurrent(P_Z, P_ghZAnti, P_WPlus, P_ghWm, 3442)
      call createCurrent(P_ghZ, P_Z, P_ghWpAnti, P_WPlus, 1646)
      call createCurrent(P_ghA, P_ghAAnti, P_GMinus, P_GPlus, 6938)
      call createCurrent(P_Hl, P_Z, P_A, P_Hl, 9548)
      call createCurrent(P_HhQ, P_Ha, P_HMinus, P_HQPlus, 1588)
      call createCurrent(P_ghWp, P_Hh, P_ghWpAnti, P_HaQ, 4335)
      call createCurrent(P_WQMinus, P_ghZ, P_WPlus, P_ghZAnti, 6802)
      call createCurrent(P_ghWpAnti, P_Z, P_ghWp, P_A, 7679)
      call createCurrent(P_Ha, P_HMinus, P_ghAAnti, P_ghWp, 2170)
      call createCurrent(P_ZQ, P_WMinus, P_WQPlus, 8031)
      call createCurrent(P_ghWpAnti, P_Hh, P_Hl, P_ghWp, 8908)
      call createCurrent(P_HaQ, P_Ha, P_Hh, P_HhQ, 550)
      call createCurrent(P_HlQ, P_GMinus, P_HPlus, P_HhQ, 165)
      call createCurrent(P_WMinus, P_Hl, P_WPlus, P_Hl, 7558)
      call createCurrent(P_GQ0, P_GMinus, P_GPlus, P_GQ0, 2704)
      call createCurrent(P_HPlus, P_GMinus, P_ghAAnti, P_ghZ, 9122)
      call createCurrent(P_ghZ, P_HMinus, P_HPlus, P_ghZAnti, 1561)
      call createCurrent(P_WQMinus, P_HMinus, P_HPlus, P_WQPlus, 7881)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, 5605)
      call createCurrent(P_GQPlus, P_HMinus, P_Ha, P_HaQ, 204)
      call createCurrent(P_ePlus, P_Hh, P_eMinus, 4192)
      call createCurrent(P_HMinus, P_ghA, P_Ha, P_ghWmAnti, 422)
      call createCurrent(P_GMinus, P_ghAAnti, P_ghWp, P_Ha, 9614)
      call createCurrent(P_GQ0, P_Hl, P_HMinus, P_GQPlus, 7312)
      call createCurrent(P_HPlus, P_A, P_HMinus, P_A, 419)
      call createCurrent(P_c, P_cAnti, P_gQ, 8010)
      call createCurrent(P_GQPlus, P_HMinus, P_Hl, P_HhQ, 7643)
      call createCurrent(P_Z, P_G0, P_WMinus, P_GPlus, 5379)
      call createCurrent(P_G0, P_Ha, P_GMinus, P_HPlus, 39)
      call createCurrent(P_HaQ, P_ghWm, P_Hh, P_ghWmAnti, 5321)
      call createCurrent(P_GQ0, P_WMinus, P_GQPlus, 2222)
      call createCurrent(P_AQ, P_eMinus, P_ePlus, 6988)
      call createCurrent(P_GQ0, P_GPlus, P_HMinus, P_GQ0, 7579)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hh, P_GQMinus, 8196)
      call createCurrent(P_ghWmAnti, P_ghWm, P_GPlus, P_HMinus, 9396)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HlQ, 6590)
      call createCurrent(P_Hl, P_G0, P_Ha, 2230)
      call createCurrent(P_GPlus, P_GMinus, P_G0, P_Hh, 9787)
      call createCurrent(P_Hl, P_G0, P_G0, P_Hl, 6704)
      call createCurrent(P_Hh, P_Hl, P_WPlus, P_WMinus, 9038)
      call createCurrent(P_Hl, P_HMinus, P_ghAAnti, P_ghWp, 8563)
      call createCurrent(P_WPlus, P_Hh, P_A, P_GMinus, 133)
      call createCurrent(P_GPlus, P_Hh, P_GMinus, 6470)
      call createCurrent(P_WQPlus, P_ghZAnti, P_ghWm, 2091)
      call createCurrent(P_Hh, P_Hl, 10013)
      call createCurrent(P_WQMinus, P_nu_tau, P_tauPlus, 396)
      call createCurrent(P_GQPlus, P_nu_eAnti, P_eMinus, 8554)
      call createCurrent(P_Z, P_WPlus, P_ghZ, P_ghWpAnti, 1407)
      call createCurrent(P_GQPlus, P_d, P_uAnti, 5798)
      call createCurrent(P_HPlus, P_HMinus, P_Hl, 7309)
      call createCurrent(P_A, P_G0, P_HPlus, P_WMinus, 8944)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghWm, P_Hh, 5595)
      call createCurrent(P_GPlus, P_A, P_A, P_HMinus, 9979)
      call createCurrent(P_Hh, P_Z, P_WPlus, P_HMinus, 4963)
      call createCurrent(P_ghZAnti, P_Ha, P_ghZ, P_G0, 9780)
      call createCurrent(P_WMinus, P_ghZ, P_WPlus, P_ghAAnti, 1648)
      call createCurrent(P_Ha, P_HPlus, P_WMinus, P_A, 4851)
      call createCurrent(P_WQMinus, P_HPlus, P_A, P_HhQ, 6804)
      call createCurrent(P_HaQ, P_Ha, P_HMinus, P_GQPlus, 6221)
      call createCurrent(P_HMinus, P_GPlus, P_G0, P_Ha, 2971)
      call createCurrent(P_ghAAnti, P_ghZ, P_HMinus, P_HPlus, 8194)
      call createCurrent(P_tAnti, P_b, P_GPlus, 5902)
      call createCurrent(P_g, P_HPlus, P_HMinus, P_g, 10773)
      call createCurrent(P_ghWp, P_ghWpAnti, P_G0, P_GQ0, 4847)
      call createCurrent(P_HMinus, P_Hl, P_Z, P_WPlus, 7854)
      call createCurrent(P_G0, P_ghWp, P_ghAAnti, P_HMinus, 9924)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hh, P_HMinus, 1580)
      call createCurrent(P_HPlus, P_G0, P_ghAAnti, P_ghWm, 10273)
      call createCurrent(P_Ha, P_ghWp, P_ghAAnti, P_GMinus, 10391)
      call createCurrent(P_ghZAnti, P_G0, P_ghWm, P_GQPlus, 3165)
      call createCurrent(P_GMinus, P_Hl, P_G0, P_GPlus, 10284)
      call createCurrent(P_ghWp, P_WMinus, P_Z, P_ghAAnti, 4292)
      call createCurrent(P_ZQ, P_Hl, P_GPlus, P_WQMinus, 7886)
      call createCurrent(P_HhQ, P_HPlus, P_G0, P_HQMinus, 632)
      call createCurrent(P_Z, P_A, P_HMinus, P_GPlus, 10461)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HhQ, 3623)
      call createCurrent(P_GQMinus, P_ghZAnti, P_Hl, P_ghWp, 3832)
      call createCurrent(P_HPlus, P_G0, P_ghWpAnti, P_ghA, 9000)
      call createCurrent(P_HQMinus, P_Hh, P_Hl, P_GQPlus, 1895)
      call createCurrent(P_GQMinus, P_G0, P_HPlus, P_HaQ, 3697)
      call createCurrent(P_A, P_HPlus, P_A, P_GMinus, 10469)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Ha, P_HaQ, 4458)
      call createCurrent(P_ghWpAnti, P_Ha, P_HPlus, P_ghA, 2016)
      call createCurrent(P_Hl, P_ghWp, P_ghZAnti, P_HMinus, 7686)
      call createCurrent(P_Hl, P_HMinus, P_HPlus, P_Ha, 9694)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Ha, P_G0, 8960)
      call createCurrent(P_WPlus, P_A, P_Ha, P_GMinus, 9207)
      call createCurrent(P_GPlus, P_A, P_WMinus, P_Hl, 7559)
      call createCurrent(P_WQMinus, P_GPlus, P_ZQ, 3491)
      call createCurrent(P_HlQ, P_Ha, P_HaQ, 5553)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Ha, P_HlQ, 6332)
      call createCurrent(P_WMinus, P_G0, P_Z, P_GPlus, 3301)
      call createCurrent(P_HlQ, P_Ha, P_HMinus, P_HQPlus, 2083)
      call createCurrent(P_sAnti, P_s, P_Z, 3801)
      call createCurrent(P_HQMinus, P_HPlus, P_A, P_AQ, 3701)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghZ, P_A, 1342)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_Hh, 265)
      call createCurrent(P_Hh, P_HPlus, P_GMinus, P_Ha, 1183)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghA, P_Hh, 6381)
      call createCurrent(P_g, P_Z, P_Z, P_g, 10664)
      call createCurrent(P_A, P_WPlus, P_HMinus, 7283)
      call createCurrent(P_GQPlus, P_A, P_WMinus, P_HlQ, 941)
      call createCurrent(P_WPlus, P_Z, P_HMinus, P_Hh, 4976)
      call createCurrent(P_tauMinus, P_nu_tauAnti, P_HPlus, 281)
      call createCurrent(P_s, P_HPlus, P_cAnti, 7545)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghA, P_Hl, 765)
      call createCurrent(P_HaQ, P_G0, P_G0, P_HaQ, 121)
      call createCurrent(P_WMinus, P_Hl, P_Hh, P_WPlus, 9551)
      call createCurrent(P_HPlus, P_HMinus, P_G0, P_Hh, 9240)
      call createCurrent(P_Z, P_Z, P_GMinus, P_GPlus, 3469)
      call createCurrent(P_ghWm, P_A, P_WPlus, P_ghAAnti, 8663)
      call createCurrent(P_Z, P_A, P_ghWp, P_ghWpAnti, 5774)
      call createCurrent(P_GQMinus, P_ghA, P_Hl, P_ghWmAnti, 7576)
      call createCurrent(P_ghZ, P_ghAAnti, P_WPlus, P_WQMinus, 2658)
      call createCurrent(P_WQMinus, P_ghA, P_WPlus, P_ghAAnti, 1839)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghWp, P_AQ, 6267)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghZ, P_WQMinus, 4203)
      call createCurrent(P_Hh, P_GMinus, P_Ha, P_HPlus, 6222)
      call createCurrent(P_ghA, P_HMinus, P_Ha, P_ghWmAnti, 13)
      call createCurrent(P_G0, P_tAnti, P_t, 6411)
      call createCurrent(P_AQ, P_WPlus, P_ghWm, P_ghAAnti, 1156)
      call createCurrent(P_HPlus, P_Z, P_Z, P_HMinus, 1692)
      call createCurrent(P_Hl, P_WPlus, P_HMinus, P_A, 1722)
      call createCurrent(P_uAnti, P_u, P_HhQ, 2641)
      call createCurrent(P_s, P_GPlus, P_cAnti, 625)
      call createCurrent(P_HMinus, P_G0, P_GPlus, P_Hh, 8697)
      call createCurrent(P_Hh, P_Hl, P_ghZ, P_ghZAnti, 10093)
      call createCurrent(P_WMinus, P_GPlus, P_Z, P_Hl, 5511)
      call createCurrent(P_HPlus, P_ghAAnti, P_G0, P_ghWm, 10226)
      call createCurrent(P_GQPlus, P_G0, P_GMinus, P_HlQ, 3960)
      call createCurrent(P_c, P_sAnti, P_HQMinus, 7735)
      call createCurrent(P_ghZ, P_GMinus, P_GPlus, P_ghZAnti, 1443)
      call createCurrent(P_WQMinus, P_ghZ, P_ghWmAnti, 4156)
      call createCurrent(P_c, P_cAnti, 9915)
      call createCurrent(P_Hh, P_HMinus, P_Hh, P_HPlus, 752)
      call createCurrent(P_WQMinus, P_ghWp, P_WPlus, P_ghWpAnti, 1223)
      call createCurrent(P_G0, P_ghWp, P_HMinus, P_ghAAnti, 9793)
      call createCurrent(P_A, P_Hh, P_Z, P_Hl, 10590)
      call createCurrent(P_ghZAnti, P_HMinus, P_GPlus, P_ghA, 9157)
      call createCurrent(P_GMinus, P_ghWmAnti, P_HPlus, P_ghWm, 9581)
      call createCurrent(P_Z, P_GPlus, P_Hl, P_WMinus, 5679)
      call createCurrent(P_GPlus, P_s, P_cAnti, 7563)
      call createCurrent(P_GPlus, P_ghA, P_ghAAnti, P_GMinus, 4507)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_Ha, P_ghA, 657)
      call createCurrent(P_WPlus, P_ghAAnti, P_WMinus, P_ghA, 6255)
      call createCurrent(P_GMinus, P_Ha, P_GPlus, P_Ha, 5936)
      call createCurrent(P_ghAAnti, P_Hh, P_HMinus, P_ghWp, 2229)
      call createCurrent(P_GQMinus, P_WMinus, P_GPlus, P_WQPlus, 5216)
      call createCurrent(P_A, P_WMinus, P_ghWmAnti, P_ghA, 7855)
      call createCurrent(P_ghWmAnti, P_G0, P_ghWm, P_Ha, 8962)
      call createCurrent(P_ghWpAnti, P_ghA, P_WPlus, 1647)
      call createCurrent(P_Ha, P_Z, P_GPlus, P_WMinus, 9227)
      call createCurrent(P_Hh, P_GPlus, P_GMinus, P_Ha, 9211)
      call createCurrent(P_GQPlus, P_Hl, P_HMinus, P_HaQ, 2403)
      call createCurrent(P_HMinus, P_HPlus, P_Z, P_Z, 583)
      call createCurrent(P_WMinus, P_HPlus, P_WPlus, P_GMinus, 8988)
      call createCurrent(P_WQPlus, P_HMinus, P_AQ, 1599)
      call createCurrent(P_Hl, P_HPlus, P_GMinus, P_Hh, 1678)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GQPlus, 4488)
      call createCurrent(P_HMinus, P_Ha, P_ghZ, P_ghWmAnti, 4242)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_GQ0, 418)
      call createCurrent(P_G0, P_Hh, P_Ha, P_Hl, 3524)
      call createCurrent(P_WPlus, P_ghZAnti, P_ghZ, P_WMinus, 4460)
      call createCurrent(P_ghWm, P_ghAAnti, P_G0, P_GPlus, 4667)
      call createCurrent(P_HPlus, P_ghWp, P_ghWpAnti, P_HMinus, 1087)
      call createCurrent(P_WMinus, P_WPlus, P_Hl, P_Hl, 1481)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HaQ, 1979)
      call createCurrent(P_ghZ, P_ghZAnti, P_GMinus, P_GPlus, 6314)
      call createCurrent(P_Hl, P_GMinus, P_ghA, P_ghWmAnti, 4605)
      call createCurrent(P_G0, P_ghWmAnti, P_Ha, P_ghWm, 10223)
      call createCurrent(P_Hl, P_HPlus, P_Ha, P_HMinus, 10408)
      call createCurrent(P_g, P_g, P_HPlus, P_HMinus, 10749)
      call createCurrent(P_HMinus, P_G0, P_ghZ, P_ghWmAnti, 10268)
      call createCurrent(P_WQPlus, P_WMinus, P_HPlus, P_HQMinus, 4742)
      call createCurrent(P_WMinus, P_G0, P_HPlus, 9944)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghA, P_HPlus, 7347)
      call createCurrent(P_Z, P_Z, P_HMinus, P_HPlus, 6709)
      call createCurrent(P_GQPlus, P_HMinus, P_Ha, P_GQ0, 4542)
      call createCurrent(P_ghA, P_Hh, P_ghWpAnti, P_HQPlus, 8667)
      call createCurrent(P_WMinus, P_ghA, P_ghWmAnti, P_A, 2321)
      call createCurrent(P_Z, P_WPlus, P_HMinus, P_Hh, 8747)
      call createCurrent(P_HlQ, P_WPlus, P_HQMinus, 3064)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghA, P_G0, 10207)
      call createCurrent(P_WMinus, P_HPlus, P_Hh, P_Z, 1432)
      call createCurrent(P_G0, P_d, P_dAnti, 634)
      call createCurrent(P_HPlus, P_Z, P_GMinus, 10247)
      call createCurrent(P_ghWp, P_ghZAnti, P_GQMinus, 2687)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Ha, P_G0, 9161)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_HlQ, 6300)
      call createCurrent(P_Ha, P_A, P_HMinus, P_WPlus, 8128)
      call createCurrent(P_ghA, P_Z, P_WPlus, P_ghWpAnti, 4246)
      call createCurrent(P_Ha, P_Ha, P_ghWp, P_ghWpAnti, 4613)
      call createCurrent(P_A, P_Hh, P_G0, 9299)
      call createCurrent(P_GPlus, P_Ha, P_HMinus, P_Hl, 4546)
      call createCurrent(P_Z, P_HMinus, P_HPlus, 7955)
      call createCurrent(P_HhQ, P_A, P_GPlus, P_WQMinus, 1234)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HMinus, P_HQPlus, 3206)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_HlQ, 95)
      call createCurrent(P_bAnti, P_Z, P_b, 7475)
      call createCurrent(P_Hl, P_ghZAnti, P_HPlus, P_ghWm, 6443)
      call createCurrent(P_Ha, P_G0, P_HPlus, P_HMinus, 2313)
      call createCurrent(P_d, P_dAnti, P_A, 355)
      call createCurrent(P_Z, P_ghWp, P_ghWpAnti, P_Z, 7050)
      call createCurrent(P_WMinus, P_Z, P_Hl, P_GPlus, 2715)
      call createCurrent(P_muMinus, P_muPlus, 9110)
      call createCurrent(P_GMinus, P_ghAAnti, P_Hh, P_ghWp, 5374)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_HhQ, 4096)
      call createCurrent(P_HhQ, P_HPlus, P_ghWm, P_ghZAnti, 1723)
      call createCurrent(P_WQPlus, P_GMinus, P_HlQ, 6284)
      call createCurrent(P_GPlus, P_HMinus, P_Ha, P_Ha, 86)
      call createCurrent(P_A, P_HPlus, P_HMinus, 7803)
      call createCurrent(P_GQ0, P_WPlus, P_WQMinus, 658)
      call createCurrent(P_Ha, P_HPlus, P_GMinus, P_Hh, 4910)
      call createCurrent(P_GQ0, P_ghZAnti, P_G0, P_ghZ, 7928)
      call createCurrent(P_GQ0, P_Ha, P_Hh, P_HlQ, 5098)
      call createCurrent(P_HQPlus, P_GMinus, P_HPlus, P_GQMinus, 6947)
      call createCurrent(P_HMinus, P_Z, P_Ha, P_WPlus, 5861)
      call createCurrent(P_Hl, P_G0, P_HMinus, P_HPlus, 9556)
      call createCurrent(P_GMinus, P_G0, P_Hl, P_GPlus, 9607)
      call createCurrent(P_ghA, P_Hh, P_ghWpAnti, P_GQPlus, 3795)
      call createCurrent(P_HPlus, P_Hh, P_HMinus, P_Hl, 741)
      call createCurrent(P_ghZAnti, P_WPlus, P_A, P_ghWm, 7235)
      call createCurrent(P_GMinus, P_ghZAnti, P_GPlus, P_ghZ, 718)
      call createCurrent(P_G0, P_ghWp, P_Ha, P_ghWpAnti, 10064)
      call createCurrent(P_ghWpAnti, P_G0, P_GPlus, P_ghA, 1688)
      call createCurrent(P_Z, P_GPlus, P_WMinus, P_Ha, 9472)
      call createCurrent(P_HMinus, P_Hl, P_WPlus, P_Z, 2034)
      call createCurrent(P_HPlus, P_GMinus, P_Ha, 6880)
      call createCurrent(P_Hl, P_ghZ, P_ghZAnti, P_Hl, 7356)
      call createCurrent(P_muPlus, P_A, P_muMinus, 149)
      call createCurrent(P_G0, P_GMinus, P_WPlus, P_A, 2500)
      call createCurrent(P_Ha, P_A, P_Z, P_G0, 10674)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_GQ0, 1080)
      call createCurrent(P_HaQ, P_Hh, P_GPlus, P_GQMinus, 4039)
      call createCurrent(P_HQPlus, P_HPlus, P_HMinus, P_GQMinus, 4673)
      call createCurrent(P_HhQ, P_GPlus, P_ghWm, P_ghAAnti, 1060)
      call createCurrent(P_AQ, P_WMinus, P_G0, P_GQPlus, 3510)
      call createCurrent(P_ghWp, P_Z, P_ghWpAnti, 273)
      call createCurrent(P_ghWm, P_GMinus, P_ghWmAnti, P_GPlus, 8513)
      call createCurrent(P_HlQ, P_GPlus, P_HMinus, P_HlQ, 6447)
      call createCurrent(P_HaQ, P_HMinus, P_Z, P_WQPlus, 228)
      call createCurrent(P_GQMinus, P_HPlus, P_Hh, P_GQ0, 8686)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_Hh, 2965)
      call createCurrent(P_HaQ, P_GMinus, P_Ha, P_GQPlus, 4486)
      call createCurrent(P_ghWmAnti, P_A, P_ghZ, P_WQMinus, 1981)
      call createCurrent(P_WQMinus, P_nu_e, P_ePlus, 7669)
      call createCurrent(P_Ha, P_G0, P_HMinus, P_HPlus, 7865)
      call createCurrent(P_HQPlus, P_Hl, P_GMinus, P_HaQ, 6344)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Ha, P_HaQ, 3185)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghZ, P_HQPlus, 3963)
      call createCurrent(P_A, P_Ha, P_Hl, 8958)
      call createCurrent(P_Ha, P_G0, P_Hl, P_Hh, 3770)
      call createCurrent(P_HQPlus, P_G0, P_G0, P_GQMinus, 4174)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, 6263)
      call createCurrent(P_WPlus, P_HMinus, P_Z, P_Hh, 4083)
      call createCurrent(P_GPlus, P_Hh, P_HMinus, P_Ha, 8179)
      call createCurrent(P_Ha, P_GMinus, P_HPlus, P_Hl, 8507)
      call createCurrent(P_HhQ, P_GPlus, P_HMinus, P_HhQ, 3298)
      call createCurrent(P_GQMinus, P_G0, P_HPlus, P_HlQ, 2221)
      call createCurrent(P_HPlus, P_nu_eAnti, P_eMinus, 2366)
      call createCurrent(P_GQPlus, P_GMinus, P_G0, P_HaQ, 7875)
      call createCurrent(P_WQPlus, P_ghA, P_A, P_ghWpAnti, 1066)
      call createCurrent(P_HQPlus, P_GPlus, P_HMinus, P_HQMinus, 2797)
      call createCurrent(P_GPlus, P_ghWmAnti, P_ghWm, P_GMinus, 3635)
      call createCurrent(P_ghA, P_G0, P_GPlus, P_ghWpAnti, 7908)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghWm, P_Hl, 432)
      call createCurrent(P_ghZAnti, P_Ha, P_GMinus, P_ghWp, 9685)
      call createCurrent(P_ghA, P_ghZAnti, P_HPlus, P_GMinus, 10473)
      call createCurrent(P_Ha, P_GPlus, P_ghWpAnti, P_ghA, 10108)
      call createCurrent(P_HQMinus, P_G0, P_G0, P_GQPlus, 3813)
      call createCurrent(P_ghWp, P_ghZAnti, P_G0, P_GQMinus, 251)
      call createCurrent(P_WPlus, P_Z, P_GMinus, P_Ha, 8856)
      call createCurrent(P_A, P_Ha, P_A, P_Ha, 10624)
      call createCurrent(P_HhQ, P_GMinus, P_ghZ, P_ghWmAnti, 5530)
      call createCurrent(P_G0, P_HMinus, P_WPlus, P_Z, 9214)
      call createCurrent(P_d, P_g, P_dAnti, 3052)
      call createCurrent(P_Hl, P_ghAAnti, P_GPlus, P_ghWm, 4986)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, 2492)
      call createCurrent(P_G0, P_HPlus, P_HMinus, P_Ha, 1181)
      call createCurrent(P_s, P_sAnti, P_AQ, 5237)
      call createCurrent(P_c, P_cAnti, P_HhQ, 3053)
      call createCurrent(P_Z, P_WPlus, P_Hh, P_HMinus, 423)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hl, P_HQPlus, 8585)
      call createCurrent(P_HQPlus, P_s, P_cAnti, 6950)
      call createCurrent(P_GQPlus, P_GMinus, P_Ha, P_HlQ, 2456)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, 5680)
      call createCurrent(P_ghA, P_ghWmAnti, P_Z, P_WQMinus, 1380)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghZ, 1807)
      call createCurrent(P_WPlus, P_WMinus, P_Hl, P_Hh, 9499)
      call createCurrent(P_GQ0, P_WPlus, P_Z, P_GQMinus, 8214)
      call createCurrent(P_ghA, P_ghWpAnti, P_GPlus, P_Hl, 1422)
      call createCurrent(P_G0, P_ghAAnti, P_ghWm, P_HPlus, 9876)
      call createCurrent(P_nu_e, P_nu_eAnti, P_ZQ, 2061)
      call createCurrent(P_HaQ, P_GPlus, P_HMinus, P_GQ0, 3535)
      call createCurrent(P_GQ0, P_GPlus, P_HQMinus, 714)
      call createCurrent(P_WMinus, P_Hh, P_A, P_GPlus, 5607)
      call createCurrent(P_G0, P_HPlus, P_A, P_WMinus, 9506)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghWm, P_Ha, 3823)
      call createCurrent(P_HlQ, P_Hh, P_HlQ, 1540)
      call createCurrent(P_t, P_bAnti, P_WMinus, 7972)
      call createCurrent(P_Hl, P_Hh, P_A, P_Z, 10700)
      call createCurrent(P_ghZ, P_WPlus, P_ghWpAnti, P_A, 1389)
      call createCurrent(P_ghZAnti, P_ghWm, P_Ha, P_HPlus, 4294)
      call createCurrent(P_GQ0, P_ghWmAnti, P_G0, P_ghWm, 4750)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghZ, P_HMinus, 2101)
      call createCurrent(P_WQPlus, P_ghAAnti, P_WMinus, P_ghZ, 7102)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghA, P_GMinus, 9680)
      call createCurrent(P_ghA, P_A, P_WMinus, P_ghWmAnti, 2011)
      call createCurrent(P_WPlus, P_ghA, P_ghWpAnti, P_A, 29)
      call createCurrent(P_WPlus, P_WMinus, P_Z, 7244)
      call createCurrent(P_HlQ, P_WPlus, P_GMinus, P_ZQ, 3060)
      call createCurrent(P_ghA, P_ghWmAnti, P_G0, P_GMinus, 7671)
      call createCurrent(P_bAnti, P_t, P_WQMinus, 5254)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_Ha, 9744)
      call createCurrent(P_HlQ, P_WPlus, P_Z, P_HQMinus, 8046)
      call createCurrent(P_HaQ, P_Hh, P_WPlus, P_WQMinus, 6400)
      call createCurrent(P_HQMinus, P_Hh, P_GPlus, P_HhQ, 1321)
      call createCurrent(P_ghZAnti, P_ghWp, P_GQMinus, 3260)
      call createCurrent(P_Hh, P_ghWm, P_ghZAnti, P_HPlus, 1714)
      call createCurrent(P_GMinus, P_Z, P_Ha, P_WPlus, 9994)
      call createCurrent(P_GPlus, P_Ha, P_ghZAnti, P_ghWm, 9673)
      call createCurrent(P_ghWpAnti, P_A, P_Z, P_ghWp, 548)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_G0, 5371)
      call createCurrent(P_GPlus, P_ghAAnti, P_GMinus, P_ghA, 3930)
      call createCurrent(P_HQMinus, P_Ha, P_GPlus, P_GQ0, 3196)
      call createCurrent(P_A, P_WPlus, P_G0, P_GMinus, 5802)
      call createCurrent(P_Hl, P_GPlus, P_G0, P_GMinus, 9032)
      call createCurrent(P_tauMinus, P_tauPlus, P_Hl, 7925)
      call createCurrent(P_ghWmAnti, P_ghZ, P_HMinus, P_Ha, 3793)
      call createCurrent(P_GQ0, P_HPlus, P_Hl, P_GQMinus, 5532)
      call createCurrent(P_HQPlus, P_HMinus, P_Hl, P_GQ0, 6620)
      call createCurrent(P_GQMinus, P_Hh, P_ghWmAnti, P_ghA, 5846)
      call createCurrent(P_WPlus, P_Ha, P_A, P_GMinus, 9778)
      call createCurrent(P_A, P_Ha, P_HMinus, P_WPlus, 430)
      call createCurrent(P_Hh, P_GMinus, P_Hh, P_GPlus, 7912)
      call createCurrent(P_ghWm, P_ghWmAnti, P_G0, P_HlQ, 5959)
      call createCurrent(P_gQ, P_ghGAnti, P_g, P_ghG, 5738)
      call createCurrent(P_G0, P_A, P_G0, P_Z, 9842)
      call createCurrent(P_AQ, P_muPlus, P_muMinus, 3891)
      call createCurrent(P_GPlus, P_ghAAnti, P_Hl, P_ghWm, 647)
      call createCurrent(P_HMinus, P_A, P_Hh, P_WPlus, 8084)
      call createCurrent(P_ghWmAnti, P_ghA, P_Hh, P_GQMinus, 8657)
      call createCurrent(P_Hl, P_Hh, P_HPlus, P_HMinus, 2355)
      call createCurrent(P_HPlus, P_GMinus, P_ghAAnti, P_ghA, 9021)
      call createCurrent(P_HPlus, P_ghWm, P_Hh, P_ghAAnti, 4681)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghA, P_Hl, 8018)
      call createCurrent(P_HaQ, P_ghWpAnti, P_Hh, P_ghWp, 1068)
      call createCurrent(P_ghWpAnti, P_ghZ, P_G0, P_GQPlus, 7165)
      call createCurrent(P_HhQ, P_ghWp, P_G0, P_ghWpAnti, 6797)
      call createCurrent(P_Hl, P_GMinus, P_Ha, P_GPlus, 9343)
      call createCurrent(P_Hh, P_ghZ, P_GPlus, P_ghWpAnti, 5360)
      call createCurrent(P_ghGAnti, P_ghG, P_gQ, 4701)
      call createCurrent(P_ghWp, P_ghAAnti, P_Hh, P_HMinus, 3015)
      call createCurrent(P_GQPlus, P_G0, P_HQMinus, 2494)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_HlQ, 4010)
      call createCurrent(P_gQ, P_tAnti, P_t, 3460)
      call createCurrent(P_AQ, P_WMinus, P_Hh, P_GQPlus, 5872)
      call createCurrent(P_GPlus, P_ghWpAnti, P_Hl, P_ghZ, 2726)
      call createCurrent(P_WQPlus, P_HMinus, P_Hh, P_ZQ, 991)
      call createCurrent(P_AQ, P_ghWm, P_WPlus, P_ghZAnti, 1916)
      call createCurrent(P_HMinus, P_ghA, P_GPlus, P_ghZAnti, 10283)
      call createCurrent(P_HhQ, P_WMinus, P_A, P_GQPlus, 4816)
      call createCurrent(P_AQ, P_WMinus, P_A, P_WQPlus, 4761)
      call createCurrent(P_ghWm, P_WPlus, P_A, P_ghAAnti, 1628)
      call createCurrent(P_ghWpAnti, P_Ha, P_Ha, P_ghWp, 6700)
      call createCurrent(P_HQPlus, P_HMinus, P_Hh, P_HhQ, 1872)
      call createCurrent(P_HMinus, P_Ha, P_HPlus, P_Hl, 9124)
      call createCurrent(P_GPlus, P_Hh, P_Ha, P_GMinus, 9874)
      call createCurrent(P_HPlus, P_Ha, P_HMinus, P_Ha, 1409)
      call createCurrent(P_eMinus, P_ePlus, P_A, 2097)
      call createCurrent(P_Ha, P_Hh, P_GMinus, P_GPlus, 9128)
      call createCurrent(P_HMinus, P_GPlus, P_ghWpAnti, P_ghWp, 9875)
      call createCurrent(P_GQPlus, P_GMinus, P_HMinus, P_GQPlus, 1841)
      call createCurrent(P_HaQ, P_tauMinus, P_tauPlus, 8632)
      call createCurrent(P_HMinus, P_GPlus, P_ghWp, P_ghWpAnti, 10256)
      call createCurrent(P_GPlus, P_ghWmAnti, P_GMinus, P_ghWm, 7535)
      call createCurrent(P_WQPlus, P_GPlus, P_WMinus, P_GQMinus, 2199)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hh, P_GQMinus, 5266)
      call createCurrent(P_A, P_HMinus, P_GPlus, 9325)
      call createCurrent(P_G0, P_Z, P_Ha, P_Z, 9138)
      call createCurrent(P_Ha, P_Hl, P_GMinus, P_HPlus, 4419)
      call createCurrent(P_ghWpAnti, P_ghWp, P_GMinus, P_GPlus, 3955)
      call createCurrent(P_G0, P_HMinus, P_WPlus, P_A, 8940)
      call createCurrent(P_WPlus, P_WMinus, P_ghAAnti, P_ghA, 8177)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_HlQ, 1447)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, P_HhQ, 1353)
      call createCurrent(P_GMinus, P_GPlus, P_G0, P_Hh, 9358)
      call createCurrent(P_HaQ, P_Ha, P_HPlus, P_HQMinus, 7998)
      call createCurrent(P_Hl, P_WMinus, P_Z, P_HPlus, 1049)
      call createCurrent(P_Hh, P_GMinus, P_Hl, P_HPlus, 372)
      call createCurrent(P_ghA, P_GMinus, P_ghZAnti, P_GQPlus, 1416)
      call createCurrent(P_ghWmAnti, P_GPlus, P_GMinus, P_ghWm, 2464)
      call createCurrent(P_WMinus, P_WPlus, P_WMinus, P_WPlus, 3146)
      call createCurrent(P_Ha, P_HPlus, P_WMinus, P_Z, 5874)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_Hh, 7246)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghA, P_GQPlus, 6877)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghZ, P_HMinus, 9020)
      call createCurrent(P_G0, P_HPlus, P_Ha, P_GMinus, 3480)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, 8644)
      call createCurrent(P_Ha, P_WMinus, P_Z, P_GPlus, 9587)
      call createCurrent(P_Hh, P_uAnti, P_u, 875)
      call createCurrent(P_Hl, P_Hh, P_g, P_g, 10645)
      call createCurrent(P_HaQ, P_HPlus, P_Ha, P_GQMinus, 2824)
      call createCurrent(P_WQPlus, P_HMinus, P_Ha, P_ZQ, 3614)
      call createCurrent(P_Hh, P_GMinus, P_WPlus, P_Z, 8718)
      call createCurrent(P_HhQ, P_WPlus, P_Z, P_HQMinus, 2912)
      call createCurrent(P_Ha, P_g, P_Ha, P_g, 10573)
      call createCurrent(P_A, P_HMinus, P_HPlus, 5180)
      call createCurrent(P_Ha, P_ghWmAnti, P_ghA, P_HMinus, 971)
      call createCurrent(P_HlQ, P_GPlus, P_Hh, P_GQMinus, 7916)
      call createCurrent(P_GQPlus, P_ghA, P_Hh, P_ghWpAnti, 326)
      call createCurrent(P_Z, P_Hl, P_A, 9210)
      call createCurrent(P_HPlus, P_GMinus, 9933)
      call createCurrent(P_WPlus, P_Hl, P_WMinus, 4447)
      call createCurrent(P_Z, P_WPlus, P_G0, P_GMinus, 0)
      call createCurrent(P_ghA, P_GPlus, P_G0, P_ghWpAnti, 4672)
      call createCurrent(P_ghA, P_ghAAnti, P_WPlus, P_WQMinus, 3279)
      call createCurrent(P_GQMinus, P_Hl, P_Hh, P_HQPlus, 8019)
      call createCurrent(P_HQPlus, P_HMinus, P_Hh, P_HaQ, 8415)
      call createCurrent(P_Hh, P_Hl, P_Hl, P_Hh, 557)
      call createCurrent(P_Hl, P_HMinus, P_Ha, P_GPlus, 8746)
      call createCurrent(P_HhQ, P_A, P_WMinus, P_HQPlus, 6328)
      call createCurrent(P_Z, P_WMinus, P_GPlus, P_Hl, 1487)
      call createCurrent(P_Ha, P_d, P_dAnti, 2305)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, P_Ha, 8346)
      call createCurrent(P_GQMinus, P_Z, P_WQPlus, 491)
      call createCurrent(P_WMinus, P_Z, P_HPlus, P_Ha, 8465)
      call createCurrent(P_WQMinus, P_WPlus, P_ghWpAnti, P_ghWp, 4608)
      call createCurrent(P_WQMinus, P_GPlus, P_GQ0, 8291)
      call createCurrent(P_WPlus, P_Z, P_Hh, P_HMinus, 5060)
      call createCurrent(P_HPlus, P_G0, P_ghWpAnti, P_ghZ, 9950)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWp, P_GMinus, 7463)
      call createCurrent(P_gQ, P_g, P_g, P_gQ, 7792)
      call createCurrent(P_ZQ, P_A, P_HPlus, P_HQMinus, 6007)
      call createCurrent(P_G0, P_ghWmAnti, P_GMinus, P_ghA, 5253)
      call createCurrent(P_GMinus, P_WPlus, P_A, P_G0, 5637)
      call createCurrent(P_GMinus, P_Hh, P_ghWp, P_ghZAnti, 2985)
      call createCurrent(P_ghWmAnti, P_ghZ, P_G0, P_GMinus, 3893)
      call createCurrent(P_ghWp, P_Hh, P_ghWpAnti, P_Hl, 10032)
      call createCurrent(P_GMinus, P_ghZ, P_GPlus, P_ghZAnti, 8313)
      call createCurrent(P_HhQ, P_Ha, P_GMinus, P_HQPlus, 8574)
      call createCurrent(P_ghAAnti, P_G0, P_ghWm, P_HPlus, 10384)
      call createCurrent(P_WPlus, P_nu_eAnti, P_eMinus, 3069)
      call createCurrent(P_Hl, P_A, P_WMinus, P_GPlus, 4050)
      call createCurrent(P_GQ0, P_GPlus, P_WMinus, P_ZQ, 5515)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghZ, P_GPlus, 4205)
      call createCurrent(P_ghWpAnti, P_ghWp, P_AQ, 3561)
      call createCurrent(P_Hh, P_GMinus, P_WPlus, 8416)
      call createCurrent(P_ghA, P_ghZAnti, P_HMinus, P_GPlus, 9113)
      call createCurrent(P_HhQ, P_ghZAnti, P_Hh, P_ghZ, 5599)
      call createCurrent(P_G0, P_GMinus, P_ghWp, P_ghZAnti, 3410)
      call createCurrent(P_GQMinus, P_Hl, P_GPlus, P_GQ0, 4433)
      call createCurrent(P_c, P_cAnti, P_Hh, 4385)
      call createCurrent(P_ghZ, P_ghZAnti, P_GMinus, P_GQPlus, 7618)
      call createCurrent(P_A, P_Hl, P_GMinus, P_WPlus, 4659)
      call createCurrent(P_Hl, P_HMinus, P_ghWp, P_ghAAnti, 6298)
      call createCurrent(P_Z, P_ghWp, P_ghWpAnti, P_A, 2813)
      call createCurrent(P_HMinus, P_Hl, P_ghWmAnti, P_ghZ, 6525)
      call createCurrent(P_HhQ, P_HMinus, P_HQPlus, 5812)
      call createCurrent(P_ghAAnti, P_Hl, P_GMinus, P_ghWp, 4334)
      call createCurrent(P_WPlus, P_Hh, P_HMinus, 3014)
      call createCurrent(P_WMinus, P_Hh, P_Z, P_HPlus, 6533)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, 5440)
      call createCurrent(P_HPlus, P_Hh, P_Z, P_WMinus, 2571)
      call createCurrent(P_A, P_G0, P_WMinus, P_GPlus, 1235)
      call createCurrent(P_nu_tauAnti, P_tauMinus, P_HPlus, 1777)
      call createCurrent(P_ghWmAnti, P_A, P_ghZ, P_WMinus, 5403)
      call createCurrent(P_u, P_g, P_uAnti, 1519)
      call createCurrent(P_GPlus, P_G0, P_Z, P_WMinus, 1919)
      call createCurrent(P_ghWm, P_HPlus, P_Ha, P_ghZAnti, 7091)
      call createCurrent(P_GMinus, P_WPlus, P_Ha, P_Z, 9819)
      call createCurrent(P_ghWm, P_Hl, P_ghZAnti, P_GPlus, 5654)
      call createCurrent(P_Z, P_WMinus, P_HPlus, P_G0, 9637)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Hl, P_Hl, 7611)
      call createCurrent(P_nu_mu, P_muPlus, P_WQMinus, 7191)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghA, P_HQPlus, 5955)
      call createCurrent(P_ghWpAnti, P_WPlus, P_WPlus, P_ghWm, 8435)
      call createCurrent(P_ghWm, P_Ha, P_ghWmAnti, P_Ha, 1988)
      call createCurrent(P_Hh, P_GMinus, P_ghWmAnti, P_ghA, 3161)
      call createCurrent(P_HQPlus, P_Hl, P_ghWm, P_ghZAnti, 2901)
      call createCurrent(P_ePlus, P_eMinus, P_A, 8373)
      call createCurrent(P_WMinus, P_ghWp, P_WPlus, P_ghWpAnti, 3987)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghZ, P_GQPlus, 4212)
      call createCurrent(P_GMinus, P_G0, P_GPlus, P_Hl, 10091)
      call createCurrent(P_GQMinus, P_Hh, P_ghA, P_ghWmAnti, 2074)
      call createCurrent(P_Hl, P_WPlus, P_A, P_HMinus, 6230)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_G0, 10056)
      call createCurrent(P_HPlus, P_Hl, P_ghWpAnti, P_ghZ, 5903)
      call createCurrent(P_ghWpAnti, P_ghA, P_Hh, P_GQPlus, 2666)
      call createCurrent(P_HPlus, P_ghWm, P_ghAAnti, P_Hh, 5020)
      call createCurrent(P_Hh, P_GMinus, P_HPlus, P_Hl, 6317)
      call createCurrent(P_GQ0, P_GMinus, P_HQPlus, 5078)
      call createCurrent(P_HaQ, P_Ha, P_WPlus, P_WQMinus, 4967)
      call createCurrent(P_ghWmAnti, P_GPlus, P_ghWm, P_HMinus, 9841)
      call createCurrent(P_dAnti, P_HMinus, P_u, 8191)
      call createCurrent(P_WQPlus, P_ghA, P_WMinus, P_ghZAnti, 3663)
      call createCurrent(P_GPlus, P_Ha, P_ghWpAnti, P_ghA, 10075)
      call createCurrent(P_HPlus, P_Z, P_Hh, P_WMinus, 5373)
      call createCurrent(P_ZQ, P_ghWmAnti, P_A, P_ghWm, 2892)
      call createCurrent(P_ZQ, P_tAnti, P_t, 266)
      call createCurrent(P_HlQ, P_HPlus, P_ghZAnti, P_ghWm, 8166)
      call createCurrent(P_GPlus, P_ghWm, P_ghAAnti, P_Hh, 8282)
      call createCurrent(P_HlQ, P_Hl, P_G0, P_GQ0, 2039)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghZ, P_GMinus, 2880)
      call createCurrent(P_Hh, P_ghWpAnti, P_HPlus, P_ghZ, 4998)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_Hl, 472)
      call createCurrent(P_G0, P_ghWp, P_GMinus, P_ghAAnti, 1475)
      call createCurrent(P_HMinus, P_WPlus, P_G0, P_A, 8951)
      call createCurrent(P_G0, P_Ha, P_ghZAnti, P_ghZ, 9728)
      call createCurrent(P_GMinus, P_WPlus, P_A, P_Hl, 4134)
      call createCurrent(P_HMinus, P_G0, P_ghWp, P_ghZAnti, 9935)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_Hl, P_ghA, 7338)
      call createCurrent(P_HPlus, P_HMinus, P_HMinus, P_HPlus, 2621)
      call createCurrent(P_ghWm, P_ghWpAnti, P_WPlus, P_WPlus, 5327)
      call createCurrent(P_Ha, P_Z, P_HPlus, P_WMinus, 5426)
      call createCurrent(P_ghA, P_HMinus, P_GPlus, P_ghAAnti, 9769)
      call createCurrent(P_HaQ, P_ghWm, P_HPlus, P_ghZAnti, 6105)
      call createCurrent(P_sAnti, P_s, P_Hh, 1251)
      call createCurrent(P_Hh, P_ghZAnti, P_Hh, P_ghZ, 3303)
      call createCurrent(P_Hh, P_HPlus, P_HMinus, P_Hh, 4547)
      call createCurrent(P_GQMinus, P_Ha, P_Hh, P_HQPlus, 5328)
      call createCurrent(P_HhQ, P_Z, P_HPlus, P_WQMinus, 4714)
      call createCurrent(P_GQ0, P_Ha, P_HlQ, 4340)
      call createCurrent(P_Z, P_ghWm, P_ghWmAnti, P_Z, 8174)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_HhQ, 4896)
      call createCurrent(P_HlQ, P_WMinus, P_Ha, P_WQPlus, 1523)
      call createCurrent(P_HMinus, P_HPlus, P_G0, P_Hh, 10297)
      call createCurrent(P_ghWmAnti, P_ghWp, P_WMinus, P_WQMinus, 6408)
      call createCurrent(P_ghWm, P_Hh, P_ghWmAnti, P_HhQ, 2597)
      call createCurrent(P_dAnti, P_u, P_GMinus, 2334)
      call createCurrent(P_GQ0, P_G0, P_HPlus, P_GQMinus, 1613)
      call createCurrent(P_Z, P_c, P_cAnti, 710)
      call createCurrent(P_HMinus, P_GPlus, P_GMinus, P_GPlus, 6739)
      call createCurrent(P_Hh, P_WPlus, P_Hl, P_WMinus, 9837)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghA, P_Hh, 508)
      call createCurrent(P_Hl, P_cAnti, P_c, 5634)
      call createCurrent(P_GQPlus, P_uAnti, P_d, 7578)
      call createCurrent(P_ghAAnti, P_G0, P_ghWp, P_HMinus, 10440)
      call createCurrent(P_Hl, P_A, P_HMinus, P_WPlus, 6852)
      call createCurrent(P_HlQ, P_Ha, P_WPlus, P_WQMinus, 5428)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghA, P_HQPlus, 6351)
      call createCurrent(P_HPlus, P_ghWpAnti, P_Hh, P_ghA, 2122)
      call createCurrent(P_HQMinus, P_A, P_HQPlus, 1063)
      call createCurrent(P_gQ, P_sAnti, P_s, 1950)
      call createCurrent(P_WPlus, P_Z, P_ghWpAnti, P_ghZ, 5924)
      call createCurrent(P_ghWmAnti, P_G0, P_HMinus, P_ghA, 9722)
      call createCurrent(P_A, P_HMinus, P_WPlus, 7689)
      call createCurrent(P_HhQ, P_HPlus, P_HMinus, P_HhQ, 6498)
      call createCurrent(P_HlQ, P_ghWpAnti, P_G0, P_ghWp, 245)
      call createCurrent(P_Hh, P_ghZ, P_Hh, P_ghZAnti, 8571)
      call createCurrent(P_HMinus, P_t, P_bAnti, 511)
      call createCurrent(P_GQMinus, P_Z, P_Hl, P_WQPlus, 722)
      call createCurrent(P_GMinus, P_ghA, P_ghZAnti, P_GPlus, 6079)
      call createCurrent(P_GQPlus, P_Hl, P_Hl, P_HQMinus, 4299)
      call createCurrent(P_GQ0, P_Hl, P_ZQ, 1482)
      call createCurrent(P_ZQ, P_HPlus, P_HQMinus, 7067)
      call createCurrent(P_bAnti, P_b, P_HlQ, 2654)
      call createCurrent(P_Ha, P_GPlus, P_GMinus, P_Hl, 9008)
      call createCurrent(P_eMinus, P_nu_eAnti, P_WQPlus, 7251)
      call createCurrent(P_ghWmAnti, P_HMinus, P_ghA, P_Hh, 1273)
      call createCurrent(P_HMinus, P_HPlus, P_Hh, P_Hl, 3569)
      call createCurrent(P_ghWmAnti, P_WMinus, P_ghA, P_A, 5456)
      call createCurrent(P_G0, P_Hl, P_G0, 1136)
      call createCurrent(P_ghWmAnti, P_Hl, P_Hl, P_ghWm, 1404)
      call createCurrent(P_HPlus, P_Ha, P_WMinus, 7568)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hl, P_Hl, 1643)
      call createCurrent(P_GQ0, P_HMinus, P_Hh, P_HQPlus, 616)
      call createCurrent(P_g, P_g, P_G0, P_Ha, 10521)
      call createCurrent(P_s, P_A, P_sAnti, 774)
      call createCurrent(P_HQMinus, P_ghWmAnti, P_ghA, 4689)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_GQ0, 931)
      call createCurrent(P_ghAAnti, P_ghWm, P_Hh, P_GPlus, 8486)
      call createCurrent(P_GMinus, P_HPlus, P_WMinus, P_WPlus, 10035)
      call createCurrent(P_muPlus, P_muMinus, P_G0, 1323)
      call createCurrent(P_HPlus, P_Hl, P_HMinus, P_Ha, 10300)
      call createCurrent(P_ZQ, P_ghWp, P_WMinus, P_ghZAnti, 158)
      call createCurrent(P_GPlus, P_HMinus, P_G0, P_Ha, 8564)
      call createCurrent(P_Hl, P_HPlus, P_ghAAnti, P_ghWm, 2783)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_Hl, 868)
      call createCurrent(P_ghWpAnti, P_ghA, P_Ha, P_HQPlus, 555)
      call createCurrent(P_GQMinus, P_Ha, P_G0, P_HQPlus, 2220)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_Hl, 6573)
      call createCurrent(P_Ha, P_ghWmAnti, P_G0, P_ghWm, 10068)
      call createCurrent(P_Hl, P_HPlus, P_HMinus, P_Hl, 7185)
      call createCurrent(P_HQPlus, P_Ha, P_WMinus, P_ZQ, 4949)
      call createCurrent(P_Z, P_HPlus, P_WMinus, P_Ha, 8798)
      call createCurrent(P_ghA, P_GPlus, P_GMinus, P_ghAAnti, 8298)
      call createCurrent(P_HMinus, P_Hh, P_GPlus, P_Hl, 5384)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghZ, P_GMinus, 10458)
      call createCurrent(P_WQPlus, P_WMinus, P_Hh, P_HhQ, 819)
      call createCurrent(P_A, P_Hh, P_Hl, P_A, 10619)
      call createCurrent(P_ghWp, P_G0, P_Ha, P_ghWpAnti, 9193)
      call createCurrent(P_tauPlus, P_G0, P_tauMinus, 4451)
      call createCurrent(P_WQMinus, P_Z, P_GPlus, P_HlQ, 7369)
      call createCurrent(P_A, P_Hl, P_WPlus, P_GMinus, 3382)
      call createCurrent(P_ghWmAnti, P_Hh, P_ghA, P_GMinus, 4620)
      call createCurrent(P_HMinus, P_Ha, P_HPlus, P_G0, 296)
      call createCurrent(P_Z, P_ghWmAnti, P_A, P_ghWm, 1148)
      call createCurrent(P_WMinus, P_ghZAnti, P_ghWp, 244)
      call createCurrent(P_AQ, P_s, P_sAnti, 2839)
      call createCurrent(P_G0, P_WMinus, P_GPlus, P_Z, 1593)
      call createCurrent(P_HlQ, P_ghZAnti, P_Hl, P_ghZ, 6697)
      call createCurrent(P_ghZ, P_GMinus, P_ghZAnti, P_HPlus, 10356)
      call createCurrent(P_HlQ, P_GMinus, P_HQPlus, 7266)
      call createCurrent(P_Hh, P_HPlus, P_WMinus, P_A, 3400)
      call createCurrent(P_ghWmAnti, P_Z, P_ghWm, P_Z, 1681)
      call createCurrent(P_HMinus, P_Hh, P_Hl, P_HPlus, 516)
      call createCurrent(P_GPlus, P_WMinus, P_Z, P_Hh, 8244)
      call createCurrent(P_HPlus, P_ghA, P_ghZAnti, P_HMinus, 7709)
      call createCurrent(P_A, P_A, P_GPlus, P_GMinus, 5620)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_Hl, 222)
      call createCurrent(P_WMinus, P_ghA, P_Z, P_ghWmAnti, 1732)
      call createCurrent(P_HMinus, P_Z, P_HPlus, P_A, 7086)
      call createCurrent(P_G0, P_Hl, P_G0, P_Ha, 10508)
      call createCurrent(P_HQMinus, P_HPlus, P_HMinus, P_HQPlus, 6483)
      call createCurrent(P_GQMinus, P_GPlus, P_HMinus, P_GQPlus, 6724)
      call createCurrent(P_g, P_GMinus, P_GPlus, P_g, 10551)
      call createCurrent(P_WPlus, P_GMinus, P_WMinus, P_HPlus, 8841)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, 156)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghZ, P_HPlus, 10394)
      call createCurrent(P_bAnti, P_b, P_AQ, 5083)
      call createCurrent(P_HMinus, P_Z, P_G0, P_WPlus, 10453)
      call createCurrent(P_WQPlus, P_G0, P_GMinus, P_ZQ, 1940)
      call createCurrent(P_G0, P_Hh, P_Hl, P_G0, 2798)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_HhQ, 2923)
      call createCurrent(P_sAnti, P_s, P_Ha, 3952)
      call createCurrent(P_Ha, P_GPlus, P_WMinus, P_A, 8849)
      call createCurrent(P_A, P_Z, 10126)
      call createCurrent(P_ghWmAnti, P_A, P_ghA, P_WMinus, 1019)
      call createCurrent(P_Ha, P_Ha, P_WMinus, P_WPlus, 401)
      call createCurrent(P_Hh, P_Hl, P_Z, P_A, 10557)
      call createCurrent(P_GQMinus, P_GPlus, P_GMinus, P_GQPlus, 4002)
      call createCurrent(P_A, P_A, P_G0, P_Ha, 10553)
      call createCurrent(P_Hl, P_WPlus, P_GMinus, P_Z, 603)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, P_Hl, 8428)
      call createCurrent(P_ZQ, P_Z, P_ghWp, P_ghWpAnti, 3618)
      call createCurrent(P_ghZ, P_GMinus, P_Hl, P_ghWmAnti, 2007)
      call createCurrent(P_HQPlus, P_ghZAnti, P_Ha, P_ghWm, 7892)
      call createCurrent(P_WMinus, P_G0, P_HPlus, P_Z, 8961)
      call createCurrent(P_Z, P_ghA, P_WMinus, P_ghWmAnti, 6570)
      call createCurrent(P_ghAAnti, P_ghZ, P_GPlus, P_GMinus, 2764)
      call createCurrent(P_HhQ, P_WPlus, P_HQMinus, 286)
      call createCurrent(P_WMinus, P_Hl, P_Z, P_GPlus, 3877)
      call createCurrent(P_HaQ, P_Z, P_HlQ, 438)
      call createCurrent(P_GMinus, P_Z, P_GPlus, P_A, 2957)
      call createCurrent(P_ghA, P_ghWpAnti, P_WQPlus, 7241)
      call createCurrent(P_HQPlus, P_Ha, P_ghA, P_ghWpAnti, 1925)
      call createCurrent(P_Ha, P_Hl, P_GPlus, P_GMinus, 10208)
      call createCurrent(P_HaQ, P_GPlus, P_Ha, P_HQMinus, 6881)
      call createCurrent(P_ghWpAnti, P_A, P_ghWp, P_ZQ, 2636)
      call createCurrent(P_ghWmAnti, P_ghWm, P_G0, P_GQ0, 3624)
      call createCurrent(P_HhQ, P_HPlus, P_HQMinus, 5990)
      call createCurrent(P_HhQ, P_ghZAnti, P_HPlus, P_ghWm, 2276)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_HlQ, 2802)
      call createCurrent(P_ghA, P_HMinus, P_GPlus, P_ghZAnti, 9338)
      call createCurrent(P_WQMinus, P_Z, P_HPlus, P_HaQ, 3387)
      call createCurrent(P_WQMinus, P_GPlus, P_AQ, 3945)
      call createCurrent(P_HQPlus, P_eMinus, P_nu_eAnti, 1865)
      call createCurrent(P_A, P_Ha, P_Z, P_Ha, 10402)
      call createCurrent(P_ghGAnti, P_g, P_ghG, 5480)
      call createCurrent(P_Hh, P_Ha, P_Ha, P_Hh, 7414)
      call createCurrent(P_Z, P_Ha, P_WMinus, P_HPlus, 1954)
      call createCurrent(P_s, P_cAnti, P_WPlus, 6630)
      call createCurrent(P_ghWp, P_HPlus, P_ghWpAnti, P_HQMinus, 306)
      call createCurrent(P_WMinus, P_Hh, P_HPlus, P_Z, 4782)
      call createCurrent(P_HlQ, P_ghZAnti, P_HMinus, P_ghWp, 7927)
      call createCurrent(P_HlQ, P_ghAAnti, P_HPlus, P_ghWm, 1982)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_HaQ, 5750)
      call createCurrent(P_Hl, P_ghZAnti, P_ghZ, 720)
      call createCurrent(P_HQMinus, P_G0, P_GPlus, P_HaQ, 790)
      call createCurrent(P_ghZ, P_HPlus, P_GMinus, P_ghZAnti, 9304)
      call createCurrent(P_HQPlus, P_Hh, P_ghZAnti, P_ghWm, 8364)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghA, P_GMinus, 4268)
      call createCurrent(P_WQMinus, P_G0, P_Z, P_GQPlus, 2194)
      call createCurrent(P_GQPlus, P_Hh, P_Hl, P_HQMinus, 2921)
      call createCurrent(P_Hl, P_Z, P_WMinus, P_GPlus, 2760)
      call createCurrent(P_ZQ, P_c, P_cAnti, 4679)
      call createCurrent(P_HQMinus, P_Z, P_Z, P_HQPlus, 8623)
      call createCurrent(P_HaQ, P_t, P_tAnti, 7239)
      call createCurrent(P_Hh, P_Z, P_Hh, P_Z, 771)
      call createCurrent(P_Hl, P_HMinus, P_Hl, P_HPlus, 6194)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_GQ0, 7890)
      call createCurrent(P_G0, P_HMinus, P_ghZAnti, P_ghWp, 9121)
      call createCurrent(P_Ha, P_A, P_Ha, P_A, 10731)
      call createCurrent(P_Hh, P_ghWm, P_ghAAnti, P_HPlus, 755)
      call createCurrent(P_GPlus, P_ghZ, P_ghZAnti, P_HMinus, 9288)
      call createCurrent(P_WMinus, P_ghWmAnti, P_A, P_ghA, 141)
      call createCurrent(P_GQPlus, P_WMinus, P_Z, P_GQ0, 3416)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghA, P_WPlus, 5980)
      call createCurrent(P_HhQ, P_G0, P_HaQ, 8573)
      call createCurrent(P_ghWm, P_GPlus, P_Ha, P_ghZAnti, 10374)
      call createCurrent(P_A, P_HMinus, P_Hl, P_WPlus, 4448)
      call createCurrent(P_WMinus, P_Z, P_A, P_WPlus, 2113)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Hl, P_Hl, 2463)
      call createCurrent(P_ePlus, P_eMinus, P_HlQ, 7847)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghA, P_Z, 8355)
      call createCurrent(P_ghWmAnti, P_ghA, P_HMinus, P_Ha, 1365)
      call createCurrent(P_ghZAnti, P_ghA, P_WPlus, P_WMinus, 3545)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Ha, P_HMinus, 6442)
      call createCurrent(P_Hl, P_GPlus, P_GMinus, 8437)
      call createCurrent(P_ZQ, P_ghWpAnti, P_Z, P_ghWp, 2788)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hl, P_HaQ, 168)
      call createCurrent(P_nu_mu, P_A, P_nu_muAnti, 9838)
      call createCurrent(P_Hl, P_Hl, P_Hh, P_Hh, 8539)
      call createCurrent(P_HlQ, P_b, P_bAnti, 7358)
      call createCurrent(P_GPlus, P_HMinus, P_Hh, 4270)
      call createCurrent(P_WPlus, P_ghZAnti, P_ghA, P_WMinus, 6049)
      call createCurrent(P_Ha, P_GMinus, P_ghZ, P_ghWmAnti, 10146)
      call createCurrent(P_G0, P_ghAAnti, P_GPlus, P_ghWm, 8234)
      call createCurrent(P_GQ0, P_muPlus, P_muMinus, 5406)
      call createCurrent(P_HlQ, P_Hh, P_HPlus, P_GQMinus, 1512)
      call createCurrent(P_G0, P_bAnti, P_b, 6176)
      call createCurrent(P_GMinus, P_ghA, P_Hl, P_ghWmAnti, 5064)
      call createCurrent(P_ZQ, P_nu_tauAnti, P_nu_tau, 177)
      call createCurrent(P_Z, P_Z, P_Hl, P_Hl, 3372)
      call createCurrent(P_ghWmAnti, P_HMinus, P_Hl, P_ghA, 203)
      call createCurrent(P_Z, P_WPlus, P_WMinus, P_A, 6203)
      call createCurrent(P_HQMinus, P_A, P_WPlus, P_HhQ, 5441)
      call createCurrent(P_GQ0, P_HPlus, P_GMinus, P_HhQ, 680)
      call createCurrent(P_ghWp, P_Hh, P_ghWpAnti, P_HhQ, 4514)
      call createCurrent(P_WQMinus, P_WPlus, P_Hh, P_HaQ, 3393)
      call createCurrent(P_AQ, P_ghWm, P_A, P_ghWmAnti, 7401)
      call createCurrent(P_GPlus, P_ghAAnti, P_ghWm, P_Hl, 3704)
      call createCurrent(P_GMinus, P_tauPlus, P_nu_tau, 3284)
      call createCurrent(P_HPlus, P_Z, P_Ha, P_WMinus, 1932)
      call createCurrent(P_muPlus, P_Ha, P_muMinus, 3401)
      call createCurrent(P_G0, P_ghA, P_GPlus, P_ghWpAnti, 813)
      call createCurrent(P_A, P_ghWpAnti, P_WPlus, P_ghZ, 2107)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_Hl, 2043)
      call createCurrent(P_HhQ, P_ghWm, P_HPlus, P_ghAAnti, 4303)
      call createCurrent(P_HlQ, P_WPlus, P_A, P_HQMinus, 1247)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hh, P_GQPlus, 8470)
      call createCurrent(P_HQMinus, P_Hl, P_ghZ, P_ghWmAnti, 6815)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Z, P_AQ, 3799)
      call createCurrent(P_Z, P_b, P_bAnti, 865)
      call createCurrent(P_ghWp, P_GMinus, P_Hh, P_ghZAnti, 6775)
      call createCurrent(P_GPlus, P_ghWmAnti, P_HMinus, P_ghWm, 9189)
      call createCurrent(P_WPlus, P_WMinus, P_Z, P_A, 7590)
      call createCurrent(P_GQMinus, P_Hl, P_HPlus, P_HhQ, 8618)
      call createCurrent(P_WQPlus, P_WPlus, P_ghWm, P_ghWpAnti, 5476)
      call createCurrent(P_WMinus, P_ghZAnti, P_ghWp, P_A, 1933)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghWp, P_Ha, 8590)
      call createCurrent(P_WPlus, P_tAnti, P_b, 1670)
      call createCurrent(P_GPlus, P_G0, P_Hl, P_HMinus, 4122)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghWp, P_Ha, 1272)
      call createCurrent(P_GQ0, P_Hh, P_ghWp, P_ghWpAnti, 7771)
      call createCurrent(P_ghZ, P_GMinus, P_ghWmAnti, P_G0, 299)
      call createCurrent(P_HhQ, P_G0, P_ghWmAnti, P_ghWm, 2219)
      call createCurrent(P_WQMinus, P_WPlus, P_Hl, P_GQ0, 4007)
      call createCurrent(P_WPlus, P_G0, P_HMinus, P_Z, 10397)
      call createCurrent(P_GMinus, P_G0, P_Ha, P_GPlus, 2532)
      call createCurrent(P_Z, P_Hl, P_HPlus, P_WMinus, 4752)
      call createCurrent(P_HhQ, P_Ha, P_ghWm, P_ghWmAnti, 4013)
      call createCurrent(P_GQ0, P_G0, P_ghWm, P_ghWmAnti, 5200)
      call createCurrent(P_HlQ, P_G0, P_Ha, P_HhQ, 3323)
      call createCurrent(P_GMinus, P_Hl, P_HPlus, P_Hh, 1536)
      call createCurrent(P_ePlus, P_eMinus, P_HaQ, 2385)
      call createCurrent(P_GQPlus, P_ghWm, P_ghZAnti, 7660)
      call createCurrent(P_GQPlus, P_WMinus, P_A, P_HhQ, 3202)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_HhQ, 7549)
      call createCurrent(P_A, P_A, P_HMinus, P_GPlus, 10454)
      call createCurrent(P_GPlus, P_A, P_GMinus, 5863)
      call createCurrent(P_ghWm, P_ghWmAnti, P_A, 7123)
      call createCurrent(P_GMinus, P_G0, P_WPlus, P_A, 6168)
      call createCurrent(P_Z, P_u, P_uAnti, 368)
      call createCurrent(P_WMinus, P_ghA, P_WPlus, P_ghZAnti, 1173)
      call createCurrent(P_ghZ, P_Hl, P_GMinus, P_ghWmAnti, 5058)
      call createCurrent(P_ZQ, P_WPlus, P_Hl, P_HQMinus, 4817)
      call createCurrent(P_Ha, P_HPlus, P_ghWm, P_ghZAnti, 6622)
      call createCurrent(P_Hh, P_A, P_Z, 9243)
      call createCurrent(P_HQPlus, P_Ha, P_Hl, P_GQMinus, 2431)
      call createCurrent(P_WPlus, P_ghAAnti, P_ghZ, P_WMinus, 8357)
      call createCurrent(P_HQMinus, P_Ha, P_GPlus, P_HaQ, 1081)
      call createCurrent(P_ghWmAnti, P_ghWm, P_G0, P_HhQ, 5517)
      call createCurrent(P_Z, P_HMinus, P_WPlus, P_Ha, 1252)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Z, 5478)
      call createCurrent(P_Hl, P_Z, P_Hh, P_A, 10584)
      call createCurrent(P_Z, P_g, P_g, P_A, 10745)
      call createCurrent(P_ghZAnti, P_ghWm, P_WPlus, P_Z, 6918)
      call createCurrent(P_sAnti, P_s, P_AQ, 1862)
      call createCurrent(P_ZQ, P_Z, P_HPlus, P_HQMinus, 7742)
      call createCurrent(P_ghA, P_WPlus, P_ghWpAnti, P_A, 7006)
      call createCurrent(P_ghWm, P_Ha, P_ghWmAnti, P_HhQ, 8280)
      call createCurrent(P_GQPlus, P_Hh, P_ghZ, P_ghWpAnti, 8096)
      call createCurrent(P_g, P_ghG, P_ghGAnti, P_g, 1652)
      call createCurrent(P_ghZ, P_ghZAnti, P_G0, P_G0, 1607)
      call createCurrent(P_HQPlus, P_Hh, P_G0, P_GQMinus, 2324)
      call createCurrent(P_Hh, P_HMinus, P_ghWmAnti, P_ghA, 4698)
      call createCurrent(P_G0, P_G0, P_Hl, 2284)
      call createCurrent(P_sAnti, P_c, P_WQMinus, 7213)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, 3427)
      call createCurrent(P_ghA, P_G0, P_ghWmAnti, P_GQMinus, 7125)
      call createCurrent(P_WPlus, P_A, P_Z, P_WMinus, 3091)
      call createCurrent(P_Z, P_HPlus, P_HMinus, P_Z, 8521)
      call createCurrent(P_GMinus, P_A, P_HPlus, P_Z, 9455)
      call createCurrent(P_ghZAnti, P_Hl, P_ghZ, 5276)
      call createCurrent(P_ghA, P_ghWmAnti, P_HMinus, 8504)
      call createCurrent(P_HMinus, P_ghAAnti, P_G0, P_ghWp, 9900)
      call createCurrent(P_WQPlus, P_Z, P_WMinus, P_ZQ, 7454)
      call createCurrent(P_g, P_g, P_A, P_g, 10713)
      call createCurrent(P_WPlus, P_Ha, P_Ha, P_WMinus, 5904)
      call createCurrent(P_HPlus, P_WMinus, P_Z, P_Hl, 6936)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_Ha, 10333)
      call createCurrent(P_bAnti, P_b, P_ZQ, 3508)
      call createCurrent(P_b, P_bAnti, P_HhQ, 688)
      call createCurrent(P_ghWp, P_ghWpAnti, P_A, P_ZQ, 5336)
      call createCurrent(P_HlQ, P_GPlus, P_Z, P_WQMinus, 5660)
      call createCurrent(P_ghA, P_Ha, P_ghWpAnti, P_HPlus, 3027)
      call createCurrent(P_ghAAnti, P_GMinus, P_ghWp, P_G0, 1477)
      call createCurrent(P_GMinus, P_Hh, P_ghWmAnti, P_ghZ, 6934)
      call createCurrent(P_GQPlus, P_ghWm, P_GMinus, P_ghWmAnti, 4440)
      call createCurrent(P_nu_e, P_nu_eAnti, 10348)
      call createCurrent(P_Hl, P_ghWp, P_GMinus, P_ghAAnti, 1609)
      call createCurrent(P_GQMinus, P_GPlus, P_Hl, P_GQ0, 4765)
      call createCurrent(P_HPlus, P_eMinus, P_nu_eAnti, 3648)
      call createCurrent(P_G0, P_A, P_Z, P_Ha, 10654)
      call createCurrent(P_Ha, P_WMinus, P_GPlus, P_A, 9801)
      call createCurrent(P_Hl, P_Hh, P_ghWm, P_ghWmAnti, 9860)
      call createCurrent(P_GQPlus, P_WMinus, P_Z, P_HlQ, 5301)
      call createCurrent(P_Hh, P_ghWp, P_ghAAnti, P_HMinus, 5786)
      call createCurrent(P_WQPlus, P_Z, P_Hh, P_GQMinus, 4493)
      call createCurrent(P_ghWmAnti, P_Z, P_ghWm, 6336)
      call createCurrent(P_GPlus, P_Ha, P_ghWm, P_ghAAnti, 9681)
      call createCurrent(P_GQ0, P_G0, P_Ha, P_GQ0, 5347)
      call createCurrent(P_HaQ, P_Ha, P_Hl, P_HlQ, 6817)
      call createCurrent(P_HQMinus, P_Ha, P_Hh, P_GQPlus, 2523)
      call createCurrent(P_HMinus, P_Hh, P_ghWmAnti, P_ghZ, 5185)
      call createCurrent(P_A, P_Z, P_HPlus, P_GMinus, 8837)
      call createCurrent(P_HaQ, P_ghZ, P_Ha, P_ghZAnti, 3741)
      call createCurrent(P_ghWp, P_GMinus, P_GPlus, P_ghWpAnti, 2718)
      call createCurrent(P_GPlus, P_WPlus, P_GMinus, P_WMinus, 3300)
      call createCurrent(P_GPlus, P_Ha, P_Hh, P_HMinus, 6379)
      call createCurrent(P_HhQ, P_G0, P_Hl, P_HaQ, 6376)
      call createCurrent(P_ghZAnti, P_Hh, P_ghWp, P_HQMinus, 1944)
      call createCurrent(P_Hh, P_HPlus, P_Ha, P_HMinus, 10475)
      call createCurrent(P_Z, P_ghWm, P_WPlus, P_ghZAnti, 2470)
      call createCurrent(P_ZQ, P_nu_tau, P_nu_tauAnti, 457)
      call createCurrent(P_HlQ, P_ghWm, P_GPlus, P_ghAAnti, 917)
      call createCurrent(P_uAnti, P_d, P_HQPlus, 7476)
      call createCurrent(P_G0, P_ghWpAnti, P_ghWp, P_G0, 8688)
      call createCurrent(P_ghZ, P_GMinus, P_HPlus, P_ghZAnti, 9275)
      call createCurrent(P_WPlus, P_HPlus, P_WMinus, P_GMinus, 10363)
      call createCurrent(P_ghWm, P_Ha, P_ghAAnti, P_GPlus, 9773)
      call createCurrent(P_Hl, P_Z, P_WPlus, P_GMinus, 2499)
      call createCurrent(P_GQPlus, P_HMinus, P_G0, P_HlQ, 4391)
      call createCurrent(P_WPlus, P_ghWpAnti, P_WMinus, P_ghWp, 1611)
      call createCurrent(P_HQPlus, P_G0, P_Hl, P_GQMinus, 1821)
      call createCurrent(P_ghZ, P_GPlus, P_ghAAnti, P_HMinus, 9784)
      call createCurrent(P_eMinus, P_ePlus, P_Ha, 1164)
      call createCurrent(P_GPlus, P_ghWm, P_ghAAnti, P_G0, 7133)
      call createCurrent(P_Hl, P_ghAAnti, P_GMinus, P_ghWp, 1137)
      call createCurrent(P_A, P_ghWpAnti, P_A, P_ghWp, 2269)
      call createCurrent(P_ZQ, P_Z, P_GPlus, P_GQMinus, 5232)
      call createCurrent(P_HQPlus, P_HMinus, P_HhQ, 7921)
      call createCurrent(P_ghAAnti, P_WMinus, P_ghWp, P_A, 3424)
      call createCurrent(P_ghA, P_ghWpAnti, P_WPlus, P_Z, 1634)
      call createCurrent(P_GPlus, P_Ha, P_HMinus, 3151)
      call createCurrent(P_WMinus, P_A, P_GPlus, P_Hh, 6753)
      call createCurrent(P_AQ, P_ghWpAnti, P_A, P_ghWp, 6756)
      call createCurrent(P_WQMinus, P_G0, P_GPlus, P_AQ, 7224)
      call createCurrent(P_HaQ, P_Hl, P_Hh, P_HaQ, 5317)
      call createCurrent(P_WQPlus, P_WMinus, P_Z, P_ZQ, 5783)
      call createCurrent(P_HPlus, P_Hl, P_GMinus, 8524)
      call createCurrent(P_ghAAnti, P_ghWm, P_WPlus, 676)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghWp, 2885)
      call createCurrent(P_WPlus, P_WMinus, P_WPlus, P_WMinus, 2807)
      call createCurrent(P_ghA, P_HPlus, P_Hh, P_ghWpAnti, 6478)
      call createCurrent(P_HQPlus, P_HMinus, P_Ha, P_GQ0, 1454)
      call createCurrent(P_HlQ, P_WMinus, P_WQPlus, 567)
      call createCurrent(P_WMinus, P_GPlus, P_Hl, P_Z, 5962)
      call createCurrent(P_G0, P_ghZAnti, P_GPlus, P_ghWm, 3842)
      call createCurrent(P_GQ0, P_GMinus, P_HPlus, P_HhQ, 1797)
      call createCurrent(P_WQPlus, P_ghZAnti, P_WMinus, P_ghA, 1445)
      call createCurrent(P_HhQ, P_G0, P_ghWm, P_ghWmAnti, 5278)
      call createCurrent(P_WMinus, P_ghZAnti, P_ghWp, P_Z, 783)
      call createCurrent(P_ghWp, P_ghWpAnti, P_Ha, P_Ha, 4734)
      call createCurrent(P_Z, P_HMinus, P_WPlus, P_G0, 9400)
      call createCurrent(P_GPlus, P_Ha, P_WMinus, P_A, 9379)
      call createCurrent(P_HQPlus, P_WPlus, P_WMinus, P_HQMinus, 485)
      call createCurrent(P_GQMinus, P_HPlus, P_HMinus, P_GQPlus, 803)
      call createCurrent(P_HMinus, P_Hh, P_Ha, P_GPlus, 3046)
      call createCurrent(P_GMinus, P_G0, P_Hh, P_HPlus, 3579)
      call createCurrent(P_AQ, P_Hh, P_GMinus, P_WQPlus, 4838)
      call createCurrent(P_GMinus, P_Hl, P_HPlus, P_Hl, 5167)
      call createCurrent(P_Hl, P_ghWpAnti, P_GPlus, P_ghZ, 75)
      call createCurrent(P_HMinus, P_G0, P_A, P_WPlus, 8919)
      call createCurrent(P_ghWmAnti, P_WMinus, P_Z, P_ghA, 7635)
      call createCurrent(P_ghWm, P_ghWmAnti, P_G0, P_HhQ, 1556)
      call createCurrent(P_HhQ, P_Hh, P_Hh, P_HlQ, 922)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HQPlus, 8163)
      call createCurrent(P_ghA, P_G0, P_ghWpAnti, P_GQPlus, 7975)
      call createCurrent(P_WQMinus, P_Hh, P_WPlus, P_HhQ, 4069)
      call createCurrent(P_G0, P_Hh, P_G0, P_Ha, 10649)
      call createCurrent(P_tauPlus, P_GMinus, P_nu_tau, 2969)
      call createCurrent(P_WQPlus, P_ghWpAnti, P_A, P_ghA, 3456)
      call createCurrent(P_ghWp, P_ghZAnti, P_WMinus, 6294)
      call createCurrent(P_HQMinus, P_HPlus, P_HhQ, 6074)
      call createCurrent(P_HQPlus, P_Ha, P_Ha, P_GQMinus, 7304)
      call createCurrent(P_ZQ, P_HPlus, P_WMinus, P_HhQ, 3791)
      call createCurrent(P_Ha, P_HMinus, P_ghWmAnti, P_ghZ, 2672)
      call createCurrent(P_GPlus, P_ghAAnti, P_Hh, P_ghWm, 5958)
      call createCurrent(P_Hh, P_HPlus, P_GMinus, 2546)
      call createCurrent(P_ghA, P_WPlus, P_ghZAnti, P_WQMinus, 120)
      call createCurrent(P_ghZ, P_HPlus, P_ghWpAnti, P_HhQ, 2428)
      call createCurrent(P_ghZAnti, P_ghZ, P_HMinus, P_HPlus, 891)
      call createCurrent(P_HQMinus, P_A, P_HPlus, P_ZQ, 4774)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghA, P_HPlus, 2304)
      call createCurrent(P_A, P_GPlus, P_HMinus, P_A, 9209)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hl, P_HPlus, 807)
      call createCurrent(P_G0, P_Hh, P_HPlus, P_HMinus, 9864)
      call createCurrent(P_nu_tauAnti, P_tauMinus, P_GPlus, 5598)
      call createCurrent(P_WPlus, P_ghWpAnti, P_Z, P_ghZ, 8427)
      call createCurrent(P_GQ0, P_ghWm, P_GPlus, P_ghZAnti, 1553)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghA, P_Ha, 678)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, 782)
      call createCurrent(P_ghWm, P_ghZAnti, P_HPlus, P_Hl, 7229)
      call createCurrent(P_HQMinus, P_HPlus, P_ghAAnti, P_ghA, 2506)
      call createCurrent(P_ghWpAnti, P_ghZ, P_WQPlus, 2596)
      call createCurrent(P_HlQ, P_GMinus, P_Hh, P_HQPlus, 2342)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Z, P_Z, 2397)
      call createCurrent(P_AQ, P_HMinus, P_Ha, P_WQPlus, 6438)
      call createCurrent(P_ghWp, P_WPlus, P_ghWpAnti, P_WQMinus, 363)
      call createCurrent(P_Hh, P_Hh, P_HPlus, P_GMinus, 494)
      call createCurrent(P_GQPlus, P_Hh, P_GQMinus, 4159)
      call createCurrent(P_G0, P_ghZAnti, P_HMinus, P_ghWp, 9503)
      call createCurrent(P_Z, P_Ha, P_HMinus, P_WPlus, 967)
      call createCurrent(P_ZQ, P_ghWp, P_Z, P_ghWpAnti, 3237)
      call createCurrent(P_G0, P_Hl, P_Hh, P_G0, 5544)
      call createCurrent(P_ghAAnti, P_ghA, P_HPlus, P_GMinus, 9208)
      call createCurrent(P_HPlus, P_WMinus, P_A, P_G0, 9618)
      call createCurrent(P_Hh, P_HMinus, P_ghWp, P_ghZAnti, 399)
      call createCurrent(P_HMinus, P_Ha, P_HPlus, P_Ha, 3670)
      call createCurrent(P_WPlus, P_G0, P_Z, P_HMinus, 10409)
      call createCurrent(P_G0, P_Ha, P_G0, P_G0, 7214)
      call createCurrent(P_GMinus, P_ghAAnti, P_G0, P_ghWp, 3565)
      call createCurrent(P_HQMinus, P_Ha, P_ghZ, P_ghWmAnti, 3534)
      call createCurrent(P_A, P_ghWm, P_ghZAnti, P_WPlus, 1861)
      call createCurrent(P_ghZAnti, P_G0, P_ghZ, P_Ha, 9836)
      call createCurrent(P_G0, P_Hh, P_HPlus, P_GMinus, 8187)
      call createCurrent(P_A, P_tAnti, P_t, 2983)
      call createCurrent(P_GPlus, P_ghA, P_ghWpAnti, P_Ha, 9746)
      call createCurrent(P_WQPlus, P_Z, P_GMinus, P_HlQ, 6615)
      call createCurrent(P_u, P_uAnti, P_GQ0, 5644)
      call createCurrent(P_Ha, P_Z, P_WMinus, P_HPlus, 6083)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hh, P_GMinus, 981)
      call createCurrent(P_HQPlus, P_HMinus, P_HMinus, P_HQPlus, 58)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghWp, P_GQMinus, 4171)
      call createCurrent(P_GQMinus, P_Hl, P_Hl, P_GQPlus, 2363)
      call createCurrent(P_Hl, P_GMinus, P_GPlus, P_Hh, 1875)
      call createCurrent(P_ghA, P_ghAAnti, P_GPlus, P_GQMinus, 8764)
      call createCurrent(P_HMinus, P_G0, P_Hl, P_HPlus, 9159)
      call createCurrent(P_ghAAnti, P_GMinus, P_HPlus, P_ghA, 10420)
      call createCurrent(P_ePlus, P_nu_e, P_GMinus, 4360)
      call createCurrent(P_HQMinus, P_Hh, P_HPlus, P_HhQ, 5827)
      call createCurrent(P_HlQ, P_GMinus, P_ghWp, P_ghAAnti, 6451)
      call createCurrent(P_eMinus, P_ePlus, P_Hh, 3854)
      call createCurrent(P_Ha, P_Z, P_WPlus, P_GMinus, 9241)
      call createCurrent(P_GMinus, P_ghZAnti, P_G0, P_ghWp, 5422)
      call createCurrent(P_ghA, P_WPlus, P_A, P_ghWpAnti, 1317)
      call createCurrent(P_b, P_A, P_bAnti, 3362)
      call createCurrent(P_WMinus, P_WPlus, P_WPlus, P_WMinus, 7932)
      call createCurrent(P_GQPlus, P_Z, P_WMinus, P_GQ0, 1782)
      call createCurrent(P_ghWm, P_GPlus, P_ghAAnti, P_HlQ, 2982)
      call createCurrent(P_A, P_WMinus, P_GPlus, P_Ha, 9273)
      call createCurrent(P_ghZAnti, P_ghWp, P_GMinus, 8305)
      call createCurrent(P_Hh, P_GPlus, P_GMinus, 3862)
      call createCurrent(P_g, P_A, P_A, P_g, 10548)
      call createCurrent(P_WMinus, P_GPlus, P_A, P_Hl, 3979)
      call createCurrent(P_GPlus, P_ghZ, P_ghWpAnti, P_G0, 4265)
      call createCurrent(P_HQPlus, P_Hh, P_ghA, P_ghWpAnti, 4869)
      call createCurrent(P_WQMinus, P_ghAAnti, P_WPlus, P_ghZ, 2056)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HaQ, 7597)
      call createCurrent(P_WMinus, P_GPlus, P_G0, P_A, 1527)
      call createCurrent(P_GQPlus, P_GMinus, P_Z, P_ZQ, 8458)
      call createCurrent(P_ghA, P_Hl, P_ghWpAnti, P_HPlus, 50)
      call createCurrent(P_ZQ, P_GPlus, P_WMinus, P_GQ0, 5259)
      call createCurrent(P_s, P_Hh, P_sAnti, 4513)
      call createCurrent(P_HlQ, P_G0, P_HPlus, P_GQMinus, 4092)
      call createCurrent(P_eMinus, P_nu_eAnti, P_HQPlus, 1754)
      call createCurrent(P_Z, P_Hl, P_Hh, P_A, 10597)
      call createCurrent(P_HPlus, P_Hh, P_ghZ, P_ghWpAnti, 6378)
      call createCurrent(P_HMinus, P_HPlus, P_Hl, 7481)
      call createCurrent(P_nu_tauAnti, P_nu_tau, P_A, 9098)
      call createCurrent(P_ghZ, P_ghWpAnti, P_G0, P_GPlus, 3836)
      call createCurrent(P_HPlus, P_ghZAnti, P_G0, P_ghWm, 8997)
      call createCurrent(P_A, P_G0, P_Ha, P_Z, 10629)
      call createCurrent(P_HQPlus, P_A, P_WMinus, P_HaQ, 3958)
      call createCurrent(P_WPlus, P_Ha, P_GMinus, 9922)
      call createCurrent(P_ghWm, P_A, P_A, P_ghWmAnti, 8599)
      call createCurrent(P_HMinus, P_GPlus, P_ghA, P_ghZAnti, 9560)
      call createCurrent(P_HlQ, P_HMinus, P_Ha, P_HQPlus, 927)
      call createCurrent(P_WMinus, P_GPlus, P_Hl, P_A, 4095)
      call createCurrent(P_ghWm, P_ghWmAnti, P_WMinus, P_WPlus, 8176)
      call createCurrent(P_GQMinus, P_HPlus, P_Ha, P_GQ0, 1357)
      call createCurrent(P_ZQ, P_tauMinus, P_tauPlus, 5027)
      call createCurrent(P_GPlus, P_HPlus, P_GMinus, P_GMinus, 6020)
      call createCurrent(P_GPlus, P_GMinus, P_GPlus, P_GMinus, 8206)
      call createCurrent(P_HaQ, P_GPlus, P_GQMinus, 5788)
      call createCurrent(P_HMinus, P_HPlus, P_g, P_g, 10699)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, 262)
      call createCurrent(P_GQPlus, P_tauMinus, P_nu_tauAnti, 3339)
      call createCurrent(P_ghAAnti, P_HPlus, P_G0, P_ghWm, 9168)
      call createCurrent(P_c, P_HMinus, P_sAnti, 5136)
      call createCurrent(P_ghA, P_ghWmAnti, P_Ha, P_HQMinus, 7280)
      call createCurrent(P_GQPlus, P_HMinus, P_HPlus, P_GQMinus, 1091)
      call createCurrent(P_GQPlus, P_G0, P_ghWpAnti, P_ghZ, 1256)
      call createCurrent(P_WPlus, P_GMinus, 9444)
      call createCurrent(P_HaQ, P_GPlus, P_Hl, P_GQMinus, 6123)
      call createCurrent(P_ghA, P_Ha, P_ghWmAnti, P_HQMinus, 1449)
      call createCurrent(P_HMinus, P_Ha, P_Hl, P_GPlus, 4787)
      call createCurrent(P_WPlus, P_WPlus, P_WMinus, P_WMinus, 1877)
      call createCurrent(P_ghZ, P_GPlus, P_GMinus, P_ghZAnti, 6631)
      call createCurrent(P_ZQ, P_Hh, P_Z, P_HhQ, 2720)
      call createCurrent(P_Hh, P_ghWm, P_HPlus, P_ghAAnti, 1479)
      call createCurrent(P_GMinus, P_ghZAnti, P_ghZ, P_GPlus, 6303)
      call createCurrent(P_WQMinus, P_GPlus, P_HhQ, 2611)
      call createCurrent(P_ghAAnti, P_Hl, P_ghWm, P_HPlus, 3671)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghZ, 7457)
      call createCurrent(P_G0, P_Z, P_Z, P_Ha, 9913)
      call createCurrent(P_HQMinus, P_HMinus, P_HPlus, P_HQPlus, 6902)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HlQ, 4225)
      call createCurrent(P_A, P_WPlus, P_GMinus, P_Hh, 2029)
      call createCurrent(P_Z, P_ghAAnti, P_WMinus, P_ghWp, 6624)
      call createCurrent(P_GMinus, P_Hh, P_GPlus, 8552)
      call createCurrent(P_GQ0, P_Ha, P_GMinus, P_HQPlus, 6353)
      call createCurrent(P_WPlus, P_ghWm, P_WPlus, P_ghWpAnti, 2322)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_Hl, P_ghZ, 560)
      call createCurrent(P_ghWpAnti, P_Ha, P_GPlus, P_ghZ, 8930)
      call createCurrent(P_ghZ, P_WMinus, P_ghZAnti, P_WQPlus, 8330)
      call createCurrent(P_ghWpAnti, P_Hh, P_HPlus, P_ghZ, 1384)
      call createCurrent(P_HPlus, P_ghA, P_ghWpAnti, P_Hl, 6432)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Z, P_AQ, 7834)
      call createCurrent(P_G0, P_ghZ, P_Ha, P_ghZAnti, 10263)
      call createCurrent(P_HPlus, P_ghWpAnti, P_GMinus, P_ghWp, 10395)
      call createCurrent(P_GQPlus, P_Hh, P_HMinus, P_HlQ, 4118)
      call createCurrent(P_ghWp, P_HPlus, P_ghWpAnti, P_HMinus, 3902)
      call createCurrent(P_GPlus, P_ghA, P_ghWpAnti, P_G0, 208)
      call createCurrent(P_HMinus, P_ghAAnti, P_Ha, P_ghWp, 5054)
      call createCurrent(P_ghWm, P_ghAAnti, P_HPlus, P_HhQ, 3288)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Z, P_WQPlus, 4326)
      call createCurrent(P_HQMinus, P_HPlus, P_Hh, P_HlQ, 272)
      call createCurrent(P_GMinus, P_HPlus, P_Hh, P_G0, 7464)
      call createCurrent(P_HQPlus, P_HMinus, P_ghWm, P_ghWmAnti, 6299)
      call createCurrent(P_HQMinus, P_Hl, P_HPlus, P_HaQ, 2088)
      call createCurrent(P_A, P_Hh, P_GMinus, P_WPlus, 2184)
      call createCurrent(P_GMinus, P_G0, P_GPlus, P_G0, 5975)
      call createCurrent(P_HQPlus, P_nu_eAnti, P_eMinus, 2192)
      call createCurrent(P_Z, P_ghWp, P_ghZAnti, P_WMinus, 8004)
      call createCurrent(P_ghWp, P_G0, P_ghZAnti, P_HMinus, 10326)
      call createCurrent(P_GMinus, P_HPlus, P_Hl, P_Ha, 5852)
      call createCurrent(P_GQ0, P_G0, P_Hh, P_HhQ, 1747)
      call createCurrent(P_HMinus, P_Ha, P_Ha, P_GPlus, 3499)
      call createCurrent(P_HhQ, P_eMinus, P_ePlus, 3814)
      call createCurrent(P_GQ0, P_GMinus, P_GQPlus, 7896)
      call createCurrent(P_Z, P_WMinus, P_WPlus, P_Z, 420)
      call createCurrent(P_ZQ, P_WPlus, P_Hh, P_GQMinus, 459)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghA, 538)
      call createCurrent(P_WPlus, P_HMinus, 10436)
      call createCurrent(P_HaQ, P_Z, P_HMinus, P_WQPlus, 4418)
      call createCurrent(P_HPlus, P_Ha, P_HMinus, P_G0, 5699)
      call createCurrent(P_HaQ, P_Ha, P_ghZAnti, P_ghZ, 6316)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghZ, P_GMinus, 1167)
      call createCurrent(P_GPlus, P_GMinus, P_Hl, P_Ha, 8843)
      call createCurrent(P_GPlus, P_ghWm, P_ghWmAnti, P_HMinus, 9492)
      call createCurrent(P_ghWp, P_ghWpAnti, P_GMinus, P_HPlus, 10431)
      call createCurrent(P_G0, P_ghWmAnti, P_ghA, P_GMinus, 2945)
      call createCurrent(P_ghA, P_GPlus, P_ghZAnti, P_GQMinus, 6391)
      call createCurrent(P_AQ, P_HMinus, P_HPlus, P_AQ, 2553)
      call createCurrent(P_ghA, P_GMinus, P_HPlus, P_ghAAnti, 10031)
      call createCurrent(P_WQMinus, P_Hl, P_WPlus, P_GQ0, 2154)
      call createCurrent(P_GPlus, P_HMinus, P_ghZ, P_ghAAnti, 10148)
      call createCurrent(P_ghWmAnti, P_ghZ, P_Ha, P_HQMinus, 1328)
      call createCurrent(P_GQPlus, P_Hh, P_GMinus, P_HaQ, 2126)
      call createCurrent(P_ghWm, P_A, P_ghWmAnti, P_AQ, 267)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, 5024)
      call createCurrent(P_ghWp, P_GPlus, P_HMinus, P_ghWpAnti, 9698)
      call createCurrent(P_ZQ, P_d, P_dAnti, 8384)
      call createCurrent(P_A, P_ghWm, P_ghWmAnti, P_A, 8143)
      call createCurrent(P_Z, P_ghWm, P_ghZAnti, P_WPlus, 6061)
      call createCurrent(P_HPlus, P_Hl, P_ghWm, P_ghAAnti, 5412)
      call createCurrent(P_HaQ, P_d, P_dAnti, 1351)
      call createCurrent(P_GMinus, P_nu_mu, P_muPlus, 6783)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, 8276)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hl, P_GPlus, 2207)
      call createCurrent(P_G0, P_G0, P_A, P_Z, 10293)
      call createCurrent(P_Hh, P_Z, P_Z, P_Hl, 8932)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, P_Hh, 4665)
      call createCurrent(P_ePlus, P_eMinus, 9959)
      call createCurrent(P_nu_tau, P_nu_tauAnti, P_Z, 7594)
      call createCurrent(P_HMinus, P_GPlus, P_G0, 3803)
      call createCurrent(P_ghWpAnti, P_HPlus, P_Ha, P_ghA, 8408)
      call createCurrent(P_GQ0, P_GMinus, P_HPlus, P_HaQ, 1013)
      call createCurrent(P_Ha, P_ghA, P_GPlus, P_ghWpAnti, 10030)
      call createCurrent(P_HlQ, P_HMinus, P_ghZAnti, P_ghWp, 5534)
      call createCurrent(P_A, P_Z, P_Hh, 10364)
      call createCurrent(P_HlQ, P_Hl, P_Hl, P_HhQ, 3576)
      call createCurrent(P_HQPlus, P_Hl, P_GMinus, P_GQ0, 179)
      call createCurrent(P_HaQ, P_GMinus, P_Hh, P_GQPlus, 5635)
      call createCurrent(P_HhQ, P_Z, P_GMinus, P_WQPlus, 5883)
      call createCurrent(P_GPlus, P_G0, P_HMinus, P_Ha, 7206)
      call createCurrent(P_HMinus, P_Hl, P_ghZAnti, P_ghWp, 7349)
      call createCurrent(P_Z, P_tauMinus, P_tauPlus, 4426)
      call createCurrent(P_G0, P_ghWmAnti, P_ghZ, P_HMinus, 9370)
      call createCurrent(P_WPlus, P_WMinus, P_ghZ, P_ghAAnti, 7887)
      call createCurrent(P_GQMinus, P_HMinus, P_GPlus, P_GQPlus, 8789)
      call createCurrent(P_ghZ, P_Hl, P_ghWmAnti, P_GQMinus, 4666)
      call createCurrent(P_GMinus, P_HMinus, P_HPlus, P_HPlus, 2182)
      call createCurrent(P_Ha, P_A, P_WPlus, P_HMinus, 474)
      call createCurrent(P_ghZAnti, P_ghA, P_GPlus, P_GMinus, 3691)
      call createCurrent(P_ghWpAnti, P_WPlus, P_ghA, P_Z, 8622)
      call createCurrent(P_HaQ, P_Hh, P_Ha, P_HlQ, 2116)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hl, P_HPlus, 1035)
      call createCurrent(P_GQPlus, P_GMinus, P_GPlus, P_GQMinus, 4377)
      call createCurrent(P_WQMinus, P_WPlus, P_ZQ, 7255)
      call createCurrent(P_nu_muAnti, P_muMinus, P_HPlus, 3397)
      call createCurrent(P_HlQ, P_A, P_WPlus, P_HQMinus, 4381)
      call createCurrent(P_GMinus, P_HPlus, P_Ha, P_Hl, 5648)
      call createCurrent(P_HQMinus, P_HPlus, P_ghWp, P_ghWpAnti, 3306)
      call createCurrent(P_WMinus, P_Hh, P_GPlus, P_A, 8049)
      call createCurrent(P_HQMinus, P_Z, P_Ha, P_WQPlus, 3276)
      call createCurrent(P_ghZ, P_ghWpAnti, P_WQPlus, 4245)
      call createCurrent(P_HPlus, P_Hl, P_Hh, P_HMinus, 4195)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Z, P_WMinus, 5442)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghZ, P_HMinus, 6036)
      call createCurrent(P_GPlus, P_HMinus, P_Hh, P_Hh, 6672)
      call createCurrent(P_ghZAnti, P_ghZ, P_WPlus, P_WMinus, 3106)
      call createCurrent(P_WMinus, P_HPlus, P_Hl, 5638)
      call createCurrent(P_A, P_WMinus, P_ghWmAnti, P_ghZ, 8712)
      call createCurrent(P_ghWpAnti, P_GPlus, P_G0, P_ghA, 2626)
      call createCurrent(P_GQ0, P_ghZ, P_GPlus, P_ghWpAnti, 5070)
      call createCurrent(P_A, P_ghA, P_WMinus, P_ghWmAnti, 760)
      call createCurrent(P_bAnti, P_b, P_Hl, 797)
      call createCurrent(P_GQPlus, P_G0, P_G0, P_GQMinus, 6813)
      call createCurrent(P_Hl, P_Hh, P_ghWmAnti, P_ghWm, 9286)
      call createCurrent(P_ghAAnti, P_WPlus, P_ghWm, 6254)
      call createCurrent(P_ghZAnti, P_Ha, P_ghWp, P_GMinus, 10276)
      call createCurrent(P_Ha, P_A, P_Hl, 10022)
      call createCurrent(P_GPlus, P_HMinus, P_ghWpAnti, P_ghWp, 9910)
      call createCurrent(P_HPlus, P_GPlus, P_HMinus, P_HMinus, 1701)
      call createCurrent(P_HMinus, P_GPlus, P_GMinus, P_HPlus, 3056)
      call createCurrent(P_ghWp, P_HMinus, P_ghWpAnti, P_GPlus, 9751)
      call createCurrent(P_muMinus, P_nu_muAnti, P_HPlus, 6911)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghA, P_HPlus, 5190)
      call createCurrent(P_ghZ, P_ghWpAnti, P_GPlus, P_G0, 3461)
      call createCurrent(P_s, P_G0, P_sAnti, 2914)
      call createCurrent(P_Hl, P_A, P_Hl, P_A, 10621)
      call createCurrent(P_Z, P_A, P_ghWmAnti, P_ghWm, 2711)
      call createCurrent(P_Ha, P_GPlus, P_WMinus, 10216)
      call createCurrent(P_HMinus, P_Ha, P_Z, P_WPlus, 1765)
      call createCurrent(P_Z, P_Z, P_Hh, P_Hh, 8799)
      call createCurrent(P_A, P_GPlus, P_Ha, P_WMinus, 9313)
      call createCurrent(P_GPlus, P_WMinus, P_GMinus, P_WPlus, 1959)
      call createCurrent(P_sAnti, P_c, P_GQMinus, 478)
      call createCurrent(P_HQMinus, P_G0, P_Hh, P_GQPlus, 3100)
      call createCurrent(P_nu_tauAnti, P_tauMinus, P_WPlus, 3590)
      call createCurrent(P_nu_e, P_nu_eAnti, P_A, 9483)
      call createCurrent(P_ghAAnti, P_GPlus, P_ghZ, P_HMinus, 9510)
      call createCurrent(P_Hl, P_GMinus, P_ghWp, P_ghZAnti, 2852)
      call createCurrent(P_d, P_A, P_dAnti, 4481)
      call createCurrent(P_HPlus, P_WMinus, P_WPlus, P_HMinus, 1518)
      call createCurrent(P_Hl, P_ghWpAnti, P_Hl, P_ghWp, 3709)
      call createCurrent(P_HlQ, P_ghWpAnti, P_HPlus, P_ghZ, 2548)
      call createCurrent(P_HMinus, P_ghA, P_HPlus, P_ghAAnti, 7285)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghZ, P_HQPlus, 8394)
      call createCurrent(P_HQPlus, P_WMinus, P_A, P_HlQ, 5238)
      call createCurrent(P_GQPlus, P_Z, P_WQMinus, 122)
      call createCurrent(P_GQ0, P_HPlus, P_G0, P_HQMinus, 2660)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_G0, 9069)
      call createCurrent(P_ghAAnti, P_Hh, P_ghWm, P_HPlus, 829)
      call createCurrent(P_ghWm, P_WPlus, P_WPlus, P_ghWpAnti, 1054)
      call createCurrent(P_Hh, P_ghWm, P_HPlus, P_ghZAnti, 3107)
      call createCurrent(P_ZQ, P_Hh, P_WPlus, P_GQMinus, 8544)
      call createCurrent(P_GQPlus, P_GMinus, P_ghZ, P_ghZAnti, 3313)
      call createCurrent(P_Hh, P_ghWpAnti, P_GPlus, P_ghZ, 3753)
      call createCurrent(P_Hh, P_HMinus, P_Hl, P_HPlus, 5708)
      call createCurrent(P_G0, P_HMinus, P_GPlus, P_G0, 2045)
      call createCurrent(P_ghWp, P_WMinus, P_ghAAnti, 7913)
      call createCurrent(P_ghZ, P_Hh, P_ghZAnti, 894)
      call createCurrent(P_GQ0, P_G0, P_G0, P_HaQ, 5021)
      call createCurrent(P_A, P_WPlus, P_WMinus, P_Z, 1418)
      call createCurrent(P_Hh, P_ghAAnti, P_ghWm, P_HPlus, 487)
      call createCurrent(P_A, P_WMinus, P_Hh, P_GPlus, 1953)
      call createCurrent(P_A, P_WPlus, P_GMinus, 1878)
      call createCurrent(P_Hh, P_GMinus, P_ghAAnti, P_ghWp, 4421)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_Hh, 8348)
      call createCurrent(P_HQPlus, P_Hl, P_Hh, P_HQMinus, 547)
      call createCurrent(P_HQPlus, P_HMinus, P_GPlus, P_GQMinus, 1420)
      call createCurrent(P_A, P_A, P_ghWp, P_ghWpAnti, 2271)
      call createCurrent(P_HPlus, P_G0, P_ghA, P_ghWpAnti, 9781)
      call createCurrent(P_WMinus, P_A, P_Hl, P_GPlus, 7443)
      call createCurrent(P_Ha, P_ghZAnti, P_HMinus, P_ghWp, 2826)
      call createCurrent(P_GQPlus, P_A, P_GMinus, P_AQ, 342)
      call createCurrent(P_sAnti, P_c, P_HQMinus, 1764)
      call createCurrent(P_HMinus, P_GPlus, P_Ha, P_Hh, 5701)
      call createCurrent(P_ghA, P_Ha, P_ghWpAnti, P_HQPlus, 1995)
      call createCurrent(P_HMinus, P_ghWp, P_ghZAnti, P_Ha, 1608)
      call createCurrent(P_HPlus, P_ghA, P_Hh, P_ghWpAnti, 4645)
      call createCurrent(P_HMinus, P_ghWp, P_HPlus, P_ghWpAnti, 8492)
      call createCurrent(P_ghA, P_HPlus, P_ghWpAnti, P_Hl, 4904)
      call createCurrent(P_HMinus, P_G0, P_ghA, P_ghWmAnti, 9716)
      call createCurrent(P_Hl, P_HMinus, P_Z, P_WPlus, 364)
      call createCurrent(P_Z, P_ghWpAnti, P_ghWp, P_A, 8807)
      call createCurrent(P_ghWp, P_Hh, P_ghZAnti, P_GQMinus, 7149)
      call createCurrent(P_HlQ, P_GMinus, P_WQPlus, 3925)
      call createCurrent(P_Hh, P_ghAAnti, P_HMinus, P_ghWp, 3835)
      call createCurrent(P_A, P_WMinus, P_HPlus, P_Hh, 5765)
      call createCurrent(P_G0, P_GPlus, P_Z, P_WMinus, 5653)
      call createCurrent(P_A, P_GPlus, P_WMinus, P_Hh, 5590)
      call createCurrent(P_HPlus, P_ghAAnti, P_GMinus, P_ghZ, 9173)
      call createCurrent(P_A, P_ghWp, P_A, P_ghWpAnti, 2767)
      call createCurrent(P_GMinus, P_GPlus, P_ghA, P_ghZAnti, 8691)
      call createCurrent(P_HQPlus, P_GMinus, P_G0, P_GQ0, 496)
      call createCurrent(P_Ha, P_Ha, P_Hh, P_Hh, 4188)
      call createCurrent(P_HMinus, P_Hh, P_Hl, P_GPlus, 3392)
      call createCurrent(P_WMinus, P_ghWm, P_WPlus, P_ghWmAnti, 2255)
      call createCurrent(P_GPlus, P_ghAAnti, P_HMinus, P_ghA, 10281)
      call createCurrent(P_ghZAnti, P_Ha, P_ghZ, P_Ha, 2630)
      call createCurrent(P_Z, P_Hh, P_G0, 6056)
      call createCurrent(P_ghWm, P_ghWmAnti, P_GMinus, P_GQPlus, 7323)
      call createCurrent(P_Hh, P_ghZ, P_GMinus, P_ghWmAnti, 7983)
      call createCurrent(P_AQ, P_Hl, P_HMinus, P_WQPlus, 5945)
      call createCurrent(P_g, P_G0, P_Ha, P_g, 10608)
      call createCurrent(P_HaQ, P_ghWm, P_ghWmAnti, 1784)
      call createCurrent(P_HPlus, P_G0, P_G0, P_GMinus, 3018)
      call createCurrent(P_HQMinus, P_Hh, P_ghZAnti, P_ghWp, 3719)
      call createCurrent(P_HQPlus, P_ghZAnti, P_HMinus, P_ghA, 3996)
      call createCurrent(P_WPlus, P_HMinus, P_A, P_Hh, 4239)
      call createCurrent(P_Hl, P_Hl, 8970)
      call createCurrent(P_Hl, P_ghWpAnti, P_HPlus, P_ghZ, 7546)
      call createCurrent(P_WQPlus, P_ghWm, P_ghAAnti, 6251)
      call createCurrent(P_g, P_g, P_Hh, 10571)
      call createCurrent(P_GQMinus, P_ghAAnti, P_GPlus, P_ghA, 1207)
      call createCurrent(P_GPlus, P_ghZ, P_ghWpAnti, P_Ha, 8972)
      call createCurrent(P_Z, P_GMinus, P_Hl, P_WPlus, 7714)
      call createCurrent(P_ghA, P_GMinus, P_ghWmAnti, P_Hl, 6280)
      call createCurrent(P_GPlus, P_ghWm, P_ghZAnti, P_Hh, 5855)
      call createCurrent(P_G0, P_ghZAnti, P_ghZ, P_G0, 7030)
      call createCurrent(P_Z, P_HPlus, P_G0, P_WMinus, 8866)
      call createCurrent(P_HaQ, P_Ha, P_WMinus, P_WQPlus, 1057)
      call createCurrent(P_WMinus, P_Ha, P_GPlus, 10378)
      call createCurrent(P_WMinus, P_Ha, P_Z, P_HPlus, 4622)
      call createCurrent(P_GQPlus, P_G0, P_Ha, P_HQMinus, 7134)
      call createCurrent(P_HhQ, P_A, P_WMinus, P_GQPlus, 8515)
      call createCurrent(P_ghZ, P_HMinus, P_ghWmAnti, 4309)
      call createCurrent(P_HQPlus, P_GMinus, P_GMinus, P_HQPlus, 7866)
      call createCurrent(P_HPlus, P_Ha, P_G0, P_GMinus, 5481)
      call createCurrent(P_HhQ, P_HPlus, P_ghWpAnti, P_ghZ, 8526)
      call createCurrent(P_Hh, P_Z, P_Hh, P_A, 9119)
      call createCurrent(P_GMinus, P_G0, P_Hh, P_GPlus, 10252)
      call createCurrent(P_ghZAnti, P_G0, P_HMinus, P_ghWp, 9621)
      call createCurrent(P_Hh, P_GPlus, P_Hl, P_HMinus, 8038)
      call createCurrent(P_Hh, P_HPlus, P_ghWpAnti, P_ghZ, 993)
      call createCurrent(P_Hl, P_WPlus, P_HMinus, P_Z, 6886)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghZ, P_WMinus, 7936)
      call createCurrent(P_ZQ, P_GMinus, P_Z, P_GQPlus, 6390)
      call createCurrent(P_ghWmAnti, P_ghA, P_HMinus, P_Hl, 4575)
      call createCurrent(P_Z, P_WPlus, P_GMinus, P_Hl, 2489)
      call createCurrent(P_ZQ, P_GPlus, P_Hl, P_WQMinus, 5947)
      call createCurrent(P_g, P_d, P_dAnti, 2796)
      call createCurrent(P_ghAAnti, P_A, P_ghWm, P_WPlus, 2724)
      call createCurrent(P_HQMinus, P_WPlus, P_Hl, P_ZQ, 7641)
      call createCurrent(P_HhQ, P_WMinus, P_GPlus, P_ZQ, 6436)
      call createCurrent(P_WQMinus, P_Z, P_ghA, P_ghWmAnti, 1555)
      call createCurrent(P_HPlus, P_G0, P_Hh, P_HMinus, 10272)
      call createCurrent(P_A, P_cAnti, P_c, 6496)
      call createCurrent(P_Ha, P_ghZAnti, P_ghWm, P_GPlus, 9170)
      call createCurrent(P_u, P_dAnti, P_GQMinus, 1177)
      call createCurrent(P_WPlus, P_ghA, P_A, P_ghWpAnti, 7466)
      call createCurrent(P_Ha, P_Hl, P_Ha, P_Hl, 8648)
      call createCurrent(P_ghGAnti, P_ghG, P_g, P_g, 5772)
      call createCurrent(P_HhQ, P_Hh, P_ghZAnti, P_ghZ, 4229)
      call createCurrent(P_g, P_tAnti, P_t, 443)
      call createCurrent(P_GQPlus, P_GMinus, P_ghA, P_ghZAnti, 6766)
      call createCurrent(P_GPlus, P_g, P_g, P_HMinus, 10612)
      call createCurrent(P_WPlus, P_Hl, P_HMinus, P_Z, 5768)
      call createCurrent(P_g, P_HMinus, P_HPlus, P_g, 10519)
      call createCurrent(P_Hh, P_A, P_Hh, P_A, 10708)
      call createCurrent(P_Hh, P_GMinus, P_GPlus, P_Hh, 1058)
      call createCurrent(P_HhQ, P_HMinus, P_ghWmAnti, P_ghZ, 7357)
      call createCurrent(P_ghZ, P_ghZAnti, P_Ha, P_HaQ, 6641)
      call createCurrent(P_WQMinus, P_ghWmAnti, P_A, P_ghZ, 4717)
      call createCurrent(P_HQPlus, P_b, P_tAnti, 3270)
      call createCurrent(P_Hh, P_WPlus, P_A, P_GMinus, 452)
      call createCurrent(P_HaQ, P_Hl, P_ghWmAnti, P_ghWm, 7498)
      call createCurrent(P_HaQ, P_G0, P_Hl, P_HhQ, 4736)
      call createCurrent(P_HQPlus, P_HMinus, P_ZQ, 531)
      call createCurrent(P_Hh, P_WPlus, P_Hh, P_WMinus, 1444)
      call createCurrent(P_HQMinus, P_Hl, P_Ha, P_GQPlus, 5518)
      call createCurrent(P_WQMinus, P_WPlus, P_Hl, P_HaQ, 2311)
      call createCurrent(P_ghA, P_ghZAnti, P_HPlus, P_HQMinus, 6735)
      call createCurrent(P_GQMinus, P_HMinus, P_HPlus, P_HQPlus, 2604)
      call createCurrent(P_GQ0, P_HPlus, P_GQMinus, 712)
      call createCurrent(P_AQ, P_Z, P_ghWm, P_ghWmAnti, 7093)
      call createCurrent(P_ghWmAnti, P_ghA, P_GMinus, 853)
      call createCurrent(P_G0, P_G0, P_HPlus, P_HMinus, 4420)
      call createCurrent(P_WPlus, P_WMinus, P_ghA, P_ghAAnti, 4558)
      call createCurrent(P_HhQ, P_Ha, P_G0, P_HlQ, 3273)
      call createCurrent(P_WPlus, P_Hl, P_GMinus, P_Z, 6071)
      call createCurrent(P_WPlus, P_ghWpAnti, P_A, P_ghZ, 495)
      call createCurrent(P_HMinus, P_Hl, P_Ha, P_GPlus, 8310)
      call createCurrent(P_HQPlus, P_HMinus, P_ghAAnti, P_ghA, 1712)
      call createCurrent(P_ghWp, P_ghAAnti, P_GMinus, P_GQ0, 4792)
      call createCurrent(P_A, P_HMinus, P_A, P_GPlus, 9080)
      call createCurrent(P_WPlus, P_A, P_A, P_WMinus, 5009)
      call createCurrent(P_WMinus, P_HPlus, P_Hl, P_A, 923)
      call createCurrent(P_ghAAnti, P_ghZ, P_HMinus, P_GPlus, 9041)
      call createCurrent(P_Z, P_ghWmAnti, P_ghWm, P_Z, 1672)
      call createCurrent(P_Ha, P_Z, P_Ha, P_A, 10476)
      call createCurrent(P_WMinus, P_Ha, P_HPlus, P_A, 3969)
      call createCurrent(P_HhQ, P_A, P_HMinus, P_WQPlus, 5834)
      call createCurrent(P_muPlus, P_muMinus, 9192)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_GQ0, 7171)
      call createCurrent(P_Ha, P_G0, P_GMinus, P_GPlus, 8043)
      call createCurrent(P_HQMinus, P_GPlus, P_Hl, P_HaQ, 7166)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hh, P_HQPlus, 4861)
      call createCurrent(P_WPlus, P_WMinus, P_A, P_A, 6862)
      call createCurrent(P_A, P_HPlus, P_G0, P_WMinus, 9563)
      call createCurrent(P_Hh, P_WMinus, P_GPlus, P_A, 344)
      call createCurrent(P_HaQ, P_HMinus, P_HPlus, P_GQ0, 7518)
      call createCurrent(P_Z, P_A, P_Hh, P_Hl, 10546)
      call createCurrent(P_ZQ, P_GMinus, P_Hl, P_WQPlus, 1125)
      call createCurrent(P_WQMinus, P_A, P_ghWmAnti, P_ghZ, 6903)
      call createCurrent(P_uAnti, P_d, P_GQPlus, 1227)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HPlus, P_GMinus, 9737)
      call createCurrent(P_ghZ, P_WMinus, P_WPlus, P_ghAAnti, 966)
      call createCurrent(P_HPlus, P_HMinus, P_ghWp, P_ghWpAnti, 4940)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghWm, P_HlQ, 3501)
      call createCurrent(P_GPlus, P_Hl, P_GMinus, P_G0, 8846)
      call createCurrent(P_GPlus, P_ghWm, P_HMinus, P_ghWmAnti, 9026)
      call createCurrent(P_WQPlus, P_WMinus, P_Ha, P_HhQ, 4073)
      call createCurrent(P_Ha, P_ghZAnti, P_ghWm, P_HPlus, 42)
      call createCurrent(P_ghWp, P_ghZAnti, P_Hl, P_HQMinus, 1798)
      call createCurrent(P_HMinus, P_G0, P_HPlus, 9782)
      call createCurrent(P_muMinus, P_Hl, P_muPlus, 6113)
      call createCurrent(P_Z, P_GMinus, P_G0, P_WPlus, 3035)
      call createCurrent(P_HQMinus, P_Hh, P_Hh, P_GQPlus, 6290)
      call createCurrent(P_ghA, P_ghWpAnti, P_G0, P_HPlus, 10246)
      call createCurrent(P_HhQ, P_Ha, P_GPlus, P_GQMinus, 6810)
      call createCurrent(P_ghZ, P_Hh, P_ghWpAnti, P_GPlus, 7694)
      call createCurrent(P_ghZAnti, P_ghWp, P_WMinus, P_A, 801)
      call createCurrent(P_ghWp, P_Z, P_ghWpAnti, P_AQ, 4942)
      call createCurrent(P_Hh, P_Hh, P_HMinus, P_HPlus, 5353)
      call createCurrent(P_ghAAnti, P_HPlus, P_ghZ, P_GMinus, 9672)
      call createCurrent(P_G0, P_ghZAnti, P_ghWp, P_HMinus, 9526)
      call createCurrent(P_ghA, P_Hl, P_ghWpAnti, P_HQPlus, 4199)
      call createCurrent(P_A, P_HPlus, P_HMinus, P_A, 4487)
      call createCurrent(P_ghZ, P_Hl, P_ghWpAnti, P_GQPlus, 778)
      call createCurrent(P_WQPlus, P_Z, P_A, P_WQMinus, 2348)
      call createCurrent(P_WQPlus, P_Z, P_ghZ, P_ghWpAnti, 6706)
      call createCurrent(P_WMinus, P_HMinus, P_GPlus, P_WPlus, 10277)
      call createCurrent(P_AQ, P_Hl, P_GMinus, P_WQPlus, 6175)
      call createCurrent(P_Hl, P_ghWm, P_HPlus, P_ghZAnti, 4509)
      call createCurrent(P_HPlus, P_GMinus, P_GMinus, P_HPlus, 3468)
      call createCurrent(P_HQPlus, P_Hh, P_WMinus, P_AQ, 6021)
      call createCurrent(P_ghZ, P_HPlus, P_ghZAnti, P_GMinus, 9588)
      call createCurrent(P_G0, P_ghWm, P_ghAAnti, P_HPlus, 10043)
      call createCurrent(P_WQMinus, P_WPlus, P_GQ0, 7981)
      call createCurrent(P_ghA, P_ghAAnti, P_GPlus, P_HMinus, 9049)
      call createCurrent(P_sAnti, P_s, P_A, 35)
      call createCurrent(P_ghZ, P_Hl, P_GPlus, P_ghWpAnti, 7079)
      call createCurrent(P_WQMinus, P_WPlus, P_ghZ, P_ghZAnti, 7956)
      call createCurrent(P_GQMinus, P_Z, P_Hh, P_WQPlus, 3802)
      call createCurrent(P_WQPlus, P_A, P_ghWpAnti, P_ghA, 7157)
      call createCurrent(P_HlQ, P_A, P_GPlus, P_WQMinus, 4944)
      call createCurrent(P_HQPlus, P_Ha, P_ghZAnti, P_ghWm, 7752)
      call createCurrent(P_ghZAnti, P_ghWm, P_HPlus, P_Hh, 5495)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hh, P_HPlus, 1650)
      call createCurrent(P_Z, P_WMinus, P_ghZ, P_ghWmAnti, 3857)
      call createCurrent(P_HQPlus, P_Z, P_Hh, P_WQMinus, 52)
      call createCurrent(P_GMinus, P_G0, P_HPlus, P_Ha, 6375)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GQMinus, 5822)
      call createCurrent(P_A, P_HPlus, P_Hh, P_WMinus, 2960)
      call createCurrent(P_HhQ, P_HMinus, P_HPlus, P_HhQ, 4313)
      call createCurrent(P_GMinus, P_HPlus, P_HMinus, P_HPlus, 8490)
      call createCurrent(P_HlQ, P_ghWp, P_G0, P_ghWpAnti, 2940)
      call createCurrent(P_WQPlus, P_Hl, P_GQMinus, 997)
      call createCurrent(P_GMinus, P_ePlus, P_nu_e, 8512)
      call createCurrent(P_G0, P_A, P_GPlus, P_WMinus, 4424)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghZ, P_HPlus, 9315)
      call createCurrent(P_Ha, P_HMinus, P_GPlus, P_Hl, 8736)
      call createCurrent(P_GPlus, P_G0, P_GMinus, 10320)
      call createCurrent(P_HlQ, P_Hh, P_Hh, P_HhQ, 8569)
      call createCurrent(P_ghWp, P_ghWpAnti, P_HMinus, P_GPlus, 9678)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Ha, P_HhQ, 1228)
      call createCurrent(P_GPlus, P_ghWm, P_Hh, P_ghAAnti, 7482)
      call createCurrent(P_ZQ, P_GMinus, P_WPlus, P_GQ0, 10)
      call createCurrent(P_HPlus, P_Hh, P_ghWm, P_ghAAnti, 7230)
      call createCurrent(P_GQMinus, P_Hh, P_HPlus, P_GQ0, 639)
      call createCurrent(P_WQPlus, P_HMinus, P_ZQ, 1182)
      call createCurrent(P_HQPlus, P_GMinus, P_HlQ, 7906)
      call createCurrent(P_WQPlus, P_WMinus, P_Hh, P_GQ0, 6629)
      call createCurrent(P_cAnti, P_Ha, P_c, 6809)
      call createCurrent(P_HQPlus, P_Z, P_Ha, P_WQMinus, 6192)
      call createCurrent(P_ghWm, P_G0, P_ghZAnti, P_GPlus, 3299)
      call createCurrent(P_t, P_tAnti, P_HhQ, 3726)
      call createCurrent(P_HPlus, P_ghWm, P_G0, P_ghZAnti, 8977)
      call createCurrent(P_ghZAnti, P_ghWp, P_Hh, P_GMinus, 3311)
      call createCurrent(P_Z, P_Ha, 9205)
      call createCurrent(P_HPlus, P_HMinus, P_WPlus, P_WMinus, 7141)
      call createCurrent(P_HPlus, P_WMinus, 9931)
      call createCurrent(P_GQ0, P_uAnti, P_u, 3904)
      call createCurrent(P_GQPlus, P_ghZAnti, P_GMinus, P_ghZ, 5170)
      call createCurrent(P_ghWpAnti, P_G0, P_Ha, P_ghWp, 9239)
      call createCurrent(P_HlQ, P_GPlus, P_ghZAnti, P_ghWm, 1542)
      call createCurrent(P_Ha, P_ghA, P_ghWmAnti, P_GMinus, 10171)
      call createCurrent(P_ghWm, P_G0, P_ghZAnti, P_GQPlus, 6999)
      call createCurrent(P_ghA, P_ghWpAnti, P_Z, P_WQPlus, 2171)
      call createCurrent(P_ZQ, P_Hh, P_GQ0, 6958)
      call createCurrent(P_HQPlus, P_WPlus, P_HMinus, P_WQMinus, 6128)
      call createCurrent(P_HaQ, P_Hl, P_WPlus, P_WQMinus, 7501)
      call createCurrent(P_GMinus, P_Hl, P_HPlus, P_Ha, 817)
      call createCurrent(P_ghWm, P_ghWmAnti, P_HaQ, 1206)
      call createCurrent(P_cAnti, P_c, P_Z, 6441)
      call createCurrent(P_HlQ, P_WMinus, P_Z, P_HQPlus, 3860)
      call createCurrent(P_WPlus, P_ghWpAnti, P_Z, P_ghA, 1727)
      call createCurrent(P_Z, P_Ha, P_G0, P_A, 10739)
      call createCurrent(P_dAnti, P_Ha, P_d, 3822)
      call createCurrent(P_HMinus, P_tauPlus, P_nu_tau, 2849)
      call createCurrent(P_GQ0, P_t, P_tAnti, 1174)
      call createCurrent(P_HQMinus, P_WPlus, P_WMinus, P_HQPlus, 7015)
      call createCurrent(P_GPlus, P_GMinus, P_HPlus, P_HMinus, 4031)
      call createCurrent(P_Ha, P_ghWpAnti, P_GPlus, P_ghZ, 10383)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_G0, 5577)
      call createCurrent(P_ghWp, P_ghWpAnti, P_WPlus, P_WQMinus, 4429)
      call createCurrent(P_HQPlus, P_Ha, P_WQMinus, 5372)
      call createCurrent(P_HaQ, P_Ha, P_G0, P_HaQ, 8587)
      call createCurrent(P_ghZ, P_G0, P_HMinus, P_ghWmAnti, 10459)
      call createCurrent(P_WPlus, P_A, P_HMinus, 5072)
      call createCurrent(P_ghAAnti, P_GPlus, P_Hl, P_ghWm, 7366)
      call createCurrent(P_ghWp, P_GMinus, P_Ha, P_ghZAnti, 9383)
      call createCurrent(P_Hl, P_ghA, P_ghWpAnti, P_HPlus, 4404)
      call createCurrent(P_u, P_Hl, P_uAnti, 2405)
      call createCurrent(P_HlQ, P_ghZ, P_HPlus, P_ghWpAnti, 866)
      call createCurrent(P_Hh, P_HPlus, P_Ha, P_GMinus, 3356)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghZ, P_WPlus, 6444)
      call createCurrent(P_b, P_bAnti, P_A, 5753)
      call createCurrent(P_ghWm, P_ghWmAnti, P_AQ, 7937)
      call createCurrent(P_HQPlus, P_uAnti, P_d, 8540)
      call createCurrent(P_A, P_ghWpAnti, P_ghWp, P_A, 175)
      call createCurrent(P_GMinus, P_GPlus, P_Ha, P_Hh, 9805)
      call createCurrent(P_Ha, P_G0, P_HPlus, P_GMinus, 3333)
      call createCurrent(P_ZQ, P_HPlus, P_WMinus, P_HlQ, 7456)
      call createCurrent(P_GQPlus, P_G0, P_WQMinus, 6972)
      call createCurrent(P_GQPlus, P_Ha, P_GMinus, P_HlQ, 3731)
      call createCurrent(P_nu_e, P_A, P_nu_eAnti, 10140)
      call createCurrent(P_GPlus, P_HMinus, P_ghWp, P_ghWpAnti, 9638)
      call createCurrent(P_ghA, P_ghAAnti, P_HMinus, P_HQPlus, 5369)
      call createCurrent(P_WPlus, P_Z, P_WMinus, P_Z, 815)
      call createCurrent(P_WQMinus, P_WPlus, P_ghAAnti, P_ghZ, 2655)
      call createCurrent(P_G0, P_HMinus, P_ghZ, P_ghWmAnti, 9509)
      call createCurrent(P_A, P_G0, 10195)
      call createCurrent(P_Ha, P_tauMinus, P_tauPlus, 1506)
      call createCurrent(P_A, P_A, P_Hh, P_Hl, 10501)
      call createCurrent(P_HQPlus, P_Ha, P_G0, P_GQMinus, 5343)
      call createCurrent(P_GMinus, P_HPlus, P_Z, P_Z, 9331)
      call createCurrent(P_muMinus, P_Z, P_muPlus, 1)
      call createCurrent(P_HPlus, P_GMinus, P_Hl, P_G0, 4641)
      call createCurrent(P_Ha, P_HMinus, P_ghZAnti, P_ghWp, 170)
      call createCurrent(P_ghWp, P_ghWpAnti, P_ZQ, 3030)
      call createCurrent(P_GQMinus, P_WPlus, P_Z, P_HlQ, 7661)
      call createCurrent(P_HaQ, P_G0, P_HhQ, 6333)
      call createCurrent(P_ZQ, P_WMinus, P_GPlus, P_HhQ, 503)
      call createCurrent(P_GPlus, P_WMinus, P_HMinus, P_WPlus, 9142)
      call createCurrent(P_ghWpAnti, P_ghZ, P_Hl, P_HPlus, 6435)
      call createCurrent(P_GMinus, P_ghA, P_ghWmAnti, P_Hh, 2529)
      call createCurrent(P_WMinus, P_ghWmAnti, P_ghZ, 8483)
      call createCurrent(P_HQPlus, P_HMinus, P_G0, P_HhQ, 5979)
      call createCurrent(P_WMinus, P_A, P_ghAAnti, P_ghWp, 6117)
      call createCurrent(P_HQPlus, P_ghWm, P_ghZAnti, 4794)
      call createCurrent(P_HlQ, P_u, P_uAnti, 3143)
      call createCurrent(P_Z, P_ghWpAnti, P_A, P_ghWp, 925)
      call createCurrent(P_A, P_A, P_ghWpAnti, P_ghWp, 4455)
      call createCurrent(P_ghWm, P_ghWmAnti, P_GPlus, P_GMinus, 3938)
      call createCurrent(P_g, P_g, P_g, P_g, 5749)
      call createCurrent(P_ghAAnti, P_ghZ, P_WPlus, P_WMinus, 2164)
      call createCurrent(P_Z, P_Z, P_A, P_Z, 10711)
      call createCurrent(P_A, P_WPlus, P_WMinus, P_A, 5771)
      call createCurrent(P_ghWpAnti, P_Z, P_ghA, P_WQPlus, 8012)
      call createCurrent(P_GMinus, P_Hl, P_ghWmAnti, P_ghZ, 2926)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hl, P_Hh, 10071)
      call createCurrent(P_Ha, P_GMinus, P_ghWmAnti, P_ghA, 9281)
      call createCurrent(P_WPlus, P_ghWm, P_ghWpAnti, P_WPlus, 2131)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_G0, P_ghA, 7198)
      call createCurrent(P_GQ0, P_G0, P_WPlus, P_WQMinus, 3838)
      call createCurrent(P_GMinus, P_ghWp, P_ghAAnti, P_Hh, 8301)
      call createCurrent(P_ePlus, P_eMinus, P_GQ0, 5676)
      call createCurrent(P_HPlus, P_Hl, P_ghA, P_ghWpAnti, 4543)
      call createCurrent(P_bAnti, P_t, P_HQMinus, 2471)
      call createCurrent(P_GPlus, P_HMinus, P_A, P_A, 9961)
      call createCurrent(P_Ha, P_ghWp, P_GMinus, P_ghZAnti, 9361)
      call createCurrent(P_ghWp, P_ghAAnti, P_HMinus, P_HhQ, 7146)
      call createCurrent(P_HQMinus, P_WMinus, P_WPlus, P_HQPlus, 744)
      call createCurrent(P_HhQ, P_muPlus, P_muMinus, 30)
      call createCurrent(P_b, P_bAnti, P_Hl, 3695)
      call createCurrent(P_GQMinus, P_WPlus, P_Hl, P_AQ, 2613)
      call createCurrent(P_GMinus, P_Hl, P_GPlus, P_G0, 10467)
      call createCurrent(P_ghWp, P_GMinus, P_Hl, P_ghAAnti, 2875)
      call createCurrent(P_GQ0, P_WMinus, P_Hl, P_WQPlus, 5234)
      call createCurrent(P_gQ, P_s, P_sAnti, 3785)
      call createCurrent(P_GQPlus, P_GMinus, P_ZQ, 1808)
      call createCurrent(P_Hh, P_GMinus, P_WPlus, P_A, 4933)
      call createCurrent(P_HPlus, P_ghWm, P_Hh, P_ghZAnti, 6528)
      call createCurrent(P_Z, P_WPlus, P_GMinus, P_G0, 3465)
      call createCurrent(P_G0, P_G0, P_ghZ, P_ghZAnti, 1239)
      call createCurrent(P_ghAAnti, P_Ha, P_ghWp, P_GMinus, 10430)
      call createCurrent(P_ghWp, P_WMinus, P_A, P_ghAAnti, 5228)
      call createCurrent(P_Hh, P_HPlus, P_GMinus, P_G0, 5323)
      call createCurrent(P_G0, P_ghZAnti, P_GMinus, P_ghWp, 6348)
      call createCurrent(P_WPlus, P_GMinus, P_GPlus, P_WMinus, 7099)
      call createCurrent(P_ghAAnti, P_ghA, P_WMinus, P_WQPlus, 3683)
      call createCurrent(P_HlQ, P_GMinus, P_Hl, P_HQPlus, 3408)
      call createCurrent(P_ghWp, P_ghZAnti, P_WMinus, P_ZQ, 2632)
      call createCurrent(P_WQPlus, P_nu_muAnti, P_muMinus, 8396)
      call createCurrent(P_GPlus, P_HMinus, P_ghWm, P_ghWmAnti, 9676)
      call createCurrent(P_ghZAnti, P_Ha, P_HMinus, P_ghWp, 2279)
      call createCurrent(P_WQPlus, P_Z, P_WMinus, P_AQ, 825)
      call createCurrent(P_HhQ, P_HMinus, P_Hh, P_GQPlus, 6688)
      call createCurrent(P_ghA, P_A, P_ghWpAnti, P_WPlus, 1359)
      call createCurrent(P_WPlus, P_HMinus, P_Z, P_G0, 10214)
      call createCurrent(P_HQPlus, P_G0, P_GMinus, P_GQ0, 1711)
      call createCurrent(P_ghWm, P_ghWmAnti, P_WPlus, P_WMinus, 7637)
      call createCurrent(P_G0, P_ghZAnti, P_ghWm, P_GPlus, 695)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_Ha, 6501)
      call createCurrent(P_Ha, P_Hl, P_G0, P_Hl, 3790)
      call createCurrent(P_WMinus, P_WPlus, P_Z, P_Z, 4647)
      call createCurrent(P_ePlus, P_nu_e, P_HQMinus, 7626)
      call createCurrent(P_HaQ, P_Ha, P_ghWm, P_ghWmAnti, 112)
      call createCurrent(P_WQPlus, P_Z, P_HMinus, P_HlQ, 3607)
      call createCurrent(P_ghZ, P_ghAAnti, P_WPlus, P_WMinus, 3226)
      call createCurrent(P_Hh, P_HMinus, P_Ha, P_HPlus, 8905)
      call createCurrent(P_WQMinus, P_A, P_Z, P_WQPlus, 3739)
      call createCurrent(P_ghWmAnti, P_Hl, P_ghWm, 5460)
      call createCurrent(P_Ha, P_Z, P_GMinus, P_WPlus, 10478)
      call createCurrent(P_GMinus, P_HPlus, P_G0, P_Ha, 8242)
      call createCurrent(P_Z, P_ghAAnti, P_WPlus, P_ghWm, 8057)
      call createCurrent(P_WQMinus, P_Z, P_HPlus, P_HlQ, 1920)
      call createCurrent(P_HPlus, P_Hl, P_GMinus, P_Hl, 3625)
      call createCurrent(P_ghWp, P_WMinus, P_ghAAnti, P_Z, 8556)
      call createCurrent(P_G0, P_Hl, P_A, 9496)
      call createCurrent(P_HlQ, P_GPlus, P_A, P_WQMinus, 6563)
      call createCurrent(P_ghWp, P_GMinus, P_Hh, P_ghAAnti, 1324)
      call createCurrent(P_HMinus, P_GPlus, P_A, 8903)
      call createCurrent(P_HQMinus, P_WPlus, P_Hh, P_ZQ, 1002)
      call createCurrent(P_GQPlus, P_Hl, P_GMinus, P_HlQ, 4793)
      call createCurrent(P_g, P_GMinus, P_HPlus, P_g, 10631)
      call createCurrent(P_GMinus, P_Hh, P_GPlus, P_Hl, 8029)
      call createCurrent(P_HMinus, P_ghWmAnti, P_Hh, P_ghZ, 207)
      call createCurrent(P_G0, P_Z, P_GMinus, P_WPlus, 1792)
      call createCurrent(P_ghZAnti, P_ghWm, P_WPlus, 5891)
      call createCurrent(P_HMinus, P_Hh, P_HPlus, P_G0, 9679)
      call createCurrent(P_AQ, P_Ha, P_WPlus, P_HQMinus, 6246)
      call createCurrent(P_ghZ, P_A, P_ghWmAnti, P_WQMinus, 579)
      call createCurrent(P_ghWmAnti, P_A, P_WMinus, P_ghA, 1386)
      call createCurrent(P_AQ, P_ghWpAnti, P_ghWp, 8351)
      call createCurrent(P_ghWm, P_ghAAnti, P_Ha, P_HPlus, 6028)
      call createCurrent(P_ghWp, P_ghZAnti, P_GMinus, P_Ha, 9630)
      call createCurrent(P_ghWmAnti, P_ghZ, P_WMinus, P_Z, 8690)
      call createCurrent(P_HQPlus, P_A, P_HQMinus, 8748)
      call createCurrent(P_Hh, P_Ha, P_GPlus, P_GMinus, 9682)
      call createCurrent(P_HPlus, P_G0, P_Z, P_WMinus, 9821)
      call createCurrent(P_A, P_ghWpAnti, P_ghA, P_WPlus, 4227)
      call createCurrent(P_Ha, P_G0, P_WPlus, P_WMinus, 9242)
      call createCurrent(P_HPlus, P_HMinus, P_Hl, P_Ha, 9077)
      call createCurrent(P_ZQ, P_Ha, P_Z, P_HaQ, 2035)
      call createCurrent(P_G0, P_G0, P_Ha, P_Ha, 428)
      call createCurrent(P_Z, P_GMinus, P_Ha, P_WPlus, 9217)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_Hl, 2761)
      call createCurrent(P_d, P_Hh, P_dAnti, 4327)
      call createCurrent(P_WQPlus, P_HMinus, P_A, P_HaQ, 2100)
      call createCurrent(P_Ha, P_cAnti, P_c, 6142)
      call createCurrent(P_HQPlus, P_G0, P_G0, P_HQMinus, 4405)
      call createCurrent(P_HlQ, P_HPlus, P_ghWpAnti, P_ghZ, 4997)
      call createCurrent(P_Ha, P_ghZ, P_ghWmAnti, P_HMinus, 530)
      call createCurrent(P_WQMinus, P_WPlus, P_HhQ, 8380)
      call createCurrent(P_HPlus, P_Hl, P_WMinus, P_Z, 7005)
      call createCurrent(P_HlQ, P_Ha, P_Hl, P_GQ0, 3447)
      call createCurrent(P_ghZAnti, P_GMinus, P_HPlus, P_ghZ, 8967)
      call createCurrent(P_HMinus, P_Z, P_A, P_HPlus, 7433)
      call createCurrent(P_muPlus, P_G0, P_muMinus, 4163)
      call createCurrent(P_HPlus, P_G0, P_HMinus, P_Ha, 3095)
      call createCurrent(P_nu_eAnti, P_A, P_nu_e, 10335)
      call createCurrent(P_Hl, P_Hl, P_WMinus, P_WPlus, 6101)
      call createCurrent(P_HhQ, P_GMinus, P_Ha, P_HQPlus, 139)
      call createCurrent(P_AQ, P_WMinus, P_ghZAnti, P_ghWp, 641)
      call createCurrent(P_nu_tau, P_tauPlus, P_GMinus, 6491)
      call createCurrent(P_HMinus, P_GPlus, P_Hh, P_G0, 7018)
      call createCurrent(P_Z, P_ghWmAnti, P_Z, P_ghWm, 5991)
      call createCurrent(P_GQPlus, P_Ha, P_G0, P_GQMinus, 8192)
      call createCurrent(P_Hh, P_GPlus, P_ghA, P_ghWpAnti, 1879)
      call createCurrent(P_ghZAnti, P_A, P_WPlus, P_ghWm, 5267)
      call createCurrent(P_muMinus, P_HPlus, P_nu_muAnti, 8411)
      call createCurrent(P_GMinus, P_Hh, P_HPlus, 4609)
      call createCurrent(P_Hh, P_ghWmAnti, P_GMinus, P_ghZ, 856)
      call createCurrent(P_ghWm, P_WMinus, P_ghWmAnti, P_WPlus, 1700)
      call createCurrent(P_ghWm, P_ghAAnti, P_Hh, P_HPlus, 4864)
      call createCurrent(P_Hl, P_HPlus, P_HMinus, 5498)
      call createCurrent(P_Hh, P_GMinus, P_ghWp, P_ghZAnti, 8097)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghWp, 5004)
      call createCurrent(P_s, P_cAnti, P_WQPlus, 2778)
      call createCurrent(P_Hh, P_HPlus, P_ghZ, P_ghWpAnti, 4322)
      call createCurrent(P_ghA, P_HPlus, P_Ha, P_ghWpAnti, 8687)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_HhQ, 6459)
      call createCurrent(P_Hl, P_Hl, P_HMinus, P_GPlus, 3038)
      call createCurrent(P_WQMinus, P_A, P_HPlus, P_HaQ, 7943)
      call createCurrent(P_GMinus, P_nu_e, P_ePlus, 2690)
      call createCurrent(P_HQMinus, P_HPlus, P_HPlus, P_HQMinus, 5081)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghZ, P_HPlus, 6957)
      call createCurrent(P_G0, P_A, P_A, P_Ha, 10540)
      call createCurrent(P_WMinus, P_ePlus, P_nu_e, 5742)
      call createCurrent(P_WPlus, P_G0, P_WMinus, P_G0, 3572)
      call createCurrent(P_GQPlus, P_WMinus, P_ZQ, 4586)
      call createCurrent(P_HQPlus, P_ghA, P_ghWpAnti, 7984)
      call createCurrent(P_HMinus, P_A, P_GPlus, 9559)
      call createCurrent(P_G0, P_ghWp, P_ghWpAnti, P_Ha, 9002)
      call createCurrent(P_HaQ, P_Hh, P_HMinus, P_HQPlus, 1349)
      call createCurrent(P_HaQ, P_Z, P_WMinus, P_HQPlus, 433)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_GQ0, 3019)
      call createCurrent(P_WMinus, P_Z, P_ghWmAnti, P_ghA, 7760)
      call createCurrent(P_GPlus, P_WMinus, P_A, P_Hl, 3595)
      call createCurrent(P_HQPlus, P_HMinus, P_Hh, P_GQ0, 3886)
      call createCurrent(P_Ha, P_HPlus, P_ghWpAnti, P_ghA, 2180)
      call createCurrent(P_cAnti, P_GPlus, P_s, 7860)
      call createCurrent(P_HaQ, P_GMinus, P_GPlus, P_GQ0, 950)
      call createCurrent(P_GMinus, P_HPlus, P_ghWp, P_ghWpAnti, 9395)
      call createCurrent(P_HlQ, P_WPlus, P_Z, P_GQMinus, 2503)
      call createCurrent(P_ghAAnti, P_ghWm, P_WPlus, P_Z, 2332)
      call createCurrent(P_Hl, P_ghWp, P_HMinus, P_ghZAnti, 4162)
      call createCurrent(P_ghWpAnti, P_A, P_WPlus, P_ghA, 1457)
      call createCurrent(P_AQ, P_WMinus, P_WPlus, P_ZQ, 4865)
      call createCurrent(P_muPlus, P_nu_mu, P_GMinus, 4060)
      call createCurrent(P_AQ, P_WPlus, P_Hh, P_GQMinus, 1562)
      call createCurrent(P_GPlus, P_ghZ, P_HMinus, P_ghAAnti, 9287)
      call createCurrent(P_HQPlus, P_A, P_A, P_HQMinus, 5097)
      call createCurrent(P_WMinus, P_ghZ, P_ghWmAnti, P_Z, 8533)
      call createCurrent(P_Hh, P_Hh, 9328)
      call createCurrent(P_ghWp, P_WMinus, P_A, P_ghZAnti, 1563)
      call createCurrent(P_ghAAnti, P_ghWp, P_Z, P_WMinus, 5453)
      call createCurrent(P_AQ, P_Hl, P_WMinus, P_HQPlus, 2600)
      call createCurrent(P_ghWmAnti, P_ghA, P_WMinus, P_Z, 6510)
      call createCurrent(P_Ha, P_ghWm, P_ghZAnti, P_HPlus, 1452)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hh, 7445)
      call createCurrent(P_WMinus, P_ghWmAnti, P_WMinus, P_ghWp, 3348)
      call createCurrent(P_ghWpAnti, P_ghA, P_GPlus, P_Hh, 6111)
      call createCurrent(P_Z, P_Ha, P_WMinus, P_GPlus, 10266)
      call createCurrent(P_GQPlus, P_Hh, P_WMinus, P_AQ, 2544)
      call createCurrent(P_tauMinus, P_tauPlus, P_Z, 1191)
      call createCurrent(P_ghZAnti, P_G0, P_ghWp, P_GQMinus, 8034)
      call createCurrent(P_b, P_bAnti, P_g, 424)
      call createCurrent(P_ghZAnti, P_ghA, P_WMinus, P_WQPlus, 2026)
      call createCurrent(P_Hl, P_HPlus, P_Hh, P_HMinus, 6750)
      call createCurrent(P_ghWpAnti, P_G0, P_ghA, P_HPlus, 9962)
      call createCurrent(P_HPlus, P_Ha, P_ghA, P_ghWpAnti, 8487)
      call createCurrent(P_nu_mu, P_WMinus, P_muPlus, 6713)
      call createCurrent(P_ghWm, P_Hh, P_GPlus, P_ghZAnti, 5613)
      call createCurrent(P_ghZ, P_Hh, P_ghWmAnti, P_GMinus, 8045)
      call createCurrent(P_WQPlus, P_WMinus, P_ghZAnti, P_ghZ, 6883)
      call createCurrent(P_G0, P_GMinus, P_ghAAnti, P_ghWp, 315)
      call createCurrent(P_A, P_A, P_WMinus, P_WPlus, 4937)
      call createCurrent(P_HhQ, P_Ha, P_ghWpAnti, P_ghWp, 6362)
      call createCurrent(P_ghA, P_ghWpAnti, P_Hh, P_GQPlus, 4516)
      call createCurrent(P_Hh, P_ghWmAnti, P_ghZ, P_HMinus, 6646)
      call createCurrent(P_Hh, P_HPlus, P_ghA, P_ghWpAnti, 3111)
      call createCurrent(P_HlQ, P_HPlus, P_A, P_WQMinus, 5199)
      call createCurrent(P_GMinus, P_G0, P_ghWmAnti, P_ghA, 8126)
      call createCurrent(P_A, P_WPlus, P_Hl, P_GMinus, 2364)
      call createCurrent(P_HlQ, P_Z, P_WMinus, P_HQPlus, 5948)
      call createCurrent(P_ghZAnti, P_Hl, P_Hl, P_ghZ, 7696)
      call createCurrent(P_HQMinus, P_Ha, P_GPlus, P_HhQ, 2387)
      call createCurrent(P_ghWm, P_WPlus, P_Z, P_ghZAnti, 1636)
      call createCurrent(P_GMinus, P_ghZ, P_ghZAnti, P_HPlus, 8899)
      call createCurrent(P_HaQ, P_ghWm, P_HPlus, P_ghAAnti, 1028)
      call createCurrent(P_Hh, P_GMinus, P_ghZ, P_ghWmAnti, 3973)
      call createCurrent(P_HQPlus, P_WMinus, P_Hh, P_AQ, 2791)
      call createCurrent(P_WMinus, P_ghZAnti, P_WPlus, P_ghA, 7000)
      call createCurrent(P_HQPlus, P_Hh, P_Z, P_WQMinus, 6803)
      call createCurrent(P_HhQ, P_Ha, P_HPlus, P_HQMinus, 1073)
      call createCurrent(P_Hh, P_G0, P_Ha, P_Hh, 6319)
      call createCurrent(P_ghZAnti, P_HMinus, P_Hh, P_ghWp, 608)
      call createCurrent(P_HlQ, P_G0, P_Hh, P_GQ0, 3357)
      call createCurrent(P_Hl, P_GMinus, P_HPlus, P_Hl, 7638)
      call createCurrent(P_GQPlus, P_Ha, P_Hl, P_HQMinus, 5741)
      call createCurrent(P_HQMinus, P_ghZAnti, P_ghWp, 2156)
      call createCurrent(P_b, P_tAnti, P_WQPlus, 4032)
      call createCurrent(P_ghWmAnti, P_G0, P_GMinus, P_ghZ, 7493)
      call createCurrent(P_HQMinus, P_Z, P_A, P_HQPlus, 1201)
      call createCurrent(P_WPlus, P_Z, P_Hh, P_GMinus, 2103)
      call createCurrent(P_HQMinus, P_Hl, P_Hh, P_GQPlus, 2033)
      call createCurrent(P_HQMinus, P_nu_tau, P_tauPlus, 1021)
      call createCurrent(P_ZQ, P_GPlus, P_GQMinus, 1924)
      call createCurrent(P_A, P_WMinus, P_Hl, P_HPlus, 4332)
      call createCurrent(P_GQ0, P_u, P_uAnti, 8390)
      call createCurrent(P_ghA, P_Hl, P_HPlus, P_ghWpAnti, 2528)
      call createCurrent(P_ghAAnti, P_ghZ, P_WMinus, P_WQPlus, 5821)
      call createCurrent(P_G0, P_ghWmAnti, P_G0, P_ghWm, 3703)
      call createCurrent(P_ePlus, P_GMinus, P_nu_e, 8527)
      call createCurrent(P_ghZ, P_ghAAnti, P_WMinus, P_WPlus, 1284)
      call createCurrent(P_ghAAnti, P_ghA, P_HMinus, P_HPlus, 5494)
      call createCurrent(P_HaQ, P_WPlus, P_WQMinus, 7136)
      call createCurrent(P_ghA, P_ghWmAnti, P_HMinus, P_Hl, 7624)
      call createCurrent(P_GPlus, P_WMinus, 10176)
      call createCurrent(P_Ha, P_ghWpAnti, P_HPlus, P_ghZ, 4980)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_HhQ, 6522)
      call createCurrent(P_WQMinus, P_WMinus, P_WPlus, P_WQPlus, 2564)
      call createCurrent(P_Hh, P_Hl, P_ghWm, P_ghWmAnti, 9706)
      call createCurrent(P_GMinus, P_ghZ, P_G0, P_ghWmAnti, 6842)
      call createCurrent(P_tauMinus, P_GPlus, P_nu_tauAnti, 4014)
      call createCurrent(P_ghWm, P_ghAAnti, P_GPlus, P_HlQ, 4519)
      call createCurrent(P_ghAAnti, P_HMinus, P_ghWp, P_HlQ, 7818)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hh, P_HPlus, 7081)
      call createCurrent(P_A, P_Hl, P_WMinus, P_GPlus, 5923)
      call createCurrent(P_GMinus, P_Ha, P_ghZAnti, P_ghWp, 9308)
      call createCurrent(P_s, P_sAnti, 10011)
      call createCurrent(P_ghWm, P_ghZAnti, P_Hh, P_GPlus, 4705)
      call createCurrent(P_ghWp, P_A, P_ghWpAnti, P_ZQ, 7567)
      call createCurrent(P_GQ0, P_ghWm, P_ghWmAnti, 7657)
      call createCurrent(P_ghWm, P_GMinus, P_GPlus, P_ghWmAnti, 2586)
      call createCurrent(P_HQPlus, P_ghWmAnti, P_HMinus, P_ghWm, 3853)
      call createCurrent(P_GMinus, P_GPlus, P_Hh, P_Hh, 8273)
      call createCurrent(P_GMinus, P_GPlus, P_Z, P_Z, 4168)
      call createCurrent(P_bAnti, P_A, P_b, 8460)
      call createCurrent(P_Ha, P_Hh, P_Hh, P_Ha, 988)
      call createCurrent(P_Hl, P_Ha, P_Hl, P_G0, 8132)
      call createCurrent(P_GMinus, P_Hl, P_HPlus, P_G0, 2577)
      call createCurrent(P_Ha, P_ghZ, P_G0, P_ghZAnti, 10147)
      call createCurrent(P_Ha, P_ghWp, P_G0, P_ghWpAnti, 9514)
      call createCurrent(P_ghWm, P_ghWmAnti, P_A, P_ZQ, 2576)
      call createCurrent(P_Ha, P_ghZ, P_GMinus, P_ghWmAnti, 9010)
      call createCurrent(P_Z, P_G0, P_A, P_Ha, 10723)
      call createCurrent(P_Hh, P_ghWm, P_ghWmAnti, 3446)
      call createCurrent(P_GQMinus, P_HPlus, P_HMinus, P_HQPlus, 6428)
      call createCurrent(P_G0, P_ghWm, P_GPlus, P_ghAAnti, 5264)
      call createCurrent(P_Z, P_ghZAnti, P_ghWm, P_WPlus, 4130)
      call createCurrent(P_GMinus, P_HPlus, 9366)
      call createCurrent(P_WPlus, P_Hl, P_Hh, P_WMinus, 9040)
      call createCurrent(P_bAnti, P_t, P_GMinus, 1661)
      call createCurrent(P_GMinus, P_Ha, P_HPlus, P_Hh, 5015)
      call createCurrent(P_WQPlus, P_Z, P_ghWpAnti, P_ghA, 5916)
      call createCurrent(P_G0, P_GPlus, P_ghZ, P_ghWpAnti, 7734)
      call createCurrent(P_HaQ, P_Ha, P_Hl, P_HhQ, 391)
      call createCurrent(P_GPlus, P_ghZAnti, P_HMinus, P_ghA, 9566)
      call createCurrent(P_HhQ, P_WPlus, P_Hh, P_WQMinus, 3756)
      call createCurrent(P_ghZAnti, P_WMinus, P_WPlus, P_ghA, 1494)
      call createCurrent(P_HhQ, P_ghZAnti, P_GMinus, P_ghWp, 5691)
      call createCurrent(P_HMinus, P_ghA, P_ghWmAnti, 1620)
      call createCurrent(P_GQPlus, P_GMinus, P_Hh, P_HaQ, 7496)
      call createCurrent(P_HQPlus, P_Hh, P_Hl, P_HQMinus, 8024)
      call createCurrent(P_tauPlus, P_Hl, P_tauMinus, 6610)
      call createCurrent(P_HMinus, P_A, P_WPlus, P_Ha, 492)
      call createCurrent(P_G0, P_GMinus, P_WPlus, 8093)
      call createCurrent(P_GPlus, P_HMinus, P_Hl, P_G0, 5210)
      call createCurrent(P_muMinus, P_nu_muAnti, P_GQPlus, 5733)
      call createCurrent(P_A, P_GPlus, P_A, P_GMinus, 903)
      call createCurrent(P_A, P_GPlus, P_Hh, P_WMinus, 2441)
      call createCurrent(P_GQMinus, P_GPlus, P_Ha, P_HaQ, 5941)
      call createCurrent(P_ghWmAnti, P_ghA, P_G0, P_HMinus, 9195)
      call createCurrent(P_GMinus, P_WPlus, P_Hh, P_A, 1055)
      call createCurrent(P_sAnti, P_GMinus, P_c, 4336)
      call createCurrent(P_Hl, P_sAnti, P_s, 3910)
      call createCurrent(P_Ha, P_WMinus, P_WPlus, P_G0, 9461)
      call createCurrent(P_ghAAnti, P_ghWp, P_WMinus, P_ZQ, 4193)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_HaQ, 958)
      call createCurrent(P_WMinus, P_Hl, P_GPlus, P_A, 5978)
      call createCurrent(P_WPlus, P_Z, P_G0, P_HMinus, 9201)
      call createCurrent(P_Ha, P_G0, P_Hh, P_Hl, 8)
      call createCurrent(P_WQPlus, P_Hl, P_WMinus, P_HaQ, 7918)
      call createCurrent(P_Hl, P_WMinus, P_Hl, P_WPlus, 7781)
      call createCurrent(P_ghA, P_HPlus, P_ghWpAnti, P_Ha, 6377)
      call createCurrent(P_GPlus, P_Z, P_A, P_HMinus, 10342)
      call createCurrent(P_GPlus, P_Hl, P_Ha, P_GMinus, 10189)
      call createCurrent(P_u, P_GMinus, P_dAnti, 8193)
      call createCurrent(P_Z, P_Z, P_ghWmAnti, P_ghWm, 1972)
      call createCurrent(P_GPlus, P_g, P_g, P_GMinus, 10694)
      call createCurrent(P_HQPlus, P_muMinus, P_nu_muAnti, 3231)
      call createCurrent(P_ghAAnti, P_HMinus, P_Hl, P_ghWp, 1535)
      call createCurrent(P_ghA, P_WPlus, P_ghZAnti, P_WMinus, 6648)
      call createCurrent(P_ghWpAnti, P_ghWp, P_GPlus, P_GMinus, 3049)
      call createCurrent(P_HaQ, P_ghWp, P_ghWpAnti, 1281)
      call createCurrent(P_sAnti, P_s, P_HlQ, 7031)
      call createCurrent(P_HPlus, P_Hl, P_ghAAnti, P_ghWm, 8268)
      call createCurrent(P_Hh, P_HPlus, P_Hl, P_GMinus, 5135)
      call createCurrent(P_GQPlus, P_ghZ, P_GMinus, P_ghZAnti, 7122)
      call createCurrent(P_Hl, P_Hl, P_GMinus, P_GPlus, 2748)
      call createCurrent(P_HhQ, P_Hl, P_HMinus, P_HQPlus, 7552)
      call createCurrent(P_GQ0, P_Ha, P_HMinus, P_GQPlus, 2308)
      call createCurrent(P_HMinus, P_Ha, P_GPlus, P_Hh, 1560)
      call createCurrent(P_A, P_Z, P_Hh, P_Hl, 10709)
      call createCurrent(P_muPlus, P_nu_mu, P_HQMinus, 3720)
      call createCurrent(P_HMinus, P_ghWmAnti, P_Hl, P_ghA, 2518)
      call createCurrent(P_ghWpAnti, P_ghZ, P_G0, P_GPlus, 5388)
      call createCurrent(P_Hl, P_GPlus, P_G0, P_HMinus, 524)
      call createCurrent(P_GQMinus, P_WPlus, P_GQ0, 4037)
      call createCurrent(P_ghAAnti, P_Ha, P_HPlus, P_ghWm, 3630)
      call createCurrent(P_Hl, P_HPlus, P_Hl, P_HMinus, 5091)
      call createCurrent(P_dAnti, P_d, P_HlQ, 587)
      call createCurrent(P_Hh, P_WPlus, P_WMinus, P_Hh, 564)
      call createCurrent(P_ghWm, P_ghZAnti, P_GPlus, P_Ha, 9260)
      call createCurrent(P_WMinus, P_ghWp, P_Z, P_ghAAnti, 1250)
      call createCurrent(P_A, P_GPlus, P_G0, P_WMinus, 4741)
      call createCurrent(P_Hl, P_HMinus, P_ghA, P_ghWmAnti, 5896)
      call createCurrent(P_HaQ, P_G0, P_GMinus, P_GQPlus, 5066)
      call createCurrent(P_HaQ, P_GMinus, P_Ha, P_HQPlus, 8774)
      call createCurrent(P_ghAAnti, P_ghWm, P_GPlus, 7427)
      call createCurrent(P_HPlus, P_ghZAnti, P_ghWm, 6023)
      call createCurrent(P_A, P_Ha, P_WPlus, P_GMinus, 9278)
      call createCurrent(P_A, P_G0, P_WMinus, P_HPlus, 10016)
      call createCurrent(P_d, P_dAnti, P_AQ, 3336)
      call createCurrent(P_WQPlus, P_ghAAnti, P_ghWm, 7880)
      call createCurrent(P_HPlus, P_HMinus, P_GPlus, P_GMinus, 2434)
      call createCurrent(P_GPlus, P_Ha, P_A, P_WMinus, 10143)
      call createCurrent(P_WPlus, P_WMinus, P_Hh, P_Hl, 9707)
      call createCurrent(P_HQMinus, P_GPlus, P_HhQ, 656)
      call createCurrent(P_HaQ, P_WMinus, P_Hl, P_WQPlus, 2964)
      call createCurrent(P_HQPlus, P_HPlus, P_HMinus, P_HQMinus, 3612)
      call createCurrent(P_WQMinus, P_A, P_WPlus, P_AQ, 3901)
      call createCurrent(P_HhQ, P_Hh, P_ghWm, P_ghWmAnti, 3636)
      call createCurrent(P_GMinus, P_HPlus, P_G0, 2530)
      call createCurrent(P_GQPlus, P_Ha, P_Hh, P_HQMinus, 2624)
      call createCurrent(P_Hh, P_Ha, P_HPlus, P_HMinus, 9798)
      call createCurrent(P_HPlus, P_GMinus, P_Hl, P_Hh, 4856)
      call createCurrent(P_G0, P_G0, P_ghZAnti, P_ghZ, 809)
      call createCurrent(P_HhQ, P_ghWmAnti, P_Ha, P_ghWm, 3033)
      call createCurrent(P_ZQ, P_Hh, P_WMinus, P_HQPlus, 3563)
      call createCurrent(P_HlQ, P_Z, P_HMinus, P_WQPlus, 6064)
      call createCurrent(P_ghWpAnti, P_WPlus, P_A, P_ghZ, 2866)
      call createCurrent(P_GQ0, P_HPlus, P_GMinus, P_GQ0, 5400)
      call createCurrent(P_Ha, P_u, P_uAnti, 1038)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghZ, P_GMinus, 9561)
      call createCurrent(P_ghZ, P_A, P_ghWmAnti, P_WMinus, 5642)
      call createCurrent(P_HPlus, P_Ha, P_ghWm, P_ghZAnti, 2785)
      call createCurrent(P_ghZ, P_Hh, P_GPlus, P_ghWpAnti, 5085)
      call createCurrent(P_GQPlus, P_HMinus, P_Ha, P_HhQ, 8266)
      call createCurrent(P_GQ0, P_Ha, P_HPlus, P_GQMinus, 4145)
      call createCurrent(P_ghZAnti, P_ghA, P_HMinus, P_HPlus, 2326)
      call createCurrent(P_HMinus, P_Ha, P_WPlus, P_A, 4029)
      call createCurrent(P_G0, P_GMinus, P_GPlus, P_Hl, 9074)
      call createCurrent(P_G0, P_ghZ, P_G0, P_ghZAnti, 1978)
      call createCurrent(P_G0, P_GPlus, P_GMinus, P_Ha, 4553)
      call createCurrent(P_Z, P_A, P_HMinus, P_HPlus, 7991)
      call createCurrent(P_GMinus, P_HPlus, P_GMinus, P_GPlus, 7324)
      call createCurrent(P_GQPlus, P_ghWpAnti, P_Hh, P_ghA, 1263)
      call createCurrent(P_G0, P_ghWp, P_HMinus, P_ghZAnti, 9485)
      call createCurrent(P_HhQ, P_ghAAnti, P_GMinus, P_ghWp, 5841)
      call createCurrent(P_Z, P_Hh, P_Z, P_Hl, 9394)
      call createCurrent(P_HlQ, P_muPlus, P_muMinus, 4962)
      call createCurrent(P_ghA, P_Ha, P_HPlus, P_ghWpAnti, 7647)
      call createCurrent(P_GQMinus, P_ghA, P_Hh, P_ghWmAnti, 1398)
      call createCurrent(P_Z, P_HPlus, P_Z, P_HMinus, 4977)
      call createCurrent(P_ghZ, P_G0, P_ghWpAnti, P_HPlus, 8931)
      call createCurrent(P_HlQ, P_Z, P_GMinus, P_WQPlus, 5531)
      call createCurrent(P_Ha, P_HMinus, P_A, P_WPlus, 3121)
      call createCurrent(P_Hl, P_HPlus, P_Hh, P_GMinus, 5563)
      call createCurrent(P_tauPlus, P_nu_tau, P_HMinus, 1710)
      call createCurrent(P_HPlus, P_Z, P_WMinus, P_Hh, 3141)
      call createCurrent(P_HhQ, P_Hl, P_HMinus, P_GQPlus, 7738)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghWp, P_Hh, 7824)
      call createCurrent(P_Hh, P_GPlus, P_WMinus, 2579)
      call createCurrent(P_A, P_A, P_HPlus, P_GMinus, 9376)
      call createCurrent(P_GQ0, P_Hl, P_GPlus, P_HQMinus, 2899)
      call createCurrent(P_Hh, P_HMinus, P_G0, P_HPlus, 8848)
      call createCurrent(P_WMinus, P_GPlus, 9604)
      call createCurrent(P_G0, P_g, P_Ha, P_g, 10587)
      call createCurrent(P_Ha, P_WPlus, P_GMinus, P_Z, 10344)
      call createCurrent(P_G0, P_HMinus, P_WPlus, 9078)
      call createCurrent(P_ghZ, P_Z, P_WMinus, P_ghWmAnti, 4738)
      call createCurrent(P_Hl, P_Hh, P_HMinus, P_HPlus, 4137)
      call createCurrent(P_WPlus, P_WMinus, P_HMinus, P_GPlus, 9263)
      call createCurrent(P_GPlus, P_GMinus, P_Ha, P_Hl, 9666)
      call createCurrent(P_GMinus, P_ghZ, P_ghZAnti, P_GPlus, 2155)
      call createCurrent(P_ZQ, P_Z, P_GMinus, P_GQPlus, 6053)
      call createCurrent(P_HlQ, P_G0, P_GPlus, P_HQMinus, 7336)
      call createCurrent(P_HQPlus, P_HMinus, P_ghA, P_ghZAnti, 8473)
      call createCurrent(P_u, P_uAnti, P_gQ, 3157)
      call createCurrent(P_G0, P_A, 9753)
      call createCurrent(P_HPlus, P_Hh, P_ghZAnti, P_ghWm, 3560)
      call createCurrent(P_GPlus, P_HMinus, P_ghZAnti, P_ghA, 9335)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_Hl, 2556)
      call createCurrent(P_HQPlus, P_G0, P_GQMinus, 5018)
      call createCurrent(P_HPlus, P_G0, P_HMinus, 10435)
      call createCurrent(P_HaQ, P_GPlus, P_GMinus, P_GQ0, 611)
      call createCurrent(P_HlQ, P_ghWm, P_Hl, P_ghWmAnti, 2520)
      call createCurrent(P_HPlus, P_GMinus, P_HPlus, P_GMinus, 5399)
      call createCurrent(P_Hl, P_A, P_WMinus, P_HPlus, 6233)
      call createCurrent(P_tauPlus, P_nu_tau, P_WMinus, 5010)
      call createCurrent(P_GMinus, P_ghWp, P_Hl, P_ghZAnti, 4754)
      call createCurrent(P_ghWpAnti, P_ghA, P_Z, P_WPlus, 5594)
      call createCurrent(P_HhQ, P_Hl, P_G0, P_HaQ, 6449)
      call createCurrent(P_ghWp, P_Ha, P_ghZAnti, P_HQMinus, 6586)
      call createCurrent(P_ghZAnti, P_ghWm, P_WPlus, P_A, 1921)
      call createCurrent(P_ghWm, P_Hh, P_ghZAnti, P_HPlus, 2398)
      call createCurrent(P_GPlus, P_HMinus, P_ghAAnti, P_ghA, 8850)
      call createCurrent(P_HPlus, P_cAnti, P_s, 4049)
      call createCurrent(P_HMinus, P_ghWp, P_Hl, P_ghZAnti, 7212)
      call createCurrent(P_WMinus, P_HMinus, P_WPlus, P_GPlus, 10067)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_Hl, 3909)
      call createCurrent(P_ghWp, P_Ha, P_GMinus, P_ghAAnti, 9824)
      call createCurrent(P_A, P_Hh, P_A, P_Hl, 10610)
      call createCurrent(P_ghZAnti, P_GMinus, P_ghWp, P_G0, 4764)
      call createCurrent(P_nu_mu, P_muPlus, P_GMinus, 1973)
      call createCurrent(P_Ha, P_ghZAnti, P_ghZ, P_G0, 9897)
      call createCurrent(P_WQPlus, P_GMinus, P_Hl, P_AQ, 7350)
      call createCurrent(P_G0, P_WPlus, P_Z, P_GMinus, 6579)
      call createCurrent(P_WMinus, P_HPlus, P_Z, P_Ha, 2676)
      call createCurrent(P_Hl, P_Z, P_A, 9850)
      call createCurrent(P_ZQ, P_A, P_WMinus, P_WQPlus, 3548)
      call createCurrent(P_g, P_GMinus, P_g, P_GPlus, 10736)
      call createCurrent(P_HaQ, P_Ha, P_Ha, P_GQ0, 6266)
      call createCurrent(P_GMinus, P_Z, P_HPlus, P_A, 9415)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghA, P_HPlus, 3338)
      call createCurrent(P_ghWmAnti, P_ghWm, P_HlQ, 8683)
      call createCurrent(P_HQMinus, P_ghWp, P_ghZAnti, 4442)
      call createCurrent(P_Hh, P_A, P_A, 10727)
      call createCurrent(P_ghAAnti, P_Hl, P_GPlus, P_ghWm, 2935)
      call createCurrent(P_bAnti, P_Hh, P_b, 884)
      call createCurrent(P_A, P_A, P_Hh, 10541)
      call createCurrent(P_ghWm, P_G0, P_HPlus, P_ghZAnti, 9470)
      call createCurrent(P_ghAAnti, P_A, P_WPlus, P_ghWm, 8042)
      call createCurrent(P_HPlus, P_HMinus, P_Ha, P_Hl, 9212)
      call createCurrent(P_HaQ, P_WMinus, P_Hh, P_WQPlus, 8782)
      call createCurrent(P_WPlus, P_ghWm, P_ghAAnti, P_Z, 6836)
      call createCurrent(P_ghWp, P_Hl, P_HMinus, P_ghAAnti, 7596)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghWp, P_HlQ, 5307)
      call createCurrent(P_HaQ, P_GMinus, P_HPlus, P_GQ0, 7823)
      call createCurrent(P_HhQ, P_Ha, P_HMinus, P_GQPlus, 337)
      call createCurrent(P_Hl, P_Hh, P_WPlus, P_WMinus, 10018)
      call createCurrent(P_WQPlus, P_G0, P_WMinus, P_GQ0, 885)
      call createCurrent(P_HMinus, P_ghZ, P_G0, P_ghWmAnti, 9181)
      call createCurrent(P_G0, P_ghWmAnti, P_ghWm, P_G0, 7514)
      call createCurrent(P_A, P_GPlus, P_HMinus, 10400)
      call createCurrent(P_Hl, P_ghWmAnti, P_GMinus, P_ghZ, 229)
      call createCurrent(P_ghWmAnti, P_G0, P_Ha, P_ghWm, 9406)
      call createCurrent(P_Hl, P_GPlus, P_WMinus, 7762)
      call createCurrent(P_ghWm, P_HPlus, P_ghAAnti, P_G0, 9747)
      call createCurrent(P_GMinus, P_ghAAnti, P_Hl, P_ghWp, 8025)
      call createCurrent(P_WQPlus, P_Z, P_HMinus, P_HaQ, 5076)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Z, P_ZQ, 4131)
      call createCurrent(P_ghAAnti, P_ghWm, P_A, P_WPlus, 3713)
      call createCurrent(P_A, P_ghAAnti, P_ghWp, P_WMinus, 4703)
      call createCurrent(P_ghA, P_Hl, P_ghWmAnti, P_HQMinus, 7440)
      call createCurrent(P_ghWp, P_Hh, P_ghWpAnti, P_GQ0, 3193)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghA, 5273)
      call createCurrent(P_WPlus, P_HMinus, P_A, P_G0, 9454)
      call createCurrent(P_tAnti, P_t, 9532)
      call createCurrent(P_ghWp, P_HMinus, P_ghAAnti, P_HaQ, 604)
      call createCurrent(P_GQMinus, P_G0, P_ghA, P_ghWmAnti, 4427)
      call createCurrent(P_A, P_Hl, P_Z, 8839)
      call createCurrent(P_Z, P_Ha, P_Ha, P_A, 9284)
      call createCurrent(P_WMinus, P_ghZ, P_ghAAnti, P_WPlus, 620)
      call createCurrent(P_ghWm, P_Ha, P_HPlus, P_ghZAnti, 8243)
      call createCurrent(P_ghZAnti, P_WMinus, P_Z, P_ghWp, 6571)
      call createCurrent(P_ghWp, P_Z, P_WMinus, P_ghAAnti, 1937)
      call createCurrent(P_Hh, P_GPlus, P_ghWm, P_ghAAnti, 5842)
      call createCurrent(P_A, P_WPlus, P_HMinus, P_Ha, 1730)
      call createCurrent(P_HaQ, P_HMinus, P_GPlus, P_HaQ, 5387)
      call createCurrent(P_GQMinus, P_GPlus, P_G0, P_HaQ, 2511)
      call createCurrent(P_ghWp, P_GMinus, P_ghAAnti, 7156)
      call createCurrent(P_g, P_sAnti, P_s, 7719)
      call createCurrent(P_HQPlus, P_Ha, P_GMinus, P_HhQ, 3882)
      call createCurrent(P_Z, P_Hl, P_WPlus, P_HMinus, 2002)
      call createCurrent(P_Ha, P_G0, P_G0, P_Hh, 10656)
      call createCurrent(P_HPlus, P_GPlus, P_GMinus, P_HMinus, 57)
      call createCurrent(P_Z, P_HMinus, P_HPlus, P_Z, 4855)
      call createCurrent(P_WMinus, P_WPlus, P_ghA, P_ghAAnti, 682)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghWm, P_Z, 3498)
      call createCurrent(P_GPlus, P_WMinus, P_A, P_Hh, 4183)
      call createCurrent(P_A, P_GMinus, P_WPlus, P_Hh, 6605)
      call createCurrent(P_ghA, P_ghWpAnti, P_Z, P_WPlus, 5808)
      call createCurrent(P_WMinus, P_Hh, P_A, P_HPlus, 544)
      call createCurrent(P_ghZAnti, P_Hl, P_ghWp, P_HQMinus, 3082)
      call createCurrent(P_Hh, P_ghZ, P_ghZAnti, P_Hl, 9851)
      call createCurrent(P_WPlus, P_Z, P_WMinus, 7316)
      call createCurrent(P_GMinus, P_Hl, P_Hh, P_HPlus, 561)
      call createCurrent(P_HhQ, P_Hh, P_HPlus, P_HQMinus, 8438)
      call createCurrent(P_ghA, P_ghWmAnti, P_HMinus, P_Hh, 5884)
      call createCurrent(P_WMinus, P_ghWmAnti, P_Z, P_ghZ, 8162)
      call createCurrent(P_G0, P_Hh, P_GPlus, P_GMinus, 9651)
      call createCurrent(P_GQ0, P_Hh, P_WMinus, P_WQPlus, 3222)
      call createCurrent(P_HlQ, P_G0, P_ghWpAnti, P_ghWp, 1575)
      call createCurrent(P_ghZAnti, P_ghA, P_HPlus, P_GMinus, 9450)
      call createCurrent(P_ghZAnti, P_WPlus, P_ghWm, 1370)
      call createCurrent(P_ghWm, P_WPlus, P_ghAAnti, P_ZQ, 66)
      call createCurrent(P_HMinus, P_ghWm, P_HPlus, P_ghWmAnti, 2683)
      call createCurrent(P_uAnti, P_HPlus, P_d, 6595)
      call createCurrent(P_d, P_dAnti, P_ZQ, 450)
      call createCurrent(P_HaQ, P_GMinus, P_HPlus, P_HaQ, 3269)
      call createCurrent(P_HaQ, P_A, P_HPlus, P_WQMinus, 2827)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghZ, P_HQPlus, 4287)
      call createCurrent(P_HMinus, P_Ha, P_GPlus, P_Hl, 977)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghWp, P_G0, 10152)
      call createCurrent(P_G0, P_WMinus, P_G0, P_WPlus, 7809)
      call createCurrent(P_A, P_ghZAnti, P_WMinus, P_ghWp, 3411)
      call createCurrent(P_ghZ, P_Hh, P_ghWmAnti, P_GQMinus, 44)
      call createCurrent(P_Z, P_Z, P_ghWp, P_ghWpAnti, 5148)
      call createCurrent(P_ePlus, P_HMinus, P_nu_e, 2183)
      call createCurrent(P_HlQ, P_GMinus, P_HPlus, P_GQ0, 448)
      call createCurrent(P_GMinus, P_HPlus, P_GPlus, P_HMinus, 2754)
      call createCurrent(P_ghZ, P_ghWmAnti, P_GMinus, P_Ha, 9440)
      call createCurrent(P_GQMinus, P_HPlus, P_Hh, P_HaQ, 5377)
      call createCurrent(P_Z, P_Hl, P_A, P_Hl, 9564)
      call createCurrent(P_Z, P_GMinus, P_WPlus, P_Hh, 4693)
      call createCurrent(P_ZQ, P_ghWpAnti, P_ghWp, 2010)
      call createCurrent(P_Ha, P_GMinus, P_Hl, P_GPlus, 9852)
      call createCurrent(P_GMinus, P_GPlus, P_Hl, P_G0, 9001)
      call createCurrent(P_AQ, P_WPlus, P_HQMinus, 4836)
      call createCurrent(P_HaQ, P_HMinus, P_WPlus, P_ZQ, 4834)
      call createCurrent(P_GPlus, P_GMinus, P_HMinus, P_GPlus, 6937)
      call createCurrent(P_WPlus, P_ghWpAnti, P_ghWp, P_WMinus, 8219)
      call createCurrent(P_GMinus, P_ghA, P_ghZAnti, P_HPlus, 9986)
      call createCurrent(P_G0, P_HPlus, P_HMinus, P_Hl, 9268)
      call createCurrent(P_HQPlus, P_Hh, P_Ha, P_GQMinus, 3915)
      call createCurrent(P_WPlus, P_GPlus, P_HMinus, P_WMinus, 9655)
      call createCurrent(P_WMinus, P_WMinus, P_ghWp, P_ghWmAnti, 7225)
      call createCurrent(P_ghWm, P_Ha, P_Ha, P_ghWmAnti, 3031)
      call createCurrent(P_HhQ, P_HPlus, P_GMinus, P_GQ0, 6801)
      call createCurrent(P_Hh, P_WMinus, P_GPlus, 1400)
      call createCurrent(P_HQPlus, P_Ha, P_HMinus, P_HlQ, 382)
      call createCurrent(P_GQMinus, P_HPlus, P_HlQ, 4200)
      call createCurrent(P_G0, P_WMinus, P_HPlus, 10089)
      call createCurrent(P_GQ0, P_Ha, P_HPlus, P_HQMinus, 1796)
      call createCurrent(P_GMinus, P_ghWpAnti, P_GPlus, P_ghWp, 473)
      call createCurrent(P_HPlus, P_A, P_GMinus, P_A, 10034)
      call createCurrent(P_GQMinus, P_t, P_bAnti, 3645)
      call createCurrent(P_ghZ, P_ghWmAnti, P_WMinus, 1756)
      call createCurrent(P_ghWm, P_Ha, P_HPlus, P_ghAAnti, 7180)
      call createCurrent(P_G0, P_ghWm, P_G0, P_ghWmAnti, 8337)
      call createCurrent(P_ghWpAnti, P_Hh, P_ghZ, P_HQPlus, 6126)
      call createCurrent(P_A, P_u, P_uAnti, 2559)
      call createCurrent(P_WQPlus, P_GMinus, P_Hh, P_AQ, 1205)
      call createCurrent(P_A, P_Z, P_g, P_g, 10737)
      call createCurrent(P_Z, P_A, P_Hh, P_Hh, 9675)
      call createCurrent(P_WMinus, P_Z, P_HPlus, P_G0, 9940)
      call createCurrent(P_HQPlus, P_Hl, P_HMinus, P_HaQ, 5101)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, P_HhQ, 8211)
      call createCurrent(P_ghWpAnti, P_Z, P_A, P_ghWp, 2390)
      call createCurrent(P_GPlus, P_GMinus, P_GPlus, P_HMinus, 4272)
      call createCurrent(P_Z, P_GPlus, P_HMinus, P_A, 9297)
      call createCurrent(P_HPlus, P_HMinus, P_ghZAnti, P_ghZ, 4135)
      call createCurrent(P_HhQ, P_Ha, P_ZQ, 4968)
      call createCurrent(P_A, P_WPlus, P_ghZAnti, P_ghWm, 7485)
      call createCurrent(P_Z, P_GPlus, P_GMinus, 59)
      call createCurrent(P_HMinus, P_GPlus, 10107)
      call createCurrent(P_G0, P_ghAAnti, P_GMinus, P_ghWp, 1618)
      call createCurrent(P_WQMinus, P_WPlus, P_G0, P_GQ0, 6374)
      call createCurrent(P_HPlus, P_ghWm, P_ghWmAnti, P_GMinus, 9476)
      call createCurrent(P_ghA, P_HPlus, P_ghZAnti, P_HMinus, 3103)
      call createCurrent(P_dAnti, P_GMinus, P_u, 858)
      call createCurrent(P_HaQ, P_HPlus, P_ghWm, P_ghZAnti, 1382)
      call createCurrent(P_HQPlus, P_HMinus, P_ghZ, P_ghAAnti, 277)
      call createCurrent(P_HMinus, P_Hh, P_ghWp, P_ghAAnti, 6257)
      call createCurrent(P_g, P_HMinus, P_g, P_GPlus, 10683)
      call createCurrent(P_GQPlus, P_WMinus, P_Hl, P_AQ, 6834)
      call createCurrent(P_A, P_WPlus, P_WMinus, 8783)
      call createCurrent(P_HQMinus, P_GPlus, P_Hl, P_HlQ, 8152)
      call createCurrent(P_Ha, P_ghAAnti, P_HPlus, P_ghWm, 6628)
      call createCurrent(P_GQMinus, P_bAnti, P_t, 1346)
      call createCurrent(P_HMinus, P_Ha, P_ghWmAnti, P_ghZ, 7268)
      call createCurrent(P_WPlus, P_HMinus, P_WMinus, P_GPlus, 9353)
      call createCurrent(P_HMinus, P_ghZAnti, P_HPlus, P_ghA, 8682)
      call createCurrent(P_GPlus, P_Ha, P_HMinus, P_G0, 2292)
      call createCurrent(P_GQPlus, P_ghWmAnti, P_GMinus, P_ghWm, 974)
      call createCurrent(P_WPlus, P_ghA, P_WMinus, P_ghZAnti, 3477)
      call createCurrent(P_ghWp, P_ghZAnti, P_G0, P_HMinus, 9145)
      call createCurrent(P_Hh, P_ePlus, P_eMinus, 7940)
      call createCurrent(P_HhQ, P_HMinus, P_ghZ, P_ghWmAnti, 7846)
      call createCurrent(P_HhQ, P_HMinus, P_Ha, P_GQPlus, 6398)
      call createCurrent(P_Hh, P_A, P_Hl, P_Z, 10544)
      call createCurrent(P_WMinus, P_Ha, P_A, P_HPlus, 1759)
      call createCurrent(P_ghWm, P_GPlus, P_ghZAnti, P_Hh, 7941)
      call createCurrent(P_WPlus, P_A, P_GMinus, P_Hh, 493)
      call createCurrent(P_WPlus, P_ghWm, P_Z, P_ghAAnti, 5050)
      call createCurrent(P_GPlus, P_GPlus, P_HMinus, P_GMinus, 4070)
      call createCurrent(P_G0, P_ghWmAnti, P_ghZ, P_GMinus, 7254)
      call createCurrent(P_GPlus, P_G0, P_G0, P_GMinus, 4495)
      call createCurrent(P_ghA, P_GMinus, P_ghAAnti, P_GPlus, 2173)
      call createCurrent(P_HQPlus, P_HMinus, P_Z, P_AQ, 1781)
      call createCurrent(P_ghWm, P_ghAAnti, P_WPlus, P_Z, 4392)
      call createCurrent(P_ghA, P_GPlus, P_ghWpAnti, P_G0, 3332)
      call createCurrent(P_GQ0, P_Z, P_GPlus, P_WQMinus, 6136)
      call createCurrent(P_HQMinus, P_ghA, P_HPlus, P_ghAAnti, 909)
      call createCurrent(P_GQPlus, P_G0, P_ghA, P_ghWpAnti, 4702)
      call createCurrent(P_ghZ, P_ghAAnti, P_GMinus, P_GQPlus, 7258)
      call createCurrent(P_WMinus, P_ghWp, P_ghAAnti, P_A, 4273)
      call createCurrent(P_GPlus, P_ghZAnti, P_HMinus, P_ghZ, 10463)
      call createCurrent(P_A, P_WMinus, P_ghA, P_ghWmAnti, 1484)
      call createCurrent(P_ghWmAnti, P_ghZ, P_GMinus, P_Hl, 3070)
      call createCurrent(P_ghWpAnti, P_GPlus, P_ghZ, P_HlQ, 92)
      call createCurrent(P_d, P_uAnti, P_GQPlus, 2833)
      call createCurrent(P_ghZ, P_GMinus, P_G0, P_ghWmAnti, 5757)
      call createCurrent(P_GMinus, P_WPlus, P_Ha, P_A, 9031)
      call createCurrent(P_WMinus, P_HPlus, P_Z, P_G0, 9442)
      call createCurrent(P_c, P_Hl, P_cAnti, 4840)
      call createCurrent(P_G0, P_Ha, P_ghWp, P_ghWpAnti, 10222)
      call createCurrent(P_A, P_Ha, P_A, P_G0, 10696)
      call createCurrent(P_Hl, P_GPlus, P_Ha, P_HMinus, 1543)
      call createCurrent(P_WQPlus, P_HMinus, P_Z, P_HaQ, 5964)
      call createCurrent(P_HMinus, P_ghA, P_HPlus, P_ghZAnti, 2136)
      call createCurrent(P_ghWm, P_ghWmAnti, P_Hh, P_Hl, 9101)
      call createCurrent(P_ZQ, P_WPlus, P_ghAAnti, P_ghWm, 3102)
      call createCurrent(P_g, P_g, P_Z, P_g, 10756)
      call createCurrent(P_WQPlus, P_ghA, P_WMinus, P_ghAAnti, 81)
      call createCurrent(P_WMinus, P_WPlus, P_ghZAnti, P_ghZ, 3742)
      call createCurrent(P_Hh, P_HMinus, P_GPlus, P_Ha, 3847)
      call createCurrent(P_Ha, P_tauPlus, P_tauMinus, 5036)
      call createCurrent(P_Hh, P_GMinus, P_ghWp, P_ghAAnti, 4380)
      call createCurrent(P_muPlus, P_nu_mu, P_HMinus, 8790)
      call createCurrent(P_ghA, P_WMinus, P_ghWmAnti, 6632)
      call createCurrent(P_HhQ, P_ghZAnti, P_HMinus, P_ghWp, 7104)
      call createCurrent(P_GMinus, P_Hh, P_ghZ, P_ghWmAnti, 3852)
      call createCurrent(P_g, P_g, P_HMinus, P_GPlus, 10770)
      call createCurrent(P_Z, P_GMinus, P_GPlus, 5356)
      call createCurrent(P_ghAAnti, P_G0, P_GPlus, P_ghWm, 2137)
      call createCurrent(P_WPlus, P_WMinus, P_Z, P_Z, 2432)
      call createCurrent(P_HQPlus, P_HMinus, P_Ha, P_HlQ, 5825)
      call createCurrent(P_GQ0, P_GMinus, P_G0, P_GQPlus, 8076)
      call createCurrent(P_ZQ, P_ghWm, P_WPlus, P_ghAAnti, 6959)
      call createCurrent(P_WQMinus, P_nu_mu, P_muPlus, 404)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_Ha, 9917)
      call createCurrent(P_ghWpAnti, P_ghZ, P_GPlus, P_HhQ, 5104)
      call createCurrent(P_HMinus, P_Hl, P_WPlus, P_A, 7189)
      call createCurrent(P_ghZAnti, P_G0, P_ghZ, P_G0, 6689)
      call createCurrent(P_WMinus, P_WPlus, P_Hh, P_Hl, 8911)
      call createCurrent(P_Hh, P_Hl, P_Hl, 6183)
      call createCurrent(P_Hh, P_ghZAnti, P_ghWp, P_HMinus, 8441)
      call createCurrent(P_AQ, P_bAnti, P_b, 6534)
      call createCurrent(P_Hl, P_ghWpAnti, P_ghZ, P_GPlus, 1695)
      call createCurrent(P_GPlus, P_g, P_GMinus, P_g, 10538)
      call createCurrent(P_WQPlus, P_WMinus, P_ghA, P_ghZAnti, 7297)
      call createCurrent(P_HPlus, P_Hl, P_GMinus, P_Ha, 3316)
      call createCurrent(P_ghWpAnti, P_ghZ, P_HPlus, P_Hh, 1288)
      call createCurrent(P_ghAAnti, P_G0, P_ghWm, P_GPlus, 6307)
      call createCurrent(P_WMinus, P_Hl, P_A, P_HPlus, 1120)
      call createCurrent(P_Z, P_Ha, P_G0, P_Z, 10081)
      call createCurrent(P_g, P_A, P_Z, P_g, 10772)
      call createCurrent(P_Z, P_WMinus, P_G0, P_HPlus, 10455)
      call createCurrent(P_ghZAnti, P_G0, P_ghWm, P_HPlus, 10014)
      call createCurrent(P_ghZAnti, P_GPlus, P_HMinus, P_ghZ, 9645)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghZ, P_Hl, 7052)
      call createCurrent(P_Hl, P_GPlus, P_ghA, P_ghWpAnti, 3992)
      call createCurrent(P_Hh, P_HMinus, P_HPlus, P_Ha, 8890)
      call createCurrent(P_GQPlus, P_G0, P_HMinus, P_HaQ, 7371)
      call createCurrent(P_ghWm, P_Hl, P_Hl, P_ghWmAnti, 4877)
      call createCurrent(P_HhQ, P_Z, P_GPlus, P_WQMinus, 3342)
      call createCurrent(P_G0, P_Z, P_HPlus, P_WMinus, 9987)
      call createCurrent(P_HPlus, P_GMinus, P_G0, 2717)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_HaQ, 1327)
      call createCurrent(P_ghWp, P_Hl, P_ghWpAnti, 1113)
      call createCurrent(P_ghWmAnti, P_GMinus, P_ghWm, P_HPlus, 9247)
      call createCurrent(P_WPlus, P_A, P_ghA, P_ghWpAnti, 1936)
      call createCurrent(P_GQMinus, P_ghWmAnti, P_Hh, P_ghA, 5339)
      call createCurrent(P_GQPlus, P_Ha, P_HQMinus, 2085)
      call createCurrent(P_ghZ, P_G0, P_ghZAnti, P_Ha, 9800)
      call createCurrent(P_WQMinus, P_HPlus, P_HaQ, 8508)
      call createCurrent(P_GQ0, P_G0, P_HMinus, P_HQPlus, 509)
      call createCurrent(P_HhQ, P_uAnti, P_u, 7253)
      call createCurrent(P_Ha, P_WMinus, P_HPlus, 8358)
      call createCurrent(P_ghZ, P_ghWpAnti, P_HPlus, P_HhQ, 5084)
      call createCurrent(P_HlQ, P_Hh, P_HMinus, P_GQPlus, 5394)
      call createCurrent(P_GPlus, P_ghA, P_HMinus, P_ghAAnti, 8871)
      call createCurrent(P_WMinus, P_ghZ, P_A, P_ghWmAnti, 1761)
      call createCurrent(P_G0, P_GMinus, P_ghZAnti, P_ghWp, 2642)
      call createCurrent(P_HhQ, P_ghZAnti, P_ghZ, 4034)
      call createCurrent(P_GQPlus, P_muMinus, P_nu_muAnti, 814)
      call createCurrent(P_GPlus, P_ghZ, P_ghAAnti, P_HMinus, 10330)
      call createCurrent(P_HMinus, P_A, P_Z, P_GPlus, 9817)
      call createCurrent(P_GPlus, P_GMinus, P_ghWpAnti, P_ghWp, 1825)
      call createCurrent(P_ghWpAnti, P_Hl, P_GPlus, P_ghA, 7289)
      call createCurrent(P_Ha, P_GMinus, P_Hh, P_HPlus, 8495)
      call createCurrent(P_ghWpAnti, P_GPlus, P_Hh, P_ghA, 2650)
      call createCurrent(P_Hl, P_Z, P_HPlus, P_WMinus, 2381)
      call createCurrent(P_ZQ, P_GMinus, P_Hh, P_WQPlus, 6249)
      call createCurrent(P_GQPlus, P_ghA, P_Hl, P_ghWpAnti, 745)
      call createCurrent(P_Hl, P_GPlus, P_ghWm, P_ghZAnti, 3673)
      call createCurrent(P_Z, P_GPlus, P_GMinus, P_Z, 8302)
      call createCurrent(P_sAnti, P_WMinus, P_c, 3718)
      call createCurrent(P_uAnti, P_u, P_G0, 2834)
      call createCurrent(P_GPlus, P_tAnti, P_b, 5239)
      call createCurrent(P_HQPlus, P_GMinus, P_Hh, P_HaQ, 2934)
      call createCurrent(P_GMinus, P_Hl, P_GPlus, P_Hl, 348)
      call createCurrent(P_HMinus, P_ghZ, P_ghWmAnti, P_Ha, 7227)
      call createCurrent(P_WMinus, P_ghAAnti, P_WPlus, P_ghZ, 827)
      call createCurrent(P_AQ, P_WPlus, P_Hh, P_HQMinus, 2211)
      call createCurrent(P_HMinus, P_Ha, P_ghWp, P_ghAAnti, 1042)
      call createCurrent(P_GMinus, P_ghWp, P_ghWpAnti, P_GPlus, 2793)
      call createCurrent(P_Hh, P_GPlus, P_A, P_WMinus, 6001)
      call createCurrent(P_GPlus, P_G0, P_ghZAnti, P_ghWm, 6508)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghZ, P_GPlus, 8957)
      call createCurrent(P_GPlus, P_Ha, P_Z, P_WMinus, 9269)
      call createCurrent(P_WMinus, P_WPlus, P_ghWp, P_ghWpAnti, 4328)
      call createCurrent(P_G0, P_GPlus, P_GMinus, 9356)
      call createCurrent(P_Z, P_GPlus, P_G0, P_WMinus, 6066)
      call createCurrent(P_ghA, P_Hh, P_GPlus, P_ghWpAnti, 1794)
      call createCurrent(P_Hh, P_HMinus, P_ghWp, P_ghAAnti, 6876)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghZ, P_HQMinus, 961)
      call createCurrent(P_ghA, P_G0, P_HPlus, P_ghWpAnti, 9062)
      call createCurrent(P_tAnti, P_t, P_Hl, 408)
      call createCurrent(P_GQMinus, P_A, P_GPlus, P_ZQ, 6715)
      call createCurrent(P_GMinus, P_HPlus, P_A, P_Z, 8918)
      call createCurrent(P_HPlus, P_HMinus, P_ghA, P_ghZAnti, 7003)
      call createCurrent(P_GQMinus, P_HPlus, P_G0, P_HlQ, 3085)
      call createCurrent(P_GPlus, P_HMinus, P_Z, P_A, 10082)
      call createCurrent(P_Hl, P_GMinus, P_G0, P_HPlus, 2303)
      call createCurrent(P_Ha, P_Ha, P_Hl, P_Hh, 287)
      call createCurrent(P_ghZAnti, P_ghWm, P_GPlus, P_GQ0, 8559)
      call createCurrent(P_GPlus, P_Hh, P_ghWpAnti, P_ghA, 2918)
      call createCurrent(P_GPlus, P_Hh, P_Z, P_WMinus, 8796)
      call createCurrent(P_GQMinus, P_GPlus, P_G0, P_GQ0, 6730)
      call createCurrent(P_WPlus, P_Ha, P_Z, P_GMinus, 10274)
      call createCurrent(P_GQMinus, P_G0, P_G0, P_GQPlus, 8724)
      call createCurrent(P_Hh, P_ghWmAnti, P_Hh, P_ghWm, 4887)
      call createCurrent(P_GPlus, P_HMinus, P_ghZAnti, P_ghZ, 9674)
      call createCurrent(P_ghWm, P_Hl, P_ghWmAnti, P_Hh, 9820)
      call createCurrent(P_HQPlus, P_ghWpAnti, P_Ha, P_ghZ, 6345)
      call createCurrent(P_Hl, P_WPlus, P_Z, P_GMinus, 6346)
      call createCurrent(P_HPlus, P_A, P_WMinus, P_Hh, 2049)
      call createCurrent(P_Z, P_G0, P_Ha, P_Z, 8959)
      call createCurrent(P_Hh, P_A, P_Z, P_Hl, 10776)
      call createCurrent(P_Z, P_Ha, P_Ha, P_Z, 3895)
      call createCurrent(P_uAnti, P_u, P_ZQ, 4066)
      call createCurrent(P_WPlus, P_muMinus, P_nu_muAnti, 3519)
      call createCurrent(P_ghZAnti, P_HPlus, P_HMinus, P_ghZ, 1887)
      call createCurrent(P_ZQ, P_Ha, P_HMinus, P_WQPlus, 1689)
      call createCurrent(P_A, P_ghWmAnti, P_A, P_ghWm, 5316)
      call createCurrent(P_HMinus, P_Hl, P_Hh, P_HPlus, 2995)
      call createCurrent(P_ZQ, P_GPlus, P_WMinus, P_HlQ, 4880)
      call createCurrent(P_GPlus, P_ghWm, P_G0, P_ghAAnti, 26)
      call createCurrent(P_Hl, P_Hl, P_ghZ, P_ghZAnti, 1913)
      call createCurrent(P_HhQ, P_HPlus, P_GMinus, P_HhQ, 1047)
      call createCurrent(P_GMinus, P_GPlus, P_A, 3989)
      call createCurrent(P_HhQ, P_GMinus, P_HPlus, P_HaQ, 7702)
      call createCurrent(P_HQMinus, P_Hl, P_HPlus, P_GQ0, 8704)
      call createCurrent(P_HMinus, P_GPlus, P_Ha, 388)
      call createCurrent(P_HlQ, P_t, P_tAnti, 365)
      call createCurrent(P_HMinus, P_WPlus, P_A, P_Ha, 3028)
      call createCurrent(P_ghWpAnti, P_Ha, P_ghA, P_HPlus, 846)
      call createCurrent(P_HPlus, P_HMinus, P_Hl, P_Hh, 8770)
      call createCurrent(P_GQMinus, P_Hh, P_GPlus, P_GQ0, 5314)
      call createCurrent(P_Hh, P_HMinus, P_Z, P_WPlus, 8596)
      call createCurrent(P_Hl, P_tAnti, P_t, 2393)
      call createCurrent(P_ghZ, P_ghWpAnti, P_G0, P_HPlus, 10053)
      call createCurrent(P_g, P_g, P_Z, P_Z, 10531)
      call createCurrent(P_GQ0, P_Hl, P_GQ0, 7361)
      call createCurrent(P_Ha, P_ghZ, P_HMinus, P_ghWmAnti, 8440)
      call createCurrent(P_GQMinus, P_Hl, P_ghWmAnti, P_ghZ, 3484)
      call createCurrent(P_HhQ, P_HPlus, P_Ha, P_GQMinus, 21)
      call createCurrent(P_ghA, P_ghWpAnti, P_HPlus, 8701)
      call createCurrent(P_HMinus, P_HPlus, P_HPlus, P_GMinus, 549)
      call createCurrent(P_GQ0, P_Hh, P_GMinus, P_GQPlus, 7793)
      call createCurrent(P_WPlus, P_GMinus, P_Hl, P_A, 4788)
      call createCurrent(P_GMinus, P_g, P_g, P_HPlus, 10628)
      call createCurrent(P_Z, P_sAnti, P_s, 4403)
      call createCurrent(P_GPlus, P_G0, P_GMinus, P_Hh, 8864)
      call createCurrent(P_HPlus, P_GMinus, P_HPlus, P_HMinus, 2380)
      call createCurrent(P_ghA, P_Hl, P_ghWpAnti, P_GPlus, 5093)
      call createCurrent(P_WQMinus, P_WMinus, P_ghWmAnti, P_ghWp, 1968)
      call createCurrent(P_GMinus, P_ghA, P_ghAAnti, P_GPlus, 4812)
      call createCurrent(P_G0, P_G0, P_Hh, P_Ha, 10577)
      call createCurrent(P_ghWpAnti, P_ghWp, P_GPlus, P_GQMinus, 5536)
      call createCurrent(P_AQ, P_HPlus, P_Hh, P_WQMinus, 1222)
      call createCurrent(P_Hl, P_Ha, P_G0, P_G0, 10726)
      call createCurrent(P_ghZAnti, P_ghZ, P_GMinus, P_HPlus, 8934)
      call createCurrent(P_ghWp, P_Ha, P_ghZAnti, P_HMinus, 397)
      call createCurrent(P_G0, P_HMinus, P_HPlus, P_Ha, 1912)
      call createCurrent(P_ghWm, P_ghAAnti, P_WPlus, P_AQ, 6994)
      call createCurrent(P_HQPlus, P_HMinus, P_HPlus, P_HQMinus, 3577)
      call createCurrent(P_Hl, P_Z, P_WMinus, P_HPlus, 7028)
      call createCurrent(P_ghAAnti, P_ghWm, P_HPlus, 1320)
      call createCurrent(P_Hl, P_Hh, P_HMinus, P_GPlus, 5734)
      call createCurrent(P_WPlus, P_Hh, P_Z, P_HMinus, 3513)
      call createCurrent(P_ghWm, P_HPlus, P_Hl, P_ghZAnti, 1547)
      call createCurrent(P_nu_muAnti, P_Z, P_nu_mu, 4025)
      call createCurrent(P_ghWm, P_A, P_ghWmAnti, P_ZQ, 4648)
      call createCurrent(P_HPlus, P_ghZAnti, P_HMinus, P_ghZ, 5714)
      call createCurrent(P_WMinus, P_Z, P_GPlus, P_Hl, 2315)
      call createCurrent(P_GPlus, P_nu_eAnti, P_eMinus, 3834)
      call createCurrent(P_HPlus, P_Hl, P_Hl, P_GMinus, 2811)
      call createCurrent(P_HQMinus, P_HPlus, P_ghWpAnti, P_ghWp, 1413)
      call createCurrent(P_HhQ, P_t, P_tAnti, 24)
      call createCurrent(P_WMinus, P_G0, P_GPlus, P_A, 1577)
      call createCurrent(P_Ha, P_HPlus, P_ghZ, P_ghWpAnti, 210)
      call createCurrent(P_A, P_Z, P_G0, P_G0, 9523)
      call createCurrent(P_GQ0, P_GPlus, P_Ha, P_HQMinus, 1406)
      call createCurrent(P_Ha, P_GMinus, P_GPlus, P_G0, 3728)
      call createCurrent(P_Ha, P_Hl, P_Hl, P_G0, 8454)
      call createCurrent(P_WMinus, P_Z, P_G0, P_GPlus, 3936)
      call createCurrent(P_GQPlus, P_Hl, P_Hh, P_HQMinus, 8297)
      call createCurrent(P_GMinus, P_Z, P_G0, P_WPlus, 6283)
      call createCurrent(P_HhQ, P_HMinus, P_A, P_WQPlus, 6232)
      call createCurrent(P_HQMinus, P_HPlus, P_WMinus, P_WQPlus, 1892)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghA, P_HQMinus, 5048)
      call createCurrent(P_GMinus, P_A, P_HPlus, 9677)
      call createCurrent(P_ghWmAnti, P_Ha, P_ghWm, P_HhQ, 2694)
      call createCurrent(P_WMinus, P_GPlus, P_WPlus, P_HMinus, 9605)
      call createCurrent(P_ghWp, P_Ha, P_GMinus, P_ghZAnti, 9530)
      call createCurrent(P_sAnti, P_Hl, P_s, 5140)
      call createCurrent(P_ghAAnti, P_Ha, P_GMinus, P_ghWp, 10001)
      call createCurrent(P_HaQ, P_HMinus, P_ghZAnti, P_ghWp, 5501)
      call createCurrent(P_ghWm, P_Hl, P_ghAAnti, P_HPlus, 2723)
      call createCurrent(P_WQMinus, P_HPlus, P_Z, P_HhQ, 4366)
      call createCurrent(P_HaQ, P_WPlus, P_Hh, P_WQMinus, 3245)
      call createCurrent(P_A, P_HMinus, P_GPlus, P_A, 9112)
      call createCurrent(P_WMinus, P_WPlus, 9896)
      call createCurrent(P_HPlus, P_WMinus, P_GMinus, P_WPlus, 10100)
      call createCurrent(P_GMinus, P_Hh, P_ghA, P_ghWmAnti, 4915)
      call createCurrent(P_Ha, P_GMinus, P_WPlus, 10288)
      call createCurrent(P_HhQ, P_ghWm, P_Hh, P_ghWmAnti, 5250)
      call createCurrent(P_A, P_WMinus, P_Ha, P_HPlus, 5502)
      call createCurrent(P_Hh, P_GMinus, P_G0, P_GPlus, 10197)
      call createCurrent(P_WPlus, P_Hh, P_Hl, P_WMinus, 10123)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hh, P_GQPlus, 5552)
      call createCurrent(P_ghWp, P_ghAAnti, P_G0, P_GMinus, 2861)
      call createCurrent(P_HPlus, P_Hh, P_GMinus, P_Ha, 7585)
      call createCurrent(P_Hh, P_Hl, P_ghWpAnti, P_ghWp, 8878)
      call createCurrent(P_ghZAnti, P_ghWm, P_WPlus, P_ZQ, 4260)
      call createCurrent(P_HhQ, P_Ha, P_Hh, P_GQ0, 4751)
      call createCurrent(P_ghA, P_ghWmAnti, P_Ha, P_HMinus, 3585)
      call createCurrent(P_d, P_dAnti, P_Hh, 7023)
      call createCurrent(P_Hl, P_WPlus, P_GMinus, P_A, 6642)
      call createCurrent(P_muMinus, P_muPlus, P_GQ0, 4922)
      call createCurrent(P_GMinus, P_ghWmAnti, P_ghA, P_Hh, 1337)
      call createCurrent(P_ghWm, P_Hh, P_ghZAnti, P_GQPlus, 3881)
      call createCurrent(P_eMinus, P_G0, P_ePlus, 6385)
      call createCurrent(P_G0, P_HMinus, P_GPlus, P_Hh, 111)
      call createCurrent(P_HhQ, P_GMinus, P_Z, P_WQPlus, 8772)
      call createCurrent(P_WQPlus, P_Hl, P_HQMinus, 7375)
      call createCurrent(P_HlQ, P_Hl, P_WPlus, P_WQMinus, 2262)
      call createCurrent(P_ghWpAnti, P_ghWm, P_WPlus, P_WQPlus, 1675)
      call createCurrent(P_HhQ, P_Hh, P_Ha, P_GQ0, 37)
      call createCurrent(P_GQPlus, P_Z, P_Hl, P_WQMinus, 3632)
      call createCurrent(P_c, P_A, P_cAnti, 2842)
      call createCurrent(P_sAnti, P_s, P_HaQ, 5851)
      call createCurrent(P_GMinus, P_G0, P_ghZ, P_ghWmAnti, 2077)
      call createCurrent(P_ghWpAnti, P_ghA, P_HPlus, 1438)
      call createCurrent(P_tauPlus, P_Z, P_tauMinus, 6481)
      call createCurrent(P_GQMinus, P_ghWp, P_Hh, P_ghZAnti, 2545)
      call createCurrent(P_Ha, P_G0, P_Hl, 8804)
      call createCurrent(P_GPlus, P_GMinus, P_G0, P_G0, 1504)
      call createCurrent(P_A, P_GMinus, P_Hl, P_WPlus, 4763)
      call createCurrent(P_c, P_Hh, P_cAnti, 3521)
      call createCurrent(P_GQPlus, P_Z, P_Z, P_GQMinus, 630)
      call createCurrent(P_Hh, P_ghWmAnti, P_HMinus, P_ghA, 6927)
      call createCurrent(P_A, P_WPlus, P_ghZ, P_ghWpAnti, 2418)
      call createCurrent(P_GQPlus, P_ghAAnti, P_GMinus, P_ghZ, 5911)
      call createCurrent(P_HQMinus, P_Hl, P_GPlus, P_HlQ, 7467)
      call createCurrent(P_Ha, P_Hl, P_HPlus, P_GMinus, 696)
      call createCurrent(P_GQPlus, P_s, P_cAnti, 4797)
      call createCurrent(P_ghWp, P_Z, P_ghWpAnti, P_Z, 2384)
      call createCurrent(P_ghWm, P_HMinus, P_ghWmAnti, P_HPlus, 571)
      call createCurrent(P_t, P_tAnti, P_Z, 545)
      call createCurrent(P_WQPlus, P_HPlus, P_HMinus, P_WQMinus, 8414)
      call createCurrent(P_G0, P_GPlus, P_ghA, P_ghWpAnti, 1310)
      call createCurrent(P_ghZ, P_G0, P_HPlus, P_ghWpAnti, 9393)
      call createCurrent(P_nu_tau, P_tauPlus, P_WMinus, 279)
      call createCurrent(P_ghA, P_GMinus, P_GPlus, P_ghZAnti, 3660)
      call createCurrent(P_Hl, P_Hl, P_HPlus, P_HMinus, 6048)
      call createCurrent(P_ZQ, P_WPlus, P_WMinus, P_AQ, 5031)
      call createCurrent(P_GQPlus, P_G0, P_Ha, P_GQMinus, 7701)
      call createCurrent(P_Ha, P_WPlus, P_A, P_HMinus, 7788)
      call createCurrent(P_ghWm, P_ghZAnti, P_G0, P_HPlus, 9497)
      call createCurrent(P_WMinus, P_GPlus, P_G0, P_Z, 5191)
      call createCurrent(P_GQ0, P_Hh, P_GQ0, 4501)
      call createCurrent(P_WQMinus, P_HPlus, P_Hl, P_ZQ, 390)
      call createCurrent(P_GQPlus, P_Hh, P_HMinus, P_HhQ, 5162)
      call createCurrent(P_Z, P_Hh, P_A, 9795)
      call createCurrent(P_A, P_muMinus, P_muPlus, 53)
      call createCurrent(P_HQMinus, P_GPlus, P_Ha, P_HaQ, 756)
      call createCurrent(P_WMinus, P_A, P_HPlus, P_G0, 10004)
      call createCurrent(P_HMinus, P_Hh, P_ghZAnti, P_ghWp, 1150)
      call createCurrent(P_WMinus, P_Z, P_WPlus, 8681)
      call createCurrent(P_AQ, P_HPlus, P_Ha, P_WQMinus, 8451)
      call createCurrent(P_ghA, P_WMinus, P_ghAAnti, P_WPlus, 3547)
      call createCurrent(P_Hl, P_ghAAnti, P_ghWp, P_GMinus, 8753)
      call createCurrent(P_GPlus, P_Hh, P_ghZ, P_ghWpAnti, 7822)
      call createCurrent(P_HhQ, P_Ha, P_WMinus, P_WQPlus, 4033)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, P_GQ0, 7190)
      call createCurrent(P_G0, P_WMinus, P_HPlus, P_A, 9230)
      call createCurrent(P_Hh, P_Hh, P_Hh, 3485)
      call createCurrent(P_HPlus, P_HMinus, P_Ha, 10338)
      call createCurrent(P_HPlus, P_muMinus, P_nu_muAnti, 8548)
      call createCurrent(P_HMinus, P_ghWmAnti, P_ghZ, P_G0, 9323)
      call createCurrent(P_WPlus, P_ghZ, P_WMinus, P_ghAAnti, 3944)
      call createCurrent(P_ghA, P_G0, P_GMinus, P_ghWmAnti, 5037)
      call createCurrent(P_ghZAnti, P_ghWp, P_Ha, P_HQMinus, 7589)
      call createCurrent(P_HMinus, P_A, P_WPlus, P_G0, 9972)
      call createCurrent(P_WPlus, P_WMinus, P_ghZAnti, P_ghZ, 2930)
      call createCurrent(P_ghZ, P_Ha, P_ghZAnti, P_HaQ, 5108)
      call createCurrent(P_HaQ, P_GMinus, P_Hl, P_GQPlus, 1110)
      call createCurrent(P_ghZAnti, P_ghWm, P_Hl, P_GPlus, 8775)
      call createCurrent(P_ghZAnti, P_ghZ, P_GPlus, P_GMinus, 5973)
      call createCurrent(P_WQPlus, P_WMinus, P_G0, P_GQ0, 8827)
      call createCurrent(P_G0, P_GPlus, P_HMinus, P_Hl, 510)
      call createCurrent(P_gQ, P_cAnti, P_c, 2541)
      call createCurrent(P_ghWp, P_ghZAnti, P_HMinus, P_Hh, 1844)
      call createCurrent(P_GQ0, P_Hh, P_HPlus, P_HQMinus, 3127)
      call createCurrent(P_HlQ, P_WPlus, P_Hl, P_WQMinus, 6211)
      call createCurrent(P_HMinus, P_ghA, P_ghAAnti, P_HPlus, 5726)
      call createCurrent(P_ghG, P_ghGAnti, P_g, 6536)
      call createCurrent(P_Ha, P_ghAAnti, P_ghWp, P_HMinus, 6814)
      call createCurrent(P_ghZ, P_ghWmAnti, P_HMinus, 5075)
      call createCurrent(P_s, P_sAnti, P_G0, 7377)
      call createCurrent(P_ghZAnti, P_Ha, P_Ha, P_ghZ, 76)
      call createCurrent(P_ghZ, P_ghAAnti, P_WMinus, P_WQPlus, 117)
      call createCurrent(P_HMinus, P_Hl, P_Hh, P_GPlus, 5192)
      call createCurrent(P_HlQ, P_GMinus, P_G0, P_GQPlus, 2410)
      call createCurrent(P_ghZAnti, P_GPlus, P_ghWm, 1405)
      call createCurrent(P_ghWmAnti, P_ghWm, P_Hl, 5525)
      call createCurrent(P_ghWp, P_HMinus, P_GPlus, P_ghWpAnti, 10375)
      call createCurrent(P_Hh, P_A, P_Ha, 9882)
      call createCurrent(P_nu_muAnti, P_WPlus, P_muMinus, 5848)
      call createCurrent(P_ZQ, P_G0, P_GMinus, P_WQPlus, 2962)
      call createCurrent(P_WPlus, P_A, P_WMinus, P_Z, 6121)
      call createCurrent(P_HhQ, P_Hl, P_GMinus, P_GQPlus, 3277)
      call createCurrent(P_G0, P_ghWmAnti, P_GMinus, P_ghZ, 5992)
      call createCurrent(P_Hl, P_HMinus, P_GPlus, P_Hh, 4758)
      call createCurrent(P_HaQ, P_WPlus, P_Ha, P_WQMinus, 2447)
      call createCurrent(P_HMinus, P_Hh, P_G0, P_GPlus, 4022)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghA, 7835)
      call createCurrent(P_HPlus, P_ghAAnti, P_ghWm, P_Ha, 4054)
      call createCurrent(P_ghAAnti, P_ghWm, P_WPlus, P_ZQ, 2686)
      call createCurrent(P_HhQ, P_GPlus, P_HMinus, P_GQ0, 1004)
      call createCurrent(P_Hh, P_ghWpAnti, P_ghA, P_GPlus, 1525)
      call createCurrent(P_WMinus, P_ghA, P_ghAAnti, P_WPlus, 677)
      call createCurrent(P_GQPlus, P_Hh, P_ghZAnti, P_ghWm, 6009)
      call createCurrent(P_WPlus, P_ghZAnti, P_Z, P_ghWm, 7300)
      call createCurrent(P_HQPlus, P_HPlus, P_GMinus, P_HQMinus, 6259)
      call createCurrent(P_HhQ, P_HMinus, P_GPlus, P_GQ0, 4526)
      call createCurrent(P_WQPlus, P_GMinus, P_Z, P_HhQ, 7173)
      call createCurrent(P_GMinus, P_Ha, P_Ha, P_GPlus, 1996)
      call createCurrent(P_HQPlus, P_GMinus, P_Ha, P_HlQ, 8740)
      call createCurrent(P_A, P_HMinus, P_Hh, P_WPlus, 6084)
      call createCurrent(P_ZQ, P_Hh, P_HMinus, P_WQPlus, 5029)
      call createCurrent(P_ghWm, P_ghZAnti, P_WPlus, P_AQ, 6439)
      call createCurrent(P_GMinus, P_G0, P_ghA, P_ghWmAnti, 4806)
      call createCurrent(P_GMinus, P_Hh, P_G0, P_GPlus, 8896)
      call createCurrent(P_HPlus, P_HMinus, P_ghZ, P_ghAAnti, 2429)
      call createCurrent(P_WQPlus, P_ghZ, P_A, P_ghWpAnti, 3526)
      call createCurrent(P_d, P_dAnti, P_Ha, 4269)
      call createCurrent(P_Ha, P_GPlus, P_Ha, P_GMinus, 5673)
      call createCurrent(P_HMinus, P_GPlus, P_Hh, 6523)
      call createCurrent(P_Ha, P_A, P_GMinus, P_WPlus, 9569)
      call createCurrent(P_HQMinus, P_Z, P_HPlus, P_ZQ, 1898)
      call createCurrent(P_ghZ, P_ghWpAnti, P_Hl, P_HPlus, 4370)
      call createCurrent(P_Ha, P_HMinus, P_Hh, P_GPlus, 6884)
      call createCurrent(P_ghA, P_GMinus, P_ghWmAnti, 7425)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_HlQ, 6765)
      call createCurrent(P_HQPlus, P_ghWm, P_HMinus, P_ghWmAnti, 5294)
      call createCurrent(P_WQMinus, P_GPlus, P_A, P_HlQ, 5186)
      call createCurrent(P_WPlus, P_A, P_HMinus, P_Hl, 1220)
      call createCurrent(P_GQ0, P_WMinus, P_Z, P_GQPlus, 1155)
      call createCurrent(P_AQ, P_ghWp, P_WMinus, P_ghZAnti, 1242)
      call createCurrent(P_Hl, P_ghWm, P_ghZAnti, P_GPlus, 3067)
      call createCurrent(P_ghWm, P_HMinus, P_GPlus, P_ghWmAnti, 9945)
      call createCurrent(P_HQMinus, P_GMinus, P_GPlus, P_GQPlus, 6600)
      call createCurrent(P_Ha, P_ghWpAnti, P_ghA, P_GPlus, 10358)
      call createCurrent(P_HhQ, P_G0, P_WMinus, P_WQPlus, 8499)
      call createCurrent(P_ghAAnti, P_GMinus, P_HPlus, P_ghZ, 9653)
      call createCurrent(P_AQ, P_cAnti, P_c, 6141)
      call createCurrent(P_HhQ, P_HPlus, P_ghWm, P_ghAAnti, 6888)
      call createCurrent(P_HhQ, P_ghWmAnti, P_ghWm, 2522)
      call createCurrent(P_Hl, P_ghWm, P_GPlus, P_ghZAnti, 1411)
      call createCurrent(P_GQ0, P_ghWp, P_G0, P_ghWpAnti, 5929)
      call createCurrent(P_HhQ, P_Hl, P_Hh, P_HlQ, 2803)
      call createCurrent(P_AQ, P_dAnti, P_d, 5790)
      call createCurrent(P_GMinus, P_ghA, P_Hh, P_ghWmAnti, 3261)
      call createCurrent(P_WQMinus, P_Hh, P_WQPlus, 1835)
      call createCurrent(P_ghA, P_ghWpAnti, P_Ha, P_GPlus, 9921)
      call createCurrent(P_GQPlus, P_WMinus, P_GQ0, 2323)
      call createCurrent(P_ghAAnti, P_ghWp, P_GMinus, P_G0, 3859)
      call createCurrent(P_GMinus, P_Z, P_Z, P_HPlus, 10121)
      call createCurrent(P_ZQ, P_Z, P_Hl, P_HlQ, 7022)
      call createCurrent(P_HMinus, P_ghWp, P_Ha, P_ghAAnti, 4563)
      call createCurrent(P_GMinus, P_Hl, P_WPlus, P_Z, 4901)
      call createCurrent(P_HlQ, P_ghWm, P_Ha, P_ghWmAnti, 8052)
      call createCurrent(P_GPlus, P_Hh, P_HMinus, P_Hl, 4184)
      call createCurrent(P_Ha, P_GMinus, P_HPlus, P_Ha, 3995)
      call createCurrent(P_ghWm, P_GPlus, P_Hl, P_ghZAnti, 6220)
      call createCurrent(P_HQMinus, P_Z, P_Hh, P_WQPlus, 4484)
      call createCurrent(P_G0, P_ghZ, P_GMinus, P_ghWmAnti, 3078)
      call createCurrent(P_HMinus, P_ghZ, P_Hl, P_ghWmAnti, 3571)
      call createCurrent(P_Z, P_G0, P_WPlus, P_HMinus, 10144)
      call createCurrent(P_GQ0, P_dAnti, P_d, 2569)
      call createCurrent(P_WMinus, P_GPlus, P_HMinus, P_WPlus, 9839)
      call createCurrent(P_Hl, P_GPlus, P_Z, P_WMinus, 5119)
      call createCurrent(P_Hh, P_ghWpAnti, P_HPlus, P_ghA, 8595)
      call createCurrent(P_HPlus, P_WMinus, P_A, P_Ha, 6597)
      call createCurrent(P_ghZAnti, P_HPlus, P_ghWm, P_Ha, 5376)
      call createCurrent(P_ghWpAnti, P_HPlus, P_ghZ, P_Ha, 908)
      call createCurrent(P_ghWp, P_ghAAnti, P_Ha, P_HMinus, 8235)
      call createCurrent(P_HQPlus, P_A, P_WMinus, P_HlQ, 6454)
      call createCurrent(P_Hh, P_Z, P_Ha, 5152)
      call createCurrent(P_HlQ, P_HMinus, P_Hl, P_GQPlus, 6749)
      call createCurrent(P_HPlus, P_HMinus, P_Hh, 2329)
      call createCurrent(P_ghZAnti, P_Ha, P_GPlus, P_ghWm, 9225)
      call createCurrent(P_Ha, P_GPlus, P_ghWpAnti, P_ghZ, 10065)
      call createCurrent(P_HPlus, P_GMinus, P_Hh, P_G0, 2095)
      call createCurrent(P_GMinus, P_ghA, P_HPlus, P_ghZAnti, 9845)
      call createCurrent(P_GQMinus, P_G0, P_Ha, P_GQPlus, 497)
      call createCurrent(P_HaQ, P_ePlus, P_eMinus, 3826)
      call createCurrent(P_bAnti, P_GMinus, P_t, 5201)
      call createCurrent(P_Ha, P_GMinus, P_HPlus, P_G0, 2812)
      call createCurrent(P_G0, P_HMinus, P_ghWmAnti, P_ghZ, 10102)
      call createCurrent(P_HlQ, P_G0, P_Hl, P_HaQ, 517)
      call createCurrent(P_ghZAnti, P_Hl, P_HPlus, P_ghWm, 4288)
      call createCurrent(P_WQMinus, P_HPlus, P_Z, P_HaQ, 4729)
      call createCurrent(P_A, P_Hl, P_G0, 9928)
      call createCurrent(P_ghWmAnti, P_A, P_ghWm, P_ZQ, 3538)
      call createCurrent(P_ghWpAnti, P_Hl, P_ghWp, P_Hl, 7473)
      call createCurrent(P_ghWpAnti, P_HPlus, P_G0, P_ghA, 9993)
      call createCurrent(P_HQPlus, P_GMinus, P_Ha, P_HhQ, 3489)
      call createCurrent(P_HMinus, P_Hl, P_HPlus, P_Hh, 7690)
      call createCurrent(P_Hl, P_ePlus, P_eMinus, 6671)
      call createCurrent(P_WQPlus, P_Hh, P_WMinus, P_GQ0, 4593)
      call createCurrent(P_HlQ, P_ghWmAnti, P_Hl, P_ghWm, 7730)
      call createCurrent(P_Ha, P_GMinus, P_HPlus, P_Hh, 4739)
      call createCurrent(P_WPlus, P_Hl, P_WMinus, P_Hl, 114)
      call createCurrent(P_HPlus, P_HMinus, 9199)
      call createCurrent(P_HPlus, P_HMinus, P_Ha, P_Ha, 8383)
      call createCurrent(P_HPlus, P_ghZ, P_HMinus, P_ghZAnti, 1037)
      call createCurrent(P_Hl, P_WPlus, P_WMinus, P_Hh, 10452)
      call createCurrent(P_ghZ, P_ghAAnti, P_GMinus, P_GPlus, 461)
      call createCurrent(P_ghWm, P_HPlus, P_ghZAnti, P_Hh, 6980)
      call createCurrent(P_WQPlus, P_WMinus, P_Ha, P_HaQ, 4824)
      call createCurrent(P_A, P_G0, P_GMinus, P_WPlus, 4994)
      call createCurrent(P_Ha, P_GMinus, P_ghZAnti, P_ghWp, 9155)
      call createCurrent(P_tauPlus, P_tauMinus, P_AQ, 6745)
      call createCurrent(P_GPlus, P_Hl, P_Hh, P_HMinus, 7479)
      call createCurrent(P_ghZAnti, P_ghZ, P_Hl, P_HlQ, 1023)
      call createCurrent(P_HhQ, P_WMinus, P_WPlus, P_HhQ, 5023)
      call createCurrent(P_WMinus, P_WPlus, P_Hh, P_Hh, 5308)
      call createCurrent(P_G0, P_WPlus, P_GMinus, 6941)
      call createCurrent(P_ghZAnti, P_HMinus, P_ghZ, P_GPlus, 8963)
      call createCurrent(P_GQMinus, P_HMinus, P_HPlus, P_GQPlus, 2246)
      call createCurrent(P_g, P_WPlus, P_g, P_WMinus, 10755)
      call createCurrent(P_HPlus, P_Hh, P_HMinus, P_G0, 10250)
      call createCurrent(P_ghZ, P_ghWmAnti, P_Ha, P_HQMinus, 6092)
      call createCurrent(P_Hl, P_Z, P_Hl, P_A, 9264)
      call createCurrent(P_HQPlus, P_WMinus, P_Ha, P_AQ, 1468)
      call createCurrent(P_ghZ, P_ghZAnti, P_WPlus, P_WQMinus, 3889)
      call createCurrent(P_HMinus, P_ghAAnti, P_ghA, P_GPlus, 9768)
      call createCurrent(P_Hl, P_HMinus, P_HPlus, P_Hl, 8340)
      call createCurrent(P_A, P_A, 9414)
      call createCurrent(P_ghWmAnti, P_G0, P_ghWm, P_HlQ, 3617)
      call createCurrent(P_cAnti, P_c, P_HlQ, 2128)
      call createCurrent(P_HhQ, P_HMinus, P_ghAAnti, P_ghWp, 614)
      call createCurrent(P_GQPlus, P_Hh, P_ghA, P_ghWpAnti, 1975)
      call createCurrent(P_GQPlus, P_WPlus, P_GMinus, P_WQMinus, 5817)
      call createCurrent(P_ghWmAnti, P_Z, P_ghWm, P_AQ, 4228)
      call createCurrent(P_GPlus, P_WMinus, P_Z, P_Ha, 9372)
      call createCurrent(P_ghWpAnti, P_ghA, P_Ha, P_GPlus, 10343)
      call createCurrent(P_HhQ, P_ghWp, P_Hh, P_ghWpAnti, 4056)
      call createCurrent(P_tAnti, P_g, P_t, 1090)
      call createCurrent(P_eMinus, P_ePlus, P_ZQ, 6199)
      call createCurrent(P_WMinus, P_ghZAnti, P_WPlus, P_ghZ, 7982)
      call createCurrent(P_Hl, P_GPlus, P_ghZ, P_ghWpAnti, 1843)
      call createCurrent(P_Ha, P_HPlus, P_Hl, P_HMinus, 9044)
      call createCurrent(P_ghZAnti, P_WMinus, P_ghA, P_WQPlus, 2951)
      call createCurrent(P_ghAAnti, P_ghWp, P_Hh, P_HMinus, 4970)
      call createCurrent(P_ghZ, P_GPlus, P_ghWpAnti, 5334)
      call createCurrent(P_HQMinus, P_G0, P_GPlus, P_HhQ, 2249)
      call createCurrent(P_WQMinus, P_Ha, P_WPlus, P_HaQ, 4110)
      call createCurrent(P_ZQ, P_HPlus, P_Ha, P_WQMinus, 8732)
      call createCurrent(P_HPlus, P_ghWpAnti, P_ghA, P_Ha, 8041)
      call createCurrent(P_cAnti, P_s, P_HPlus, 4859)
      call createCurrent(P_G0, P_HMinus, P_Hh, P_HPlus, 9152)
      call createCurrent(P_c, P_cAnti, P_ZQ, 2976)
      call createCurrent(P_HMinus, P_ghWp, P_ghWpAnti, P_GPlus, 9797)
      call createCurrent(P_HQMinus, P_Ha, P_ghWmAnti, P_ghZ, 2228)
      call createCurrent(P_tAnti, P_HPlus, P_b, 5504)
      call createCurrent(P_Hl, P_s, P_sAnti, 4201)
      call createCurrent(P_Ha, P_GPlus, P_WMinus, P_Z, 8855)
      call createCurrent(P_GQ0, P_Hl, P_Hl, P_HaQ, 5914)
      call createCurrent(P_ghAAnti, P_ghWm, P_WQPlus, 4456)
      call createCurrent(P_A, P_WPlus, P_A, P_WMinus, 7692)
      call createCurrent(P_GPlus, P_ghZ, P_G0, P_ghWpAnti, 7186)
      call createCurrent(P_WPlus, P_Hh, P_A, P_HMinus, 1880)
      call createCurrent(P_Hl, P_Hl, P_ghWpAnti, P_ghWp, 5194)
      call createCurrent(P_nu_eAnti, P_eMinus, P_HPlus, 4329)
      call createCurrent(P_HMinus, P_ghWm, P_GPlus, P_ghWmAnti, 9591)
      call createCurrent(P_ghWpAnti, P_ghWp, P_HMinus, P_GPlus, 9019)
      call createCurrent(P_GQ0, P_HPlus, P_Hh, P_GQMinus, 1015)
      call createCurrent(P_Ha, P_Ha, P_A, P_A, 10660)
      call createCurrent(P_HhQ, P_GMinus, P_G0, P_HQPlus, 1980)
      call createCurrent(P_HQPlus, P_Ha, P_HMinus, P_HhQ, 3984)
      call createCurrent(P_GPlus, P_WMinus, P_Hl, P_Z, 8071)
      call createCurrent(P_Ha, P_Ha, P_ghWpAnti, P_ghWp, 8279)
      call createCurrent(P_ghZAnti, P_WPlus, P_WMinus, P_ghZ, 7836)
      call createCurrent(P_t, P_bAnti, P_WQMinus, 1815)
      call createCurrent(P_Ha, P_HPlus, P_HMinus, P_G0, 3562)
      call createCurrent(P_AQ, P_WPlus, P_WMinus, P_ZQ, 5274)
      call createCurrent(P_ghZAnti, P_ghWp, P_HMinus, P_Ha, 6789)
      call createCurrent(P_G0, P_ghZ, P_ghWmAnti, P_GMinus, 1763)
      call createCurrent(P_G0, P_WMinus, P_Ha, P_WPlus, 9326)
      call createCurrent(P_ghWpAnti, P_ghWp, P_Ha, P_Ha, 2306)
      call createCurrent(P_Hl, P_muPlus, P_muMinus, 6846)
      call createCurrent(P_HhQ, P_HPlus, P_WQMinus, 8442)
      call createCurrent(P_HQPlus, P_Hh, P_HMinus, P_GQ0, 5418)
      call createCurrent(P_ghWm, P_ghZAnti, P_Ha, P_HQPlus, 3608)
      call createCurrent(P_AQ, P_HMinus, P_A, P_HQPlus, 334)
      call createCurrent(P_ghWmAnti, P_Z, P_WMinus, P_ghZ, 2594)
      call createCurrent(P_Hh, P_Hl, P_GMinus, P_GPlus, 1141)
    end subroutine init_vertices_mdl

    subroutine check_vertices_mdl()
      integer(kind=8) :: config
      integer :: res, address
      logical :: found
      call createParticleConfig(P_ghWp, P_Hh, P_ghZAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2208
      if ( res /= 2208) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8258
      if ( res /= 8258) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7529
      if ( res /= 7529) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_b, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",913
      if ( res /= 913) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2667
      if ( res /= 2667) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6486
      if ( res /= 6486) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10298
      if ( res /= 10298) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WPlus, P_ghWpAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8333
      if ( res /= 8333) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5677
      if ( res /= 5677) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9346
      if ( res /= 9346) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9133
      if ( res /= 9133) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8535
      if ( res /= 8535) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1050
      if ( res /= 1050) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10132
      if ( res /= 10132) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8208
      if ( res /= 8208) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6208
      if ( res /= 6208) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3868
      if ( res /= 3868) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4522
      if ( res /= 4522) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10264
      if ( res /= 10264) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10087
      if ( res /= 10087) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghA, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5713
      if ( res /= 5713) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWpAnti, P_HPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",706
      if ( res /= 706) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4807
      if ( res /= 4807) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7063
      if ( res /= 7063) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3667
      if ( res /= 3667) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1022
      if ( res /= 1022) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3943
      if ( res /= 3943) then
        stop
      end if
      call createParticleConfig(P_G0, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",601
      if ( res /= 601) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3777
      if ( res /= 3777) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4261
      if ( res /= 4261) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1941
      if ( res /= 1941) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3259
      if ( res /= 3259) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8715
      if ( res /= 8715) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7989
      if ( res /= 7989) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9543
      if ( res /= 9543) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7810
      if ( res /= 7810) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6820
      if ( res /= 6820) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4497
      if ( res /= 4497) then
        stop
      end if
      call createParticleConfig(P_g, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3128
      if ( res /= 3128) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10175
      if ( res /= 10175) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5811
      if ( res /= 5811) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6917
      if ( res /= 6917) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6751
      if ( res /= 6751) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4561
      if ( res /= 4561) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7062
      if ( res /= 7062) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9544
      if ( res /= 9544) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4611
      if ( res /= 4611) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghWp, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8121
      if ( res /= 8121) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8303
      if ( res /= 8303) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6416
      if ( res /= 6416) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5900
      if ( res /= 5900) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5065
      if ( res /= 5065) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8138
      if ( res /= 8138) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1534
      if ( res /= 1534) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2758
      if ( res /= 2758) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7650
      if ( res /= 7650) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2787
      if ( res /= 2787) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6722
      if ( res /= 6722) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9846
      if ( res /= 9846) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10188
      if ( res /= 10188) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3156
      if ( res /= 3156) then
        stop
      end if
      call createParticleConfig(P_g, P_Hh, P_g, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10580
      if ( res /= 10580) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1146
      if ( res /= 1146) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1965
      if ( res /= 1965) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6714
      if ( res /= 6714) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1192
      if ( res /= 1192) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",900
      if ( res /= 900) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1232
      if ( res /= 1232) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7240
      if ( res /= 7240) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",264
      if ( res /= 264) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4402
      if ( res /= 4402) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2099
      if ( res /= 2099) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7821
      if ( res /= 7821) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",507
      if ( res /= 507) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",69
      if ( res /= 69) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghZ, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4267
      if ( res /= 4267) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4756
      if ( res /= 4756) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5173
      if ( res /= 5173) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2524
      if ( res /= 2524) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9726
      if ( res /= 9726) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6490
      if ( res /= 6490) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10172
      if ( res /= 10172) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7346
      if ( res /= 7346) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5345
      if ( res /= 5345) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8589
      if ( res /= 8589) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1951
      if ( res /= 1951) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1373
      if ( res /= 1373) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10520
      if ( res /= 10520) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5780
      if ( res /= 5780) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1774
      if ( res /= 1774) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9355
      if ( res /= 9355) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWp, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4087
      if ( res /= 4087) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5295
      if ( res /= 5295) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWp, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5943
      if ( res /= 5943) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7174
      if ( res /= 7174) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1248
      if ( res /= 1248) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",302
      if ( res /= 302) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8818
      if ( res /= 8818) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZ, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",558
      if ( res /= 558) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9318
      if ( res /= 9318) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5994
      if ( res /= 5994) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8141
      if ( res /= 8141) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4301
      if ( res /= 4301) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1964
      if ( res /= 1964) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2795
      if ( res /= 2795) then
        stop
      end if
      call createParticleConfig(P_Z, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7044
      if ( res /= 7044) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2302
      if ( res /= 2302) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2475
      if ( res /= 2475) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5926
      if ( res /= 5926) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6446
      if ( res /= 6446) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7132
      if ( res /= 7132) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6492
      if ( res /= 6492) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6769
      if ( res /= 6769) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3733
      if ( res /= 3733) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8406
      if ( res /= 8406) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8593
      if ( res /= 8593) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",298
      if ( res /= 298) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghZAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",945
      if ( res /= 945) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",970
      if ( res /= 970) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2881
      if ( res /= 2881) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6716
      if ( res /= 6716) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2742
      if ( res /= 2742) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6996
      if ( res /= 6996) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4275
      if ( res /= 4275) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HPlus, P_GMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1539
      if ( res /= 1539) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8557
      if ( res /= 8557) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9088
      if ( res /= 9088) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",716
      if ( res /= 716) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2939
      if ( res /= 2939) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8806
      if ( res /= 8806) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10639
      if ( res /= 10639) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4291
      if ( res /= 4291) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",734
      if ( res /= 734) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8528
      if ( res /= 8528) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8399
      if ( res /= 8399) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6625
      if ( res /= 6625) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3334
      if ( res /= 3334) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1515
      if ( res /= 1515) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7572
      if ( res /= 7572) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5040
      if ( res /= 5040) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_nu_muAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4899
      if ( res /= 4899) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6031
      if ( res /= 6031) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9578
      if ( res /= 9578) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2240
      if ( res /= 2240) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",400
      if ( res /= 400) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6426
      if ( res /= 6426) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWm, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7517
      if ( res /= 7517) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1145
      if ( res /= 1145) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3217
      if ( res /= 3217) then
        stop
      end if
      call createParticleConfig(P_g, P_WPlus, P_WMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10518
      if ( res /= 10518) then
        stop
      end if
      call createParticleConfig(P_g, P_A, P_g, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10653
      if ( res /= 10653) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8757
      if ( res /= 8757) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8429
      if ( res /= 8429) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5724
      if ( res /= 5724) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8620
      if ( res /= 8620) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GMinus, P_HPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10419
      if ( res /= 10419) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9178
      if ( res /= 9178) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4298
      if ( res /= 4298) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4100
      if ( res /= 4100) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3366
      if ( res /= 3366) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6396
      if ( res /= 6396) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2094
      if ( res /= 2094) then
        stop
      end if
      call createParticleConfig(P_ghG, P_g, P_ghGAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3615
      if ( res /= 3615) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8793
      if ( res /= 8793) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8870
      if ( res /= 8870) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3555
      if ( res /= 3555) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3831
      if ( res /= 3831) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9755
      if ( res /= 9755) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2887
      if ( res /= 2887) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3811
      if ( res /= 3811) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9806
      if ( res /= 9806) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7720
      if ( res /= 7720) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9717
      if ( res /= 9717) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_nu_muAnti, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3528
      if ( res /= 3528) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9486
      if ( res /= 9486) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8991
      if ( res /= 8991) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9329
      if ( res /= 9329) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6276
      if ( res /= 6276) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9914
      if ( res /= 9914) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_muMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7505
      if ( res /= 7505) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3584
      if ( res /= 3584) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",643
      if ( res /= 643) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghZAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4463
      if ( res /= 4463) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4692
      if ( res /= 4692) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_A, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",94
      if ( res /= 94) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_HMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4945
      if ( res /= 4945) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5882
      if ( res /= 5882) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",933
      if ( res /= 933) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GPlus, P_ghWpAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8597
      if ( res /= 8597) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2515
      if ( res /= 2515) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_c, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7851
      if ( res /= 7851) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2163
      if ( res /= 2163) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",899
      if ( res /= 899) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1012
      if ( res /= 1012) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5349
      if ( res /= 5349) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1799
      if ( res /= 1799) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6291
      if ( res /= 6291) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2631
      if ( res /= 2631) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9443
      if ( res /= 9443) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_dAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4413
      if ( res /= 4413) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9641
      if ( res /= 9641) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4579
      if ( res /= 4579) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5931
      if ( res /= 5931) then
        stop
      end if
      call createParticleConfig(P_c, P_sAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6234
      if ( res /= 6234) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2615
      if ( res /= 2615) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10712
      if ( res /= 10712) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",104
      if ( res /= 104) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7217
      if ( res /= 7217) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5261
      if ( res /= 5261) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9580
      if ( res /= 9580) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9620
      if ( res /= 9620) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",728
      if ( res /= 728) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghWm, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7646
      if ( res /= 7646) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8079
      if ( res /= 8079) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10027
      if ( res /= 10027) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",954
      if ( res /= 954) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3527
      if ( res /= 3527) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4219
      if ( res /= 4219) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghZAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2627
      if ( res /= 2627) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GPlus, P_HMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3523
      if ( res /= 3523) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6513
      if ( res /= 6513) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5352
      if ( res /= 5352) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7236
      if ( res /= 7236) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6819
      if ( res /= 6819) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_t, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7798
      if ( res /= 7798) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2555
      if ( res /= 2555) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9715
      if ( res /= 9715) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8614
      if ( res /= 8614) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8251
      if ( res /= 8251) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3399
      if ( res /= 3399) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10547
      if ( res /= 10547) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10743
      if ( res /= 10743) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3769
      if ( res /= 3769) then
        stop
      end if
      call createParticleConfig(P_AQ, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8278
      if ( res /= 8278) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9365
      if ( res /= 9365) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9535
      if ( res /= 9535) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1045
      if ( res /= 1045) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9194
      if ( res /= 9194) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6081
      if ( res /= 6081) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6560
      if ( res /= 6560) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1208
      if ( res /= 1208) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9345
      if ( res /= 9345) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5235
      if ( res /= 5235) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10595
      if ( res /= 10595) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4911
      if ( res /= 4911) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9283
      if ( res /= 9283) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6405
      if ( res /= 6405) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7573
      if ( res /= 7573) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1758
      if ( res /= 1758) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2181
      if ( res /= 2181) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4688
      if ( res /= 4688) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5506
      if ( res /= 5506) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10301
      if ( res /= 10301) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8085
      if ( res /= 8085) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5277
      if ( res /= 5277) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8257
      if ( res /= 8257) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9206
      if ( res /= 9206) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6425
      if ( res /= 6425) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4585
      if ( res /= 4585) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8744
      if ( res /= 8744) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8921
      if ( res /= 8921) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1030
      if ( res /= 1030) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWp, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7017
      if ( res /= 7017) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",704
      if ( res /= 704) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1745
      if ( res /= 1745) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8805
      if ( res /= 8805) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8641
      if ( res /= 8641) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4683
      if ( res /= 4683) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",32
      if ( res /= 32) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1093
      if ( res /= 1093) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9794
      if ( res /= 9794) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10062
      if ( res /= 10062) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",500
      if ( res /= 500) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6997
      if ( res /= 6997) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4510
      if ( res /= 4510) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8816
      if ( res /= 8816) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_g, P_ghG, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6677
      if ( res /= 6677) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5019
      if ( res /= 5019) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9298
      if ( res /= 9298) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1870
      if ( res /= 1870) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6916
      if ( res /= 6916) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4872
      if ( res /= 4872) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4743
      if ( res /= 4743) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8295
      if ( res /= 8295) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6423
      if ( res /= 6423) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9932
      if ( res /= 9932) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8928
      if ( res /= 8928) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4472
      if ( res /= 4472) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2404
      if ( res /= 2404) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2467
      if ( res /= 2467) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2125
      if ( res /= 2125) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8098
      if ( res /= 8098) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6077
      if ( res /= 6077) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8873
      if ( res /= 8873) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7489
      if ( res /= 7489) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3675
      if ( res /= 3675) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9783
      if ( res /= 9783) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9515
      if ( res /= 9515) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4363
      if ( res /= 4363) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",217
      if ( res /= 217) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWm, P_HPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8964
      if ( res /= 8964) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6489
      if ( res /= 6489) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2461
      if ( res /= 2461) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",904
      if ( res /= 904) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_A, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6939
      if ( res /= 6939) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6275
      if ( res /= 6275) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3077
      if ( res /= 3077) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2176
      if ( res /= 2176) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",242
      if ( res /= 242) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10347
      if ( res /= 10347) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4129
      if ( res /= 4129) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",791
      if ( res /= 791) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2635
      if ( res /= 2635) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2017
      if ( res /= 2017) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3685
      if ( res /= 3685) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8137
      if ( res /= 8137) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10730
      if ( res /= 10730) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7856
      if ( res /= 7856) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7859
      if ( res /= 7859) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10291
      if ( res /= 10291) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5845
      if ( res /= 5845) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5541
      if ( res /= 5541) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8225
      if ( res /= 8225) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",702
      if ( res /= 702) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1161
      if ( res /= 1161) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7208
      if ( res /= 7208) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3201
      if ( res /= 3201) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1948
      if ( res /= 1948) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5392
      if ( res /= 5392) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_u, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",879
      if ( res /= 879) then
        stop
      end if
      call createParticleConfig(P_Hh, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6718
      if ( res /= 6718) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10569
      if ( res /= 10569) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5247
      if ( res /= 5247) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6752
      if ( res /= 6752) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7038
      if ( res /= 7038) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9202
      if ( res /= 9202) then
        stop
      end if
      call createParticleConfig(P_u, P_dAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7269
      if ( res /= 7269) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6982
      if ( res /= 6982) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1339
      if ( res /= 1339) then
        stop
      end if
      call createParticleConfig(P_t, P_Hh, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",128
      if ( res /= 128) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8980
      if ( res /= 8980) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_ePlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",748
      if ( res /= 748) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9464
      if ( res /= 9464) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1362
      if ( res /= 1362) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5068
      if ( res /= 5068) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghZAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7284
      if ( res /= 7284) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6663
      if ( res /= 6663) then
        stop
      end if
      call createParticleConfig(P_Z, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3229
      if ( res /= 3229) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3235
      if ( res /= 3235) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8400
      if ( res /= 8400) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2497
      if ( res /= 2497) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWm, P_ghWmAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7878
      if ( res /= 7878) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9324
      if ( res /= 9324) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",721
      if ( res /= 721) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7509
      if ( res /= 7509) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7963
      if ( res /= 7963) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3875
      if ( res /= 3875) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghA, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7437
      if ( res /= 7437) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_G0, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7899
      if ( res /= 7899) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7536
      if ( res /= 7536) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6089
      if ( res /= 6089) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2966
      if ( res /= 2966) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5218
      if ( res /= 5218) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9603
      if ( res /= 9603) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7188
      if ( res /= 7188) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8954
      if ( res /= 8954) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2937
      if ( res /= 2937) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8207
      if ( res /= 8207) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_nu_tauAnti, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7746
      if ( res /= 7746) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5573
      if ( res /= 5573) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6135
      if ( res /= 6135) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_dAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4823
      if ( res /= 4823) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",216
      if ( res /= 216) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1011
      if ( res /= 1011) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10185
      if ( res /= 10185) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8859
      if ( res /= 8859) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7952
      if ( res /= 7952) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1450
      if ( res /= 1450) then
        stop
      end if
      call createParticleConfig(P_G0, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8430
      if ( res /= 8430) then
        stop
      end if
      call createParticleConfig(P_G0, P_g, P_G0, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10618
      if ( res /= 10618) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8745
      if ( res /= 8745) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7819
      if ( res /= 7819) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_Ha, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8127
      if ( res /= 8127) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8656
      if ( res /= 8656) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4372
      if ( res /= 4372) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2037
      if ( res /= 2037) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9317
      if ( res /= 9317) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4878
      if ( res /= 4878) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghWmAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4984
      if ( res /= 4984) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9097
      if ( res /= 9097) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9258
      if ( res /= 9258) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5182
      if ( res /= 5182) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8743
      if ( res /= 8743) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5177
      if ( res /= 5177) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2853
      if ( res /= 2853) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5722
      if ( res /= 5722) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7914
      if ( res /= 7914) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",870
      if ( res /= 870) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",131
      if ( res /= 131) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8345
      if ( res /= 8345) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4257
      if ( res /= 4257) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5272
      if ( res /= 5272) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10471
      if ( res /= 10471) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2570
      if ( res /= 2570) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4064
      if ( res /= 4064) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6157
      if ( res /= 6157) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6238
      if ( res /= 6238) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9789
      if ( res /= 9789) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6245
      if ( res /= 6245) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3377
      if ( res /= 3377) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",48
      if ( res /= 48) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9043
      if ( res /= 9043) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7019
      if ( res /= 7019) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2064
      if ( res /= 2064) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8002
      if ( res /= 8002) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3177
      if ( res /= 3177) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7024
      if ( res /= 7024) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8902
      if ( res /= 8902) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7391
      if ( res /= 7391) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10702
      if ( res /= 10702) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6018
      if ( res /= 6018) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3829
      if ( res /= 3829) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7682
      if ( res /= 7682) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",979
      if ( res /= 979) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9047
      if ( res /= 9047) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1576
      if ( res /= 1576) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3737
      if ( res /= 3737) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2377
      if ( res /= 2377) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",110
      if ( res /= 110) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3466
      if ( res /= 3466) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6322
      if ( res /= 6322) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6705
      if ( res /= 6705) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8836
      if ( res /= 8836) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10746
      if ( res /= 10746) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_c, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3359
      if ( res /= 3359) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6415
      if ( res /= 6415) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_A, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3677
      if ( res /= 3677) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8705
      if ( res /= 8705) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",56
      if ( res /= 56) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWp, P_ghWpAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9971
      if ( res /= 9971) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5130
      if ( res /= 5130) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4776
      if ( res /= 4776) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6954
      if ( res /= 6954) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8070
      if ( res /= 8070) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4640
      if ( res /= 4640) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghZAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8444
      if ( res /= 8444) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4027
      if ( res /= 4027) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3694
      if ( res /= 3694) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1025
      if ( res /= 1025) then
        stop
      end if
      call createParticleConfig(P_A, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4182
      if ( res /= 4182) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7374
      if ( res /= 7374) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8634
      if ( res /= 8634) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6253
      if ( res /= 6253) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4534
      if ( res /= 4534) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9899
      if ( res /= 9899) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7520
      if ( res /= 7520) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5348
      if ( res /= 5348) then
        stop
      end if
      call createParticleConfig(P_g, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6977
      if ( res /= 6977) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3172
      if ( res /= 3172) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1287
      if ( res /= 1287) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2609
      if ( res /= 2609) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10135
      if ( res /= 10135) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWpAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1016
      if ( res /= 1016) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6469
      if ( res /= 6469) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2214
      if ( res /= 2214) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10504
      if ( res /= 10504) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2740
      if ( res /= 2740) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2698
      if ( res /= 2698) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_A, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7209
      if ( res /= 7209) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9735
      if ( res /= 9735) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4058
      if ( res /= 4058) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7299
      if ( res /= 7299) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6647
      if ( res /= 6647) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7494
      if ( res /= 7494) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7184
      if ( res /= 7184) then
        stop
      end if
      call createParticleConfig(P_s, P_Ha, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6191
      if ( res /= 6191) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4278
      if ( res /= 4278) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5623
      if ( res /= 5623) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8371
      if ( res /= 8371) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3340
      if ( res /= 3340) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",662
      if ( res /= 662) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6651
      if ( res /= 6651) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3530
      if ( res /= 3530) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3238
      if ( res /= 3238) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWm, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3329
      if ( res /= 3329) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7740
      if ( res /= 7740) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_muPlus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3470
      if ( res /= 3470) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",330
      if ( res /= 330) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2623
      if ( res /= 2623) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4459
      if ( res /= 4459) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10349
      if ( res /= 10349) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5034
      if ( res /= 5034) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3437
      if ( res /= 3437) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2468
      if ( res /= 2468) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7537
      if ( res /= 7537) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_nu_tau, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4208
      if ( res /= 4208) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",93
      if ( res /= 93) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6355
      if ( res /= 6355) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3610
      if ( res /= 3610) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",471
      if ( res /= 471) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5622
      if ( res /= 5622) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",14
      if ( res /= 14) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",593
      if ( res /= 593) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3474
      if ( res /= 3474) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6543
      if ( res /= 6543) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3307
      if ( res /= 3307) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3650
      if ( res /= 3650) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9172
      if ( res /= 9172) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3421
      if ( res /= 3421) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10585
      if ( res /= 10585) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6181
      if ( res /= 6181) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1503
      if ( res /= 1503) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6931
      if ( res /= 6931) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1488
      if ( res /= 1488) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3317
      if ( res /= 3317) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10323
      if ( res /= 10323) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghWmAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3009
      if ( res /= 3009) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1702
      if ( res /= 1702) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",323
      if ( res /= 323) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6594
      if ( res /= 6594) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5322
      if ( res /= 5322) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3870
      if ( res /= 3870) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3573
      if ( res /= 3573) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5766
      if ( res /= 5766) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghA, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3654
      if ( res /= 3654) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4559
      if ( res /= 4559) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",18
      if ( res /= 18) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2335
      if ( res /= 2335) then
        stop
      end if
      call createParticleConfig(P_b, P_Hl, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7056
      if ( res /= 7056) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8626
      if ( res /= 8626) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9068
      if ( res /= 9068) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3167
      if ( res /= 3167) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1361
      if ( res /= 1361) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9686
      if ( res /= 9686) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_t, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",308
      if ( res /= 308) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2438
      if ( res /= 2438) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9327
      if ( res /= 9327) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",377
      if ( res /= 377) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6315
      if ( res /= 6315) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3388
      if ( res /= 3388) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9786
      if ( res /= 9786) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10191
      if ( res /= 10191) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",469
      if ( res /= 469) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",16
      if ( res /= 16) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5217
      if ( res /= 5217) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5047
      if ( res /= 5047) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9056
      if ( res /= 9056) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6740
      if ( res /= 6740) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8239
      if ( res /= 8239) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1837
      if ( res /= 1837) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10392
      if ( res /= 10392) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4090
      if ( res /= 4090) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8252
      if ( res /= 8252) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5662
      if ( res /= 5662) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9156
      if ( res /= 9156) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7343
      if ( res /= 7343) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9412
      if ( res /= 9412) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8925
      if ( res /= 8925) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5408
      if ( res /= 5408) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10670
      if ( res /= 10670) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7073
      if ( res /= 7073) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Ha, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2988
      if ( res /= 2988) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_c, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",297
      if ( res /= 297) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3603
      if ( res /= 3603) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4804
      if ( res /= 4804) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1225
      if ( res /= 1225) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2681
      if ( res /= 2681) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3646
      if ( res /= 3646) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1967
      if ( res /= 1967) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8530
      if ( res /= 8530) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1828
      if ( res /= 1828) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8352
      if ( res /= 8352) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",119
      if ( res /= 119) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7898
      if ( res /= 7898) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1776
      if ( res /= 1776) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9197
      if ( res /= 9197) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4016
      if ( res /= 4016) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8612
      if ( res /= 8612) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7471
      if ( res /= 7471) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7826
      if ( res /= 7826) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6984
      if ( res /= 6984) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2851
      if ( res /= 2851) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6781
      if ( res /= 6781) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8168
      if ( res /= 8168) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8233
      if ( res /= 8233) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8241
      if ( res /= 8241) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3148
      if ( res /= 3148) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6562
      if ( res /= 6562) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10040
      if ( res /= 10040) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5721
      if ( res /= 5721) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7969
      if ( res /= 7969) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4614
      if ( res /= 4614) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1773
      if ( res /= 1773) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2386
      if ( res /= 2386) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9992
      if ( res /= 9992) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2850
      if ( res /= 2850) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_nu_e, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1917
      if ( res /= 1917) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1746
      if ( res /= 1746) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1117
      if ( res /= 1117) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7755
      if ( res /= 7755) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2894
      if ( res /= 2894) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2567
      if ( res /= 2567) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7218
      if ( res /= 7218) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10242
      if ( res /= 10242) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3752
      if ( res /= 3752) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1312
      if ( res /= 1312) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2106
      if ( res /= 2106) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6943
      if ( res /= 6943) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8710
      if ( res /= 8710) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3084
      if ( res /= 3084) then
        stop
      end if
      call createParticleConfig(P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9525
      if ( res /= 9525) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5870
      if ( res /= 5870) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",282
      if ( res /= 282) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7263
      if ( res /= 7263) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWp, P_HMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1163
      if ( res /= 1163) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3631
      if ( res /= 3631) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",311
      if ( res /= 311) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",626
      if ( res /= 626) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10381
      if ( res /= 10381) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4304
      if ( res /= 4304) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_b, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",883
      if ( res /= 883) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10025
      if ( res /= 10025) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7995
      if ( res /= 7995) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9246
      if ( res /= 9246) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1814
      if ( res /= 1814) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5129
      if ( res /= 5129) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1237
      if ( res /= 1237) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4670
      if ( res /= 4670) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10742
      if ( res /= 10742) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7582
      if ( res /= 7582) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1430
      if ( res /= 1430) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1408
      if ( res /= 1408) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",160
      if ( res /= 160) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5205
      if ( res /= 5205) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8089
      if ( res /= 8089) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2864
      if ( res /= 2864) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7127
      if ( res /= 7127) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1378
      if ( res /= 1378) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",563
      if ( res /= 563) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4061
      if ( res /= 4061) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2395
      if ( res /= 2395) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3898
      if ( res /= 3898) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",163
      if ( res /= 163) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4715
      if ( res /= 4715) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7364
      if ( res /= 7364) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1853
      if ( res /= 1853) then
        stop
      end if
      call createParticleConfig(P_ghG, P_ghGAnti, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5589
      if ( res /= 5589) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_HMinus, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4569
      if ( res /= 4569) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7367
      if ( res /= 7367) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7976
      if ( res /= 7976) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8067
      if ( res /= 8067) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2958
      if ( res /= 2958) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2661
      if ( res /= 2661) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7857
      if ( res /= 7857) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10083
      if ( res /= 10083) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10248
      if ( res /= 10248) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8367
      if ( res /= 8367) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4206
      if ( res /= 4206) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4144
      if ( res /= 4144) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9695
      if ( res /= 9695) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",592
      if ( res /= 592) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9081
      if ( res /= 9081) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4230
      if ( res /= 4230) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8979
      if ( res /= 8979) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",383
      if ( res /= 383) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5707
      if ( res /= 5707) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6655
      if ( res /= 6655) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",654
      if ( res /= 654) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3445
      if ( res /= 3445) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6270
      if ( res /= 6270) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4892
      if ( res /= 4892) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4464
      if ( res /= 4464) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6511
      if ( res /= 6511) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7238
      if ( res /= 7238) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7131
      if ( res /= 7131) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5390
      if ( res /= 5390) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7078
      if ( res /= 7078) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghAAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1303
      if ( res /= 1303) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghWm, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3385
      if ( res /= 3385) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9586
      if ( res /= 9586) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1084
      if ( res /= 1084) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",369
      if ( res /= 369) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3386
      if ( res /= 3386) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4658
      if ( res /= 4658) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9538
      if ( res /= 9538) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghAAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",482
      if ( res /= 482) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8424
      if ( res /= 8424) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghA, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1286
      if ( res /= 1286) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5645
      if ( res /= 5645) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2233
      if ( res /= 2233) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1188
      if ( res /= 1188) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4255
      if ( res /= 4255) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4875
      if ( res /= 4875) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8728
      if ( res /= 8728) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWm, P_ghWmAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8881
      if ( res /= 8881) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8950
      if ( res /= 8950) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_tauMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",701
      if ( res /= 701) then
        stop
      end if
      call createParticleConfig(P_g, P_HMinus, P_g, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10503
      if ( res /= 10503) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4850
      if ( res /= 4850) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5163
      if ( res /= 5163) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZ, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4412
      if ( res /= 4412) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",743
      if ( res /= 743) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",664
      if ( res /= 664) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2483
      if ( res /= 2483) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1528
      if ( res /= 1528) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1860
      if ( res /= 1860) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1552
      if ( res /= 1552) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8865
      if ( res /= 8865) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6361
      if ( res /= 6361) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1827
      if ( res /= 1827) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9151
      if ( res /= 9151) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10432
      if ( res /= 10432) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_Hh, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2424
      if ( res /= 2424) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7868
      if ( res /= 7868) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9643
      if ( res /= 9643) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5465
      if ( res /= 5465) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5976
      if ( res /= 5976) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ePlus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2735
      if ( res /= 2735) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",22
      if ( res /= 22) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10472
      if ( res /= 10472) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5669
      if ( res /= 5669) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_tAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6794
      if ( res /= 6794) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWmAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7521
      if ( res /= 7521) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6429
      if ( res /= 6429) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8754
      if ( res /= 8754) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8983
      if ( res /= 8983) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2288
      if ( res /= 2288) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6414
      if ( res /= 6414) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ePlus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",317
      if ( res /= 317) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4798
      if ( res /= 4798) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1200
      if ( res /= 1200) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7334
      if ( res /= 7334) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_g, P_g, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10669
      if ( res /= 10669) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10366
      if ( res /= 10366) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3324
      if ( res /= 3324) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2158
      if ( res /= 2158) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9668
      if ( res /= 9668) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",607
      if ( res /= 607) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8994
      if ( res /= 8994) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2237
      if ( res /= 2237) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4004
      if ( res /= 4004) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",975
      if ( res /= 975) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1270
      if ( res /= 1270) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8175
      if ( res /= 8175) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6033
      if ( res /= 6033) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7170
      if ( res /= 7170) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10676
      if ( res /= 10676) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8090
      if ( res /= 8090) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9632
      if ( res /= 9632) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7515
      if ( res /= 7515) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10296
      if ( res /= 10296) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2780
      if ( res /= 2780) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7291
      if ( res /= 7291) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8935
      if ( res /= 8935) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8468
      if ( res /= 8468) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10255
      if ( res /= 10255) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2810
      if ( res /= 2810) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3899
      if ( res /= 3899) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2263
      if ( res /= 2263) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_d, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5730
      if ( res /= 5730) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghWpAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7872
      if ( res /= 7872) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6785
      if ( res /= 6785) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_bAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7271
      if ( res /= 7271) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10186
      if ( res /= 10186) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9084
      if ( res /= 9084) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10080
      if ( res /= 10080) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8909
      if ( res /= 8909) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1635
      if ( res /= 1635) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10502
      if ( res /= 10502) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3255
      if ( res /= 3255) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HPlus, P_GMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1139
      if ( res /= 1139) then
        stop
      end if
      call createParticleConfig(P_G0, P_g, P_g, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10698
      if ( res /= 10698) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3458
      if ( res /= 3458) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9285
      if ( res /= 9285) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5030
      if ( res /= 5030) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GPlus, P_GMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1738
      if ( res /= 1738) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3551
      if ( res /= 3551) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8072
      if ( res /= 8072) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2920
      if ( res /= 2920) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",261
      if ( res /= 261) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3674
      if ( res /= 3674) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",99
      if ( res /= 99) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9221
      if ( res /= 9221) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2349
      if ( res /= 2349) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWm, P_ghWmAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10235
      if ( res /= 10235) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3818
      if ( res /= 3818) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghAAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2531
      if ( res /= 2531) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Z, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1510
      if ( res /= 1510) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9399
      if ( res /= 9399) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",380
      if ( res /= 380) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4274
      if ( res /= 4274) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9942
      if ( res /= 9942) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2777
      if ( res /= 2777) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5758
      if ( res /= 5758) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4873
      if ( res /= 4873) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10564
      if ( res /= 10564) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_g, P_HMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10693
      if ( res /= 10693) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6356
      if ( res /= 6356) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6717
      if ( res /= 6717) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10280
      if ( res /= 10280) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10118
      if ( res /= 10118) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5556
      if ( res /= 5556) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6649
      if ( res /= 6649) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10527
      if ( res /= 10527) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9518
      if ( res /= 9518) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3679
      if ( res /= 3679) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",108
      if ( res /= 108) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8874
      if ( res /= 8874) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8823
      if ( res /= 8823) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",683
      if ( res /= 683) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",589
      if ( res /= 589) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2501
      if ( res /= 2501) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1410
      if ( res /= 1410) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_A, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1448
      if ( res /= 1448) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3932
      if ( res /= 3932) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6914
      if ( res /= 6914) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3972
      if ( res /= 3972) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9890
      if ( res /= 9890) then
        stop
      end if
      call createParticleConfig(P_gQ, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1642
      if ( res /= 1642) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8140
      if ( res /= 8140) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7426
      if ( res /= 7426) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4307
      if ( res /= 4307) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6858
      if ( res /= 6858) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3839
      if ( res /= 3839) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",166
      if ( res /= 166) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1098
      if ( res /= 1098) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3865
      if ( res /= 3865) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9883
      if ( res /= 9883) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7620
      if ( res /= 7620) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1849
      if ( res /= 1849) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3234
      if ( res /= 3234) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6962
      if ( res /= 6962) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10434
      if ( res /= 10434) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWp, P_ghWpAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",795
      if ( res /= 795) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2383
      if ( res /= 2383) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8594
      if ( res /= 8594) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10010
      if ( res /= 10010) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1130
      if ( res /= 1130) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",793
      if ( res /= 793) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",539
      if ( res /= 539) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10193
      if ( res /= 10193) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2299
      if ( res /= 2299) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8974
      if ( res /= 8974) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10219
      if ( res /= 10219) then
        stop
      end if
      call createParticleConfig(P_Hl, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5260
      if ( res /= 5260) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1249
      if ( res /= 1249) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5432
      if ( res /= 5432) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7966
      if ( res /= 7966) then
        stop
      end if
      call createParticleConfig(P_Hl, P_g, P_Hh, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10605
      if ( res /= 10605) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5815
      if ( res /= 5815) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2746
      if ( res /= 2746) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3341
      if ( res /= 3341) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9163
      if ( res /= 9163) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1513
      if ( res /= 1513) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9721
      if ( res /= 9721) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4590
      if ( res /= 4590) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4043
      if ( res /= 4043) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4117
      if ( res /= 4117) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8377
      if ( res /= 8377) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6826
      if ( res /= 6826) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6258
      if ( res /= 6258) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9139
      if ( res /= 9139) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2089
      if ( res /= 2089) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6550
      if ( res /= 6550) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1739
      if ( res /= 1739) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9037
      if ( res /= 9037) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2474
      if ( res /= 2474) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3119
      if ( res /= 3119) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4535
      if ( res /= 4535) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghWmAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2829
      if ( res /= 2829) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5624
      if ( res /= 5624) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7223
      if ( res /= 7223) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HMinus, P_WPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7129
      if ( res /= 7129) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",357
      if ( res /= 357) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7281
      if ( res /= 7281) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6067
      if ( res /= 6067) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7436
      if ( res /= 7436) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9832
      if ( res /= 9832) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7799
      if ( res /= 7799) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",982
      if ( res /= 982) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5002
      if ( res /= 5002) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_A, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",566
      if ( res /= 566) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4251
      if ( res /= 4251) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5472
      if ( res /= 5472) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10625
      if ( res /= 10625) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghWm, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7542
      if ( res /= 7542) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5935
      if ( res /= 5935) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6239
      if ( res /= 6239) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7840
      if ( res /= 7840) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9849
      if ( res /= 9849) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5523
      if ( res /= 5523) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5354
      if ( res /= 5354) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3841
      if ( res /= 3841) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6159
      if ( res /= 6159) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5981
      if ( res /= 5981) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7586
      if ( res /= 7586) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10337
      if ( res /= 10337) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10387
      if ( res /= 10387) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1456
      if ( res /= 1456) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",672
      if ( res /= 672) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3689
      if ( res /= 3689) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2147
      if ( res /= 2147) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3011
      if ( res /= 3011) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9103
      if ( res /= 9103) then
        stop
      end if
      call createParticleConfig(P_g, P_G0, P_G0, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10721
      if ( res /= 10721) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",685
      if ( res /= 685) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghZ, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8835
      if ( res /= 8835) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5703
      if ( res /= 5703) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9748
      if ( res /= 9748) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3426
      if ( res /= 3426) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9908
      if ( res /= 9908) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_nu_muAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8844
      if ( res /= 8844) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",347
      if ( res /= 347) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3016
      if ( res /= 3016) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2889
      if ( res /= 2889) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",64
      if ( res /= 64) then
        stop
      end if
      call createParticleConfig(P_g, P_Hh, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10682
      if ( res /= 10682) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9491
      if ( res /= 9491) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5944
      if ( res /= 5944) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3830
      if ( res /= 3830) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6521
      if ( res /= 6521) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2582
      if ( res /= 2582) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10190
      if ( res /= 10190) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6430
      if ( res /= 6430) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4909
      if ( res /= 4909) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",197
      if ( res /= 197) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7406
      if ( res /= 7406) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghA, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3927
      if ( res /= 3927) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9810
      if ( res /= 9810) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8503
      if ( res /= 8503) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1437
      if ( res /= 1437) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1295
      if ( res /= 1295) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_A, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5814
      if ( res /= 5814) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",320
      if ( res /= 320) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3869
      if ( res /= 3869) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6022
      if ( res /= 6022) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2443
      if ( res /= 2443) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4706
      if ( res /= 4706) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9519
      if ( res /= 9519) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HMinus, P_ghWmAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10426
      if ( res /= 10426) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1000
      if ( res /= 1000) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5378
      if ( res /= 5378) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10042
      if ( res /= 10042) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5150
      if ( res /= 5150) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",881
      if ( res /= 881) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9869
      if ( res /= 9869) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3047
      if ( res /= 3047) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3682
      if ( res /= 3682) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6995
      if ( res /= 6995) then
        stop
      end if
      call createParticleConfig(P_AQ, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",661
      if ( res /= 661) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10405
      if ( res /= 10405) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10524
      if ( res /= 10524) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5663
      if ( res /= 5663) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_c, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5381
      if ( res /= 5381) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5477
      if ( res /= 5477) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6678
      if ( res /= 6678) then
        stop
      end if
      call createParticleConfig(P_g, P_ghGAnti, P_ghG, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1118
      if ( res /= 1118) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3611
      if ( res /= 3611) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3114
      if ( res /= 3114) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8606
      if ( res /= 8606) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",642
      if ( res /= 642) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9947
      if ( res /= 9947) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7210
      if ( res /= 7210) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6363
      if ( res /= 6363) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3418
      if ( res /= 3418) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3240
      if ( res /= 3240) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5636
      if ( res /= 5636) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",857
      if ( res /= 857) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8579
      if ( res /= 8579) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9363
      if ( res /= 9363) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3325
      if ( res /= 3325) then
        stop
      end if
      call createParticleConfig(P_g, P_HPlus, P_GMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10707
      if ( res /= 10707) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2589
      if ( res /= 2589) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4091
      if ( res /= 4091) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1903
      if ( res /= 1903) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1318
      if ( res /= 1318) then
        stop
      end if
      call createParticleConfig(P_b, P_Z, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1666
      if ( res /= 1666) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9616
      if ( res /= 9616) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_WPlus, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3986
      if ( res /= 3986) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7449
      if ( res /= 7449) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6506
      if ( res /= 6506) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1592
      if ( res /= 1592) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",898
      if ( res /= 898) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7678
      if ( res /= 7678) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8479
      if ( res /= 8479) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5828
      if ( res /= 5828) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",164
      if ( res /= 164) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6922
      if ( res /= 6922) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10151
      if ( res /= 10151) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3613
      if ( res /= 3613) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10523
      if ( res /= 10523) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6515
      if ( res /= 6515) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2651
      if ( res /= 2651) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2027
      if ( res /= 2027) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9320
      if ( res /= 9320) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2023
      if ( res /= 2023) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2200
      if ( res /= 2200) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2213
      if ( res /= 2213) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7724
      if ( res /= 7724) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9521
      if ( res /= 9521) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",17
      if ( res /= 17) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1829
      if ( res /= 1829) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5279
      if ( res /= 5279) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6874
      if ( res /= 6874) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_nu_tau, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8576
      if ( res /= 8576) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWp, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7608
      if ( res /= 7608) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1869
      if ( res /= 1869) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9232
      if ( res /= 9232) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3746
      if ( res /= 3746) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1526
      if ( res /= 1526) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4436
      if ( res /= 4436) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5700
      if ( res /= 5700) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8462
      if ( res /= 8462) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5898
      if ( res /= 5898) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3407
      if ( res /= 3407) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghWm, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4408
      if ( res /= 4408) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_Z, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7654
      if ( res /= 7654) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5918
      if ( res /= 5918) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9466
      if ( res /= 9466) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3209
      if ( res /= 3209) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5895
      if ( res /= 5895) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9437
      if ( res /= 9437) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6754
      if ( res /= 6754) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",193
      if ( res /= 193) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",546
      if ( res /= 546) then
        stop
      end if
      call createParticleConfig(P_A, P_nu_e, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10377
      if ( res /= 10377) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2389
      if ( res /= 2389) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",370
      if ( res /= 370) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_t, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8015
      if ( res /= 8015) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10331
      if ( res /= 10331) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4211
      if ( res /= 4211) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8842
      if ( res /= 8842) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7328
      if ( res /= 7328) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9115
      if ( res /= 9115) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4617
      if ( res /= 4617) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3566
      if ( res /= 3566) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7783
      if ( res /= 7783) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1572
      if ( res /= 1572) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8344
      if ( res /= 8344) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2931
      if ( res /= 2931) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6237
      if ( res /= 6237) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7554
      if ( res /= 7554) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2679
      if ( res /= 2679) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8104
      if ( res /= 8104) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1627
      if ( res /= 1627) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4094
      if ( res /= 4094) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7074
      if ( res /= 7074) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4394
      if ( res /= 4394) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8698
      if ( res /= 8698) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6882
      if ( res /= 6882) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3398
      if ( res /= 3398) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6973
      if ( res /= 6973) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghAAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3142
      if ( res /= 3142) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8892
      if ( res /= 8892) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",155
      if ( res /= 155) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3092
      if ( res /= 3092) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_A, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8013
      if ( res /= 8013) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_Z, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6656
      if ( res /= 6656) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_nu_e, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10336
      if ( res /= 10336) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6870
      if ( res /= 6870) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7748
      if ( res /= 7748) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5380
      if ( res /= 5380) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9048
      if ( res /= 9048) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2592
      if ( res /= 2592) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3349
      if ( res /= 3349) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1134
      if ( res /= 1134) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10070
      if ( res /= 10070) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7725
      if ( res /= 7725) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9743
      if ( res /= 9743) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4020
      if ( res /= 4020) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1778
      if ( res /= 1778) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1360
      if ( res /= 1360) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",867
      if ( res /= 867) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2517
      if ( res /= 2517) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4841
      if ( res /= 4841) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3379
      if ( res /= 3379) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1300
      if ( res /= 1300) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWmAnti, P_WMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6726
      if ( res /= 6726) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghZ, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3104
      if ( res /= 3104) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9919
      if ( res /= 9919) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3481
      if ( res /= 3481) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5008
      if ( res /= 5008) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_A, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",136
      if ( res /= 136) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6243
      if ( res /= 6243) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9017
      if ( res /= 9017) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3173
      if ( res /= 3173) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9180
      if ( res /= 9180) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4113
      if ( res /= 4113) then
        stop
      end if
      call createParticleConfig(P_g, P_Ha, P_Ha, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10526
      if ( res /= 10526) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_nu_tauAnti, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8812
      if ( res /= 8812) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9533
      if ( res /= 9533) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5818
      if ( res /= 5818) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",301
      if ( res /= 301) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6645
      if ( res /= 6645) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",132
      if ( res /= 132) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10448
      if ( res /= 10448) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1375
      if ( res /= 1375) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2350
      if ( res /= 2350) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8821
      if ( res /= 8821) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2174
      if ( res /= 2174) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1651
      if ( res /= 1651) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8133
      if ( res /= 8133) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3797
      if ( res /= 3797) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4305
      if ( res /= 4305) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5865
      if ( res /= 5865) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2823
      if ( res /= 2823) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6421
      if ( res /= 6421) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6125
      if ( res /= 6125) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3810
      if ( res /= 3810) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9516
      if ( res /= 9516) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5364
      if ( res /= 5364) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_c, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8178
      if ( res /= 8178) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2732
      if ( res /= 2732) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6155
      if ( res /= 6155) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8350
      if ( res /= 8350) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3514
      if ( res /= 3514) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GPlus, P_HMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5110
      if ( res /= 5110) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10490
      if ( res /= 10490) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8222
      if ( res /= 8222) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_nu_mu, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8813
      if ( res /= 8813) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8520
      if ( res /= 8520) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",157
      if ( res /= 157) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4284
      if ( res /= 4284) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3061
      if ( res /= 3061) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6011
      if ( res /= 6011) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1180
      if ( res /= 1180) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9684
      if ( res /= 9684) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6981
      if ( res /= 6981) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8164
      if ( res /= 8164) then
        stop
      end if
      call createParticleConfig(P_Hh, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4179
      if ( res /= 4179) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_d, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5881
      if ( res /= 5881) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4746
      if ( res /= 4746) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7785
      if ( res /= 7785) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5876
      if ( res /= 5876) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5652
      if ( res /= 5652) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9191
      if ( res /= 9191) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1817
      if ( res /= 1817) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",651
      if ( res /= 651) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6673
      if ( res /= 6673) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghZAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3559
      if ( res /= 3559) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",184
      if ( res /= 184) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4353
      if ( res /= 4353) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10099
      if ( res /= 10099) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7459
      if ( res /= 7459) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3583
      if ( res /= 3583) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5389
      if ( res /= 5389) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghZAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6900
      if ( res /= 6900) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6069
      if ( res /= 6069) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_GMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5770
      if ( res /= 5770) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_A, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",880
      if ( res /= 880) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9282
      if ( res /= 9282) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8643
      if ( res /= 8643) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1215
      if ( res /= 1215) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3134
      if ( res /= 3134) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghZ, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8751
      if ( res /= 8751) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6555
      if ( res /= 6555) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2968
      if ( res /= 2968) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7871
      if ( res /= 7871) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZ, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4744
      if ( res /= 4744) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_Z, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5293
      if ( res /= 5293) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1079
      if ( res /= 1079) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8802
      if ( res /= 8802) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9576
      if ( res /= 9576) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10009
      if ( res /= 10009) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",962
      if ( res /= 962) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3580
      if ( res /= 3580) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5745
      if ( res /= 5745) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8580
      if ( res /= 8580) then
        stop
      end if
      call createParticleConfig(P_Ha, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",733
      if ( res /= 733) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_Hl, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4390
      if ( res /= 4390) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8831
      if ( res /= 8831) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7794
      if ( res /= 7794) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5572
      if ( res /= 5572) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9419
      if ( res /= 9419) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8978
      if ( res /= 8978) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8432
      if ( res /= 8432) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2396
      if ( res /= 2396) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8114
      if ( res /= 8114) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5189
      if ( res /= 5189) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1483
      if ( res /= 1483) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5809
      if ( res /= 5809) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9907
      if ( res /= 9907) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8506
      if ( res /= 8506) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5681
      if ( res /= 5681) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4649
      if ( res /= 4649) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10275
      if ( res /= 10275) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6216
      if ( res /= 6216) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5249
      if ( res /= 5249) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1377
      if ( res /= 1377) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",445
      if ( res /= 445) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_Ha, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5664
      if ( res /= 5664) then
        stop
      end if
      call createParticleConfig(P_Hl, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1645
      if ( res /= 1645) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7782
      if ( res /= 7782) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1621
      if ( res /= 1621) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_muPlus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",257
      if ( res /= 257) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_A, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6144
      if ( res /= 6144) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3505
      if ( res /= 3505) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9647
      if ( res /= 9647) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1885
      if ( res /= 1885) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10415
      if ( res /= 10415) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9498
      if ( res /= 9498) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_Z, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7105
      if ( res /= 7105) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6864
      if ( res /= 6864) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1187
      if ( res /= 1187) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6609
      if ( res /= 6609) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1728
      if ( res /= 1728) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1031
      if ( res /= 1031) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",821
      if ( res /= 821) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3358
      if ( res /= 3358) then
        stop
      end if
      call createParticleConfig(P_ghG, P_g, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2603
      if ( res /= 2603) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5977
      if ( res /= 5977) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9799
      if ( res /= 9799) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6963
      if ( res /= 6963) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1470
      if ( res /= 1470) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6151
      if ( res /= 6151) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7474
      if ( res /= 7474) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5717
      if ( res /= 5717) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4917
      if ( res /= 4917) then
        stop
      end if
      call createParticleConfig(P_A, P_g, P_g, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10741
      if ( res /= 10741) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1691
      if ( res /= 1691) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",834
      if ( res /= 834) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9023
      if ( res /= 9023) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghA, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5826
      if ( res /= 5826) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5930
      if ( res /= 5930) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10218
      if ( res /= 10218) then
        stop
      end if
      call createParticleConfig(P_AQ, P_G0, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4178
      if ( res /= 4178) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2133
      if ( res /= 2133) then
        stop
      end if
      call createParticleConfig(P_g, P_Z, P_g, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10775
      if ( res /= 10775) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10019
      if ( res /= 10019) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2268
      if ( res /= 2268) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1840
      if ( res /= 1840) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_bAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3736
      if ( res /= 3736) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HMinus, P_GPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",130
      if ( res /= 130) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6450
      if ( res /= 6450) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5546
      if ( res /= 5546) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2086
      if ( res /= 2086) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4469
      if ( res /= 4469) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2657
      if ( res /= 2657) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6134
      if ( res /= 6134) then
        stop
      end if
      call createParticleConfig(P_ghA, P_A, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3775
      if ( res /= 3775) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6634
      if ( res /= 6634) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2425
      if ( res /= 2425) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4333
      if ( res /= 4333) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1970
      if ( res /= 1970) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6857
      if ( res /= 6857) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2895
      if ( res /= 2895) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7850
      if ( res /= 7850) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7322
      if ( res /= 7322) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6269
      if ( res /= 6269) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2806
      if ( res /= 2806) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",890
      if ( res /= 890) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9407
      if ( res /= 9407) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10581
      if ( res /= 10581) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8250
      if ( res /= 8250) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4478
      if ( res /= 4478) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6306
      if ( res /= 6306) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5071
      if ( res /= 5071) then
        stop
      end if
      call createParticleConfig(P_G0, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4598
      if ( res /= 4598) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",451
      if ( res /= 451) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10762
      if ( res /= 10762) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4629
      if ( res /= 4629) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8976
      if ( res /= 8976) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10655
      if ( res /= 10655) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2273
      if ( res /= 2273) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5611
      if ( res /= 5611) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1511
      if ( res /= 1511) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_Hl, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7524
      if ( res /= 7524) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6892
      if ( res /= 6892) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",766
      if ( res /= 766) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4157
      if ( res /= 4157) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2979
      if ( res /= 2979) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7897
      if ( res /= 7897) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10049
      if ( res /= 10049) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6932
      if ( res /= 6932) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10449
      if ( res /= 10449) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3998
      if ( res /= 3998) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7177
      if ( res /= 7177) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5490
      if ( res /= 5490) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1565
      if ( res /= 1565) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7547
      if ( res /= 7547) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",963
      if ( res /= 963) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6504
      if ( res /= 6504) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5297
      if ( res /= 5297) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6224
      if ( res /= 6224) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1132
      if ( res /= 1132) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9946
      if ( res /= 9946) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",447
      if ( res /= 447) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8810
      if ( res /= 8810) then
        stop
      end if
      call createParticleConfig(P_t, P_bAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1160
      if ( res /= 1160) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10456
      if ( res /= 10456) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6153
      if ( res /= 6153) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8824
      if ( res /= 8824) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4893
      if ( res /= 4893) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9623
      if ( res /= 9623) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4632
      if ( res /= 4632) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7778
      if ( res /= 7778) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1018
      if ( res /= 1018) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5672
      if ( res /= 5672) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5549
      if ( res /= 5549) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1612
      if ( res /= 1612) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1153
      if ( res /= 1153) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9833
      if ( res /= 9833) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2700
      if ( res /= 2700) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",246
      if ( res /= 246) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",248
      if ( res /= 248) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1414
      if ( res /= 1414) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1417
      if ( res /= 1417) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10345
      if ( res /= 10345) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",869
      if ( res /= 869) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8498
      if ( res /= 8498) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3536
      if ( res /= 3536) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8872
      if ( res /= 8872) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZ, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7942
      if ( res /= 7942) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6339
      if ( res /= 6339) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",928
      if ( res /= 928) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8955
      if ( res /= 8955) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",346
      if ( res /= 346) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3540
      if ( res /= 3540) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2274
      if ( res /= 2274) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_A, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3878
      if ( res /= 3878) then
        stop
      end if
      call createParticleConfig(P_g, P_GPlus, P_g, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10600
      if ( res /= 10600) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5775
      if ( res /= 5775) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5950
      if ( res /= 5950) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2001
      if ( res /= 2001) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4434
      if ( res /= 4434) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_d, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1724
      if ( res /= 1724) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4938
      if ( res /= 4938) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5913
      if ( res /= 5913) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7601
      if ( res /= 7601) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5963
      if ( res /= 5963) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1559
      if ( res /= 1559) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1358
      if ( res /= 1358) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5154
      if ( res /= 5154) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10142
      if ( res /= 10142) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10262
      if ( res /= 10262) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWp, P_HMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10447
      if ( res /= 10447) then
        stop
      end if
      call createParticleConfig(P_Z, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1546
      if ( res /= 1546) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",754
      if ( res /= 754) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7089
      if ( res /= 7089) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1131
      if ( res /= 1131) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6833
      if ( res /= 6833) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7812
      if ( res /= 7812) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZ, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4490
      if ( res /= 4490) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6885
      if ( res /= 6885) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6295
      if ( res /= 6295) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3717
      if ( res /= 3717) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2771
      if ( res /= 2771) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_Hh, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4539
      if ( res /= 4539) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2774
      if ( res /= 2774) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5529
      if ( res /= 5529) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5720
      if ( res /= 5720) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10477
      if ( res /= 10477) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7564
      if ( res /= 7564) then
        stop
      end if
      call createParticleConfig(P_A, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4802
      if ( res /= 4802) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6109
      if ( res /= 6109) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWm, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1788
      if ( res /= 1788) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9027
      if ( res /= 9027) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6326
      if ( res /= 6326) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5578
      if ( res /= 5578) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2340
      if ( res /= 2340) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9185
      if ( res /= 9185) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4661
      if ( res /= 4661) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8830
      if ( res /= 8830) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2606
      if ( res /= 2606) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2725
      if ( res /= 2725) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4400
      if ( res /= 4400) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6162
      if ( res /= 6162) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6229
      if ( res /= 6229) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6549
      if ( res /= 6549) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2169
      if ( res /= 2169) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4190
      if ( res /= 4190) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_A, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",405
      if ( res /= 405) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2065
      if ( res /= 2065) then
        stop
      end if
      call createParticleConfig(P_b, P_Hh, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6816
      if ( res /= 6816) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghZ, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5474
      if ( res /= 5474) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1997
      if ( res /= 1997) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5951
      if ( res /= 5951) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4630
      if ( res /= 4630) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7172
      if ( res /= 7172) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9055
      if ( res /= 9055) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5824
      if ( res /= 5824) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5358
      if ( res /= 5358) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",206
      if ( res /= 206) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3967
      if ( res /= 3967) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9439
      if ( res /= 9439) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6087
      if ( res /= 6087) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4399
      if ( res /= 4399) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9204
      if ( res /= 9204) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",951
      if ( res /= 951) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7418
      if ( res /= 7418) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9879
      if ( res /= 9879) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",234
      if ( res /= 234) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8068
      if ( res /= 8068) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9887
      if ( res /= 9887) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_bAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1659
      if ( res /= 1659) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9271
      if ( res /= 9271) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghZAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4565
      if ( res /= 4565) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3940
      if ( res /= 3940) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_g, P_GPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10688
      if ( res /= 10688) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3135
      if ( res /= 3135) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3022
      if ( res /= 3022) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4023
      if ( res /= 4023) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6879
      if ( res /= 6879) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10059
      if ( res /= 10059) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2730
      if ( res /= 2730) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1740
      if ( res /= 1740) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1493
      if ( res /= 1493) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_Hl, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6553
      if ( res /= 6553) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4531
      if ( res /= 4531) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4660
      if ( res /= 4660) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghZ, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6607
      if ( res /= 6607) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10757
      if ( res /= 10757) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9392
      if ( res /= 9392) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8343
      if ( res /= 8343) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10438
      if ( res /= 10438) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",129
      if ( res /= 129) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghA, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3680
      if ( res /= 3680) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4281
      if ( res /= 4281) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9524
      if ( res /= 9524) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9927
      if ( res /= 9927) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6516
      if ( res /= 6516) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7923
      if ( res /= 7923) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9441
      if ( res /= 9441) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5401
      if ( res /= 5401) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2149
      if ( res /= 2149) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7345
      if ( res /= 7345) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5351
      if ( res /= 5351) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2436
      if ( res /= 2436) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9183
      if ( res /= 9183) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9740
      if ( res /= 9740) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1653
      if ( res /= 1653) then
        stop
      end if
      call createParticleConfig(P_u, P_Hh, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1492
      if ( res /= 1492) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7790
      if ( res /= 7790) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1499
      if ( res /= 1499) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6806
      if ( res /= 6806) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1603
      if ( res /= 1603) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_G0, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4236
      if ( res /= 4236) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9111
      if ( res /= 9111) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1352
      if ( res /= 1352) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6759
      if ( res /= 6759) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2822
      if ( res /= 2822) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",409
      if ( res /= 409) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Z, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",46
      if ( res /= 46) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7010
      if ( res /= 7010) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2416
      if ( res /= 2416) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3693
      if ( res /= 3693) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",68
      if ( res /= 68) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9431
      if ( res /= 9431) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6694
      if ( res /= 6694) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4099
      if ( res /= 4099) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1138
      if ( res /= 1138) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8450
      if ( res /= 8450) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2283
      if ( res /= 2283) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10316
      if ( res /= 10316) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8924
      if ( res /= 8924) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6509
      if ( res /= 6509) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",385
      if ( res /= 385) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7202
      if ( res /= 7202) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7749
      if ( res /= 7749) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2527
      if ( res /= 2527) then
        stop
      end if
      call createParticleConfig(P_g, P_Ha, P_G0, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10657
      if ( res /= 10657) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4972
      if ( res /= 4972) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghAAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",436
      if ( res /= 436) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWm, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4820
      if ( res /= 4820) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",513
      if ( res /= 513) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8445
      if ( res /= 8445) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4644
      if ( res /= 4644) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",529
      if ( res /= 529) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2738
      if ( res /= 2738) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_A, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4668
      if ( res /= 4668) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1902
      if ( res /= 1902) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5754
      if ( res /= 5754) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5398
      if ( res /= 5398) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1772
      if ( res /= 1772) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10174
      if ( res /= 10174) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2820
      if ( res /= 2820) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3642
      if ( res /= 3642) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3782
      if ( res /= 3782) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",830
      if ( res /= 830) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2860
      if ( res /= 2860) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10594
      if ( res /= 10594) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_Ha, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8801
      if ( res /= 8801) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9760
      if ( res /= 9760) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghAAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8051
      if ( res /= 8051) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",906
      if ( res /= 906) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5938
      if ( res /= 5938) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5125
      if ( res /= 5125) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2392
      if ( res /= 2392) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7751
      if ( res /= 7751) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2491
      if ( res /= 2491) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7492
      if ( res /= 7492) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8943
      if ( res /= 8943) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWpAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4109
      if ( res /= 4109) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3939
      if ( res /= 3939) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2870
      if ( res /= 2870) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8159
      if ( res /= 8159) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6431
      if ( res /= 6431) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7670
      if ( res /= 7670) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3502
      if ( res /= 3502) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HPlus, P_GMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9303
      if ( res /= 9303) then
        stop
      end if
      call createParticleConfig(P_gQ, P_g, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6587
      if ( res /= 6587) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWm, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5801
      if ( res /= 5801) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7237
      if ( res /= 7237) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_ghG, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",910
      if ( res /= 910) then
        stop
      end if
      call createParticleConfig(P_t, P_Ha, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1336
      if ( res /= 1336) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3004
      if ( res /= 3004) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",537
      if ( res /= 537) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10066
      if ( res /= 10066) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",833
      if ( res /= 833) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7318
      if ( res /= 7318) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_nu_e, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2358
      if ( res /= 2358) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7907
      if ( res /= 7907) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3593
      if ( res /= 3593) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3771
      if ( res /= 3771) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4341
      if ( res /= 4341) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4231
      if ( res /= 4231) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2558
      if ( res /= 2558) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_Hh, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5157
      if ( res /= 5157) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3627
      if ( res /= 3627) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_ePlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3122
      if ( res /= 3122) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10627
      if ( res /= 10627) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7039
      if ( res /= 7039) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HPlus, P_HMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5602
      if ( res /= 5602) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9522
      if ( res /= 9522) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10556
      if ( res /= 10556) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9408
      if ( res /= 9408) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7080
      if ( res /= 7080) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7403
      if ( res /= 7403) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWpAnti, P_HPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9134
      if ( res /= 9134) then
        stop
      end if
      call createParticleConfig(P_A, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6370
      if ( res /= 6370) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9402
      if ( res /= 9402) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1356
      if ( res /= 1356) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1896
      if ( res /= 1896) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9567
      if ( res /= 9567) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5894
      if ( res /= 5894) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1429
      if ( res /= 1429) then
        stop
      end if
      call createParticleConfig(P_AQ, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8466
      if ( res /= 8466) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5006
      if ( res /= 5006) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4317
      if ( res /= 4317) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9449
      if ( res /= 9449) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2890
      if ( res /= 2890) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",540
      if ( res /= 540) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghAAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",668
      if ( res /= 668) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10177
      if ( res /= 10177) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3001
      if ( res /= 3001) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3543
      if ( res /= 3543) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8635
      if ( res /= 8635) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7770
      if ( res /= 7770) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghAAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2652
      if ( res /= 2652) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2142
      if ( res /= 2142) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7903
      if ( res /= 7903) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10028
      if ( res /= 10028) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10389
      if ( res /= 10389) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9042
      if ( res /= 9042) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWpAnti, P_WPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8136
      if ( res /= 8136) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9089
      if ( res /= 9089) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2000
      if ( res /= 2000) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4653
      if ( res /= 4653) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_s, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2178
      if ( res /= 2178) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6992
      if ( res /= 6992) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10632
      if ( res /= 10632) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4153
      if ( res /= 4153) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2059
      if ( res /= 2059) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",389
      if ( res /= 389) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4954
      if ( res /= 4954) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3365
      if ( res /= 3365) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3282
      if ( res /= 3282) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",826
      if ( res /= 826) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1304
      if ( res /= 1304) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWmAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7411
      if ( res /= 7411) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3293
      if ( res /= 3293) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3840
      if ( res /= 3840) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1517
      if ( res /= 1517) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1573
      if ( res /= 1573) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",882
      if ( res /= 882) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4604
      if ( res /= 4604) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghZ, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2786
      if ( res /= 2786) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",350
      if ( res /= 350) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9409
      if ( res /= 9409) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10224
      if ( res /= 10224) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghZAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",973
      if ( res /= 973) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghZAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7353
      if ( res /= 7353) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6866
      if ( res /= 6866) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6616
      if ( res /= 6616) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4417
      if ( res /= 4417) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWmAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6748
      if ( res /= 6748) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",181
      if ( res /= 181) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8395
      if ( res /= 8395) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",212
      if ( res /= 212) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWp, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5257
      if ( res /= 5257) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6
      if ( res /= 6) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8245
      if ( res /= 8245) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6352
      if ( res /= 6352) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3118
      if ( res /= 3118) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1472
      if ( res /= 1472) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2691
      if ( res /= 2691) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9004
      if ( res /= 9004) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",873
      if ( res /= 873) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5362
      if ( res /= 5362) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3218
      if ( res /= 3218) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",792
      if ( res /= 792) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8016
      if ( res /= 8016) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",31
      if ( res /= 31) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8541
      if ( res /= 8541) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghZ, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5320
      if ( res /= 5320) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",439
      if ( res /= 439) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4204
      if ( res /= 4204) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2648
      if ( res /= 2648) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",624
      if ( res /= 624) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghA, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3433
      if ( res /= 3433) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9340
      if ( res /= 9340) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7957
      if ( res /= 7957) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6991
      if ( res /= 6991) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3588
      if ( res /= 3588) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6024
      if ( res /= 6024) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",781
      if ( res /= 781) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3463
      if ( res /= 3463) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2454
      if ( res /= 2454) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5073
      if ( res /= 5073) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10598
      if ( res /= 10598) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1751
      if ( res /= 1751) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2840
      if ( res /= 2840) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4958
      if ( res /= 4958) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWpAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6171
      if ( res /= 6171) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",77
      if ( res /= 77) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_A, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8124
      if ( res /= 8124) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1322
      if ( res /= 1322) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8185
      if ( res /= 8185) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8549
      if ( res /= 8549) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5095
      if ( res /= 5095) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7541
      if ( res /= 7541) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8115
      if ( res /= 8115) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",574
      if ( res /= 574) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7499
      if ( res /= 7499) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",528
      if ( res /= 528) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2019
      if ( res /= 2019) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2090
      if ( res /= 2090) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2808
      if ( res /= 2808) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6746
      if ( res /= 6746) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2172
      if ( res /= 2172) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2911
      if ( res /= 2911) then
        stop
      end if
      call createParticleConfig(P_gQ, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",888
      if ( res /= 888) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",562
      if ( res /= 562) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1538
      if ( res /= 1538) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6212
      if ( res /= 6212) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7461
      if ( res /= 7461) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3275
      if ( res /= 3275) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_A, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3971
      if ( res /= 3971) then
        stop
      end if
      call createParticleConfig(P_A, P_g, P_Z, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10636
      if ( res /= 10636) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4581
      if ( res /= 4581) then
        stop
      end if
      call createParticleConfig(P_g, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4948
      if ( res /= 4948) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8570
      if ( res /= 8570) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8936
      if ( res /= 8936) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWp, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3747
      if ( res /= 3747) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2479
      if ( res /= 2479) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4165
      if ( res /= 4165) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8100
      if ( res /= 8100) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5449
      if ( res /= 5449) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7557
      if ( res /= 7557) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3792
      if ( res /= 3792) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8617
      if ( res /= 8617) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghZ, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4572
      if ( res /= 4572) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10417
      if ( res /= 10417) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_Hl, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2612
      if ( res /= 2612) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5969
      if ( res /= 5969) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1786
      if ( res /= 1786) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6386
      if ( res /= 6386) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5367
      if ( res /= 5367) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7021
      if ( res /= 7021) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9552
      if ( res /= 9552) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6895
      if ( res /= 6895) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1836
      if ( res /= 1836) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_Z, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8755
      if ( res /= 8755) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3705
      if ( res /= 3705) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GPlus, P_GMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8661
      if ( res /= 8661) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10637
      if ( res /= 10637) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5629
      if ( res /= 5629) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_g, P_g, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10758
      if ( res /= 10758) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3262
      if ( res /= 3262) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10493
      if ( res /= 10493) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7817
      if ( res /= 7817) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2789
      if ( res /= 2789) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",144
      if ( res /= 144) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7680
      if ( res /= 7680) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4297
      if ( res /= 4297) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",444
      if ( res /= 444) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2817
      if ( res /= 2817) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10357
      if ( res /= 10357) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10601
      if ( res /= 10601) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9898
      if ( res /= 9898) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3395
      if ( res /= 3395) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8536
      if ( res /= 8536) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10748
      if ( res /= 10748) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8182
      if ( res /= 8182) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10304
      if ( res /= 10304) then
        stop
      end if
      call createParticleConfig(P_Z, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1172
      if ( res /= 1172) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10517
      if ( res /= 10517) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10511
      if ( res /= 10511) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HPlus, P_GMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3734
      if ( res /= 3734) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3594
      if ( res /= 3594) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8363
      if ( res /= 8363) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3075
      if ( res /= 3075) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1590
      if ( res /= 1590) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9102
      if ( res /= 9102) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10592
      if ( res /= 10592) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4749
      if ( res /= 4749) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6003
      if ( res /= 6003) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7025
      if ( res /= 7025) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10312
      if ( res /= 10312) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3846
      if ( res /= 3846) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_A, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6551
      if ( res /= 6551) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6854
      if ( res /= 6854) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3068
      if ( res /= 3068) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7726
      if ( res /= 7726) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",247
      if ( res /= 247) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6951
      if ( res /= 6951) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9991
      if ( res /= 9991) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3759
      if ( res /= 3759) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1541
      if ( res /= 1541) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2859
      if ( res /= 2859) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1434
      if ( res /= 1434) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",426
      if ( res /= 426) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6923
      if ( res /= 6923) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",708
      if ( res /= 708) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9823
      if ( res /= 9823) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1126
      if ( res /= 1126) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4568
      if ( res /= 4568) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",915
      if ( res /= 915) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9750
      if ( res /= 9750) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9382
      if ( res /= 9382) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2168
      if ( res /= 2168) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9963
      if ( res /= 9963) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2075
      if ( res /= 2075) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_A, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1633
      if ( res /= 1633) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghA, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4479
      if ( res /= 4479) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6372
      if ( res /= 6372) then
        stop
      end if
      call createParticleConfig(P_t, P_bAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1554
      if ( res /= 1554) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9954
      if ( res /= 9954) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9380
      if ( res /= 9380) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7228
      if ( res /= 7228) then
        stop
      end if
      call createParticleConfig(P_g, P_GMinus, P_g, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10562
      if ( res /= 10562) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9350
      if ( res /= 9350) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",848
      if ( res /= 848) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5657
      if ( res /= 5657) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2747
      if ( res /= 2747) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",824
      if ( res /= 824) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2588
      if ( res /= 2588) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1329
      if ( res /= 1329) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3922
      if ( res /= 3922) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7383
      if ( res /= 7383) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_G0, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2257
      if ( res /= 2257) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9493
      if ( res /= 9493) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3305
      if ( res /= 3305) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6557
      if ( res /= 6557) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3036
      if ( res /= 3036) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4
      if ( res /= 4) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4348
      if ( res /= 4348) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9071
      if ( res /= 9071) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1383
      if ( res /= 1383) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_Z, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3414
      if ( res /= 3414) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5858
      if ( res /= 5858) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5729
      if ( res /= 5729) then
        stop
      end if
      call createParticleConfig(P_Hh, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7203
      if ( res /= 7203) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3180
      if ( res /= 3180) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8491
      if ( res /= 8491) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4345
      if ( res /= 4345) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3885
      if ( res /= 3885) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9835
      if ( res /= 9835) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10249
      if ( res /= 10249) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2411
      if ( res /= 2411) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3309
      if ( res /= 3309) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2668
      if ( res /= 2668) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7869
      if ( res /= 7869) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",831
      if ( res /= 831) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9766
      if ( res /= 9766) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3773
      if ( res /= 3773) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",628
      if ( res /= 628) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4635
      if ( res /= 4635) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9200
      if ( res /= 9200) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9249
      if ( res /= 9249) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5107
      if ( res /= 5107) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8091
      if ( res /= 8091) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10036
      if ( res /= 10036) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3750
      if ( res /= 3750) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3364
      if ( res /= 3364) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7685
      if ( res /= 7685) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",648
      if ( res /= 648) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1780
      if ( res /= 1780) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4772
      if ( res /= 4772) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7109
      if ( res /= 7109) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2753
      if ( res /= 2753) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1696
      if ( res /= 1696) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",730
      if ( res /= 730) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8003
      if ( res /= 8003) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2238
      if ( res /= 2238) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7526
      if ( res /= 7526) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10023
      if ( res /= 10023) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6158
      if ( res /= 6158) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9639
      if ( res /= 9639) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10497
      if ( res /= 10497) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7438
      if ( res /= 7438) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9295
      if ( res /= 9295) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_A, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6225
      if ( res /= 6225) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWmAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6743
      if ( res /= 6743) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6196
      if ( res /= 6196) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4498
      if ( res /= 4498) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",255
      if ( res /= 255) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10622
      if ( res /= 10622) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1805
      if ( res /= 1805) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9417
      if ( res /= 9417) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10038
      if ( res /= 10038) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghZ, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8431
      if ( res /= 8431) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9083
      if ( res /= 9083) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8489
      if ( res /= 8489) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10154
      if ( res /= 10154) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6458
      if ( res /= 6458) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8822
      if ( res /= 8822) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1154
      if ( res /= 1154) then
        stop
      end if
      call createParticleConfig(P_s, P_cAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8354
      if ( res /= 8354) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",585
      if ( res /= 585) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2595
      if ( res /= 2595) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5184
      if ( res /= 5184) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10470
      if ( res /= 10470) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_b, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6471
      if ( res /= 6471) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8985
      if ( res /= 8985) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2217
      if ( res /= 2217) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7222
      if ( res /= 7222) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5271
      if ( res /= 5271) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7451
      if ( res /= 7451) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6070
      if ( res /= 6070) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",692
      if ( res /= 692) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4293
      if ( res /= 4293) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3707
      if ( res /= 3707) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1121
      if ( res /= 1121) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6893
      if ( res /= 6893) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3204
      if ( res /= 3204) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3354
      if ( res /= 3354) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7490
      if ( res /= 7490) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_nu_mu, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5458
      if ( res /= 5458) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",180
      if ( res /= 180) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3044
      if ( res /= 3044) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3278
      if ( res /= 3278) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7971
      if ( res /= 7971) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8200
      if ( res /= 8200) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10164
      if ( res /= 10164) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8893
      if ( res /= 8893) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3355
      if ( res /= 3355) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3760
      if ( res /= 3760) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6653
      if ( res /= 6653) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5853
      if ( res /= 5853) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8476
      if ( res /= 8476) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9296
      if ( res /= 9296) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9484
      if ( res /= 9484) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghWmAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3072
      if ( res /= 3072) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3450
      if ( res /= 3450) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4883
      if ( res /= 4883) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8044
      if ( res /= 8044) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2466
      if ( res /= 2466) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7962
      if ( res /= 7962) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7973
      if ( res /= 7973) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5942
      if ( res /= 5942) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4932
      if ( res /= 4932) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1241
      if ( res /= 1241) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4952
      if ( res /= 4952) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3403
      if ( res /= 3403) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",338
      if ( res /= 338) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWm, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2685
      if ( res /= 2685) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8274
      if ( res /= 8274) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3817
      if ( res /= 3817) then
        stop
      end if
      call createParticleConfig(P_gQ, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6731
      if ( res /= 6731) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3888
      if ( res /= 3888) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2400
      if ( res /= 2400) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4677
      if ( res /= 4677) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",598
      if ( res /= 598) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6076
      if ( res /= 6076) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10133
      if ( res /= 10133) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghWm, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1803
      if ( res /= 1803) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7915
      if ( res /= 7915) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6719
      if ( res /= 6719) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",431
      if ( res /= 431) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9245
      if ( res /= 9245) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWpAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1345
      if ( res /= 1345) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6812
      if ( res /= 6812) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5433
      if ( res /= 5433) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1114
      if ( res /= 1114) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WPlus, P_ghWm, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",732
      if ( res /= 732) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7532
      if ( res /= 7532) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2451
      if ( res /= 2451) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10427
      if ( res /= 10427) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8000
      if ( res /= 8000) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2048
      if ( res /= 2048) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWm, P_ghWmAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2295
      if ( res /= 2295) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10505
      if ( res /= 10505) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4943
      if ( res /= 4943) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1999
      if ( res /= 1999) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3532
      if ( res /= 3532) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5571
      if ( res /= 5571) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4920
      if ( res /= 4920) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5739
      if ( res /= 5739) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4602
      if ( res /= 4602) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5787
      if ( res /= 5787) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6480
      if ( res /= 6480) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2516
      if ( res /= 2516) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2264
      if ( res /= 2264) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10073
      if ( res /= 10073) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3026
      if ( res /= 3026) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HMinus, P_HPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5796
      if ( res /= 5796) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7423
      if ( res /= 7423) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1831
      if ( res /= 1831) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8788
      if ( res /= 8788) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1622
      if ( res /= 1622) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2928
      if ( res /= 2928) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghZ, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3243
      if ( res /= 3243) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2209
      if ( res /= 2209) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3236
      if ( res /= 3236) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5213
      if ( res /= 5213) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",985
      if ( res /= 985) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3168
      if ( res /= 3168) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2886
      if ( res /= 2886) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",735
      if ( res /= 735) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10200
      if ( res /= 10200) then
        stop
      end if
      call createParticleConfig(P_d, P_uAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1316
      if ( res /= 1316) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10350
      if ( res /= 10350) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",455
      if ( res /= 455) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5139
      if ( res /= 5139) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",442
      if ( res /= 442) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6394
      if ( res /= 6394) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10078
      if ( res /= 10078) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5439
      if ( res /= 5439) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4300
      if ( res /= 4300) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_nu_muAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",134
      if ( res /= 134) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9100
      if ( res /= 9100) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4669
      if ( res /= 4669) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WPlus, P_WMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",892
      if ( res /= 892) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",11
      if ( res /= 11) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",667
      if ( res /= 667) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7917
      if ( res /= 7917) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10120
      if ( res /= 10120) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10380
      if ( res /= 10380) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3639
      if ( res /= 3639) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3912
      if ( res /= 3912) then
        stop
      end if
      call createParticleConfig(P_b, P_GPlus, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5124
      if ( res /= 5124) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10717
      if ( res /= 10717) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4931
      if ( res /= 4931) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4473
      if ( res /= 4473) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3017
      if ( res /= 3017) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7791
      if ( res /= 7791) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6330
      if ( res /= 6330) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9729
      if ( res /= 9729) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4114
      if ( res /= 4114) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghA, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3867
      if ( res /= 3867) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9859
      if ( res /= 9859) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3088
      if ( res /= 3088) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9488
      if ( res /= 9488) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",709
      if ( res /= 709) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3263
      if ( res /= 3263) then
        stop
      end if
      call createParticleConfig(P_b, P_WPlus, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1313
      if ( res /= 1313) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1100
      if ( res /= 1100) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10321
      if ( res /= 10321) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",605
      if ( res /= 605) then
        stop
      end if
      call createParticleConfig(P_t, P_bAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3851
      if ( res /= 3851) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6112
      if ( res /= 6112) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",726
      if ( res /= 726) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8317
      if ( res /= 8317) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7631
      if ( res /= 7631) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8605
      if ( res /= 8605) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9649
      if ( res /= 9649) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2616
      if ( res /= 2616) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2653
      if ( res /= 2653) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3389
      if ( res /= 3389) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_nu_tau, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6558
      if ( res /= 6558) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2580
      if ( res /= 2580) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Z, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3849
      if ( res /= 3849) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6187
      if ( res /= 6187) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10365
      if ( res /= 10365) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3351
      if ( res /= 3351) then
        stop
      end if
      call createParticleConfig(P_u, P_dAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3656
      if ( res /= 3656) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2105
      if ( res /= 2105) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6584
      if ( res /= 6584) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3199
      if ( res /= 3199) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7277
      if ( res /= 7277) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8625
      if ( res /= 8625) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5756
      if ( res /= 5756) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghZ, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2857
      if ( res /= 2857) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1943
      if ( res /= 1943) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1956
      if ( res /= 1956) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5890
      if ( res /= 5890) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WMinus, P_WPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7627
      if ( res /= 7627) then
        stop
      end if
      call createParticleConfig(P_Hl, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4210
      if ( res /= 4210) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8989
      if ( res /= 8989) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",536
      if ( res /= 536) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_A, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4846
      if ( res /= 4846) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2267
      if ( res /= 2267) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4411
      if ( res /= 4411) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2512
      if ( res /= 2512) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2333
      if ( res /= 2333) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4416
      if ( res /= 4416) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2643
      if ( res /= 2643) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6699
      if ( res /= 6699) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9036
      if ( res /= 9036) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10155
      if ( res /= 10155) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2713
      if ( res /= 2713) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_A, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5056
      if ( res /= 5056) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWpAnti, P_ghWp, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6160
      if ( res /= 6160) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4973
      if ( res /= 4973) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7639
      if ( res /= 7639) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5566
      if ( res /= 5566) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10092
      if ( res /= 10092) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6897
      if ( res /= 6897) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4373
      if ( res /= 4373) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5203
      if ( res /= 5203) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1069
      if ( res /= 1069) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8984
      if ( res /= 8984) then
        stop
      end if
      call createParticleConfig(P_c, P_G0, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4220
      if ( res /= 4220) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",467
      if ( res /= 467) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3246
      if ( res /= 3246) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ePlus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3678
      if ( res /= 3678) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2731
      if ( res /= 2731) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9336
      if ( res /= 9336) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9087
      if ( res /= 9087) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8926
      if ( res /= 8926) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5174
      if ( res /= 5174) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_g, P_g, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10648
      if ( res /= 10648) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9277
      if ( res /= 9277) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7787
      if ( res /= 7787) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7372
      if ( res /= 7372) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2504
      if ( res /= 2504) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3006
      if ( res /= 3006) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1664
      if ( res /= 1664) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3154
      if ( res /= 3154) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4650
      if ( res /= 4650) then
        stop
      end if
      call createParticleConfig(P_Hh, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3659
      if ( res /= 3659) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9923
      if ( res /= 9923) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8584
      if ( res /= 8584) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6580
      if ( res /= 6580) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GMinus, P_ghWmAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10098
      if ( res /= 10098) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9709
      if ( res /= 9709) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7531
      if ( res /= 7531) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8861
      if ( res /= 8861) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7562
      if ( res /= 7562) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10238
      if ( res /= 10238) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2869
      if ( res /= 2869) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6140
      if ( res /= 6140) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7001
      if ( res /= 7001) then
        stop
      end if
      call createParticleConfig(P_ghG, P_ghGAnti, P_g, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3029
      if ( res /= 3029) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghZAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8077
      if ( res /= 8077) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7759
      if ( res /= 7759) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9602
      if ( res /= 9602) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3658
      if ( res /= 3658) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3343
      if ( res /= 3343) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1238
      if ( res /= 1238) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7713
      if ( res /= 7713) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2770
      if ( res /= 2770) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4694
      if ( res /= 4694) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GMinus, P_HPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10460
      if ( res /= 10460) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7721
      if ( res /= 7721) then
        stop
      end if
      call createParticleConfig(P_c, P_sAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8341
      if ( res /= 8341) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",590
      if ( res /= 590) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Z, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5338
      if ( res /= 5338) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1116
      if ( res /= 1116) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5702
      if ( res /= 5702) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7617
      if ( res /= 7617) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1876
      if ( res /= 1876) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3147
      if ( res /= 3147) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6039
      if ( res /= 6039) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",215
      if ( res /= 215) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4521
      if ( res /= 4521) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",612
      if ( res /= 612) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9460
      if ( res /= 9460) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9568
      if ( res /= 9568) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWmAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6679
      if ( res /= 6679) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1863
      if ( res /= 1863) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_A, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8621
      if ( res /= 8621) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghAAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6150
      if ( res /= 6150) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5178
      if ( res /= 5178) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3454
      if ( res /= 3454) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4651
      if ( res /= 4651) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6335
      if ( res /= 6335) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9468
      if ( res /= 9468) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",918
      if ( res /= 918) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1144
      if ( res /= 1144) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8518
      if ( res /= 8518) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6768
      if ( res /= 6768) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8645
      if ( res /= 8645) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWm, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1421
      if ( res /= 1421) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9425
      if ( res /= 9425) then
        stop
      end if
      call createParticleConfig(P_ghA, P_A, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5012
      if ( res /= 5012) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10373
      if ( res /= 10373) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6357
      if ( res /= 6357) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9418
      if ( res /= 9418) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4233
      if ( res /= 4233) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_tauMinus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5646
      if ( res /= 5646) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4138
      if ( res /= 4138) then
        stop
      end if
      call createParticleConfig(P_Hl, P_g, P_g, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10611
      if ( res /= 10611) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7029
      if ( res /= 7029) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1326
      if ( res /= 1326) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4570
      if ( res /= 4570) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1476
      if ( res /= 1476) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7994
      if ( res /= 7994) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",36
      if ( res /= 36) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Ha, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5784
      if ( res /= 5784) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7413
      if ( res /= 7413) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2871
      if ( res /= 2871) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZ, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7553
      if ( res /= 7553) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9384
      if ( res /= 9384) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8551
      if ( res /= 8551) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5769
      if ( res /= 5769) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1802
      if ( res /= 1802) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10466
      if ( res /= 10466) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2352
      if ( res /= 2352) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWmAnti, P_HMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8651
      if ( res /= 8651) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1852
      if ( res /= 1852) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6811
      if ( res /= 6811) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10101
      if ( res /= 10101) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7695
      if ( res /= 7695) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8253
      if ( res /= 8253) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",91
      if ( res /= 91) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4839
      if ( res /= 4839) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8463
      if ( res /= 8463) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1325
      if ( res /= 1325) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3264
      if ( res /= 3264) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2344
      if ( res /= 2344) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3353
      if ( res /= 3353) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3367
      if ( res /= 3367) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9362
      if ( res /= 9362) then
        stop
      end if
      call createParticleConfig(P_d, P_HPlus, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4374
      if ( res /= 4374) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7302
      if ( res /= 7302) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8778
      if ( res /= 8778) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6042
      if ( res /= 6042) then
        stop
      end if
      call createParticleConfig(P_u, P_A, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4842
      if ( res /= 4842) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5909
      if ( res /= 5909) then
        stop
      end if
      call createParticleConfig(P_gQ, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1302
      if ( res /= 1302) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9066
      if ( res /= 9066) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5463
      if ( res /= 5463) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10061
      if ( res /= 10061) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9916
      if ( res /= 9916) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4523
      if ( res /= 4523) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4600
      if ( res /= 4600) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7630
      if ( res /= 7630) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5873
      if ( res /= 5873) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6349
      if ( res /= 6349) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2999
      if ( res /= 2999) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4697
      if ( res /= 4697) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2195
      if ( res /= 2195) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8312
      if ( res /= 8312) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7756
      if ( res /= 7756) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4527
      if ( res /= 4527) then
        stop
      end if
      call createParticleConfig(P_g, P_A, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10509
      if ( res /= 10509) then
        stop
      end if
      call createParticleConfig(P_Ha, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3600
      if ( res /= 3600) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8887
      if ( res /= 8887) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9005
      if ( res /= 9005) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8436
      if ( res /= 8436) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7527
      if ( res /= 7527) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1309
      if ( res /= 1309) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7867
      if ( res /= 7867) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3455
      if ( res /= 3455) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2047
      if ( res /= 2047) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3432
      if ( res /= 3432) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10088
      if ( res /= 10088) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1147
      if ( res /= 1147) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8107
      if ( res /= 8107) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9644
      if ( res /= 9644) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9507
      if ( res /= 9507) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7378
      if ( res /= 7378) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_Z, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7326
      if ( res /= 7326) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",770
      if ( res /= 770) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3163
      if ( res /= 3163) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3903
      if ( res /= 3903) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8108
      if ( res /= 8108) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6619
      if ( res /= 6619) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",921
      if ( res /= 921) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6598
      if ( res /= 6598) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8443
      if ( res /= 8443) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3512
      if ( res /= 3512) then
        stop
      end if
      call createParticleConfig(P_c, P_sAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2185
      if ( res /= 2185) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8679
      if ( res /= 8679) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1655
      if ( res /= 1655) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4271
      if ( res /= 4271) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5993
      if ( res /= 5993) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_u, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6989
      if ( res /= 6989) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2310
      if ( res /= 2310) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9144
      if ( res /= 9144) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4243
      if ( res /= 4243) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10097
      if ( res /= 10097) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10697
      if ( res /= 10697) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5740
      if ( res /= 5740) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7979
      if ( res /= 7979) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3281
      if ( res /= 3281) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2167
      if ( res /= 2167) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Ha, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",84
      if ( res /= 84) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8075
      if ( res /= 8075) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8658
      if ( res /= 8658) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3974
      if ( res /= 3974) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5211
      if ( res /= 5211) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_WMinus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7839
      if ( res /= 7839) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9508
      if ( res /= 9508) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2766
      if ( res /= 2766) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5520
      if ( res /= 5520) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghZAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6177
      if ( res /= 6177) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6575
      if ( res /= 6575) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2998
      if ( res /= 2998) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4862
      if ( res /= 4862) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9831
      if ( res /= 9831) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3483
      if ( res /= 3483) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7873
      if ( res /= 7873) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7084
      if ( res /= 7084) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2417
      if ( res /= 2417) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8403
      if ( res /= 8403) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7207
      if ( res /= 7207) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1435
      if ( res /= 1435) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6488
      if ( res /= 6488) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6544
      if ( res /= 6544) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8014
      if ( res /= 8014) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_Hh, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7767
      if ( res /= 7767) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_Ha, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5907
      if ( res /= 5907) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1104
      if ( res /= 1104) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3436
      if ( res /= 3436) then
        stop
      end if
      call createParticleConfig(P_Ha, P_g, P_g, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10765
      if ( res /= 10765) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Z, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5302
      if ( res /= 5302) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9132
      if ( res /= 9132) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2884
      if ( res /= 2884) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",140
      if ( res /= 140) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7417
      if ( res /= 7417) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",980
      if ( res /= 980) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8389
      if ( res /= 8389) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2346
      if ( res /= 2346) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4176
      if ( res /= 4176) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2123
      if ( res /= 2123) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1960
      if ( res /= 1960) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7674
      if ( res /= 7674) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2224
      if ( res /= 2224) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2585
      if ( res /= 2585) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",666
      if ( res /= 666) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2809
      if ( res /= 2809) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7421
      if ( res /= 7421) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8387
      if ( res /= 8387) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_WPlus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6068
      if ( res /= 6068) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2677
      if ( res /= 2677) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6242
      if ( res /= 6242) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",316
      if ( res /= 316) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",425
      if ( res /= 425) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5242
      if ( res /= 5242) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",356
      if ( res /= 356) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8761
      if ( res /= 8761) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8920
      if ( res /= 8920) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZ, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5704
      if ( res /= 5704) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2452
      if ( res /= 2452) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",816
      if ( res /= 816) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4198
      if ( res /= 4198) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3390
      if ( res /= 3390) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1998
      if ( res /= 1998) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7606
      if ( res /= 7606) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8968
      if ( res /= 8968) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3007
      if ( res /= 3007) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3274
      if ( res /= 3274) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4592
      if ( res /= 4592) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4597
      if ( res /= 4597) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5137
      if ( res /= 5137) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6707
      if ( res /= 6707) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9583
      if ( res /= 9583) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Z, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2917
      if ( res /= 2917) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3699
      if ( res /= 3699) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8293
      if ( res /= 8293) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8325
      if ( res /= 8325) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4676
      if ( res /= 4676) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_Z, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",705
      if ( res /= 705) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8624
      if ( res /= 8624) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8338
      if ( res /= 8338) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5220
      if ( res /= 5220) then
        stop
      end if
      call createParticleConfig(P_g, P_ghG, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7566
      if ( res /= 7566) then
        stop
      end if
      call createParticleConfig(P_AQ, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2373
      if ( res /= 2373) then
        stop
      end if
      call createParticleConfig(P_ghA, P_A, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7439
      if ( res /= 7439) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",717
      if ( res /= 717) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4283
      if ( res /= 4283) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghWp, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5747
      if ( res /= 5747) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5183
      if ( res /= 5183) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6210
      if ( res /= 6210) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1958
      if ( res /= 1958) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5737
      if ( res /= 5737) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWp, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7513
      if ( res /= 7513) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",460
      if ( res /= 460) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6668
      if ( res /= 6668) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8477
      if ( res /= 8477) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZ, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",375
      if ( res /= 375) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",366
      if ( res /= 366) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3116
      if ( res /= 3116) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3380
      if ( res /= 3380) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3350
      if ( res /= 3350) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5510
      if ( res /= 5510) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8135
      if ( res /= 8135) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghZAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5043
      if ( res /= 5043) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9446
      if ( res /= 9446) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5988
      if ( res /= 5988) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5171
      if ( res /= 5171) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7386
      if ( res /= 7386) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2445
      if ( res /= 2445) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8719
      if ( res /= 8719) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9628
      if ( res /= 9628) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4026
      if ( res /= 4026) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9553
      if ( res /= 9553) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6552
      if ( res /= 6552) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",476
      if ( res /= 476) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2055
      if ( res /= 2055) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5719
      if ( res /= 5719) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10057
      if ( res /= 10057) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2025
      if ( res /= 2025) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8675
      if ( res /= 8675) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3089
      if ( res /= 3089) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8183
      if ( res /= 8183) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7538
      if ( res /= 7538) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2225
      if ( res /= 2225) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6654
      if ( res /= 6654) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4217
      if ( res /= 4217) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3921
      if ( res /= 3921) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2159
      if ( res /= 2159) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5115
      if ( res /= 5115) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10339
      if ( res /= 10339) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3914
      if ( res /= 3914) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6289
      if ( res /= 6289) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5375
      if ( res /= 5375) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4854
      if ( res /= 4854) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1949
      if ( res /= 1949) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8592
      if ( res /= 8592) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2140
      if ( res /= 2140) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2371
      if ( res /= 2371) then
        stop
      end if
      call createParticleConfig(P_A, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3087
      if ( res /= 3087) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3578
      if ( res /= 3578) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5625
      if ( res /= 5625) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4387
      if ( res /= 4387) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6085
      if ( res /= 6085) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4587
      if ( res /= 4587) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9830
      if ( res /= 9830) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5312
      if ( res /= 5312) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9016
      if ( res /= 9016) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_muMinus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5581
      if ( res /= 5581) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2112
      if ( res /= 2112) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9064
      if ( res /= 9064) then
        stop
      end if
      call createParticleConfig(P_Ha, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5618
      if ( res /= 5618) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5437
      if ( res /= 5437) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5906
      if ( res /= 5906) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10026
      if ( res /= 10026) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_tauPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2617
      if ( res /= 2617) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4826
      if ( res /= 4826) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1992
      if ( res /= 1992) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2507
      if ( res /= 2507) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4038
      if ( res /= 4038) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9397
      if ( res /= 9397) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4591
      if ( res /= 4591) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8226
      if ( res /= 8226) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",675
      if ( res /= 675) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",274
      if ( res /= 274) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4358
      if ( res /= 4358) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",609
      if ( res /= 609) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8150
      if ( res /= 8150) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9234
      if ( res /= 9234) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9109
      if ( res /= 9109) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8670
      if ( res /= 8670) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8058
      if ( res /= 8058) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghA, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2883
      if ( res /= 2883) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1266
      if ( res /= 1266) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10234
      if ( res /= 10234) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1826
      if ( res /= 1826) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2734
      if ( res /= 2734) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3907
      if ( res /= 3907) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",263
      if ( res /= 263) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1221
      if ( res /= 1221) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6209
      if ( res /= 6209) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5298
      if ( res /= 5298) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1255
      if ( res /= 1255) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3531
      if ( res /= 3531) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1076
      if ( res /= 1076) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4035
      if ( res /= 4035) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",464
      if ( res /= 464) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",27
      if ( res /= 27) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2759
      if ( res /= 2759) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3804
      if ( res /= 3804) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1231
      if ( res /= 1231) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2407
      if ( res /= 2407) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7014
      if ( res /= 7014) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5795
      if ( res /= 5795) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7175
      if ( res /= 7175) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",105
      if ( res /= 105) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2818
      if ( res /= 2818) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5275
      if ( res /= 5275) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",421
      if ( res /= 421) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9342
      if ( res /= 9342) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6131
      if ( res /= 6131) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3999
      if ( res /= 3999) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7980
      if ( res /= 7980) then
        stop
      end if
      call createParticleConfig(P_A, P_ghAAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8223
      if ( res /= 8223) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4656
      if ( res /= 4656) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWmAnti, P_WMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8756
      if ( res /= 8756) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7267
      if ( res /= 7267) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8256
      if ( res /= 8256) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8699
      if ( res /= 8699) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10468
      if ( res /= 10468) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1415
      if ( res /= 1415) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9652
      if ( res /= 9652) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1319
      if ( res /= 1319) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4320
      if ( res /= 4320) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5166
      if ( res /= 5166) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9814
      if ( res /= 9814) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3819
      if ( res /= 3819) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WMinus, P_WPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9854
      if ( res /= 9854) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1983
      if ( res /= 1983) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2563
      if ( res /= 2563) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8144
      if ( res /= 8144) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1537
      if ( res /= 1537) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2716
      if ( res /= 2716) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8359
      if ( res /= 8359) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2922
      if ( res /= 2922) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",940
      if ( res /= 940) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9257
      if ( res /= 9257) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2179
      if ( res /= 2179) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8288
      if ( res /= 8288) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",211
      if ( res /= 211) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6755
      if ( res /= 6755) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GMinus, P_HPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3352
      if ( res /= 3352) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4566
      if ( res /= 4566) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1083
      if ( res /= 1083) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6843
      if ( res /= 6843) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6040
      if ( res /= 6040) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3094
      if ( res /= 3094) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6166
      if ( res /= 6166) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10289
      if ( res /= 10289) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",454
      if ( res /= 454) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2828
      if ( res /= 2828) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2297
      if ( res /= 2297) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghAAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2838
      if ( res /= 2838) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8347
      if ( res /= 8347) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9825
      if ( res /= 9825) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5330
      if ( res /= 5330) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3905
      if ( res /= 3905) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HPlus, P_GMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1809
      if ( res /= 1809) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9160
      if ( res /= 9160) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3871
      if ( res /= 3871) then
        stop
      end if
      call createParticleConfig(P_Hh, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7317
      if ( res /= 7317) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6217
      if ( res /= 6217) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9022
      if ( res /= 9022) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9096
      if ( res /= 9096) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5468
      if ( res /= 5468) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4196
      if ( res /= 4196) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8553
      if ( res /= 8553) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1568
      if ( res /= 1568) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10613
      if ( res /= 10613) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9827
      if ( res /= 9827) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",351
      if ( res /= 351) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7036
      if ( res /= 7036) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7110
      if ( res /= 7110) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",568
      if ( res /= 568) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2799
      if ( res /= 2799) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8198
      if ( res /= 8198) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4808
      if ( res /= 4808) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5262
      if ( res /= 5262) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWmAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3644
      if ( res /= 3644) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4352
      if ( res /= 4352) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3681
      if ( res /= 3681) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4517
      if ( res /= 4517) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWmAnti, P_WMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4891
      if ( res /= 4891) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2477
      if ( res /= 2477) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9131
      if ( res /= 9131) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8481
      if ( res /= 8481) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3964
      if ( res /= 3964) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2161
      if ( res /= 2161) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10050
      if ( res /= 10050) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3057
      if ( res /= 3057) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3676
      if ( res /= 3676) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5405
      if ( res /= 5405) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6252
      if ( res /= 6252) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7672
      if ( res /= 7672) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4140
      if ( res /= 4140) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4879
      if ( res /= 4879) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GMinus, P_GPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5482
      if ( res /= 5482) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1769
      if ( res /= 1769) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6482
      if ( res /= 6482) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6577
      if ( res /= 6577) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_u, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",374
      if ( res /= 374) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWmAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5222
      if ( res /= 5222) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9403
      if ( res /= 9403) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10510
      if ( res /= 10510) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2835
      if ( res /= 2835) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7342
      if ( res /= 7342) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_A, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2378
      if ( res /= 2378) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9949
      if ( res /= 9949) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5555
      if ( res /= 5555) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1086
      if ( res /= 1086) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",407
      if ( res /= 407) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6091
      if ( res /= 6091) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_nu_eAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8671
      if ( res /= 8671) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10352
      if ( res /= 10352) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_tauPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2904
      if ( res /= 2904) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9660
      if ( res /= 9660) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6341
      if ( res /= 6341) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3097
      if ( res /= 3097) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6030
      if ( res /= 6030) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3978
      if ( res /= 3978) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GMinus, P_ghWp, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",674
      if ( res /= 674) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10122
      if ( res /= 10122) then
        stop
      end if
      call createParticleConfig(P_Z, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4089
      if ( res /= 4089) then
        stop
      end if
      call createParticleConfig(P_d, P_G0, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4240
      if ( res /= 4240) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3402
      if ( res /= 3402) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8741
      if ( res /= 8741) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Ha, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7033
      if ( res /= 7033) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5052
      if ( res /= 5052) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4564
      if ( res /= 4564) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9752
      if ( res /= 9752) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9943
      if ( res /= 9943) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10425
      if ( res /= 10425) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2493
      if ( res /= 2493) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4991
      if ( res /= 4991) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_Hl, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7325
      if ( res /= 7325) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8419
      if ( res /= 8419) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10052
      if ( res /= 10052) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3066
      if ( res /= 3066) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5910
      if ( res /= 5910) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1977
      if ( res /= 1977) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3643
      if ( res /= 3643) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",861
      if ( res /= 861) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8117
      if ( res /= 8117) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9960
      if ( res /= 9960) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3370
      if ( res /= 3370) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3994
      if ( res /= 3994) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2665
      if ( res /= 2665) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5304
      if ( res /= 5304) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWm, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7159
      if ( res /= 7159) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2841
      if ( res /= 2841) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_b, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2244
      if ( res /= 2244) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6110
      if ( res /= 6110) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9166
      if ( res /= 9166) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5649
      if ( res /= 5649) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghA, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",729
      if ( res /= 729) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2729
      if ( res /= 2729) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1040
      if ( res /= 1040) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_d, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7727
      if ( res /= 7727) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9870
      if ( res /= 9870) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8328
      if ( res /= 8328) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1419
      if ( res /= 1419) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HPlus, P_HMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1216
      if ( res /= 1216) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4324
      if ( res /= 4324) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1873
      if ( res /= 1873) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7103
      if ( res /= 7103) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5087
      if ( res /= 5087) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5041
      if ( res /= 5041) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2902
      if ( res /= 2902) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7889
      if ( res /= 7889) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6676
      if ( res /= 6676) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3249
      if ( res /= 3249) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9812
      if ( res /= 9812) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4957
      if ( res /= 4957) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7327
      if ( res /= 7327) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4432
      if ( res /= 4432) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4779
      if ( res /= 4779) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5391
      if ( res /= 5391) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3805
      if ( res /= 3805) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2575
      if ( res /= 2575) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GPlus, P_GMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6188
      if ( res /= 6188) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6915
      if ( res /= 6915) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5143
      if ( res /= 5143) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4529
      if ( res /= 4529) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3184
      if ( res /= 3184) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3982
      if ( res /= 3982) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1186
      if ( res /= 1186) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2496
      if ( res /= 2496) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10751
      if ( res /= 10751) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WMinus, P_ghWmAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6132
      if ( res /= 6132) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWmAnti, P_ghWm, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1024
      if ( res /= 1024) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3178
      if ( res /= 3178) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",723
      if ( res /= 723) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9182
      if ( res /= 9182) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_Ha, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",989
      if ( res /= 989) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2444
      if ( res /= 2444) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6554
      if ( res /= 6554) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6499
      if ( res /= 6499) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWmAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8050
      if ( res /= 8050) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9154
      if ( res /= 9154) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",600
      if ( res /= 600) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7555
      if ( res /= 7555) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3789
      if ( res /= 3789) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9129
      if ( res /= 9129) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghWmAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5306
      if ( res /= 5306) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",862
      if ( res /= 862) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8900
      if ( res /= 8900) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5778
      if ( res /= 5778) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",269
      if ( res /= 269) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3651
      if ( res /= 3651) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7565
      if ( res /= 7565) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2187
      if ( res /= 2187) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7697
      if ( res /= 7697) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1532
      if ( res /= 1532) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWpAnti, P_ghWp, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5871
      if ( res /= 5871) then
        stop
      end if
      call createParticleConfig(P_t, P_A, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",893
      if ( res /= 893) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2659
      if ( res /= 2659) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6683
      if ( res /= 6683) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9585
      if ( res /= 9585) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5755
      if ( res /= 5755) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4975
      if ( res /= 4975) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9090
      if ( res /= 9090) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2560
      if ( res /= 2560) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_sAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5503
      if ( res /= 5503) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7718
      if ( res /= 7718) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8516
      if ( res /= 8516) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6184
      if ( res /= 6184) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10446
      if ( res /= 10446) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7092
      if ( res /= 7092) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5996
      if ( res /= 5996) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10626
      if ( res /= 10626) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2703
      if ( res /= 2703) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8011
      if ( res /= 8011) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7161
      if ( res /= 7161) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_A, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4277
      if ( res /= 4277) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2514
      if ( res /= 2514) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1257
      if ( res /= 1257) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9861
      if ( res /= 9861) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6287
      if ( res /= 6287) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7772
      if ( res /= 7772) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8073
      if ( res /= 8073) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3665
      if ( res /= 3665) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghAAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5813
      if ( res /= 5813) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_nu_tau, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4800
      if ( res /= 4800) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9213
      if ( res /= 9213) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7002
      if ( res /= 7002) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7634
      if ( res /= 7634) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4055
      if ( res /= 4055) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghWmAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3589
      if ( res /= 3589) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1428
      if ( res /= 1428) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5244
      if ( res /= 5244) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5999
      if ( res /= 5999) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_G0, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1074
      if ( res /= 1074) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8613
      if ( res /= 8613) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2948
      if ( res /= 2948) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3755
      if ( res /= 3755) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2231
      if ( res /= 2231) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10692
      if ( res /= 10692) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9901
      if ( res /= 9901) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1945
      if ( res /= 1945) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1726
      if ( res /= 1726) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4120
      if ( res /= 4120) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_cAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5309
      if ( res /= 5309) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5319
      if ( res /= 5319) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9736
      if ( res /= 9736) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8361
      if ( res /= 8361) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1485
      if ( res /= 1485) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1767
      if ( res /= 1767) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8555
      if ( res /= 8555) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10251
      if ( res /= 10251) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7354
      if ( res /= 7354) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghWp, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2566
      if ( res /= 2566) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWpAnti, P_ghWp, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10008
      if ( res /= 10008) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",187
      if ( res /= 187) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7625
      if ( res /= 7625) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8405
      if ( res /= 8405) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9388
      if ( res /= 9388) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3653
      if ( res /= 3653) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4079
      if ( res /= 4079) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10103
      if ( res /= 10103) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4207
      if ( res /= 4207) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8103
      if ( res /= 8103) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8664
      if ( res /= 8664) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",162
      if ( res /= 162) then
        stop
      end if
      call createParticleConfig(P_Hh, P_g, P_g, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10500
      if ( res /= 10500) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7844
      if ( res /= 7844) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2607
      if ( res /= 2607) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5208
      if ( res /= 5208) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2974
      if ( res /= 2974) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8001
      if ( res /= 8001) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1350
      if ( res /= 1350) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9955
      if ( res /= 9955) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10149
      if ( res /= 10149) then
        stop
      end if
      call createParticleConfig(P_g, P_Hh, P_Hh, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10686
      if ( res /= 10686) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2779
      if ( res /= 2779) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8534
      if ( res /= 8534) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7779
      if ( res /= 7779) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8879
      if ( res /= 8879) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WPlus, P_WMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8113
      if ( res /= 8113) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7447
      if ( res /= 7447) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7862
      if ( res /= 7862) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6680
      if ( res /= 6680) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5011
      if ( res /= 5011) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4946
      if ( res /= 4946) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4610
      if ( res /= 4610) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8083
      if ( res /= 8083) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3475
      if ( res /= 3475) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",393
      if ( res /= 393) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",911
      if ( res /= 911) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7891
      if ( res /= 7891) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9941
      if ( res /= 9941) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2359
      if ( res /= 2359) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7035
      if ( res /= 7035) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2166
      if ( res /= 2166) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7396
      if ( res /= 7396) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8998
      if ( res /= 8998) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2649
      if ( res /= 2649) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",599
      if ( res /= 599) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1900
      if ( res /= 1900) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6293
      if ( res /= 6293) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghA, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",836
      if ( res /= 836) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWp, P_GMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8946
      if ( res /= 8946) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8318
      if ( res /= 8318) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_Z, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8102
      if ( res /= 8102) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2151
      if ( res /= 2151) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10586
      if ( res /= 10586) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",582
      if ( res /= 582) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5854
      if ( res /= 5854) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7706
      if ( res /= 7706) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5039
      if ( res /= 5039) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghA, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7360
      if ( res /= 7360) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2498
      if ( res /= 2498) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4799
      if ( res /= 4799) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8550
      if ( res /= 8550) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",719
      if ( res /= 719) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7205
      if ( res /= 7205) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1233
      if ( res /= 1233) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2970
      if ( res /= 2970) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghZAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4999
      if ( res /= 4999) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8496
      if ( res /= 8496) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7219
      if ( res /= 7219) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8694
      if ( res /= 8694) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2946
      if ( res /= 2946) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7487
      if ( res /= 7487) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghZAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8600
      if ( res /= 8600) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6961
      if ( res /= 6961) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1744
      if ( res /= 1744) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",679
      if ( res /= 679) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghAAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7420
      if ( res /= 7420) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6964
      if ( res /= 6964) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1516
      if ( res /= 1516) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2836
      if ( res /= 2836) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8118
      if ( res /= 8118) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9095
      if ( res /= 9095) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3546
      if ( res /= 3546) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7221
      if ( res /= 7221) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5231
      if ( res /= 5231) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1115
      if ( res /= 1115) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2972
      if ( res /= 2972) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5893
      if ( res /= 5893) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3233
      if ( res /= 3233) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GMinus, P_HPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4627
      if ( res /= 4627) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6422
      if ( res /= 6422) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10094
      if ( res /= 10094) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1124
      if ( res /= 1124) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6044
      if ( res /= 6044) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3213
      if ( res /= 3213) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_eMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8232
      if ( res /= 8232) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4116
      if ( res /= 4116) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3735
      if ( res /= 3735) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3144
      if ( res /= 3144) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10479
      if ( res /= 10479) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9457
      if ( res /= 9457) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2341
      if ( res /= 2341) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Z, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3533
      if ( res /= 3533) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3423
      if ( res /= 3423) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5616
      if ( res /= 5616) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9889
      if ( res /= 9889) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10492
      if ( res /= 10492) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10359
      if ( res /= 10359) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",463
      if ( res /= 463) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7551
      if ( res /= 7551) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2903
      if ( res /= 2903) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1041
      if ( res /= 1041) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9059
      if ( res /= 9059) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7902
      if ( res /= 7902) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6535
      if ( res /= 6535) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4678
      if ( res /= 4678) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8915
      if ( res /= 8915) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3438
      if ( res /= 3438) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8992
      if ( res /= 8992) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7707
      if ( res /= 7707) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",360
      if ( res /= 360) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4368
      if ( res /= 4368) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2876
      if ( res /= 2876) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWpAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9
      if ( res /= 9) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2415
      if ( res /= 2415) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",845
      if ( res /= 845) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8167
      if ( res /= 8167) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_Z, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3641
      if ( res /= 3641) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9427
      if ( res /= 9427) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4221
      if ( res /= 4221) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_WPlus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1851
      if ( res /= 1851) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",167
      if ( res /= 167) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4040
      if ( res /= 4040) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6912
      if ( res /= 6912) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7622
      if ( res /= 7622) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9179
      if ( res /= 9179) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4506
      if ( res /= 4506) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6129
      if ( res /= 6129) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3124
      if ( res /= 3124) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7453
      if ( res /= 7453) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2250
      if ( res /= 2250) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8750
      if ( res /= 8750) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1713
      if ( res /= 1713) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10681
      if ( res /= 10681) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",860
      if ( res /= 860) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8791
      if ( res /= 8791) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4723
      if ( res /= 4723) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8730
      if ( res /= 8730) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7128
      if ( res /= 7128) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6627
      if ( res /= 6627) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5161
      if ( res /= 5161) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_dAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",415
      if ( res /= 415) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",411
      if ( res /= 411) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5164
      if ( res /= 5164) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6953
      if ( res /= 6953) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7195
      if ( res /= 7195) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1906
      if ( res /= 1906) then
        stop
      end if
      call createParticleConfig(P_A, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3295
      if ( res /= 3295) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9974
      if ( res /= 9974) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4941
      if ( res /= 4941) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3267
      if ( res /= 3267) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1355
      if ( res /= 1355) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1791
      if ( res /= 1791) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9456
      if ( res /= 9456) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2232
      if ( res /= 2232) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5370
      if ( res /= 5370) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5001
      if ( res /= 5001) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3957
      if ( res /= 3957) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9375
      if ( res /= 9375) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5305
      if ( res /= 5305) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3093
      if ( res /= 3093) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5542
      if ( res /= 5542) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghZ, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2203
      if ( res /= 2203) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9149
      if ( res /= 9149) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2755
      if ( res /= 2755) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",944
      if ( res /= 944) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_g, P_WPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10763
      if ( res /= 10763) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9599
      if ( res /= 9599) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6202
      if ( res /= 6202) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1064
      if ( res /= 1064) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2845
      if ( res /= 2845) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2413
      if ( res /= 2413) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_nu_muAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7604
      if ( res /= 7604) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",638
      if ( res /= 638) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6780
      if ( res /= 6780) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9028
      if ( res /= 9028) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4594
      if ( res /= 4594) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",135
      if ( res /= 135) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWm, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2455
      if ( res /= 2455) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5773
      if ( res /= 5773) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5286
      if ( res /= 5286) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghWm, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6364
      if ( res /= 6364) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6929
      if ( res /= 6929) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6427
      if ( res /= 6427) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4365
      if ( res /= 4365) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4691
      if ( res /= 4691) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9969
      if ( res /= 9969) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6611
      if ( res /= 6611) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3422
      if ( res /= 3422) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWm, P_ghWmAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2032
      if ( res /= 2032) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10769
      if ( res /= 10769) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",806
      if ( res /= 806) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8472
      if ( res /= 8472) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1388
      if ( res /= 1388) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7644
      if ( res /= 7644) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7211
      if ( res /= 7211) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3710
      if ( res /= 3710) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1578
      if ( res /= 1578) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_nu_mu, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5016
      if ( res /= 5016) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1258
      if ( res /= 1258) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8482
      if ( res /= 8482) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10058
      if ( res /= 10058) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghAAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7569
      if ( res /= 7569) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3406
      if ( res /= 3406) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2620
      if ( res /= 2620) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7708
      if ( res /= 7708) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7196
      if ( res /= 7196) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7262
      if ( res /= 7262) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6455
      if ( res /= 6455) then
        stop
      end if
      call createParticleConfig(P_Ha, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3394
      if ( res /= 3394) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9956
      if ( res /= 9956) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3417
      if ( res /= 3417) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5363
      if ( res /= 5363) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10113
      if ( res /= 10113) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8292
      if ( res /= 8292) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",295
      if ( res /= 295) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4238
      if ( res /= 4238) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8531
      if ( res /= 8531) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9433
      if ( res /= 9433) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1095
      if ( res /= 1095) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9424
      if ( res /= 9424) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7488
      if ( res /= 7488) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",60
      if ( res /= 60) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4084
      if ( res /= 4084) then
        stop
      end if
      call createParticleConfig(P_A, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5591
      if ( res /= 5591) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5932
      if ( res /= 5932) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2721
      if ( res /= 2721) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_tauMinus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4870
      if ( res /= 4870) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10225
      if ( res /= 10225) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4259
      if ( res /= 4259) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5596
      if ( res /= 5596) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8036
      if ( res /= 8036) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_muPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1170
      if ( res /= 1170) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5715
      if ( res /= 5715) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4992
      if ( res /= 4992) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1260
      if ( res /= 1260) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",523
      if ( res /= 523) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3537
      if ( res /= 3537) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",519
      if ( res /= 519) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9130
      if ( res /= 9130) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10529
      if ( res /= 10529) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",874
      if ( res /= 874) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7355
      if ( res /= 7355) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9453
      if ( res /= 9453) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1787
      if ( res /= 1787) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8224
      if ( res /= 8224) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6650
      if ( res /= 6650) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9435
      if ( res /= 9435) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9742
      if ( res /= 9742) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4286
      if ( res /= 4286) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9627
      if ( res /= 9627) then
        stop
      end if
      call createParticleConfig(P_A, P_nu_muAnti, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9808
      if ( res /= 9808) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3328
      if ( res /= 3328) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3112
      if ( res /= 3112) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8262
      if ( res /= 8262) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10422
      if ( res /= 10422) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4222
      if ( res /= 4222) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4104
      if ( res /= 4104) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3435
      if ( res /= 3435) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9147
      if ( res /= 9147) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9401
      if ( res /= 9401) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",239
      if ( res /= 239) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10244
      if ( res /= 10244) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3175
      if ( res /= 3175) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5230
      if ( res /= 5230) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10265
      if ( res /= 10265) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5475
      if ( res /= 5475) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GMinus, P_HPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10236
      if ( res /= 10236) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4425
      if ( res /= 4425) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8840
      if ( res /= 8840) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5674
      if ( res /= 5674) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3073
      if ( res /= 3073) then
        stop
      end if
      call createParticleConfig(P_ghA, P_A, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5857
      if ( res /= 5857) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9964
      if ( res /= 9964) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4735
      if ( res /= 4735) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",844
      if ( res /= 844) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghAAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",313
      if ( res /= 313) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3054
      if ( res /= 3054) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWpAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7027
      if ( res /= 7027) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9228
      if ( res /= 9228) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6946
      if ( res /= 6946) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3024
      if ( res /= 3024) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",359
      if ( res /= 359) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2709
      if ( res /= 2709) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1785
      if ( res /= 1785) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4149
      if ( res /= 4149) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghZAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6790
      if ( res /= 6790) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6497
      if ( res /= 6497) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9292
      if ( res /= 9292) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",578
      if ( res /= 578) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6387
      if ( res /= 6387) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZ, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4215
      if ( res /= 4215) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10615
      if ( res /= 10615) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5658
      if ( res /= 5658) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3257
      if ( res /= 3257) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6741
      if ( res /= 6741) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",97
      if ( res /= 97) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8759
      if ( res /= 8759) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7298
      if ( res /= 7298) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1605
      if ( res /= 1605) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5804
      if ( res /= 5804) then
        stop
      end if
      call createParticleConfig(P_d, P_uAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3977
      if ( res /= 3977) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6986
      if ( res /= 6986) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6599
      if ( res /= 6599) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6264
      if ( res /= 6264) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1667
      if ( res /= 1667) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8677
      if ( res /= 8677) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8833
      if ( res /= 8833) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3473
      if ( res /= 3473) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4634
      if ( res /= 4634) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1582
      if ( res /= 1582) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1709
      if ( res /= 1709) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7398
      if ( res /= 7398) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6218
      if ( res /= 6218) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4556
      if ( res /= 4556) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9226
      if ( res /= 9226) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_Z, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3786
      if ( res /= 3786) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8975
      if ( res /= 8975) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6906
      if ( res /= 6906) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3043
      if ( res /= 3043) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4541
      if ( res /= 4541) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7924
      if ( res /= 7924) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2036
      if ( res /= 2036) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_G0, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7676
      if ( res /= 7676) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghWpAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8064
      if ( res /= 8064) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_G0, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7801
      if ( res /= 7801) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5984
      if ( res /= 5984) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HMinus, P_HPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",74
      if ( res /= 74) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4491
      if ( res /= 4491) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3738
      if ( res /= 3738) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5331
      if ( res /= 5331) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",953
      if ( res /= 953) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_Z, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1056
      if ( res /= 1056) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3574
      if ( res /= 3574) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4720
      if ( res /= 4720) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HPlus, P_GMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9631
      if ( res /= 9631) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3529
      if ( res /= 3529) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4431
      if ( res /= 4431) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5484
      if ( res /= 5484) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10115
      if ( res /= 10115) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1099
      if ( res /= 1099) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6310
      if ( res /= 6310) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3776
      if ( res /= 3776) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8006
      if ( res /= 8006) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7303
      if ( res /= 7303) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",727
      if ( res /= 727) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7580
      if ( res /= 7580) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7307
      if ( res /= 7307) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2963
      if ( res /= 2963) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",731
      if ( res /= 731) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9478
      if ( res /= 9478) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9025
      if ( res /= 9025) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5867
      if ( res /= 5867) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3522
      if ( res /= 3522) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2656
      if ( res /= 2656) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4828
      if ( res /= 4828) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWp, P_ghWpAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6494
      if ( res /= 6494) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10334
      if ( res /= 10334) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_nu_tau, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",610
      if ( res /= 610) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3783
      if ( res /= 3783) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6420
      if ( res /= 6420) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5168
      if ( res /= 5168) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_A, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",751
      if ( res /= 751) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",223
      if ( res /= 223) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1810
      if ( res /= 1810) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10128
      if ( res /= 10128) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1280
      if ( res /= 1280) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5285
      if ( res /= 5285) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3171
      if ( res /= 3171) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7774
      if ( res /= 7774) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2684
      if ( res /= 2684) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9829
      if ( res /= 9829) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4633
      if ( res /= 4633) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2794
      if ( res /= 2794) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7645
      if ( res /= 7645) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4567
      if ( res /= 4567) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5839
      if ( res /= 5839) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6440
      if ( res /= 6440) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10239
      if ( res /= 10239) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghWm, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1686
      if ( res /= 1686) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4599
      if ( res /= 4599) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6041
      if ( res /= 6041) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3430
      if ( res /= 3430) then
        stop
      end if
      call createParticleConfig(P_g, P_G0, P_g, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10642
      if ( res /= 10642) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9165
      if ( res /= 9165) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3439
      if ( res /= 3439) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9413
      if ( res /= 9413) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",999
      if ( res /= 999) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2790
      if ( res /= 2790) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4445
      if ( res /= 4445) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3692
      if ( res /= 3692) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1354
      if ( res /= 1354) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4791
      if ( res /= 4791) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_A, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8971
      if ( res /= 8971) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_Hl, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1591
      if ( res /= 1591) then
        stop
      end if
      call createParticleConfig(P_Z, P_nu_tauAnti, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2992
      if ( res /= 2992) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1226
      if ( res /= 1226) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",699
      if ( res /= 699) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9788
      if ( res /= 9788) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5836
      if ( res /= 5836) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7279
      if ( res /= 7279) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1888
      if ( res /= 1888) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6970
      if ( res /= 6970) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9565
      if ( res /= 9565) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10137
      if ( res /= 10137) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6840
      if ( res /= 6840) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4396
      if ( res /= 4396) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7523
      if ( res /= 7523) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3890
      if ( res /= 3890) then
        stop
      end if
      call createParticleConfig(P_Ha, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4903
      if ( res /= 4903) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",518
      if ( res /= 518) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9368
      if ( res /= 9368) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6484
      if ( res /= 6484) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5100
      if ( res /= 5100) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_u, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6383
      if ( res /= 6383) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7249
      if ( res /= 7249) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1824
      if ( res /= 1824) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_b, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8610
      if ( res /= 8610) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_d, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2339
      if ( res /= 2339) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5528
      if ( res /= 5528) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3520
      if ( res /= 3520) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3444
      if ( res /= 3444) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GMinus, P_HPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6130
      if ( res /= 6130) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4830
      if ( res /= 4830) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6869
      if ( res /= 6869) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6928
      if ( res /= 6928) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4256
      if ( res /= 4256) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8993
      if ( res /= 8993) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2052
      if ( res /= 2052) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2573
      if ( res /= 2573) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",230
      if ( res /= 230) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8762
      if ( res /= 8762) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7632
      if ( res /= 7632) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6792
      if ( res /= 6792) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1401
      if ( res /= 1401) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1165
      if ( res /= 1165) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",787
      if ( res /= 787) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5296
      if ( res /= 5296) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HPlus, P_GMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6382
      if ( res /= 6382) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5269
      if ( res /= 5269) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7442
      if ( res /= 7442) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3609
      if ( res /= 3609) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_b, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2414
      if ( res /= 2414) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2488
      if ( res /= 2488) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2534
      if ( res /= 2534) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9310
      if ( res /= 9310) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5395
      if ( res /= 5395) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5539
      if ( res /= 5539) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghAAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",214
      if ( res /= 214) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8263
      if ( res /= 8263) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5077
      if ( res /= 5077) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2419
      if ( res /= 2419) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9981
      if ( res /= 9981) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5592
      if ( res /= 5592) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8863
      if ( res /= 8863) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5263
      if ( res /= 5263) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghAAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1676
      if ( res /= 1676) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8497
      if ( res /= 8497) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9369
      if ( res /= 9369) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2374
      if ( res /= 2374) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3575
      if ( res /= 3575) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1006
      if ( res /= 1006) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3208
      if ( res /= 3208) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6518
      if ( res /= 6518) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9236
      if ( res /= 9236) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9404
      if ( res /= 9404) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1010
      if ( res /= 1010) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_nu_tauAnti, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5940
      if ( res /= 5940) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1062
      if ( res /= 1062) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",872
      if ( res /= 872) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2057
      if ( res /= 2057) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2749
      if ( res /= 2749) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7160
      if ( res /= 7160) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5313
      if ( res /= 5313) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9774
      if ( res /= 9774) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6568
      if ( res /= 6568) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",12
      if ( res /= 12) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6448
      if ( res /= 6448) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10243
      if ( res /= 10243) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1500
      if ( res /= 1500) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9015
      if ( res /= 9015) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10404
      if ( res /= 10404) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5709
      if ( res /= 5709) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8205
      if ( res /= 8205) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5763
      if ( res /= 5763) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9487
      if ( res /= 9487) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3042
      if ( res /= 3042) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10672
      if ( res /= 10672) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4155
      if ( res /= 4155) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7290
      if ( res /= 7290) then
        stop
      end if
      call createParticleConfig(P_g, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7314
      if ( res /= 7314) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1750
      if ( res /= 1750) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9667
      if ( res /= 9667) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5229
      if ( res /= 5229) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10117
      if ( res /= 10117) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",70
      if ( res /= 70) then
        stop
      end if
      call createParticleConfig(P_AQ, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1372
      if ( res /= 1372) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8255
      if ( res /= 8255) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4364
      if ( res /= 4364) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6463
      if ( res /= 6463) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2805
      if ( res /= 2805) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10386
      if ( res /= 10386) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2446
      if ( res /= 2446) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1198
      if ( res /= 1198) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10125
      if ( res /= 10125) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7588
      if ( res /= 7588) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8867
      if ( res /= 8867) then
        stop
      end if
      call createParticleConfig(P_g, P_Hh, P_g, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10496
      if ( res /= 10496) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10024
      if ( res /= 10024) then
        stop
      end if
      call createParticleConfig(P_b, P_HPlus, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7613
      if ( res /= 7613) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2042
      if ( res /= 2042) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3412
      if ( res /= 3412) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4042
      if ( res /= 4042) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6548
      if ( res /= 6548) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2347
      if ( res /= 2347) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3688
      if ( res /= 3688) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2316
      if ( res /= 2316) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9858
      if ( res /= 9858) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7192
      if ( res /= 7192) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9624
      if ( res /= 9624) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9713
      if ( res /= 9713) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",294
      if ( res /= 294) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10396
      if ( res /= 10396) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9009
      if ( res /= 9009) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1014
      if ( res /= 1014) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6015
      if ( res /= 6015) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4111
      if ( res /= 4111) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5099
      if ( res /= 5099) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3672
      if ( res /= 3672) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7194
      if ( res /= 7194) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9613
      if ( res /= 9613) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6163
      if ( res /= 6163) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WMinus, P_ghWp, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",955
      if ( res /= 955) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1179
      if ( res /= 1179) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",145
      if ( res /= 145) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1558
      if ( res /= 1558) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7814
      if ( res /= 7814) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10491
      if ( res /= 10491) then
        stop
      end if
      call createParticleConfig(P_u, P_G0, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4827
      if ( res /= 4827) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6161
      if ( res /= 6161) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9765
      if ( res /= 9765) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4588
      if ( res /= 4588) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2439
      if ( res /= 2439) then
        stop
      end if
      call createParticleConfig(P_AQ, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4175
      if ( res /= 4175) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7431
      if ( res /= 7431) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",240
      if ( res /= 240) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4803
      if ( res /= 4803) then
        stop
      end if
      call createParticleConfig(P_G0, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4902
      if ( res /= 4902) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9322
      if ( res /= 9322) then
        stop
      end if
      call createParticleConfig(P_Z, P_g, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10534
      if ( res /= 10534) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4057
      if ( res /= 4057) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4173
      if ( res /= 4173) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2098
      if ( res /= 2098) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3993
      if ( res /= 3993) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8808
      if ( res /= 8808) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8763
      if ( res /= 8763) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3448
      if ( res /= 3448) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5255
      if ( res /= 5255) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",202
      if ( res /= 202) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",72
      if ( res /= 72) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_t, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5533
      if ( res /= 5533) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3361
      if ( res /= 3361) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8474
      if ( res /= 8474) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10433
      if ( res /= 10433) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10718
      if ( res /= 10718) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9625
      if ( res /= 9625) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6567
      if ( res /= 6567) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3916
      if ( res /= 3916) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8714
      if ( res /= 8714) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2867
      if ( res /= 2867) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",85
      if ( res /= 85) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",490
      if ( res /= 490) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5467
      if ( res /= 5467) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7877
      if ( res /= 7877) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3758
      if ( res /= 3758) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6956
      if ( res /= 6956) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Ha, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5007
      if ( res /= 5007) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2285
      if ( res /= 2285) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2331
      if ( res /= 2331) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5760
      if ( res /= 5760) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",644
      if ( res /= 644) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4549
      if ( res /= 4549) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1804
      if ( res /= 1804) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7399
      if ( res /= 7399) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9650
      if ( res /= 9650) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4716
      if ( res /= 4716) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HMinus, P_ghWmAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8201
      if ( res /= 8201) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10760
      if ( res /= 10760) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10539
      if ( res /= 10539) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8973
      if ( res /= 8973) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1436
      if ( res /= 1436) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7950
      if ( res /= 7950) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_g, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10774
      if ( res /= 10774) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_A, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9646
      if ( res /= 9646) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4686
      if ( res /= 4686) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10781
      if ( res /= 10781) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_WMinus, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2893
      if ( res /= 2893) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5402
      if ( res /= 5402) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5799
      if ( res /= 5799) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",535
      if ( res /= 535) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9886
      if ( res /= 9886) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_GMinus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5485
      if ( res /= 5485) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3182
      if ( res /= 3182) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1820
      if ( res /= 1820) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10187
      if ( res /= 10187) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4695
      if ( res /= 4695) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2423
      if ( res /= 2423) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9570
      if ( res /= 9570) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",703
      if ( res /= 703) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3459
      if ( res /= 3459) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9574
      if ( res /= 9574) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1842
      if ( res /= 1842) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghZAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",828
      if ( res /= 828) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",143
      if ( res /= 143) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9997
      if ( res /= 9997) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8652
      if ( res /= 8652) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5937
      if ( res /= 5937) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6281
      if ( res /= 6281) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5885
      if ( res /= 5885) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8271
      if ( res /= 8271) then
        stop
      end if
      call createParticleConfig(P_c, P_Z, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3063
      if ( res /= 3063) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8787
      if ( res /= 8787) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1770
      if ( res /= 1770) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2994
      if ( res /= 2994) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2645
      if ( res /= 2645) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6008
      if ( res /= 6008) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7933
      if ( res /= 7933) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1833
      if ( res /= 1833) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3326
      if ( res /= 3326) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1442
      if ( res /= 1442) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10779
      if ( res /= 10779) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1955
      if ( res /= 1955) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9612
      if ( res /= 9612) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",361
      if ( res /= 361) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4964
      if ( res /= 4964) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7990
      if ( res /= 7990) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10116
      if ( res /= 10116) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3181
      if ( res /= 3181) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZ, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1571
      if ( res /= 1571) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3727
      if ( res /= 3727) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8646
      if ( res /= 8646) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",484
      if ( res /= 484) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2959
      if ( res /= 2959) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3926
      if ( res /= 3926) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6561
      if ( res /= 6561) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2253
      if ( res /= 2253) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10183
      if ( res /= 10183) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1901
      if ( res /= 1901) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3897
      if ( res /= 3897) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4916
      if ( res /= 4916) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2816
      if ( res /= 2816) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7649
      if ( res /= 7649) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1341
      if ( res /= 1341) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",964
      if ( res /= 964) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9426
      if ( res /= 9426) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10766
      if ( res /= 10766) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",103
      if ( res /= 103) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3780
      if ( res /= 3780) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghWp, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1567
      if ( res /= 1567) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3289
      if ( res /= 3289) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7116
      if ( res /= 7116) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1376
      if ( res /= 1376) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4671
      if ( res /= 4671) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10055
      if ( res /= 10055) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1830
      if ( res /= 1830) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7610
      if ( res /= 7610) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8889
      if ( res /= 8889) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2152
      if ( res /= 2152) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9410
      if ( res /= 9410) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8794
      if ( res /= 8794) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4524
      if ( res /= 4524) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5878
      if ( res /= 5878) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8853
      if ( res /= 8853) then
        stop
      end if
      call createParticleConfig(P_d, P_GPlus, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7750
      if ( res /= 7750) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_GPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2076
      if ( res /= 2076) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7607
      if ( res /= 7607) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7124
      if ( res /= 7124) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10136
      if ( res /= 10136) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3504
      if ( res /= 3504) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8267
      if ( res /= 8267) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5505
      if ( res /= 5505) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5299
      if ( res /= 5299) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7310
      if ( res /= 7310) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_A, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4554
      if ( res /= 4554) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10254
      if ( res /= 10254) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1290
      if ( res /= 1290) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6736
      if ( res /= 6736) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6215
      if ( res /= 6215) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3743
      if ( res /= 3743) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",769
      if ( res /= 769) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1039
      if ( res /= 1039) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8784
      if ( res /= 8784) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10421
      if ( res /= 10421) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_nu_muAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",832
      if ( res /= 832) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6778
      if ( res /= 6778) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5499
      if ( res /= 5499) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2680
      if ( res /= 2680) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6445
      if ( res /= 6445) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7894
      if ( res /= 7894) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6050
      if ( res /= 6050) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2638
      if ( res /= 2638) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7741
      if ( res /= 7741) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",96
      if ( res /= 96) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",126
      if ( res /= 126) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8510
      if ( res /= 8510) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10561
      if ( res /= 10561) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghZ, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",660
      if ( res /= 660) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9233
      if ( res /= 9233) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9174
      if ( res /= 9174) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10483
      if ( res /= 10483) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5671
      if ( res /= 5671) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8063
      if ( res /= 8063) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghZAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4821
      if ( res /= 4821) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1259
      if ( res /= 1259) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2605
      if ( res /= 2605) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4314
      if ( res /= 4314) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4185
      if ( res /= 4185) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9904
      if ( res /= 9904) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_Z, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4881
      if ( res /= 4881) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4607
      if ( res /= 4607) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8020
      if ( res /= 8020) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8145
      if ( res /= 8145) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9373
      if ( res /= 9373) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",127
      if ( res /= 127) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1459
      if ( res /= 1459) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4898
      if ( res /= 4898) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_HPlus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",737
      if ( res /= 737) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7992
      if ( res /= 7992) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4062
      if ( res /= 4062) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7042
      if ( res /= 7042) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6593
      if ( res /= 6593) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1690
      if ( res /= 1690) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2975
      if ( res /= 2975) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_nu_mu, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",362
      if ( res /= 362) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7804
      if ( res /= 7804) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3225
      if ( res /= 3225) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6072
      if ( res /= 6072) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1469
      if ( res /= 1469) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2148
      if ( res /= 2148) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5621
      if ( res /= 5621) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7216
      if ( res /= 7216) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7339
      if ( res /= 7339) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10650
      if ( res /= 10650) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",43
      if ( res /= 43) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7158
      if ( res /= 7158) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3376
      if ( res /= 3376) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4654
      if ( res /= 4654) then
        stop
      end if
      call createParticleConfig(P_u, P_dAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6057
      if ( res /= 6057) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_GPlus, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5685
      if ( res /= 5685) then
        stop
      end if
      call createParticleConfig(P_Hl, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10643
      if ( res /= 10643) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8229
      if ( res /= 8229) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5933
      if ( res /= 5933) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3850
      if ( res /= 3850) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4601
      if ( res /= 4601) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3620
      if ( res /= 3620) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWmAnti, P_WMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",190
      if ( res /= 190) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7747
      if ( res /= 7747) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10596
      if ( res /= 10596) then
        stop
      end if
      call createParticleConfig(P_Hh, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5659
      if ( res /= 5659) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10606
      if ( res /= 10606) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3892
      if ( res /= 3892) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4435
      if ( res /= 4435) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3883
      if ( res /= 3883) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4452
      if ( res /= 4452) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3314
      if ( res /= 3314) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",62
      if ( res /= 62) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7143
      if ( res /= 7143) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3360
      if ( res /= 3360) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9067
      if ( res /= 9067) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6180
      if ( res /= 6180) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1193
      if ( res /= 1193) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9337
      if ( res /= 9337) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2830
      if ( res /= 2830) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7472
      if ( res /= 7472) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3194
      if ( res /= 3194) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_c, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4995
      if ( res /= 4995) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10559
      if ( res /= 10559) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4988
      if ( res /= 4988) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",887
      if ( res /= 887) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8640
      if ( res /= 8640) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghA, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4410
      if ( res /= 4410) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6723
      if ( res /= 6723) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",219
      if ( res /= 219) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9718
      if ( res /= 9718) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3544
      if ( res /= 3544) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3227
      if ( res /= 3227) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghA, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6711
      if ( res /= 6711) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5610
      if ( res /= 5610) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9776
      if ( res /= 9776) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3251
      if ( res /= 3251) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_A, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3451
      if ( res /= 3451) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6732
      if ( res /= 6732) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10659
      if ( res /= 10659) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWp, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",201
      if ( res /= 201) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",318
      if ( res /= 318) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2165
      if ( res /= 2165) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8188
      if ( res /= 8188) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4376
      if ( res /= 4376) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",386
      if ( res /= 386) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7183
      if ( res /= 7183) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1077
      if ( res /= 1077) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8413
      if ( res /= 8413) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9030
      if ( res /= 9030) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6808
      if ( res /= 6808) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_s, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4354
      if ( res /= 4354) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghZAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8638
      if ( res /= 8638) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8180
      if ( res /= 8180) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6825
      if ( res /= 6825) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2291
      if ( res /= 2291) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5291
      if ( res /= 5291) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8308
      if ( res /= 8308) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5256
      if ( res /= 5256) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",526
      if ( res /= 526) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8459
      if ( res /= 8459) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_nu_muAnti, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5956
      if ( res /= 5956) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3250
      if ( res /= 3250) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7118
      if ( res /= 7118) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_c, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3296
      if ( res /= 3296) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10157
      if ( res /= 10157) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1922
      if ( res /= 1922) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1001
      if ( res /= 1001) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4768
      if ( res /= 4768) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8110
      if ( res /= 8110) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6350
      if ( res /= 6350) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6090
      if ( res /= 6090) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",199
      if ( res /= 199) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4454
      if ( res /= 4454) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4959
      if ( res /= 4959) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9475
      if ( res /= 9475) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3935
      if ( res /= 3935) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9430
      if ( res /= 9430) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",63
      if ( res /= 63) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4151
      if ( res /= 4151) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4088
      if ( res /= 4088) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HMinus, P_GPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10401
      if ( res /= 10401) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4989
      if ( res /= 4989) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9099
      if ( res /= 9099) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1625
      if ( res /= 1625) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10489
      if ( res /= 10489) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9479
      if ( res /= 9479) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6702
      if ( res /= 6702) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6837
      if ( res /= 6837) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2719
      if ( res /= 2719) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5340
      if ( res /= 5340) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7675
      if ( res /= 7675) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",854
      if ( res /= 854) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GMinus, P_HPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6406
      if ( res /= 6406) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",335
      if ( res /= 335) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2865
      if ( res /= 2865) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8410
      if ( res /= 8410) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10588
      if ( res /= 10588) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6327
      if ( res /= 6327) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",498
      if ( res /= 498) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",499
      if ( res /= 499) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1854
      if ( res /= 1854) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8537
      if ( res /= 8537) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1229
      if ( res /= 1229) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9252
      if ( res /= 9252) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9772
      if ( res /= 9772) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9259
      if ( res /= 9259) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2825
      if ( res /= 2825) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7502
      if ( res /= 7502) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3894
      if ( res /= 3894) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3873
      if ( res /= 3873) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7296
      if ( res /= 7296) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4664
      if ( res /= 4664) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1166
      if ( res /= 1166) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10199
      if ( res /= 10199) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8803
      if ( res /= 8803) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",920
      if ( res /= 920) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4711
      if ( res /= 4711) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3767
      if ( res /= 3767) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3556
      if ( res /= 3556) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8765
      if ( res /= 8765) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10227
      if ( res /= 10227) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5038
      if ( res /= 5038) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1866
      if ( res /= 1866) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8286
      if ( res /= 8286) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1301
      if ( res /= 1301) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",198
      if ( res /= 198) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9075
      if ( res /= 9075) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10267
      if ( res /= 10267) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3781
      if ( res /= 3781) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3517
      if ( res /= 3517) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5791
      if ( res /= 5791) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1638
      if ( res /= 1638) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5196
      if ( res /= 5196) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4628
      if ( res /= 4628) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10450
      if ( res /= 10450) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9640
      if ( res /= 9640) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4822
      if ( res /= 4822) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1848
      if ( res /= 1848) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6919
      if ( res /= 6919) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1289
      if ( res /= 1289) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4077
      if ( res /= 4077) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2242
      if ( res /= 2242) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3253
      if ( res /= 3253) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7204
      if ( res /= 7204) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3788
      if ( res /= 3788) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4562
      if ( res /= 4562) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_nu_tauAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",855
      if ( res /= 855) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2435
      if ( res /= 2435) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4103
      if ( res /= 4103) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8283
      if ( res /= 8283) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",691
      if ( res /= 691) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4906
      if ( res /= 4906) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8095
      if ( res /= 8095) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghA, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7651
      if ( res /= 7651) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6476
      if ( res /= 6476) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3497
      if ( res /= 3497) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6945
      if ( res /= 6945) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4453
      if ( res /= 4453) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1649
      if ( res /= 1649) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1584
      if ( res /= 1584) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",943
      if ( res /= 943) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HMinus, P_HPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",811
      if ( res /= 811) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9188
      if ( res /= 9188) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5580
      if ( res /= 5580) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10362
      if ( res /= 10362) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4500
      if ( res /= 4500) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9033
      if ( res /= 9033) then
        stop
      end if
      call createParticleConfig(P_b, P_tAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3137
      if ( res /= 3137) then
        stop
      end if
      call createParticleConfig(P_c, P_sAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6503
      if ( res /= 6503) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",804
      if ( res /= 804) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5416
      if ( res /= 5416) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6710
      if ( res /= 6710) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",446
      if ( res /= 446) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7385
      if ( res /= 7385) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_eMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",959
      if ( res /= 959) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7664
      if ( res /= 7664) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8349
      if ( res /= 8349) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2013
      if ( res /= 2013) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10623
      if ( res /= 10623) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9465
      if ( res /= 9465) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_tauPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7653
      if ( res /= 7653) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2402
      if ( res /= 2402) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2388
      if ( res /= 2388) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",289
      if ( res /= 289) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8922
      if ( res /= 8922) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZ, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5767
      if ( res /= 5767) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5410
      if ( res /= 5410) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghWm, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",957
      if ( res /= 957) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",580
      if ( res /= 580) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2562
      if ( res /= 2562) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2038
      if ( res /= 2038) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8215
      if ( res /= 8215) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GPlus, P_WMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10382
      if ( res /= 10382) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7861
      if ( res /= 7861) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4684
      if ( res /= 4684) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghWm, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4996
      if ( res /= 4996) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2587
      if ( res /= 2587) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9034
      if ( res /= 9034) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4709
      if ( res /= 4709) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10112
      if ( res /= 10112) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GPlus, P_HMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3640
      if ( res /= 3640) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9589
      if ( res /= 9589) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9294
      if ( res /= 9294) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3188
      if ( res /= 3188) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5693
      if ( res /= 5693) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7574
      if ( res /= 7574) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7776
      if ( res /= 7776) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8228
      if ( res /= 8228) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",842
      if ( res /= 842) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",278
      if ( res /= 278) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3493
      if ( res /= 3493) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2256
      if ( res /= 2256) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",169
      if ( res /= 169) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4181
      if ( res /= 4181) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1412
      if ( res /= 1412) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_uAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1101
      if ( res /= 1101) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8561
      if ( res /= 8561) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7652
      if ( res /= 7652) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghWmAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2936
      if ( res /= 2936) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5949
      if ( res /= 5949) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4017
      if ( res /= 4017) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4492
      if ( res /= 4492) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9754
      if ( res /= 9754) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3302
      if ( res /= 3302) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GPlus, P_ghWm, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5120
      if ( res /= 5120) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8558
      if ( res /= 8558) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1159
      if ( res /= 1159) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7311
      if ( res /= 7311) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10237
      if ( res /= 10237) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_A, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5365
      if ( res /= 5365) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9758
      if ( res /= 9758) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3391
      if ( res /= 3391) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6666
      if ( res /= 6666) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3266
      if ( res /= 3266) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8062
      if ( res /= 8062) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghZ, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2547
      if ( res /= 2547) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10044
      if ( res /= 10044) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghWm, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6169
      if ( res /= 6169) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10651
      if ( res /= 10651) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9371
      if ( res /= 9371) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7808
      if ( res /= 7808) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8854
      if ( res /= 8854) then
        stop
      end if
      call createParticleConfig(P_A, P_nu_eAnti, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10424
      if ( res /= 10424) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",623
      if ( res /= 623) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9804
      if ( res /= 9804) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZ, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6844
      if ( res /= 6844) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6278
      if ( res /= 6278) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2143
      if ( res /= 2143) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2041
      if ( res /= 2041) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6465
      if ( res /= 6465) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7958
      if ( res /= 7958) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_Z, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6621
      if ( res /= 6621) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5643
      if ( res /= 5643) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4052
      if ( res /= 4052) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10313
      if ( res /= 10313) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6658
      if ( res /= 6658) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9622
      if ( res /= 9622) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7404
      if ( res /= 7404) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7256
      if ( res /= 7256) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6681
      if ( res /= 6681) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5270
      if ( res /= 5270) then
        stop
      end if
      call createParticleConfig(P_Ha, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6791
      if ( res /= 6791) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9554
      if ( res /= 9554) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3844
      if ( res /= 3844) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8619
      if ( res /= 8619) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2628
      if ( res /= 2628) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8385
      if ( res /= 8385) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8307
      if ( res /= 8307) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3633
      if ( res /= 3633) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GMinus, P_WPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1914
      if ( res /= 1914) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",154
      if ( res /= 154) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10204
      if ( res /= 10204) then
        stop
      end if
      call createParticleConfig(P_G0, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7665
      if ( res /= 7665) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10033
      if ( res /= 10033) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4470
      if ( res /= 4470) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GMinus, P_GPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1586
      if ( res /= 1586) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",309
      if ( res /= 309) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7621
      if ( res /= 7621) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",123
      if ( res /= 123) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8525
      if ( res /= 8525) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9120
      if ( res /= 9120) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2450
      if ( res /= 2450) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6850
      if ( res /= 6850) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9309
      if ( res /= 9309) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10229
      if ( res /= 10229) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3346
      if ( res /= 3346) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2087
      if ( res /= 2087) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",226
      if ( res /= 226) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_Ha, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4843
      if ( res /= 4843) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3224
      if ( res /= 3224) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6433
      if ( res /= 6433) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9502
      if ( res /= 9502) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9661
      if ( res /= 9661) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7272
      if ( res /= 7272) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5917
      if ( res /= 5917) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10110
      if ( res /= 10110) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2846
      if ( res /= 2846) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6640
      if ( res /= 6640) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9429
      if ( res /= 9429) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8637
      if ( res /= 8637) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghAAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4415
      if ( res /= 4415) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5003
      if ( res /= 5003) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10351
      if ( res /= 10351) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghZ, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1097
      if ( res /= 1097) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9123
      if ( res /= 9123) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Z, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1085
      if ( res /= 1085) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",480
      if ( res /= 480) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_WPlus, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3405
      if ( res /= 3405) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",808
      if ( res /= 808) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4813
      if ( res /= 4813) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8586
      if ( res /= 8586) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9125
      if ( res /= 9125) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_WPlus, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4818
      if ( res /= 4818) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10416
      if ( res /= 10416) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2804
      if ( res /= 2804) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4721
      if ( res /= 4721) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1963
      if ( res /= 1963) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7276
      if ( res /= 7276) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9692
      if ( res /= 9692) then
        stop
      end if
      call createParticleConfig(P_g, P_Hl, P_g, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10560
      if ( res /= 10560) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3828
      if ( res /= 3828) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1663
      if ( res /= 1663) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6274
      if ( res /= 6274) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8517
      if ( res /= 8517) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10414
      if ( res /= 10414) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9595
      if ( res /= 9595) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2252
      if ( res /= 2252) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_sAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6539
      if ( res /= 6539) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3621
      if ( res /= 3621) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6417
      if ( res /= 6417) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6479
      if ( res /= 6479) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6531
      if ( res /= 6531) then
        stop
      end if
      call createParticleConfig(P_AQ, P_G0, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6733
      if ( res /= 6733) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10411
      if ( res /= 10411) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10572
      if ( res /= 10572) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_g, P_HPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10617
      if ( res /= 10617) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4690
      if ( res /= 4690) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8981
      if ( res /= 8981) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",514
      if ( res /= 514) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5252
      if ( res /= 5252) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4783
      if ( res /= 4783) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_nu_e, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3768
      if ( res /= 3768) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9312
      if ( res /= 9312) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3169
      if ( res /= 3169) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",23
      if ( res /= 23) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3634
      if ( res /= 3634) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6601
      if ( res /= 6601) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7765
      if ( res /= 7765) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9996
      if ( res /= 9996) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7135
      if ( res /= 7135) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10633
      if ( res /= 10633) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3787
      if ( res /= 3787) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8575
      if ( res /= 8575) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1687
      if ( res /= 1687) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HPlus, P_HMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3192
      if ( res /= 3192) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6762
      if ( res /= 6762) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6369
      if ( res /= 6369) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3486
      if ( res /= 3486) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2448
      if ( res /= 2448) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8289
      if ( res /= 8289) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghWp, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3039
      if ( res /= 3039) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10445
      if ( res /= 10445) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7233
      if ( res /= 7233) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2773
      if ( res /= 2773) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8392
      if ( res /= 8392) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1157
      if ( res /= 1157) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6308
      if ( res /= 6308) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6359
      if ( res /= 6359) then
        stop
      end if
      call createParticleConfig(P_Z, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",159
      if ( res /= 159) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7879
      if ( res /= 7879) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5816
      if ( res /= 5816) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4615
      if ( res /= 4615) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7911
      if ( res /= 7911) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4398
      if ( res /= 4398) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9267
      if ( res /= 9267) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9815
      if ( res /= 9815) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7287
      if ( res /= 7287) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWpAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4476
      if ( res /= 4476) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4631
      if ( res /= 4631) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7260
      if ( res /= 7260) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3722
      if ( res /= 3722) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghZAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5604
      if ( res /= 5604) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4753
      if ( res /= 4753) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghA, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2354
      if ( res /= 2354) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3190
      if ( res /= 3190) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5290
      if ( res /= 5290) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3239
      if ( res /= 3239) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7397
      if ( res /= 7397) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",594
      if ( res /= 594) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5925
      if ( res /= 5925) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3946
      if ( res /= 3946) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_s, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4655
      if ( res /= 4655) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2675
      if ( res /= 2675) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3037
      if ( res /= 3037) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1368
      if ( res /= 1368) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9266
      if ( res /= 9266) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GPlus, P_ghWpAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2457
      if ( res /= 2457) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WMinus, P_WPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3872
      if ( res /= 3872) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5862
      if ( res /= 5862) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7400
      if ( res /= 7400) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6340
      if ( res /= 6340) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2549
      if ( res /= 2549) then
        stop
      end if
      call createParticleConfig(P_A, P_nu_tau, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9657
      if ( res /= 9657) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5519
      if ( res /= 5519) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10322
      if ( res /= 10322) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7434
      if ( res /= 7434) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6987
      if ( res /= 6987) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghZ, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8081
      if ( res /= 8081) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_nu_tau, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2317
      if ( res /= 2317) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9527
      if ( res /= 9527) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1393
      if ( res /= 1393) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghAAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",588
      if ( res /= 588) then
        stop
      end if
      call createParticleConfig(P_A, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2664
      if ( res /= 2664) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1834
      if ( res /= 1834) then
        stop
      end if
      call createParticleConfig(P_b, P_G0, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3166
      if ( res /= 3166) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4343
      if ( res /= 4343) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10179
      if ( res /= 10179) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10257
      if ( res /= 10257) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1495
      if ( res /= 1495) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8727
      if ( res /= 8727) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10647
      if ( res /= 10647) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",61
      if ( res /= 61) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8700
      if ( res /= 8700) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10308
      if ( res /= 10308) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9739
      if ( res /= 9739) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1581
      if ( res /= 1581) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3247
      if ( res /= 3247) then
        stop
      end if
      call createParticleConfig(P_Ha, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5226
      if ( res /= 5226) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWm, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5554
      if ( res /= 5554) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9114
      if ( res /= 9114) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3170
      if ( res /= 3170) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3696
      if ( res /= 3696) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",146
      if ( res /= 146) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2205
      if ( res /= 2205) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6976
      if ( res /= 6976) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2054
      if ( res /= 2054) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6527
      if ( res /= 6527) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1246
      if ( res /= 1246) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6118
      if ( res /= 6118) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1003
      if ( res /= 1003) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7416
      if ( res /= 7416) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2412
      if ( res /= 2412) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4642
      if ( res /= 4642) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9251
      if ( res /= 9251) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_ePlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5687
      if ( res /= 5687) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9473
      if ( res /= 9473) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3845
      if ( res /= 3845) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghA, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1715
      if ( res /= 1715) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9731
      if ( res /= 9731) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8819
      if ( res /= 8819) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6698
      if ( res /= 6698) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4704
      if ( res /= 4704) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2014
      if ( res /= 2014) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3214
      if ( res /= 3214) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",649
      if ( res /= 649) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WPlus, P_WMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",690
      if ( res /= 690) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1194
      if ( res /= 1194) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10003
      if ( res /= 10003) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4169
      if ( res /= 4169) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9344
      if ( res /= 9344) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5844
      if ( res /= 5844) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8665
      if ( res /= 8665) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1947
      if ( res /= 1947) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10307
      if ( res /= 10307) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1884
      if ( res /= 1884) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1455
      if ( res /= 1455) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6296
      if ( res /= 6296) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",584
      if ( res /= 584) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",453
      if ( res /= 453) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HPlus, P_HMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7083
      if ( res /= 7083) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5287
      if ( res /= 5287) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8423
      if ( res /= 8423) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7199
      if ( res /= 7199) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9076
      if ( res /= 9076) then
        stop
      end if
      call createParticleConfig(P_g, P_Hl, P_Hh, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10599
      if ( res /= 10599) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWp, P_GMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4457
      if ( res /= 4457) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4011
      if ( res /= 4011) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8729
      if ( res /= 8729) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8320
      if ( res /= 8320) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2908
      if ( res /= 2908) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10258
      if ( res /= 10258) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10160
      if ( res /= 10160) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1502
      if ( res /= 1502) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5116
      if ( res /= 5116) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2762
      if ( res /= 2762) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_tauMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6933
      if ( res /= 6933) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WMinus, P_ghWp, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",798
      if ( res /= 798) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2409
      if ( res /= 2409) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4483
      if ( res /= 4483) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4139
      if ( res /= 4139) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_Hh, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7539
      if ( res /= 7539) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3155
      if ( res /= 3155) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9775
      if ( res /= 9775) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2993
      if ( res /= 2993) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_c, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5500
      if ( res /= 5500) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3003
      if ( res /= 3003) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2898
      if ( res /= 2898) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7506
      if ( res /= 7506) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3962
      if ( res /= 3962) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_g, P_g, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5411
      if ( res /= 5411) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10614
      if ( res /= 10614) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3176
      if ( res /= 3176) then
        stop
      end if
      call createParticleConfig(P_A, P_g, P_g, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10691
      if ( res /= 10691) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",522
      if ( res /= 522) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4389
      if ( res /= 4389) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GMinus, P_GPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6978
      if ( res /= 6978) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghZAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2266
      if ( res /= 2266) then
        stop
      end if
      call createParticleConfig(P_Hl, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3884
      if ( res /= 3884) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6822
      if ( res /= 6822) then
        stop
      end if
      call createParticleConfig(P_Hh, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2251
      if ( res /= 2251) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7584
      if ( res /= 7584) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7648
      if ( res /= 7648) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6016
      if ( res /= 6016) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4785
      if ( res /= 4785) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4330
      if ( res /= 4330) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HPlus, P_HMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3210
      if ( res /= 3210) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2924
      if ( res /= 2924) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5803
      if ( res /= 5803) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10314
      if ( res /= 10314) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5586
      if ( res /= 5586) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",150
      if ( res /= 150) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",173
      if ( res /= 173) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",34
      if ( res /= 34) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4837
      if ( res /= 4837) then
        stop
      end if
      call createParticleConfig(P_Ha, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6365
      if ( res /= 6365) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8639
      if ( res /= 8639) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5221
      if ( res /= 5221) then
        stop
      end if
      call createParticleConfig(P_G0, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8082
      if ( res /= 8082) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1343
      if ( res /= 1343) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7893
      if ( res /= 7893) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",153
      if ( res /= 153) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7066
      if ( res /= 7066) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9381
      if ( res /= 9381) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3495
      if ( res /= 3495) then
        stop
      end if
      call createParticleConfig(P_AQ, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3
      if ( res /= 3) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6006
      if ( res /= 6006) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",124
      if ( res /= 124) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8869
      if ( res /= 8869) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4008
      if ( res /= 4008) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5593
      if ( res /= 5593) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7071
      if ( res /= 7071) then
        stop
      end if
      call createParticleConfig(P_Z, P_g, P_A, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10666
      if ( res /= 10666) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9334
      if ( res /= 9334) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_nu_muAnti, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1894
      if ( res /= 1894) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghZ, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7577
      if ( res /= 7577) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4154
      if ( res /= 4154) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8927
      if ( res /= 8927) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghAAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1315
      if ( res /= 1315) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5156
      if ( res /= 5156) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9708
      if ( res /= 9708) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9571
      if ( res /= 9571) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3109
      if ( res /= 3109) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4342
      if ( res /= 4342) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3230
      if ( res /= 3230) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3005
      if ( res /= 3005) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9732
      if ( res /= 9732) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2239
      if ( res /= 2239) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5859
      if ( res /= 5859) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7153
      if ( res /= 7153) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4819
      if ( res /= 4819) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9167
      if ( res /= 9167) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",65
      if ( res /= 65) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7796
      if ( res /= 7796) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1741
      if ( res /= 1741) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9912
      if ( res /= 9912) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4053
      if ( res /= 4053) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4045
      if ( res /= 4045) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9250
      if ( res /= 9250) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3179
      if ( res /= 3179) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_nu_eAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8382
      if ( res /= 8382) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2858
      if ( res /= 2858) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1918
      if ( res /= 1918) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7430
      if ( res /= 7430) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2050
      if ( res /= 2050) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2382
      if ( res /= 2382) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7926
      if ( res /= 7926) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10269
      if ( res /= 10269) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10703
      if ( res /= 10703) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2882
      if ( res /= 2882) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",796
      if ( res /= 796) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8287
      if ( res /= 8287) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1059
      if ( res /= 1059) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4512
      if ( res /= 4512) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9828
      if ( res /= 9828) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7243
      if ( res /= 7243) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1910
      if ( res /= 1910) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3132
      if ( res /= 3132) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6247
      if ( res /= 6247) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_A, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4732
      if ( res /= 4732) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4612
      if ( res /= 4612) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5807
      if ( res /= 5807) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6060
      if ( res /= 6060) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",652
      if ( res /= 652) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7920
      if ( res /= 7920) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6635
      if ( res /= 6635) then
        stop
      end if
      call createParticleConfig(P_u, P_Z, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8236
      if ( res /= 8236) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9730
      if ( res /= 9730) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5268
      if ( res /= 5268) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8820
      if ( res /= 8820) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8952
      if ( res /= 8952) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_A, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1178
      if ( res /= 1178) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3655
      if ( res /= 3655) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4953
      if ( res /= 4953) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5908
      if ( res /= 5908) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghWpAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3807
      if ( res /= 3807) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3110
      if ( res /= 3110) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2855
      if ( res /= 2855) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6178
      if ( res /= 6178) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",946
      if ( res /= 946) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6115
      if ( res /= 6115) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7026
      if ( res /= 7026) then
        stop
      end if
      call createParticleConfig(P_Z, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",835
      if ( res /= 835) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9662
      if ( res /= 9662) then
        stop
      end if
      call createParticleConfig(P_g, P_Ha, P_g, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10558
      if ( res /= 10558) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10407
      if ( res /= 10407) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4515
      if ( res /= 4515) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2068
      if ( res /= 2068) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8311
      if ( res /= 8311) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10423
      if ( res /= 10423) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6108
      if ( res /= 6108) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1268
      if ( res /= 1268) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5982
      if ( res /= 5982) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10690
      if ( res /= 10690) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1195
      if ( res /= 1195) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3917
      if ( res /= 3917) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3462
      if ( res /= 3462) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8054
      if ( res /= 8054) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghA, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3153
      if ( res /= 3153) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1463
      if ( res /= 1463) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6213
      if ( res /= 6213) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8023
      if ( res /= 8023) then
        stop
      end if
      call createParticleConfig(P_Z, P_g, P_Z, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10678
      if ( res /= 10678) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4775
      if ( res /= 4775) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_nu_eAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2275
      if ( res /= 2275) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7330
      if ( res /= 7330) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_nu_tauAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4186
      if ( res /= 4186) then
        stop
      end if
      call createParticleConfig(P_s, P_g, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8706
      if ( res /= 8706) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1685
      if ( res /= 1685) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9223
      if ( res /= 9223) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4730
      if ( res /= 4730) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6875
      if ( res /= 6875) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",416
      if ( res /= 416) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8471
      if ( res /= 8471) then
        stop
      end if
      call createParticleConfig(P_A, P_g, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10646
      if ( res /= 10646) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5092
      if ( res /= 5092) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5288
      if ( res /= 5288) then
        stop
      end if
      call createParticleConfig(P_s, P_cAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2843
      if ( res /= 2843) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9314
      if ( res /= 9314) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",851
      if ( res /= 851) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7058
      if ( res /= 7058) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ePlus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2320
      if ( res /= 2320) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6127
      if ( res /= 6127) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10567
      if ( res /= 10567) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1721
      if ( res /= 1721) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5233
      if ( res /= 5233) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",367
      if ( res /= 367) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4375
      if ( res /= 4375) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1994
      if ( res /= 1994) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7
      if ( res /= 7) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10114
      if ( res /= 10114) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4028
      if ( res /= 4028) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9386
      if ( res /= 9386) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1731
      if ( res /= 1731) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4657
      if ( res /= 4657) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6805
      if ( res /= 6805) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",258
      if ( res /= 258) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3991
      if ( res /= 3991) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1278
      if ( res /= 1278) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4950
      if ( res /= 4950) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9759
      if ( res /= 9759) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5337
      if ( res /= 5337) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10360
      if ( res /= 10360) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6761
      if ( res /= 6761) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10228
      if ( res /= 10228) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_eMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1218
      if ( res /= 1218) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4682
      if ( res /= 4682) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6073
      if ( res /= 6073) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_tauPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",470
      if ( res /= 470) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6576
      if ( res /= 6576) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2801
      if ( res /= 2801) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8060
      if ( res /= 8060) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10368
      if ( res /= 10368) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5579
      if ( res /= 5579) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9126
      if ( res /= 9126) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2278
      if ( res /= 2278) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2372
      if ( res /= 2372) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10641
      if ( res /= 10641) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5819
      if ( res /= 5819) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWpAnti, P_GPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2051
      if ( res /= 2051) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2712
      if ( res /= 2712) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GPlus, P_HMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3415
      if ( res /= 3415) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3115
      if ( res /= 3115) then
        stop
      end if
      call createParticleConfig(P_t, P_g, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8028
      if ( res /= 8028) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7097
      if ( res /= 7097) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2459
      if ( res /= 2459) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2281
      if ( res /= 2281) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3827
      if ( res /= 3827) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",552
      if ( res /= 552) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9274
      if ( res /= 9274) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1974
      if ( res /= 1974) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7922
      if ( res /= 7922) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9300
      if ( res /= 9300) then
        stop
      end if
      call createParticleConfig(P_G0, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",310
      if ( res /= 310) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1684
      if ( res /= 1684) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4379
      if ( res /= 4379) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7996
      if ( res /= 7996) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3604
      if ( res /= 3604) then
        stop
      end if
      call createParticleConfig(P_Z, P_nu_muAnti, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6248
      if ( res /= 6248) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6034
      if ( res /= 6034) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5666
      if ( res /= 5666) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6119
      if ( res /= 6119) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1204
      if ( res /= 1204) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",919
      if ( res /= 919) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8047
      if ( res /= 8047) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4574
      if ( res /= 4574) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7393
      if ( res /= 7393) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8966
      if ( res /= 8966) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1331
      if ( res /= 1331) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghAAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5355
      if ( res /= 5355) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6032
      if ( res /= 6032) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1507
      if ( res /= 1507) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10732
      if ( res /= 10732) then
        stop
      end if
      call createParticleConfig(P_Hl, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",998
      if ( res /= 998) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10635
      if ( res /= 10635) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5570
      if ( res /= 5570) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3924
      if ( res /= 3924) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5102
      if ( res /= 5102) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9777
      if ( res /= 9777) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8615
      if ( res /= 8615) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5797
      if ( res /= 5797) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4895
      if ( res /= 4895) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3542
      if ( res /= 3542) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9104
      if ( res /= 9104) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1471
      if ( res /= 1471) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8322
      if ( res /= 8322) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WMinus, P_HPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1596
      if ( res /= 1596) then
        stop
      end if
      call createParticleConfig(P_g, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4637
      if ( res /= 4637) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2226
      if ( res /= 2226) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8376
      if ( res /= 8376) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10213
      if ( res /= 10213) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9880
      if ( res /= 9880) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6324
      if ( res /= 6324) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_GPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8502
      if ( res /= 8502) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3000
      if ( res /= 3000) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5725
      if ( res /= 5725) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7666
      if ( res /= 7666) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",171
      if ( res /= 171) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1931
      if ( res /= 1931) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5887
      if ( res /= 5887) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1043
      if ( res /= 1043) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4582
      if ( res /= 4582) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6260
      if ( res /= 6260) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8949
      if ( res /= 8949) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7182
      if ( res /= 7182) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7910
      if ( res /= 7910) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9405
      if ( res /= 9405) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7087
      if ( res /= 7087) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",805
      if ( res /= 805) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10679
      if ( res /= 10679) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8173
      if ( res /= 8173) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1789
      if ( res /= 1789) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6652
      if ( res /= 6652) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10241
      if ( res /= 10241) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6343
      if ( res /= 6343) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WPlus, P_GMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8882
      if ( res /= 8882) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4979
      if ( res /= 4979) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5574
      if ( res /= 5574) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10684
      if ( res /= 10684) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1460
      if ( res /= 1460) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3937
      if ( res /= 3937) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",776
      if ( res /= 776) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4315
      if ( res /= 4315) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5151
      if ( res /= 5151) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7716
      if ( res /= 7716) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7901
      if ( res /= 7901) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6059
      if ( res /= 6059) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9770
      if ( res /= 9770) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HPlus, P_HMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1008
      if ( res /= 1008) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1034
      if ( res /= 1034) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1061
      if ( res /= 1061) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10150
      if ( res /= 10150) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2519
      if ( res /= 2519) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9619
      if ( res /= 9619) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GPlus, P_GMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6721
      if ( res /= 6721) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghA, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3784
      if ( res /= 3784) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5397
      if ( res /= 5397) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6080
      if ( res /= 6080) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2538
      if ( res /= 2538) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4747
      if ( res /= 4747) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWp, P_WMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8040
      if ( res /= 8040) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2537
      if ( res /= 2537) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5952
      if ( res /= 5952) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1604
      if ( res /= 1604) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5258
      if ( res /= 5258) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8161
      if ( res /= 8161) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",271
      if ( res /= 271) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5823
      if ( res /= 5823) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6828
      if ( res /= 6828) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghA, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5172
      if ( res /= 5172) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10037
      if ( res /= 10037) then
        stop
      end if
      call createParticleConfig(P_Hh, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5997
      if ( res /= 5997) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7348
      if ( res /= 7348) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WPlus, P_WMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3815
      if ( res /= 3815) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2509
      if ( res /= 2509) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4538
      if ( res /= 4538) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_Ha, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8449
      if ( res /= 8449) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10673
      if ( res /= 10673) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8636
      if ( res /= 8636) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4502
      if ( res /= 4502) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1168
      if ( res /= 1168) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",939
      if ( res /= 939) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6227
      if ( res /= 6227) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1524
      if ( res /= 1524) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",581
      if ( res /= 581) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1819
      if ( res /= 1819) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10084
      if ( res /= 10084) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_Z, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7273
      if ( res /= 7273) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3108
      if ( res /= 3108) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8894
      if ( res /= 8894) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8230
      if ( res /= 8230) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1240
      if ( res /= 1240) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghAAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2913
      if ( res /= 2913) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9092
      if ( res /= 9092) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8237
      if ( res /= 8237) then
        stop
      end if
      call createParticleConfig(P_Hl, P_g, P_Hl, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10754
      if ( res /= 10754) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8693
      if ( res /= 8693) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4929
      if ( res /= 4929) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZ, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2391
      if ( res /= 2391) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",914
      if ( res /= 914) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5678
      if ( res /= 5678) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2981
      if ( res /= 2981) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8022
      if ( res /= 8022) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghWpAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",935
      if ( res /= 935) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4344
      if ( res /= 4344) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",633
      if ( res /= 633) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_g, P_GMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10488
      if ( res /= 10488) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",354
      if ( res /= 354) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4127
      if ( res /= 4127) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5138
      if ( res /= 5138) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_tauPlus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5361
      if ( res /= 5361) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7825
      if ( res /= 7825) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9629
      if ( res /= 9629) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2644
      if ( res /= 2644) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5710
      if ( res /= 5710) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1265
      if ( res /= 1265) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6667
      if ( res /= 6667) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WPlus, P_HMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",465
      if ( res /= 465) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2896
      if ( res /= 2896) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7176
      if ( res /= 7176) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4108
      if ( res /= 4108) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1293
      if ( res /= 1293) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7163
      if ( res /= 7163) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWp, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6845
      if ( res /= 6845) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10530
      if ( res /= 10530) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8880
      if ( res /= 8880) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3159
      if ( res /= 3159) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5548
      if ( res /= 5548) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7043
      if ( res /= 7043) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9658
      if ( res /= 9658) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8825
      if ( res /= 8825) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5614
      if ( res /= 5614) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9601
      if ( res /= 9601) then
        stop
      end if
      call createParticleConfig(P_gQ, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4759
      if ( res /= 4759) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3374
      if ( res /= 3374) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10039
      if ( res /= 10039) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",697
      if ( res /= 697) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7381
      if ( res /= 7381) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",41
      if ( res /= 41) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4621
      if ( res /= 4621) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9577
      if ( res /= 9577) then
        stop
      end if
      call createParticleConfig(P_u, P_dAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8153
      if ( res /= 8153) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2557
      if ( res /= 2557) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5212
      if ( res /= 5212) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10728
      if ( res /= 10728) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7777
      if ( res /= 7777) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2328
      if ( res /= 2328) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",670
      if ( res /= 670) then
        stop
      end if
      call createParticleConfig(P_u, P_dAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3130
      if ( res /= 3130) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3321
      if ( res /= 3321) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1602
      if ( res /= 1602) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7197
      if ( res /= 7197) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6693
      if ( res /= 6693) then
        stop
      end if
      call createParticleConfig(P_Hh, P_g, P_Hl, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10568
      if ( res /= 10568) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3706
      if ( res /= 3706) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WPlus, P_ghWm, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2996
      if ( res /= 2996) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GMinus, P_HPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8378
      if ( res /= 8378) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghWp, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3745
      if ( res /= 3745) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8446
      if ( res /= 8446) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8519
      if ( res /= 8519) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8891
      if ( res /= 8891) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8154
      if ( res /= 8154) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3586
      if ( res /= 3586) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4557
      if ( res /= 4557) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4643
      if ( res /= 4643) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8817
      if ( res /= 8817) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3164
      if ( res /= 3164) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",655
      if ( res /= 655) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",986
      if ( res /= 986) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10462
      if ( res /= 10462) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWpAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",984
      if ( res /= 984) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6569
      if ( res /= 6569) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2066
      if ( res /= 2066) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7954
      if ( res /= 7954) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",749
      if ( res /= 749) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1521
      if ( res /= 1521) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_GMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3373
      if ( res /= 3373) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_eMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6147
      if ( res /= 6147) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6474
      if ( res /= 6474) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7874
      if ( res /= 7874) then
        stop
      end if
      call createParticleConfig(P_Hh, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6456
      if ( res /= 6456) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2009
      if ( res /= 2009) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8809
      if ( res /= 8809) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3404
      if ( res /= 3404) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2674
      if ( res /= 2674) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9785
      if ( res /= 9785) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8480
      if ( res /= 8480) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3254
      if ( res /= 3254) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1088
      if ( res /= 1088) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10578
      if ( res /= 10578) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4985
      if ( res /= 4985) then
        stop
      end if
      call createParticleConfig(P_g, P_WMinus, P_WPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10574
      if ( res /= 10574) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6507
      if ( res /= 6507) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",178
      if ( res /= 178) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4888
      if ( res /= 4888) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghZ, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",462
      if ( res /= 462) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3058
      if ( res /= 3058) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6608
      if ( res /= 6608) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4266
      if ( res /= 4266) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5086
      if ( res /= 5086) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4477
      if ( res /= 4477) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",543
      if ( res /= 543) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1886
      if ( res /= 1886) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2353
      if ( res /= 2353) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghZ, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1135
      if ( res /= 1135) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2437
      if ( res /= 2437) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1264
      if ( res /= 1264) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7335
      if ( res /= 7335) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_WPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8768
      if ( res /= 8768) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghA, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3794
      if ( res /= 3794) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3048
      if ( res /= 3048) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1219
      if ( res /= 1219) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9818
      if ( res /= 9818) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4576
      if ( res /= 4576) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1067
      if ( res /= 1067) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2583
      if ( res /= 2583) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",434
      if ( res /= 434) then
        stop
      end if
      call createParticleConfig(P_s, P_cAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8760
      if ( res /= 8760) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7365
      if ( res /= 7365) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2536
      if ( res /= 2536) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10318
      if ( res /= 10318) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10202
      if ( res /= 10202) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2671
      if ( res /= 2671) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9452
      if ( res /= 9452) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",373
      if ( res /= 373) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2223
      if ( res /= 2223) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10665
      if ( res /= 10665) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_muPlus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4098
      if ( res /= 4098) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3766
      if ( res /= 3766) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10095
      if ( res /= 10095) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2421
      if ( res /= 2421) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1905
      if ( res /= 1905) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1369
      if ( res /= 1369) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9878
      if ( res /= 9878) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5567
      if ( res /= 5567) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2202
      if ( res /= 2202) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10046
      if ( res /= 10046) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5922
      if ( res /= 5922) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4786
      if ( res /= 4786) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7231
      if ( res /= 7231) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6047
      if ( res /= 6047) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4005
      if ( res /= 4005) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8990
      if ( res /= 8990) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5082
      if ( res /= 5082) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HMinus, P_HPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2104
      if ( res /= 2104) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5134
      if ( res /= 5134) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8467
      if ( res /= 8467) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWpAnti, P_ghWp, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10324
      if ( res /= 10324) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9512
      if ( res /= 9512) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4262
      if ( res /= 4262) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWp, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4845
      if ( res /= 4845) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1616
      if ( res /= 1616) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2505
      if ( res /= 2505) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9436
      if ( res /= 9436) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9222
      if ( res /= 9222) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",724
      if ( res /= 724) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5806
      if ( res /= 5806) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9903
      if ( res /= 9903) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2433
      if ( res /= 2433) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3552
      if ( res /= 3552) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4771
      if ( res /= 4771) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8672
      if ( res /= 8672) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4810
      if ( res /= 4810) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",972
      if ( res /= 972) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5419
      if ( res /= 5419) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",969
      if ( res /= 969) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8326
      if ( res /= 8326) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7144
      if ( res /= 7144) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghZAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8074
      if ( res /= 8074) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4700
      if ( res /= 4700) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9843
      if ( res /= 9843) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6692
      if ( res /= 6692) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6312
      if ( res /= 6312) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5718
      if ( res /= 5718) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8190
      if ( res /= 8190) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6371
      if ( res /= 6371) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3981
      if ( res /= 3981) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",847
      if ( res /= 847) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2247
      if ( res /= 2247) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",371
      if ( res /= 371) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8056
      if ( res /= 8056) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_HMinus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5954
      if ( res /= 5954) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3322
      if ( res /= 3322) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7181
      if ( res /= 7181) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",152
      if ( res /= 152) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8111
      if ( res /= 8111) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5582
      if ( res /= 5582) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",542
      if ( res /= 542) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5149
      if ( res /= 5149) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1123
      if ( res /= 1123) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",799
      if ( res /= 799) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5059
      if ( res /= 5059) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6088
      if ( res /= 6088) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1399
      if ( res /= 1399) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3500
      if ( res /= 3500) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5972
      if ( res /= 5972) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8181
      if ( res /= 8181) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Z, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",275
      if ( res /= 275) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_A, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3113
      if ( res /= 3113) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghZAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",864
      if ( res /= 864) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2950
      if ( res /= 2950) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZ, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8401
      if ( res /= 8401) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7076
      if ( res /= 7076) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2473
      if ( res /= 2473) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5600
      if ( res /= 5600) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10593
      if ( res /= 10593) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_Z, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",934
      if ( res /= 934) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10701
      if ( res /= 10701) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2030
      if ( res /= 2030) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWm, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4990
      if ( res /= 4990) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",324
      if ( res /= 324) then
        stop
      end if
      call createParticleConfig(P_Z, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7384
      if ( res /= 7384) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghWm, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4773
      if ( res /= 4773) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",976
      if ( res /= 976) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghA, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2356
      if ( res /= 2356) then
        stop
      end if
      call createParticleConfig(P_b, P_tAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7841
      if ( res /= 7841) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HMinus, P_GPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8300
      if ( res /= 8300) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",713
      if ( res /= 713) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3509
      if ( res /= 3509) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10159
      if ( res /= 10159) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10695
      if ( res /= 10695) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6664
      if ( res /= 6664) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",687
      if ( res /= 687) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3772
      if ( res /= 3772) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5921
      if ( res /= 5921) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6574
      if ( res /= 6574) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6512
      if ( res /= 6512) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4036
      if ( res /= 4036) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5329
      if ( res /= 5329) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7986
      if ( res /= 7986) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6063
      if ( res /= 6063) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3228
      if ( res /= 3228) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_nu_muAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3748
      if ( res /= 3748) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7164
      if ( res /= 7164) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",195
      if ( res /= 195) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3211
      if ( res /= 3211) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9664
      if ( res /= 9664) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4471
      if ( res /= 4471) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10287
      if ( res /= 10287) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8669
      if ( res /= 8669) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7516
      if ( res /= 7516) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3515
      if ( res /= 3515) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5409
      if ( res /= 5409) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8404
      if ( res /= 8404) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4356
      if ( res /= 4356) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWp, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7193
      if ( res /= 7193) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8945
      if ( res /= 8945) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7805
      if ( res /= 7805) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3090
      if ( res /= 3090) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10168
      if ( res /= 10168) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6737
      if ( res /= 6737) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5850
      if ( res /= 5850) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GMinus, P_ghWp, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9881
      if ( res /= 9881) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6219
      if ( res /= 6219) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6318
      if ( res /= 6318) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_sAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5698
      if ( res /= 5698) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghWm, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",172
      if ( res /= 172) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghZ, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",947
      if ( res /= 947) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7061
      if ( res /= 7061) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6974
      if ( res /= 6974) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5300
      if ( res /= 5300) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghZAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2737
      if ( res /= 2737) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1296
      if ( res /= 1296) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6998
      if ( res /= 6998) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9663
      if ( res /= 9663) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1752
      if ( res /= 1752) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5265
      if ( res /= 5265) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5126
      if ( res /= 5126) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_A, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5438
      if ( res /= 5438) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8832
      if ( res /= 8832) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9811
      if ( res /= 9811) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",822
      if ( res /= 822) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_bAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",183
      if ( res /= 183) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_A, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4871
      if ( res /= 4871) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8299
      if ( res /= 8299) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6054
      if ( res /= 6054) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8666
      if ( res /= 8666) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10638
      if ( res /= 10638) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10000
      if ( res /= 10000) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4928
      if ( res /= 4928) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8457
      if ( res /= 8457) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4015
      if ( res /= 4015) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3506
      if ( res /= 3506) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2897
      if ( res /= 2897) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2190
      if ( res /= 2190) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6282
      if ( res /= 6282) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4712
      if ( res /= 4712) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3602
      if ( res /= 3602) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2768
      if ( res /= 2768) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4235
      if ( res /= 4235) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2070
      if ( res /= 2070) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9905
      if ( res /= 9905) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9459
      if ( res /= 9459) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4191
      if ( res /= 4191) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2196
      if ( res /= 2196) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10205
      if ( res /= 10205) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9977
      if ( res /= 9977) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8707
      if ( res /= 8707) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4128
      if ( res /= 4128) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWm, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8368
      if ( res /= 8368) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5603
      if ( res /= 5603) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_GPlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1813
      if ( res /= 1813) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1505
      if ( res /= 1505) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2705
      if ( res /= 2705) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4166
      if ( res /= 4166) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5488
      if ( res /= 5488) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2486
      if ( res /= 2486) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWmAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8142
      if ( res /= 8142) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10209
      if ( res /= 10209) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",232
      if ( res /= 232) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6942
      if ( res /= 6942) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1052
      if ( res /= 1052) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1395
      if ( res /= 1395) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4930
      if ( res /= 4930) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4071
      if ( res /= 4071) then
        stop
      end if
      call createParticleConfig(P_g, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4384
      if ( res /= 4384) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1277
      if ( res /= 1277) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5777
      if ( res /= 5777) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5111
      if ( res /= 5111) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3626
      if ( res /= 3626) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5089
      if ( res /= 5089) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1344
      if ( res /= 1344) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3071
      if ( res /= 3071) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10253
      if ( res /= 10253) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7288
      if ( res /= 7288) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4197
      if ( res /= 4197) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_A, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",489
      if ( res /= 489) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7270
      if ( res /= 7270) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8248
      if ( res /= 8248) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7331
      if ( res /= 7331) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9573
      if ( res /= 9573) then
        stop
      end if
      call createParticleConfig(P_c, P_sAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",233
      if ( res /= 233) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_sAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7904
      if ( res /= 7904) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",629
      if ( res /= 629) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2127
      if ( res /= 2127) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",268
      if ( res /= 268) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6891
      if ( res /= 6891) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7232
      if ( res /= 7232) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9853
      if ( res /= 9853) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2046
      if ( res /= 2046) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8725
      if ( res /= 8725) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7723
      if ( res /= 7723) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10047
      if ( res /= 10047) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",905
      if ( res /= 905) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZ, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5967
      if ( res /= 5967) then
        stop
      end if
      call createParticleConfig(P_G0, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3951
      if ( res /= 3951) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5188
      if ( res /= 5188) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3597
      if ( res /= 3597) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1065
      if ( res /= 1065) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9039
      if ( res /= 9039) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9065
      if ( res /= 9065) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8478
      if ( res /= 8478) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5601
      if ( res /= 5601) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8560
      if ( res /= 8560) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2689
      if ( res /= 2689) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5843
      if ( res /= 5843) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10579
      if ( res /= 10579) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8204
      if ( res /= 8204) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7999
      if ( res /= 7999) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_WMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5429
      if ( res /= 5429) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5934
      if ( res /= 5934) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4264
      if ( res /= 4264) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8771
      if ( res /= 8771) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8769
      if ( res /= 8769) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GMinus, P_GPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2987
      if ( res /= 2987) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2482
      if ( res /= 2482) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4143
      if ( res /= 4143) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5022
      if ( res /= 5022) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8485
      if ( res /= 8485) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6795
      if ( res /= 6795) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5049
      if ( res /= 5049) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5617
      if ( res /= 5617) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9980
      if ( res /= 9980) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",189
      if ( res /= 189) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6146
      if ( res /= 6146) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7370
      if ( res /= 7370) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5688
      if ( res /= 5688) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4209
      if ( res /= 4209) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10340
      if ( res /= 10340) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1509
      if ( res /= 1509) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7736
      if ( res /= 7736) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10217
      if ( res /= 10217) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HPlus, P_ghWpAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8986
      if ( res /= 8986) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2111
      if ( res /= 2111) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9545
      if ( res /= 9545) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6856
      if ( res /= 6856) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10182
      if ( res /= 10182) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3965
      if ( res /= 3965) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2118
      if ( res /= 2118) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7528
      if ( res /= 7528) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7147
      if ( res /= 7147) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4361
      if ( res /= 4361) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9374
      if ( res /= 9374) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3724
      if ( res /= 3724) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",118
      if ( res /= 118) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6434
      if ( res /= 6434) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WMinus, P_WPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3582
      if ( res /= 3582) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9389
      if ( res /= 9389) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1966
      if ( res /= 1966) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5493
      if ( res /= 5493) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",788
      if ( res /= 788) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5840
      if ( res /= 5840) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5966
      if ( res /= 5966) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7978
      if ( res /= 7978) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2129
      if ( res /= 2129) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",49
      if ( res /= 49) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWm, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7265
      if ( res /= 7265) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3140
      if ( res /= 3140) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3051
      if ( res /= 3051) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghZAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7951
      if ( res /= 7951) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9462
      if ( res /= 9462) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8425
      if ( res /= 8425) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7484
      if ( res /= 7484) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3961
      if ( res /= 3961) then
        stop
      end if
      call createParticleConfig(P_Ha, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7428
      if ( res /= 7428) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7264
      if ( res /= 7264) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7658
      if ( res /= 7658) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",902
      if ( res /= 902) then
        stop
      end if
      call createParticleConfig(P_Z, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1486
      if ( res /= 1486) then
        stop
      end if
      call createParticleConfig(P_b, P_Ha, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4978
      if ( res /= 4978) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2215
      if ( res /= 2215) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5251
      if ( res /= 5251) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2831
      if ( res /= 2831) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7491
      if ( res /= 7491) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6045
      if ( res /= 6045) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8147
      if ( res /= 8147) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1271
      if ( res /= 1271) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7600
      if ( res /= 7600) then
        stop
      end if
      call createParticleConfig(P_G0, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7732
      if ( res /= 7732) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3806
      if ( res /= 3806) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6174
      if ( res /= 6174) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",213
      if ( res /= 213) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4030
      if ( res /= 4030) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5608
      if ( res /= 5608) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWm, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6062
      if ( res /= 6062) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9146
      if ( res /= 9146) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WPlus, P_ghWpAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",576
      if ( res /= 576) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5090
      if ( res /= 5090) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2293
      if ( res /= 2293) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1668
      if ( res /= 1668) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2775
      if ( res /= 2775) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5886
      if ( res /= 5886) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8381
      if ( res /= 8381) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4085
      if ( res /= 4085) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7293
      if ( res /= 7293) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2925
      if ( res /= 2925) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10161
      if ( res /= 10161) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_nu_e, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3637
      if ( res /= 3637) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8092
      if ( res /= 8092) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4393
      if ( res /= 4393) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3363
      if ( res /= 3363) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3929
      if ( res /= 3929) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6419
      if ( res /= 6419) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9438
      if ( res /= 9438) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10341
      if ( res /= 10341) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10537
      if ( res /= 10537) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1140
      if ( res /= 1140) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_d, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4136
      if ( res /= 4136) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1868
      if ( res /= 1868) then
        stop
      end if
      call createParticleConfig(P_AQ, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7960
      if ( res /= 7960) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghWpAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5112
      if ( res /= 5112) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",322
      if ( res /= 322) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_A, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4907
      if ( res /= 4907) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9683
      if ( res /= 9683) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7077
      if ( res /= 7077) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",968
      if ( res /= 968) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2312
      if ( res /= 2312) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_g, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10675
      if ( res /= 10675) then
        stop
      end if
      call createParticleConfig(P_d, P_Z, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",115
      if ( res /= 115) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6223
      if ( res /= 6223) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9937
      if ( res /= 9937) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1846
      if ( res /= 1846) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9626
      if ( res /= 9626) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3101
      if ( res /= 3101) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",784
      if ( res /= 784) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8170
      if ( res /= 8170) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10514
      if ( res /= 10514) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5459
      if ( res /= 5459) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6849
      if ( res /= 6849) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7113
      if ( res /= 7113) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",276
      if ( res /= 276) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GPlus, P_GMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5789
      if ( res /= 5789) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1105
      if ( res /= 1105) then
        stop
      end if
      call createParticleConfig(P_gQ, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2952
      if ( res /= 2952) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1497
      if ( res /= 1497) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10180
      if ( res /= 10180) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7405
      if ( res /= 7405) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1942
      if ( res /= 1942) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2453
      if ( res /= 2453) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10747
      if ( res /= 10747) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1717
      if ( res /= 1717) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4346
      if ( res /= 4346) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_HPlus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7838
      if ( res /= 7838) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8342
      if ( res /= 8342) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5028
      if ( res /= 5028) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",292
      if ( res /= 292) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7101
      if ( res /= 7101) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",392
      if ( res /= 392) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8488
      if ( res /= 8488) then
        stop
      end if
      call createParticleConfig(P_t, P_G0, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6004
      if ( res /= 6004) then
        stop
      end if
      call createParticleConfig(P_g, P_Hl, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10533
      if ( res /= 10533) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3335
      if ( res /= 3335) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4046
      if ( res /= 4046) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",889
      if ( res /= 889) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",483
      if ( res /= 483) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5451
      if ( res /= 5451) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8904
      if ( res /= 8904) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1654
      if ( res /= 1654) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9505
      if ( res /= 9505) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4560
      if ( res /= 4560) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10109
      if ( res /= 10109) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10499
      if ( res /= 10499) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2369
      if ( res /= 2369) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8365
      if ( res /= 8365) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5421
      if ( res /= 5421) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2487
      if ( res /= 2487) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6052
      if ( res /= 6052) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9451
      if ( res /= 9451) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5455
      if ( res /= 5455) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4295
      if ( res /= 4295) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8088
      if ( res /= 8088) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10233
      if ( res /= 10233) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghA, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4831
      if ( res /= 4831) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9196
      if ( res /= 9196) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3711
      if ( res /= 3711) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10443
      if ( res /= 10443) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",236
      if ( res /= 236) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWpAnti, P_HPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2130
      if ( res /= 2130) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3369
      if ( res /= 3369) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3471
      if ( res /= 3471) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9687
      if ( res /= 9687) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghWp, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",949
      if ( res /= 949) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3554
      if ( res /= 3554) then
        stop
      end if
      call createParticleConfig(P_t, P_GMinus, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5193
      if ( res /= 5193) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWpAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1790
      if ( res /= 1790) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8659
      if ( res /= 8659) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5655
      if ( res /= 5655) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8172
      if ( res /= 8172) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7587
      if ( res /= 7587) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10127
      if ( res /= 10127) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3664
      if ( res /= 3664) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9448
      if ( res /= 9448) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5983
      if ( res /= 5983) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",556
      if ( res /= 556) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6399
      if ( res /= 6399) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5970
      if ( res /= 5970) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4351
      if ( res /= 4351) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6545
      if ( res /= 6545) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3189
      if ( res /= 3189) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5051
      if ( res /= 5051) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3145
      if ( res /= 3145) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9215
      if ( res /= 9215) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3120
      if ( res /= 3120) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2427
      if ( res /= 2427) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7012
      if ( res /= 7012) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7683
      if ( res /= 7683) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10398
      if ( res /= 10398) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7120
      if ( res /= 7120) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3223
      if ( res /= 3223) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10007
      if ( res /= 10007) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6154
      if ( res /= 6154) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10543
      if ( res /= 10543) then
        stop
      end if
      call createParticleConfig(P_gQ, P_g, P_ghGAnti, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7947
      if ( res /= 7947) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10437
      if ( res /= 10437) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2084
      if ( res /= 2084) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7715
      if ( res /= 7715) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3215
      if ( res /= 3215) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3968
      if ( res /= 3968) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4860
      if ( res /= 4860) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",343
      if ( res /= 343) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1864
      if ( res /= 1864) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2625
      if ( res /= 2625) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2873
      if ( res /= 2873) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8660
      if ( res /= 8660) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6644
      if ( res /= 6644) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6824
      if ( res /= 6824) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",403
      if ( res /= 403) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4583
      if ( res /= 4583) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",823
      if ( res /= 823) then
        stop
      end if
      call createParticleConfig(P_G0, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8811
      if ( res /= 8811) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2044
      if ( res /= 2044) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6637
      if ( res /= 6637) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6261
      if ( res /= 6261) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_nu_tau, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1162
      if ( res /= 1162) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1938
      if ( res /= 1938) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1725
      if ( res /= 1725) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7919
      if ( res /= 7919) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWp, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6493
      if ( res /= 6493) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1433
      if ( res /= 1433) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7072
      if ( res /= 7072) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",895
      if ( res /= 895) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5117
      if ( res /= 5117) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1017
      if ( res /= 1017) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2933
      if ( res /= 2933) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3203
      if ( res /= 3203) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5508
      if ( res /= 5508) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8696
      if ( res /= 8696) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5833
      if ( res /= 5833) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9593
      if ( res /= 9593) then
        stop
      end if
      call createParticleConfig(P_AQ, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5989
      if ( res /= 5989) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9844
      if ( res /= 9844) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2082
      if ( res /= 2082) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10361
      if ( res /= 10361) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6787
      if ( res /= 6787) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7040
      if ( res /= 7040) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9537
      if ( res /= 9537) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",186
      if ( res /= 186) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1760
      if ( res /= 1760) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6380
      if ( res /= 6380) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8426
      if ( res /= 8426) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",125
      if ( res /= 125) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1569
      if ( res /= 1569) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",707
      if ( res /= 707) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7967
      if ( res /= 7967) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5971
      if ( res /= 5971) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2629
      if ( res /= 2629) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10166
      if ( res /= 10166) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5118
      if ( res /= 5118) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6547
      if ( res /= 6547) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1597
      if ( res /= 1597) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZ, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7151
      if ( res /= 7151) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6807
      if ( res /= 6807) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7507
      if ( res /= 7507) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",636
      if ( res /= 636) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2490
      if ( res /= 2490) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10542
      if ( res /= 10542) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1094
      if ( res /= 1094) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6262
      if ( res /= 6262) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_t, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7446
      if ( res /= 7446) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9872
      if ( res /= 9872) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9082
      if ( res /= 9082) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2485
      if ( res /= 2485) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6818
      if ( res /= 6818) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1783
      if ( res /= 1783) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_WMinus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3453
      if ( res /= 3453) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7392
      if ( res /= 7392) then
        stop
      end if
      call createParticleConfig(P_g, P_HPlus, P_g, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10667
      if ( res /= 10667) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7807
      if ( res /= 7807) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",458
      if ( res /= 458) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4961
      if ( res /= 4961) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghA, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4857
      if ( res /= 4857) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10439
      if ( res /= 10439) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3700
      if ( res /= 3700) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6055
      if ( res /= 6055) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5450
      if ( res /= 5450) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",15
      if ( res /= 15) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10325
      if ( res /= 10325) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9428
      if ( res /= 9428) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5551
      if ( res /= 5551) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3843
      if ( res /= 3843) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_nu_e, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8334
      if ( res /= 8334) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7691
      if ( res /= 7691) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6464
      if ( res /= 6464) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7813
      if ( res /= 7813) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10522
      if ( res /= 10522) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",414
      if ( res /= 414) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1364
      if ( res /= 1364) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2254
      if ( res /= 2254) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4021
      if ( res /= 4021) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2248
      if ( res /= 2248) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6684
      if ( res /= 6684) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4306
      if ( res /= 4306) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9347
      if ( res /= 9347) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",220
      if ( res /= 220) then
        stop
      end if
      call createParticleConfig(P_c, P_g, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2552
      if ( res /= 2552) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2298
      if ( res /= 2298) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",80
      if ( res /= 80) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GPlus, P_HMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5346
      if ( res /= 5346) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10722
      if ( res /= 10722) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5514
      if ( res /= 5514) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3080
      if ( res /= 3080) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HPlus, P_GMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6847
      if ( res /= 6847) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8923
      if ( res /= 8923) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",700
      if ( res /= 700) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6331
      if ( res /= 6331) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5042
      if ( res /= 5042) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",325
      if ( res /= 325) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5483
      if ( res /= 5483) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9973
      if ( res /= 9973) then
        stop
      end if
      call createParticleConfig(P_u, P_Ha, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1533
      if ( res /= 1533) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8895
      if ( res /= 8895) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3200
      if ( res /= 3200) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5953
      if ( res /= 5953) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",746
      if ( res /= 746) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8493
      if ( res /= 8493) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5899
      if ( res /= 5899) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7802
      if ( res /= 7802) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3975
      if ( res /= 3975) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2955
      if ( res /= 2955) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7993
      if ( res /= 7993) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9634
      if ( res /= 9634) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6268
      if ( res /= 6268) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1753
      if ( res /= 1753) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9046
      if ( res /= 9046) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5521
      if ( res /= 5521) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5359
      if ( res /= 5359) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4885
      if ( res /= 4885) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WMinus, P_WPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6565
      if ( res /= 6565) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_tAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4318
      if ( res /= 4318) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWp, P_WMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8721
      if ( res /= 8721) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9053
      if ( res /= 9053) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2426
      if ( res /= 2426) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1771
      if ( res /= 1771) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2018
      if ( res /= 2018) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4619
      if ( res /= 4619) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8668
      if ( res /= 8668) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8941
      if ( res /= 8941) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3232
      if ( res /= 3232) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4890
      if ( res /= 4890) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2863
      if ( res /= 2863) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2906
      if ( res /= 2906) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4544
      if ( res /= 4544) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9360
      if ( res /= 9360) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1583
      if ( res /= 1583) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9254
      if ( res /= 9254) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6841
      if ( res /= 6841) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghWmAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",283
      if ( res /= 283) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2138
      if ( res /= 2138) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8538
      if ( res /= 8538) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7593
      if ( res /= 7593) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6292
      if ( res /= 6292) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWp, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6354
      if ( res /= 6354) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3764
      if ( res /= 3764) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2944
      if ( res /= 2944) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3757
      if ( res /= 3757) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4578
      if ( res /= 4578) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10085
      if ( res /= 10085) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9902
      if ( res /= 9902) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9321
      if ( res /= 9321) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8562
      if ( res /= 8562) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9255
      if ( res /= 9255) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6285
      if ( res /= 6285) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6475
      if ( res /= 6475) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3708
      if ( res /= 3708) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GMinus, P_WPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5509
      if ( res /= 5509) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_nu_mu, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2062
      if ( res /= 2062) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5447
      if ( res /= 5447) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2745
      if ( res /= 2745) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1211
      if ( res /= 1211) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8602
      if ( res /= 8602) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6149
      if ( res /= 6149) then
        stop
      end if
      call createParticleConfig(P_s, P_Hl, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4308
      if ( res /= 4308) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_s, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5985
      if ( res /= 5985) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6025
      if ( res /= 6025) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8393
      if ( res /= 8393) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1939
      if ( res /= 1939) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4125
      if ( res /= 4125) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6413
      if ( res /= 6413) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6848
      if ( res /= 6848) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6542
      if ( res /= 6542) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_HPlus, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6889
      if ( res /= 6889) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9873
      if ( res /= 9873) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5728
      if ( res /= 5728) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2727
      if ( res /= 2727) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7051
      if ( res /= 7051) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",28
      if ( res /= 28) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5000
      if ( res /= 5000) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7468
      if ( res /= 7468) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6320
      if ( res /= 6320) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3449
      if ( res /= 3449) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9235
      if ( res /= 9235) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5915
      if ( res /= 5915) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3443
      if ( res /= 3443) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1733
      if ( res /= 1733) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2060
      if ( res /= 2060) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4580
      if ( res /= 4580) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8601
      if ( res /= 8601) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5223
      if ( res /= 5223) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4974
      if ( res /= 4974) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3420
      if ( res /= 3420) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",107
      if ( res /= 107) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghZAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3431
      if ( res /= 3431) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4784
      if ( res /= 4784) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Ha, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8316
      if ( res /= 8316) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8030
      if ( res /= 8030) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6402
      if ( res /= 6402) then
        stop
      end if
      call createParticleConfig(P_Hl, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",188
      if ( res /= 188) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1348
      if ( res /= 1348) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8261
      if ( res /= 8261) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8581
      if ( res /= 8581) then
        stop
      end if
      call createParticleConfig(P_Hh, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1657
      if ( res /= 1657) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8708
      if ( res /= 8708) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",488
      if ( res /= 488) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2481
      if ( res /= 2481) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7452
      if ( res /= 7452) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3980
      if ( res /= 3980) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5670
      if ( res /= 5670) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8412
      if ( res /= 8412) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7220
      if ( res /= 7220) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10576
      if ( res /= 10576) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7320
      if ( res /= 7320) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4422
      if ( res /= 4422) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghWpAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",747
      if ( res /= 747) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1152
      if ( res /= 1152) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5564
      if ( res /= 5564) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2973
      if ( res /= 2973) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_g, P_g, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10570
      if ( res /= 10570) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3258
      if ( res /= 3258) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3371
      if ( res /= 3371) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2193
      if ( res /= 2193) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_G0, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4757
      if ( res /= 4757) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8685
      if ( res /= 8685) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WPlus, P_WMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1757
      if ( res /= 1757) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10211
      if ( res /= 10211) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",859
      if ( res /= 859) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5507
      if ( res /= 5507) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3833
      if ( res /= 3833) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWmAnti, P_ghWm, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10051
      if ( res /= 10051) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9176
      if ( res /= 9176) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8027
      if ( res /= 8027) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7581
      if ( res /= 7581) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3948
      if ( res /= 3948) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6437
      if ( res /= 6437) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5651
      if ( res /= 5651) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6103
      if ( res /= 6103) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6017
      if ( res /= 6017) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4107
      if ( res /= 4107) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5866
      if ( res /= 5866) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3798
      if ( res /= 3798) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8588
      if ( res /= 8588) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_ePlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5417
      if ( res /= 5417) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1818
      if ( res /= 1818) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10644
      if ( res /= 10644) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10516
      if ( res /= 10516) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4699
      if ( res /= 4699) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5794
      if ( res /= 5794) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4325
      if ( res /= 4325) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",174
      if ( res /= 174) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2819
      if ( res /= 2819) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4724
      if ( res /= 4724) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10305
      if ( res /= 10305) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9691
      if ( res /= 9691) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1051
      if ( res /= 1051) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghAAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6612
      if ( res /= 6612) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3518
      if ( res /= 3518) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",441
      if ( res /= 441) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8402
      if ( res /= 8402) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7242
      if ( res /= 7242) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",646
      if ( res /= 646) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7681
      if ( res /= 7681) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3596
      if ( res /= 3596) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9018
      if ( res /= 9018) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",358
      if ( res /= 358) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2290
      if ( res /= 2290) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10355
      if ( res /= 10355) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",802
      if ( res /= 802) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2338
      if ( res /= 2338) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2301
      if ( res /= 2301) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9330
      if ( res /= 9330) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghA, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",602
      if ( res /= 602) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6279
      if ( res /= 6279) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_Z, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4441
      if ( res /= 4441) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1202
      if ( res /= 1202) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10734
      if ( res /= 10734) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1107
      if ( res /= 1107) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4710
      if ( res /= 4710) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7257
      if ( res /= 7257) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_tauMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7448
      if ( res /= 7448) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9306
      if ( res /= 9306) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",501
      if ( res /= 501) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",740
      if ( res /= 740) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3268
      if ( res /= 3268) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WPlus, P_ghWpAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",773
      if ( res /= 773) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8059
      if ( res /= 8059) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4646
      if ( res /= 4646) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7155
      if ( res /= 7155) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9895
      if ( res /= 9895) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10069
      if ( res /= 10069) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6397
      if ( res /= 6397) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_g, P_g, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10735
      if ( res /= 10735) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",577
      if ( res /= 577) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1297
      if ( res /= 1297) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghZAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3855
      if ( res /= 3855) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_nu_eAnti, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1606
      if ( res /= 1606) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9596
      if ( res /= 9596) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4825
      if ( res /= 4825) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7444
      if ( res /= 7444) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6412
      if ( res /= 6412) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7115
      if ( res /= 7115) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2462
      if ( res /= 2462) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8885
      if ( res /= 8885) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5661
      if ( res /= 5661) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9700
      if ( res /= 9700) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5583
      if ( res /= 5583) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8514
      if ( res /= 8514) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6000
      if ( res /= 6000) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5103
      if ( res /= 5103) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",249
      if ( res /= 249) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_ePlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8146
      if ( res /= 8146) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4252
      if ( res /= 4252) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9764
      if ( res /= 9764) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9359
      if ( res /= 9359) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4770
      if ( res /= 4770) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6114
      if ( res /= 6114) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4748
      if ( res /= 4748) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9968
      if ( res /= 9968) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4912
      if ( res /= 4912) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6913
      if ( res /= 6913) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4367
      if ( res /= 4367) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6859
      if ( res /= 6859) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9517
      if ( res /= 9517) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9958
      if ( res /= 9958) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6979
      if ( res /= 6979) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4760
      if ( res /= 4760) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_tauMinus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7315
      if ( res /= 7315) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1811
      if ( res /= 1811) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4852
      if ( res /= 4852) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghWpAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4936
      if ( res /= 4936) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1615
      if ( res /= 1615) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7389
      if ( res /= 7389) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5366
      if ( res /= 5366) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7668
      if ( res /= 7668) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_ePlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2695
      if ( res /= 2695) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",715
      if ( res /= 715) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10451
      if ( res /= 10451) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9906
      if ( res /= 9906) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWm, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8447
      if ( res /= 8447) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_Ha, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",897
      if ( res /= 897) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3476
      if ( res /= 3476) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1934
      if ( res /= 1934) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7519
      if ( res /= 7519) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10677
      if ( res /= 10677) then
        stop
      end if
      call createParticleConfig(P_g, P_Z, P_A, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10604
      if ( res /= 10604) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4927
      if ( res /= 4927) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8906
      if ( res /= 8906) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_nu_tau, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7543
      if ( res /= 7543) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6038
      if ( res /= 6038) then
        stop
      end if
      call createParticleConfig(P_b, P_g, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2525
      if ( res /= 2525) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7295
      if ( res /= 7295) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6185
      if ( res /= 6185) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6887
      if ( res /= 6887) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5837
      if ( res /= 5837) then
        stop
      end if
      call createParticleConfig(P_ghG, P_g, P_ghGAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3931
      if ( res /= 3931) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_G0, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8086
      if ( res /= 8086) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10230
      if ( res /= 10230) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10124
      if ( res /= 10124) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",575
      if ( res /= 575) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5187
      if ( res /= 5187) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",270
      if ( res /= 270) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1907
      if ( res /= 1907) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1987
      if ( res /= 1987) then
        stop
      end if
      call createParticleConfig(P_g, P_Hh, P_Hl, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10535
      if ( res /= 10535) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3304
      if ( res /= 3304) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6727
      if ( res /= 6727) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4152
      if ( res /= 4152) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9600
      if ( res /= 9600) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10259
      if ( res /= 10259) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",253
      if ( res /= 253) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8565
      if ( res /= 8565) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9265
      if ( res /= 9265) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5537
      if ( res /= 5537) then
        stop
      end if
      call createParticleConfig(P_Hl, P_g, P_g, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10532
      if ( res /= 10532) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7020
      if ( res /= 7020) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9834
      if ( res /= 9834) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1127
      if ( res /= 1127) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1092
      if ( res /= 1092) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5344
      if ( res /= 5344) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_uAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2633
      if ( res /= 2633) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10744
      if ( res /= 10744) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5464
      if ( res /= 5464) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghA, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8160
      if ( res /= 8160) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1007
      if ( res /= 1007) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7786
      if ( res /= 7786) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2092
      if ( res /= 2092) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2956
      if ( res /= 2956) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",627
      if ( res /= 627) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9985
      if ( res /= 9985) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghAAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1832
      if ( res /= 1832) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1213
      if ( res /= 1213) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6602
      if ( res /= 6602) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWpAnti, P_WPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6638
      if ( res /= 6638) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9611
      if ( res /= 9611) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_g, P_GPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10687
      if ( res /= 10687) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8654
      if ( res /= 8654) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5113
      if ( res /= 5113) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",300
      if ( res /= 300) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6388
      if ( res /= 6388) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghA, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7642
      if ( res /= 7642) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",352
      if ( res /= 352) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWmAnti, P_ghWm, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5289
      if ( res /= 5289) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_tauMinus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2375
      if ( res /= 2375) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5335
      if ( res /= 5335) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZ, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2008
      if ( res /= 2008) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_t, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1306
      if ( res /= 1306) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2394
      if ( res /= 2394) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",671
      if ( res /= 671) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1564
      if ( res /= 1564) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6779
      if ( res /= 6779) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghA, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5284
      if ( res /= 5284) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1335
      if ( res /= 1335) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZ, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2144
      if ( res /= 2144) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1874
      if ( res /= 1874) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6925
      if ( res /= 6925) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5706
      if ( res /= 5706) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4525
      if ( res /= 4525) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",924
      if ( res /= 924) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",339
      if ( res /= 339) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3911
      if ( res /= 3911) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5241
      if ( res /= 5241) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1129
      if ( res /= 1129) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6323
      if ( res /= 6323) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1549
      if ( res /= 1549) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10710
      if ( res /= 10710) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GMinus, P_HPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3472
      if ( res /= 3472) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6742
      if ( res /= 6742) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2325
      if ( res /= 2325) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2670
      if ( res /= 2670) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6051
      if ( res /= 6051) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",937
      if ( res /= 937) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9469
      if ( res /= 9469) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3887
      if ( res /= 3887) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1680
      if ( res /= 1680) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5325
      if ( res /= 5325) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4908
      if ( res /= 4908) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1294
      if ( res /= 1294) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3712
      if ( res /= 3712) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6367
      if ( res /= 6367) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10441
      if ( res /= 10441) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6983
      if ( res /= 6983) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1882
      if ( res /= 1882) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2495
      if ( res /= 2495) then
        stop
      end if
      call createParticleConfig(P_g, P_ghGAnti, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4777
      if ( res /= 4777) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10634
      if ( res /= 10634) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",82
      if ( res /= 82) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7049
      if ( res /= 7049) then
        stop
      end if
      call createParticleConfig(P_Hh, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3440
      if ( res /= 3440) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4382
      if ( res /= 4382) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9354
      if ( res /= 9354) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2708
      if ( res /= 2708) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6603
      if ( res /= 6603) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4141
      if ( res /= 4141) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",142
      if ( res /= 142) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2856
      if ( res /= 2856) then
        stop
      end if
      call createParticleConfig(P_Hh, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8630
      if ( res /= 8630) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6657
      if ( res /= 6657) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1033
      if ( res /= 1033) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6725
      if ( res /= 6725) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_nu_e, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1671
      if ( res /= 1671) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",254
      if ( res /= 254) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",863
      if ( res /= 863) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1338
      if ( res /= 1338) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9141
      if ( res /= 9141) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZ, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2277
      if ( res /= 2277) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghAAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2102
      if ( res /= 2102) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1928
      if ( res /= 1928) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4423
      if ( res /= 4423) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",109
      if ( res /= 109) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3186
      if ( res /= 3186) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1032
      if ( res /= 1032) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghA, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",843
      if ( res /= 843) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10486
      if ( res /= 10486) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4194
      if ( res /= 4194) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghWp, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7739
      if ( res /= 7739) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5559
      if ( res /= 5559) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9513
      if ( res /= 9513) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWm, P_GPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6985
      if ( res /= 6985) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4740
      if ( res /= 4740) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6738
      if ( res /= 6738) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2188
      if ( res /= 2188) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1673
      if ( res /= 1673) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghA, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7591
      if ( res /= 7591) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3023
      if ( res /= 3023) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10412
      if ( res /= 10412) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6636
      if ( res /= 6636) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2862
      if ( res /= 2862) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6926
      if ( res /= 6926) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2259
      if ( res /= 2259) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5044
      if ( res /= 5044) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4075
      if ( res /= 4075) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4718
      if ( res /= 4718) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4573
      if ( res /= 4573) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1694
      if ( res /= 1694) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1427
      if ( res /= 1427) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5939
      if ( res /= 5939) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7833
      if ( res /= 7833) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4867
      if ( res /= 4867) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5158
      if ( res /= 5158) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3434
      if ( res /= 3434) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1149
      if ( res /= 1149) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1027
      if ( res /= 1027) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9711
      if ( res /= 9711) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HPlus, P_WMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",161
      if ( res /= 161) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7500
      if ( res /= 7500) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GPlus, P_GMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3248
      if ( res /= 3248) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4965
      if ( res /= 4965) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghA, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5283
      if ( res /= 5283) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8362
      if ( res /= 8362) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",711
      if ( res /= 711) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8216
      if ( res /= 8216) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9749
      if ( res /= 9749) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",291
      if ( res /= 291) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7410
      if ( res /= 7410) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6728
      if ( res /= 6728) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9050
      if ( res /= 9050) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4874
      if ( res /= 4874) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",596
      if ( res /= 596) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HMinus, P_WPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1822
      if ( res /= 1822) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1619
      if ( res /= 1619) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7673
      if ( res /= 7673) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9756
      if ( res /= 9756) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1151
      if ( res /= 1151) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10194
      if ( res /= 10194) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",381
      if ( res /= 381) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4150
      if ( res /= 4150) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8284
      if ( res /= 8284) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9978
      if ( res /= 9978) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8157
      if ( res /= 8157) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6250
      if ( res /= 6250) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4347
      if ( res /= 4347) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7247
      if ( res /= 7247) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",191
      if ( res /= 191) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10260
      if ( res /= 10260) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",990
      if ( res /= 990) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6182
      if ( res /= 6182) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1285
      if ( res /= 1285) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10212
      if ( res /= 10212) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2282
      if ( res /= 2282) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2153
      if ( res /= 2153) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4234
      if ( res /= 4234) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_g, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2772
      if ( res /= 2772) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3191
      if ( res /= 3191) then
        stop
      end if
      call createParticleConfig(P_A, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8119
      if ( res /= 8119) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghZ, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7570
      if ( res /= 7570) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7609
      if ( res /= 7609) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4161
      if ( res /= 4161) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9885
      if ( res /= 9885) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4044
      if ( res /= 4044) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWpAnti, P_WPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1669
      if ( res /= 1669) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7107
      if ( res /= 7107) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",775
      if ( res /= 775) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9579
      if ( res /= 9579) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10315
      if ( res /= 10315) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghWpAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",398
      if ( res /= 398) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8545
      if ( res /= 8545) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7845
      if ( res /= 7845) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1044
      if ( res /= 1044) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_nu_tauAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4923
      if ( res /= 4923) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9256
      if ( res /= 9256) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WPlus, P_WMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1705
      if ( res /= 1705) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_dAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5692
      if ( res /= 5692) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9490
      if ( res /= 9490) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9158
      if ( res /= 9158) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1601
      if ( res /= 1601) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6410
      if ( res /= 6410) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5069
      if ( res /= 5069) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9477
      if ( res /= 9477) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3652
      if ( res /= 3652) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8500
      if ( res /= 8500) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_muMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4530
      if ( res /= 4530) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7130
      if ( res /= 7130) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7815
      if ( res /= 7815) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7795
      if ( res /= 7795) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4795
      if ( res /= 4795) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2162
      if ( res /= 2162) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_nu_mu, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9989
      if ( res /= 9989) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1347
      if ( res /= 1347) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9070
      if ( res /= 9070) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1644
      if ( res /= 1644) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5315
      if ( res /= 5315) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10410
      if ( res /= 10410) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6520
      if ( res /= 6520) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3669
      if ( res /= 3669) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6530
      if ( res /= 6530) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9725
      if ( res /= 9725) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8065
      if ( res /= 8065) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8995
      if ( res /= 8995) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6948
      if ( res /= 6948) then
        stop
      end if
      call createParticleConfig(P_s, P_cAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5927
      if ( res /= 5927) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2848
      if ( res /= 2848) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9558
      if ( res /= 9558) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8422
      if ( res /= 8422) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7876
      if ( res /= 7876) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3698
      if ( res /= 3698) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_WPlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7522
      if ( res /= 7522) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8857
      if ( res /= 8857) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3541
      if ( res /= 3541) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4571
      if ( res /= 4571) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1720
      if ( res /= 1720) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2235
      if ( res /= 2235) then
        stop
      end if
      call createParticleConfig(P_A, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7351
      if ( res /= 7351) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7974
      if ( res /= 7974) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8629
      if ( res /= 8629) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5318
      if ( res /= 5318) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10767
      if ( res /= 10767) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1703
      if ( res /= 1703) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4279
      if ( res /= 4279) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7712
      if ( res /= 7712) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",280
      if ( res /= 280) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_ghG, P_g, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6167
      if ( res /= 6167) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3567
      if ( res /= 3567) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghZAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1005
      if ( res /= 1005) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9447
      if ( res /= 9447) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8339
      if ( res /= 8339) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghA, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2343
      if ( res /= 2343) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10733
      if ( res /= 10733) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",440
      if ( res /= 440) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9572
      if ( res /= 9572) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8703
      if ( res /= 8703) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6058
      if ( res /= 6058) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3808
      if ( res /= 3808) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2480
      if ( res /= 2480) then
        stop
      end if
      call createParticleConfig(P_Hh, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8021
      if ( res /= 8021) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2319
      if ( res /= 2319) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10203
      if ( res /= 10203) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1991
      if ( res /= 1991) then
        stop
      end if
      call createParticleConfig(P_t, P_Hl, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8766
      if ( res /= 8766) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6581
      if ( res /= 6581) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1775
      if ( res /= 1775) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1379
      if ( res /= 1379) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GMinus, P_GPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7858
      if ( res /= 7858) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7780
      if ( res /= 7780) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4913
      if ( res /= 4913) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2728
      if ( res /= 2728) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4282
      if ( res /= 4282) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7382
      if ( res /= 7382) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4048
      if ( res /= 4048) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",886
      if ( res /= 886) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7663
      if ( res /= 7663) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9975
      if ( res /= 9975) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9118
      if ( res /= 9118) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4868
      if ( res /= 4868) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9761
      if ( res /= 9761) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9307
      if ( res /= 9307) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7729
      if ( res /= 7729) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6907
      if ( res /= 6907) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7150
      if ( res /= 7150) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6591
      if ( res /= 6591) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6098
      if ( res /= 6098) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4350
      if ( res /= 4350) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4926
      if ( res /= 4926) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9467
      if ( res /= 9467) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8165
      if ( res /= 8165) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1217
      if ( res /= 1217) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWm, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3647
      if ( res /= 3647) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3291
      if ( res /= 3291) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7169
      if ( res /= 7169) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4728
      if ( res /= 4728) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3496
      if ( res /= 3496) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_nu_tau, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",194
      if ( res /= 194) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10076
      if ( res /= 10076) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZ, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7629
      if ( res /= 7629) then
        stop
      end if
      call createParticleConfig(P_Z, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7722
      if ( res /= 7722) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1311
      if ( res /= 1311) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7895
      if ( res /= 7895) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7929
      if ( res /= 7929) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GMinus, P_HPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6935
      if ( res /= 6935) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_GPlus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5452
      if ( res /= 5452) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8773
      if ( res /= 8773) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9659
      if ( res /= 9659) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6969
      if ( res /= 6969) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10465
      if ( res /= 10465) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_g, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2980
      if ( res /= 2980) then
        stop
      end if
      call createParticleConfig(P_Ha, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5165
      if ( res /= 5165) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2069
      if ( res /= 2069) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_nu_tauAnti, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4074
      if ( res /= 4074) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2150
      if ( res /= 2150) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3553
      if ( res /= 3553) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7278
      if ( res /= 7278) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1119
      if ( res /= 1119) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9938
      if ( res /= 9938) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7636
      if ( res /= 7636) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2241
      if ( res /= 2241) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6559
      if ( res /= 6559) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10129
      if ( res /= 10129) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1812
      if ( res /= 1812) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5342
      if ( res /= 5342) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HMinus, P_GPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7717
      if ( res /= 7717) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3105
      if ( res /= 3105) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_HMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5568
      if ( res /= 5568) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5236
      if ( res /= 5236) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3749
      if ( res /= 3749) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9813
      if ( res /= 9813) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",83
      if ( res /= 83) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghZAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6297
      if ( res /= 6297) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9866
      if ( res /= 9866) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6838
      if ( res /= 6838) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5686
      if ( res /= 5686) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6623
      if ( res /= 6623) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4080
      if ( res /= 4080) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HPlus, P_HMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7853
      if ( res /= 7853) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2610
      if ( res /= 2610) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6758
      if ( res /= 6758) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3821
      if ( res /= 3821) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4638
      if ( res /= 4638) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3131
      if ( res /= 3131) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_A, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5462
      if ( res /= 5462) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5415
      if ( res /= 5415) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7409
      if ( res /= 7409) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7450
      if ( res /= 7450) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8184
      if ( res /= 8184) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WMinus, P_GPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9504
      if ( res /= 9504) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8026
      if ( res /= 8026) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6904
      if ( res /= 6904) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HPlus, P_HMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5682
      if ( res /= 5682) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7540
      if ( res /= 7540) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7842
      if ( res /= 7842) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5723
      if ( res /= 5723) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5641
      if ( res /= 5641) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8240
      if ( res /= 8240) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9807
      if ( res /= 9807) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",224
      if ( res /= 224) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_A, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1366
      if ( res /= 1366) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1742
      if ( res /= 1742) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9862
      if ( res /= 9862) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7698
      if ( res /= 7698) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1133
      if ( res /= 1133) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",838
      if ( res /= 838) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2535
      if ( res /= 2535) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8916
      if ( res /= 8916) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7178
      if ( res /= 7178) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5240
      if ( res /= 5240) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8420
      if ( res /= 8420) then
        stop
      end if
      call createParticleConfig(P_AQ, P_G0, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1589
      if ( res /= 1589) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10494
      if ( res /= 10494) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",481
      if ( res /= 481) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9856
      if ( res /= 9856) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8742
      if ( res /= 8742) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghZ, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8786
      if ( res /= 8786) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWmAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7677
      if ( res /= 7677) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7615
      if ( res /= 7615) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2216
      if ( res /= 2216) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWpAnti, P_ghWp, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3942
      if ( res /= 3942) then
        stop
      end if
      call createParticleConfig(P_Z, P_nu_mu, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7486
      if ( res /= 7486) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7114
      if ( res /= 7114) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4853
      if ( res /= 4853) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7148
      if ( res /= 7148) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5545
      if ( res /= 5545) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8372
      if ( res /= 8372) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10609
      if ( res /= 10609) then
        stop
      end if
      call createParticleConfig(P_g, P_ghG, P_g, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2702
      if ( res /= 2702) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6152
      if ( res /= 6152) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",757
      if ( res /= 757) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7096
      if ( res /= 7096) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",952
      if ( res /= 952) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",102
      if ( res /= 102) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4102
      if ( res /= 4102) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8109
      if ( res /= 8109) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",486
      if ( res /= 486) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",758
      if ( res /= 758) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9520
      if ( res /= 9520) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9926
      if ( res /= 9926) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2915
      if ( res /= 2915) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",849
      if ( res /= 849) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10346
      if ( res /= 10346) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2135
      if ( res /= 2135) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1332
      if ( res /= 1332) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7460
      if ( res /= 7460) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3086
      if ( res /= 3086) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_A, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5628
      if ( res /= 5628) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4494
      if ( res /= 4494) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10515
      if ( res /= 10515) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3079
      if ( res /= 3079) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9934
      if ( res /= 9934) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6617
      if ( res /= 6617) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7784
      if ( res /= 7784) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4475
      if ( res /= 4475) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3880
      if ( res /= 3880) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6930
      if ( res /= 6930) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9610
      if ( res /= 9610) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2132
      if ( res /= 2132) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5731
      if ( res /= 5731) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1768
      if ( res /= 1768) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9474
      if ( res /= 9474) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8008
      if ( res /= 8008) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10406
      if ( res /= 10406) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5667
      if ( res /= 5667) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6618
      if ( res /= 6618) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6682
      if ( res /= 6682) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5522
      if ( res /= 5522) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7329
      if ( res /= 7329) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4148
      if ( res /= 4148) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6890
      if ( res /= 6890) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",698
      if ( res /= 698) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5169
      if ( res /= 5169) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8061
      if ( res /= 8061) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1391
      if ( res /= 1391) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6899
      if ( res /= 6899) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8713
      if ( res /= 8713) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2465
      if ( res /= 2465) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7763
      if ( res /= 7763) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5694
      if ( res /= 5694) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6120
      if ( res /= 6120) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6901
      if ( res /= 6901) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6524
      if ( res /= 6524) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7909
      if ( res /= 7909) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8439
      if ( res /= 8439) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4018
      if ( res /= 4018) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5469
      if ( res /= 5469) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9261
      if ( res /= 9261) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",527
      if ( res /= 527) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",800
      if ( res /= 800) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1441
      if ( res /= 1441) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10780
      if ( res /= 10780) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4024
      if ( res /= 4024) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",466
      if ( res /= 466) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5445
      if ( res /= 5445) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2243
      if ( res /= 2243) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2929
      if ( res /= 2929) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3174
      if ( res /= 3174) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5805
      if ( res /= 5805) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8826
      if ( res /= 8826) then
        stop
      end if
      call createParticleConfig(P_A, P_ghA, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",936
      if ( res /= 936) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5711
      if ( res /= 5711) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2078
      if ( res /= 2078) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghZAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8129
      if ( res /= 8129) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3207
      if ( res /= 3207) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8767
      if ( res /= 8767) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7699
      if ( res /= 7699) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2141
      if ( res /= 2141) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9423
      if ( res /= 9423) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4121
      if ( res /= 4121) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_GPlus, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1048
      if ( res /= 1048) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5245
      if ( res /= 5245) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2739
      if ( res /= 2739) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6010
      if ( res /= 6010) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6578
      if ( res /= 6578) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2990
      if ( res /= 2990) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10603
      if ( res /= 10603) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",192
      if ( res /= 192) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1658
      if ( res /= 1658) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3761
      if ( res /= 3761) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5094
      if ( res /= 5094) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4624
      if ( res /= 4624) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3592
      if ( res /= 3592) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_muMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1990
      if ( res /= 1990) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1718
      if ( res /= 1718) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZ, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3666
      if ( res /= 3666) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5471
      if ( res /= 5471) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8858
      if ( res /= 8858) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",938
      if ( res /= 938) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9416
      if ( res /= 9416) then
        stop
      end if
      call createParticleConfig(P_Z, P_nu_e, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2669
      if ( res /= 2669) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1856
      if ( res /= 1856) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghWpAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8720
      if ( res /= 8720) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6122
      if ( res /= 6122) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2119
      if ( res /= 2119) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5830
      if ( res /= 5830) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9720
      if ( res /= 9720) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6207
      if ( res /= 6207) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9079
      if ( res /= 9079) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5615
      if ( res /= 5615) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",54
      if ( res /= 54) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7215
      if ( res /= 7215) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4180
      if ( res /= 4180) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4012
      if ( res /= 4012) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8269
      if ( res /= 8269) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_nu_muAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5127
      if ( res /= 5127) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HPlus, P_WMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8417
      if ( res /= 8417) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8898
      if ( res /= 8898) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3198
      if ( res /= 3198) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",570
      if ( res /= 570) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GPlus, P_HMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10181
      if ( res /= 10181) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",413
      if ( res /= 413) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7764
      if ( res /= 7764) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8529
      if ( res /= 8529) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",681
      if ( res /= 681) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9863
      if ( res /= 9863) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5121
      if ( res /= 5121) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",565
      if ( res /= 565) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",176
      if ( res /= 176) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2701
      if ( res /= 2701) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_ghWm, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",810
      if ( res /= 810) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3606
      if ( res /= 3606) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_s, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8604
      if ( res /= 8604) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GMinus, P_ghWmAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3947
      if ( res /= 3947) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10215
      if ( res /= 10215) then
        stop
      end if
      call createParticleConfig(P_ghG, P_ghGAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8566
      if ( res /= 8566) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9341
      if ( res /= 9341) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",205
      if ( res /= 205) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",780
      if ( res /= 780) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HMinus, P_ghWp, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6784
      if ( res /= 6784) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6366
      if ( res /= 6366) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4675
      if ( res /= 4675) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3330
      if ( res /= 3330) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9135
      if ( res /= 9135) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1845
      if ( res /= 1845) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7704
      if ( res /= 7704) then
        stop
      end if
      call createParticleConfig(P_g, P_GPlus, P_g, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10513
      if ( res /= 10513) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8655
      if ( res /= 8655) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3021
      if ( res /= 3021) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10106
      if ( res /= 10106) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4438
      if ( res /= 4438) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10369
      if ( res /= 10369) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6661
      if ( res /= 6661) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3628
      if ( res /= 3628) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9186
      if ( res /= 9186) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5181
      if ( res /= 5181) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7733
      if ( res /= 7733) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_Hh, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8734
      if ( res /= 8734) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3598
      if ( res /= 3598) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2079
      if ( res /= 2079) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9528
      if ( res /= 9528) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7478
      if ( res /= 7478) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7070
      if ( res /= 7070) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2699
      if ( res /= 2699) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1461
      if ( res /= 1461) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1908
      if ( res /= 1908) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10048
      if ( res /= 10048) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4296
      if ( res /= 4296) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8331
      if ( res /= 8331) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4884
      if ( res /= 4884) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2618
      if ( res /= 2618) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9688
      if ( res /= 9688) then
        stop
      end if
      call createParticleConfig(P_s, P_WPlus, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5141
      if ( res /= 5141) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4189
      if ( res /= 4189) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2916
      if ( res /= 2916) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8275
      if ( res /= 8275) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5227
      if ( res /= 5227) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_WMinus, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1276
      if ( res /= 1276) then
        stop
      end if
      call createParticleConfig(P_Hh, P_g, P_g, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10555
      if ( res /= 10555) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9107
      if ( res /= 9107) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZ, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",260
      if ( res /= 260) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2120
      if ( res /= 2120) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5225
      if ( res /= 5225) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6519
      if ( res /= 6519) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2408
      if ( res /= 2408) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6674
      if ( res /= 6674) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10536
      if ( res /= 10536) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4848
      if ( res /= 4848) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8195
      if ( res /= 8195) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7800
      if ( res /= 7800) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9333
      if ( res /= 9333) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9999
      if ( res /= 9999) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",116
      if ( res /= 116) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1665
      if ( res /= 1665) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8369
      if ( res /= 8369) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10141
      if ( res /= 10141) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6305
      if ( res /= 6305) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2977
      if ( res /= 2977) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",686
      if ( res /= 686) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3344
      if ( res /= 3344) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9072
      if ( res /= 9072) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8996
      if ( res /= 8996) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WMinus, P_HPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3800
      if ( res /= 3800) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8456
      if ( res /= 8456) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1292
      if ( res /= 1292) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9541
      if ( res /= 9541) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3220
      if ( res /= 3220) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3136
      if ( res /= 3136) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7332
      if ( res /= 7332) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8523
      if ( res /= 8523) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7275
      if ( res /= 7275) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1855
      if ( res /= 1855) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",659
      if ( res /= 659) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3139
      if ( res /= 3139) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8329
      if ( res /= 8329) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1976
      if ( res /= 1976) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",871
      if ( res /= 871) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8709
      if ( res /= 8709) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3315
      if ( res /= 3315) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7603
      if ( res /= 7603) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3557
      if ( res /= 3557) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GMinus, P_GPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4009
      if ( res /= 4009) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7065
      if ( res /= 7065) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7154
      if ( res /= 7154) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10620
      if ( res /= 10620) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4371
      if ( res /= 4371) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HMinus, P_GPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4461
      if ( res /= 4461) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1176
      if ( res /= 1176) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7286
      if ( res /= 7286) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9642
      if ( res /= 9642) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3812
      if ( res /= 3812) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",477
      if ( res /= 477) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WMinus, P_WPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9920
      if ( res /= 9920) then
        stop
      end if
      call createParticleConfig(P_g, P_Z, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10528
      if ( res /= 10528) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4041
      if ( res /= 4041) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3050
      if ( res /= 3050) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10074
      if ( res /= 10074) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9617
      if ( res /= 9617) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5897
      if ( res /= 5897) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HPlus, P_HMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3638
      if ( res /= 3638) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_nu_e, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7390
      if ( res /= 7390) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10077
      if ( res /= 10077) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_cAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",435
      if ( res /= 435) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9289
      if ( res /= 9289) then
        stop
      end if
      call createParticleConfig(P_Hh, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2399
      if ( res /= 2399) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6256
      if ( res /= 6256) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2601
      if ( res /= 2601) then
        stop
      end if
      call createParticleConfig(P_A, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1683
      if ( res /= 1683) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_Z, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6871
      if ( res /= 6871) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2521
      if ( res /= 2521) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3507
      if ( res /= 3507) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8398
      if ( res /= 8398) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",106
      if ( res /= 106) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",665
      if ( res /= 665) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1915
      if ( res /= 1915) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8953
      if ( res /= 8953) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2561
      if ( res /= 2561) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8304
      if ( res /= 8304) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZ, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1587
      if ( res /= 1587) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7561
      if ( res /= 7561) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4285
      if ( res /= 4285) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",777
      if ( res /= 777) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4232
      if ( res /= 4232) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8264
      if ( res /= 8264) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10245
      if ( res /= 10245) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9411
      if ( res /= 9411) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghWp, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6529
      if ( res /= 6529) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",515
      if ( res /= 515) then
        stop
      end if
      call createParticleConfig(P_g, P_GPlus, P_GMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10685
      if ( res /= 10685) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_s, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6452
      if ( res /= 6452) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8776
      if ( res /= 8776) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3900
      if ( res /= 3900) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10162
      if ( res /= 10162) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9802
      if ( res /= 9802) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4101
      if ( res /= 4101) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1766
      if ( res /= 1766) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6075
      if ( res /= 6075) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghA, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8391
      if ( res /= 8391) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6467
      if ( res /= 6467) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HMinus, P_HPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8360
      if ( res /= 8360) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2533
      if ( res /= 2533) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5987
      if ( res /= 5987) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4302
      if ( res /= 4302) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9590
      if ( res /= 9590) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8834
      if ( res /= 8834) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7688
      if ( res /= 7688) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10464
      if ( res /= 10464) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghZAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",618
      if ( res /= 618) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5961
      if ( res /= 5961) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9169
      if ( res /= 9169) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1762
      if ( res /= 1762) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1881
      if ( res /= 1881) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWpAnti, P_GPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9953
      if ( res /= 9953) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5735
      if ( res /= 5735) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9073
      if ( res /= 9073) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10545
      if ( res /= 10545) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8650
      if ( res /= 8650) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1439
      if ( res /= 1439) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7953
      if ( res /= 7953) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8112
      if ( res /= 8112) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9117
      if ( res /= 9117) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7753
      if ( res /= 7753) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6687
      if ( res /= 6687) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8374
      if ( res /= 8374) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10498
      if ( res /= 10498) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5492
      if ( res /= 5492) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_nu_e, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7905
      if ( res /= 7905) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9445
      if ( res /= 9445) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8509
      if ( res /= 8509) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",252
      if ( res /= 252) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4934
      if ( res /= 4934) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",794
      if ( res /= 794) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_u, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",238
      if ( res /= 238) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4956
      if ( res /= 4956) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3966
      if ( res /= 3966) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",569
      if ( res /= 569) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1551
      if ( res /= 1551) then
        stop
      end if
      call createParticleConfig(P_Hl, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4894
      if ( res /= 4894) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8433
      if ( res /= 8433) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3396
      if ( res /= 3396) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWmAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4158
      if ( res /= 4158) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5448
      if ( res /= 5448) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2663
      if ( res /= 2663) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1755
      if ( res /= 1755) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",235
      if ( res /= 235) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9703
      if ( res /= 9703) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6403
      if ( res /= 6403) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6757
      if ( res /= 6757) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3511
      if ( res /= 3511) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9387
      if ( res /= 9387) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8591
      if ( res /= 8591) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7997
      if ( res /= 7997) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4809
      if ( res /= 4809) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8929
      if ( res /= 8929) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1480
      if ( res /= 1480) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1923
      if ( res /= 1923) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3271
      if ( res /= 3271) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6273
      if ( res /= 6273) then
        stop
      end if
      call createParticleConfig(P_Z, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8272
      if ( res /= 8272) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5207
      if ( res /= 5207) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4218
      if ( res /= 4218) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7121
      if ( res /= 7121) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1729
      if ( res /= 1729) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9511
      if ( res /= 9511) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4789
      if ( res /= 4789) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2814
      if ( res /= 2814) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2540
      if ( res /= 2540) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4388
      if ( res /= 4388) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5214
      if ( res /= 5214) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghZ, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3684
      if ( res /= 3684) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8134
      if ( res /= 8134) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4359
      if ( res /= 4359) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10752
      if ( res /= 10752) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6572
      if ( res /= 6572) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2175
      if ( res /= 2175) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",573
      if ( res /= 573) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7811
      if ( res /= 7811) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3539
      if ( res /= 3539) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",742
      if ( res /= 742) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10005
      if ( res /= 10005) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1496
      if ( res /= 1496) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5013
      if ( res /= 5013) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_g, P_ghG, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4428
      if ( res /= 4428) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7379
      if ( res /= 7379) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1929
      if ( res /= 1929) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_nu_tauAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7935
      if ( res /= 7935) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1305
      if ( res /= 1305) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6288
      if ( res /= 6288) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_cAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7744
      if ( res /= 7744) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10565
      if ( res /= 10565) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6013
      if ( res /= 6013) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7888
      if ( res /= 7888) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9976
      if ( res /= 9976) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2234
      if ( res /= 2234) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5106
      if ( res /= 5106) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",762
      if ( res /= 762) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7162
      if ( res /= 7162) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2401
      if ( res /= 2401) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9779
      if ( res /= 9779) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7422
      if ( res /= 7422) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9162
      if ( res /= 9162) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10376
      if ( res /= 10376) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2941
      if ( res /= 2941) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3384
      if ( res /= 3384) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9006
      if ( res /= 9006) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",138
      if ( res /= 138) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5880
      if ( res /= 5880) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_nu_mu, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6485
      if ( res /= 6485) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4981
      if ( res /= 4981) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2847
      if ( res /= 2847) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6309
      if ( res /= 6309) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7248
      if ( res /= 7248) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1850
      if ( res /= 1850) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3923
      if ( res /= 3923) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4093
      if ( res /= 4093) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7016
      if ( res /= 7016) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2593
      if ( res /= 2593) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2877
      if ( res /= 2877) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6863
      if ( res /= 6863) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4518
      if ( res /= 4518) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9058
      if ( res /= 9058) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghWmAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4550
      if ( res /= 4550) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4164
      if ( res /= 4164) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1112
      if ( res /= 1112) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9984
      if ( res /= 9984) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2422
      if ( res /= 2422) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6206
      if ( res /= 6206) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GMinus, P_GPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2792
      if ( res /= 2792) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5569
      if ( res /= 5569) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",479
      if ( res /= 479) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8702
      if ( res /= 8702) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9635
      if ( res /= 9635) then
        stop
      end if
      call createParticleConfig(P_Hh, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10487
      if ( res /= 10487) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1446
      if ( res /= 1446) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9723
      if ( res /= 9723) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4766
      if ( res /= 4766) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10294
      if ( res /= 10294) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10156
      if ( res /= 10156) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9656
      if ( res /= 9656) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7655
      if ( res /= 7655) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6407
      if ( res /= 6407) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2227
      if ( res /= 2227) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5919
      if ( res /= 5919) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",852
      if ( res /= 852) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghWp, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2327
      if ( res /= 2327) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8397
      if ( res /= 8397) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3083
      if ( res /= 3083) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4955
      if ( res /= 4955) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8852
      if ( res /= 8852) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10198
      if ( res /= 10198) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6517
      if ( res /= 6517) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2289
      if ( res /= 2289) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9291
      if ( res /= 9291) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1036
      if ( res /= 1036) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",468
      if ( res /= 468) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",218
      if ( res /= 218) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1490
      if ( res /= 1490) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1029
      if ( res /= 1029) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",406
      if ( res /= 406) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4951
      if ( res /= 4951) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4876
      if ( res /= 4876) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1897
      if ( res /= 1897) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6967
      if ( res /= 6967) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9108
      if ( res /= 9108) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",753
      if ( res /= 753) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10311
      if ( res /= 10311) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4465
      if ( res /= 4465) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6767
      if ( res /= 6767) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7069
      if ( res /= 7069) then
        stop
      end if
      call createParticleConfig(P_A, P_nu_mu, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9790
      if ( res /= 9790) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9822
      if ( res /= 9822) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10079
      if ( res /= 10079) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",378
      if ( res /= 378) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2961
      if ( res /= 2961) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3876
      if ( res /= 3876) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7761
      if ( res /= 7761) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6894
      if ( res /= 6894) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9734
      if ( res /= 9734) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1623
      if ( res /= 1623) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9529
      if ( res /= 9529) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghAAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",520
      if ( res /= 520) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",738
      if ( res /= 738) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8735
      if ( res /= 8735) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10662
      if ( res /= 10662) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6514
      if ( res /= 6514) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1340
      if ( res /= 1340) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2006
      if ( res /= 2006) then
        stop
      end if
      call createParticleConfig(P_d, P_WPlus, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1158
      if ( res /= 1158) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10582
      if ( res /= 10582) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7510
      if ( res /= 7510) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1594
      if ( res /= 1594) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_Hh, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1089
      if ( res /= 1089) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10090
      if ( res /= 10090) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",554
      if ( res /= 554) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4312
      if ( res /= 4312) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5587
      if ( res /= 5587) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1071
      if ( res /= 1071) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4796
      if ( res /= 4796) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9948
      if ( res /= 9948) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",850
      if ( res /= 850) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9270
      if ( res /= 9270) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9809
      if ( res /= 9809) then
        stop
      end if
      call createParticleConfig(P_G0, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",221
      if ( res /= 221) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8598
      if ( res /= 8598) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10403
      if ( res /= 10403) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5357
      if ( res /= 5357) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2647
      if ( res /= 2647) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10165
      if ( res /= 10165) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",521
      if ( res /= 521) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9293
      if ( res /= 9293) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2619
      if ( res /= 2619) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6404
      if ( res /= 6404) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3816
      if ( res /= 3816) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Z, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3762
      if ( res /= 3762) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10552
      if ( res /= 10552) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8315
      if ( res /= 8315) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2021
      if ( res /= 2021) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9280
      if ( res /= 9280) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5147
      if ( res /= 5147) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6190
      if ( res /= 6190) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8603
      if ( res /= 8603) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8868
      if ( res /= 8868) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZ, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5324
      if ( res /= 5324) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZ, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2307
      if ( res /= 2307) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7187
      if ( res /= 7187) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10753
      if ( res /= 10753) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9840
      if ( res /= 9840) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7503
      if ( res /= 7503) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3913
      if ( res /= 3913) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",992
      if ( res /= 992) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5800
      if ( res /= 5800) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1189
      if ( res /= 1189) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5145
      if ( res /= 5145) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1385
      if ( res /= 1385) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6940
      if ( res /= 6940) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3906
      if ( res /= 3906) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8290
      if ( res /= 8290) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3687
      if ( res /= 3687) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_g, P_g, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10680
      if ( res /= 10680) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5341
      if ( res /= 5341) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8938
      if ( res /= 8938) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10104
      if ( res /= 10104) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1736
      if ( res /= 1736) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5832
      if ( res /= 5832) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8105
      if ( res /= 8105) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9763
      if ( res /= 9763) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9153
      if ( res /= 9153) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6861
      if ( res /= 6861) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2110
      if ( res /= 2110) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",837
      if ( res /= 837) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3441
      if ( res /= 3441) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4504
      if ( res /= 4504) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_WPlus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6302
      if ( res /= 6302) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_muPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4663
      if ( res /= 4663) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5131
      if ( res /= 5131) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10671
      if ( res /= 10671) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_nu_eAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",225
      if ( res /= 225) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6241
      if ( res /= 6241) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5146
      if ( res /= 5146) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2900
      if ( res /= 2900) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7373
      if ( res /= 7373) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4662
      if ( res /= 4662) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6582
      if ( res /= 6582) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5385
      if ( res /= 5385) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9224
      if ( res /= 9224) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4383
      if ( res /= 4383) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5526
      if ( res /= 5526) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8039
      if ( res /= 8039) then
        stop
      end if
      call createParticleConfig(P_Z, P_nu_tau, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1203
      if ( res /= 1203) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2751
      if ( res /= 2751) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5062
      if ( res /= 5062) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7126
      if ( res /= 7126) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4858
      if ( res /= 4858) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2160
      if ( res /= 2160) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_nu_tauAnti, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4244
      if ( res /= 4244) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1719
      if ( res /= 1719) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8370
      if ( res /= 8370) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4072
      if ( res /= 4072) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8875
      if ( res /= 8875) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3487
      if ( res /= 3487) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghWmAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7388
      if ( res /= 7388) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10015
      if ( res /= 10015) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4555
      if ( res /= 4555) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1624
      if ( res /= 1624) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9481
      if ( res /= 9481) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3570
      if ( res /= 3570) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6502
      if ( res /= 6502) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2854
      if ( res /= 2854) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6908
      if ( res /= 6908) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1660
      if ( res /= 1660) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7145
      if ( res /= 7145) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",621
      if ( res /= 621) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1704
      if ( res /= 1704) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8673
      if ( res /= 8673) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9262
      if ( res /= 9262) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4814
      if ( res /= 4814) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10299
      if ( res /= 10299) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2258
      if ( res /= 2258) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2139
      if ( res /= 2139) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3287
      if ( res /= 3287) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8815
      if ( res /= 8815) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1209
      if ( res /= 1209) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghZAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5014
      if ( res /= 5014) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9982
      if ( res /= 9982) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5557
      if ( res /= 5557) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_dAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1020
      if ( res /= 1020) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8628
      if ( res /= 8628) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7504
      if ( res /= 7504) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5928
      if ( res /= 5928) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10086
      if ( res /= 10086) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1779
      if ( res /= 1779) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",684
      if ( res /= 684) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10002
      if ( res /= 10002) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4733
      if ( res /= 4733) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3581
      if ( res /= 3581) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2109
      if ( res /= 2109) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8212
      if ( res /= 8212) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_b, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4993
      if ( res /= 4993) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWp, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5535
      if ( res /= 5535) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWpAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5640
      if ( res /= 5640) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7057
      if ( res /= 7057) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_sAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7462
      if ( res /= 7462) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9597
      if ( res /= 9597) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10029
      if ( res /= 10029) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10482
      if ( res /= 10482) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10327
      if ( res /= 10327) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10777
      if ( res /= 10777) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1128
      if ( res /= 1128) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3297
      if ( res /= 3297) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WMinus, P_HPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10442
      if ( res /= 10442) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_A, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10261
      if ( res /= 10261) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4755
      if ( res /= 4755) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4409
      if ( res /= 4409) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9988
      if ( res /= 9988) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9187
      if ( res /= 9187) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2844
      if ( res /= 2844) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4745
      if ( res /= 4745) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6774
      if ( res /= 6774) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6878
      if ( res /= 6878) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7934
      if ( res /= 7934) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWp, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2949
      if ( res /= 2949) then
        stop
      end if
      call createParticleConfig(P_Z, P_g, P_g, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10719
      if ( res /= 10719) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZ, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4378
      if ( res /= 4378) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1806
      if ( res /= 1806) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_g, P_HMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10602
      if ( res /= 10602) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4319
      if ( res /= 4319) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9052
      if ( res /= 9052) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5576
      if ( res /= 5576) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8007
      if ( res /= 8007) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",767
      if ( res /= 767) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1698
      if ( res /= 1698) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2954
      if ( res /= 2954) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7435
      if ( res /= 7435) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6541
      if ( res /= 6541) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZ, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",148
      if ( res /= 148) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2800
      if ( res /= 2800) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5793
      if ( res /= 5793) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7094
      if ( res /= 7094) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3751
      if ( res /= 3751) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",534
      if ( res /= 534) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10060
      if ( res /= 10060) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7138
      if ( res /= 7138) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2909
      if ( res /= 2909) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4971
      if ( res /= 4971) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9045
      if ( res /= 9045) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10184
      if ( res /= 10184) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HMinus, P_HPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6639
      if ( res /= 6639) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_HPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7011
      if ( res /= 7011) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1632
      if ( res /= 1632) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3824
      if ( res /= 3824) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1847
      if ( res /= 1847) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghAAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8653
      if ( res /= 8653) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",147
      if ( res /= 147) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9929
      if ( res /= 9929) then
        stop
      end if
      call createParticleConfig(P_gQ, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5280
      if ( res /= 5280) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2191
      if ( res /= 2191) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8156
      if ( res /= 8156) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghZ, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3252
      if ( res /= 3252) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_nu_mu, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1199
      if ( res /= 1199) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4437
      if ( res /= 4437) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",591
      if ( res /= 591) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",619
      if ( res /= 619) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",90
      if ( res /= 90) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6701
      if ( res /= 6701) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",101
      if ( res /= 101) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1299
      if ( res /= 1299) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7968
      if ( res /= 7968) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5998
      if ( res /= 5998) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7703
      if ( res /= 7703) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2458
      if ( res /= 2458) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",669
      if ( res /= 669) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",694
      if ( res /= 694) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7797
      if ( res /= 7797) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_muPlus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1544
      if ( res /= 1544) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1946
      if ( res /= 1946) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9164
      if ( res /= 9164) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1108
      if ( res /= 1108) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8939
      if ( res /= 8939) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8033
      if ( res /= 8033) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8409
      if ( res /= 8409) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",994
      if ( res /= 994) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3452
      if ( res /= 3452) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_u, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5785
      if ( res /= 5785) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5668
      if ( res /= 5668) then
        stop
      end if
      call createParticleConfig(P_g, P_Ha, P_g, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10759
      if ( res /= 10759) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5684
      if ( res /= 5684) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6626
      if ( res /= 6626) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6165
      if ( res /= 6165) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5912
      if ( res /= 5912) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3864
      if ( res /= 3864) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3949
      if ( res /= 3949) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2781
      if ( res /= 2781) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8101
      if ( res /= 8101) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2115
      if ( res /= 2115) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6851
      if ( res /= 6851) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4625
      if ( res /= 4625) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3032
      if ( res /= 3032) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2692
      if ( res /= 2692) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4545
      if ( res /= 4545) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6708
      if ( res /= 6708) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7931
      if ( res /= 7931) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5626
      if ( res /= 5626) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7754
      if ( res /= 7754) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1106
      if ( res /= 1106) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3294
      if ( res /= 3294) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_A, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3256
      if ( res /= 3256) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10428
      if ( res /= 10428) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3725
      if ( res /= 3725) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6342
      if ( res /= 6342) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8758
      if ( res /= 8758) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5609
      if ( res /= 5609) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5473
      if ( res /= 5473) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9696
      if ( res /= 9696) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",394
      if ( res /= 394) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4468
      if ( res /= 4468) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10390
      if ( res /= 10390) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9792
      if ( res /= 9792) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3221
      if ( res /= 3221) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9276
      if ( res /= 9276) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3896
      if ( res /= 3896) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4499
      if ( res /= 4499) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",572
      if ( res /= 572) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7508
      if ( res /= 7508) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4253
      if ( res /= 4253) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4726
      if ( res /= 4726) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4160
      if ( res /= 4160) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8434
      if ( res /= 8434) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2440
      if ( res /= 2440) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7045
      if ( res /= 7045) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_c, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2300
      if ( res /= 2300) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2967
      if ( res /= 2967) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3763
      if ( res /= 3763) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10232
      if ( res /= 10232) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1009
      if ( res /= 1009) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5743
      if ( res /= 5743) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6540
      if ( res /= 6540) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",663
      if ( res /= 663) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_A, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3345
      if ( res /= 3345) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghA, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3729
      if ( res /= 3729) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7849
      if ( res /= 7849) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6301
      if ( res /= 6301) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9091
      if ( res /= 9091) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8461
      if ( res /= 8461) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10167
      if ( res /= 10167) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7106
      if ( res /= 7106) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1859
      if ( res /= 1859) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1465
      if ( res /= 1465) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6872
      if ( res /= 6872) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6401
      if ( res /= 6401) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2907
      if ( res /= 2907) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8448
      if ( res /= 8448) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",929
      if ( res /= 929) then
        stop
      end if
      call createParticleConfig(P_d, P_uAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8676
      if ( res /= 8676) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9738
      if ( res /= 9738) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_WMinus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6002
      if ( res /= 6002) then
        stop
      end if
      call createParticleConfig(P_b, P_tAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7930
      if ( res /= 7930) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",45
      if ( res /= 45) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5088
      if ( res /= 5088) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9877
      if ( res /= 9877) then
        stop
      end if
      call createParticleConfig(P_u, P_HMinus, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3478
      if ( res /= 3478) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3848
      if ( res /= 3848) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7667
      if ( res /= 7667) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5779
      if ( res /= 5779) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3308
      if ( res /= 3308) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3272
      if ( res /= 3272) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",693
      if ( res /= 693) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",303
      if ( res /= 303) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7139
      if ( res /= 7139) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",640
      if ( res /= 640) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8120
      if ( res /= 8120) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1171
      if ( res /= 1171) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10714
      if ( res /= 10714) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_cAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1458
      if ( res /= 1458) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1390
      if ( res /= 1390) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_eMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2072
      if ( res /= 2072) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9636
      if ( res /= 9636) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1716
      if ( res /= 1716) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4489
      if ( res /= 4489) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9791
      if ( res /= 9791) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",689
      if ( res /= 689) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8738
      if ( res /= 8738) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghAAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5619
      if ( res /= 5619) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10704
      if ( res /= 10704) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4216
      if ( res /= 4216) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6771
      if ( res /= 6771) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5396
      if ( res /= 5396) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6139
      if ( res /= 6139) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7458
      if ( res /= 7458) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",597
      if ( res /= 597) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5386
      if ( res /= 5386) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10474
      if ( res /= 10474) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6099
      if ( res /= 6099) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6198
      if ( res /= 6198) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7961
      if ( res /= 7961) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9220
      if ( res /= 9220) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3879
      if ( res /= 3879) then
        stop
      end if
      call createParticleConfig(P_gQ, P_ghG, P_g, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1196
      if ( res /= 1196) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6179
      if ( res /= 6179) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2189
      if ( res /= 2189) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9014
      if ( res /= 9014) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7469
      if ( res /= 7469) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6037
      if ( res /= 6037) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6104
      if ( res /= 6104) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4713
      if ( res /= 4713) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9888
      if ( res /= 9888) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1930
      if ( res /= 1930) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_nu_tau, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5527
      if ( res /= 5527) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9029
      if ( res /= 9029) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_eMinus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6392
      if ( res /= 6392) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6418
      if ( res /= 6418) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5588
      if ( res /= 5588) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1102
      if ( res /= 1102) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9458
      if ( res /= 9458) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",113
      if ( res /= 113) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5960
      if ( res /= 5960) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",47
      if ( res /= 47) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghWp, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4395
      if ( res /= 4395) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4618
      if ( res /= 4618) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2646
      if ( res /= 2646) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HPlus, P_ghWm, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5540
      if ( res /= 5540) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8838
      if ( res /= 8838) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6035
      if ( res /= 6035) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5246
      if ( res /= 5246) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4496
      if ( res /= 4496) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",412
      if ( res /= 412) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5712
      if ( res /= 5712) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_nu_muAnti, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1072
      if ( res /= 1072) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9867
      if ( res /= 9867) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8202
      if ( res /= 8202) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",376
      if ( res /= 376) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4414
      if ( res /= 4414) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1243
      if ( res /= 1243) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1708
      if ( res /= 1708) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4722
      if ( res /= 4722) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",250
      if ( res /= 250) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2765
      if ( res /= 2765) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2769
      if ( res /= 2769) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6596
      if ( res /= 6596) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8785
      if ( res /= 8785) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6137
      if ( res /= 6137) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5889
      if ( res /= 5889) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4338
      if ( res /= 4338) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2330
      if ( res /= 2330) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",456
      if ( res /= 456) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8914
      if ( res /= 8914) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6764
      if ( res /= 6764) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3482
      if ( res /= 3482) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2198
      if ( res /= 2198) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3716
      if ( res /= 3716) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9248
      if ( res /= 9248) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8452
      if ( res /= 8452) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4213
      if ( res /= 4213) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1699
      if ( res /= 1699) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8883
      if ( res /= 8883) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2406
      if ( res /= 2406) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_g, P_HPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10495
      if ( res /= 10495) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6665
      if ( res /= 6665) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7595
      if ( res /= 7595) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9094
      if ( res /= 9094) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5202
      if ( res /= 5202) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1109
      if ( res /= 1109) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6873
      if ( res /= 6873) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GMinus, P_HPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4626
      if ( res /= 4626) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9332
      if ( res /= 9332) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5142
      if ( res /= 5142) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWm, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5748
      if ( res /= 5748) then
        stop
      end if
      call createParticleConfig(P_b, P_tAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3702
      if ( res /= 3702) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2598
      if ( res /= 2598) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6592
      if ( res /= 6592) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8139
      if ( res /= 8139) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4719
      if ( res /= 4719) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5461
      if ( res /= 5461) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7728
      if ( res /= 7728) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5444
      if ( res /= 5444) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9757
      if ( res /= 9757) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5282
      if ( res /= 5282) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5053
      if ( res /= 5053) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2932
      if ( res /= 2932) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5606
      if ( res /= 5606) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9398
      if ( res /= 9398) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1737
      if ( res /= 1737) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8032
      if ( res /= 8032) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2146
      if ( res /= 2146) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3013
      if ( res /= 3013) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9378
      if ( res /= 9378) then
        stop
      end if
      call createParticleConfig(P_g, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6395
      if ( res /= 6395) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4439
      if ( res /= 4439) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghAAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6921
      if ( res /= 6921) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10285
      if ( res /= 10285) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2204
      if ( res /= 2204) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6690
      if ( res /= 6690) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4331
      if ( res /= 4331) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_muPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4966
      if ( res /= 4966) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",345
      if ( res /= 345) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8749
      if ( res /= 8749) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9710
      if ( res /= 9710) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4540
      if ( res /= 4540) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5776
      if ( res /= 5776) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1392
      if ( res /= 1392) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4769
      if ( res /= 4769) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7037
      if ( res /= 7037) then
        stop
      end if
      call createParticleConfig(P_A, P_nu_tauAnti, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10105
      if ( res /= 10105) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2318
      if ( res /= 2318) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10020
      if ( res /= 10020) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9669
      if ( res /= 9669) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7616
      if ( res /= 7616) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2197
      if ( res /= 2197) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWpAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3378
      if ( res /= 3378) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4708
      if ( res /= 4708) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2379
      if ( res /= 2379) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10563
      if ( res /= 10563) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8862
      if ( res /= 8862) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghA, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8609
      if ( res /= 8609) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9689
      if ( res /= 9689) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6867
      if ( res /= 6867) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_A, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2688
      if ( res /= 2688) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",321
      if ( res /= 321) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8543
      if ( res /= 8543) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2550
      if ( res /= 2550) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1466
      if ( res /= 1466) then
        stop
      end if
      call createParticleConfig(P_Z, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2568
      if ( res /= 2568) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10372
      if ( res /= 10372) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",786
      if ( res /= 786) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2919
      if ( res /= 2919) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",290
      if ( res /= 290) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10021
      if ( res /= 10021) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3129
      if ( res /= 3129) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1629
      if ( res /= 1629) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1626
      if ( res /= 1626) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9970
      if ( res /= 9970) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2093
      if ( res /= 2093) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4078
      if ( res /= 4078) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7806
      if ( res /= 7806) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1639
      if ( res /= 1639) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8572
      if ( res /= 8572) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",25
      if ( res /= 25) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Z, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7085
      if ( res /= 7085) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9311
      if ( res /= 9311) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5063
      if ( res /= 5063) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9482
      if ( res /= 9482) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1867
      if ( res /= 1867) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6173
      if ( res /= 6173) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3954
      if ( res /= 3954) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5869
      if ( res /= 5869) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6777
      if ( res /= 6777) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",288
      if ( res /= 288) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1838
      if ( res /= 1838) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5026
      if ( res /= 5026) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2608
      if ( res /= 2608) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5650
      if ( res /= 5650) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8080
      if ( res /= 8080) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10017
      if ( res /= 10017) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",942
      if ( res /= 942) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5005
      if ( res /= 5005) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4316
      if ( res /= 4316) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_b, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1262
      if ( res /= 1262) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6772
      if ( res /= 6772) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7340
      if ( res /= 7340) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6133
      if ( res /= 6133) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8227
      if ( res /= 8227) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghAAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1381
      if ( res /= 1381) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2145
      if ( res /= 2145) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",622
      if ( res /= 622) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5198
      if ( res /= 5198) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8886
      if ( res /= 8886) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghWm, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8336
      if ( res /= 8336) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Z, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1952
      if ( res /= 1952) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8469
      if ( res /= 8469) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8231
      if ( res /= 8231) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10270
      if ( res /= 10270) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",88
      if ( res /= 88) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4250
      if ( res /= 4250) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8726
      if ( res /= 8726) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8171
      if ( res /= 8171) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3212
      if ( res /= 3212) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",196
      if ( res /= 196) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghZ, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5560
      if ( res /= 5560) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5965
      if ( res /= 5965) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3690
      if ( res /= 3690) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3858
      if ( res /= 3858) then
        stop
      end if
      call createParticleConfig(P_u, P_WMinus, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1637
      if ( res /= 1637) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3138
      if ( res /= 3138) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWm, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5109
      if ( res /= 5109) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1330
      if ( res /= 1330) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",51
      if ( res /= 51) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1425
      if ( res /= 1425) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3976
      if ( res /= 3976) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghWp, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6313
      if ( res /= 6313) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWm, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4762
      if ( res /= 4762) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_HMinus, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6304
      if ( res /= 6304) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_Z, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4142
      if ( res /= 4142) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6660
      if ( res /= 6660) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",764
      if ( res /= 764) then
        stop
      end if
      call createParticleConfig(P_A, P_ghA, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",960
      if ( res /= 960) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7710
      if ( res /= 7710) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5762
      if ( res /= 5762) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7282
      if ( res /= 7282) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1989
      if ( res /= 1989) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghA, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3126
      if ( res /= 3126) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Z, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6472
      if ( res /= 6472) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3219
      if ( res /= 3219) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7864
      if ( res /= 7864) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8421
      if ( res /= 8421) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3008
      if ( res /= 3008) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10329
      if ( res /= 10329) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3549
      if ( res /= 3549) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2058
      if ( res /= 2058) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_nu_mu, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7068
      if ( res /= 7068) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9471
      if ( res /= 9471) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6368
      if ( res /= 6368) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3205
      if ( res /= 3205) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7200
      if ( res /= 7200) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9339
      if ( res /= 9339) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghA, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7614
      if ( res /= 7614) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10163
      if ( res /= 10163) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3020
      if ( res /= 3020) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5683
      if ( res /= 5683) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2986
      if ( res /= 2986) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7054
      if ( res /= 7054) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8616
      if ( res /= 8616) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8048
      if ( res /= 8048) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7949
      if ( res /= 7949) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",227
      if ( res /= 227) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1230
      if ( res /= 1230) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2367
      if ( res /= 2367) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6588
      if ( res /= 6588) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1656
      if ( res /= 1656) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghA, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2081
      if ( res /= 2081) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8583
      if ( res /= 8583) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6898
      if ( res /= 6898) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6106
      if ( res /= 6106) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6145
      if ( res /= 6145) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8260
      if ( res /= 8260) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9714
      if ( res /= 9714) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1749
      if ( res /= 1749) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7441
      if ( res /= 7441) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5414
      if ( res /= 5414) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7408
      if ( res /= 7408) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6240
      if ( res /= 6240) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5829
      if ( res /= 5829) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9420
      if ( res /= 9420) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_HMinus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5430
      if ( res /= 5430) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4258
      if ( res /= 4258) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1986
      if ( res /= 1986) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4687
      if ( res /= 4687) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1269
      if ( res /= 1269) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3779
      if ( res /= 3779) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2071
      if ( res /= 2071) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3149
      if ( res /= 3149) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1574
      if ( res /= 1574) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8722
      if ( res /= 8722) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5986
      if ( res /= 5986) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6952
      if ( res /= 6952) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_G0, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7477
      if ( res /= 7477) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1748
      if ( res /= 1748) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1595
      if ( res /= 1595) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",304
      if ( res /= 304) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5427
      if ( res /= 5427) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6462
      if ( res /= 6462) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8684
      if ( res /= 8684) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghA, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2910
      if ( res /= 2910) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6773
      if ( res /= 6773) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7095
      if ( res /= 7095) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7534
      if ( res /= 7534) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4833
      if ( res /= 4833) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2879
      if ( res /= 2879) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3686
      if ( res /= 3686) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2591
      if ( res /= 2591) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10485
      if ( res /= 10485) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10725
      if ( res /= 10725) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9480
      if ( res /= 9480) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3347
      if ( res /= 3347) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9633
      if ( res /= 9633) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9238
      if ( res /= 9238) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9391
      if ( res /= 9391) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1969
      if ( res /= 1969) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1367
      if ( res /= 1367) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9127
      if ( res /= 9127) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6734
      if ( res /= 6734) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8155
      if ( res /= 8155) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10134
      if ( res /= 10134) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_tauPlus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7292
      if ( res /= 7292) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",329
      if ( res /= 329) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10413
      if ( res /= 10413) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4339
      if ( res /= 4339) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1046
      if ( res /= 1046) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WPlus, P_WMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9143
      if ( res /= 9143) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5764
      if ( res /= 5764) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1501
      if ( res /= 1501) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5781
      if ( res /= 5781) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9608
      if ( res /= 9608) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9305
      if ( res /= 9305) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5436
      if ( res /= 5436) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",907
      if ( res /= 907) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9951
      if ( res /= 9951) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6799
      if ( res /= 6799) then
        stop
      end if
      call createParticleConfig(P_Ha, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3723
      if ( res /= 3723) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghWp, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5792
      if ( res /= 5792) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1244
      if ( res /= 1244) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1961
      if ( res /= 1961) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2286
      if ( res /= 2286) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9741
      if ( res /= 9741) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3959
      if ( res /= 3959) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10429
      if ( res /= 10429) then
        stop
      end if
      call createParticleConfig(P_AQ, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4349
      if ( res /= 4349) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9555
      if ( res /= 9555) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9826
      if ( res /= 9826) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1800
      if ( res /= 1800) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1308
      if ( res /= 1308) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_nu_tauAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2905
      if ( res /= 2905) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9966
      if ( res /= 9966) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",595
      if ( res /= 595) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7544
      if ( res /= 7544) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4226
      if ( res /= 4226) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2510
      if ( res /= 2510) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1314
      if ( res /= 1314) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6796
      if ( res /= 6796) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7046
      if ( res /= 7046) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10566
      if ( res /= 10566) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7946
      if ( res /= 7946) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghWmAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1333
      if ( res /= 1333) then
        stop
      end if
      call createParticleConfig(P_A, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3479
      if ( res /= 3479) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1402
      if ( res /= 1402) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",349
      if ( res /= 349) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9557
      if ( res /= 9557) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GPlus, P_GMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3244
      if ( res /= 3244) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8017
      if ( res /= 8017) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5350
      if ( res /= 5350) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5368
      if ( res /= 5368) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",761
      if ( res /= 761) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3197
      if ( res /= 3197) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_HPlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2710
      if ( res /= 2710) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5153
      if ( res /= 5153) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_A, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2476
      if ( res /= 2476) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6473
      if ( res /= 6473) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2484
      if ( res /= 2484) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4106
      if ( res /= 4106) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6453
      if ( res /= 6453) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_tAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7075
      if ( res /= 7075) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9745
      if ( res /= 9745) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10045
      if ( res /= 10045) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4115
      if ( res /= 4115) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9150
      if ( res /= 9150) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8169
      if ( res /= 8169) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6670
      if ( res /= 6670) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3796
      if ( res /= 3796) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2578
      if ( res /= 2578) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9198
      if ( res /= 9198) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9957
      if ( res /= 9957) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8847
      if ( res /= 8847) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_GMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8238
      if ( res /= 8238) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2590
      if ( res /= 2590) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1334
      if ( res /= 1334) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5413
      if ( res /= 5413) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7769
      if ( res /= 7769) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2752
      if ( res /= 2752) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5470
      if ( res /= 5470) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6855
      if ( res /= 6855) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10367
      if ( res /= 10367) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8956
      if ( res /= 8956) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2565
      if ( res /= 2565) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3002
      if ( res /= 3002) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8203
      if ( res /= 8203) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5466
      if ( res /= 5466) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZ, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1214
      if ( res /= 1214) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7137
      if ( res /= 7137) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4781
      if ( res /= 4781) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9349
      if ( res /= 9349) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3601
      if ( res /= 3601) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3983
      if ( res /= 3983) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5875
      if ( res /= 5875) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2361
      if ( res /= 2361) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8965
      if ( res /= 8965) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4097
      if ( res /= 4097) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",504
      if ( res /= 504) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8631
      if ( res /= 8631) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9534
      if ( res /= 9534) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6703
      if ( res /= 6703) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4124
      if ( res /= 4124) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_g, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6325
      if ( res /= 6325) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3856
      if ( res /= 3856) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6633
      if ( res /= 6633) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",978
      if ( res /= 978) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2947
      if ( res /= 2947) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7032
      if ( res /= 7032) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6193
      if ( res /= 6193) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",417
      if ( res /= 417) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7633
      if ( res /= 7633) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1520
      if ( res /= 1520) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",98
      if ( res /= 98) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6831
      if ( res /= 6831) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5565
      if ( res /= 5565) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4063
      if ( res /= 4063) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7429
      if ( res /= 7429) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6776
      if ( res /= 6776) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2157
      if ( res /= 2157) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6614
      if ( res /= 6614) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1707
      if ( res /= 1707) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7412
      if ( res /= 7412) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8795
      if ( res /= 8795) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2662
      if ( res /= 2662) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2117
      if ( res /= 2117) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3990
      if ( res /= 3990) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7419
      if ( res /= 7419) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",89
      if ( res /= 89) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10131
      if ( res /= 10131) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",736
      if ( res /= 736) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9106
      if ( res /= 9106) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3062
      if ( res /= 3062) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6347
      if ( res /= 6347) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6082
      if ( res /= 6082) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8884
      if ( res /= 8884) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9983
      if ( res /= 9983) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8475
      if ( res /= 8475) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6321
      if ( res /= 6321) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3283
      if ( res /= 3283) then
        stop
      end if
      call createParticleConfig(P_Hl, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5215
      if ( res /= 5215) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2028
      if ( res /= 2028) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2776
      if ( res /= 2776) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4551
      if ( res /= 4551) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2878
      if ( res /= 2878) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",387
      if ( res /= 387) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6086
      if ( res /= 6086) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1082
      if ( res /= 1082) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4450
      if ( res /= 4450) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4076
      if ( res /= 4076) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3368
      if ( res /= 3368) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghAAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6944
      if ( res /= 6944) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1464
      if ( res /= 1464) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1957
      if ( res /= 1957) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10130
      if ( res /= 10130) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1026
      if ( res /= 1026) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8197
      if ( res /= 8197) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4577
      if ( res /= 4577) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2872
      if ( res /= 2872) then
        stop
      end if
      call createParticleConfig(P_d, P_Hl, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5647
      if ( res /= 5647) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3045
      if ( res /= 3045) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4126
      if ( res /= 4126) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6827
      if ( res /= 6827) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6027
      if ( res /= 6027) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9952
      if ( res /= 9952) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5114
      if ( res /= 5114) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3310
      if ( res /= 3310) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2430
      if ( res /= 2430) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7592
      if ( res /= 7592) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9385
      if ( res /= 9385) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10006
      if ( res /= 10006) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10607
      if ( res /= 10607) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2260
      if ( res /= 2260) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9733
      if ( res /= 9733) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3286
      if ( res /= 3286) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_nu_e, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6990
      if ( res /= 6990) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4725
      if ( res /= 4725) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8186
      if ( res /= 8186) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7108
      if ( res /= 7108) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GMinus, P_GPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1498
      if ( res /= 1498) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4863
      if ( res /= 4863) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8969
      if ( res /= 8969) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1674
      if ( res /= 1674) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_Z, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2314
      if ( res /= 2314) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2460
      if ( res /= 2460) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",926
      if ( res /= 926) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10054
      if ( res /= 10054) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6231
      if ( res /= 6231) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5856
      if ( res /= 5856) then
        stop
      end if
      call createParticleConfig(P_g, P_Hl, P_g, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10591
      if ( res /= 10591) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5057
      if ( res /= 5057) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5209
      if ( res /= 5209) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5746
      if ( res /= 5746) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4247
      if ( res /= 4247) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2005
      if ( res /= 2005) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3662
      if ( res /= 3662) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",532
      if ( res /= 532) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWp, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4835
      if ( res /= 4835) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4987
      if ( res /= 4987) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3866
      if ( res /= 3866) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1440
      if ( res /= 1440) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GPlus, P_WMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2706
      if ( res /= 2706) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7662
      if ( res /= 7662) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",395
      if ( res /= 395) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1274
      if ( res /= 1274) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10286
      if ( res /= 10286) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6329
      if ( res /= 6329) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4925
      if ( res /= 4925) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1550
      if ( res /= 1550) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_Ha, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5175
      if ( res /= 5175) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10630
      if ( res /= 10630) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2637
      if ( res /= 2637) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9803
      if ( res /= 9803) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7885
      if ( res /= 7885) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9995
      if ( res /= 9995) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5732
      if ( res /= 5732) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3117
      if ( res /= 3117) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8375
      if ( res /= 8375) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7470
      if ( res /= 7470) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5382
      if ( res /= 5382) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3457
      if ( res /= 3457) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2073
      if ( res /= 2073) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",932
      if ( res /= 932) then
        stop
      end if
      call createParticleConfig(P_A, P_g, P_A, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10768
      if ( res /= 10768) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6186
      if ( res /= 6186) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8678
      if ( res /= 8678) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3409
      if ( res /= 3409) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10616
      if ( res /= 10616) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5888
      if ( res /= 5888) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4170
      if ( res /= 4170) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10145
      if ( res /= 10145) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8662
      if ( res /= 8662) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10196
      if ( res /= 10196) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7598
      if ( res /= 7598) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10319
      if ( res /= 10319) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",820
      if ( res /= 820) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_Z, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1514
      if ( res /= 1514) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6172
      if ( res /= 6172) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9085
      if ( res /= 9085) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7882
      if ( res /= 7882) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8484
      if ( res /= 8484) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4829
      if ( res /= 4829) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",241
      if ( res /= 241) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2953
      if ( res /= 2953) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_t, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2782
      if ( res /= 2782) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5633
      if ( res /= 5633) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",20
      if ( res /= 20) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8723
      if ( res /= 8723) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8828
      if ( res /= 8828) then
        stop
      end if
      call createParticleConfig(P_G0, P_g, P_g, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10705
      if ( res /= 10705) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2365
      if ( res /= 2365) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1508
      if ( res /= 1508) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9105
      if ( res /= 9105) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4815
      if ( res /= 4815) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6097
      if ( res /= 6097) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_u, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4059
      if ( res /= 4059) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4921
      if ( res /= 4921) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWmAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6744
      if ( res /= 6744) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8851
      if ( res /= 8851) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7167
      if ( res /= 7167) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7179
      if ( res /= 7179) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10173
      if ( res /= 10173) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3765
      if ( res /= 3765) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10310
      if ( res /= 10310) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",645
      if ( res /= 645) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8933
      if ( res /= 8933) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5132
      if ( res /= 5132) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8937
      if ( res /= 8937) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_G0, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2744
      if ( res /= 2744) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_ghG, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1662
      if ( res /= 1662) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",293
      if ( res /= 293) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4177
      if ( res /= 4177) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HMinus, P_ghWp, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9319
      if ( res /= 9319) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",505
      if ( res /= 505) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3041
      if ( res /= 3041) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4323
      if ( res /= 4323) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",840
      if ( res /= 840) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5105
      if ( res /= 5105) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWp, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4310
      if ( res /= 4310) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9998
      if ( res /= 9998) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4446
      if ( res /= 4446) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3133
      if ( res /= 3133) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghZ, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7870
      if ( res /= 7870) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2927
      if ( res /= 2927) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10111
      if ( res /= 10111) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2943
      if ( res /= 2943) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4146
      if ( res /= 4146) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10231
      if ( res /= 10231) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6760
      if ( res /= 6760) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2370
      if ( res /= 2370) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_b, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2634
      if ( res /= 2634) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5332
      if ( res /= 5332) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",739
      if ( res /= 739) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5656
      if ( res /= 5656) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",912
      if ( res /= 912) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1075
      if ( res /= 1075) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3558
      if ( res /= 3558) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10290
      if ( res /= 10290) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5705
      if ( res /= 5705) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8217
      if ( res /= 8217) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6393
      if ( res /= 6393) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9316
      if ( res /= 9316) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9218
      if ( res /= 9218) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10663
      if ( res /= 10663) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_s, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8281
      if ( res /= 8281) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5407
      if ( res /= 5407) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3778
      if ( res /= 3778) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_G0, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7628
      if ( res /= 7628) then
        stop
      end if
      call createParticleConfig(P_t, P_WMinus, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5631
      if ( res /= 5631) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7731
      if ( res /= 7731) then
        stop
      end if
      call createParticleConfig(P_Hl, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7693
      if ( res /= 7693) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1210
      if ( res /= 1210) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9865
      if ( res /= 9865) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2201
      if ( res /= 2201) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",502
      if ( res /= 502) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7684
      if ( res /= 7684) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WPlus, P_HMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10303
      if ( res /= 10303) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1857
      if ( res /= 1857) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",67
      if ( res /= 67) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6747
      if ( res /= 6747) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7965
      if ( res /= 7965) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4623
      if ( res /= 4623) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1614
      if ( res /= 1614) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWp, P_ghWpAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9816
      if ( res /= 9816) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghZAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4086
      if ( res /= 4086) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",40
      if ( res /= 40) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5584
      if ( res /= 5584) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10388
      if ( res /= 10388) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2888
      if ( res /= 2888) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4081
      if ( res /= 4081) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8249
      if ( res /= 8249) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9024
      if ( res /= 9024) then
        stop
      end if
      call createParticleConfig(P_G0, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6102
      if ( res /= 6102) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2874
      if ( res /= 2874) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_nu_eAnti, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8246
      if ( res /= 8246) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_Hl, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1530
      if ( res /= 1530) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9697
      if ( res /= 9697) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7988
      if ( res /= 7988) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_g, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3616
      if ( res /= 3616) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",331
      if ( res /= 331) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6107
      if ( res /= 6107) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_A, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6589
      if ( res /= 6589) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4082
      if ( res /= 4082) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",185
      if ( res /= 185) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4616
      if ( res /= 4616) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8148
      if ( res /= 8148) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1307
      if ( res /= 1307) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10481
      if ( res /= 10481) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2756
      if ( res /= 2756) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",285
      if ( res /= 285) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2
      if ( res /= 2) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",475
      if ( res /= 475) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWmAnti, P_GMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9003
      if ( res /= 9003) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghWp, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7827
      if ( res /= 7827) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5516
      if ( res /= 5516) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7512
      if ( res /= 7512) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10652
      if ( res /= 10652) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4866
      if ( res /= 4866) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4589
      if ( res /= 4589) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2265
      if ( res /= 2265) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1641
      if ( res /= 1641) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6868
      if ( res /= 6868) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1735
      if ( res /= 1735) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9930
      if ( res /= 9930) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2622
      if ( res /= 2622) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_G0, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",930
      if ( res /= 930) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5025
      if ( res /= 5025) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3059
      if ( res /= 3059) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9013
      if ( res /= 9013) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2080
      if ( res /= 2080) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10550
      if ( res /= 10550) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6235
      if ( res /= 6235) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_u, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8453
      if ( res /= 8453) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6334
      if ( res /= 6334) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4290
      if ( res /= 4290) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8270
      if ( res /= 8270) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4241
      if ( res /= 4241) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9539
      if ( res /= 9539) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6205
      if ( res /= 6205) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9719
      if ( res /= 9719) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3419
      if ( res /= 3419) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3587
      if ( res /= 3587) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8999
      if ( res /= 8999) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6556
      if ( res /= 6556) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2108
      if ( res /= 2108) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghAAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6078
      if ( res /= 6078) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWm, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8418
      if ( res /= 8418) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6695
      if ( res /= 6695) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7362
      if ( res /= 7362) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3908
      if ( res /= 3908) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",100
      if ( res /= 100) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1261
      if ( res /= 1261) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6286
      if ( res /= 6286) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8254
      if ( res /= 8254) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7034
      if ( res /= 7034) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10332
      if ( res /= 10332) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8642
      if ( res /= 8642) then
        stop
      end if
      call createParticleConfig(P_A, P_ghA, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7082
      if ( res /= 7082) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10689
      if ( res /= 10689) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",340
      if ( res /= 340) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8464
      if ( res /= 8464) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6839
      if ( res /= 6839) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8319
      if ( res /= 8319) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2177
      if ( res /= 2177) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2270
      if ( res /= 2270) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6585
      if ( res /= 6585) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8116
      if ( res /= 8116) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8876
      if ( res /= 8876) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4132
      if ( res /= 4132) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9990
      if ( res /= 9990) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3874
      if ( res /= 3874) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1823
      if ( res /= 1823) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2837
      if ( res /= 2837) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2206
      if ( res /= 2206) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3076
      if ( res /= 3076) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3099
      if ( res /= 3099) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWp, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7530
      if ( res /= 7530) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_G0, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6164
      if ( res /= 6164) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5133
      if ( res /= 5133) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8277
      if ( res /= 8277) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9390
      if ( res /= 9390) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWm, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2376
      if ( res /= 2376) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5479
      if ( res /= 5479) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7112
      if ( res /= 7112) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5968
      if ( res /= 5968) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9348
      if ( res /= 9348) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5454
      if ( res /= 5454) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10012
      if ( res /= 10012) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",259
      if ( res /= 259) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1904
      if ( res /= 1904) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Z, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7306
      if ( res /= 7306) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWm, P_WPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6830
      if ( res /= 6830) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7830
      if ( res /= 7830) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5224
      if ( res /= 5224) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5055
      if ( res /= 5055) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8306
      if ( res /= 8306) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10729
      if ( res /= 10729) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9892
      if ( res /= 9892) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9712
      if ( res /= 9712) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6265
      if ( res /= 6265) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8739
      if ( res /= 8739) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2997
      if ( res /= 2997) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9911
      if ( res /= 9911) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4596
      if ( res /= 4596) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8607
      if ( res /= 8607) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6971
      if ( res /= 6971) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",553
      if ( res /= 553) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3985
      if ( res /= 3985) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3619
      if ( res /= 3619) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8647
      if ( res /= 8647) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9967
      if ( res /= 9967) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7274
      if ( res /= 7274) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7852
      if ( res /= 7852) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2031
      if ( res /= 2031) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7987
      if ( res /= 7987) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7119
      if ( res /= 7119) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6968
      if ( res /= 6968) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghA, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7789
      if ( res /= 7789) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_d, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8388
      if ( res /= 8388) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7432
      if ( res /= 7432) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9116
      if ( res /= 9116) then
        stop
      end if
      call createParticleConfig(P_Z, P_nu_eAnti, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4511
      if ( res /= 4511) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6148
      if ( res /= 6148) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1103
      if ( res /= 1103) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5311
      if ( res /= 5311) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8218
      if ( res /= 8218) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5630
      if ( res /= 5630) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2832
      if ( res /= 2832) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_u, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1197
      if ( res /= 1197) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2040
      if ( res /= 2040) then
        stop
      end if
      call createParticleConfig(P_d, P_Ha, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1185
      if ( res /= 1185) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",312
      if ( res /= 312) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6195
      if ( res /= 6195) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4939
      if ( res /= 4939) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1706
      if ( res /= 1706) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3564
      if ( res /= 3564) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2526
      if ( res /= 2526) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5443
      if ( res /= 5443) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3714
      if ( res /= 3714) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9909
      if ( res /= 9909) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",38
      if ( res /= 38) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",673
      if ( res /= 673) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6798
      if ( res /= 6798) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6821
      if ( res /= 6821) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",996
      if ( res /= 996) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZ, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",336
      if ( res /= 336) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_eMinus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8332
      if ( res /= 8332) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7743
      if ( res /= 7743) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4068
      if ( res /= 4068) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8220
      if ( res /= 8220) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8649
      if ( res /= 8649) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7556
      if ( res /= 7556) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WMinus, P_ghWmAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5782
      if ( res /= 5782) then
        stop
      end if
      call createParticleConfig(P_Ha, P_g, P_g, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10668
      if ( res /= 10668) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_nu_e, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8546
      if ( res /= 8546) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5074
      if ( res /= 5074) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1899
      if ( res /= 1899) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",333
      if ( res /= 333) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWpAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8711
      if ( res /= 8711) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghWpAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7201
      if ( res /= 7201) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2236
      if ( res /= 2236) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",615
      if ( res /= 615) then
        stop
      end if
      call createParticleConfig(P_AQ, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3183
      if ( res /= 3183) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1474
      if ( res /= 1474) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2639
      if ( res /= 2639) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6384
      if ( res /= 6384) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2697
      if ( res /= 2697) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10278
      if ( res /= 10278) then
        stop
      end if
      call createParticleConfig(P_b, P_tAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5424
      if ( res /= 5424) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghAAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8608
      if ( res /= 8608) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3668
      if ( res /= 3668) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7599
      if ( res /= 7599) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9301
      if ( res /= 9301) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",353
      if ( res /= 353) then
        stop
      end if
      call createParticleConfig(P_Z, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5905
      if ( res /= 5905) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10418
      if ( res /= 10418) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7321
      if ( res /= 7321) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3488
      if ( res /= 3488) then
        stop
      end if
      call createParticleConfig(P_A, P_ghAAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2096
      if ( res /= 2096) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6532
      if ( res /= 6532) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",78
      if ( res /= 78) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5486
      if ( res /= 5486) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9231
      if ( res /= 9231) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",768
      if ( res /= 768) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8130
      if ( res /= 8130) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7041
      if ( res /= 7041) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2296
      if ( res /= 2296) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8285
      if ( res /= 8285) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3464
      if ( res /= 3464) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6466
      if ( res /= 6466) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10295
      if ( res /= 10295) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5032
      if ( res /= 5032) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7090
      if ( res /= 7090) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6094
      if ( res /= 6094) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5752
      if ( res /= 5752) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1096
      if ( res /= 1096) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6236
      if ( res /= 6236) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6829
      if ( res /= 6829) then
        stop
      end if
      call createParticleConfig(P_g, P_G0, P_g, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10549
      if ( res /= 10549) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4918
      if ( res /= 4918) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8005
      if ( res /= 8005) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8265
      if ( res /= 8265) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8582
      if ( res /= 8582) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1371
      if ( res /= 1371) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4919
      if ( res /= 4919) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6026
      if ( res /= 6026) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3429
      if ( res /= 3429) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",759
      if ( res /= 759) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2714
      if ( res /= 2714) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10771
      if ( res /= 10771) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6896
      if ( res /= 6896) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWmAnti, P_ghWm, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8897
      if ( res /= 8897) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9434
      if ( res /= 9434) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_nu_muAnti, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3467
      if ( res /= 3467) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9237
      if ( res /= 9237) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6019
      if ( res /= 6019) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5697
      if ( res /= 5697) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10778
      if ( res /= 10778) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4982
      if ( res /= 4982) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1801
      if ( res /= 1801) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2502
      if ( res /= 2502) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1545
      if ( res /= 1545) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_HMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",533
      if ( res /= 533) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7140
      if ( res /= 7140) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",437
      if ( res /= 437) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9893
      if ( res /= 9893) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2004
      if ( res /= 2004) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10484
      if ( res /= 10484) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",327
      if ( res /= 327) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1891
      if ( res /= 1891) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7234
      if ( res /= 7234) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10221
      if ( res /= 10221) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6477
      if ( res /= 6477) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",332
      if ( res /= 332) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6096
      if ( res /= 6096) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3012
      if ( res /= 3012) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5831
      if ( res /= 5831) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3040
      if ( res /= 3040) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8800
      if ( res /= 8800) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7959
      if ( res /= 7959) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8353
      if ( res /= 8353) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8069
      if ( res /= 8069) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_nu_mu, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",965
      if ( res /= 965) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9699
      if ( res /= 9699) then
        stop
      end if
      call createParticleConfig(P_Hh, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5197
      if ( res /= 5197) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_muMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2513
      if ( res /= 2513) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4369
      if ( res /= 4369) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5457
      if ( res /= 5457) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5122
      if ( res /= 5122) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_dAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2678
      if ( res /= 2678) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9432
      if ( res /= 9432) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2015
      if ( res /= 2015) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5067
      if ( res /= 5067) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5675
      if ( res /= 5675) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1184
      if ( res /= 1184) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8942
      if ( res /= 8942) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3158
      if ( res /= 3158) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8907
      if ( res /= 8907) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4123
      if ( res /= 4123) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2673
      if ( res /= 2673) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8845
      if ( res /= 8845) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",506
      if ( res /= 506) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GPlus, P_GMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5513
      if ( res /= 5513) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghZ, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",319
      if ( res /= 319) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2736
      if ( res /= 2736) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4778
      if ( res /= 4778) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_nu_mu, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2067
      if ( res /= 2067) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4202
      if ( res /= 4202) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1585
      if ( res /= 1585) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",789
      if ( res /= 789) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_eMinus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5727
      if ( res /= 5727) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_muPlus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1053
      if ( res /= 1053) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1190
      if ( res /= 1190) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1291
      if ( res /= 1291) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4680
      if ( res /= 4680) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4889
      if ( res /= 4889) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3599
      if ( res /= 3599) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1677
      if ( res /= 1677) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10282
      if ( res /= 10282) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8309
      if ( res /= 8309) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7773
      if ( res /= 7773) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3381
      if ( res /= 3381) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2287
      if ( res /= 2287) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2984
      if ( res /= 2984) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4397
      if ( res /= 4397) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghWpAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4006
      if ( res /= 4006) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghA, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6457
      if ( res /= 6457) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7848
      if ( res /= 7848) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9086
      if ( res /= 9086) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6360
      if ( res /= 6360) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5744
      if ( res /= 5744) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4603
      if ( res /= 4603) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8366
      if ( res /= 8366) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_muPlus, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3920
      if ( res /= 3920) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7766
      if ( res /= 7766) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",983
      if ( res /= 983) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7612
      if ( res /= 7612) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3494
      if ( res /= 3494) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8680
      if ( res /= 8680) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2784
      if ( res /= 2784) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2542
      if ( res /= 2542) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9884
      if ( res /= 9884) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2294
      if ( res /= 2294) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7757
      if ( res /= 7757) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_nu_eAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4727
      if ( res /= 4727) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6204
      if ( res /= 6204) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7394
      if ( res /= 7394) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6675
      if ( res /= 6675) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7816
      if ( res /= 7816) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2572
      if ( res /= 2572) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3820
      if ( res /= 3820) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4503
      if ( res /= 4503) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9540
      if ( res /= 9540) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghA, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5128
      if ( res /= 5128) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1793
      if ( res /= 1793) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2743
      if ( res /= 2743) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4505
      if ( res /= 4505) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZ, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5491
      if ( res /= 5491) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4355
      if ( res /= 4355) then
        stop
      end if
      call createParticleConfig(P_g, P_GPlus, P_HMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10761
      if ( res /= 10761) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7745
      if ( res /= 7745) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7319
      if ( res /= 7319) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7583
      if ( res /= 7583) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9542
      if ( res /= 9542) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2022
      if ( res /= 2022) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2212
      if ( res /= 2212) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2581
      if ( res /= 2581) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9648
      if ( res /= 9648) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7152
      if ( res /= 7152) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6244
      if ( res /= 6244) then
        stop
      end if
      call createParticleConfig(P_g, P_HPlus, P_g, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10512
      if ( res /= 10512) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8247
      if ( res /= 8247) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWmAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2763
      if ( res /= 2763) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_u, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8522
      if ( res /= 8522) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HMinus, P_GPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6065
      if ( res /= 6065) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7098
      if ( res /= 7098) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5080
      if ( res /= 5080) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9190
      if ( res /= 9190) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1600
      if ( res /= 1600) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6975
      if ( res /= 6975) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghZ, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2508
      if ( res /= 2508) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_A, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8717
      if ( res /= 8717) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4731
      if ( res /= 4731) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HMinus, P_GPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6373
      if ( res /= 6373) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7009
      if ( res /= 7009) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10158
      if ( res /= 10158) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10139
      if ( res /= 10139) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7775
      if ( res /= 7775) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_g, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4886
      if ( res /= 4886) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4844
      if ( res /= 4844) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2121
      if ( res /= 2121) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5176
      if ( res /= 5176) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5558
      if ( res /= 5558) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",429
      if ( res /= 429) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9175
      if ( res /= 9175) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WMinus, P_WPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4357
      if ( res /= 4357) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6338
      if ( res /= 6338) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",307
      if ( res /= 307) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1858
      if ( res /= 1858) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1911
      if ( res /= 1911) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3605
      if ( res /= 3605) then
        stop
      end if
      call createParticleConfig(P_gQ, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6468
      if ( res /= 6468) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",586
      if ( res /= 586) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9302
      if ( res /= 9302) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2942
      if ( res /= 2942) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghWp, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7820
      if ( res /= 7820) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6358
      if ( res /= 6358) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_sAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1610
      if ( res /= 1610) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8797
      if ( res /= 8797) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5446
      if ( res /= 5446) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",631
      if ( res /= 631) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7308
      if ( res /= 7308) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2186
      if ( res /= 2186) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9421
      if ( res /= 9421) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZ, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3242
      if ( res /= 3242) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6613
      if ( res /= 6613) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4105
      if ( res /= 4105) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1245
      if ( res /= 1245) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_nu_e, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5045
      if ( res /= 5045) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3096
      if ( res /= 3096) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4280
      if ( res /= 4280) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4067
      if ( res /= 4067) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7828
      if ( res /= 7828) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9546
      if ( res /= 9546) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10220
      if ( res /= 10220) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3081
      if ( res /= 3081) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9061
      if ( res /= 9061) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6138
      if ( res /= 6138) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5901
      if ( res /= 5901) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3568
      if ( res /= 3568) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4248
      if ( res /= 4248) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9229
      if ( res /= 9229) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8386
      if ( res /= 8386) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4249
      if ( res /= 4249) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9357
      if ( res /= 9357) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3162
      if ( res /= 3162) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_muMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",750
      if ( res /= 750) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5079
      if ( res /= 5079) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Z, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2543
      if ( res /= 2543) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1693
      if ( res /= 1693) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5035
      if ( res /= 5035) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7376
      if ( res /= 7376) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5046
      if ( res /= 5046) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2345
      if ( res /= 2345) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4443
      if ( res /= 4443) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2003
      if ( res /= 2003) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8901
      if ( res /= 8901) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9796
      if ( res /= 9796) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5695
      if ( res /= 5695) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7944
      if ( res /= 7944) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4449
      if ( res /= 4449) then
        stop
      end if
      call createParticleConfig(P_g, P_ghGAnti, P_g, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4133
      if ( res /= 4133) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10716
      if ( res /= 10716) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3715
      if ( res /= 3715) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9140
      if ( res /= 9140) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6993
      if ( res /= 6993) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10072
      if ( res /= 10072) then
        stop
      end if
      call createParticleConfig(P_gQ, P_ghG, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",617
      if ( res /= 617) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9272
      if ( res /= 9272) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6214
      if ( res /= 6214) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7359
      if ( res /= 7359) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10096
      if ( res /= 10096) then
        stop
      end if
      call createParticleConfig(P_A, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1253
      if ( res /= 1253) then
        stop
      end if
      call createParticleConfig(P_A, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6669
      if ( res /= 6669) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5123
      if ( res /= 5123) then
        stop
      end if
      call createParticleConfig(P_gQ, P_ghGAnti, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1394
      if ( res /= 1394) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3383
      if ( res /= 3383) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9918
      if ( res /= 9918) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",151
      if ( res /= 151) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1236
      if ( res /= 1236) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8323
      if ( res /= 8323) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3730
      if ( res /= 3730) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1431
      if ( res /= 1431) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3740
      if ( res /= 3740) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2210
      if ( res /= 2210) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_Z, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7605
      if ( res /= 7605) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8578
      if ( res /= 8578) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7948
      if ( res /= 7948) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3265
      if ( res /= 3265) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",606
      if ( res /= 606) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4112
      if ( res /= 4112) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6865
      if ( res /= 6865) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6093
      if ( res /= 6093) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2351
      if ( res /= 2351) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",33
      if ( res /= 33) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_G0, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7007
      if ( res /= 7007) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6832
      if ( res /= 6832) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8321
      if ( res /= 8321) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9060
      if ( res /= 9060) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghA, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3319
      if ( res /= 3319) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_Hl, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3425
      if ( res /= 3425) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8294
      if ( res /= 8294) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6460
      if ( res /= 6460) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6271
      if ( res /= 6271) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7333
      if ( res /= 7333) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",877
      if ( res /= 877) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9011
      if ( res /= 9011) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8149
      if ( res /= 8149) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6424
      if ( res /= 6424) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",812
      if ( res /= 812) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7344
      if ( res /= 7344) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10750
      if ( res /= 10750) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5
      if ( res /= 5) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",763
      if ( res /= 763) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4674
      if ( res /= 4674) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9422
      if ( res /= 9422) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7048
      if ( res /= 7048) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3055
      if ( res /= 3055) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3721
      if ( res /= 3721) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghAAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",402
      if ( res /= 402) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2309
      if ( res /= 2309) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3774
      if ( res /= 3774) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4386
      if ( res /= 4386) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4187
      if ( res /= 4187) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",427
      if ( res /= 427) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5159
      if ( res /= 5159) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9177
      if ( res /= 9177) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5204
      if ( res /= 5204) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6835
      if ( res /= 6835) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7525
      if ( res /= 7525) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6955
      if ( res /= 6955) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7939
      if ( res /= 7939) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghZ, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6659
      if ( res /= 6659) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9704
      if ( res /= 9704) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7829
      if ( res /= 7829) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1212
      if ( res /= 1212) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4811
      if ( res /= 4811) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9547
      if ( res /= 9547) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9136
      if ( res /= 9136) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3934
      if ( res /= 3934) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",256
      if ( res /= 256) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5547
      if ( res /= 5547) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6228
      if ( res /= 6228) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6691
      if ( res /= 6691) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9939
      if ( res /= 9939) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1566
      if ( res /= 1566) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWm, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1282
      if ( res /= 1282) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6606
      if ( res /= 6606) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_Hl, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6823
      if ( res /= 6823) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8910
      if ( res /= 8910) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7261
      if ( res /= 7261) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3918
      if ( res /= 3918) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4430
      if ( res /= 4430) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5434
      if ( res /= 5434) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8199
      if ( res /= 8199) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2989
      if ( res /= 2989) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4019
      if ( res /= 4019) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3428
      if ( res /= 3428) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10370
      if ( res /= 10370) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3160
      if ( res /= 3160) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9936
      if ( res /= 9936) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1489
      if ( res /= 1489) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",284
      if ( res /= 284) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5383
      if ( res /= 5383) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_muPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4462
      if ( res /= 4462) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_ghGAnti, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1734
      if ( res /= 1734) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6226
      if ( res /= 6226) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Z, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3622
      if ( res /= 3622) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10292
      if ( res /= 10292) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8106
      if ( res /= 8106) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3933
      if ( res /= 3933) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1424
      if ( res /= 1424) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3413
      if ( res /= 3413) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2449
      if ( res /= 2449) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3863
      if ( res /= 3863) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1426
      if ( res /= 1426) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6537
      if ( res /= 6537) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7700
      if ( res /= 7700) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",512
      if ( res /= 512) then
        stop
      end if
      call createParticleConfig(P_Hh, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8737
      if ( res /= 8737) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4480
      if ( res /= 4480) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3503
      if ( res /= 3503) then
        stop
      end if
      call createParticleConfig(P_g, P_WMinus, P_g, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10720
      if ( res /= 10720) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3195
      if ( res /= 3195) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10379
      if ( res /= 10379) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7843
      if ( res /= 7843) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6495
      if ( res /= 6495) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7548
      if ( res /= 7548) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8781
      if ( res /= 8781) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7758
      if ( res /= 7758) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10371
      if ( res /= 10371) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_muMinus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2733
      if ( res /= 2733) then
        stop
      end if
      call createParticleConfig(P_c, P_Ha, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4467
      if ( res /= 4467) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7064
      if ( res /= 7064) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5333
      if ( res /= 5333) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_G0, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3280
      if ( res /= 3280) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8917
      if ( res /= 8917) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_Z, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1111
      if ( res /= 1111) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8094
      if ( res /= 8094) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9701
      if ( res /= 9701) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10444
      if ( res /= 10444) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4444
      if ( res /= 4444) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7100
      if ( res /= 7100) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5195
      if ( res /= 5195) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3754
      if ( res /= 3754) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghA, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6965
      if ( res /= 6965) then
        stop
      end if
      call createParticleConfig(P_g, P_A, P_g, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10554
      if ( res /= 10554) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5868
      if ( res /= 5868) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4466
      if ( res /= 4466) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",541
      if ( res /= 541) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HMinus, P_GPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7884
      if ( res /= 7884) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6043
      if ( res /= 6043) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8814
      if ( res /= 8814) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3320
      if ( res /= 3320) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7395
      if ( res /= 7395) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8066
      if ( res /= 8066) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_HPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7705
      if ( res /= 7705) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7837
      if ( res /= 7837) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2124
      if ( res /= 2124) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4969
      if ( res /= 4969) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5835
      if ( res /= 5835) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",379
      if ( res /= 379) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7305
      if ( res /= 7305) then
        stop
      end if
      call createParticleConfig(P_d, P_uAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6685
      if ( res /= 6685) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_tauPlus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3837
      if ( res /= 3837) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7465
      if ( res /= 7465) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8627
      if ( res /= 8627) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5849
      if ( res /= 5849) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1462
      if ( res /= 1462) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5538
      if ( res /= 5538) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",779
      if ( res /= 779) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10480
      if ( res /= 10480) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8296
      if ( res /= 8296) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3327
      if ( res /= 3327) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6696
      if ( res /= 6696) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1169
      if ( res /= 1169) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_Hh, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7497
      if ( res /= 7497) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6095
      if ( res /= 6095) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1275
      if ( res /= 1275) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8948
      if ( res /= 8948) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6800
      if ( res /= 6800) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2020
      if ( res /= 2020) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2821
      if ( res /= 2821) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4552
      if ( res /= 4552) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",384
      if ( res /= 384) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7008
      if ( res /= 7008) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3187
      if ( res /= 3187) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GPlus, P_GMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7483
      if ( res /= 7483) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8987
      if ( res /= 8987) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7656
      if ( res /= 7656) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10457
      if ( res /= 10457) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5627
      if ( res /= 5627) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4983
      if ( res /= 4983) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",725
      if ( res /= 725) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1522
      if ( res /= 1522) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5243
      if ( res /= 5243) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_tauPlus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6272
      if ( res /= 6272) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7380
      if ( res /= 7380) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9693
      if ( res /= 9693) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7964
      if ( res /= 7964) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2815
      if ( res /= 2815) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9671
      if ( res /= 9671) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5061
      if ( res /= 5061) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_bAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8151
      if ( res /= 8151) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2693
      if ( res /= 2693) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8189
      if ( res /= 8189) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1374
      if ( res /= 1374) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5206
      if ( res /= 5206) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4897
      if ( res /= 4897) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7250
      if ( res /= 7250) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3490
      if ( res /= 3490) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6029
      if ( res /= 6029) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4548
      if ( res /= 4548) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghA, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2114
      if ( res /= 2114) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10317
      if ( res /= 10317) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5689
      if ( res /= 5689) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2614
      if ( res /= 2614) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7560
      if ( res /= 7560) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWp, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1795
      if ( res /= 1795) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3241
      if ( res /= 3241) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6949
      if ( res /= 6949) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZ, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1491
      if ( res /= 1491) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4780
      if ( res /= 4780) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Ha, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5496
      if ( res /= 5496) then
        stop
      end if
      call createParticleConfig(P_Hl, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5543
      if ( res /= 5543) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9352
      if ( res /= 9352) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4533
      if ( res /= 4533) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6782
      if ( res /= 6782) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7168
      if ( res /= 7168) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghWpAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9592
      if ( res /= 9592) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_ghAAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1993
      if ( res /= 1993) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",772
      if ( res /= 772) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9965
      if ( res /= 9965) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7900
      if ( res /= 7900) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2469
      if ( res /= 2469) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10507
      if ( res /= 10507) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",878
      if ( res /= 878) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10271
      if ( res /= 10271) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10583
      if ( res /= 10583) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9549
      if ( res /= 9549) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7832
      if ( res /= 7832) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1985
      if ( res /= 1985) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghA, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",200
      if ( res /= 200) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9594
      if ( res /= 9594) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6526
      if ( res /= 6526) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZ, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8511
      if ( res /= 8511) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1078
      if ( res /= 1078) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4737
      if ( res /= 4737) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",653
      if ( res /= 653) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8501
      if ( res /= 8501) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3657
      if ( res /= 3657) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2551
      if ( res /= 2551) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10328
      if ( res /= 10328) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HMinus, P_HPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8055
      if ( res /= 8055) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9054
      if ( res /= 9054) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",305
      if ( res /= 305) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5639
      if ( res /= 5639) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4696
      if ( res /= 4696) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7602
      if ( res /= 7602) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7575
      if ( res /= 7575) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10119
      if ( res /= 10119) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5810
      if ( res /= 5810) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3516
      if ( res /= 3516) then
        stop
      end if
      call createParticleConfig(P_t, P_bAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6500
      if ( res /= 6500) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2420
      if ( res /= 2420) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7368
      if ( res /= 7368) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1871
      if ( res /= 1871) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10041
      if ( res /= 10041) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GPlus, P_ghWpAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9253
      if ( res /= 9253) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghWm, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1640
      if ( res /= 1640) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7387
      if ( res /= 7387) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5892
      if ( res /= 5892) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4900
      if ( res /= 4900) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6686
      if ( res /= 6686) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2722
      if ( res /= 2722) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10201
      if ( res /= 10201) then
        stop
      end if
      call createParticleConfig(P_g, P_Z, P_g, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10706
      if ( res /= 10706) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6643
      if ( res /= 6643) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4407
      if ( res /= 4407) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7047
      if ( res /= 7047) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3950
      if ( res /= 3950) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4337
      if ( res /= 4337) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",613
      if ( res /= 613) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7117
      if ( res /= 7117) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8695
      if ( res /= 8695) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7711
      if ( res /= 7711) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1267
      if ( res /= 1267) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_tauMinus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",410
      if ( res /= 410) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",637
      if ( res /= 637) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1387
      if ( res /= 1387) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",87
      if ( res /= 87) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8324
      if ( res /= 8324) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4119
      if ( res /= 4119) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9495
      if ( res /= 9495) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10153
      if ( res /= 10153) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8213
      if ( res /= 8213) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9171
      if ( res /= 9171) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_tAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1926
      if ( res /= 1926) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10279
      if ( res /= 10279) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghZAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5879
      if ( res /= 5879) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6538
      if ( res /= 6538) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10169
      if ( res /= 10169) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2053
      if ( res /= 2053) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6905
      if ( res /= 6905) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4595
      if ( res /= 4595) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1971
      if ( res /= 1971) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9219
      if ( res /= 9219) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4947
      if ( res /= 4947) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GPlus, P_GMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8692
      if ( res /= 8692) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4882
      if ( res /= 4882) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9771
      if ( res /= 9771) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3550
      if ( res /= 3550) then
        stop
      end if
      call createParticleConfig(P_c, P_GMinus, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6770
      if ( res /= 6770) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6487
      if ( res /= 6487) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7245
      if ( res /= 7245) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_s, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",995
      if ( res /= 995) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7977
      if ( res /= 7977) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8210
      if ( res /= 8210) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9203
      if ( res /= 9203) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3629
      if ( res /= 3629) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZ, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7294
      if ( res /= 7294) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7511
      if ( res /= 7511) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6005
      if ( res /= 6005) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghZ, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7252
      if ( res /= 7252) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_uAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4474
      if ( res /= 4474) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HPlus, P_GMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9727
      if ( res /= 9727) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7985
      if ( res /= 7985) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6277
      if ( res /= 6277) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5033
      if ( res /= 5033) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghAAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6564
      if ( res /= 6564) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3956
      if ( res /= 3956) then
        stop
      end if
      call createParticleConfig(P_g, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4321
      if ( res /= 4321) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10399
      if ( res /= 10399) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8860
      if ( res /= 8860) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6712
      if ( res /= 6712) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9575
      if ( res /= 9575) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6910
      if ( res /= 6910) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3809
      if ( res /= 3809) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4276
      if ( res /= 4276) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1143
      if ( res /= 1143) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",948
      if ( res /= 948) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2280
      if ( res /= 2280) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9665
      if ( res /= 9665) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1889
      if ( res /= 1889) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2218
      if ( res /= 2218) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8087
      if ( res /= 8087) then
        stop
      end if
      call createParticleConfig(P_Hl, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2682
      if ( res /= 2682) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7970
      if ( res /= 7970) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1962
      if ( res /= 1962) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1298
      if ( res /= 1298) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7831
      if ( res /= 7831) then
        stop
      end if
      call createParticleConfig(P_Hh, P_g, P_Hh, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10506
      if ( res /= 10506) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3290
      if ( res /= 3290) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8407
      if ( res /= 8407) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6012
      if ( res /= 6012) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_g, P_g, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10738
      if ( res /= 10738) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghWm, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8542
      if ( res /= 8542) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3331
      if ( res /= 3331) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6337
      if ( res /= 6337) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",559
      if ( res /= 559) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2442
      if ( res /= 2442) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8532
      if ( res /= 8532) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10740
      if ( res /= 10740) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4508
      if ( res /= 4508) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10063
      if ( res /= 10063) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_uAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",137
      if ( res /= 137) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7659
      if ( res /= 7659) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8209
      if ( res /= 8209) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2707
      if ( res /= 2707) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghZAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4147
      if ( res /= 4147) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9531
      if ( res /= 9531) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8567
      if ( res /= 8567) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1531
      if ( res /= 1531) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_Hh, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1743
      if ( res /= 1743) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4639
      if ( res /= 4639) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5326
      if ( res /= 5326) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10764
      if ( res /= 10764) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2574
      if ( res /= 2574) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WMinus, P_GPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7111
      if ( res /= 7111) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7495
      if ( res /= 7495) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1423
      if ( res /= 1423) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6853
      if ( res /= 6853) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10309
      if ( res /= 10309) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10589
      if ( res /= 10589) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",73
      if ( res /= 73) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_GPlus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8455
      if ( res /= 8455) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4528
      if ( res /= 4528) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4224
      if ( res /= 4224) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10138
      if ( res /= 10138) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3970
      if ( res /= 3970) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4167
      if ( res /= 4167) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5877
      if ( res /= 5877) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_Z, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2024
      if ( res /= 2024) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_nu_muAnti, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6200
      if ( res /= 6200) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4520
      if ( res /= 4520) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4214
      if ( res /= 4214) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10353
      if ( res /= 10353) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5585
      if ( res /= 5585) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2360
      if ( res /= 2360) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2245
      if ( res /= 2245) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5096
      if ( res /= 5096) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5690
      if ( res /= 5690) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3744
      if ( res /= 3744) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2599
      if ( res /= 2599) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_Z, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4289
      if ( res /= 4289) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9582
      if ( res /= 9582) then
        stop
      end if
      call createParticleConfig(P_A, P_ghAAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7013
      if ( res /= 7013) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5751
      if ( res /= 5751) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",449
      if ( res /= 449) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9290
      if ( res /= 9290) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_d, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6156
      if ( res /= 6156) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7480
      if ( res /= 7480) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5957
      if ( res /= 5957) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6729
      if ( res /= 6729) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4805
      if ( res /= 4805) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1697
      if ( res /= 1697) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3292
      if ( res /= 3292) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GMinus, P_WPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9848
      if ( res /= 9848) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",956
      if ( res /= 956) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3649
      if ( res /= 3649) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8009
      if ( res /= 8009) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1909
      if ( res /= 1909) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10306
      if ( res /= 10306) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9148
      if ( res /= 9148) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7341
      if ( res /= 7341) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8913
      if ( res /= 8913) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",314
      if ( res /= 314) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3010
      if ( res /= 3010) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6583
      if ( res /= 6583) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5155
      if ( res /= 5155) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8356
      if ( res /= 8356) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9489
      if ( res /= 9489) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9536
      if ( res /= 9536) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",818
      if ( res /= 818) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2602
      if ( res /= 2602) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",841
      if ( res /= 841) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9925
      if ( res /= 9925) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2868
      if ( res /= 2868) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6461
      if ( res /= 6461) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8494
      if ( res /= 8494) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8379
      if ( res /= 8379) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8221
      if ( res /= 8221) then
        stop
      end if
      call createParticleConfig(P_gQ, P_g, P_ghG, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2063
      if ( res /= 2063) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_A, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6201
      if ( res /= 6201) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9377
      if ( res /= 9377) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6909
      if ( res /= 6909) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6566
      if ( res /= 6566) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5524
      if ( res /= 5524) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9702
      if ( res /= 9702) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1630
      if ( res /= 1630) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3492
      if ( res /= 3492) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5423
      if ( res /= 5423) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9606
      if ( res /= 9606) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2012
      if ( res /= 2012) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4001
      if ( res /= 4001) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",209
      if ( res /= 209) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_nu_tauAnti, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2336
      if ( res /= 2336) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5946
      if ( res /= 5946) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3150
      if ( res /= 3150) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8335
      if ( res /= 8335) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9857
      if ( res /= 9857) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5310
      if ( res /= 5310) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5512
      if ( res /= 5512) then
        stop
      end if
      call createParticleConfig(P_t, P_HMinus, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7863
      if ( res /= 7863) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5425
      if ( res /= 5425) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6662
      if ( res /= 6662) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6763
      if ( res /= 6763) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",525
      if ( res /= 525) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8779
      if ( res /= 8779) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4051
      if ( res /= 4051) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4406
      if ( res /= 4406) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7352
      if ( res /= 7352) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6124
      if ( res /= 6124) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9063
      if ( res /= 9063) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5561
      if ( res /= 5561) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7313
      if ( res /= 7313) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghWmAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7737
      if ( res /= 7737) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8792
      if ( res /= 8792) then
        stop
      end if
      call createParticleConfig(P_AQ, P_A, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6311
      if ( res /= 6311) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8123
      if ( res /= 8123) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5920
      if ( res /= 5920) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5612
      if ( res /= 5612) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1283
      if ( res /= 1283) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8780
      if ( res /= 8780) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9216
      if ( res /= 9216) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10302
      if ( res /= 10302) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7623
      if ( res /= 7623) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3375
      if ( res /= 3375) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9494
      if ( res /= 9494) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7053
      if ( res /= 7053) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1893
      if ( res /= 1893) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10393
      if ( res /= 10393) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1984
      if ( res /= 1984) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6920
      if ( res /= 6920) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5435
      if ( res /= 5435) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4636
      if ( res /= 4636) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10170
      if ( res /= 10170) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4263
      if ( res /= 4263) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7142
      if ( res /= 7142) then
        stop
      end if
      call createParticleConfig(P_g, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4652
      if ( res /= 4652) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3591
      if ( res /= 3591) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_d, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8035
      if ( res /= 8035) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8099
      if ( res /= 8099) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5248
      if ( res /= 5248) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8131
      if ( res /= 8131) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9584
      if ( res /= 9584) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8547
      if ( res /= 8547) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3065
      if ( res /= 3065) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3074
      if ( res /= 3074) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5017
      if ( res /= 5017) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7055
      if ( res /= 7055) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9035
      if ( res /= 9035) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3941
      if ( res /= 3941) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WPlus, P_WMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9562
      if ( res /= 9562) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2539
      if ( res /= 2539) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1679
      if ( res /= 1679) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4362
      if ( res /= 4362) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8259
      if ( res /= 8259) then
        stop
      end if
      call createParticleConfig(P_A, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4000
      if ( res /= 4000) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7402
      if ( res /= 7402) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4801
      if ( res /= 4801) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5716
      if ( res /= 5716) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7883
      if ( res /= 7883) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5696
      if ( res /= 5696) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5820
      if ( res /= 5820) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8716
      if ( res /= 8716) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9244
      if ( res /= 9244) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",551
      if ( res /= 551) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9463
      if ( res /= 9463) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1070
      if ( res /= 1070) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3928
      if ( res /= 3928) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5847
      if ( res /= 5847) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5404
      if ( res /= 5404) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4707
      if ( res /= 4707) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9670
      if ( res /= 9670) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9724
      if ( res /= 9724) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1453
      if ( res /= 1453) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5393
      if ( res /= 5393) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5292
      if ( res /= 5292) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghWm, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8674
      if ( res /= 8674) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6788
      if ( res /= 6788) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZ, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3034
      if ( res /= 3034) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9550
      if ( res /= 9550) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6143
      if ( res /= 6143) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_b, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1579
      if ( res /= 1579) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6720
      if ( res /= 6720) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2584
      if ( res /= 2584) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9871
      if ( res /= 9871) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9012
      if ( res /= 9012) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5431
      if ( res /= 5431) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8158
      if ( res /= 8158) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2554
      if ( res /= 2554) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4172
      if ( res /= 4172) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6786
      if ( res /= 6786) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Z, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5995
      if ( res /= 5995) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",635
      if ( res /= 635) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7619
      if ( res /= 7619) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9894
      if ( res /= 9894) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",243
      if ( res /= 243) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1451
      if ( res /= 1451) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6170
      if ( res /= 6170) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6014
      if ( res /= 6014) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4914
      if ( res /= 4914) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1935
      if ( res /= 1935) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WMinus, P_WPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5860
      if ( res /= 5860) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9351
      if ( res /= 9351) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6100
      if ( res /= 6100) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6409
      if ( res /= 6409) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9855
      if ( res /= 9855) then
        stop
      end if
      call createParticleConfig(P_c, P_WMinus, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4790
      if ( res /= 4790) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1631
      if ( res /= 1631) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9705
      if ( res /= 9705) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1557
      if ( res /= 1557) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7533
      if ( res /= 7533) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6189
      if ( res /= 6189) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2368
      if ( res /= 2368) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_Z, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4482
      if ( res /= 4482) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9767
      if ( res /= 9767) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2741
      if ( res /= 2741) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1403
      if ( res /= 1403) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5562
      if ( res /= 5562) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4485
      if ( res /= 4485) then
        stop
      end if
      call createParticleConfig(P_g, P_Hl, P_Hl, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10658
      if ( res /= 10658) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5281
      if ( res /= 5281) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4584
      if ( res /= 4584) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9367
      if ( res /= 9367) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7768
      if ( res /= 7768) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2337
      if ( res /= 2337) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",19
      if ( res /= 19) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2750
      if ( res /= 2750) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3318
      if ( res /= 3318) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GMinus, P_GPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1890
      if ( res /= 1890) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9051
      if ( res /= 9051) then
        stop
      end if
      call createParticleConfig(P_G0, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7640
      if ( res /= 7640) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7571
      if ( res /= 7571) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10178
      if ( res /= 10178) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3337
      if ( res /= 3337) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9891
      if ( res /= 9891) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9847
      if ( res /= 9847) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4960
      if ( res /= 4960) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5497
      if ( res /= 5497) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7337
      if ( res /= 7337) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9690
      if ( res /= 9690) then
        stop
      end if
      call createParticleConfig(P_AQ, P_GMinus, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6546
      if ( res /= 6546) then
        stop
      end if
      call createParticleConfig(P_g, P_HMinus, P_GPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10715
      if ( res /= 10715) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8611
      if ( res /= 8611) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6604
      if ( res /= 6604) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghZ, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8505
      if ( res /= 8505) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1175
      if ( res /= 1175) then
        stop
      end if
      call createParticleConfig(P_ghG, P_g, P_g, P_ghGAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8314
      if ( res /= 8314) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5974
      if ( res /= 5974) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4924
      if ( res /= 4924) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghWm, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7226
      if ( res /= 7226) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",79
      if ( res /= 79) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5219
      if ( res /= 5219) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8982
      if ( res /= 8982) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5632
      if ( res /= 5632) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6966
      if ( res /= 6966) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HPlus, P_ghWm, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9137
      if ( res /= 9137) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2134
      if ( res /= 2134) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",839
      if ( res /= 839) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2891
      if ( res /= 2891) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9762
      if ( res /= 9762) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4254
      if ( res /= 4254) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4532
      if ( res /= 4532) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",182
      if ( res /= 182) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1397
      if ( res /= 1397) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3152
      if ( res /= 3152) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1529
      if ( res /= 1529) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8777
      if ( res /= 8777) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8829
      if ( res /= 8829) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5759
      if ( res /= 5759) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3098
      if ( res /= 3098) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9868
      if ( res /= 9868) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7407
      if ( res /= 7407) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_u, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7687
      if ( res /= 7687) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",916
      if ( res /= 916) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2362
      if ( res /= 2362) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10640
      if ( res /= 10640) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2938
      if ( res /= 2938) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4905
      if ( res /= 4905) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2640
      if ( res /= 2640) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Z, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4767
      if ( res /= 4767) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",341
      if ( res /= 341) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghA, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4223
      if ( res /= 4223) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5179
      if ( res /= 5179) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10525
      if ( res /= 10525) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9598
      if ( res /= 9598) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8125
      if ( res /= 8125) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6389
      if ( res /= 6389) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8577
      if ( res /= 8577) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1254
      if ( res /= 1254) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7004
      if ( res /= 7004) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghWp, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2261
      if ( res /= 2261) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4606
      if ( res /= 4606) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2472
      if ( res /= 2472) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6505
      if ( res /= 6505) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7415
      if ( res /= 7415) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",785
      if ( res /= 785) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8888
      if ( res /= 8888) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10206
      if ( res /= 10206) then
        stop
      end if
      call createParticleConfig(P_AQ, P_G0, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3123
      if ( res /= 3123) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_cAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3125
      if ( res /= 3125) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1478
      if ( res /= 1478) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",237
      if ( res /= 237) then
        stop
      end if
      call createParticleConfig(P_d, P_uAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4849
      if ( res /= 4849) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7363
      if ( res /= 7363) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7550
      if ( res /= 7550) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8877
      if ( res /= 8877) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6197
      if ( res /= 6197) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2478
      if ( res /= 2478) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9007
      if ( res /= 9007) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3285
      if ( res /= 3285) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWp, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3988
      if ( res /= 3988) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",901
      if ( res /= 901) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3216
      if ( res /= 3216) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3825
      if ( res /= 3825) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6860
      if ( res /= 6860) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4537
      if ( res /= 4537) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10354
      if ( res /= 10354) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7060
      if ( res /= 7060) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10575
      if ( res /= 10575) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5864
      if ( res /= 5864) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8633
      if ( res /= 8633) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10210
      if ( res /= 10210) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1396
      if ( res /= 1396) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1467
      if ( res /= 1467) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1682
      if ( res /= 1682) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",987
      if ( res /= 987) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8731
      if ( res /= 8731) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4935
      if ( res /= 4935) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3732
      if ( res /= 3732) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8037
      if ( res /= 8037) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",328
      if ( res /= 328) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1363
      if ( res /= 1363) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6116
      if ( res /= 6116) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6960
      if ( res /= 6960) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8912
      if ( res /= 8912) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8327
      if ( res /= 8327) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9609
      if ( res /= 9609) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10192
      if ( res /= 10192) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4401
      if ( res /= 4401) then
        stop
      end if
      call createParticleConfig(P_t, P_Z, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6046
      if ( res /= 6046) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghAAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5420
      if ( res /= 5420) then
        stop
      end if
      call createParticleConfig(P_G0, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3919
      if ( res /= 3919) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1617
      if ( res /= 1617) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9615
      if ( res /= 9615) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3312
      if ( res /= 3312) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1816
      if ( res /= 1816) then
        stop
      end if
      call createParticleConfig(P_s, P_Z, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4047
      if ( res /= 4047) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7059
      if ( res /= 7059) then
        stop
      end if
      call createParticleConfig(P_Hl, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8053
      if ( res /= 8053) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8733
      if ( res /= 8733) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_WMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3661
      if ( res /= 3661) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",55
      if ( res /= 55) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8122
      if ( res /= 8122) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5597
      if ( res /= 5597) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_Hh, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5761
      if ( res /= 5761) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1927
      if ( res /= 1927) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5736
      if ( res /= 5736) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4065
      if ( res /= 4065) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2272
      if ( res /= 2272) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4536
      if ( res /= 4536) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5838
      if ( res /= 5838) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1548
      if ( res /= 1548) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7938
      if ( res /= 7938) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5487
      if ( res /= 5487) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghA, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3997
      if ( res /= 3997) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9184
      if ( res /= 9184) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6924
      if ( res /= 6924) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5160
      if ( res /= 5160) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3861
      if ( res /= 3861) then
        stop
      end if
      call createParticleConfig(P_G0, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",71
      if ( res /= 71) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",650
      if ( res /= 650) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7455
      if ( res /= 7455) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2757
      if ( res /= 2757) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8078
      if ( res /= 8078) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5303
      if ( res /= 5303) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5575
      if ( res /= 5575) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1224
      if ( res /= 1224) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1598
      if ( res /= 1598) then
        stop
      end if
      call createParticleConfig(P_Ha, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7088
      if ( res /= 7088) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2991
      if ( res /= 2991) then
        stop
      end if
      call createParticleConfig(P_Ha, P_g, P_G0, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10724
      if ( res /= 10724) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_nu_tauAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10385
      if ( res /= 10385) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",231
      if ( res /= 231) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3953
      if ( res /= 3953) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GMinus, P_GPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7424
      if ( res /= 7424) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1142
      if ( res /= 1142) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4311
      if ( res /= 4311) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8947
      if ( res /= 8947) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2696
      if ( res /= 2696) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5144
      if ( res /= 5144) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8568
      if ( res /= 8568) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6793
      if ( res /= 6793) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9279
      if ( res /= 9279) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5665
      if ( res /= 5665) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4237
      if ( res /= 4237) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_eMinus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2978
      if ( res /= 2978) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5550
      if ( res /= 5550) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",896
      if ( res /= 896) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4832
      if ( res /= 4832) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9057
      if ( res /= 9057) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2357
      if ( res /= 2357) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9501
      if ( res /= 9501) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10240
      if ( res /= 10240) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1473
      if ( res /= 1473) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4003
      if ( res /= 4003) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9654
      if ( res /= 9654) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3525
      if ( res /= 3525) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7301
      if ( res /= 7301) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8689
      if ( res /= 8689) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",876
      if ( res /= 876) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8752
      if ( res /= 8752) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7259
      if ( res /= 7259) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4685
      if ( res /= 4685) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9093
      if ( res /= 9093) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3025
      if ( res /= 3025) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1570
      if ( res /= 1570) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9364
      if ( res /= 9364) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5489
      if ( res /= 5489) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_g, P_WMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10661
      if ( res /= 10661) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7945
      if ( res /= 7945) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1883
      if ( res /= 1883) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_nu_tauAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1122
      if ( res /= 1122) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9500
      if ( res /= 9500) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1279
      if ( res /= 1279) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3442
      if ( res /= 3442) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Z, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1646
      if ( res /= 1646) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6938
      if ( res /= 6938) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9548
      if ( res /= 9548) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1588
      if ( res /= 1588) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghWpAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4335
      if ( res /= 4335) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZ, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6802
      if ( res /= 6802) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7679
      if ( res /= 7679) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2170
      if ( res /= 2170) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8031
      if ( res /= 8031) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8908
      if ( res /= 8908) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",550
      if ( res /= 550) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",165
      if ( res /= 165) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_WPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7558
      if ( res /= 7558) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2704
      if ( res /= 2704) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9122
      if ( res /= 9122) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1561
      if ( res /= 1561) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HMinus, P_HPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7881
      if ( res /= 7881) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5605
      if ( res /= 5605) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",204
      if ( res /= 204) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_Hh, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4192
      if ( res /= 4192) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",422
      if ( res /= 422) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9614
      if ( res /= 9614) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7312
      if ( res /= 7312) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",419
      if ( res /= 419) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8010
      if ( res /= 8010) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7643
      if ( res /= 7643) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5379
      if ( res /= 5379) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",39
      if ( res /= 39) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWm, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5321
      if ( res /= 5321) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2222
      if ( res /= 2222) then
        stop
      end if
      call createParticleConfig(P_AQ, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6988
      if ( res /= 6988) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7579
      if ( res /= 7579) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8196
      if ( res /= 8196) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9396
      if ( res /= 9396) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6590
      if ( res /= 6590) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2230
      if ( res /= 2230) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9787
      if ( res /= 9787) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6704
      if ( res /= 6704) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9038
      if ( res /= 9038) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8563
      if ( res /= 8563) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",133
      if ( res /= 133) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6470
      if ( res /= 6470) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2091
      if ( res /= 2091) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10013
      if ( res /= 10013) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_nu_tau, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",396
      if ( res /= 396) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_nu_eAnti, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8554
      if ( res /= 8554) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1407
      if ( res /= 1407) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_d, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5798
      if ( res /= 5798) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7309
      if ( res /= 7309) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8944
      if ( res /= 8944) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghWm, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5595
      if ( res /= 5595) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9979
      if ( res /= 9979) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4963
      if ( res /= 4963) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9780
      if ( res /= 9780) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1648
      if ( res /= 1648) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4851
      if ( res /= 4851) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6804
      if ( res /= 6804) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6221
      if ( res /= 6221) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2971
      if ( res /= 2971) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8194
      if ( res /= 8194) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_b, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5902
      if ( res /= 5902) then
        stop
      end if
      call createParticleConfig(P_g, P_HPlus, P_HMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10773
      if ( res /= 10773) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4847
      if ( res /= 4847) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7854
      if ( res /= 7854) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9924
      if ( res /= 9924) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1580
      if ( res /= 1580) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10273
      if ( res /= 10273) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10391
      if ( res /= 10391) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghWm, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3165
      if ( res /= 3165) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10284
      if ( res /= 10284) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_Z, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4292
      if ( res /= 4292) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hl, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7886
      if ( res /= 7886) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",632
      if ( res /= 632) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10461
      if ( res /= 10461) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3623
      if ( res /= 3623) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghZAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3832
      if ( res /= 3832) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9000
      if ( res /= 9000) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1895
      if ( res /= 1895) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3697
      if ( res /= 3697) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10469
      if ( res /= 10469) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4458
      if ( res /= 4458) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2016
      if ( res /= 2016) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7686
      if ( res /= 7686) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9694
      if ( res /= 9694) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8960
      if ( res /= 8960) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9207
      if ( res /= 9207) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7559
      if ( res /= 7559) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3491
      if ( res /= 3491) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5553
      if ( res /= 5553) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6332
      if ( res /= 6332) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3301
      if ( res /= 3301) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2083
      if ( res /= 2083) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3801
      if ( res /= 3801) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_A, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3701
      if ( res /= 3701) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghZ, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1342
      if ( res /= 1342) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",265
      if ( res /= 265) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1183
      if ( res /= 1183) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6381
      if ( res /= 6381) then
        stop
      end if
      call createParticleConfig(P_g, P_Z, P_Z, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10664
      if ( res /= 10664) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7283
      if ( res /= 7283) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",941
      if ( res /= 941) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4976
      if ( res /= 4976) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_nu_tauAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",281
      if ( res /= 281) then
        stop
      end if
      call createParticleConfig(P_s, P_HPlus, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7545
      if ( res /= 7545) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",765
      if ( res /= 765) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",121
      if ( res /= 121) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9551
      if ( res /= 9551) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9240
      if ( res /= 9240) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3469
      if ( res /= 3469) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8663
      if ( res /= 8663) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5774
      if ( res /= 5774) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghA, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7576
      if ( res /= 7576) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2658
      if ( res /= 2658) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghA, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1839
      if ( res /= 1839) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghWp, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6267
      if ( res /= 6267) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghZ, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4203
      if ( res /= 4203) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6222
      if ( res /= 6222) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",13
      if ( res /= 13) then
        stop
      end if
      call createParticleConfig(P_G0, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6411
      if ( res /= 6411) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1156
      if ( res /= 1156) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1692
      if ( res /= 1692) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1722
      if ( res /= 1722) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2641
      if ( res /= 2641) then
        stop
      end if
      call createParticleConfig(P_s, P_GPlus, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",625
      if ( res /= 625) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8697
      if ( res /= 8697) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10093
      if ( res /= 10093) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5511
      if ( res /= 5511) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10226
      if ( res /= 10226) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3960
      if ( res /= 3960) then
        stop
      end if
      call createParticleConfig(P_c, P_sAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7735
      if ( res /= 7735) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1443
      if ( res /= 1443) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4156
      if ( res /= 4156) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9915
      if ( res /= 9915) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",752
      if ( res /= 752) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWp, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1223
      if ( res /= 1223) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9793
      if ( res /= 9793) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10590
      if ( res /= 10590) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9157
      if ( res /= 9157) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9581
      if ( res /= 9581) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5679
      if ( res /= 5679) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_s, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7563
      if ( res /= 7563) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4507
      if ( res /= 4507) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",657
      if ( res /= 657) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6255
      if ( res /= 6255) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5936
      if ( res /= 5936) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2229
      if ( res /= 2229) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WMinus, P_GPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5216
      if ( res /= 5216) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7855
      if ( res /= 7855) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8962
      if ( res /= 8962) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1647
      if ( res /= 1647) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9227
      if ( res /= 9227) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9211
      if ( res /= 9211) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2403
      if ( res /= 2403) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",583
      if ( res /= 583) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8988
      if ( res /= 8988) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1599
      if ( res /= 1599) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1678
      if ( res /= 1678) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4488
      if ( res /= 4488) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4242
      if ( res /= 4242) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",418
      if ( res /= 418) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3524
      if ( res /= 3524) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4460
      if ( res /= 4460) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4667
      if ( res /= 4667) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWp, P_ghWpAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1087
      if ( res /= 1087) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1481
      if ( res /= 1481) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1979
      if ( res /= 1979) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6314
      if ( res /= 6314) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4605
      if ( res /= 4605) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10223
      if ( res /= 10223) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10408
      if ( res /= 10408) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10749
      if ( res /= 10749) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10268
      if ( res /= 10268) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4742
      if ( res /= 4742) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9944
      if ( res /= 9944) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7347
      if ( res /= 7347) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6709
      if ( res /= 6709) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4542
      if ( res /= 4542) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8667
      if ( res /= 8667) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2321
      if ( res /= 2321) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8747
      if ( res /= 8747) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3064
      if ( res /= 3064) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghA, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10207
      if ( res /= 10207) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1432
      if ( res /= 1432) then
        stop
      end if
      call createParticleConfig(P_G0, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",634
      if ( res /= 634) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10247
      if ( res /= 10247) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2687
      if ( res /= 2687) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Ha, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9161
      if ( res /= 9161) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6300
      if ( res /= 6300) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8128
      if ( res /= 8128) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Z, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4246
      if ( res /= 4246) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4613
      if ( res /= 4613) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9299
      if ( res /= 9299) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4546
      if ( res /= 4546) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7955
      if ( res /= 7955) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1234
      if ( res /= 1234) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3206
      if ( res /= 3206) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",95
      if ( res /= 95) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_Z, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7475
      if ( res /= 7475) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6443
      if ( res /= 6443) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2313
      if ( res /= 2313) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",355
      if ( res /= 355) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7050
      if ( res /= 7050) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2715
      if ( res /= 2715) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9110
      if ( res /= 9110) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5374
      if ( res /= 5374) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4096
      if ( res /= 4096) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1723
      if ( res /= 1723) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6284
      if ( res /= 6284) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",86
      if ( res /= 86) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7803
      if ( res /= 7803) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",658
      if ( res /= 658) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4910
      if ( res /= 4910) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghZAnti, P_G0, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7928
      if ( res /= 7928) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5098
      if ( res /= 5098) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6947
      if ( res /= 6947) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5861
      if ( res /= 5861) then
        stop
      end if
      call createParticleConfig(P_Hl, P_G0, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9556
      if ( res /= 9556) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9607
      if ( res /= 9607) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3795
      if ( res /= 3795) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",741
      if ( res /= 741) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7235
      if ( res /= 7235) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",718
      if ( res /= 718) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10064
      if ( res /= 10064) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1688
      if ( res /= 1688) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9472
      if ( res /= 9472) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2034
      if ( res /= 2034) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6880
      if ( res /= 6880) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZ, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7356
      if ( res /= 7356) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_A, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",149
      if ( res /= 149) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2500
      if ( res /= 2500) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10674
      if ( res /= 10674) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1080
      if ( res /= 1080) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4039
      if ( res /= 4039) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HPlus, P_HMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4673
      if ( res /= 4673) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1060
      if ( res /= 1060) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3510
      if ( res /= 3510) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",273
      if ( res /= 273) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GMinus, P_ghWmAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8513
      if ( res /= 8513) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6447
      if ( res /= 6447) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",228
      if ( res /= 228) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8686
      if ( res /= 8686) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2965
      if ( res /= 2965) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4486
      if ( res /= 4486) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghZ, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1981
      if ( res /= 1981) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_nu_e, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7669
      if ( res /= 7669) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7865
      if ( res /= 7865) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6344
      if ( res /= 6344) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3185
      if ( res /= 3185) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghZ, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3963
      if ( res /= 3963) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8958
      if ( res /= 8958) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3770
      if ( res /= 3770) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4174
      if ( res /= 4174) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6263
      if ( res /= 6263) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4083
      if ( res /= 4083) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8179
      if ( res /= 8179) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8507
      if ( res /= 8507) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3298
      if ( res /= 3298) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2221
      if ( res /= 2221) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_nu_eAnti, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2366
      if ( res /= 2366) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7875
      if ( res /= 7875) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghA, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1066
      if ( res /= 1066) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GPlus, P_HMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2797
      if ( res /= 2797) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWmAnti, P_ghWm, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3635
      if ( res /= 3635) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7908
      if ( res /= 7908) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",432
      if ( res /= 432) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9685
      if ( res /= 9685) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10473
      if ( res /= 10473) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10108
      if ( res /= 10108) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3813
      if ( res /= 3813) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",251
      if ( res /= 251) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8856
      if ( res /= 8856) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10624
      if ( res /= 10624) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5530
      if ( res /= 5530) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9214
      if ( res /= 9214) then
        stop
      end if
      call createParticleConfig(P_d, P_g, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3052
      if ( res /= 3052) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4986
      if ( res /= 4986) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2492
      if ( res /= 2492) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1181
      if ( res /= 1181) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5237
      if ( res /= 5237) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3053
      if ( res /= 3053) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",423
      if ( res /= 423) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8585
      if ( res /= 8585) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_s, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6950
      if ( res /= 6950) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2456
      if ( res /= 2456) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5680
      if ( res /= 5680) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1380
      if ( res /= 1380) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1807
      if ( res /= 1807) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9499
      if ( res /= 9499) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8214
      if ( res /= 8214) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1422
      if ( res /= 1422) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9876
      if ( res /= 9876) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_nu_eAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2061
      if ( res /= 2061) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3535
      if ( res /= 3535) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",714
      if ( res /= 714) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5607
      if ( res /= 5607) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9506
      if ( res /= 9506) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3823
      if ( res /= 3823) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1540
      if ( res /= 1540) then
        stop
      end if
      call createParticleConfig(P_t, P_bAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7972
      if ( res /= 7972) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10700
      if ( res /= 10700) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WPlus, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1389
      if ( res /= 1389) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4294
      if ( res /= 4294) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWmAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4750
      if ( res /= 4750) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2101
      if ( res /= 2101) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghAAnti, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7102
      if ( res /= 7102) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9680
      if ( res /= 9680) then
        stop
      end if
      call createParticleConfig(P_ghA, P_A, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2011
      if ( res /= 2011) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",29
      if ( res /= 29) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7244
      if ( res /= 7244) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3060
      if ( res /= 3060) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7671
      if ( res /= 7671) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_t, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5254
      if ( res /= 5254) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9744
      if ( res /= 9744) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8046
      if ( res /= 8046) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6400
      if ( res /= 6400) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1321
      if ( res /= 1321) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3260
      if ( res /= 3260) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1714
      if ( res /= 1714) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9994
      if ( res /= 9994) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9673
      if ( res /= 9673) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",548
      if ( res /= 548) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5371
      if ( res /= 5371) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3930
      if ( res /= 3930) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3196
      if ( res /= 3196) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5802
      if ( res /= 5802) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9032
      if ( res /= 9032) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7925
      if ( res /= 7925) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3793
      if ( res /= 3793) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5532
      if ( res /= 5532) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6620
      if ( res /= 6620) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5846
      if ( res /= 5846) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9778
      if ( res /= 9778) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",430
      if ( res /= 430) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7912
      if ( res /= 7912) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5959
      if ( res /= 5959) then
        stop
      end if
      call createParticleConfig(P_gQ, P_ghGAnti, P_g, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5738
      if ( res /= 5738) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9842
      if ( res /= 9842) then
        stop
      end if
      call createParticleConfig(P_AQ, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3891
      if ( res /= 3891) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",647
      if ( res /= 647) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8084
      if ( res /= 8084) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8657
      if ( res /= 8657) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2355
      if ( res /= 2355) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9021
      if ( res /= 9021) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4681
      if ( res /= 4681) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghA, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8018
      if ( res /= 8018) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWpAnti, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1068
      if ( res /= 1068) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7165
      if ( res /= 7165) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6797
      if ( res /= 6797) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9343
      if ( res /= 9343) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5360
      if ( res /= 5360) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_ghG, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4701
      if ( res /= 4701) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3015
      if ( res /= 3015) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2494
      if ( res /= 2494) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4010
      if ( res /= 4010) then
        stop
      end if
      call createParticleConfig(P_gQ, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3460
      if ( res /= 3460) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5872
      if ( res /= 5872) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWpAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2726
      if ( res /= 2726) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",991
      if ( res /= 991) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWm, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1916
      if ( res /= 1916) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10283
      if ( res /= 10283) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_A, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4816
      if ( res /= 4816) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4761
      if ( res /= 4761) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_A, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1628
      if ( res /= 1628) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6700
      if ( res /= 6700) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1872
      if ( res /= 1872) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9124
      if ( res /= 9124) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9874
      if ( res /= 9874) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1409
      if ( res /= 1409) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2097
      if ( res /= 2097) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9128
      if ( res /= 9128) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9875
      if ( res /= 9875) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1841
      if ( res /= 1841) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8632
      if ( res /= 8632) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10256
      if ( res /= 10256) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWmAnti, P_GMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7535
      if ( res /= 7535) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GPlus, P_WMinus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2199
      if ( res /= 2199) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5266
      if ( res /= 5266) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9325
      if ( res /= 9325) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9138
      if ( res /= 9138) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4419
      if ( res /= 4419) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3955
      if ( res /= 3955) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8940
      if ( res /= 8940) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8177
      if ( res /= 8177) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1447
      if ( res /= 1447) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1353
      if ( res /= 1353) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9358
      if ( res /= 9358) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7998
      if ( res /= 7998) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1049
      if ( res /= 1049) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",372
      if ( res /= 372) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghZAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1416
      if ( res /= 1416) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GPlus, P_GMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2464
      if ( res /= 2464) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3146
      if ( res /= 3146) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5874
      if ( res /= 5874) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7246
      if ( res /= 7246) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghA, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6877
      if ( res /= 6877) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9020
      if ( res /= 9020) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3480
      if ( res /= 3480) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8644
      if ( res /= 8644) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9587
      if ( res /= 9587) then
        stop
      end if
      call createParticleConfig(P_Hh, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",875
      if ( res /= 875) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10645
      if ( res /= 10645) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2824
      if ( res /= 2824) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3614
      if ( res /= 3614) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8718
      if ( res /= 8718) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_Z, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2912
      if ( res /= 2912) then
        stop
      end if
      call createParticleConfig(P_Ha, P_g, P_Ha, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10573
      if ( res /= 10573) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5180
      if ( res /= 5180) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_ghA, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",971
      if ( res /= 971) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7916
      if ( res /= 7916) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghA, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",326
      if ( res /= 326) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9210
      if ( res /= 9210) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9933
      if ( res /= 9933) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4447
      if ( res /= 4447) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",0
      if ( res /= 0) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4672
      if ( res /= 4672) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3279
      if ( res /= 3279) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8019
      if ( res /= 8019) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8415
      if ( res /= 8415) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",557
      if ( res /= 557) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8746
      if ( res /= 8746) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6328
      if ( res /= 6328) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1487
      if ( res /= 1487) then
        stop
      end if
      call createParticleConfig(P_Ha, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2305
      if ( res /= 2305) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8346
      if ( res /= 8346) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",491
      if ( res /= 491) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8465
      if ( res /= 8465) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4608
      if ( res /= 4608) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8291
      if ( res /= 8291) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5060
      if ( res /= 5060) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9950
      if ( res /= 9950) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7463
      if ( res /= 7463) then
        stop
      end if
      call createParticleConfig(P_gQ, P_g, P_g, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7792
      if ( res /= 7792) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6007
      if ( res /= 6007) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_GMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5253
      if ( res /= 5253) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5637
      if ( res /= 5637) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2985
      if ( res /= 2985) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3893
      if ( res /= 3893) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10032
      if ( res /= 10032) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8313
      if ( res /= 8313) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8574
      if ( res /= 8574) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10384
      if ( res /= 10384) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_nu_eAnti, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3069
      if ( res /= 3069) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4050
      if ( res /= 4050) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5515
      if ( res /= 5515) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4205
      if ( res /= 4205) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3561
      if ( res /= 3561) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8416
      if ( res /= 8416) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9113
      if ( res /= 9113) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5599
      if ( res /= 5599) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3410
      if ( res /= 3410) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4433
      if ( res /= 4433) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4385
      if ( res /= 4385) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7618
      if ( res /= 7618) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4659
      if ( res /= 4659) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6298
      if ( res /= 6298) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2813
      if ( res /= 2813) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6525
      if ( res /= 6525) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5812
      if ( res /= 5812) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4334
      if ( res /= 4334) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3014
      if ( res /= 3014) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6533
      if ( res /= 6533) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5440
      if ( res /= 5440) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2571
      if ( res /= 2571) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1235
      if ( res /= 1235) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_tauMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1777
      if ( res /= 1777) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5403
      if ( res /= 5403) then
        stop
      end if
      call createParticleConfig(P_u, P_g, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1519
      if ( res /= 1519) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1919
      if ( res /= 1919) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7091
      if ( res /= 7091) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9819
      if ( res /= 9819) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5654
      if ( res /= 5654) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9637
      if ( res /= 9637) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7611
      if ( res /= 7611) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_muPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7191
      if ( res /= 7191) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghA, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5955
      if ( res /= 5955) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8435
      if ( res /= 8435) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1988
      if ( res /= 1988) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3161
      if ( res /= 3161) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2901
      if ( res /= 2901) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8373
      if ( res /= 8373) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3987
      if ( res /= 3987) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghZ, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4212
      if ( res /= 4212) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10091
      if ( res /= 10091) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2074
      if ( res /= 2074) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6230
      if ( res /= 6230) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10056
      if ( res /= 10056) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5903
      if ( res /= 5903) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2666
      if ( res /= 2666) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5020
      if ( res /= 5020) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6317
      if ( res /= 6317) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5078
      if ( res /= 5078) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4967
      if ( res /= 4967) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GPlus, P_ghWm, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9841
      if ( res /= 9841) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_HMinus, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8191
      if ( res /= 8191) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghA, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3663
      if ( res /= 3663) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10075
      if ( res /= 10075) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5373
      if ( res /= 5373) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWmAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2892
      if ( res /= 2892) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",266
      if ( res /= 266) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8166
      if ( res /= 8166) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8282
      if ( res /= 8282) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2039
      if ( res /= 2039) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2880
      if ( res /= 2880) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4998
      if ( res /= 4998) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",472
      if ( res /= 472) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1475
      if ( res /= 1475) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8951
      if ( res /= 8951) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9728
      if ( res /= 9728) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4134
      if ( res /= 4134) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9935
      if ( res /= 9935) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7338
      if ( res /= 7338) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2621
      if ( res /= 2621) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWpAnti, P_WPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5327
      if ( res /= 5327) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5426
      if ( res /= 5426) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9769
      if ( res /= 9769) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6105
      if ( res /= 6105) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1251
      if ( res /= 1251) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3303
      if ( res /= 3303) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4547
      if ( res /= 4547) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5328
      if ( res /= 5328) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4714
      if ( res /= 4714) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4340
      if ( res /= 4340) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8174
      if ( res /= 8174) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4896
      if ( res /= 4896) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1523
      if ( res /= 1523) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10297
      if ( res /= 10297) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWp, P_WMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6408
      if ( res /= 6408) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghWmAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2597
      if ( res /= 2597) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_u, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2334
      if ( res /= 2334) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1613
      if ( res /= 1613) then
        stop
      end if
      call createParticleConfig(P_Z, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",710
      if ( res /= 710) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6739
      if ( res /= 6739) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9837
      if ( res /= 9837) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",508
      if ( res /= 508) then
        stop
      end if
      call createParticleConfig(P_Hl, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5634
      if ( res /= 5634) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_uAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7578
      if ( res /= 7578) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10440
      if ( res /= 10440) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6852
      if ( res /= 6852) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5428
      if ( res /= 5428) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghA, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6351
      if ( res /= 6351) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2122
      if ( res /= 2122) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1063
      if ( res /= 1063) then
        stop
      end if
      call createParticleConfig(P_gQ, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1950
      if ( res /= 1950) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5924
      if ( res /= 5924) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9722
      if ( res /= 9722) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7689
      if ( res /= 7689) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6498
      if ( res /= 6498) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWpAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",245
      if ( res /= 245) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8571
      if ( res /= 8571) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_t, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",511
      if ( res /= 511) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",722
      if ( res /= 722) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6079
      if ( res /= 6079) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4299
      if ( res /= 4299) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1482
      if ( res /= 1482) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7067
      if ( res /= 7067) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2654
      if ( res /= 2654) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9008
      if ( res /= 9008) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_nu_eAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7251
      if ( res /= 7251) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1273
      if ( res /= 1273) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3569
      if ( res /= 3569) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_ghA, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5456
      if ( res /= 5456) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1136
      if ( res /= 1136) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1404
      if ( res /= 1404) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7568
      if ( res /= 7568) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1643
      if ( res /= 1643) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",616
      if ( res /= 616) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10521
      if ( res /= 10521) then
        stop
      end if
      call createParticleConfig(P_s, P_A, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",774
      if ( res /= 774) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4689
      if ( res /= 4689) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",931
      if ( res /= 931) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8486
      if ( res /= 8486) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10035
      if ( res /= 10035) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1323
      if ( res /= 1323) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10300
      if ( res /= 10300) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWp, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",158
      if ( res /= 158) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8564
      if ( res /= 8564) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2783
      if ( res /= 2783) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",868
      if ( res /= 868) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",555
      if ( res /= 555) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Ha, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2220
      if ( res /= 2220) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6573
      if ( res /= 6573) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWmAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10068
      if ( res /= 10068) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7185
      if ( res /= 7185) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4949
      if ( res /= 4949) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_WMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8798
      if ( res /= 8798) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8298
      if ( res /= 8298) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5384
      if ( res /= 5384) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10458
      if ( res /= 10458) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",819
      if ( res /= 819) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10619
      if ( res /= 10619) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9193
      if ( res /= 9193) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_G0, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4451
      if ( res /= 4451) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7369
      if ( res /= 7369) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3382
      if ( res /= 3382) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hh, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4620
      if ( res /= 4620) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",296
      if ( res /= 296) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1148
      if ( res /= 1148) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",244
      if ( res /= 244) then
        stop
      end if
      call createParticleConfig(P_AQ, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2839
      if ( res /= 2839) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_GPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1593
      if ( res /= 1593) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6697
      if ( res /= 6697) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10356
      if ( res /= 10356) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7266
      if ( res /= 7266) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3400
      if ( res /= 3400) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1681
      if ( res /= 1681) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",516
      if ( res /= 516) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8244
      if ( res /= 8244) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7709
      if ( res /= 7709) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5620
      if ( res /= 5620) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",222
      if ( res /= 222) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_Z, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1732
      if ( res /= 1732) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7086
      if ( res /= 7086) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10508
      if ( res /= 10508) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6483
      if ( res /= 6483) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6724
      if ( res /= 6724) then
        stop
      end if
      call createParticleConfig(P_g, P_GMinus, P_GPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10551
      if ( res /= 10551) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8841
      if ( res /= 8841) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",156
      if ( res /= 156) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10394
      if ( res /= 10394) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5083
      if ( res /= 5083) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10453
      if ( res /= 10453) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1940
      if ( res /= 1940) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2798
      if ( res /= 2798) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2923
      if ( res /= 2923) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3952
      if ( res /= 3952) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8849
      if ( res /= 8849) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10126
      if ( res /= 10126) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1019
      if ( res /= 1019) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",401
      if ( res /= 401) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10557
      if ( res /= 10557) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4002
      if ( res /= 4002) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10553
      if ( res /= 10553) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",603
      if ( res /= 603) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8428
      if ( res /= 8428) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3618
      if ( res /= 3618) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2007
      if ( res /= 2007) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7892
      if ( res /= 7892) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8961
      if ( res /= 8961) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghA, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6570
      if ( res /= 6570) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2764
      if ( res /= 2764) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",286
      if ( res /= 286) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3877
      if ( res /= 3877) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",438
      if ( res /= 438) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2957
      if ( res /= 2957) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7241
      if ( res /= 7241) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1925
      if ( res /= 1925) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10208
      if ( res /= 10208) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6881
      if ( res /= 6881) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_ghWp, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2636
      if ( res /= 2636) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3624
      if ( res /= 3624) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5990
      if ( res /= 5990) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2276
      if ( res /= 2276) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2802
      if ( res /= 2802) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HMinus, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9338
      if ( res /= 9338) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3387
      if ( res /= 3387) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3945
      if ( res /= 3945) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_eMinus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1865
      if ( res /= 1865) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10402
      if ( res /= 10402) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_g, P_ghG, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5480
      if ( res /= 5480) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7414
      if ( res /= 7414) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1954
      if ( res /= 1954) then
        stop
      end if
      call createParticleConfig(P_s, P_cAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6630
      if ( res /= 6630) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HPlus, P_ghWpAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",306
      if ( res /= 306) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4782
      if ( res /= 4782) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7927
      if ( res /= 7927) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1982
      if ( res /= 1982) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5750
      if ( res /= 5750) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",720
      if ( res /= 720) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",790
      if ( res /= 790) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9304
      if ( res /= 9304) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8364
      if ( res /= 8364) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4268
      if ( res /= 4268) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2194
      if ( res /= 2194) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2921
      if ( res /= 2921) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2760
      if ( res /= 2760) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_c, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4679
      if ( res /= 4679) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8623
      if ( res /= 8623) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7239
      if ( res /= 7239) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Hh, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",771
      if ( res /= 771) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6194
      if ( res /= 6194) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7890
      if ( res /= 7890) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9121
      if ( res /= 9121) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10731
      if ( res /= 10731) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",755
      if ( res /= 755) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9288
      if ( res /= 9288) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_A, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",141
      if ( res /= 141) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Z, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3416
      if ( res /= 3416) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5980
      if ( res /= 5980) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8573
      if ( res /= 8573) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10374
      if ( res /= 10374) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4448
      if ( res /= 4448) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2113
      if ( res /= 2113) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2463
      if ( res /= 2463) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7847
      if ( res /= 7847) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghA, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8355
      if ( res /= 8355) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1365
      if ( res /= 1365) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3545
      if ( res /= 3545) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6442
      if ( res /= 6442) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8437
      if ( res /= 8437) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWpAnti, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2788
      if ( res /= 2788) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",168
      if ( res /= 168) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_A, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9838
      if ( res /= 9838) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8539
      if ( res /= 8539) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7358
      if ( res /= 7358) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4270
      if ( res /= 4270) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_ghA, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6049
      if ( res /= 6049) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10146
      if ( res /= 10146) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8234
      if ( res /= 8234) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5406
      if ( res /= 5406) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1512
      if ( res /= 1512) then
        stop
      end if
      call createParticleConfig(P_G0, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6176
      if ( res /= 6176) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5064
      if ( res /= 5064) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_nu_tauAnti, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",177
      if ( res /= 177) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3372
      if ( res /= 3372) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_HMinus, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",203
      if ( res /= 203) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6203
      if ( res /= 6203) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5441
      if ( res /= 5441) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",680
      if ( res /= 680) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghWpAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4514
      if ( res /= 4514) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3393
      if ( res /= 3393) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWm, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7401
      if ( res /= 7401) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3704
      if ( res /= 3704) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_tauPlus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3284
      if ( res /= 3284) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1932
      if ( res /= 1932) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_Ha, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3401
      if ( res /= 3401) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghA, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",813
      if ( res /= 813) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2107
      if ( res /= 2107) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2043
      if ( res /= 2043) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4303
      if ( res /= 4303) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1247
      if ( res /= 1247) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8470
      if ( res /= 8470) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6815
      if ( res /= 6815) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3799
      if ( res /= 3799) then
        stop
      end if
      call createParticleConfig(P_Z, P_b, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",865
      if ( res /= 865) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6775
      if ( res /= 6775) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWmAnti, P_HMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9189
      if ( res /= 9189) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7590
      if ( res /= 7590) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8618
      if ( res /= 8618) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WPlus, P_ghWm, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5476
      if ( res /= 5476) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1933
      if ( res /= 1933) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8590
      if ( res /= 8590) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_tAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1670
      if ( res /= 1670) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4122
      if ( res /= 4122) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1272
      if ( res /= 1272) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7771
      if ( res /= 7771) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_ghWmAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",299
      if ( res /= 299) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2219
      if ( res /= 2219) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4007
      if ( res /= 4007) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10397
      if ( res /= 10397) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2532
      if ( res /= 2532) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4752
      if ( res /= 4752) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4013
      if ( res /= 4013) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5200
      if ( res /= 5200) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3323
      if ( res /= 3323) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1536
      if ( res /= 1536) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2385
      if ( res /= 2385) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7660
      if ( res /= 7660) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_A, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3202
      if ( res /= 3202) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7549
      if ( res /= 7549) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10454
      if ( res /= 10454) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5863
      if ( res /= 5863) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7123
      if ( res /= 7123) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6168
      if ( res /= 6168) then
        stop
      end if
      call createParticleConfig(P_Z, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",368
      if ( res /= 368) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1173
      if ( res /= 1173) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5058
      if ( res /= 5058) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4817
      if ( res /= 4817) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6622
      if ( res /= 6622) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9243
      if ( res /= 9243) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2431
      if ( res /= 2431) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghAAnti, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8357
      if ( res /= 8357) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1081
      if ( res /= 1081) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5517
      if ( res /= 5517) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1252
      if ( res /= 1252) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5478
      if ( res /= 5478) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10584
      if ( res /= 10584) then
        stop
      end if
      call createParticleConfig(P_Z, P_g, P_g, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10745
      if ( res /= 10745) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6918
      if ( res /= 6918) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1862
      if ( res /= 1862) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7742
      if ( res /= 7742) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghWpAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7006
      if ( res /= 7006) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghWmAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8280
      if ( res /= 8280) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8096
      if ( res /= 8096) then
        stop
      end if
      call createParticleConfig(P_g, P_ghG, P_ghGAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1652
      if ( res /= 1652) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1607
      if ( res /= 1607) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2324
      if ( res /= 2324) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4698
      if ( res /= 4698) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2284
      if ( res /= 2284) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_c, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7213
      if ( res /= 7213) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3427
      if ( res /= 3427) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7125
      if ( res /= 7125) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3091
      if ( res /= 3091) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8521
      if ( res /= 8521) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9455
      if ( res /= 9455) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5276
      if ( res /= 5276) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8504
      if ( res /= 8504) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9900
      if ( res /= 9900) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7454
      if ( res /= 7454) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_A, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10713
      if ( res /= 10713) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5904
      if ( res /= 5904) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6936
      if ( res /= 6936) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10333
      if ( res /= 10333) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3508
      if ( res /= 3508) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",688
      if ( res /= 688) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5336
      if ( res /= 5336) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5660
      if ( res /= 5660) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3027
      if ( res /= 3027) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1477
      if ( res /= 1477) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6934
      if ( res /= 6934) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWm, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4440
      if ( res /= 4440) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10348
      if ( res /= 10348) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1609
      if ( res /= 1609) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4765
      if ( res /= 4765) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_eMinus, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3648
      if ( res /= 3648) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10654
      if ( res /= 10654) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9801
      if ( res /= 9801) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9860
      if ( res /= 9860) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5301
      if ( res /= 5301) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWp, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5786
      if ( res /= 5786) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4493
      if ( res /= 4493) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6336
      if ( res /= 6336) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9681
      if ( res /= 9681) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5347
      if ( res /= 5347) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6817
      if ( res /= 6817) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2523
      if ( res /= 2523) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5185
      if ( res /= 5185) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8837
      if ( res /= 8837) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghZ, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3741
      if ( res /= 3741) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2718
      if ( res /= 2718) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WPlus, P_GMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3300
      if ( res /= 3300) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6379
      if ( res /= 6379) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6376
      if ( res /= 6376) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hh, P_ghWp, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1944
      if ( res /= 1944) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10475
      if ( res /= 10475) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_WPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2470
      if ( res /= 2470) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_nu_tau, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",457
      if ( res /= 457) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",917
      if ( res /= 917) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_d, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7476
      if ( res /= 7476) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWpAnti, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8688
      if ( res /= 8688) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9275
      if ( res /= 9275) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HPlus, P_WMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10363
      if ( res /= 10363) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9773
      if ( res /= 9773) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2499
      if ( res /= 2499) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4391
      if ( res /= 4391) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1611
      if ( res /= 1611) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1821
      if ( res /= 1821) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9784
      if ( res /= 9784) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1164
      if ( res /= 1164) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7133
      if ( res /= 7133) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1137
      if ( res /= 1137) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2269
      if ( res /= 2269) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5232
      if ( res /= 5232) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7921
      if ( res /= 7921) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WMinus, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3424
      if ( res /= 3424) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1634
      if ( res /= 1634) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3151
      if ( res /= 3151) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6753
      if ( res /= 6753) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWpAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6756
      if ( res /= 6756) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_G0, P_GPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7224
      if ( res /= 7224) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5317
      if ( res /= 5317) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5783
      if ( res /= 5783) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8524
      if ( res /= 8524) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",676
      if ( res /= 676) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2885
      if ( res /= 2885) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2807
      if ( res /= 2807) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6478
      if ( res /= 6478) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1454
      if ( res /= 1454) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",567
      if ( res /= 567) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5962
      if ( res /= 5962) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3842
      if ( res /= 3842) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1797
      if ( res /= 1797) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZAnti, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1445
      if ( res /= 1445) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5278
      if ( res /= 5278) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_ghWp, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",783
      if ( res /= 783) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4734
      if ( res /= 4734) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9400
      if ( res /= 9400) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9379
      if ( res /= 9379) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WPlus, P_WMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",485
      if ( res /= 485) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",803
      if ( res /= 803) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3046
      if ( res /= 3046) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3579
      if ( res /= 3579) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hh, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4838
      if ( res /= 4838) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5167
      if ( res /= 5167) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",75
      if ( res /= 75) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8919
      if ( res /= 8919) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_WMinus, P_Z, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7635
      if ( res /= 7635) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1556
      if ( res /= 1556) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",922
      if ( res /= 922) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8163
      if ( res /= 8163) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7975
      if ( res /= 7975) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4069
      if ( res /= 4069) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10649
      if ( res /= 10649) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_GMinus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2969
      if ( res /= 2969) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWpAnti, P_A, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3456
      if ( res /= 3456) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6294
      if ( res /= 6294) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6074
      if ( res /= 6074) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7304
      if ( res /= 7304) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_WMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3791
      if ( res /= 3791) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2672
      if ( res /= 2672) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5958
      if ( res /= 5958) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2546
      if ( res /= 2546) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghZAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",120
      if ( res /= 120) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghWpAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2428
      if ( res /= 2428) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",891
      if ( res /= 891) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_A, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4774
      if ( res /= 4774) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2304
      if ( res /= 2304) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9209
      if ( res /= 9209) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",807
      if ( res /= 807) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9864
      if ( res /= 9864) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_tauMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5598
      if ( res /= 5598) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_Z, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8427
      if ( res /= 8427) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1553
      if ( res /= 1553) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",678
      if ( res /= 678) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",782
      if ( res /= 782) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7229
      if ( res /= 7229) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2506
      if ( res /= 2506) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2596
      if ( res /= 2596) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2342
      if ( res /= 2342) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2397
      if ( res /= 2397) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6438
      if ( res /= 6438) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WPlus, P_ghWpAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",363
      if ( res /= 363) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",494
      if ( res /= 494) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4159
      if ( res /= 4159) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9503
      if ( res /= 9503) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",967
      if ( res /= 967) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWp, P_Z, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3237
      if ( res /= 3237) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5544
      if ( res /= 5544) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9208
      if ( res /= 9208) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9618
      if ( res /= 9618) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",399
      if ( res /= 399) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3670
      if ( res /= 3670) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10409
      if ( res /= 10409) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7214
      if ( res /= 7214) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3565
      if ( res /= 3565) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3534
      if ( res /= 3534) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1861
      if ( res /= 1861) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9836
      if ( res /= 9836) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8187
      if ( res /= 8187) then
        stop
      end if
      call createParticleConfig(P_A, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2983
      if ( res /= 2983) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9746
      if ( res /= 9746) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6615
      if ( res /= 6615) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5644
      if ( res /= 5644) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6083
      if ( res /= 6083) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",981
      if ( res /= 981) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",58
      if ( res /= 58) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghWp, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4171
      if ( res /= 4171) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2363
      if ( res /= 2363) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1875
      if ( res /= 1875) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8764
      if ( res /= 8764) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9159
      if ( res /= 9159) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10420
      if ( res /= 10420) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_nu_e, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4360
      if ( res /= 4360) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5827
      if ( res /= 5827) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6451
      if ( res /= 6451) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3854
      if ( res /= 3854) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9241
      if ( res /= 9241) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_G0, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5422
      if ( res /= 5422) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1317
      if ( res /= 1317) then
        stop
      end if
      call createParticleConfig(P_b, P_A, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3362
      if ( res /= 3362) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7932
      if ( res /= 7932) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1782
      if ( res /= 1782) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghAAnti, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2982
      if ( res /= 2982) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9273
      if ( res /= 9273) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8305
      if ( res /= 8305) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3862
      if ( res /= 3862) then
        stop
      end if
      call createParticleConfig(P_g, P_A, P_A, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10548
      if ( res /= 10548) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3979
      if ( res /= 3979) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4265
      if ( res /= 4265) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4869
      if ( res /= 4869) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghAAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2056
      if ( res /= 2056) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7597
      if ( res /= 7597) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1527
      if ( res /= 1527) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8458
      if ( res /= 8458) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",50
      if ( res /= 50) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5259
      if ( res /= 5259) then
        stop
      end if
      call createParticleConfig(P_s, P_Hh, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4513
      if ( res /= 4513) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4092
      if ( res /= 4092) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_nu_eAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1754
      if ( res /= 1754) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10597
      if ( res /= 10597) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6378
      if ( res /= 6378) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7481
      if ( res /= 7481) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_nu_tau, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9098
      if ( res /= 9098) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3836
      if ( res /= 3836) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8997
      if ( res /= 8997) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10629
      if ( res /= 10629) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3958
      if ( res /= 3958) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9922
      if ( res /= 9922) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8599
      if ( res /= 8599) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9560
      if ( res /= 9560) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",927
      if ( res /= 927) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4095
      if ( res /= 4095) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8176
      if ( res /= 8176) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1357
      if ( res /= 1357) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5027
      if ( res /= 5027) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HPlus, P_GMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6020
      if ( res /= 6020) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8206
      if ( res /= 8206) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5788
      if ( res /= 5788) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10699
      if ( res /= 10699) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",262
      if ( res /= 262) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_tauMinus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3339
      if ( res /= 3339) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9168
      if ( res /= 9168) then
        stop
      end if
      call createParticleConfig(P_c, P_HMinus, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5136
      if ( res /= 5136) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7280
      if ( res /= 7280) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1091
      if ( res /= 1091) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1256
      if ( res /= 1256) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9444
      if ( res /= 9444) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6123
      if ( res /= 6123) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1449
      if ( res /= 1449) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4787
      if ( res /= 4787) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WPlus, P_WMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1877
      if ( res /= 1877) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6631
      if ( res /= 6631) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2720
      if ( res /= 2720) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1479
      if ( res /= 1479) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6303
      if ( res /= 6303) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2611
      if ( res /= 2611) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3671
      if ( res /= 3671) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7457
      if ( res /= 7457) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9913
      if ( res /= 9913) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HMinus, P_HPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6902
      if ( res /= 6902) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4225
      if ( res /= 4225) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2029
      if ( res /= 2029) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghAAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6624
      if ( res /= 6624) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8552
      if ( res /= 8552) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6353
      if ( res /= 6353) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2322
      if ( res /= 2322) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",560
      if ( res /= 560) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8930
      if ( res /= 8930) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_ghZAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8330
      if ( res /= 8330) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1384
      if ( res /= 1384) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6432
      if ( res /= 6432) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7834
      if ( res /= 7834) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10263
      if ( res /= 10263) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10395
      if ( res /= 10395) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4118
      if ( res /= 4118) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HPlus, P_ghWpAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3902
      if ( res /= 3902) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",208
      if ( res /= 208) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5054
      if ( res /= 5054) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3288
      if ( res /= 3288) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4326
      if ( res /= 4326) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",272
      if ( res /= 272) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7464
      if ( res /= 7464) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6299
      if ( res /= 6299) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2088
      if ( res /= 2088) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2184
      if ( res /= 2184) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5975
      if ( res /= 5975) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_nu_eAnti, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2192
      if ( res /= 2192) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWp, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8004
      if ( res /= 8004) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_G0, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10326
      if ( res /= 10326) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5852
      if ( res /= 5852) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1747
      if ( res /= 1747) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3499
      if ( res /= 3499) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_eMinus, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3814
      if ( res /= 3814) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7896
      if ( res /= 7896) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",420
      if ( res /= 420) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",459
      if ( res /= 459) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",538
      if ( res /= 538) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10436
      if ( res /= 10436) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4418
      if ( res /= 4418) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5699
      if ( res /= 5699) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6316
      if ( res /= 6316) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1167
      if ( res /= 1167) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8843
      if ( res /= 8843) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9492
      if ( res /= 9492) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10431
      if ( res /= 10431) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2945
      if ( res /= 2945) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghZAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6391
      if ( res /= 6391) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_HPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2553
      if ( res /= 2553) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10031
      if ( res /= 10031) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hl, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2154
      if ( res /= 2154) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10148
      if ( res /= 10148) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1328
      if ( res /= 1328) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_GMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2126
      if ( res /= 2126) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghWmAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",267
      if ( res /= 267) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5024
      if ( res /= 5024) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GPlus, P_HMinus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9698
      if ( res /= 9698) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8384
      if ( res /= 8384) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWm, P_ghWmAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8143
      if ( res /= 8143) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWm, P_ghZAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6061
      if ( res /= 6061) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5412
      if ( res /= 5412) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1351
      if ( res /= 1351) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_nu_mu, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6783
      if ( res /= 6783) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8276
      if ( res /= 8276) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2207
      if ( res /= 2207) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10293
      if ( res /= 10293) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8932
      if ( res /= 8932) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4665
      if ( res /= 4665) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9959
      if ( res /= 9959) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_nu_tauAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7594
      if ( res /= 7594) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3803
      if ( res /= 3803) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_Ha, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8408
      if ( res /= 8408) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1013
      if ( res /= 1013) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10030
      if ( res /= 10030) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5534
      if ( res /= 5534) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10364
      if ( res /= 10364) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3576
      if ( res /= 3576) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",179
      if ( res /= 179) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5635
      if ( res /= 5635) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5883
      if ( res /= 5883) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7206
      if ( res /= 7206) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7349
      if ( res /= 7349) then
        stop
      end if
      call createParticleConfig(P_Z, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4426
      if ( res /= 4426) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9370
      if ( res /= 9370) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7887
      if ( res /= 7887) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HMinus, P_GPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8789
      if ( res /= 8789) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4666
      if ( res /= 4666) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HMinus, P_HPlus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2182
      if ( res /= 2182) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",474
      if ( res /= 474) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3691
      if ( res /= 3691) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_ghA, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8622
      if ( res /= 8622) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2116
      if ( res /= 2116) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1035
      if ( res /= 1035) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4377
      if ( res /= 4377) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7255
      if ( res /= 7255) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_muMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3397
      if ( res /= 3397) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4381
      if ( res /= 4381) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5648
      if ( res /= 5648) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3306
      if ( res /= 3306) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8049
      if ( res /= 8049) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_Ha, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3276
      if ( res /= 3276) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4245
      if ( res /= 4245) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4195
      if ( res /= 4195) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5442
      if ( res /= 5442) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6036
      if ( res /= 6036) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6672
      if ( res /= 6672) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3106
      if ( res /= 3106) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5638
      if ( res /= 5638) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8712
      if ( res /= 8712) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2626
      if ( res /= 2626) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5070
      if ( res /= 5070) then
        stop
      end if
      call createParticleConfig(P_A, P_ghA, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",760
      if ( res /= 760) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_b, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",797
      if ( res /= 797) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6813
      if ( res /= 6813) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9286
      if ( res /= 9286) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6254
      if ( res /= 6254) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10276
      if ( res /= 10276) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10022
      if ( res /= 10022) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9910
      if ( res /= 9910) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GPlus, P_HMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1701
      if ( res /= 1701) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3056
      if ( res /= 3056) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9751
      if ( res /= 9751) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_nu_muAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6911
      if ( res /= 6911) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5190
      if ( res /= 5190) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3461
      if ( res /= 3461) then
        stop
      end if
      call createParticleConfig(P_s, P_G0, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2914
      if ( res /= 2914) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10621
      if ( res /= 10621) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2711
      if ( res /= 2711) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10216
      if ( res /= 10216) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1765
      if ( res /= 1765) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8799
      if ( res /= 8799) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_Ha, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9313
      if ( res /= 9313) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1959
      if ( res /= 1959) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_c, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",478
      if ( res /= 478) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3100
      if ( res /= 3100) then
        stop
      end if
      call createParticleConfig(P_nu_tauAnti, P_tauMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3590
      if ( res /= 3590) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_nu_eAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9483
      if ( res /= 9483) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9510
      if ( res /= 9510) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2852
      if ( res /= 2852) then
        stop
      end if
      call createParticleConfig(P_d, P_A, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4481
      if ( res /= 4481) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1518
      if ( res /= 1518) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3709
      if ( res /= 3709) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2548
      if ( res /= 2548) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7285
      if ( res /= 7285) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghZ, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8394
      if ( res /= 8394) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5238
      if ( res /= 5238) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",122
      if ( res /= 122) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2660
      if ( res /= 2660) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9069
      if ( res /= 9069) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hh, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",829
      if ( res /= 829) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_WPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1054
      if ( res /= 1054) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3107
      if ( res /= 3107) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_WPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8544
      if ( res /= 8544) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3313
      if ( res /= 3313) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3753
      if ( res /= 3753) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5708
      if ( res /= 5708) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2045
      if ( res /= 2045) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7913
      if ( res /= 7913) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",894
      if ( res /= 894) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5021
      if ( res /= 5021) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1418
      if ( res /= 1418) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",487
      if ( res /= 487) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1953
      if ( res /= 1953) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1878
      if ( res /= 1878) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4421
      if ( res /= 4421) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8348
      if ( res /= 8348) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",547
      if ( res /= 547) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1420
      if ( res /= 1420) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2271
      if ( res /= 2271) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9781
      if ( res /= 9781) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7443
      if ( res /= 7443) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2826
      if ( res /= 2826) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_A, P_GMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",342
      if ( res /= 342) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_c, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1764
      if ( res /= 1764) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5701
      if ( res /= 5701) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1995
      if ( res /= 1995) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1608
      if ( res /= 1608) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghA, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4645
      if ( res /= 4645) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8492
      if ( res /= 8492) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghWpAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4904
      if ( res /= 4904) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9716
      if ( res /= 9716) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",364
      if ( res /= 364) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8807
      if ( res /= 8807) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghZAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7149
      if ( res /= 7149) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3925
      if ( res /= 3925) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghAAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3835
      if ( res /= 3835) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5765
      if ( res /= 5765) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5653
      if ( res /= 5653) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5590
      if ( res /= 5590) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9173
      if ( res /= 9173) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWp, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2767
      if ( res /= 2767) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8691
      if ( res /= 8691) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",496
      if ( res /= 496) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4188
      if ( res /= 4188) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3392
      if ( res /= 3392) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWm, P_WPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2255
      if ( res /= 2255) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghAAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10281
      if ( res /= 10281) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2630
      if ( res /= 2630) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6056
      if ( res /= 6056) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7323
      if ( res /= 7323) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7983
      if ( res /= 7983) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5945
      if ( res /= 5945) then
        stop
      end if
      call createParticleConfig(P_g, P_G0, P_Ha, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10608
      if ( res /= 10608) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1784
      if ( res /= 1784) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3018
      if ( res /= 3018) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3719
      if ( res /= 3719) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghZAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3996
      if ( res /= 3996) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4239
      if ( res /= 4239) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8970
      if ( res /= 8970) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7546
      if ( res /= 7546) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6251
      if ( res /= 6251) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10571
      if ( res /= 10571) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghAAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1207
      if ( res /= 1207) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8972
      if ( res /= 8972) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7714
      if ( res /= 7714) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghWmAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6280
      if ( res /= 6280) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5855
      if ( res /= 5855) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7030
      if ( res /= 7030) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8866
      if ( res /= 8866) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1057
      if ( res /= 1057) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10378
      if ( res /= 10378) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4622
      if ( res /= 4622) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7134
      if ( res /= 7134) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_WMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8515
      if ( res /= 8515) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4309
      if ( res /= 4309) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_GMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7866
      if ( res /= 7866) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5481
      if ( res /= 5481) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8526
      if ( res /= 8526) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9119
      if ( res /= 9119) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10252
      if ( res /= 10252) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9621
      if ( res /= 9621) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8038
      if ( res /= 8038) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",993
      if ( res /= 993) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6886
      if ( res /= 6886) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghZ, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7936
      if ( res /= 7936) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6390
      if ( res /= 6390) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4575
      if ( res /= 4575) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2489
      if ( res /= 2489) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5947
      if ( res /= 5947) then
        stop
      end if
      call createParticleConfig(P_g, P_d, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2796
      if ( res /= 2796) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_A, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2724
      if ( res /= 2724) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7641
      if ( res /= 7641) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6436
      if ( res /= 6436) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1555
      if ( res /= 1555) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10272
      if ( res /= 10272) then
        stop
      end if
      call createParticleConfig(P_A, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6496
      if ( res /= 6496) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9170
      if ( res /= 9170) then
        stop
      end if
      call createParticleConfig(P_u, P_dAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1177
      if ( res /= 1177) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7466
      if ( res /= 7466) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8648
      if ( res /= 8648) then
        stop
      end if
      call createParticleConfig(P_ghGAnti, P_ghG, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5772
      if ( res /= 5772) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4229
      if ( res /= 4229) then
        stop
      end if
      call createParticleConfig(P_g, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",443
      if ( res /= 443) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6766
      if ( res /= 6766) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_g, P_g, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10612
      if ( res /= 10612) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_HMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5768
      if ( res /= 5768) then
        stop
      end if
      call createParticleConfig(P_g, P_HMinus, P_HPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10519
      if ( res /= 10519) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10708
      if ( res /= 10708) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1058
      if ( res /= 1058) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7357
      if ( res /= 7357) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6641
      if ( res /= 6641) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_ghWmAnti, P_A, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4717
      if ( res /= 4717) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_b, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3270
      if ( res /= 3270) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",452
      if ( res /= 452) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7498
      if ( res /= 7498) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4736
      if ( res /= 4736) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",531
      if ( res /= 531) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1444
      if ( res /= 1444) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5518
      if ( res /= 5518) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2311
      if ( res /= 2311) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghZAnti, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6735
      if ( res /= 6735) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HMinus, P_HPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2604
      if ( res /= 2604) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",712
      if ( res /= 712) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Z, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7093
      if ( res /= 7093) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",853
      if ( res /= 853) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4420
      if ( res /= 4420) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4558
      if ( res /= 4558) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3273
      if ( res /= 3273) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6071
      if ( res /= 6071) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_A, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",495
      if ( res /= 495) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8310
      if ( res /= 8310) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1712
      if ( res /= 1712) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4792
      if ( res /= 4792) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9080
      if ( res /= 9080) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5009
      if ( res /= 5009) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",923
      if ( res /= 923) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9041
      if ( res /= 9041) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1672
      if ( res /= 1672) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10476
      if ( res /= 10476) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3969
      if ( res /= 3969) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_A, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5834
      if ( res /= 5834) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9192
      if ( res /= 9192) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7171
      if ( res /= 7171) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8043
      if ( res /= 8043) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7166
      if ( res /= 7166) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hh, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4861
      if ( res /= 4861) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6862
      if ( res /= 6862) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9563
      if ( res /= 9563) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",344
      if ( res /= 344) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7518
      if ( res /= 7518) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10546
      if ( res /= 10546) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1125
      if ( res /= 1125) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6903
      if ( res /= 6903) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_d, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1227
      if ( res /= 1227) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9737
      if ( res /= 9737) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_WMinus, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",966
      if ( res /= 966) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4940
      if ( res /= 4940) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3501
      if ( res /= 3501) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8846
      if ( res /= 8846) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9026
      if ( res /= 9026) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4073
      if ( res /= 4073) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",42
      if ( res /= 42) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1798
      if ( res /= 1798) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9782
      if ( res /= 9782) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_Hl, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6113
      if ( res /= 6113) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3035
      if ( res /= 3035) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hh, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6290
      if ( res /= 6290) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10246
      if ( res /= 10246) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6810
      if ( res /= 6810) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7694
      if ( res /= 7694) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",801
      if ( res /= 801) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghWpAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4942
      if ( res /= 4942) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5353
      if ( res /= 5353) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HPlus, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9672
      if ( res /= 9672) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9526
      if ( res /= 9526) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWpAnti, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4199
      if ( res /= 4199) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4487
      if ( res /= 4487) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_ghWpAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",778
      if ( res /= 778) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2348
      if ( res /= 2348) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6706
      if ( res /= 6706) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HMinus, P_GPlus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10277
      if ( res /= 10277) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6175
      if ( res /= 6175) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4509
      if ( res /= 4509) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3468
      if ( res /= 3468) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6021
      if ( res /= 6021) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_HPlus, P_ghZAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9588
      if ( res /= 9588) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10043
      if ( res /= 10043) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7981
      if ( res /= 7981) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9049
      if ( res /= 9049) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",35
      if ( res /= 35) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hl, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7079
      if ( res /= 7079) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7956
      if ( res /= 7956) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Z, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3802
      if ( res /= 3802) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_A, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7157
      if ( res /= 7157) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_A, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4944
      if ( res /= 4944) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7752
      if ( res /= 7752) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5495
      if ( res /= 5495) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1650
      if ( res /= 1650) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3857
      if ( res /= 3857) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",52
      if ( res /= 52) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6375
      if ( res /= 6375) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5822
      if ( res /= 5822) then
        stop
      end if
      call createParticleConfig(P_A, P_HPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2960
      if ( res /= 2960) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4313
      if ( res /= 4313) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8490
      if ( res /= 8490) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWp, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2940
      if ( res /= 2940) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",997
      if ( res /= 997) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ePlus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8512
      if ( res /= 8512) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4424
      if ( res /= 4424) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9315
      if ( res /= 9315) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8736
      if ( res /= 8736) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10320
      if ( res /= 10320) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_Hh, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8569
      if ( res /= 8569) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9678
      if ( res /= 9678) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1228
      if ( res /= 1228) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7482
      if ( res /= 7482) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10
      if ( res /= 10) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7230
      if ( res /= 7230) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",639
      if ( res /= 639) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1182
      if ( res /= 1182) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7906
      if ( res /= 7906) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6629
      if ( res /= 6629) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_Ha, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6809
      if ( res /= 6809) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Z, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6192
      if ( res /= 6192) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3299
      if ( res /= 3299) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3726
      if ( res /= 3726) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8977
      if ( res /= 8977) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3311
      if ( res /= 3311) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9205
      if ( res /= 9205) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7141
      if ( res /= 7141) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9931
      if ( res /= 9931) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3904
      if ( res /= 3904) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5170
      if ( res /= 5170) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_Ha, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9239
      if ( res /= 9239) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1542
      if ( res /= 1542) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghA, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10171
      if ( res /= 10171) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_ghZAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6999
      if ( res /= 6999) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2171
      if ( res /= 2171) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6958
      if ( res /= 6958) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WPlus, P_HMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6128
      if ( res /= 6128) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hl, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7501
      if ( res /= 7501) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",817
      if ( res /= 817) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1206
      if ( res /= 1206) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6441
      if ( res /= 6441) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WMinus, P_Z, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3860
      if ( res /= 3860) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_Z, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1727
      if ( res /= 1727) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10739
      if ( res /= 10739) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_Ha, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3822
      if ( res /= 3822) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_tauPlus, P_nu_tau, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2849
      if ( res /= 2849) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1174
      if ( res /= 1174) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7015
      if ( res /= 7015) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4031
      if ( res /= 4031) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_GPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10383
      if ( res /= 10383) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5577
      if ( res /= 5577) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4429
      if ( res /= 4429) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5372
      if ( res /= 5372) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8587
      if ( res /= 8587) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10459
      if ( res /= 10459) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5072
      if ( res /= 5072) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GPlus, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7366
      if ( res /= 7366) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_Ha, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9383
      if ( res /= 9383) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghA, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4404
      if ( res /= 4404) then
        stop
      end if
      call createParticleConfig(P_u, P_Hl, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2405
      if ( res /= 2405) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghZ, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",866
      if ( res /= 866) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3356
      if ( res /= 3356) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghZ, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6444
      if ( res /= 6444) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5753
      if ( res /= 5753) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7937
      if ( res /= 7937) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_uAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8540
      if ( res /= 8540) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_ghWp, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",175
      if ( res /= 175) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9805
      if ( res /= 9805) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3333
      if ( res /= 3333) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7456
      if ( res /= 7456) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6972
      if ( res /= 6972) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3731
      if ( res /= 3731) then
        stop
      end if
      call createParticleConfig(P_nu_e, P_A, P_nu_eAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10140
      if ( res /= 10140) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9638
      if ( res /= 9638) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghAAnti, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5369
      if ( res /= 5369) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",815
      if ( res /= 815) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_ghAAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2655
      if ( res /= 2655) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9509
      if ( res /= 9509) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10195
      if ( res /= 10195) then
        stop
      end if
      call createParticleConfig(P_Ha, P_tauMinus, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1506
      if ( res /= 1506) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10501
      if ( res /= 10501) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5343
      if ( res /= 5343) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9331
      if ( res /= 9331) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_Z, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1
      if ( res /= 1) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4641
      if ( res /= 4641) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",170
      if ( res /= 170) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghWpAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3030
      if ( res /= 3030) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Z, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7661
      if ( res /= 7661) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6333
      if ( res /= 6333) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",503
      if ( res /= 503) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9142
      if ( res /= 9142) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6435
      if ( res /= 6435) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2529
      if ( res /= 2529) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8483
      if ( res /= 8483) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_G0, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5979
      if ( res /= 5979) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6117
      if ( res /= 6117) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4794
      if ( res /= 4794) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3143
      if ( res /= 3143) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWpAnti, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",925
      if ( res /= 925) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4455
      if ( res /= 4455) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3938
      if ( res /= 3938) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5749
      if ( res /= 5749) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2164
      if ( res /= 2164) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10711
      if ( res /= 10711) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_WMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5771
      if ( res /= 5771) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_ghA, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8012
      if ( res /= 8012) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2926
      if ( res /= 2926) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10071
      if ( res /= 10071) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9281
      if ( res /= 9281) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2131
      if ( res /= 2131) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7198
      if ( res /= 7198) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3838
      if ( res /= 3838) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghAAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8301
      if ( res /= 8301) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_eMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5676
      if ( res /= 5676) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4543
      if ( res /= 4543) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_t, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2471
      if ( res /= 2471) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9961
      if ( res /= 9961) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9361
      if ( res /= 9361) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7146
      if ( res /= 7146) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WMinus, P_WPlus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",744
      if ( res /= 744) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",30
      if ( res /= 30) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3695
      if ( res /= 3695) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2613
      if ( res /= 2613) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10467
      if ( res /= 10467) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_Hl, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2875
      if ( res /= 2875) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5234
      if ( res /= 5234) then
        stop
      end if
      call createParticleConfig(P_gQ, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3785
      if ( res /= 3785) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1808
      if ( res /= 1808) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4933
      if ( res /= 4933) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6528
      if ( res /= 6528) then
        stop
      end if
      call createParticleConfig(P_Z, P_WPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3465
      if ( res /= 3465) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1239
      if ( res /= 1239) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10430
      if ( res /= 10430) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_A, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5228
      if ( res /= 5228) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5323
      if ( res /= 5323) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6348
      if ( res /= 6348) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7099
      if ( res /= 7099) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3683
      if ( res /= 3683) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_Hl, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3408
      if ( res /= 3408) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2632
      if ( res /= 2632) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_nu_muAnti, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8396
      if ( res /= 8396) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9676
      if ( res /= 9676) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2279
      if ( res /= 2279) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",825
      if ( res /= 825) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6688
      if ( res /= 6688) then
        stop
      end if
      call createParticleConfig(P_ghA, P_A, P_ghWpAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1359
      if ( res /= 1359) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10214
      if ( res /= 10214) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1711
      if ( res /= 1711) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7637
      if ( res /= 7637) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",695
      if ( res /= 695) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6501
      if ( res /= 6501) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3790
      if ( res /= 3790) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4647
      if ( res /= 4647) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_nu_e, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7626
      if ( res /= 7626) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",112
      if ( res /= 112) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3607
      if ( res /= 3607) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3226
      if ( res /= 3226) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8905
      if ( res /= 8905) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3739
      if ( res /= 3739) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5460
      if ( res /= 5460) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Z, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10478
      if ( res /= 10478) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_G0, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8242
      if ( res /= 8242) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghAAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8057
      if ( res /= 8057) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Z, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1920
      if ( res /= 1920) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3625
      if ( res /= 3625) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_ghAAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8556
      if ( res /= 8556) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9496
      if ( res /= 9496) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6563
      if ( res /= 6563) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_Hh, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1324
      if ( res /= 1324) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8903
      if ( res /= 8903) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_WPlus, P_Hh, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1002
      if ( res /= 1002) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4793
      if ( res /= 4793) then
        stop
      end if
      call createParticleConfig(P_g, P_GMinus, P_HPlus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10631
      if ( res /= 10631) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8029
      if ( res /= 8029) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_Hh, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",207
      if ( res /= 207) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1792
      if ( res /= 1792) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5891
      if ( res /= 5891) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9679
      if ( res /= 9679) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Ha, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6246
      if ( res /= 6246) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_A, P_ghWmAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",579
      if ( res /= 579) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_WMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1386
      if ( res /= 1386) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8351
      if ( res /= 8351) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6028
      if ( res /= 6028) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9630
      if ( res /= 9630) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8690
      if ( res /= 8690) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8748
      if ( res /= 8748) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9682
      if ( res /= 9682) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9821
      if ( res /= 9821) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWpAnti, P_ghA, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4227
      if ( res /= 4227) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9242
      if ( res /= 9242) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hl, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9077
      if ( res /= 9077) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2035
      if ( res /= 2035) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",428
      if ( res /= 428) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9217
      if ( res /= 9217) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2761
      if ( res /= 2761) then
        stop
      end if
      call createParticleConfig(P_d, P_Hh, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4327
      if ( res /= 4327) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_A, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2100
      if ( res /= 2100) then
        stop
      end if
      call createParticleConfig(P_Ha, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6142
      if ( res /= 6142) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_G0, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4405
      if ( res /= 4405) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4997
      if ( res /= 4997) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",530
      if ( res /= 530) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8380
      if ( res /= 8380) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7005
      if ( res /= 7005) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Ha, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3447
      if ( res /= 3447) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8967
      if ( res /= 8967) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Z, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7433
      if ( res /= 7433) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_G0, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4163
      if ( res /= 4163) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3095
      if ( res /= 3095) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_A, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10335
      if ( res /= 10335) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6101
      if ( res /= 6101) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",139
      if ( res /= 139) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",641
      if ( res /= 641) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_tauPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6491
      if ( res /= 6491) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7018
      if ( res /= 7018) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghWmAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5991
      if ( res /= 5991) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8192
      if ( res /= 8192) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1879
      if ( res /= 1879) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_A, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5267
      if ( res /= 5267) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_HPlus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8411
      if ( res /= 8411) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4609
      if ( res /= 4609) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",856
      if ( res /= 856) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WMinus, P_ghWmAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1700
      if ( res /= 1700) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4864
      if ( res /= 4864) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5498
      if ( res /= 5498) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8097
      if ( res /= 8097) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5004
      if ( res /= 5004) then
        stop
      end if
      call createParticleConfig(P_s, P_cAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2778
      if ( res /= 2778) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4322
      if ( res /= 4322) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_Ha, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8687
      if ( res /= 8687) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6459
      if ( res /= 6459) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3038
      if ( res /= 3038) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7943
      if ( res /= 7943) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_nu_e, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2690
      if ( res /= 2690) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5081
      if ( res /= 5081) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghZ, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6957
      if ( res /= 6957) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10540
      if ( res /= 10540) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ePlus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5742
      if ( res /= 5742) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_G0, P_WMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3572
      if ( res /= 3572) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4586
      if ( res /= 4586) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7984
      if ( res /= 7984) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9559
      if ( res /= 9559) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9002
      if ( res /= 9002) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Hh, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1349
      if ( res /= 1349) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Z, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",433
      if ( res /= 433) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3019
      if ( res /= 3019) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7760
      if ( res /= 7760) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3595
      if ( res /= 3595) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3886
      if ( res /= 3886) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2180
      if ( res /= 2180) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_GPlus, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7860
      if ( res /= 7860) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",950
      if ( res /= 950) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9395
      if ( res /= 9395) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2503
      if ( res /= 2503) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2332
      if ( res /= 2332) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4162
      if ( res /= 4162) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_A, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1457
      if ( res /= 1457) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4865
      if ( res /= 4865) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_nu_mu, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4060
      if ( res /= 4060) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1562
      if ( res /= 1562) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9287
      if ( res /= 9287) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_A, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5097
      if ( res /= 5097) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_ghWmAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8533
      if ( res /= 8533) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9328
      if ( res /= 9328) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_WMinus, P_A, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1563
      if ( res /= 1563) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5453
      if ( res /= 5453) then
        stop
      end if
      call createParticleConfig(P_AQ, P_Hl, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2600
      if ( res /= 2600) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6510
      if ( res /= 6510) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWm, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1452
      if ( res /= 1452) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7445
      if ( res /= 7445) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3348
      if ( res /= 3348) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6111
      if ( res /= 6111) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10266
      if ( res /= 10266) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2544
      if ( res /= 2544) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_tauPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1191
      if ( res /= 1191) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghWp, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8034
      if ( res /= 8034) then
        stop
      end if
      call createParticleConfig(P_b, P_bAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",424
      if ( res /= 424) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2026
      if ( res /= 2026) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6750
      if ( res /= 6750) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_G0, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9962
      if ( res /= 9962) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8487
      if ( res /= 8487) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_WMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6713
      if ( res /= 6713) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5613
      if ( res /= 5613) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8045
      if ( res /= 8045) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6883
      if ( res /= 6883) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",315
      if ( res /= 315) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4937
      if ( res /= 4937) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6362
      if ( res /= 6362) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4516
      if ( res /= 4516) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_ghZ, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6646
      if ( res /= 6646) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3111
      if ( res /= 3111) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HPlus, P_A, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5199
      if ( res /= 5199) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghWmAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8126
      if ( res /= 8126) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2364
      if ( res /= 2364) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5948
      if ( res /= 5948) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_Hl, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7696
      if ( res /= 7696) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2387
      if ( res /= 2387) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_Z, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1636
      if ( res /= 1636) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8899
      if ( res /= 8899) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWm, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1028
      if ( res /= 1028) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3973
      if ( res /= 3973) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2791
      if ( res /= 2791) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7000
      if ( res /= 7000) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_Z, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6803
      if ( res /= 6803) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1073
      if ( res /= 1073) then
        stop
      end if
      call createParticleConfig(P_Hh, P_G0, P_Ha, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6319
      if ( res /= 6319) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_Hh, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",608
      if ( res /= 608) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3357
      if ( res /= 3357) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7638
      if ( res /= 7638) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5741
      if ( res /= 5741) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2156
      if ( res /= 2156) then
        stop
      end if
      call createParticleConfig(P_b, P_tAnti, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4032
      if ( res /= 4032) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7493
      if ( res /= 7493) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1201
      if ( res /= 1201) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2103
      if ( res /= 2103) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2033
      if ( res /= 2033) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_nu_tau, P_tauPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1021
      if ( res /= 1021) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1924
      if ( res /= 1924) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4332
      if ( res /= 4332) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8390
      if ( res /= 8390) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2528
      if ( res /= 2528) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghZ, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5821
      if ( res /= 5821) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_G0, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3703
      if ( res /= 3703) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_GMinus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8527
      if ( res /= 8527) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1284
      if ( res /= 1284) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghA, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5494
      if ( res /= 5494) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7136
      if ( res /= 7136) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7624
      if ( res /= 7624) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10176
      if ( res /= 10176) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4980
      if ( res /= 4980) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6522
      if ( res /= 6522) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WMinus, P_WPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2564
      if ( res /= 2564) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9706
      if ( res /= 9706) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6842
      if ( res /= 6842) then
        stop
      end if
      call createParticleConfig(P_tauMinus, P_GPlus, P_nu_tauAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4014
      if ( res /= 4014) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4519
      if ( res /= 4519) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7818
      if ( res /= 7818) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7081
      if ( res /= 7081) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5923
      if ( res /= 5923) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9308
      if ( res /= 9308) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10011
      if ( res /= 10011) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4705
      if ( res /= 4705) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_A, P_ghWpAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7567
      if ( res /= 7567) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7657
      if ( res /= 7657) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GMinus, P_GPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2586
      if ( res /= 2586) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWmAnti, P_HMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3853
      if ( res /= 3853) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8273
      if ( res /= 8273) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4168
      if ( res /= 4168) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_A, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8460
      if ( res /= 8460) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hh, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",988
      if ( res /= 988) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8132
      if ( res /= 8132) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2577
      if ( res /= 2577) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10147
      if ( res /= 10147) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWp, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9514
      if ( res /= 9514) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_A, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2576
      if ( res /= 2576) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9010
      if ( res /= 9010) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10723
      if ( res /= 10723) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3446
      if ( res /= 3446) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6428
      if ( res /= 6428) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_GPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5264
      if ( res /= 5264) then
        stop
      end if
      call createParticleConfig(P_Z, P_ghZAnti, P_ghWm, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4130
      if ( res /= 4130) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9366
      if ( res /= 9366) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9040
      if ( res /= 9040) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_t, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1661
      if ( res /= 1661) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5015
      if ( res /= 5015) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5916
      if ( res /= 5916) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7734
      if ( res /= 7734) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Hl, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",391
      if ( res /= 391) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9566
      if ( res /= 9566) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3756
      if ( res /= 3756) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_WPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1494
      if ( res /= 1494) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5691
      if ( res /= 5691) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1620
      if ( res /= 1620) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_GMinus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7496
      if ( res /= 7496) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8024
      if ( res /= 8024) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_Hl, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6610
      if ( res /= 6610) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_WPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",492
      if ( res /= 492) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8093
      if ( res /= 8093) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5210
      if ( res /= 5210) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_nu_muAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5733
      if ( res /= 5733) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_A, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",903
      if ( res /= 903) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_Hh, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2441
      if ( res /= 2441) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5941
      if ( res /= 5941) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghA, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9195
      if ( res /= 9195) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1055
      if ( res /= 1055) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_GMinus, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4336
      if ( res /= 4336) then
        stop
      end if
      call createParticleConfig(P_Hl, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3910
      if ( res /= 3910) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9461
      if ( res /= 9461) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4193
      if ( res /= 4193) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",958
      if ( res /= 958) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5978
      if ( res /= 5978) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9201
      if ( res /= 9201) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8
      if ( res /= 8) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_WMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7918
      if ( res /= 7918) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7781
      if ( res /= 7781) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghWpAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6377
      if ( res /= 6377) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Z, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10342
      if ( res /= 10342) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10189
      if ( res /= 10189) then
        stop
      end if
      call createParticleConfig(P_u, P_GMinus, P_dAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8193
      if ( res /= 8193) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1972
      if ( res /= 1972) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_g, P_g, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10694
      if ( res /= 10694) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_muMinus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3231
      if ( res /= 3231) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_HMinus, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1535
      if ( res /= 1535) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WPlus, P_ghZAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6648
      if ( res /= 6648) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3049
      if ( res /= 3049) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1281
      if ( res /= 1281) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7031
      if ( res /= 7031) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8268
      if ( res /= 8268) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HPlus, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5135
      if ( res /= 5135) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghZ, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7122
      if ( res /= 7122) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2748
      if ( res /= 2748) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7552
      if ( res /= 7552) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2308
      if ( res /= 2308) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1560
      if ( res /= 1560) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10709
      if ( res /= 10709) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_nu_mu, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3720
      if ( res /= 3720) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_Hl, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2518
      if ( res /= 2518) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5388
      if ( res /= 5388) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",524
      if ( res /= 524) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_WPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4037
      if ( res /= 4037) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3630
      if ( res /= 3630) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5091
      if ( res /= 5091) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_d, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",587
      if ( res /= 587) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",564
      if ( res /= 564) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9260
      if ( res /= 9260) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_Z, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1250
      if ( res /= 1250) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4741
      if ( res /= 4741) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5896
      if ( res /= 5896) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_G0, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5066
      if ( res /= 5066) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8774
      if ( res /= 8774) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7427
      if ( res /= 7427) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6023
      if ( res /= 6023) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9278
      if ( res /= 9278) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10016
      if ( res /= 10016) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3336
      if ( res /= 3336) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7880
      if ( res /= 7880) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2434
      if ( res /= 2434) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10143
      if ( res /= 10143) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9707
      if ( res /= 9707) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",656
      if ( res /= 656) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_Hl, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2964
      if ( res /= 2964) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HPlus, P_HMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3612
      if ( res /= 3612) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_A, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3901
      if ( res /= 3901) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_ghWm, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3636
      if ( res /= 3636) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2530
      if ( res /= 2530) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2624
      if ( res /= 2624) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Ha, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9798
      if ( res /= 9798) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4856
      if ( res /= 4856) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",809
      if ( res /= 809) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWmAnti, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3033
      if ( res /= 3033) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_WMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3563
      if ( res /= 3563) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6064
      if ( res /= 6064) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_WPlus, P_A, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2866
      if ( res /= 2866) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5400
      if ( res /= 5400) then
        stop
      end if
      call createParticleConfig(P_Ha, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1038
      if ( res /= 1038) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9561
      if ( res /= 9561) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_A, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5642
      if ( res /= 5642) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Ha, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2785
      if ( res /= 2785) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5085
      if ( res /= 5085) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_HMinus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8266
      if ( res /= 8266) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_HPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4145
      if ( res /= 4145) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2326
      if ( res /= 2326) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4029
      if ( res /= 4029) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9074
      if ( res /= 9074) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_G0, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1978
      if ( res /= 1978) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4553
      if ( res /= 4553) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7991
      if ( res /= 7991) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7324
      if ( res /= 7324) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWpAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1263
      if ( res /= 1263) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWp, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9485
      if ( res /= 9485) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5841
      if ( res /= 5841) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9394
      if ( res /= 9394) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4962
      if ( res /= 4962) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Ha, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7647
      if ( res /= 7647) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghA, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1398
      if ( res /= 1398) then
        stop
      end if
      call createParticleConfig(P_Z, P_HPlus, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4977
      if ( res /= 4977) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8931
      if ( res /= 8931) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Z, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5531
      if ( res /= 5531) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3121
      if ( res /= 3121) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HPlus, P_Hh, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5563
      if ( res /= 5563) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_nu_tau, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1710
      if ( res /= 1710) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Z, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3141
      if ( res /= 3141) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7738
      if ( res /= 7738) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghWp, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7824
      if ( res /= 7824) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2579
      if ( res /= 2579) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9376
      if ( res /= 9376) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2899
      if ( res /= 2899) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8848
      if ( res /= 8848) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9604
      if ( res /= 9604) then
        stop
      end if
      call createParticleConfig(P_G0, P_g, P_Ha, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10587
      if ( res /= 10587) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10344
      if ( res /= 10344) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9078
      if ( res /= 9078) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Z, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4738
      if ( res /= 4738) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_HMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4137
      if ( res /= 4137) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9263
      if ( res /= 9263) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9666
      if ( res /= 9666) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghZ, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2155
      if ( res /= 2155) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6053
      if ( res /= 6053) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_GPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7336
      if ( res /= 7336) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8473
      if ( res /= 8473) then
        stop
      end if
      call createParticleConfig(P_u, P_uAnti, P_gQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3157
      if ( res /= 3157) then
        stop
      end if
      call createParticleConfig(P_G0, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9753
      if ( res /= 9753) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3560
      if ( res /= 3560) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghZAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9335
      if ( res /= 9335) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2556
      if ( res /= 2556) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_G0, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5018
      if ( res /= 5018) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10435
      if ( res /= 10435) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",611
      if ( res /= 611) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2520
      if ( res /= 2520) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5399
      if ( res /= 5399) then
        stop
      end if
      call createParticleConfig(P_Hl, P_A, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6233
      if ( res /= 6233) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_nu_tau, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5010
      if ( res /= 5010) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4754
      if ( res /= 4754) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5594
      if ( res /= 5594) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6449
      if ( res /= 6449) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghZAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6586
      if ( res /= 6586) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1921
      if ( res /= 1921) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2398
      if ( res /= 2398) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghAAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8850
      if ( res /= 8850) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_cAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4049
      if ( res /= 4049) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7212
      if ( res /= 7212) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HMinus, P_WPlus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10067
      if ( res /= 10067) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3909
      if ( res /= 3909) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9824
      if ( res /= 9824) then
        stop
      end if
      call createParticleConfig(P_A, P_Hh, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10610
      if ( res /= 10610) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GMinus, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4764
      if ( res /= 4764) then
        stop
      end if
      call createParticleConfig(P_nu_mu, P_muPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1973
      if ( res /= 1973) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZAnti, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9897
      if ( res /= 9897) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7350
      if ( res /= 7350) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6579
      if ( res /= 6579) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2676
      if ( res /= 2676) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9850
      if ( res /= 9850) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_A, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3548
      if ( res /= 3548) then
        stop
      end if
      call createParticleConfig(P_g, P_GMinus, P_g, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10736
      if ( res /= 10736) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_Ha, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6266
      if ( res /= 6266) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9415
      if ( res /= 9415) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3338
      if ( res /= 3338) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8683
      if ( res /= 8683) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghWp, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4442
      if ( res /= 4442) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10727
      if ( res /= 10727) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Hl, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2935
      if ( res /= 2935) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_Hh, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",884
      if ( res /= 884) then
        stop
      end if
      call createParticleConfig(P_A, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10541
      if ( res /= 10541) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_G0, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9470
      if ( res /= 9470) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_A, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8042
      if ( res /= 8042) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Ha, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9212
      if ( res /= 9212) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8782
      if ( res /= 8782) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_ghAAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6836
      if ( res /= 6836) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7596
      if ( res /= 7596) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghWp, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5307
      if ( res /= 5307) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7823
      if ( res /= 7823) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",337
      if ( res /= 337) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10018
      if ( res /= 10018) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_G0, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",885
      if ( res /= 885) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9181
      if ( res /= 9181) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_ghWm, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7514
      if ( res /= 7514) then
        stop
      end if
      call createParticleConfig(P_A, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10400
      if ( res /= 10400) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",229
      if ( res /= 229) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_Ha, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9406
      if ( res /= 9406) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7762
      if ( res /= 7762) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghAAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9747
      if ( res /= 9747) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghAAnti, P_Hl, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8025
      if ( res /= 8025) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Z, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5076
      if ( res /= 5076) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Z, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4131
      if ( res /= 4131) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_A, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3713
      if ( res /= 3713) then
        stop
      end if
      call createParticleConfig(P_A, P_ghAAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4703
      if ( res /= 4703) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWmAnti, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7440
      if ( res /= 7440) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hh, P_ghWpAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3193
      if ( res /= 3193) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5273
      if ( res /= 5273) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9454
      if ( res /= 9454) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9532
      if ( res /= 9532) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_ghAAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",604
      if ( res /= 604) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4427
      if ( res /= 4427) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8839
      if ( res /= 8839) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9284
      if ( res /= 9284) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",620
      if ( res /= 620) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8243
      if ( res /= 8243) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_Z, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6571
      if ( res /= 6571) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1937
      if ( res /= 1937) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5842
      if ( res /= 5842) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1730
      if ( res /= 1730) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_GPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5387
      if ( res /= 5387) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_G0, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2511
      if ( res /= 2511) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_GMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7156
      if ( res /= 7156) then
        stop
      end if
      call createParticleConfig(P_g, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7719
      if ( res /= 7719) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3882
      if ( res /= 3882) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2002
      if ( res /= 2002) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_G0, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10656
      if ( res /= 10656) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GPlus, P_GMinus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",57
      if ( res /= 57) then
        stop
      end if
      call createParticleConfig(P_Z, P_HMinus, P_HPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4855
      if ( res /= 4855) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghA, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",682
      if ( res /= 682) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghWm, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3498
      if ( res /= 3498) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_A, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4183
      if ( res /= 4183) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6605
      if ( res /= 6605) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5808
      if ( res /= 5808) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hh, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",544
      if ( res /= 544) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_ghWp, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3082
      if ( res /= 3082) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZ, P_ghZAnti, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9851
      if ( res /= 9851) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7316
      if ( res /= 7316) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",561
      if ( res /= 561) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8438
      if ( res /= 8438) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5884
      if ( res /= 5884) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWmAnti, P_Z, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8162
      if ( res /= 8162) then
        stop
      end if
      call createParticleConfig(P_G0, P_Hh, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9651
      if ( res /= 9651) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3222
      if ( res /= 3222) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1575
      if ( res /= 1575) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghA, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9450
      if ( res /= 9450) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1370
      if ( res /= 1370) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_WPlus, P_ghAAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",66
      if ( res /= 66) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWm, P_HPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2683
      if ( res /= 2683) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_HPlus, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6595
      if ( res /= 6595) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",450
      if ( res /= 450) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3269
      if ( res /= 3269) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_A, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2827
      if ( res /= 2827) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghZ, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4287
      if ( res /= 4287) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",977
      if ( res /= 977) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghWp, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10152
      if ( res /= 10152) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7809
      if ( res /= 7809) then
        stop
      end if
      call createParticleConfig(P_A, P_ghZAnti, P_WMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3411
      if ( res /= 3411) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Hh, P_ghWmAnti, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",44
      if ( res /= 44) then
        stop
      end if
      call createParticleConfig(P_Z, P_Z, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5148
      if ( res /= 5148) then
        stop
      end if
      call createParticleConfig(P_ePlus, P_HMinus, P_nu_e, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2183
      if ( res /= 2183) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",448
      if ( res /= 448) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2754
      if ( res /= 2754) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9440
      if ( res /= 9440) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5377
      if ( res /= 5377) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hl, P_A, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9564
      if ( res /= 9564) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_WPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4693
      if ( res /= 4693) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2010
      if ( res /= 2010) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9852
      if ( res /= 9852) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9001
      if ( res /= 9001) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4836
      if ( res /= 4836) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4834
      if ( res /= 4834) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6937
      if ( res /= 6937) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWpAnti, P_ghWp, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8219
      if ( res /= 8219) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghZAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9986
      if ( res /= 9986) then
        stop
      end if
      call createParticleConfig(P_G0, P_HPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9268
      if ( res /= 9268) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3915
      if ( res /= 3915) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GPlus, P_HMinus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9655
      if ( res /= 9655) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WMinus, P_ghWp, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7225
      if ( res /= 7225) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3031
      if ( res /= 3031) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_GMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6801
      if ( res /= 6801) then
        stop
      end if
      call createParticleConfig(P_Hh, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1400
      if ( res /= 1400) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_HMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",382
      if ( res /= 382) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4200
      if ( res /= 4200) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10089
      if ( res /= 10089) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Ha, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1796
      if ( res /= 1796) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWpAnti, P_GPlus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",473
      if ( res /= 473) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10034
      if ( res /= 10034) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_t, P_bAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3645
      if ( res /= 3645) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1756
      if ( res /= 1756) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Ha, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7180
      if ( res /= 7180) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWm, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8337
      if ( res /= 8337) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hh, P_ghZ, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6126
      if ( res /= 6126) then
        stop
      end if
      call createParticleConfig(P_A, P_u, P_uAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2559
      if ( res /= 2559) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Hh, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1205
      if ( res /= 1205) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_g, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10737
      if ( res /= 10737) then
        stop
      end if
      call createParticleConfig(P_Z, P_A, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9675
      if ( res /= 9675) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9940
      if ( res /= 9940) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hl, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5101
      if ( res /= 5101) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8211
      if ( res /= 8211) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Z, P_A, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2390
      if ( res /= 2390) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_GPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4272
      if ( res /= 4272) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_HMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9297
      if ( res /= 9297) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4135
      if ( res /= 4135) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4968
      if ( res /= 4968) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7485
      if ( res /= 7485) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",59
      if ( res /= 59) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10107
      if ( res /= 10107) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghAAnti, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1618
      if ( res /= 1618) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WPlus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6374
      if ( res /= 6374) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWm, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9476
      if ( res /= 9476) then
        stop
      end if
      call createParticleConfig(P_ghA, P_HPlus, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3103
      if ( res /= 3103) then
        stop
      end if
      call createParticleConfig(P_dAnti, P_GMinus, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",858
      if ( res /= 858) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1382
      if ( res /= 1382) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",277
      if ( res /= 277) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6257
      if ( res /= 6257) then
        stop
      end if
      call createParticleConfig(P_g, P_HMinus, P_g, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10683
      if ( res /= 10683) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_Hl, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6834
      if ( res /= 6834) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8783
      if ( res /= 8783) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8152
      if ( res /= 8152) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6628
      if ( res /= 6628) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_bAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1346
      if ( res /= 1346) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7268
      if ( res /= 7268) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_HMinus, P_WMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9353
      if ( res /= 9353) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8682
      if ( res /= 8682) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2292
      if ( res /= 2292) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghWmAnti, P_GMinus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",974
      if ( res /= 974) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghA, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3477
      if ( res /= 3477) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_G0, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9145
      if ( res /= 9145) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7940
      if ( res /= 7940) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7846
      if ( res /= 7846) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6398
      if ( res /= 6398) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10544
      if ( res /= 10544) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Ha, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1759
      if ( res /= 1759) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7941
      if ( res /= 7941) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",493
      if ( res /= 493) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghWm, P_Z, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5050
      if ( res /= 5050) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GPlus, P_HMinus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4070
      if ( res /= 4070) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_ghZ, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7254
      if ( res /= 7254) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4495
      if ( res /= 4495) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2173
      if ( res /= 2173) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Z, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1781
      if ( res /= 1781) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4392
      if ( res /= 4392) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GPlus, P_ghWpAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3332
      if ( res /= 3332) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Z, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6136
      if ( res /= 6136) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_ghA, P_HPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",909
      if ( res /= 909) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4702
      if ( res /= 4702) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7258
      if ( res /= 7258) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghWp, P_ghAAnti, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4273
      if ( res /= 4273) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10463
      if ( res /= 10463) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1484
      if ( res /= 1484) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghZ, P_GMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3070
      if ( res /= 3070) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_ghZ, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",92
      if ( res /= 92) then
        stop
      end if
      call createParticleConfig(P_d, P_uAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2833
      if ( res /= 2833) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GMinus, P_G0, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5757
      if ( res /= 5757) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_WPlus, P_Ha, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9031
      if ( res /= 9031) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_HPlus, P_Z, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9442
      if ( res /= 9442) then
        stop
      end if
      call createParticleConfig(P_c, P_Hl, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4840
      if ( res /= 4840) then
        stop
      end if
      call createParticleConfig(P_G0, P_Ha, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10222
      if ( res /= 10222) then
        stop
      end if
      call createParticleConfig(P_A, P_Ha, P_A, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10696
      if ( res /= 10696) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1543
      if ( res /= 1543) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HMinus, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5964
      if ( res /= 5964) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2136
      if ( res /= 2136) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghWmAnti, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9101
      if ( res /= 9101) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_ghAAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3102
      if ( res /= 3102) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Z, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10756
      if ( res /= 10756) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghA, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",81
      if ( res /= 81) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3742
      if ( res /= 3742) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3847
      if ( res /= 3847) then
        stop
      end if
      call createParticleConfig(P_Ha, P_tauPlus, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5036
      if ( res /= 5036) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4380
      if ( res /= 4380) then
        stop
      end if
      call createParticleConfig(P_muPlus, P_nu_mu, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8790
      if ( res /= 8790) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6632
      if ( res /= 6632) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZAnti, P_HMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7104
      if ( res /= 7104) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3852
      if ( res /= 3852) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10770
      if ( res /= 10770) then
        stop
      end if
      call createParticleConfig(P_Z, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5356
      if ( res /= 5356) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2137
      if ( res /= 2137) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2432
      if ( res /= 2432) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5825
      if ( res /= 5825) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8076
      if ( res /= 8076) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_ghWm, P_WPlus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6959
      if ( res /= 6959) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_nu_mu, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",404
      if ( res /= 404) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9917
      if ( res /= 9917) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5104
      if ( res /= 5104) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_WPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7189
      if ( res /= 7189) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6689
      if ( res /= 6689) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Hh, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8911
      if ( res /= 8911) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6183
      if ( res /= 6183) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghZAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8441
      if ( res /= 8441) then
        stop
      end if
      call createParticleConfig(P_AQ, P_bAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6534
      if ( res /= 6534) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWpAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1695
      if ( res /= 1695) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_g, P_GMinus, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10538
      if ( res /= 10538) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7297
      if ( res /= 7297) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3316
      if ( res /= 3316) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghZ, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1288
      if ( res /= 1288) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_G0, P_ghWm, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6307
      if ( res /= 6307) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Hl, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1120
      if ( res /= 1120) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10081
      if ( res /= 10081) then
        stop
      end if
      call createParticleConfig(P_g, P_A, P_Z, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10772
      if ( res /= 10772) then
        stop
      end if
      call createParticleConfig(P_Z, P_WMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10455
      if ( res /= 10455) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_G0, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10014
      if ( res /= 10014) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9645
      if ( res /= 9645) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghZ, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7052
      if ( res /= 7052) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3992
      if ( res /= 3992) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8890
      if ( res /= 8890) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7371
      if ( res /= 7371) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4877
      if ( res /= 4877) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Z, P_GPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3342
      if ( res /= 3342) then
        stop
      end if
      call createParticleConfig(P_G0, P_Z, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9987
      if ( res /= 9987) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2717
      if ( res /= 2717) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1327
      if ( res /= 1327) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1113
      if ( res /= 1113) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_GMinus, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9247
      if ( res /= 9247) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1936
      if ( res /= 1936) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWmAnti, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5339
      if ( res /= 5339) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2085
      if ( res /= 2085) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_ghZAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9800
      if ( res /= 9800) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8508
      if ( res /= 8508) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_G0, P_HMinus, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",509
      if ( res /= 509) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_uAnti, P_u, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7253
      if ( res /= 7253) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8358
      if ( res /= 8358) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_HPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5084
      if ( res /= 5084) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hh, P_HMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5394
      if ( res /= 5394) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghA, P_HMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8871
      if ( res /= 8871) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZ, P_A, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1761
      if ( res /= 1761) then
        stop
      end if
      call createParticleConfig(P_G0, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2642
      if ( res /= 2642) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4034
      if ( res /= 4034) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_muMinus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",814
      if ( res /= 814) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_ghAAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10330
      if ( res /= 10330) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_Z, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9817
      if ( res /= 9817) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1825
      if ( res /= 1825) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_GPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7289
      if ( res /= 7289) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8495
      if ( res /= 8495) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_GPlus, P_Hh, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2650
      if ( res /= 2650) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_HPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2381
      if ( res /= 2381) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6249
      if ( res /= 6249) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghA, P_Hl, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",745
      if ( res /= 745) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghWm, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3673
      if ( res /= 3673) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_GMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8302
      if ( res /= 8302) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_WMinus, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3718
      if ( res /= 3718) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2834
      if ( res /= 2834) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_tAnti, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5239
      if ( res /= 5239) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Hh, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2934
      if ( res /= 2934) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",348
      if ( res /= 348) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_ghWmAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7227
      if ( res /= 7227) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghAAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",827
      if ( res /= 827) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2211
      if ( res /= 2211) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Ha, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1042
      if ( res /= 1042) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWp, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2793
      if ( res /= 2793) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6001
      if ( res /= 6001) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6508
      if ( res /= 6508) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8957
      if ( res /= 8957) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Ha, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9269
      if ( res /= 9269) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_ghWp, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4328
      if ( res /= 4328) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9356
      if ( res /= 9356) then
        stop
      end if
      call createParticleConfig(P_Z, P_GPlus, P_G0, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6066
      if ( res /= 6066) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hh, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1794
      if ( res /= 1794) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_ghWp, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6876
      if ( res /= 6876) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghZ, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",961
      if ( res /= 961) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9062
      if ( res /= 9062) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_t, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",408
      if ( res /= 408) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_A, P_GPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6715
      if ( res /= 6715) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_HPlus, P_A, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8918
      if ( res /= 8918) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghA, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7003
      if ( res /= 7003) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HPlus, P_G0, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3085
      if ( res /= 3085) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_Z, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10082
      if ( res /= 10082) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GMinus, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2303
      if ( res /= 2303) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",287
      if ( res /= 287) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8559
      if ( res /= 8559) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2918
      if ( res /= 2918) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8796
      if ( res /= 8796) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_GPlus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6730
      if ( res /= 6730) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Ha, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10274
      if ( res /= 10274) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8724
      if ( res /= 8724) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_Hh, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4887
      if ( res /= 4887) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_HMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9674
      if ( res /= 9674) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghWmAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9820
      if ( res /= 9820) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWpAnti, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6345
      if ( res /= 6345) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_Z, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6346
      if ( res /= 6346) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_A, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2049
      if ( res /= 2049) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8959
      if ( res /= 8959) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Z, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10776
      if ( res /= 10776) then
        stop
      end if
      call createParticleConfig(P_Z, P_Ha, P_Ha, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3895
      if ( res /= 3895) then
        stop
      end if
      call createParticleConfig(P_uAnti, P_u, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4066
      if ( res /= 4066) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_muMinus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3519
      if ( res /= 3519) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1887
      if ( res /= 1887) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Ha, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1689
      if ( res /= 1689) then
        stop
      end if
      call createParticleConfig(P_A, P_ghWmAnti, P_A, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5316
      if ( res /= 5316) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2995
      if ( res /= 2995) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_GPlus, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4880
      if ( res /= 4880) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghWm, P_G0, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",26
      if ( res /= 26) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_ghZ, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1913
      if ( res /= 1913) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_GMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1047
      if ( res /= 1047) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3989
      if ( res /= 3989) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_HPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7702
      if ( res /= 7702) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_HPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8704
      if ( res /= 8704) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",388
      if ( res /= 388) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",365
      if ( res /= 365) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_WPlus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3028
      if ( res /= 3028) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Ha, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",846
      if ( res /= 846) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hl, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8770
      if ( res /= 8770) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hh, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5314
      if ( res /= 5314) then
        stop
      end if
      call createParticleConfig(P_Hh, P_HMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8596
      if ( res /= 8596) then
        stop
      end if
      call createParticleConfig(P_Hl, P_tAnti, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2393
      if ( res /= 2393) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10053
      if ( res /= 10053) then
        stop
      end if
      call createParticleConfig(P_g, P_g, P_Z, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10531
      if ( res /= 10531) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7361
      if ( res /= 7361) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghZ, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8440
      if ( res /= 8440) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_Hl, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3484
      if ( res /= 3484) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",21
      if ( res /= 21) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8701
      if ( res /= 8701) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_HPlus, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",549
      if ( res /= 549) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7793
      if ( res /= 7793) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_GMinus, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4788
      if ( res /= 4788) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_g, P_g, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10628
      if ( res /= 10628) then
        stop
      end if
      call createParticleConfig(P_Z, P_sAnti, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4403
      if ( res /= 4403) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_G0, P_GMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8864
      if ( res /= 8864) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2380
      if ( res /= 2380) then
        stop
      end if
      call createParticleConfig(P_ghA, P_Hl, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5093
      if ( res /= 5093) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_WMinus, P_ghWmAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1968
      if ( res /= 1968) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_ghAAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4812
      if ( res /= 4812) then
        stop
      end if
      call createParticleConfig(P_G0, P_G0, P_Hh, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10577
      if ( res /= 10577) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_GPlus, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5536
      if ( res /= 5536) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1222
      if ( res /= 1222) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Ha, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10726
      if ( res /= 10726) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_GMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8934
      if ( res /= 8934) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_ghZAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",397
      if ( res /= 397) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1912
      if ( res /= 1912) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghAAnti, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6994
      if ( res /= 6994) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HMinus, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3577
      if ( res /= 3577) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_WMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7028
      if ( res /= 7028) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1320
      if ( res /= 1320) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hh, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5734
      if ( res /= 5734) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_Z, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3513
      if ( res /= 3513) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1547
      if ( res /= 1547) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_Z, P_nu_mu, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4025
      if ( res /= 4025) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_A, P_ghWmAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4648
      if ( res /= 4648) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZAnti, P_HMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5714
      if ( res /= 5714) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_GPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2315
      if ( res /= 2315) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_nu_eAnti, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3834
      if ( res /= 3834) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hl, P_Hl, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2811
      if ( res /= 2811) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1413
      if ( res /= 1413) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_t, P_tAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",24
      if ( res /= 24) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_G0, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1577
      if ( res /= 1577) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",210
      if ( res /= 210) then
        stop
      end if
      call createParticleConfig(P_A, P_Z, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9523
      if ( res /= 9523) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_GPlus, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1406
      if ( res /= 1406) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_GPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3728
      if ( res /= 3728) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8454
      if ( res /= 8454) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3936
      if ( res /= 3936) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hl, P_Hh, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8297
      if ( res /= 8297) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_G0, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6283
      if ( res /= 6283) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_A, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6232
      if ( res /= 6232) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_HPlus, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1892
      if ( res /= 1892) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghA, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5048
      if ( res /= 5048) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_A, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9677
      if ( res /= 9677) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Ha, P_ghWm, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2694
      if ( res /= 2694) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9605
      if ( res /= 9605) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Ha, P_GMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9530
      if ( res /= 9530) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_Hl, P_s, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5140
      if ( res /= 5140) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_Ha, P_GMinus, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10001
      if ( res /= 10001) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_HMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5501
      if ( res /= 5501) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hl, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2723
      if ( res /= 2723) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4366
      if ( res /= 4366) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_Hh, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3245
      if ( res /= 3245) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_GPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9112
      if ( res /= 9112) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9896
      if ( res /= 9896) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10100
      if ( res /= 10100) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4915
      if ( res /= 4915) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10288
      if ( res /= 10288) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWm, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5250
      if ( res /= 5250) then
        stop
      end if
      call createParticleConfig(P_A, P_WMinus, P_Ha, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5502
      if ( res /= 5502) then
        stop
      end if
      call createParticleConfig(P_Hh, P_GMinus, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10197
      if ( res /= 10197) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_Hl, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10123
      if ( res /= 10123) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hh, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5552
      if ( res /= 5552) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_G0, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2861
      if ( res /= 2861) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_GMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7585
      if ( res /= 7585) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8878
      if ( res /= 8878) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4260
      if ( res /= 4260) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4751
      if ( res /= 4751) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWmAnti, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3585
      if ( res /= 3585) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7023
      if ( res /= 7023) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_GMinus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6642
      if ( res /= 6642) then
        stop
      end if
      call createParticleConfig(P_muMinus, P_muPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4922
      if ( res /= 4922) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghWmAnti, P_ghA, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1337
      if ( res /= 1337) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_Hh, P_ghZAnti, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3881
      if ( res /= 3881) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_G0, P_ePlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6385
      if ( res /= 6385) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",111
      if ( res /= 111) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_Z, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8772
      if ( res /= 8772) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hl, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7375
      if ( res /= 7375) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_Hl, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2262
      if ( res /= 2262) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWm, P_WPlus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1675
      if ( res /= 1675) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hh, P_Ha, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",37
      if ( res /= 37) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3632
      if ( res /= 3632) then
        stop
      end if
      call createParticleConfig(P_c, P_A, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2842
      if ( res /= 2842) then
        stop
      end if
      call createParticleConfig(P_sAnti, P_s, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5851
      if ( res /= 5851) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghZ, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2077
      if ( res /= 2077) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1438
      if ( res /= 1438) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_Z, P_tauMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6481
      if ( res /= 6481) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_ghWp, P_Hh, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2545
      if ( res /= 2545) then
        stop
      end if
      call createParticleConfig(P_Ha, P_G0, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8804
      if ( res /= 8804) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_GMinus, P_G0, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1504
      if ( res /= 1504) then
        stop
      end if
      call createParticleConfig(P_A, P_GMinus, P_Hl, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4763
      if ( res /= 4763) then
        stop
      end if
      call createParticleConfig(P_c, P_Hh, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3521
      if ( res /= 3521) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Z, P_Z, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",630
      if ( res /= 630) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWmAnti, P_HMinus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6927
      if ( res /= 6927) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2418
      if ( res /= 2418) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_ghAAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5911
      if ( res /= 5911) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Hl, P_GPlus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7467
      if ( res /= 7467) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Hl, P_HPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",696
      if ( res /= 696) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_s, P_cAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4797
      if ( res /= 4797) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_Z, P_ghWpAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2384
      if ( res /= 2384) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HMinus, P_ghWmAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",571
      if ( res /= 571) then
        stop
      end if
      call createParticleConfig(P_t, P_tAnti, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",545
      if ( res /= 545) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_HPlus, P_HMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8414
      if ( res /= 8414) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1310
      if ( res /= 1310) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_G0, P_HPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9393
      if ( res /= 9393) then
        stop
      end if
      call createParticleConfig(P_nu_tau, P_tauPlus, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",279
      if ( res /= 279) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3660
      if ( res /= 3660) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6048
      if ( res /= 6048) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_WPlus, P_WMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5031
      if ( res /= 5031) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_G0, P_Ha, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7701
      if ( res /= 7701) then
        stop
      end if
      call createParticleConfig(P_Ha, P_WPlus, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7788
      if ( res /= 7788) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_G0, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9497
      if ( res /= 9497) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_G0, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5191
      if ( res /= 5191) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4501
      if ( res /= 4501) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Hl, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",390
      if ( res /= 390) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5162
      if ( res /= 5162) then
        stop
      end if
      call createParticleConfig(P_Z, P_Hh, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9795
      if ( res /= 9795) then
        stop
      end if
      call createParticleConfig(P_A, P_muMinus, P_muPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",53
      if ( res /= 53) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GPlus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",756
      if ( res /= 756) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_A, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10004
      if ( res /= 10004) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1150
      if ( res /= 1150) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_Z, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8681
      if ( res /= 8681) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HPlus, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8451
      if ( res /= 8451) then
        stop
      end if
      call createParticleConfig(P_ghA, P_WMinus, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3547
      if ( res /= 3547) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghAAnti, P_ghWp, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8753
      if ( res /= 8753) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7822
      if ( res /= 7822) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Ha, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4033
      if ( res /= 4033) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7190
      if ( res /= 7190) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_HPlus, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9230
      if ( res /= 9230) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3485
      if ( res /= 3485) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10338
      if ( res /= 10338) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_muMinus, P_nu_muAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8548
      if ( res /= 8548) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWmAnti, P_ghZ, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9323
      if ( res /= 9323) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZ, P_WMinus, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3944
      if ( res /= 3944) then
        stop
      end if
      call createParticleConfig(P_ghA, P_G0, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5037
      if ( res /= 5037) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7589
      if ( res /= 7589) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_A, P_WPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9972
      if ( res /= 9972) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_WMinus, P_ghZAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2930
      if ( res /= 2930) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_Ha, P_ghZAnti, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5108
      if ( res /= 5108) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_GMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1110
      if ( res /= 1110) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWm, P_Hl, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8775
      if ( res /= 8775) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_GPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5973
      if ( res /= 5973) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_G0, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8827
      if ( res /= 8827) then
        stop
      end if
      call createParticleConfig(P_G0, P_GPlus, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",510
      if ( res /= 510) then
        stop
      end if
      call createParticleConfig(P_gQ, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2541
      if ( res /= 2541) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghZAnti, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1844
      if ( res /= 1844) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hh, P_HPlus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3127
      if ( res /= 3127) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_WPlus, P_Hl, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6211
      if ( res /= 6211) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghA, P_ghAAnti, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5726
      if ( res /= 5726) then
        stop
      end if
      call createParticleConfig(P_ghG, P_ghGAnti, P_g, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6536
      if ( res /= 6536) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghAAnti, P_ghWp, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6814
      if ( res /= 6814) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5075
      if ( res /= 5075) then
        stop
      end if
      call createParticleConfig(P_s, P_sAnti, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7377
      if ( res /= 7377) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_Ha, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",76
      if ( res /= 76) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",117
      if ( res /= 117) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5192
      if ( res /= 5192) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_GMinus, P_G0, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2410
      if ( res /= 2410) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1405
      if ( res /= 1405) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_ghWm, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5525
      if ( res /= 5525) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_HMinus, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10375
      if ( res /= 10375) then
        stop
      end if
      call createParticleConfig(P_Hh, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9882
      if ( res /= 9882) then
        stop
      end if
      call createParticleConfig(P_nu_muAnti, P_WPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5848
      if ( res /= 5848) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_G0, P_GMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2962
      if ( res /= 2962) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6121
      if ( res /= 6121) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_GMinus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3277
      if ( res /= 3277) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghWmAnti, P_GMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5992
      if ( res /= 5992) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4758
      if ( res /= 4758) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_WPlus, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2447
      if ( res /= 2447) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hh, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4022
      if ( res /= 4022) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7835
      if ( res /= 7835) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghAAnti, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4054
      if ( res /= 4054) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_WPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2686
      if ( res /= 2686) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GPlus, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1004
      if ( res /= 1004) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1525
      if ( res /= 1525) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghA, P_ghAAnti, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",677
      if ( res /= 677) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_ghZAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6009
      if ( res /= 6009) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_ghZAnti, P_Z, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7300
      if ( res /= 7300) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_HPlus, P_GMinus, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6259
      if ( res /= 6259) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_GPlus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4526
      if ( res /= 4526) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_GMinus, P_Z, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7173
      if ( res /= 7173) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Ha, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1996
      if ( res /= 1996) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Ha, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8740
      if ( res /= 8740) then
        stop
      end if
      call createParticleConfig(P_A, P_HMinus, P_Hh, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6084
      if ( res /= 6084) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Hh, P_HMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5029
      if ( res /= 5029) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_WPlus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6439
      if ( res /= 6439) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_G0, P_ghA, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4806
      if ( res /= 4806) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hh, P_G0, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8896
      if ( res /= 8896) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_ghZ, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2429
      if ( res /= 2429) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_ghZ, P_A, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3526
      if ( res /= 3526) then
        stop
      end if
      call createParticleConfig(P_d, P_dAnti, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4269
      if ( res /= 4269) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_Ha, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5673
      if ( res /= 5673) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_GPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6523
      if ( res /= 6523) then
        stop
      end if
      call createParticleConfig(P_Ha, P_A, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9569
      if ( res /= 9569) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_HPlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1898
      if ( res /= 1898) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWpAnti, P_Hl, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4370
      if ( res /= 4370) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HMinus, P_Hh, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6884
      if ( res /= 6884) then
        stop
      end if
      call createParticleConfig(P_ghA, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7425
      if ( res /= 7425) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6765
      if ( res /= 6765) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_ghWm, P_HMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5294
      if ( res /= 5294) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_GPlus, P_A, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5186
      if ( res /= 5186) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_A, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1220
      if ( res /= 1220) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_WMinus, P_Z, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1155
      if ( res /= 1155) then
        stop
      end if
      call createParticleConfig(P_AQ, P_ghWp, P_WMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1242
      if ( res /= 1242) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_ghZAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3067
      if ( res /= 3067) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HMinus, P_GPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9945
      if ( res /= 9945) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_GMinus, P_GPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6600
      if ( res /= 6600) then
        stop
      end if
      call createParticleConfig(P_Ha, P_ghWpAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10358
      if ( res /= 10358) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_G0, P_WMinus, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8499
      if ( res /= 8499) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_GMinus, P_HPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9653
      if ( res /= 9653) then
        stop
      end if
      call createParticleConfig(P_AQ, P_cAnti, P_c, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6141
      if ( res /= 6141) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_ghWm, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6888
      if ( res /= 6888) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWmAnti, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2522
      if ( res /= 2522) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ghWm, P_GPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1411
      if ( res /= 1411) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_ghWp, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5929
      if ( res /= 5929) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_Hl, P_Hh, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2803
      if ( res /= 2803) then
        stop
      end if
      call createParticleConfig(P_AQ, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5790
      if ( res /= 5790) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_Hh, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3261
      if ( res /= 3261) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1835
      if ( res /= 1835) then
        stop
      end if
      call createParticleConfig(P_ghA, P_ghWpAnti, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9921
      if ( res /= 9921) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2323
      if ( res /= 2323) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_GMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3859
      if ( res /= 3859) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Z, P_Z, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10121
      if ( res /= 10121) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_Z, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7022
      if ( res /= 7022) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_Ha, P_ghAAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4563
      if ( res /= 4563) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_Hl, P_WPlus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4901
      if ( res /= 4901) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWm, P_Ha, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8052
      if ( res /= 8052) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hh, P_HMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4184
      if ( res /= 4184) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_HPlus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3995
      if ( res /= 3995) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_GPlus, P_Hl, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6220
      if ( res /= 6220) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Z, P_Hh, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4484
      if ( res /= 4484) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_GMinus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3078
      if ( res /= 3078) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghZ, P_Hl, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3571
      if ( res /= 3571) then
        stop
      end if
      call createParticleConfig(P_Z, P_G0, P_WPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10144
      if ( res /= 10144) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_dAnti, P_d, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2569
      if ( res /= 2569) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_GPlus, P_HMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9839
      if ( res /= 9839) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_Z, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5119
      if ( res /= 5119) then
        stop
      end if
      call createParticleConfig(P_Hh, P_ghWpAnti, P_HPlus, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8595
      if ( res /= 8595) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_WMinus, P_A, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6597
      if ( res /= 6597) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HPlus, P_ghWm, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5376
      if ( res /= 5376) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_ghZ, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",908
      if ( res /= 908) then
        stop
      end if
      call createParticleConfig(P_ghWp, P_ghAAnti, P_Ha, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8235
      if ( res /= 8235) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_A, P_WMinus, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6454
      if ( res /= 6454) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5152
      if ( res /= 5152) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_HMinus, P_Hl, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6749
      if ( res /= 6749) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2329
      if ( res /= 2329) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Ha, P_GPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9225
      if ( res /= 9225) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_ghWpAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10065
      if ( res /= 10065) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_GMinus, P_Hh, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2095
      if ( res /= 2095) then
        stop
      end if
      call createParticleConfig(P_GMinus, P_ghA, P_HPlus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9845
      if ( res /= 9845) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_G0, P_Ha, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",497
      if ( res /= 497) then
        stop
      end if
      call createParticleConfig(P_HaQ, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3826
      if ( res /= 3826) then
        stop
      end if
      call createParticleConfig(P_bAnti, P_GMinus, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5201
      if ( res /= 5201) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_HPlus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2812
      if ( res /= 2812) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10102
      if ( res /= 10102) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_G0, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",517
      if ( res /= 517) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_Hl, P_HPlus, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4288
      if ( res /= 4288) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_HPlus, P_Z, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4729
      if ( res /= 4729) then
        stop
      end if
      call createParticleConfig(P_A, P_Hl, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9928
      if ( res /= 9928) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_A, P_ghWm, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3538
      if ( res /= 3538) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_Hl, P_ghWp, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7473
      if ( res /= 7473) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_HPlus, P_G0, P_ghA, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9993
      if ( res /= 9993) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_GMinus, P_Ha, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3489
      if ( res /= 3489) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_Hl, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7690
      if ( res /= 7690) then
        stop
      end if
      call createParticleConfig(P_Hl, P_ePlus, P_eMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6671
      if ( res /= 6671) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_Hh, P_WMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4593
      if ( res /= 4593) then
        stop
      end if
      call createParticleConfig(P_HlQ, P_ghWmAnti, P_Hl, P_ghWm, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7730
      if ( res /= 7730) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_HPlus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4739
      if ( res /= 4739) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hl, P_WMinus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",114
      if ( res /= 114) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9199
      if ( res /= 9199) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_HMinus, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8383
      if ( res /= 8383) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghZ, P_HMinus, P_ghZAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1037
      if ( res /= 1037) then
        stop
      end if
      call createParticleConfig(P_Hl, P_WPlus, P_WMinus, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10452
      if ( res /= 10452) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghAAnti, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",461
      if ( res /= 461) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_HPlus, P_ghZAnti, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6980
      if ( res /= 6980) then
        stop
      end if
      call createParticleConfig(P_WQPlus, P_WMinus, P_Ha, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4824
      if ( res /= 4824) then
        stop
      end if
      call createParticleConfig(P_A, P_G0, P_GMinus, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4994
      if ( res /= 4994) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GMinus, P_ghZAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9155
      if ( res /= 9155) then
        stop
      end if
      call createParticleConfig(P_tauPlus, P_tauMinus, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6745
      if ( res /= 6745) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_Hl, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7479
      if ( res /= 7479) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghZ, P_Hl, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1023
      if ( res /= 1023) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_WMinus, P_WPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5023
      if ( res /= 5023) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_WPlus, P_Hh, P_Hh, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5308
      if ( res /= 5308) then
        stop
      end if
      call createParticleConfig(P_G0, P_WPlus, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6941
      if ( res /= 6941) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_HMinus, P_ghZ, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8963
      if ( res /= 8963) then
        stop
      end if
      call createParticleConfig(P_GQMinus, P_HMinus, P_HPlus, P_GQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2246
      if ( res /= 2246) then
        stop
      end if
      call createParticleConfig(P_g, P_WPlus, P_g, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10755
      if ( res /= 10755) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_Hh, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10250
      if ( res /= 10250) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghWmAnti, P_Ha, P_HQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6092
      if ( res /= 6092) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Z, P_Hl, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9264
      if ( res /= 9264) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_WMinus, P_Ha, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1468
      if ( res /= 1468) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_ghZAnti, P_WPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3889
      if ( res /= 3889) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghAAnti, P_ghA, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9768
      if ( res /= 9768) then
        stop
      end if
      call createParticleConfig(P_Hl, P_HMinus, P_HPlus, P_Hl, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8340
      if ( res /= 8340) then
        stop
      end if
      call createParticleConfig(P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9414
      if ( res /= 9414) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_G0, P_ghWm, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3617
      if ( res /= 3617) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_c, P_HlQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2128
      if ( res /= 2128) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HMinus, P_ghAAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",614
      if ( res /= 614) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_Hh, P_ghA, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1975
      if ( res /= 1975) then
        stop
      end if
      call createParticleConfig(P_GQPlus, P_WPlus, P_GMinus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5817
      if ( res /= 5817) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_ghWm, P_AQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4228
      if ( res /= 4228) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Z, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9372
      if ( res /= 9372) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghA, P_Ha, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10343
      if ( res /= 10343) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_ghWp, P_Hh, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4056
      if ( res /= 4056) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_g, P_t, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1090
      if ( res /= 1090) then
        stop
      end if
      call createParticleConfig(P_eMinus, P_ePlus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6199
      if ( res /= 6199) then
        stop
      end if
      call createParticleConfig(P_WMinus, P_ghZAnti, P_WPlus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7982
      if ( res /= 7982) then
        stop
      end if
      call createParticleConfig(P_Hl, P_GPlus, P_ghZ, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1843
      if ( res /= 1843) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_Hl, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9044
      if ( res /= 9044) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WMinus, P_ghA, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2951
      if ( res /= 2951) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWp, P_Hh, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4970
      if ( res /= 4970) then
        stop
      end if
      call createParticleConfig(P_ghZ, P_GPlus, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5334
      if ( res /= 5334) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_G0, P_GPlus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2249
      if ( res /= 2249) then
        stop
      end if
      call createParticleConfig(P_WQMinus, P_Ha, P_WPlus, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4110
      if ( res /= 4110) then
        stop
      end if
      call createParticleConfig(P_ZQ, P_HPlus, P_Ha, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8732
      if ( res /= 8732) then
        stop
      end if
      call createParticleConfig(P_HPlus, P_ghWpAnti, P_ghA, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8041
      if ( res /= 8041) then
        stop
      end if
      call createParticleConfig(P_cAnti, P_s, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4859
      if ( res /= 4859) then
        stop
      end if
      call createParticleConfig(P_G0, P_HMinus, P_Hh, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9152
      if ( res /= 9152) then
        stop
      end if
      call createParticleConfig(P_c, P_cAnti, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2976
      if ( res /= 2976) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWp, P_ghWpAnti, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9797
      if ( res /= 9797) then
        stop
      end if
      call createParticleConfig(P_HQMinus, P_Ha, P_ghWmAnti, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2228
      if ( res /= 2228) then
        stop
      end if
      call createParticleConfig(P_tAnti, P_HPlus, P_b, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5504
      if ( res /= 5504) then
        stop
      end if
      call createParticleConfig(P_Hl, P_s, P_sAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4201
      if ( res /= 4201) then
        stop
      end if
      call createParticleConfig(P_Ha, P_GPlus, P_WMinus, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8855
      if ( res /= 8855) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_Hl, P_Hl, P_HaQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5914
      if ( res /= 5914) then
        stop
      end if
      call createParticleConfig(P_ghAAnti, P_ghWm, P_WQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4456
      if ( res /= 4456) then
        stop
      end if
      call createParticleConfig(P_A, P_WPlus, P_A, P_WMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7692
      if ( res /= 7692) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_ghZ, P_G0, P_ghWpAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7186
      if ( res /= 7186) then
        stop
      end if
      call createParticleConfig(P_WPlus, P_Hh, P_A, P_HMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1880
      if ( res /= 1880) then
        stop
      end if
      call createParticleConfig(P_Hl, P_Hl, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5194
      if ( res /= 5194) then
        stop
      end if
      call createParticleConfig(P_nu_eAnti, P_eMinus, P_HPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",4329
      if ( res /= 4329) then
        stop
      end if
      call createParticleConfig(P_HMinus, P_ghWm, P_GPlus, P_ghWmAnti, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9591
      if ( res /= 9591) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_HMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9019
      if ( res /= 9019) then
        stop
      end if
      call createParticleConfig(P_GQ0, P_HPlus, P_Hh, P_GQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1015
      if ( res /= 1015) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_A, P_A, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",10660
      if ( res /= 10660) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_GMinus, P_G0, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1980
      if ( res /= 1980) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Ha, P_HMinus, P_HhQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3984
      if ( res /= 3984) then
        stop
      end if
      call createParticleConfig(P_GPlus, P_WMinus, P_Hl, P_Z, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8071
      if ( res /= 8071) then
        stop
      end if
      call createParticleConfig(P_Ha, P_Ha, P_ghWpAnti, P_ghWp, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8279
      if ( res /= 8279) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_WPlus, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",7836
      if ( res /= 7836) then
        stop
      end if
      call createParticleConfig(P_t, P_bAnti, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1815
      if ( res /= 1815) then
        stop
      end if
      call createParticleConfig(P_Ha, P_HPlus, P_HMinus, P_G0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3562
      if ( res /= 3562) then
        stop
      end if
      call createParticleConfig(P_AQ, P_WPlus, P_WMinus, P_ZQ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5274
      if ( res /= 5274) then
        stop
      end if
      call createParticleConfig(P_ghZAnti, P_ghWp, P_HMinus, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6789
      if ( res /= 6789) then
        stop
      end if
      call createParticleConfig(P_G0, P_ghZ, P_ghWmAnti, P_GMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1763
      if ( res /= 1763) then
        stop
      end if
      call createParticleConfig(P_G0, P_WMinus, P_Ha, P_WPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",9326
      if ( res /= 9326) then
        stop
      end if
      call createParticleConfig(P_ghWpAnti, P_ghWp, P_Ha, P_Ha, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2306
      if ( res /= 2306) then
        stop
      end if
      call createParticleConfig(P_Hl, P_muPlus, P_muMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",6846
      if ( res /= 6846) then
        stop
      end if
      call createParticleConfig(P_HhQ, P_HPlus, P_WQMinus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",8442
      if ( res /= 8442) then
        stop
      end if
      call createParticleConfig(P_HQPlus, P_Hh, P_HMinus, P_GQ0, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",5418
      if ( res /= 5418) then
        stop
      end if
      call createParticleConfig(P_ghWm, P_ghZAnti, P_Ha, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",3608
      if ( res /= 3608) then
        stop
      end if
      call createParticleConfig(P_AQ, P_HMinus, P_A, P_HQPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",334
      if ( res /= 334) then
        stop
      end if
      call createParticleConfig(P_ghWmAnti, P_Z, P_WMinus, P_ghZ, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",2594
      if ( res /= 2594) then
        stop
      end if
      call createParticleConfig(P_Hh, P_Hl, P_GMinus, P_GPlus, config)
      call dict(config, 1, address, found, 0)
      res = dict_get_value(address)
      write (*,*) "out=", res, found, address, "shouldbe = ",1141
      if ( res /= 1141) then
        stop
      end if
    end subroutine check_vertices_mdl

    subroutine get_vertex_lorentz_rank_mdl(ty,cc,mr)
      integer, intent(in) :: ty
      logical, intent(in) :: cc(0:)
      integer, intent(out) :: mr
      select case(ty)
      case(12)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(19)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(20)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(21)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(22)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(23)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(25)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(26)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(27)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(28)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(29)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(32)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(33)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(35)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(37)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(38)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(64)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(65)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(66)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(67)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(68)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(69)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(70)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(71)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(74)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(75)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(76)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(77)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(81)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(86)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(90)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case(99)
        if (cc(0) .and. cc(1)) then
          mr = 0
        else
          mr = 1
        end if 
      case(100)
        if (cc(0)) then
          mr = 0
        else
          mr = 1
        end if 
      case default
        mr = 0
      end select 
    end subroutine get_vertex_lorentz_rank_mdl
end module class_vertices
